#include "copyright.h"
#include "port.h"
#include "oct.h"
#include "smm.h"
#include "yylex.h"

extern octObject cfacet;

#define IF_STACK 80
int if_stack[IF_STACK];
int if_ptr = 0;

char *varname[MAXVARS];		/* names of variables */
term *froms[MAXVARS];
term *tos[MAXVARS];
term *bys[MAXVARS];
int currentVal[MAXVARS];
int currentMfrom[MAXVARS];
int mkArray[MAXVARS];
octObject *mkArrayBags[MAXVARS];
int nextvar = 0;		/* first slot open for a new var */
char *strsav();


/* declare a variable */
push_var(name, from, to, by, mk_array_spec)
char *name;
term *from, *to, *by;
int mk_array_spec;
{
    if (nextvar > MAXVARS) {
	error("Sorry: too many parameters");
    }
    froms[nextvar] = from;
    tos[nextvar] = to;
    bys[nextvar] = by;
    mkArray[nextvar] = mk_array_spec;
    varname[nextvar++] = name;
}


/* declare a variable */
pop_var()
{
    free_expr(froms[--nextvar]);
    free_expr(tos[nextvar]);
    free_expr(bys[nextvar]);
    FREE(varname[nextvar]);
}


do_itag(instTag)
char *instTag;
{
    int i;
    char buf[16];

    *instTag = '\0';
    for (i=0; i<nextvar; i++) {
	if (i == 0) {
	    (void) sprintf(buf, ".%d", currentVal[i]);
	} else {
	    (void) sprintf(buf, "x%d", currentVal[i]);
	}
	(void) strcat(instTag, buf);
    }
}


#ifdef FOO
/*
 * creates the bags for mkArray
 */
make_mkabags(i)
int i;			/* which array variable */
{
    int j;
    int from, to, diff;
    char *type;
    octObject arrayStruct;

    from = expr2number(froms[i]);
    to = expr2number(tos[i]);
    diff = (to-currentVal[i] > 0) ? to-currentVal[i]+1 :
	    currentVal[i]-to+1;
    mkArrayBags[i] = ALLOC(octObject, diff);

    arrayStruct.type = OCT_BAG;
    arrayStruct.contents.bag.name = "arrayStructure";
    if (octGetOrCreate(&cfacet, &arrayStruct) != OCT_OK) {
	octError("make_inst");
	exit(-1);
    }
    if (mkArray[i] == 1) {
	type = "row";
    } else {
	type = "column";
    }
    for (j=0; j<diff; j++) {
	o_create_bag(&arrayStruct, &(mkArrayBags[i][j]), type);
    }
}
#endif


/*
 * return true if something to gen
 */
init_gen_array(instTag, makeBags)
char *instTag;
int makeBags;
{
    int i;

    for (i=0; i<nextvar; i++) {
	currentVal[i] = expr2number(froms[i]);
	currentMfrom[i] = 0;
	if (makeBags && (mkArray[i] != 0)) {
	    /* OLD code -- should not happen
		    make_mkabags(i);
	    */
	}
    }
    do_itag(instTag);
}


/*
 * return false if there is nothing to gen
 */
int gen_array(instTag)
char *instTag;
{
    int i;
    int by;

    for (i = nextvar-1; i>=0; i--) {
	by = expr2number(bys[i]);
	if (by == 0)
	    yyerror("BY is zero");

	currentVal[i] += by;
	currentMfrom[i]++;
	if (by < 0) {
	    if (currentVal[i] >= expr2number(tos[i])) break;
	} else {
	    if (currentVal[i] <= expr2number(tos[i])) break;
	}
	currentVal[i] = expr2number(froms[i]);
	currentMfrom[i] = 0;
    }
    if (i<0) return 0;

    do_itag(instTag);
    return 1;
}


get_rc(rbag, cbag)
octObject *rbag, *cbag;
{
    int i;

    cbag->objectId = rbag->objectId = oct_null_id;
    for (i=0; i<nextvar;i++) {
	if (mkArray[i] == 1) {
	    *rbag = mkArrayBags[i][currentMfrom[i]];
	}
	if (mkArray[i] == 2) {
	    *cbag = mkArrayBags[i][currentMfrom[i]];
	}
    }
}


make_const(result, iconst)
term **result;
int iconst;
{
    int i;

    *result = MALLOC(term);
    (*result)->next = NIL(term);
    for (i=0; i<MAXVARS; i++) {
	(*result)->vars[i] = 0;
    }
    (*result)->coef = iconst;
}


int make_var(result, name)
term **result;
char *name;
{
    int varnum;
    int i;
    char buf[80];

    for (varnum = 0; varnum < nextvar; varnum++) {
	if (varname[varnum] == NIL(char)) {
	    (void) sprintf(buf, "Undefined parameter %s", name);
	    yysoft(buf);
	    return 0;
	}
	if (strcmp(varname[varnum], name) == 0) {
	    break;
	}
    }
    if (varnum == nextvar) {
	(void) sprintf(buf, "Undefined parameter %s", name);
	yysoft(buf);
	return 0;
    }
	
    *result = MALLOC(term);
    (*result)->next = NIL(term);
    for (i=0; i<MAXVARS; i++) {
	(*result)->vars[i] = 0;
    }
    (*result)->vars[varnum] = 1;
    (*result)->coef = 1;
    return 1;
}


int termcmp(term1, term2)
term *term1, *term2;
{
    int i;

    for (i = MAXVARS-1; i >= 0; i--) {
	if (term1->vars[i] > term2->vars[i]) {
	    return 1;
	}
	if (term1->vars[i] < term2->vars[i]) {
	    return -1;
	}
    }
    return 0;
}


/*
 * Sort the terms of an expression and merge identical items
 * Bubble sort seems easiest.  The list will be short anyway.
 */
sort_expr(expr)
term **expr;
{
    term *ind;		/* index */
    term *last;		/* previous to ind */
    term *point;	/* from this point on, list is sorted */
    term *temp;
    int cmp;

    for (point = NIL(term); point != *expr; point = ind) {
	last = NIL(term);
	for (ind = *expr; ind->next != point; ) {
	    cmp = termcmp(ind, ind->next);
	    if (cmp == 0) {
		/* merge */
		ind->coef += ind->next->coef;
		temp = ind->next;
	        ind->next = ind->next->next;
		FREE(temp);
	    } else if (cmp > 0) {
		/* swap */
		temp = ind->next->next;
		ind->next->next = ind;
		if (last == NIL(term)) {
		    last = *expr = ind->next;
		} else {
		    last->next = ind->next;
		    last = last->next;
		}
		ind->next = temp;
	    } else {
		/* advance */
		last = ind;
		ind = ind->next;
	    }
	}
    }
}


free_expr(expr)
term *expr;
{
    term *ind;
    term *temp;

    for (ind = expr; ind != NIL(term); ) {
	temp = ind->next;
	FREE(ind);
	ind = temp;
    }
}


do_add(expr1, expr2)		/* result returned in expr1 */
term **expr1, **expr2;
{
    term *ind;

    for (ind = *expr1; ind->next != NIL(term); ind = ind->next) {
    }
    ind->next = *expr2;
    sort_expr(expr1);
    *expr2 = NIL(term);
}


do_sub(expr1, expr2)		/* result returned in expr1 */
term **expr1, **expr2;
{
    term *ind;

    for (ind = *expr1; ind->next != NIL(term); ind = ind->next) {
    }
    ind->next = *expr2;
    for (ind = *expr2; ind != NIL(term); ind = ind->next) {
	ind->coef *= -1;
    }
    sort_expr(expr1);
    *expr2 = NIL(term);
}


do_div(expr1, expr2)		/* result returned in expr1 */
term **expr1, **expr2;
{
    term *result;
    term *ind1, *ind2, *ind3;
    int i;

    ind3 = NIL(term);
    for (ind1 = *expr1; ind1 != NIL(term); ind1 = ind1->next) {
	for (ind2 = *expr2; ind2 != NIL(term); ind2 = ind2->next) {
	    if (ind3 == NIL(term)) {
		result = ind3 = MALLOC(term);
	    } else {
		ind3->next = MALLOC(term);
		ind3 = ind3->next;
	    }
	    ind3->coef = ind1->coef / ind2->coef;
	    for (i=0; i<MAXVARS; i++) {
		ind3->vars[i] = ind1->vars[i] + ind2->vars[i];
	    }
	}
    }
    ind3->next = NIL(term);
    sort_expr(&result);
    free_expr(*expr1);
    free_expr(*expr2);
    *expr1 = result;
    *expr2 = NIL(term);
}


do_mod(expr1, expr2)		/* result returned in expr1 */
term **expr1, **expr2;
{
    term *result;
    term *ind1, *ind2, *ind3;
    int i;

    ind3 = NIL(term);
    for (ind1 = *expr1; ind1 != NIL(term); ind1 = ind1->next) {
	for (ind2 = *expr2; ind2 != NIL(term); ind2 = ind2->next) {
	    if (ind3 == NIL(term)) {
		result = ind3 = MALLOC(term);
	    } else {
		ind3->next = MALLOC(term);
		ind3 = ind3->next;
	    }
	    ind3->coef = ind1->coef % ind2->coef;
	    for (i=0; i<MAXVARS; i++) {
		ind3->vars[i] = ind1->vars[i] + ind2->vars[i];
	    }
	}
    }
    ind3->next = NIL(term);
    sort_expr(&result);
    free_expr(*expr1);
    free_expr(*expr2);
    *expr1 = result;
    *expr2 = NIL(term);
}


int
pop_if ()
{
	if (if_ptr <= 0) {
		yyerror("if stack underflow");
	}
	return if_stack[--if_ptr];
}

push_if (old, value)
int old;
term *value;
{
	int result;

	if (if_ptr >= IF_STACK) {
		yyerror("if nested too deeply");
	}
	if_stack[if_ptr++] = old;

	result = expr2number (value);
	free_expr(value);

	return result;
}

do_mult(expr1, expr2)		/* result returned in expr1 */
term **expr1, **expr2;
{
    term *result;
    term *ind1, *ind2, *ind3;
    int i;

    ind3 = NIL(term);
    for (ind1 = *expr1; ind1 != NIL(term); ind1 = ind1->next) {
	for (ind2 = *expr2; ind2 != NIL(term); ind2 = ind2->next) {
	    if (ind3 == NIL(term)) {
		result = ind3 = MALLOC(term);
	    } else {
		ind3->next = MALLOC(term);
		ind3 = ind3->next;
	    }
	    ind3->coef = ind1->coef * ind2->coef;
	    for (i=0; i<MAXVARS; i++) {
		ind3->vars[i] = ind1->vars[i] + ind2->vars[i];
	    }
	}
    }
    ind3->next = NIL(term);
    sort_expr(&result);
    free_expr(*expr1);
    free_expr(*expr2);
    *expr1 = result;
    *expr2 = NIL(term);
}


int expr2number(expr)
term *expr;
{
    int number;
    term *ind;
    int j;			/* variables of term */
    int k;
    int partsum;

    number = 0;
    for (ind = expr; ind != NIL(term); ind = ind->next) {
	partsum = ind->coef;
	for (j=0; j<MAXVARS; j++) {
	    for (k = 0; k < ind->vars[j]; k++) {
		partsum *= currentVal[j];
	    }
	}
	number += partsum;
    }
    return number;
}

