#include "copyright.h"
#include "port.h"
#include "options.h"
#include "oct.h"
#include "vov.h"

int errorcount = 0;
int topdown = 0;
int singlenets = 0;
int showconnect = 0;
int showall = 0;
int warning = 1;
int noVulcan = 0;		/* Flag to decide if vulcan should be run. */
int  showBags  = 0;
int  showProps = 0;
extern int dump_tokens;

octObject cfacet;
octObject ifacet;


optionStruct optionList[] = {
    { "n",	0,	"write the netlist to standard output"},
    { "a",	0,	"when used with '-n', all the instances are output"},
    { "c",	0,	"when used with '-n', the connectors are also output"},
    { "P",	0,	"when used with '-n', all properties are output"},
    { "B",	0,	"when used with '-n', all bags are output"},
    { "i",	0,	"write an inverted inetlist to standard output (list of nets and connections rather than a list of elements and connections)"},
    { "s",	0,	"when used with '-i', only show nets with zero or 1 connection (good for looking for bad connections)"},
    { "t",	0,	"create the master if the master of an instance does not exist (for top-down design)"},
    { "w",	0,	"suppress warnings"},
    { "v",      0,      "do not run vulcan on interface facet"},
    { "d",	0,	"dump tokens (for debugging)"},
    { OPT_RARG,	"{netlist | cellname[:viewname]}",	"netlist description file or Oct facet (for '-i' and '-n')"},
    { 0,	0,	0},
};

main(argc, argv)
int argc;
char **argv;
{
    int invert = 0, nout = 0;
    char *cell = "", *view = "";
    extern int optind;
    extern char *optarg;
    int option;

    cfacet.objectId = oct_null_id;
    ifacet.objectId = oct_null_id;
    while ((option = optGetOption(argc, argv)) != EOF) {
	switch(option) {
	case 'v': noVulcan = 1; break;
	case 'w': warning = 0; break;
	case 't': topdown = 1; break;
	case 's': singlenets = 1; break;
	case 'i': invert = 1; break;
	case 'n': nout = 1; break;
	case 'c': showconnect = 1; break;
	case 'a': showall = 1; break;
	case 'P': showProps = 1; break;
	case 'B': showBags = 1; break;
	case 'd': dump_tokens = 1; break;
	}
    }

    if ( argc == 1 ) {
	optUsage();
    }
    VOVbegin( argc, argv );

    if (invert) {
	if (argc != optind + 1) {
	    optUsage();
	}
	parse_facet_spec(argv[optind], &cell, &view);
	open_old_facet(&cfacet, cell, view, "contents", "r");
	shown();
    } else if (nout) {
	if (argc != optind + 1) {
	    optUsage();
	}
	parse_facet_spec(argv[optind], &cell, &view);
	open_old_facet(&cfacet, cell, view, "contents", "r");
	open_old_facet(&ifacet, cell, view, "interface", "r");
	netout(cell, view);
    } else {
	yylex_init();
	if (optind >= argc) {
	    if (yyopenfile("(stdin)", 1, 0) < 0) {
		(void) VOVend(1);
	    }
	} else if (optind == argc-1) {
	    if (yyopenfile(argv[optind], 0, 0) < 0) {
		(void) VOVend(1);
	    }
	} else {
	    optUsage();
	}
	(void) yyparse();
    }
    if (errorcount) {
	    (void) fprintf (stderr, "Terminated with %d errors and/or warnings\n", 
		    errorcount);
    }
    VOVend( 0 );
}


parse_facet_spec(arg, cell, view)
char *arg, **cell, **view;
{
    char *colon;

    if ((colon = strchr(arg, ':')) == NULL) {
	*cell = arg;
    } else {
	*colon = '\0';
	*cell = arg;
	*view = colon+1;
    }
    if (**cell == '\0') {
	(void) fprintf(stderr, "No input cell name\n");
	(void) VOVend(-1);
    }
    if (**view == '\0') {
	*view = "symbolic";
    }
}


open_old_facet(theFacet, cell, view, facet, mode)
octObject *theFacet;
char *cell, *view, *facet, *mode;
{
    theFacet->type = OCT_FACET;
    theFacet->contents.facet.cell = cell;
    theFacet->contents.facet.view = view;
    theFacet->contents.facet.facet = facet;
    theFacet->contents.facet.mode = mode;
    theFacet->contents.facet.version = OCT_CURRENT_VERSION;
    fprintf(stderr, "bdnet: opening %s\n", ohFormatName(theFacet));
    
    VOVinputFacet( theFacet );

    if (octOpenFacet(theFacet) < OCT_OK) {
	octError("open failed");
	(void) VOVend(-1);
    }
}
