/*
 * Property and bag annotation have been added by
 * Enrico Malavasi  -- malavasi@ic.berkeley.edu
 * August 1991
 */

#include "copyright.h"
#include "port.h"
#include "oct.h"
#include "oh.h"
#include "st.h"
#include "smm.h"

extern int singlenets;
extern int showconnect;
extern int showall;
extern int  showBags;
extern int  showProps;

extern octObject cfacet;
extern octObject ifacet;

/*
 * carry out the -i option
 */
shown()
{
    octObject net, bag;
    octObject inst;
    octObject term, aterm;
    octGenerator netgen, termgen, atermgen, baggen;

    o_init_gen_contents(&cfacet, OCT_NET_MASK, &netgen);
    while (octGenerate(&netgen, &net) == OCT_OK) {
	if (singlenets) {
	    o_init_gen_contents(&net, OCT_TERM_MASK, &termgen);
	    (void) octGenerate(&termgen, &term);
	    if (octGenerate(&termgen, &term) == OCT_OK) {
		/* two connected terms are ok -- don't list */
		continue;
	    }
	}
	if (strlen (net.contents.net.name)) {
		(void) printf("%s :", net.contents.net.name);
	}
	else {
		/* give the net the term name ... */
		o_gen_first_content (&net, OCT_TERM_MASK, &term);
	    if (octIdIsNull((inst.objectId = term.contents.term.instanceId))) {
		(void) printf("n_%s :", term.contents.term.name);
	    } else {
		if (octGetById(&inst) != OCT_OK) {
		    octError("");
		    exit(-1);
		}
		(void) printf("n_%s_%s :", inst.contents.instance.name,
			term.contents.term.name);
	    }
	}

	o_init_gen_contents(&net, OCT_TERM_MASK, &termgen);
	while (octGenerate(&termgen, &term) == OCT_OK) {
	    if (octIdIsNull((inst.objectId = term.contents.term.instanceId))) {
		(void) printf(" %s", term.contents.term.name);
	    } else {
		if (octGetById(&inst) != OCT_OK) {
		    octError("");
		    exit(-1);
		}
		(void) printf(" %s/%s", inst.contents.instance.name,
			term.contents.term.name);
	    }
	}
	(void) printf("\n");
    }

    /* now check for error conditions associate with term promotion */
    o_init_gen_contents(&cfacet, OCT_TERM_MASK, &termgen);
    while (octGenerate(&termgen, &term) == OCT_OK) {
	o_init_gen_contents(&term, OCT_TERM_MASK, &atermgen);
	if (octGenerate(&atermgen, &aterm) != OCT_OK) {
	    continue;
	}

	if (octIdIsNull((inst.objectId = aterm.contents.term.instanceId))) {
	    if (octGetById(&inst) != OCT_OK) {
		octError("Strange attachment encountered");
		exit(-1);
	    }
	}

	o_init_gen_containers(&inst, OCT_BAG_MASK, &baggen);
	while (octGenerate(&baggen, &bag) == OCT_OK) {
	    if (strcmp(bag.contents.bag.name, "CONNECTORS") == OCT_OK) {
		goto cont;
	    }
	}

	o_init_gen_containers(&term, OCT_NET_MASK, &netgen);
	if (octGenerate(&netgen, &net) != OCT_OK) {
	    (void) printf("ERROR: A net must contain formal terminal %s and its promoted actual\n",
		    term.contents.term.name); 
	    continue;
	}

	if (octIsAttached(&net, &aterm) != OCT_OK) {
	    (void) printf("ERROR: Net of formal terminal %s must contain it and its promoted actual\n",
		    term.contents.term.name); 
	}
cont: ;
    }
}

static int
  standardBag( bag )
octObject *bag;
{
    if(! strcmp(bag->contents.bag.name, "INSTANCES"))     return 1;
    if(! strcmp(bag->contents.bag.name, "CONNECTORS"))    return 1;
    return 0;
}
static int
  standardProp( prop )
octObject *prop;
{
    if(! strcmp(prop->contents.prop.name, "TECHNOLOGY"))    return 1;
    if(! strcmp(prop->contents.prop.name, "VIEWTYPE"))      return 1;
    if(! strcmp(prop->contents.prop.name, "EDITSTYLE"))     return 1;
    return 0;
}

static void
  print_prop( prop )
octObject *prop;
{
    switch( prop->contents.prop.type ) {
      case OCT_STRING:
	printf("\t(CREATE SPROP \"%s\" \"%s\")\n", prop->contents.prop.name, prop->contents.prop.value.string);
	break;
      case OCT_REAL:
	printf("\t(CREATE RPROP \"%s\" %f)\n", prop->contents.prop.name, prop->contents.prop.value.real);
	break;
      case OCT_INTEGER:
	printf("\t(CREATE IPROP \"%s\" %d)\n", prop->contents.prop.name, prop->contents.prop.value.integer);
	break;
    }
}
static void
  print_bag( bag)
octObject *bag;
{
    int   fff;
    octObject inst, bbag, prop, term;
    octGenerator instGener, bagGener, propGener, termGener;

    printf("\t(CREATE BAG %s\n", bag->contents.bag.name);
    OH_ASSERT(octInitGenContents(bag, OCT_INSTANCE_MASK, &instGener));
    while (octGenerate(&instGener, &inst) == OCT_OK) {
	if( strcmp(inst.contents.instance.name, "")) {
	    printf("\t\t(INSTANCE %s", inst.contents.instance.name);
	    if( showProps ) {
		fff = 0;
		OH_ASSERT(octInitGenContents(&inst, OCT_PROP_MASK, &propGener));
		while (octGenerate(&propGener, &prop) == OCT_OK) {
		    if(! fff) {
			printf("\n");
			fff = 1;
		    }
		    print_prop( &prop );
		}
		octFreeGenerator( &propGener );
	    }
	    printf("\t)\n");
	}
    }
    octFreeGenerator(&instGener);

    OH_ASSERT(octInitGenContents(bag, OCT_TERM_MASK, &termGener));
    while (octGenerate(&termGener, &term) == OCT_OK) {
	if(octIdIsNull(term.contents.term.instanceId)) {
	    printf("\t\t(FTERM %s", term.contents.term.name);
	}
    }
    octFreeGenerator(&termGener);

    OH_ASSERT(octInitGenContents(bag, OCT_BAG_MASK, &bagGener));
    while (octGenerate(&bagGener, &bbag) == OCT_OK) {
	print_bag( &bbag );
    }
    octFreeGenerator(&bagGener);

    if( showProps ) {
	OH_ASSERT(octInitGenContents(bag, OCT_PROP_MASK, &propGener));
	while (octGenerate(&propGener, &prop) == OCT_OK) {
	    print_prop( &prop );
	}
	octFreeGenerator(&propGener);
    }

    printf("\t)\n");
}

/*
 * Carry out the -n function
 */
netout(cell, view)
char *cell, *view;
{
    char *dir, *termt;
    octObject net, inst, prop, prop2;
    octObject bag;
    octObject fterm, iterm;
    octGenerator netgen, ftermgen;

    (void) printf("MODEL \"%s\":%s;\n\n", cell, view);
    prop.type = OCT_PROP;
    prop.contents.prop.name = "TECHNOLOGY";
    if (octGetByName(&cfacet, &prop) == OCT_OK) {
	(void) printf("TECHNOLOGY %s;\n", prop.contents.prop.value.string);
    }
    prop.contents.prop.name = "VIEWTYPE";
    if (octGetByName(&cfacet, &prop) == OCT_OK) {
	(void) printf("VIEWTYPE %s;\n", prop.contents.prop.value.string);
    }
    prop.contents.prop.name = "EDITSTYLE";
    if (octGetByName(&cfacet, &prop) == OCT_OK) {
	(void) printf("EDITSTYLE %s;\n", prop.contents.prop.value.string);
    }

    o_init_gen_contents(&cfacet, OCT_TERM_MASK, &ftermgen);
    while (octGenerate(&ftermgen, &fterm) == OCT_OK) {
	iterm.type = OCT_TERM;
	iterm.contents.term.name = fterm.contents.term.name;
	if (octGetByName(&ifacet, &iterm) != OCT_OK) {
	    error("Interface facet out of date");
	}
	prop.type = OCT_PROP;
	prop.contents.prop.name = "DIRECTION";
	if (octGetByName(&iterm, &prop) != OCT_OK) {
	    prop.contents.prop.value.string = "UNSPEC";
	}
	prop2.type = OCT_PROP;
	prop2.contents.prop.name = "TERMTYPE";
	if (octGetByName(&iterm, &prop2) != OCT_OK) {
	    prop2.contents.prop.value.string = "UNSPEC";
	}
	termt = prop2.contents.prop.value.string;
	dir = prop.contents.prop.value.string;
	if (strcmp(termt, "GROUND") == 0) {
	    if (strcmp(dir, "OUTPUT") == 0) {
		(void) printf("OUTGROUND");
		} else {
		(void) printf("GROUND");
	    }
	} else if (strcmp(termt, "SUPPLY") == 0) {
	    if (strcmp(dir, "OUTPUT") == 0) {
		(void) printf("OUTSUPPLY");
		} else {
		(void) printf("SUPPLY");
	    }
	} else if (strcmp(termt, "TRISTATE") == 0) {
	    (void) printf("TRISTATE");
	} else if (strcmp(termt, "CLOCK") == 0) {
	    if (strcmp(dir, "OUTPUT") == 0) {
		(void) printf("OUTCLOCK");
	    } else {
		(void) printf("CLOCK");
	    }
	} else {
	    if (strcmp(dir, "OUTPUT") == 0) {
		(void) printf("OUTPUT");
	    } else if (strcmp(dir, "INOUT") == 0) {
		(void) printf("INOUT");
	    } else {
		(void) printf("INPUT");
	    }
	}

	o_init_gen_containers(&fterm, OCT_NET_MASK, &netgen);
	if (octGenerate(&netgen, &net) == OCT_OK) {
		if ((net.contents.net.name != NIL(char)) &&
                    (strlen(net.contents.net.name) != 0)) {
			(void) printf(" \"%s\" : \"%s\";\n",
                                      fterm.contents.term.name,
				      net.contents.net.name);
		} else {
 		    int32 xid;
		    octExternalId(&net, &xid);
		    (void) printf(" \"%s\" : \"net_%d_%d\";\n",
                                  fterm.contents.term.name, 
				  net.objectId, xid);
		}
	} else {
	    (void) printf(" \"%s\";\n", fterm.contents.term.name);
	}
    }
    (void) printf("\n");

    if (showall) {
        print_insts(&cfacet, 0);
    } else {
	    if (showconnect) {
		bag.type = OCT_BAG;
		bag.contents.bag.name = "CONNECTORS";
		if (octGetByName(&cfacet, &bag) == OCT_OK) {
		    print_insts(&bag, 1 /* connectors */);
		}
	    }
	    bag.type = OCT_BAG;
	    bag.contents.bag.name = "INSTANCES";
	    if (octGetByName(&cfacet, &bag) == OCT_OK) {
		print_insts(&bag, 0 /* instances */);
	    }
    }
    (void) printf("ENDMODEL;\n");

    if (! (showBags || showProps))   return;

    printf( "\n(FACET %s:%s:contents\n", cell, view);

    if (showBags) {
	octObject    bag;
	octGenerator bagGener;
	OH_ASSERT(octInitGenContents(&cfacet, OCT_BAG_MASK, &bagGener));
	while(octGenerate(&bagGener, &bag) == OCT_OK) {
	    if(standardBag( &bag ))   continue;
	    print_bag( &bag );
	}
	octFreeGenerator( &bagGener );
    }

    if (showProps) {
	int          fff;
	octObject    prop, inst, bbag;
	octGenerator propGener, instGener, bagGener;
	OH_ASSERT(octInitGenContents(&cfacet, OCT_PROP_MASK, &propGener));
	while(octGenerate(&propGener, &prop) == OCT_OK) {
	    if(standardProp( &prop ))   continue;
	    print_prop( &prop );
	}
	octFreeGenerator( &propGener );

	OH_ASSERT(octInitGenContents(&cfacet, OCT_INSTANCE_MASK, &instGener));
	while(octGenerate(&instGener, &inst) == OCT_OK) {
	    fff = 0;
	    if( showBags ) {
		OH_ASSERT(octInitGenContainers(&inst, OCT_BAG_MASK, &bagGener));
		while(octGenerate(&bagGener, &bbag) == OCT_OK) {
		    if(! standardBag( &bbag )) {
			fff = 1;
			break;
		    }
		}
	    }
	    if(!  fff ) {
		fff = 0;
		OH_ASSERT(octInitGenContents(&inst, OCT_PROP_MASK, &propGener));
		while(octGenerate(&propGener, &prop) == OCT_OK) {
		    if(! fff) {
			printf("\t(INSTANCE %s \n", inst.contents.instance.name);
			fff = 1;
		    }
		    print_prop( &prop );
		}
		octFreeGenerator( &propGener );
		if( fff ) {
		    printf("\t)\n");
		}
		octFreeGenerator( &propGener );
	    }
	}
    }

    printf( ")\n");
}


static char *trans_name[] = {
    "OCT_NO_TRANSFORM",
    "OCT_MIRROR_X",
    "OCT_MIRROR_Y",
    "OCT_ROT90",
    "OCT_ROT180",
    "OCT_ROT270",
    "OCT_MX_ROT90",
    "OCT_MY_ROT90",
    "OCT_FULL_TRANSFORM"
};

print_insts(bag, isconn)
octObject *bag;
int isconn;
{
    octObject inst, term, net, fterm;
    octGenerator instgen, termgen, netgen, ftermgen;

    o_init_gen_contents(bag, OCT_INSTANCE_MASK, &instgen);
    while (octGenerate(&instgen, &inst) == OCT_OK) {
        if (strcmp(inst.contents.instance.facet, "contents") == 0) {
		(void) printf("INSTANCE \"%s\":%s NAME = \"%s\"",
                              inst.contents.instance.master,
			      inst.contents.instance.view,
                              inst.contents.instance.name);
        } else {
		(void) printf("INSTANCE \"%s\":%s:%s NAME = \"%s\"",
                              inst.contents.instance.master,
			      inst.contents.instance.view,
			      inst.contents.instance.facet,
                              inst.contents.instance.name);
        }
	if (isconn) (void) printf(" CONNECTOR");
	if ((inst.contents.instance.transform.translation.x != 0) ||
	    (inst.contents.instance.transform.translation.y != 0) ||
	    (inst.contents.instance.transform.transformType != OCT_NO_TRANSFORM)) {
	    if (inst.contents.instance.transform.transformType == OCT_NO_TRANSFORM) {
		(void) printf(" [%d, %d]",
		       inst.contents.instance.transform.translation.x,
		       inst.contents.instance.transform.translation.y);
	    } else {
		(void) printf(" [%d, %d, %s]",
		       inst.contents.instance.transform.translation.x,
		       inst.contents.instance.transform.translation.y,
		       trans_name[(int) inst.contents.instance.transform.transformType]);
	    }
	}
	
	(void) printf("\n");
	o_init_gen_contents(&inst, OCT_TERM_MASK, &termgen);
	while (octGenerate(&termgen, &term) == OCT_OK) {
	    o_init_gen_containers(&term, OCT_TERM_MASK, &ftermgen);
	    o_init_gen_containers(&term, OCT_NET_MASK, &netgen);
	    if (octGenerate(&ftermgen, &fterm) == OCT_OK) {
		(void) printf("\"%s\" # \"%s\";\n", term.contents.term.name,
			fterm.contents.term.name);
	    } else if (octGenerate(&netgen, &net) == OCT_OK) {
		if ((net.contents.net.name != NIL(char)) &&
                    (strlen(net.contents.net.name) != 0)) {
		    (void) printf(" \"%s\" : \"%s\";\n",
                                  term.contents.term.name,
                                  net.contents.net.name);
		} else {
		    /* make up a name*/
 		    int32 xid;
		    octExternalId(&net, &xid);
		    (void) printf(" \"%s\" : \"net_%d_%d\";\n",
                                  term.contents.term.name, 
				  net.objectId, xid);
		}
	    } else {
		(void) printf("\"%s\" : UNCONNECTED;\n", term.contents.term.name);
	    }
	}
	(void) printf("\n");
    }
}
