#include "copyright.h"
#include "port.h"
#include "smm.h"
#include "oct.h"
#include "st.h"
#include "vov.h"

error(s)
char *s;
{
    (void) fprintf( stderr, "Error: %s\n", s );
    exit(-1);
}


char *strsav(s)
char *s;
{
    return strcpy(ALLOC(char, strlen(s) + 1), s);
}


o_attach(container, object)
octObject *container;
octObject *object;
{
    if (octAttach(container, object) < OCT_OK) {
	octError("cannot oct attach");
	exit(-1);
    }
}


o_create_bag(container, object, name)
octObject *container;
octObject *object;
char *name;
{
    object->type = OCT_BAG;
    object->contents.bag.name = name;
    if (octCreate(container, object) != OCT_OK) {
	octError("cannot oct create bag");
	exit(-1);
    }
}

static st_table *NetTable = NIL(st_table);

free_net_table()
{
	st_free_table (NetTable);
}


initialize_net_table()
{
	NetTable = st_init_table(strcmp, st_strhash);
}


o_get_net_by_name (facet, net)
octObject *facet, *net;
{
    char *ptr;
    octStatus status;

    net->type = OCT_NET;
    if (st_lookup(NetTable, net->contents.net.name, &ptr)) {
	net->objectId = *(octId *) ptr;
	if (octGetById(net) != OCT_OK) {
	    (void) fprintf(stderr, "bdnet: fatal internal error");
	    exit(-1);
	}
	return(OCT_OK);
    } else {
	if ((status = octGetByName(facet, net)) == OCT_OK) {
	    octId *id = ALLOC(octId, 1);
	    *id = net->objectId;
	    (void) st_insert(NetTable, net->contents.net.name, (char *) id);
	    return(OCT_OK);
	}
	return(status);
    }
}


o_create_net(container, object, name)
octObject *container;
octObject *object;
char *name;
{
    octId *id;

    if (st_lookup(NetTable, name, NIL(char *))) {
    (void) fprintf(stderr, "Trying to create a net that already exists: %s\n",
               name);
	exit(-1);
    }
 
    object->type = OCT_NET;
    object->contents.net.name = strsav (name);
    object->contents.net.width = 1;
    if (octCreate(container, object) != OCT_OK) {
	octError("cannot oct create net");
	exit(-1);
    }
    
    id = ALLOC(octId, 1);
    *id = object->objectId;
    (void) st_insert(NetTable, object->contents.net.name, (char *) id);
}


o_create_box(container, object, llx, lly, urx, ury)
octObject *container;
octObject *object;
octCoord llx, lly, urx, ury;
{
    object->type = OCT_BOX;
    object->contents.box.lowerLeft.x = llx;
    object->contents.box.lowerLeft.y = lly;
    object->contents.box.upperRight.x = urx;
    object->contents.box.upperRight.y = ury;
    if (octCreate(container, object) != OCT_OK) {
	octError("cannot oct create box");
	exit(-1);
    }
}


o_create_term(container, object, name)
octObject *container;
octObject *object;
char *name;
{
    object->type = OCT_TERM;
    object->contents.term.name = name;
    object->contents.term.width = 1;
    object->contents.term.instanceId = oct_null_id;
    if (octCreate(container, object) != OCT_OK) {
	octError("cannot oct create term");
	exit(-1);
    }
}


/*
 * create string property
 */
o_create_sprop(container, object, name, string)
octObject *container;
octObject *object;
char *name;
char *string;
{
    object->type = OCT_PROP;
    object->contents.prop.name = name;
    object->contents.prop.type = OCT_STRING;
    object->contents.prop.value.string = string;
    if (octCreate(container, object) != OCT_OK) {
	octError(name);
	exit(-1);
    }
}


o_delete(object)
octObject *object;
{
    if (octDelete(object) != OCT_OK) {
	octError("cannot oct delete");
	exit(-1);
    }
}


o_get_by_id(object, id)
octObject *object;
octId id;
{
    object->objectId = id;
    if ( octGetById(object) != OCT_OK ) {
	octError("Can't get by id");
	exit(-1);
	/*NOTREACHED*/
    }
}

#ifndef lint
o_get_by_name_prop(container, object, name)
octObject *container;
octObject *object;
char *name;
{
    object->type = OCT_PROP;
    object->contents.prop.name = name;
    if (octGetByName(container, object) != OCT_OK) {
	octError(name);
	exit(-1);
    }
}
#endif

o_get_by_name_term(container, object, name)
octObject *container;
octObject *object;
char *name;
{
    object->type = OCT_TERM;
    object->contents.term.name = name;
    if (octGetByName(container, object) != OCT_OK) {
	octError(name);
	exit(-1);
    }
}


o_init_gen_containers(object, mask, generator)
octObject *object;
octObjectMask mask;
octGenerator *generator;
{
    if (octInitGenContainers(object, mask, generator) != OCT_OK) {
	octError("cannot oct init gen contents");
	exit(-1);
    }
}


o_gen_first_content(container, mask, result)
octObject *container;
octObjectMask mask;
octObject *result;
{
    if (octGenFirstContent(container, mask, result) != OCT_OK) {
	octError("cannot oct gen first content");
	exit(-1);
    }
}

o_init_gen_contents(container, mask, generator)
octObject *container;
octObjectMask mask;
octGenerator *generator;
{
    if (octInitGenContents(container, mask, generator) != OCT_OK) {
	octError("cannot oct init gen contents");
	exit(-1);
    }
}


o_open_facet(ocell, cell, view, facet, mode)
octObject *ocell;
char *cell, *view, *facet, *mode;
{
    ocell->type = OCT_FACET;
    ocell->contents.facet.cell = cell;
    ocell->contents.facet.view = view;
    ocell->contents.facet.facet = facet;
    ocell->contents.facet.version = OCT_CURRENT_VERSION;
    ocell->contents.facet.mode = mode;

    switch ( mode[0] ) {
    case 'w':
	VOVoutputFacet( ocell );
	break;
    case 'r':
	VOVinputFacet( ocell );
	break;
    case 'a':
	VOVinputFacet( ocell );
	VOVoutputFacet( ocell );
	break;
    defaults:
	error( "bad mode for octOpenFacet()" );
	break;
    }
    if (octOpenFacet(ocell) < OCT_OK) {
	octError("cannot oct open facet");
	exit(-1);
    }
}


#ifdef FOO
/*
 * adds to the end of an RPC arg list
 */
add_RPCArg( list, type, length, data )
lsList *list;
long type;
long length;
Pointer data;
{
    struct RPCArg *arg;

    arg = MALLOC(struct RPCArg);
    arg->argType = type;
    arg->length = length;
    switch ( type ) {
	case 0:
	case 1:
	    arg->argData.points = (struct octPoint *) data;
	    break;

	case 4:
	    arg->argData.string = (char *) data;
	    break;

	default:
	    error("bad add_RPCArg");
    }
    if ( *list == LS_NIL ) {
	*list = lsCreate();
    }
    if ( lsNewEnd( *list, arg, 0 ) != LS_OK ) {
	error("ls fail in add_RPCArg");
    }
}


/*
 * free memory in a ls arg
 */
lsDelFunc( arg )
struct RPCArg *arg;
{
    switch ( arg->argType ) {
	case 0:
	case 1:
	    FREE(arg->argData.points);
	    break;

	case 4:
	    FREE(arg->argData.string);
	    break;

	default:
	    error("bad add_RPCArg");
    }
    FREE(arg);
}


/*
 * dummy routine for ls delete
 */
nullFuncLs( arg )
struct RPCarg *arg;
{
}


/*
 * dummy routine for t23 delete
 */
nullFunct23( arg )
char *arg;
{
}
#endif
