#include "copyright.h"
#include "port.h"
#include "smm.h"
#include "oct.h"
#include "st.h"
#include "yylex.h"
#include "errtrap.h"

#define upper(c)        (islower(c) ? toupper(c) : (c))

extern octObject cfacet;
extern octObject ifacet;
extern int topdown;
extern int noVulcan;		/*  */
static octObject tdfacet;
static int td_created = 0;
static char *savetech;

char *strsav();

int upper_strhash(key, mod)
char *key;
int mod;
{
    register int i;
    register unsigned word;

    /* Compute a hash function over the name */
    word = 0;
    for(i = 0; key[i] != '\0'; i++)
	word = (word << 1) ^ upper(key[i]);
    return word % mod;
}


int upper_strcmp(s1, s2)
register char *s1, *s2;
{
    while (upper(*s1) == upper(*s2)) {
	if (*s1 == '\0') {
	    return 0;
	}
	s1++;
	s2++;
    }
    return upper(*s1) - upper(*s2);
}


read_constant(text, base_string, value, width)
register char *text;
char *base_string;
unsigned *value;
int *width;
{
    register unsigned int len = 0, c, base;

    base = atoi(base_string);

    *value = 0;
    while ((c = *text++) != '\0') {
	if (c >= '0' && c <= '9') c -= '0'; 
	else if (c >= 'A' && c <= 'F') c = c - 'A' + 10;
	else if (c >= 'a' && c <= 'f') c = c - 'a' + 10;
	if (c >= base) {
	    yyerror("badly formed constant");
	}
	if (*value > (0xffffffff - c)/base) {
	    yyerror("Constant must be 32 bits or less");
	}
	*value = *value * base + c;
	len++;
    }
    switch(base) {
	case 2: 
	    *width = len;
	    break;
	case 8: 
	    *width = len * 3; 
	    break;
	case 10: 
	    *width = const_width(*value);
	    break;
	case 16: 
	    *width = len * 4; 
	    break;
	default:
	    yyerror("bad base for constant");
    }
    /* Allow for 11 digit octal where msd < 4 */
    if (*width > 32) {
	*width = 32;
    }
}

int const_width(x)
unsigned x;
{
    if (x == 0) {
	return 1;
    } else if (x > 0x7fffffff) {
	return 32;
    } else {
	return log((double) x+.5) / log(2.0) + 1;
    }
}


ident_free(list)
ident *list;
{
    ident *temp;

    while (list != NIL(ident)) {
	free_expr(list->hbit);
	free_expr(list->lbit);
	switch (list->type) {
	    case EXPR:
		free_expr(list->u.value);
		break;
	    case ID:
		FREE(list->u.name);
		break;
	}
	temp = list->next;
	FREE(list);
	list = temp;
    }
}


ident_add(list, new)
ident **list;
ident *new;
{
    ident *i;

    if (*list == NIL(ident)) {
	*list = new;
    } else {
	for (i = *list; i->next != NIL(ident); i = i->next) {
	}
	i->next = new;
    }
}


static char *instName;
static octObject instanceBag;
static octObject connectBag;
static char *currentTermDir;
static char viewtype[128];
static char editstyle[128];

oct_init(model_name, view)
char *model_name;
char *view;
{
    o_open_facet(&cfacet, model_name, view, "contents", "w");

    /* get cfacet by id so I can free up the strings */
    o_get_by_id(&cfacet, cfacet.objectId);

    /* set up special bags */
    o_create_bag(&cfacet, &instanceBag, "INSTANCES");
    o_create_bag(&cfacet, &connectBag, "CONNECTORS");
    (void) strcpy(viewtype, "SYMBOLIC");
    (void) strcpy(editstyle, "SYMBOLIC");

	initialize_net_table ();
}



/*
 * deal with all possible switches on instances
 */
static term *thePlacex;		/* place for instance */
static term *thePlacey;
static int promote;
static int connector;
static octTransformType theTransform;

/* set instance switches to defaults */
default_parms()
{
    make_const(&thePlacex, 0);
    make_const(&thePlacey, 0);
    theTransform = OCT_NO_TRANSFORM;
	instName = "";
    promote = 0;
    connector = 0;
}

set_name(name)
char *name;
{
    instName = name;
}

set_connector()
{
    connector = 1;
}

set_promote()
{
    promote = 1;
}

set_place(x, y, tr)
term *x,*y;
int tr;
{
    thePlacex = x;
    thePlacey = y;
    theTransform = (octTransformType) tr;
}

int make_inst_uniq = 0;
octObject arrayed_insts;

/*
 * Instantiate a submodel
 */
make_inst(master, view, facet)
char *master;
char *view;
char *facet;
{
    char buf[1024];
    char fullName[1024];
    octObject currentInst;
    octObject prop;
    octObject rbag, cbag;
    char instTag[100];

    o_create_bag(&cfacet, &arrayed_insts, "AI");
    if (*instName == '\0') {
	instName = strsav(master);
    }
    currentInst.type = OCT_INSTANCE;
    currentInst.contents.instance.version = OCT_CURRENT_VERSION;
    currentInst.contents.instance.transform.transformType = OCT_NO_TRANSFORM;
    init_gen_array(instTag, 1);
    do {
	(void) strcpy(fullName, instName);
	(void) strcat(fullName, instTag);
	currentInst.contents.instance.name = fullName;
	while (octGetByName(&cfacet, &currentInst) == OCT_OK) {
	    (void) sprintf(buf, "_%d", make_inst_uniq++);
	    (void) strcat(fullName, buf);
	    currentInst.contents.instance.name = fullName;
	}
	currentInst.contents.instance.master = master;
	currentInst.contents.instance.view = view;
	currentInst.contents.instance.facet = facet;
	currentInst.contents.instance.transform.translation.x =
		expr2number(thePlacex);
	currentInst.contents.instance.transform.translation.y =
		expr2number(thePlacey);
	currentInst.contents.instance.transform.transformType = theTransform;
	if (octCreate(&cfacet, &currentInst) != OCT_OK) {
		if (topdown) {
		    o_open_facet(&tdfacet, master, view, "contents", "w");

		    /* get cfacet by id so I can free up the strings */
		    o_get_by_id(&tdfacet, tdfacet.objectId);
 
		    o_create_sprop(&tdfacet, &prop, "TECHNOLOGY", savetech);
		    o_create_sprop(&tdfacet, &prop, "VIEWTYPE", viewtype);
		    o_create_sprop(&tdfacet, &prop, "EDITSTYLE", editstyle);

		    if (octCreate(&cfacet, &currentInst) != OCT_OK) {
	    		(void) sprintf(buf, "Cannot create master \"%s\"", master);
	    		yysoft(buf);
	    		FREE(instName);
	    		instName = strsav(""); /* flag bad inst */
			o_delete(&arrayed_insts);
	    		return;
		    }

		    td_created = 1;
		} else {
		    errRaise( "bdnet", 1, "Instance \"%s\" not found", master);

		    /* NOT REACHED */
		    yysoft(buf);
		    FREE(instName);
		    instName = strsav(""); /* flag bad inst */
		    o_delete(&arrayed_insts);
		    return;
		}
	}
	if (connector) {
	    o_attach(&connectBag, &currentInst);
	} else {
	    o_attach(&instanceBag, &currentInst);
	}
	get_rc(&rbag, &cbag);
	if (!octIdIsNull(rbag.objectId)) {
	    o_attach(&rbag, &currentInst);
	}
	if (!octIdIsNull(cbag.objectId)) {
	    o_attach(&cbag, &currentInst);
	}
	o_attach(&arrayed_insts, &currentInst);
    } while (gen_array(instTag));
}


/*
 * do post processing on instance (PROMOTEs, check for unconnected)
 */
finish_inst()
{
    octObject interfacet, save_cfacet;
    octObject currentInst;
    octObject aterm, iterm, prop, net, aterm2;
    octObject jbag;
    octGenerator baggen, itermgen;
    octGenerator termgen, netgen, instgen;
    char buf[1024];
    int bit;

	if (td_created) {
		save_cfacet = cfacet;
		cfacet = tdfacet;
		oct_close (/* end_cfacet */ 0);
		cfacet = save_cfacet;
	}

    /* if error in inst declaration then bomb out */
    if (*instName == '\0') {
	FREE(instName);
	return;
    }
 
    o_init_gen_contents(&arrayed_insts, OCT_INSTANCE_MASK, &instgen);
    if (octGenerate(&instgen, &currentInst) != OCT_OK) {
	error("finist_inst");
    }
    o_open_facet(&interfacet, currentInst.contents.instance.master,
	    currentInst.contents.instance.view, "interface", "r");

    o_init_gen_contents(&arrayed_insts, OCT_INSTANCE_MASK, &instgen);
    while (octGenerate(&instgen, &currentInst) == OCT_OK) {
	o_init_gen_contents(&currentInst, OCT_TERM_MASK, &termgen);
	while (octGenerate(&termgen, &aterm) == OCT_OK) {
		if (octIdIsNull(aterm.contents.term.instanceId)) {
			/* formal terminal: do not check... */
			continue;
		}
	    o_init_gen_containers(&aterm, OCT_NET_MASK, &netgen);
	    if (octGenerate(&netgen, &net) != OCT_OK) {
		prop.type = OCT_PROP;
		prop.contents.prop.name = "UNCONNECTED";
		if (octGetByName(&aterm, &prop) == OCT_OK) {
		    o_delete(&prop);
		    continue;
		}
		iterm.type = OCT_TERM;
		iterm.contents.term.name = aterm.contents.term.name;
		if (octGetByName(&interfacet, &iterm) != OCT_OK) {
		    (void) sprintf(buf, "Interface facet of %s out of date",
			    currentInst.contents.instance.master);
		    yyerror(buf);
		}
		jbag.objectId = oct_null_id;
		jbag.contents.bag.name = "";
		o_init_gen_containers(&iterm, OCT_BAG_MASK, &baggen);
		while (octGenerate(&baggen, &jbag) == OCT_OK) {
		    if (strcmp(jbag.contents.bag.name, "JOINED") == 0) {
			break;
		    }
		}
		if (strcmp(jbag.contents.bag.name, "JOINED") == 0) {
		    o_init_gen_contents(&jbag, OCT_TERM_MASK, &itermgen);
		    while (octGenerate(&itermgen, &iterm) == OCT_OK) {
			o_get_by_name_term(&currentInst, &aterm2,
				iterm.contents.term.name);
			o_init_gen_containers(&aterm2, OCT_NET_MASK, &netgen);
			if (octGenerate(&netgen, &net) == OCT_OK) {
			    goto cont;
			}
		    }
		}
		if (promote) {
		    if (strchr(aterm.contents.term.name, '<') ==
			    (char *) 0) {
			create_net(&net, aterm.contents.term.name, 0, 0);
			o_attach(&net, &aterm);
		    } else {
			(void) sscanf(aterm.contents.term.name,
				"%[^<]<%d>", buf, &bit);
			create_net(&net, buf, bit, 1);
			o_attach(&net, &aterm);
		    }
		} else {
		    (void) sprintf(buf,
			    "Terminal \"%s\" of instance \"%s\" is not connected",
			    aterm.contents.term.name,
			    currentInst.contents.instance.name);
		    yysoft(buf);

		}
	    }
	cont: ;
	}
    }
    FREE(instName);
    free_expr(thePlacex);
    free_expr(thePlacey);
    o_delete(&arrayed_insts);
    /* ifacet is kept open for efficiency */
}


set_technology(tech)
char *tech;
{
    octObject prop;
    o_create_sprop(&cfacet, &prop, "TECHNOLOGY", tech);
	savetech = strsav (tech);
}

set_viewtype(view)
char *view;
{
    char *p;

    (void) strcpy(viewtype, view);
    for (p = viewtype; *p != '\0'; p++) {
	*p = upper(*p);
    }
}

set_editstyle(edit)
char *edit;
{
    char *p;

    (void) strcpy(editstyle, edit);
    for (p = editstyle; *p != '\0'; p++) {
	*p = upper(*p);
    }
}

make_view_prop()
{
    octObject prop;

    o_create_sprop(&cfacet, &prop, "VIEWTYPE", viewtype);
    o_create_sprop(&cfacet, &prop, "EDITSTYLE", editstyle);
}

set_term_dir(dir)
char *dir;
{
    currentTermDir = dir;
}


/*
 * Create an fterm and attach it to a net.
 * rhs will specify a net connection.  If rhs == NIL then
 * net name = term name
 */
make_formal(info, rhs, unconnected)
ident *info, *rhs;
int unconnected;
{
    octObject fterm;
    octObject prop;
    octObject net;
    char buf[1024];
    int give_status;
    int i, hbit, lbit, isrhs;

    isrhs = (rhs != NIL(ident));
    if (isrhs) {
      init_gen_rhs(rhs);
    }

    give_status = give_hl(info, &hbit, &lbit);
    for (i=hbit; i>=lbit; i--) {
	if (give_status) {
	    (void) sprintf(buf, "%s<%d>", info->u.name, i);
	} else {
	    (void) sprintf(buf, "%s", info->u.name);
	}

	/* check whether this port exists already */
	fterm.contents.term.name = buf;
	fterm.type = OCT_TERM;
	if (octGetByName(&cfacet, &fterm) == OCT_OK) {
	    yyerror("Port defined twice");
	}

	o_create_term(&cfacet, &fterm, buf);
	/* store away direction for safe keeping */
	if ((strcmp(currentTermDir, "INPUT") == 0) ||
		(strcmp(currentTermDir, "CLOCK") == 0)) {
	    o_create_sprop(&fterm, &prop, "DIRECTION", "INPUT");
	} else if ((strcmp(currentTermDir, "SUPPLY") == 0) ||
		(strcmp(currentTermDir, "GROUND") == 0)) {
	    o_create_sprop(&fterm, &prop, "DIRECTION", "INPUT");
	} else if ((strcmp(currentTermDir, "OUTSUPPLY") == 0) ||
		(strcmp(currentTermDir, "OUTGROUND") == 0)) {
	    o_create_sprop(&fterm, &prop, "DIRECTION", "OUTPUT");
	} else if (strcmp(currentTermDir, "INOUT") == 0) {
	    o_create_sprop(&fterm, &prop, "DIRECTION", "INOUT");
	} else {
	    o_create_sprop(&fterm, &prop, "DIRECTION", "OUTPUT");
	}

	if ((strcmp(currentTermDir, "CLOCK") == 0) ||
		(strcmp(currentTermDir, "OUTCLOCK") == 0)) {
	    o_create_sprop(&fterm, &prop, "TERMTYPE", "CLOCK");
	} else if (strcmp(currentTermDir, "GROUND") == 0 ||
		strcmp(currentTermDir, "OUTGROUND") == 0) {
	    o_create_sprop(&fterm, &prop, "TERMTYPE", "GROUND");
	} else if (strcmp(currentTermDir, "SUPPLY") == 0 ||
		strcmp(currentTermDir, "OUTSUPPLY") == 0) {
	    o_create_sprop(&fterm, &prop, "TERMTYPE", "SUPPLY");
	} else if (strcmp(currentTermDir, "TRISTATE") == 0) {
	    o_create_sprop(&fterm, &prop, "TERMTYPE", "TRISTATE");
	} else {
	    o_create_sprop(&fterm, &prop, "TERMTYPE", "SIGNAL");
	}

	if (! unconnected) {
		if (isrhs) {
	   	 if (!gen_rhs(&net)) {
			yyerror("Connection not wide enough");
	   	 }
		} else {
		    o_create_net(&cfacet, &net, buf);
			if (strcmp(currentTermDir, "GROUND") == 0 ||
				strcmp(currentTermDir, "OUTGROUND") == 0) {
				o_create_sprop(&net, &prop, "NETTYPE", "GROUND");
			} else if (strcmp(currentTermDir, "SUPPLY") == 0 ||
				strcmp(currentTermDir, "OUTSUPPLY") == 0) {
				o_create_sprop(&net, &prop, "NETTYPE", "SUPPLY");
			} 
		}
		o_attach(&net, &fterm);
	      }
      }
}


/*
 * give hbit and lbit numbers.  Return 0 if hbit=NIL
 */
int give_hl(info, hbit, lbit)
ident *info;
int *hbit, *lbit;
{

    if (info->hbit == NIL(term)) {
	*hbit = *lbit = 0;
	return 0;
    } else {
	*hbit = expr2number(info->hbit);
    }
    if (info->lbit == NIL(term)) {
	*lbit = *hbit;
    } else {
	*lbit = expr2number(info->lbit);
    }
    return 1;
}


static ident *rhsPlace;			/* term of rhs */
static int rhsCurrentBit;		/* bit of rhs to give next */
static int rhsLastBit;			/* last bit of this term */
static int rhsHasSubscr;		/* current is subsrcipted */

init_gen_rhs(rhs)
ident *rhs;
{
    rhsPlace = rhs;
    rhsHasSubscr = give_hl(rhs, &rhsCurrentBit, &rhsLastBit);
}

/*
 * return next net of rhs in "net", returns 0 if no more nets.
 */
int gen_rhs(net)
octObject *net;
{
    if (rhsPlace == NIL(ident)) return 0;
    if (rhsHasSubscr) {
	create_net(net, rhsPlace->u.name, rhsCurrentBit--, 1);
    } else {
	create_net(net, rhsPlace->u.name, 0, 0);
	rhsCurrentBit--;
    }
    if (rhsCurrentBit < rhsLastBit) {
	rhsPlace = rhsPlace->next;
	if (rhsPlace != NIL(ident)) {
	    rhsHasSubscr = give_hl(rhsPlace, &rhsCurrentBit, &rhsLastBit);
	}
    }
    return 1;
}

/*
 * generate fterms based on the rhs
 */
int gen_rhs_fterms(fterm)
octObject *fterm;
{
    char buffer[256], buf[256];

    if (rhsPlace == NIL(ident)) return 0;
    if (rhsHasSubscr) {
	(void) sprintf(buf, "%s<%d>", rhsPlace->u.name, rhsCurrentBit--);
    } else {
	(void) sprintf(buf, "%s", rhsPlace->u.name);
	rhsCurrentBit--;
    }
    fterm->type = OCT_TERM;
    fterm->contents.term.name = buf;
    if (octGetByName(&cfacet, fterm) != OCT_OK) {
	(void) sprintf(buffer, "Formal terminal %s does not exist", buf);
	yysoft(buffer);
	return 0;
    }
    if (rhsCurrentBit < rhsLastBit) {
	rhsPlace = rhsPlace->next;
	if (rhsPlace != NIL(ident)) {
	    rhsHasSubscr =  give_hl(rhsPlace, &rhsCurrentBit, &rhsLastBit);
	}
    }
    return 1;
}


/*
 * gets or creates a net on the fly
 */
create_net(net, name, bit, usebit)
octObject *net;
char *name;
int bit;
int usebit;		/* no subscript if false */
{
    char buf[1024];

    if (usebit) {
	(void) sprintf(buf, "%s<%d>", name, bit);
    } else {
	(void) sprintf(buf, "%s", name);
    }
    net->type = OCT_NET;
    net->contents.net.name = buf;
    if (o_get_net_by_name(&cfacet, net) != OCT_OK) {
    	o_create_net(&cfacet, net, buf);
    }
}


/*
 * Core of make_connect  Return false if problem
 */
int m_connect(net, aterm)
octObject *net, *aterm;
{
    octObject inst, tempnet;
    octGenerator gen;
    char buf[256];

    /* Can't attach to connector aterm */
    inst.objectId = aterm->contents.term.instanceId;
    if (octIsAttached(&connectBag, &inst) == OCT_OK) {
	yysoft("Illegal connection to CONNECTOR terminal");
	return 0;
    }
    /* Check to see if the term is already connected */
    o_init_gen_containers(aterm, OCT_NET_MASK, &gen);
    if (octGenerate(&gen, &tempnet) == OCT_OK) {
	(void) sprintf(buf, "Actual terminal \"%s\" connected twice",
		aterm->contents.term.name);
	yysoft(buf);
	return 0;
    }
    o_attach(net, aterm);
    return 1;
}

/*
 * make connections to current inst
 */
make_connect(termi, rhs)
ident *termi, *rhs;
{
    octObject net, last_net, aterm;
    octObject currentInst;
    octGenerator instgen;
    int hbitt, lbitt;		/* hbit and lbit for term */
    char buf[256];
    char instTag[100];
    int i;

    /* if error in inst declaration then bomb out */
    if (*instName == '\0') return;

    init_gen_array(instTag, 0);
    o_init_gen_contents(&arrayed_insts, OCT_INSTANCE_MASK, &instgen);
    while (octGenerate(&instgen, &currentInst) == OCT_OK) {
	init_gen_rhs(rhs);
	if (!give_hl(termi, &hbitt, &lbitt)) {
	    aterm.type = OCT_TERM;
	    aterm.contents.term.name = termi->u.name;

	    /* check to see if this is a nonsubscripted term
	     * or is the this term<0> */
	    if (td_created) {
			o_create_term (&tdfacet, &aterm, termi->u.name);
			/* no longer necessary ??
			if (octCreate(&currentInst, &aterm) != OCT_OK) {
				(void) sprintf(buf, "Cannot create actual terminal \"%s\"",
					termi->u.name);
				yysoft(buf);
				return;
	    	}
			*/
		}
		if (octGetByName(&currentInst, &aterm) == OCT_OK) {
		/* This is a non-subscripted term */
		/* get net to connect */
		(void) gen_rhs(&net);
		if (!m_connect(&net, &aterm)) return;
		(void) gen_array(instTag);
		continue;
	    }
	}

	for (i=hbitt; i>=lbitt; i--) {
	    /* try to get net to connect */
	    last_net = net;
	    if (!gen_rhs(&net)) {
	      /* JCW: connect remaining terms to same net */
	      net = last_net;
	    }

	    /* get the actual term */
	    aterm.type = OCT_TERM;
	    (void) sprintf(buf, "%s<%d>", termi->u.name, i);
	    aterm.contents.term.name = buf;
	    if (td_created) {
			o_create_term (&tdfacet, &aterm, buf);
			/* no longer necessary ??
			if (octCreate(&currentInst, &aterm) != OCT_OK) {
				(void) sprintf(buf, "Cannot create actual terminal \"%s\"",
					termi->u.name);
				yysoft(buf);
				return;
	    	}
			 */
		}
		if (octGetByName(&currentInst, &aterm) == OCT_NOT_FOUND) {
		(void) sprintf(buf, "Terminal \"%s\" not part of instance \"%s\"",
			termi->u.name, currentInst.contents.instance.name);
		yysoft(buf);
		return;
	    }
	    if (!m_connect(&net, &aterm)) return;
	}
	(void) gen_array(instTag);
    }
    if (gen_rhs(&net)) {
	(void) sprintf(buf, "Connection to \"%s\" is too wide", termi->u.name);
	yysoft(buf);
	return;
    }
}


/*
 * make unconnections
 */
make_unconnect(termi)
ident *termi;
{
    octObject aterm, uprop;
    octObject currentInst;
    octGenerator instgen;
    int hbitt, lbitt;		/* hbit and lbit for term */
    char buf[1024];
    int i;

    uprop.type = OCT_PROP;
    uprop.contents.prop.name = "UNCONNECTED";
    uprop.contents.prop.type = OCT_NULL;

    /* if error in inst declaration then bomb out */
    if (*instName == '\0') return;

    o_init_gen_contents(&arrayed_insts, OCT_INSTANCE_MASK, &instgen);
    while (octGenerate(&instgen, &currentInst) == OCT_OK) {
	if (!give_hl(termi, &hbitt, &lbitt)) {
	    aterm.type = OCT_TERM;
	    aterm.contents.term.name = termi->u.name;
	    if (octGetByName(&currentInst, &aterm) == OCT_OK) {
		(void) octCreate(&aterm, &uprop);
		continue;
	    }
	}
	for (i=hbitt; i>=lbitt; i--) {

	    /* get the actual term */
	    aterm.type = OCT_TERM;
	    (void) sprintf(buf, "%s<%d>", termi->u.name, i);
	    aterm.contents.term.name = buf;
	    if (octGetByName(&currentInst, &aterm) == OCT_NOT_FOUND) {
		(void) sprintf(buf, "Terminal \"%s\" not part of instance \"%s\"",
			termi->u.name, currentInst.contents.instance.name);
		yysoft(buf);
		return;
	    }
	    (void) octCreate(&aterm, &uprop);
	}
    }
}

/*
 * Core of make_promote (return false if problem)
 */
int m_promote(fterm, aterm)
octObject *fterm, *aterm;
{
    octObject net, inst;
    octGenerator gen;
    char buf[256];

    /* Check to see if the term is already promoted */
    o_init_gen_contents(fterm, OCT_TERM_MASK, &gen);
    if (octGenerate(&gen, aterm) == OCT_OK) {
	(void) sprintf(buf, "Formal terminal \"%s\" implemented twice",
		fterm->contents.term.name);
	yysoft(buf);
	return 0;
    }
    o_init_gen_containers(aterm, OCT_TERM_MASK, &gen);
    if (octGenerate(&gen, fterm) == OCT_OK) {
	(void) sprintf(buf, "Actual Terminal \"%s\" promoted twice",
		aterm->contents.term.name);
	yysoft(buf);
	return 0;
    }
    o_attach(fterm, aterm);

    /* attach aterm to fterm's net if not CONNECTOR */
    inst.objectId = aterm->contents.term.instanceId;
    if (octIsAttached(&connectBag, &inst) == OCT_OK) {
	return 1;
    }
    o_init_gen_containers(aterm, OCT_NET_MASK, &gen);
    if (octGenerate(&gen, &net) == OCT_OK) {
	(void) sprintf(buf, "Actual Terminal \"%s\" connected twice",
		aterm->contents.term.name);
	yysoft(buf);
	return 0;
    }
    o_init_gen_containers(fterm, OCT_NET_MASK, &gen);
    if (octGenerate(&gen, &net) != OCT_OK) {
	error("internal: m_promote");
    }
    o_attach(&net, aterm);
    return 1;
}


/*
 * promote given aterms to given fterms (rhs is fterms)
 */
make_promote(termi, rhs)
ident *termi, *rhs;
{
    octObject fterm, aterm;
    octObject currentInst;
    octGenerator instgen;
    int hbitt, lbitt;		/* hbit and lbit for term */
    char buf[256];
    char instTag[100];
    int i;

    /* if error in inst declaration then bomb out */
    if (*instName == '\0') return;

    init_gen_array(instTag, 0);
    o_init_gen_contents(&arrayed_insts, OCT_INSTANCE_MASK, &instgen);
    while (octGenerate(&instgen, &currentInst) == OCT_OK) {
	init_gen_rhs(rhs);
	if (!give_hl(termi, &hbitt, &lbitt)) {
	    aterm.type = OCT_TERM;
	    aterm.contents.term.name = termi->u.name;

	    /* check to see if this is a nonsubscripted term
	     * or is the this term<0> */
	    if (octGetByName(&currentInst, &aterm) == OCT_OK) {
		/* This is a non-subscripted term */
		/* get net to connect */
		if (!gen_rhs_fterms(&fterm)) {
		    return;
		}
		if (!m_promote(&fterm, &aterm)) {
		    return;
		}
		(void) gen_array(instTag);
		continue;
	    }
	}

	for (i=hbitt; i>=lbitt; i--) {
	    /* try to get fterm to connect */
	    if (!gen_rhs_fterms(&fterm)) {
		(void) sprintf(buf, "Promotion of \"%s\" is not wide enough",
			termi->u.name);
		yysoft(buf);
		return;
	    }

	    /* get the actual term */
	    aterm.type = OCT_TERM;
	    (void) sprintf(buf, "%s<%d>", termi->u.name, i);
	    aterm.contents.term.name = buf;
	    if (octGetByName(&currentInst, &aterm) == OCT_NOT_FOUND) {
		(void) sprintf(buf, "Terminal \"%s\" not part of instance \"%s\"",
			termi->u.name, currentInst.contents.instance.name);
		yysoft(buf);
		return;
	    }

	    if (!m_promote(&fterm, &aterm)) {
		return;
	    }
	}
	(void) gen_array(instTag);
    }
    if (gen_rhs_fterms(&fterm)) {
	(void) sprintf(buf, "Promotion of \"%s\" is too wide", termi->u.name);
	yysoft(buf);
	return;
    }
}


/*
 * Move fterm props to ifacet, where they belong.
 */
fix_terms()
{
    octObject fterm, iterm;
    octObject prop, backp;
    octGenerator propgen, itermgen;

    o_init_gen_contents(&ifacet, OCT_TERM_MASK, &itermgen);
    while (octGenerate(&itermgen, &iterm) == OCT_OK) {
	o_get_by_name_term(&cfacet, &fterm, iterm.contents.term.name);
	/* move term props to ifacet */
	o_init_gen_contents(&fterm, OCT_PROP_MASK, &propgen);
	while (octGenerate(&propgen, &prop) == OCT_OK) {
	    backp.objectId = prop.objectId;
	    if (octCreate(&iterm, &prop) < OCT_OK) {
		octError("problems creating property on an interface terminal");
	    }
	    o_delete(&backp);
	}
    }
}

/*
 * delete unneeded nets.
 * If there is less than two terms (regardless of type)
 * attached to net, net is not needed.
 */
delete_nets()
{
    octObject theTerm, net;
    octGenerator termgen, netgen;
    char buf[1024];

    o_init_gen_contents(&cfacet, OCT_NET_MASK, &netgen);
    while (octGenerate(&netgen, &net) == OCT_OK) {
	o_init_gen_contents(&net, OCT_TERM_MASK, &termgen);
	(void) octGenerate(&termgen, &theTerm);
	if (octGenerate(&termgen, &theTerm) != OCT_OK) {
	    (void) sprintf(buf,
		"Warning: net %s has single connection : net deleted\n",
		net.contents.net.name);
		yysoft(buf);
	    o_delete(&net);
	}
    }
}


/*
 * clean up everything and close facets.  Run vulcan
 */
oct_close(end_cfacet)
int end_cfacet;
{
    char buffer[128];

    if (end_cfacet) {
	delete_nets();
	free_net_table ();
    }

    /* make interface facet */
    o_open_facet(&ifacet, cfacet.contents.facet.cell,
	    cfacet.contents.facet.view, "interface", "w");
    fix_terms();

    if (octCloseFacet(&cfacet) < OCT_OK) {
	octError("cfacet");
	exit(-1);
    }
    if (octCloseFacet(&ifacet) < OCT_OK) {
	octError("ifacet");
	exit(-1);
    }


    if ( !noVulcan ) {
	(void) sprintf(buffer, "vulcan %s:%s",
		       cfacet.contents.facet.cell,
		       cfacet.contents.facet.view);
	(void) system(buffer);
    }
}

/*
 * stuff to attach misc oct junk
 */

static octObject container[50];   /* can stack objects 50 levels deep */
static int level;		/* level of container */
static octObject lastInst;	/* last instance touched */
static st_table *aliases;

extra_open_facet(cell, view, facet)
char *cell, *view, *facet;
{
    open_old_facet(&cfacet, cell, view, facet, "a");
    level = 0;
    container[level] = cfacet;
    lastInst.objectId = oct_null_id;
    aliases = st_init_table(strcmp, st_strhash);
    initialize_net_table();
}

extra_close_facet()
{
    if (octCloseFacet(&cfacet) < OCT_OK) {
	octError("");
	exit(-1);
    }
    st_free_table(aliases);
    free_net_table();
}


pop_level()
{
    if (--level < 0) {
	yyerror("Too many close parens");
    }
}

make_alias(name)
char *name;
{
    octId *id = ALLOC(octId, 1);
    
    *id = container[level].objectId;
    (void) st_insert(aliases, name, (char *) id);
}

grab_alias(name)
char *name;
{
    octObject object;
    octId *id;
    char *ptr;
    char buffer[256];

    if (!st_lookup(aliases, name, &ptr)) {
	(void) sprintf(buffer, "Alias %s not found", name);
	yyerror(buffer);
    }
    o_get_by_id(&object, *(octId *) ptr);
    o_attach(&container[level], &object);

    container[++level] = object;
}

make_sprop(name, value, create)
char *name, *value;
int create;
{
    octObject prop;

    prop.type = OCT_PROP;
    prop.contents.prop.name = name;
    prop.contents.prop.type = OCT_STRING;
    prop.contents.prop.value.string = value;
    if (create) {
	(void) octCreate(&container[level], &prop);
    } else {
	(void) octCreateOrModify(&container[level], &prop);
    }
    container[++level] = prop;
}

make_iprop(name, value, create)
char *name;
int value;
int create;
{
    octObject prop;

    prop.type = OCT_PROP;
    prop.contents.prop.name = name;
    prop.contents.prop.type = OCT_INTEGER;
    prop.contents.prop.value.integer = value;
    if (create) {
	(void) octCreate(&container[level], &prop);
    } else {
	(void) octCreateOrModify(&container[level], &prop);
    }
    container[++level] = prop;
}


make_rprop(name, value, create)
char *name, *value;
int create;
{
    octObject prop;

    prop.type = OCT_PROP;
    prop.contents.prop.name = name;
    prop.contents.prop.type = OCT_REAL;
    prop.contents.prop.value.real = atof(value);
    if (create) {
	(void) octCreate(&container[level], &prop);
    } else {
	(void) octCreateOrModify(&container[level], &prop);
    }
    container[++level] = prop;
}


make_bag(name, create)
char *name;
int create;
{
    octObject bag;

    bag.type = OCT_BAG;
    bag.contents.bag.name = name;
    if (create) {
	(void) octCreate(&container[level], &bag);
    } else {
	(void) octCreateOrModify(&container[level], &bag);
    }
    container[++level] = bag;
}

make_box(llx, lly, urx, ury)
int llx, lly, urx, ury;
{
    octObject box;

    o_create_box(&container[level], &box, (octCoord) llx, (octCoord) lly,
	    (octCoord) urx, (octCoord) ury);
    container[++level] = box;
}

make_layer(name, create)
char *name;
int create;
{
    octObject layer;

    layer.type = OCT_LAYER;
    layer.contents.bag.name = name;
    if (create) {
	(void) octCreate(&container[level], &layer);
    } else {
	(void) octCreateOrModify(&container[level], &layer);
    }
    container[++level] = layer;
}

grab_instance(name)
char *name;
{
    octObject inst;

    inst.type = OCT_INSTANCE;
    inst.contents.instance.name = name;
    if (octGetByName(&cfacet, &inst) != OCT_OK) {
	yyerror("Instance must already exist");
    }
    if (!octIdsEqual(cfacet.objectId, container[level].objectId)) {
	o_attach(&container[level], &inst);
    }
    container[++level] = inst;
    lastInst = inst;
}

grab_net(termi)
ident *termi;
{
    octObject net;
    int hbit, lbit, i, flag;
    char buf[80];

    net.type = OCT_NET;
    flag = give_hl(termi, &hbit, &lbit);
    if (hbit != lbit) {
	yyerror("Only single terminals may be specified");
    }
    (void) sprintf(buf, "%s<%d>", termi->u.name, hbit);
    net.contents.net.name = buf;

    /* 2 tries -- with bit subscript, and without */
    for (i = 0; i < 2; i++) {
	if (o_get_net_by_name(&cfacet, &net) == OCT_OK) {
	    (void) strcpy(buf, termi->u.name);
	    if (!octIdsEqual(cfacet.objectId, container[level].objectId)) {
		o_attach(&container[level], &net);
	    }
	    container[++level] = net;
	    return;
	}

	/* for retry -- bit subscript must have been unspecified */
	if (flag) {
	    break;
	}
	net.contents.net.name = termi->u.name;
    }
    yyerror("Net not found");
}

grab_fterm(termi)
ident *termi;
{
    octObject oterm;
    int hbit, lbit, i, flag;
    char buf[80];

    oterm.type = OCT_TERM;
    flag = give_hl(termi, &hbit, &lbit);
    if (hbit != lbit) {
	yyerror("Only single terminals may be specified");
    }
    (void) sprintf(buf, "%s<%d>", termi->u.name, hbit);
    oterm.contents.term.name = buf;

    /* 2 tries -- with bit subscript, and without */
    for (i = 0; i < 2; i++) {
	if (octGetByName(&cfacet, &oterm) == OCT_OK) {
	    (void) strcpy(buf, termi->u.name);
	    if (!octIdsEqual(cfacet.objectId, container[level].objectId)) {
		o_attach(&container[level], &oterm);
	    }
	    container[++level] = oterm;
	    return;
	}

	/* for retry -- bit subscript must have been unspecified */
	if (flag) {
	    break;
	}
	oterm.contents.term.name = termi->u.name;
    }
    yyerror("Terminal not found");
}

grab_aterm(termi)
ident *termi;
{
    octObject oterm;
    int hbit, lbit, i, flag;
    char buf[80];

    if (octIdIsNull(lastInst.objectId)) {
	yyerror("INSTANCE must precede ATERM");
    }
    oterm.type = OCT_TERM;
    flag = give_hl(termi, &hbit, &lbit);
    if (hbit != lbit) {
	yyerror("Only single terminals may be specified");
    }
    (void) sprintf(buf, "%s<%d>", termi->u.name, hbit);
    oterm.contents.term.name = buf;

    /* 2 tries -- with bit subscript, and without */
    for (i = 0; i < 2; i++) {
	if (octGetByName(&lastInst, &oterm) == OCT_OK) {
	    (void) strcpy(buf, termi->u.name);
	    if (!octIdsEqual(lastInst.objectId, container[level].objectId)) {
		o_attach(&container[level], &oterm);
	    }
	    container[++level] = oterm;
	    return;
	}

	/* for retry -- bit subscript must have been unspecified */
	if (flag) {
	    break;
	}
	oterm.contents.term.name = termi->u.name;
    }
    yyerror("Terminal not found");
}
