%{
#include "copyright.h"
#include "port.h"
#include "yylex.h"
#include "smm.h"
#include "oct.h"

#define YTRACE(mes) if (0) (void) printf("mes\n")

char *strsav();
ident *make_concat();
static int doit = 1;
%}


%union {
    int intval;
    char *strval;
    ident *expr;
    cexpr *number;
}

%start file

%token <intval>
T_ALIAS
T_ARRAY
T_ATERM
T_BAG
T_BASE
T_BOX
T_BY
T_CLOCK
T_CLOSE_ANGLE
T_CLOSE_PAREN
T_CLOSE_SQUARE
T_COL
T_COLON
T_COMMA
T_CONCAT
T_CONNECTOR
T_CREATE
T_DIV
T_EDITSTYLE
T_ELSE
T_END
T_ENDMACRO
T_ENDMODEL
T_EQ
T_FACET
T_FROM
T_FTERM
T_GROUND
T_IF
T_INAME
T_INOUT
T_INPUT
T_INSTANCE
T_IPROP
T_LAYER
T_MACRO
T_MINUS
T_MOD
T_MODEL
T_NAME
T_NET
T_NEXT
T_NUMBER
T_OF
T_OPEN_ANGLE
T_OPEN_PAREN
T_OPEN_SQUARE
T_OUTCLOCK
T_OUTGROUND
T_OUTPUT
T_OUTSUPPLY
T_PATH
T_PLUS
T_PROMOTE
T_REQUIRE
T_ROW
T_RPROP
T_SUPPLY
T_SPROP
T_STRING
T_TECHNOLOGY
T_THEN
T_TIMES
T_TO
T_TRISTATE
T_UNCONNECTED
T_VAR
T_VIEWTYPE
T_OCT_NO_TRANSFORM
T_OCT_MIRROR_X
T_OCT_MIRROR_Y
T_OCT_ROT90
T_OCT_ROT180
T_OCT_ROT270
T_OCT_MX_ROT90
T_OCT_MY_ROT90

%type <intval>
simple_number
mk_array_spec
opt_create
transform

%type <expr>
const_identifier
rhs_identifier
/*
    const_list
    dconstant
*/

%type <strval>
number
name
var_string

%type <number>
variable
constant
expression

%left T_PLUS T_MINUS
%left T_TIMES 
%nonassoc T_DIV T_MOD
%%

file		:   description
		|   file description
		;

description 	:   model_header policy_decls
			{if(doit){make_view_prop();}}
		    port_decls instance_decls T_ENDMODEL T_NEXT
			{if(doit){oct_close(/* end_cfacet */ 1);}}
		|   extras
		;

model_header	:   T_MODEL name T_NEXT
			{if(doit){
			 oct_init($2, "symbolic");
			 FREE($2);
			 YTRACE(model_header);
			}}
		|   T_MODEL name T_COLON name T_NEXT
			{if(doit){
			 oct_init($2, $4);
			 FREE($2); FREE($4);
			 YTRACE(model_header);
			}}
		;

policy_decls	:   /* empty */
		|   policy_decls T_PATH path_list T_NEXT
		|   policy_decls T_TECHNOLOGY name T_NEXT
			{if(doit){set_technology($3);}}
		|   policy_decls T_EDITSTYLE name T_NEXT
			{if(doit){set_editstyle($3);}}
		|   policy_decls T_VIEWTYPE name T_NEXT
			{if(doit){set_viewtype($3);}}
		;

path_list	:   /* empty */
		|   path_list name
		;

port_decls	:   /* empty */
		|   port_decls port_decl
		;

port_decl	:   T_INPUT {if(doit){set_term_dir("INPUT");}}
		    opt_port_list T_NEXT
		|   T_OUTPUT {if(doit){set_term_dir("OUTPUT");}}
		    opt_port_list T_NEXT
		|   T_INOUT {if(doit){set_term_dir("INOUT");}}
		    opt_port_list T_NEXT
		|   T_CLOCK {if(doit){set_term_dir("CLOCK");}}
		    opt_port_list T_NEXT
		|   T_OUTGROUND {if(doit){set_term_dir("OUTGROUND");}}
		    opt_port_list T_NEXT
		|   T_GROUND {if(doit){set_term_dir("GROUND");}}
		    opt_port_list T_NEXT
		|   T_OUTCLOCK {if(doit){set_term_dir("OUTCLOCK");}}
		    opt_port_list T_NEXT
		|   T_TRISTATE {if(doit){set_term_dir("TRISTATE");}}
		    opt_port_list T_NEXT
		|   T_OUTSUPPLY {if(doit){set_term_dir("OUTSUPPLY");}}
		    opt_port_list T_NEXT
		|   T_SUPPLY {if(doit){set_term_dir("SUPPLY");}}
		    opt_port_list T_NEXT
		;

opt_port_list	:   /* empty */
		|   port_list
		;
port_list	:   port_spec
		|   port_list port_spec
		|   port_list T_COMMA port_spec
		;
port_spec	:  const_identifier 
			{if(doit){make_formal($1, NIL(ident), 0); ident_free($1); }}
		|  const_identifier T_COLON T_UNCONNECTED
			{if(doit){make_formal($1, NIL(ident), 1); ident_free($1); }}
		|  const_identifier T_COLON rhs_identifier
			{if(doit){make_formal($1, $3, 0); ident_free($1); ident_free($3);}}
		| T_IF expression T_THEN
			{ doit = push_if(doit, $2->value); }
		  port_spec T_ELSE
			{ doit = ! doit; }
		  port_spec T_END
			{ doit = pop_if (); }
		;

instance_decls	:   /* empty */
		|   instance_decls instance_decl
		;

instance_decl	: T_ARRAY array_instance
				| T_INSTANCE single_instance opt_connect_list
					{if(doit){finish_inst();}}
				;

array_instance: mk_array_spec var_string T_FROM expression
		    T_TO expression T_OF
			{if(doit){term *temp;
			 temp = MALLOC(term);
			 make_const(&temp, 1);
			 push_var($2, $4->value, $6->value, temp, $1);
			 FREE($4); FREE($6);
			}}
		    instance_decl
			{if(doit){pop_var();}}
		|   mk_array_spec var_string T_FROM expression
		    T_TO expression T_BY expression T_OF
			{if(doit){push_var($2, $4->value, $6->value, $8->value, $1);
			 FREE($4); FREE($6); FREE($8);
			}}
		    instance_decl
			{if(doit){pop_var();}}
		| T_IF expression T_THEN
			{ doit = push_if(doit, $2->value); }
		  array_instance T_ELSE
			{ doit = ! doit; }
		  array_instance T_END
			{ doit = pop_if (); }
		;

single_instance : name
			{if(doit){default_parms();}}
		    inst_parms
			{if(doit){make_inst($1, "symbolic", "contents"); FREE($1);}}

		|   name T_COLON name
			{if(doit){default_parms();}}
		    inst_parms
			{if(doit){make_inst($1, $3, "contents"); FREE($1); FREE($3);}}

		|   name T_COLON name T_COLON name
			{if(doit){default_parms();}}
		    inst_parms
			{if(doit){make_inst($1, $3, $5); FREE($1); FREE($3); FREE($5);}}
		| T_IF expression T_THEN
			{ doit = push_if(doit, $2->value); }
		  single_instance T_NEXT T_ELSE
			{ doit = ! doit; }
		  single_instance T_NEXT T_END
			{ doit = pop_if (); }
		;

inst_parms	:   /* empty */
		|   inst_parms T_OPEN_SQUARE expression T_COMMA
		    expression T_COMMA transform T_CLOSE_SQUARE
			{if(doit){set_place($3->value, $5->value, $7);
			 FREE($3); FREE($5);}}
		|   inst_parms T_OPEN_SQUARE expression T_COMMA
		    expression T_CLOSE_SQUARE
			{if(doit){set_place($3->value, $5->value, OCT_NO_TRANSFORM);
			 FREE($3); FREE($5);}}
		|   inst_parms T_PROMOTE
			{if(doit){set_promote();}}
		|   inst_parms T_INAME T_EQ name
			{if(doit){set_name($4);}}
		|   inst_parms T_CONNECTOR
			{if(doit){set_connector();}}
		;

mk_array_spec	:   /* empty */
			{if(doit){$$ = 0;}}
		|   T_ROW
			{if(doit){$$ = 1;}}
		|   T_COL
			{if(doit){$$ = 2;}}
		;

/*
 * oct junk attacher
 */
/* Declaration to tag garbage onto the facet */
extras		:   T_OPEN_PAREN T_FACET name T_COLON name T_COLON name
			{if(doit){extra_open_facet($3, $5, $7);
			 FREE($3);FREE($5);FREE($7);}}
		    oct_stuff_list T_CLOSE_PAREN
			{if(doit){extra_close_facet();}}
		|   T_OPEN_PAREN T_FACET name T_COLON name
			{if(doit){extra_open_facet($3, $5, "contents");
			 FREE($3);FREE($5);}}
		    oct_stuff_list T_CLOSE_PAREN
			{if(doit){extra_close_facet();}}

oct_stuff_list	:   /* empty */
		|   oct_stuff_list T_OPEN_PAREN opt_create T_SPROP name name
			{if(doit){make_sprop($5,$6,$3); FREE($5); FREE($6);}}
		    oct_stuff_list opt_alias T_CLOSE_PAREN
			{if(doit){pop_level();}}

		|   oct_stuff_list T_OPEN_PAREN opt_create T_RPROP name name
			{if(doit){make_rprop($5,$6,$3); FREE($5); FREE($6);}}
		    oct_stuff_list opt_alias T_CLOSE_PAREN
			{if(doit){pop_level();}}

		|   oct_stuff_list T_OPEN_PAREN opt_create T_IPROP name
			simple_number
			{if(doit){make_iprop($5,$6,$3); FREE($5);}}
		    oct_stuff_list opt_alias T_CLOSE_PAREN
			{if(doit){pop_level();}}

		|   oct_stuff_list T_OPEN_PAREN opt_create T_BAG name 
			{if(doit){make_bag($5,$3); FREE($5);}}
		    oct_stuff_list opt_alias T_CLOSE_PAREN
			{if(doit){pop_level();}}

		|   oct_stuff_list T_OPEN_PAREN opt_create T_LAYER name
			{if(doit){make_layer($5,$3); FREE($5);}}
		    oct_stuff_list opt_alias T_CLOSE_PAREN
			{if(doit){pop_level();}}

		|   oct_stuff_list T_OPEN_PAREN T_NET const_identifier
			{if(doit){grab_net($4); ident_free($4);}}
		    oct_stuff_list T_CLOSE_PAREN
			{if(doit){pop_level();}}

		|   oct_stuff_list T_OPEN_PAREN T_INSTANCE name
			{if(doit){grab_instance($4); FREE($4);}}
		    oct_stuff_list T_CLOSE_PAREN
			{if(doit){pop_level();}}

		|   oct_stuff_list T_OPEN_PAREN T_ATERM const_identifier
			{if(doit){grab_aterm($4); ident_free($4);}}
		    oct_stuff_list T_CLOSE_PAREN
			{if(doit){pop_level();}}

		|   oct_stuff_list T_OPEN_PAREN T_FTERM const_identifier
			{if(doit){grab_fterm($4); ident_free($4);}}
		    oct_stuff_list T_CLOSE_PAREN
			{if(doit){pop_level();}}

		|   oct_stuff_list T_OPEN_PAREN T_BOX
		    simple_number simple_number simple_number simple_number
			{if(doit){make_box($4, $5, $6, $7);}}
		    oct_stuff_list opt_alias T_CLOSE_PAREN
			{if(doit){pop_level();}}

		|   oct_stuff_list T_OPEN_PAREN T_ALIAS name
			{if(doit){grab_alias($4); FREE($4);}}
		    oct_stuff_list T_CLOSE_PAREN
			{if(doit){pop_level();}}
		;

opt_alias	:   /* empty */
		|   name
			{if(doit){make_alias($1);}}
		;

opt_create	:   /* empty */
			{if(doit){$$ = 0;}}
		|   T_CREATE
			{if(doit){$$ = 1;}}
		;

opt_connect_list:   connect_list
		;

connect_list	:   /*empty */
		|   connect_list T_NEXT
		|   connect_list const_identifier T_COLON
		    rhs_identifier T_NEXT
			{if(doit){make_connect($2, $4);
			 ident_free($2); ident_free($4);}}
		|   connect_list const_identifier T_COLON
		    T_UNCONNECTED T_NEXT
			{if(doit){make_unconnect($2); ident_free($2);}}
		|   connect_list const_identifier T_BASE
		    rhs_identifier T_NEXT
			{if(doit){make_promote($2, $4);
			 ident_free($2);ident_free($4);}}
		| connect_list T_IF expression T_THEN
			{ doit = push_if(doit, $3->value); }
		  connect_list T_ELSE
			{ doit = ! doit; }
		  connect_list T_END
			{ doit = pop_if (); }
		;

/*  Any number of concated const_identifiers */
rhs_identifier	:   const_identifier
			{if(doit){$$ = $1;}}
		|   rhs_identifier T_CONCAT const_identifier
			{if(doit){ident_add(&$1,$3);}}
		;

/*  an identifier with (optional) subscripts  */
const_identifier:   name
			{if(doit){$$=MALLOC(ident); $$->u.name=$1;
			 $$->next=NIL(ident); $$->type = ID;
			 /* The default size is based on context.
			  * In a declaration => hbit = lbit = 0.
			  * Elsewhere => from declaration */
			 $$->hbit = $$->lbit = NIL(term);}}
		|   name T_OPEN_ANGLE expression T_CLOSE_ANGLE
			{if(doit){$$=MALLOC(ident); $$->u.name=$1;
			 $$->next=NIL(ident); $$->type = ID;
			 $$->hbit = $3->value; $$->lbit = NIL(term);
			 FREE($3);}}
		|   name T_OPEN_ANGLE expression T_COLON expression T_CLOSE_ANGLE
			{if(doit){$$=MALLOC(ident); $$->u.name=$1;
			 $$->next=NIL(ident); $$->type = ID;
			 $$->hbit = $3->value; $$->lbit = $5->value;
			 FREE($3); FREE($5);}}
		;

/*
 *  expressions
 */
expression	:   constant
			{if(doit){$$ = $1;}}
		|   variable
			{if(doit){$$ = $1;}}
		|   T_OPEN_PAREN expression T_CLOSE_PAREN
			{if(doit){$$ = $2;}}

	/* unary arthimetic operators */
		|   T_PLUS expression %prec T_TIMES
			{if(doit){$$ = $2; $$->width = 32;}}
		|   T_MINUS expression %prec T_TIMES
			{if(doit){term *temp;
			 make_const(&temp, 0);
			 do_sub(&temp, &$2->value);
			 $$ = $2; $$->width = 32; $$->value = temp;}}

	/* binary arithmetic operators */
		|   expression T_PLUS expression
			{if(doit){$$ = $1;
			 do_add(&$$->value, &$3->value);
			 $$->width = 32;
			 FREE($3);}}
		|   expression T_MINUS expression
			{if(doit){$$ = $1;
			 do_sub(&$$->value, &$3->value);
			 $$->width = 32;
			 FREE($3);}}
		|   expression T_DIV expression
			{if(doit){$$ = $1;
			 do_div(&$$->value, &$3->value);
			 $$->width = 32;
			 FREE($3);}}
		|   expression T_MOD expression
			{if(doit){$$ = $1;
			 do_mod(&$$->value, &$3->value);
			 $$->width = 32;
			 FREE($3);}}
		|   expression T_TIMES expression
			{if(doit){$$ = $1;
			 do_mult(&$$->value, &$3->value);
			 $$->width = 32;
			 FREE($3);}}
		;

/* a constant node */
constant	:   number
			{if(doit){int i;
			 $$ = MALLOC(cexpr); $$->next = NIL(cexpr);
			 read_constant($1, "10", (unsigned *) &i, &$$->width);
			 make_const(&$$->value, i);
			 FREE($1);}}
		|   number T_BASE number
			{if(doit){int i;
			 $$ = MALLOC(cexpr); $$->next = NIL(cexpr);
			 read_constant($1, $3, (unsigned *) &i, &$$->width);
			 make_const(&$$->value, i);
			 FREE($1); FREE($3);}} 
		;

/* A variable node (%---) */
variable	: T_VAR 
			{if(doit){$$ = MALLOC(cexpr); $$->next = NIL(cexpr);
			 $$->width = 32;
			 if (!make_var(&$$->value, yytext))
			     make_const(&$$->value, 0);
			}}
		;

simple_number	:   number
			{if(doit){$$ = atoi($1); FREE($1);}}
		|   T_MINUS number
			{if(doit){$$ = -atoi($2); FREE($2);}}

var_string	: T_VAR 
			{if(doit){$$ = strsav(yytext);}}
		;

name		:   T_NAME
			{if(doit){$$ = strsav(yytext);}}
		;

number		: T_NAME
			{if(doit){$$ = strsav(yytext);}}
		;

transform       :   T_OCT_NO_TRANSFORM
			{if(doit){$$ = (int) OCT_NO_TRANSFORM;}}
		|   T_OCT_MIRROR_X
		        {if(doit){$$ = (int) OCT_MIRROR_X;}}
		|   T_OCT_MIRROR_Y
		        {if(doit){$$ = (int) OCT_MIRROR_Y;}}
		|   T_OCT_ROT90
		        {if(doit){$$ = (int) OCT_ROT90;}}
		|   T_OCT_ROT180
		        {if(doit){$$ = (int) OCT_ROT180;}}
		|   T_OCT_ROT270
		        {if(doit){$$ = (int) OCT_ROT270;}}
		|   T_OCT_MX_ROT90
		        {if(doit){$$ = (int) OCT_MX_ROT90;}}
		|   T_OCT_MY_ROT90
		        {if(doit){$$ = (int) OCT_MY_ROT90;}}
		;
