#include "copyright.h"
#include "port.h"
#include "defs.h"
#include "fang.h"
#include "harpoon.h"


main_chip_area(facet)
    octObject *facet;
{
  octObject		InstanceBag, ConnectorBag;
  octObject		Instance;
  octObject		Connector;
  octStatus		status;		/* to record the return codes */
  octGenerator		genInst; /* generators */
  double		inst_area;
  char			*connectorName;

  struct octBox chipBB;
  

  ConnectorBag.type = OCT_BAG;
  ConnectorBag.contents.bag.name = "CONNECTORS";
  InstanceBag.type = OCT_BAG;
  InstanceBag.contents.bag.name = "INSTANCES";

  if (octGetByName(facet,&InstanceBag)==OCT_NOT_FOUND) FatalError("Missing instance bag");
  if (octGetOrCreate(facet,&ConnectorBag)!=OCT_OK) FatalError("Problem with connector bag");
   
  OH_ASSERT(octInitGenContents(&InstanceBag,OCT_INSTANCE_MASK,&genInst));
  while ((status = octGenerate(&genInst,&Instance)) == OCT_OK) {
    num_instances++;
    inst_area =  instance_area(&Instance);
    tot_instance_area += inst_area;
    tot_instance_area2 += (double) inst_area * (double) inst_area;
  }
  if (num_instances > 0) {
    tot_instance_area_var = (tot_instance_area2 - (tot_instance_area * tot_instance_area)/num_instances) / num_instances;
  }
  if (status!=OCT_GEN_DONE) FatalError("generating instances");

  /*  Read in all the connectors that implement formal terminals also */
  OH_ASSERT(octInitGenContents(&ConnectorBag,OCT_INSTANCE_MASK,&genInst));
  while ((status = octGenerate(&genInst,&Connector)) == OCT_OK) {
    if (strrchr(Connector.contents.instance.master,'/') == 0 ) {
      connectorName = Connector.contents.instance.master;
    } else {
      connectorName = strrchr(Connector.contents.instance.master,'/');
      connectorName++;
    }
    if (strncmp(connectorName, "com2", 4) == 0) {
      num_vias++;
    }
    num_connectors++;
  }
  if (status!=OCT_GEN_DONE) FatalError("generating instances");
  
  OH_ASSERT(octBB(facet,&chipBB));

  chipWidth = chipBB.upperRight.x - chipBB.lowerLeft.x;
  chipHeight= chipBB.upperRight.y - chipBB.lowerLeft.y;

  chipWidth /= LAMBDA;
  chipHeight/= LAMBDA;
  chipArea = (double) chipWidth * chipHeight;


}


int
instance_area(InstanceP)
    octObject *InstanceP;
{
  octObject		Master;
  int			i,i1;
  int			area;
  harList		har_perimList;	/* to hold the perimeter of a Master */
  struct octPoint	*perimeter; 	/* temporary perimeter of a cell */
  int			corners;


  /* get the Master */
  Master.type=OCT_FACET;
  Master.contents.facet.facet="interface";
  Master.contents.facet.mode="r";
  if (octOpenMaster(InstanceP,&Master) < OCT_OK) {
    FatalError(sprintf(buffer,"Opening master %s:%s",
			  Master.contents.facet.cell,Master.contents.facet.view));
  }

  harFacetToPerimList(&Master, HAR_INFINITY , (int) LAMBDA * 40, &har_perimList);
    
  if ((harList *) harRestOfList(har_perimList) != NIL(harList)) {
    (void) sprintf(buffer,"More than 1 geometry to cover %s:%s", 
		    Master.contents.facet.cell,Master.contents.facet.view);
    Warning(buffer);
  }
  /* get the information from HARPOON,copy it in shared memory and then free storage */
  perimeter   = harFirstPerimArray(har_perimList);
  corners     = harFirstPerimCount(har_perimList);

  /* Compute the area contained in the perimeter */
  /* The perimeter is given counterclock wise */
  area = 0;
  for (i = 0 ; i < corners ; i++) {
    i1 = (i + 1) % corners;
    area += (perimeter[i].x  - perimeter[i1].x) * perimeter[i].y;
  }
  area /= (LAMBDA * LAMBDA);
  if (area < 0) area = -area;	/* Just to make sure the area is positive */

  harFreeList(&har_perimList);	/* free all storage used by HARPOON */
  return(area);
}
