#include "copyright.h"
#include "port.h"
#include "defs.h"

int net_length();
int net_bb_compare();
int net_length_compare();


main_net_length(facet)
    octObject *facet;
{
  int 		i;
  octObject 	net;

  num_nets = all_net_lengths(facet, &net_lengths);
  if (num_nets < 0) {
    (void) printf("Policy violation while checking nets\n");
    exit(1);
  }

  if (bb) {
    qsort((char *) net_lengths, num_nets, 
	  sizeof(struct net_length_struct), net_bb_compare);
  } else {
    qsort((char *) net_lengths, num_nets, 
	  sizeof(struct net_length_struct), net_length_compare);
  }

  if (bb) {
    for(i = 0; i < num_nets; i++) {
      OH_ASSERT(ohGetById(&net, net_lengths[i].objectId));
      total_net_length += net_lengths[i].bbox_length;
      total_net_length2 +=  net_lengths[i].bbox_length * net_lengths[i].bbox_length;
      total_net_area += net_lengths[i].area;
      total_net_area2 += net_lengths[i].area *  net_lengths[i].area;
    }
  } else {
    for(i = 0; i < num_nets; i++) {
      OH_ASSERT(ohGetById(&net, net_lengths[i].objectId));
      total_net_length += net_lengths[i].length;
      total_net_length2 +=  net_lengths[i].length * net_lengths[i].length;
      total_net_area += net_lengths[i].area;
      total_net_area2 += net_lengths[i].area *  net_lengths[i].area;
    }
  }
  if (num_nets > 0) {
    total_net_length_var = (total_net_length2 - (total_net_length * total_net_length)/num_nets) / num_nets;
    total_net_area_var   = (total_net_area2 - (total_net_area * total_net_area)/num_nets) / num_nets;
  }
  return(0);
}


int all_net_lengths(facet, net_lengths_p)
    octObject *facet;
    struct net_length_struct **net_lengths_p;
{
  int size, num_nets;
  double actual_length, bbox_length, area;
  struct net_length_struct *net_lengths;
  octGenerator gen;
  octObject net;
  octStatus status;

  num_nets = 0;
  size = 10;
  net_lengths = ALLOC(struct net_length_struct, size);

  OH_ASSERT(octInitGenContents(facet, OCT_NET_MASK, &gen));
  while ((status = octGenerate(&gen, &net)) == OCT_OK) {
    if (num_nets >= size) {
      size *= 2;
      net_lengths = REALLOC(struct net_length_struct, net_lengths, size);
    }

    /* Bail out if any errors */
    if (! net_length(&net, &actual_length, &bbox_length, &area)) {
      return -1;
    }
    net_lengths[num_nets].length = actual_length;
    net_lengths[num_nets].bbox_length = bbox_length;
    net_lengths[num_nets].area = area;
    net_lengths[num_nets].objectId = net.objectId;
    num_nets++;
  }
  OH_ASSERT(status == OCT_GEN_DONE ? OCT_OK : status);
  *net_lengths_p = net_lengths;
  return num_nets;
}


int net_length(net, actual_length, bbox_length , area)
    octObject *net;
    double *actual_length;
    double *bbox_length;
    double *area;
{
  double length, tot_length, tot_area, hypot();
  int first;
  int32 num_points;
  struct octPoint *points;
  struct octBox box, bb;
  octGenerator gen;
  octObject path, term;
  octStatus status;
  octCoord dx, dy;


  /*
   * Compute the length of all paths attached to the net
   */
  tot_length = 0.0;
  tot_area = 0.0;
  OH_ASSERT(octInitGenContents(net, OCT_PATH_MASK, &gen));
  while ((status = octGenerate(&gen, &path)) == OCT_OK) {
    ohGetPoints(&path, &num_points, &points);
    if (num_points != 2) {
      Warning(sprintf(buffer,"Net %s has a path with %d points",
		      net->contents.net.name, num_points));
      return 0; 
  }

    dx = points[0].x - points[1].x;
    dy = points[0].y - points[1].y;
    if (dx == 0) {
      length = dy < 0 ? -dy : dy;
    } else if (dy == 0) {
      length = dx < 0 ? -dx : dx;
    } else {
      length = hypot((double) dx, (double) dy);
    }
    tot_length += length;
    tot_area += length * path.contents.path.width;
  }
  OH_ASSERT(status == OCT_GEN_DONE ? OCT_OK : status);


  /*
   *  Compute the bounding box of all terminals on the net
   */
  first = 1;
  OH_ASSERT(octInitGenContents(net, OCT_TERM_MASK, &gen));
  while ((status = octGenerate(&gen, &term)) == OCT_OK) {
    if (ohTerminalBB(&term, &bb) != OCT_OK) {
      Warning(sprintf(buffer,"%s term %s on %s has no BBox",
		      (term.contents.term.instanceId==0) ? "FORMAL" : "ACTUAL",
		      term.contents.term.name,
		      net->contents.net.name));
      continue;			/* Policy violation */
    }
    if (first || bb.lowerLeft.x < box.lowerLeft.x) {
      box.lowerLeft.x = bb.lowerLeft.x;
    }
    if (first || bb.lowerLeft.y < box.lowerLeft.y) {
      box.lowerLeft.y = bb.lowerLeft.y;
    }
    if (first || bb.upperRight.x > box.upperRight.x) {
      box.upperRight.x = bb.upperRight.x;
    }
    if (first || bb.upperRight.y > box.upperRight.y) {
      box.upperRight.y = bb.upperRight.y;
    }
    first = 0;
  }
  OH_ASSERT(status == OCT_GEN_DONE ? OCT_OK : status);

  *actual_length = tot_length / (double) LAMBDA;
  *bbox_length = ((box.upperRight.x - box.lowerLeft.x) +
		  (box.upperRight.y - box.lowerLeft.y)) / LAMBDA;
  *area = tot_area / (LAMBDA * LAMBDA);
  return 1;
}


net_length_compare(a, b)
    struct net_length_struct *a, *b;
{
    if (b->length > a->length) {
	return 1;
    } else if (b->length < a->length) {
	return -1;
    } else {
	return 0;
    }
}

net_bb_compare(a, b)
    struct net_length_struct *a, *b;
{
    if (b->bbox_length > a->bbox_length) {
	return 1;
    } else if (b->bbox_length < a->bbox_length) {
	return -1;
    } else {
	return 0;
    }
}
