#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "st.h"
#include "tr.h"
#include "cif.h"
#include "cif_internal.h"

int cif_current_layer_defined = 0;
int cif_flatten;
tr_stack *cif_stack;

static st_table *cell_table;

static void
cif_init_cell_table()
{
    cell_table = st_init_table(st_numcmp, st_numhash);
    return;
}

/*ARGSUSED*/
static enum st_retval
do_cell_free(key, cptr, junk)
char *key;
char *cptr;
char *junk;
{
    cif_free_cell((struct cif_cell *) cptr);
    return ST_CONTINUE;
}

static void
cif_free_cell_table()
{
    (void) st_foreach(cell_table, do_cell_free, NIL(char));
    st_free_table(cell_table);
    return;
}

cif_process(file, flatten)
FILE *file;
int flatten;
{
    
    cif_init_cell_table();
    cif_stack = tr_create();
	
    cif_flatten = flatten;
    
    cp_parse(file);

    tr_free(cif_stack);
    cif_free_cell_table();
}


cif_add_cell(cell_num, cell)
int cell_num;
struct cif_cell *cell;
{
    (void) st_insert(cell_table, (char *) cell_num, (char *)cell);
}
    
cif_lookup_cell(cell_num, cell)
int cell_num;
struct cif_cell **cell;
{
    char *ptr;
    
    if (st_lookup(cell_table, (char *) cell_num, &ptr)) {
	*cell = (struct cif_cell *) ptr;
	return 1;
    } else {
	return 0;
    }
}

static enum st_retval
do_defdelete(key, entry, cell_num)
char *key;
char *entry;
char *cell_num;
{
    if ((int) key >= (int) cell_num) {
	cif_free_cell((struct cif_cell *) entry);
	return ST_DELETE;
    }
    return ST_CONTINUE;
}
	
cif_delete_cells(cell_num)
int cell_num;
{
    (void) st_foreach(cell_table, do_defdelete, (char *)cell_num);
}


#ifndef lint
/* force all of cif package in right now */
static void
dummy()
{
    cp_point center, dir;
    cp_path path;
    double rotation_array[2][2];
    /* struct cif_cell *cell; */

    cp_box(0, 0, center, dir);
    cp_call(0, rotation_array, 0, 0, 0);
    cp_defstart(0, 0, 0);
    cp_deffinish();
    cp_defdelete(0);
    cp_wire(0, 0, &path);
    cp_polygon(0, &path);
    cp_roundflash(0, center);
    cp_userextension(0, "hi");
    cp_end();
    cp_layer("NI");
    /* cif_flatten_call(cell, 0, rotation_array, 0, 0, 0, 0); */
}
#endif
