#include "copyright.h"
#include "port.h"
#include "errtrap.h"
#include "utility.h"
#include "tr.h"
#include "cif.h"
#include "cif_internal.h"

#define strsave(string) strcpy(malloc((unsigned)(strlen(string)+1)),string)

#define PUSH(path, ptr) {\
    if (path.head == NIL(char)) {\
	path.head = path.tail = (char *) ptr;\
	((struct cif_link *) path.tail)->link = (char *) 0;\
    } else {\
	((struct cif_link *) path.tail)->link = (char *) ptr;\
	path.tail = (char *) ptr;\
	((struct cif_link *) path.tail)->link = (char *) 0;\
    }\
}

#define SCALE(x) ((current_cell->a*(x))/current_cell->b)
#define SCALE_POINT(p) (p.x = SCALE(p.x),p.y = SCALE(p.y))

static struct cif_cell *current_cell;
static int in_definition = 0;

cp_box(length, width, center, dir)
int length,width;
cp_point center, dir;
{
    struct cif_box *ptr;
    if (in_definition) {
	ptr = ALLOC(struct cif_box, 1);
	ptr->type = CIF_BOX;
	ptr->length = SCALE(length);
	ptr->width = SCALE(width);
	ptr->center = center;
	SCALE_POINT(ptr->center);
	ptr->dir = dir;
	ptr->line = cp_line_number;
	PUSH(current_cell->geos,ptr);
    } else {
	cif_box(length, width, center, dir, cp_line_number);
    }
}

cp_call(cell_num, rotation_array, tran_x, tran_y, type)
int cell_num;
double rotation_array[2][2];
int32 tran_x, tran_y;
int type;
{
    struct cif_instance *ptr;
    struct cif_cell *cell;
    
    if (in_definition) {
	ptr = ALLOC(struct cif_instance, 1);
	ptr->type = CIF_INSTANCE;
	ptr->cell_num = cell_num;
	ptr->matrix = rotation_array;
	ptr->trans_x = SCALE(tran_x);
	ptr->trans_y = SCALE(tran_y);
	ptr->trans_type = type;
	ptr->line = cp_line_number;
	PUSH(current_cell->instances, ptr);
    } else {
	if (!cif_lookup_cell(cell_num, &cell)) {
	    errRaise(CIF_PKG_NAME, -1,
		    "Fatal Error: call of undefined cell #%d at line %d",
		    cell_num, cp_line_number);
	}

	if (cif_flatten) {
	    cif_flatten_call(cell, cell_num, rotation_array, tran_x, tran_y, type,
			     cp_line_number);
	} else {
	    cif_call(cell, cell_num, rotation_array, tran_x, tran_y, type,
		     cp_line_number);
	}
    }
}

cp_defstart(cell_num,a,b)
int cell_num,a,b;
{
    struct cif_cell *cell, *old;
    
    in_definition = 1;
    cell = ALLOC(struct cif_cell, 1);
    cell->type = CIF_CELL;
    cell->number = cell_num;
    cell->a = a;
    cell->b = b;
    cell->geos.head = cell->geos.tail = NIL(char);
    cell->instances.head = cell->instances.tail = NIL(char);
    cell->flags.being_processed = cell->flags.already_created = 0;
    cell->flags.redefined = cell->flags.child_redefined = 0;
    cell->line = cp_line_number;
    cell->users = cell->masters = NIL(struct cif_chain);
    
    if (cif_lookup_cell(cell_num, &old)) {
	(void) fprintf(stderr, "Warning: Cell #%d redefined at line %d\n",
		cell_num, cp_line_number);
	cif_free_cell(old);
    }
    current_cell = cell;
}

cp_deffinish()
{
    cif_add_cell(current_cell->number, current_cell);
    in_definition = 0;
}

cp_defdelete(cell_num)
int cell_num;
{
    cif_delete_cells(cell_num);
}

cp_wire(width,number,path)
int width;
int number;
cp_path *path;
{
    struct cif_wire *wire;
    cp_path *ptr;

    if (in_definition) {
	wire = ALLOC(struct cif_wire, 1);
	wire->type = CIF_WIRE;
	wire->width = SCALE(width);
	wire->number = number;
	for (ptr=path; ptr!= (cp_path *) 0; ptr = ptr->next) {
	    SCALE_POINT(ptr->point);
	}
	wire->path = path;
	wire->line = cp_line_number;
	PUSH(current_cell->geos,wire);
    } else {
	cif_wire(width, number, path, cp_line_number);
    }
}

cp_polygon(number,path)
int number;
cp_path *path;
{
    struct cif_polygon *polygon;
    cp_path *ptr;

    if (in_definition) {
	polygon = ALLOC(struct cif_polygon, 1);
	polygon->type = CIF_POLYGON;
	polygon->number = number;
	for (ptr=path; ptr!= (cp_path *) 0; ptr = ptr->next) {
	    SCALE_POINT(ptr->point);
	}
	polygon->path = path;
	polygon->line = cp_line_number;
	PUSH(current_cell->geos,polygon);
    } else {
	cif_polygon(number, path, cp_line_number);
    }
}

cp_roundflash(diameter,center)
int diameter;
cp_point center;
{
    struct cif_round_flash *circle;
    
    if (in_definition) {
	circle = ALLOC(struct cif_round_flash, 1);
	circle->type = CIF_ROUND_FLASH;
	circle->diameter = SCALE(diameter);
	circle->center = center;
	SCALE_POINT(center);
	circle->line = cp_line_number;
	PUSH(current_cell->geos,circle);
    } else {
	cif_round_flash(diameter, center, cp_line_number);
    }
}

cp_userextension(extension_num, text_string)
int extension_num;
char *text_string;
{
    struct cif_user_extension *ptr;
    
    if (in_definition) {
	int x, y;
	
	ptr = ALLOC(struct cif_user_extension, 1);
	ptr->type = CIF_USER_EXTENSION;
	ptr->number = extension_num;
	if (extension_num == 94) {
	    char label[1000], layer[6];
	    int retval;
	    
	    retval = sscanf(text_string, "%900s %d %d %4s", label, &x, &y, layer);
	    if (retval == 3 || retval == 4) {
	        /* this is recognizable as a label */
		(void) sprintf(label + strlen(label),
			" %d %d %s", SCALE(x), SCALE(y),
			(retval == 4 ? layer : ""));
		free(text_string);
		text_string = strsave(label);
	    }
	}
	ptr->text = text_string;
	ptr->line = cp_line_number;
	PUSH(current_cell->geos,ptr);
    } else {
	cif_user_extension(extension_num, text_string, cp_line_number);
    }
}

cp_end()
{
}

cp_layer(name)
char *name;
{
    struct cif_layer *ptr;
    
    if (in_definition) {
	ptr = ALLOC(struct cif_layer, 1);
	ptr->type = CIF_LAYER;
	ptr->name = name;
	ptr->line = cp_line_number;
	PUSH(current_cell->geos,ptr);
    } else {
	cif_layer(name, cp_line_number);
	cif_current_layer_defined = 1;
    }
}

cif_flatten_call(cell, cell_num, rotation_array, tran_x, tran_y, type, line)
struct cif_cell *cell;
int cell_num;
double rotation_array[2][2];
int32 tran_x, tran_y;
int type;
int line;
{
    int old_current_layer_defined;
    char *ptr;
    
    if (cell->flags.being_processed) {
	errRaise(CIF_PKG_NAME, -1,
		 "recursive call of cell #%d at line %d",
		 cell_num, line);
    }

    cell->flags.being_processed = 1;

    tr_push(cif_stack);

    tr_translate(cif_stack, tran_x, tran_y, 0);
    
    if (type != (int) TR_FULL_TRANSFORM) {
	tr_do_op(cif_stack, type, 0);
    } else {
	tr_rotate_matrix(cif_stack, rotation_array, 0);
    }
    
    ptr = cell->instances.head;
    while (ptr != NIL(char)) {
	struct cif_cell *sub_cell;
	struct cif_instance *inst = (struct cif_instance *) ptr;
	
	if (!cif_lookup_cell(inst->cell_num, &sub_cell)) {
	    errRaise(CIF_PKG_NAME, -1,
		    "call of undefined cell #%d at line %d",
		    inst->cell_num, cp_line_number);
	}
	
	cif_flatten_call(sub_cell, inst->cell_num, inst->matrix, inst->trans_x,
		inst->trans_y, inst->trans_type, inst->line);

	ptr = CIF_NEXT(ptr);
    }

    old_current_layer_defined = cif_current_layer_defined;
    cif_current_layer_defined = 0;

    ptr = cell->geos.head;
    while(ptr != NIL(char)) {
	switch (CIF_TYPE(ptr)) {
	case CIF_BOX :
	    {
		struct cp_point center, dir, zero;
		struct cif_box *box;

		box = (struct cif_box *) ptr;

		center = box->center;
		tr_transform(cif_stack, &center.x, &center.y);

		zero.x = zero.y = 0;
		dir = box->dir;
		tr_transform(cif_stack, &dir.x, &dir.y);
		tr_transform(cif_stack, &zero.x, &zero.y);
		dir.x -= zero.x;
		dir.y -= zero.y;

		cif_box(box->length, box->width, center, dir, box->line);
		break;
	    }
	case CIF_POLYGON :
	    {
		struct cif_polygon *polygon;
		struct cp_path *pnt, *new = NIL(cp_path), *copy;
		int first = 1;
		polygon = (struct cif_polygon *) ptr;

		pnt = polygon->path;
		while (pnt != NIL(struct cp_path)) {
		    if (first) {
			new = copy = ALLOC(struct cp_path, 1);
			first = 0;
		    } else {
			copy->next = ALLOC(struct cp_path, 1);
			copy = copy->next;
		    }
		    *copy = *pnt;
		    tr_transform(cif_stack, &copy->point.x, &copy->point.y);
		    pnt = pnt->next;
		}
		    
		cif_polygon(polygon->number, new, polygon->line);
		while (new != NIL(struct cp_path)) {
		    pnt = new->next;
		    free((char *) new);
		    new = pnt;
		}
		break;
	    }
	case CIF_WIRE :
	    {
		struct cif_wire *wire;
		struct cp_path *pnt, *new = NIL(cp_path), *copy;
		int first = 1;
		wire = (struct cif_wire *) ptr;

		pnt = wire->path;
		while (pnt != NIL(struct cp_path)) {
		    if (first) {
			new = copy = ALLOC(struct cp_path, 1);
			first = 0;
		    } else {
			copy->next = ALLOC(struct cp_path, 1);
			copy = copy->next;
		    }
		    *copy = *pnt;
		    tr_transform(cif_stack, &copy->point.x, &copy->point.y);
		    pnt = pnt->next;
		}
		    
		cif_wire(wire->width, wire->number, new, wire->line);
		
		while (new != NIL(struct cp_path)) {
		    pnt = new->next;
		    free((char *) new);
		    new = pnt;
		}
		break;
	    }
	case CIF_ROUND_FLASH :
	    {
		struct cif_round_flash *round_flash;
		struct cp_point center;
		
		round_flash = (struct cif_round_flash *) ptr;
		
		center = round_flash->center;
		tr_transform(cif_stack, &center.x, &center.y);

		cif_round_flash(round_flash->diameter, center,
				round_flash->line);
		break;
	    }
	case CIF_USER_EXTENSION :
	    {
		struct cif_user_extension *user_extension =
		             (struct cif_user_extension *) ptr;
		cif_user_extension(user_extension->number,
				   user_extension->text,
				   user_extension->line);
		break;
	    }
	case CIF_LAYER :
	    {
		struct cif_layer *layer = (struct cif_layer *) ptr;
		cif_layer(layer->name, layer->line);
		cif_current_layer_defined = 1;
		break;
	    }
	}
	ptr = CIF_NEXT(ptr);
    }

    cell->flags.being_processed = 0;
    cif_current_layer_defined = old_current_layer_defined;
    tr_pop(cif_stack);

    return 1;
}
