#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "tr.h"
#include "cif.h"
#include "cif_internal.h"

cif_free_layer(layer)
struct cif_layer *layer;
{
    FREE(layer->name);
    FREE(layer);
}

cif_free_user_extension(user_extension)
struct cif_user_extension *user_extension;
{
    FREE(user_extension->text);
    FREE(user_extension);
}

cif_free_box(box)
struct cif_box *box;
{
    FREE(box);
}

cif_free_round_flash(round_flash)
struct cif_round_flash *round_flash;
{
    FREE(round_flash);
}

cif_free_wire(wire)
struct cif_wire *wire;
{
    struct cp_path *path = wire->path, *tmp;

    while(path != NIL(cp_path)) {
	tmp = path->next;
	FREE(path);
	path = tmp;
    }
    FREE(wire);
}

cif_free_polygon(polygon)
struct cif_polygon *polygon;
{
    struct cp_path *path = polygon->path, *tmp;

    while(path != NIL(cp_path)) {
	tmp = path->next;
	FREE(path);
	path = tmp;
    }
    FREE(polygon);
}

cif_free_instance(instance)
struct cif_instance *instance;
{
    FREE(instance->matrix);
    FREE(instance);
}

cif_free_cell(cell)
struct cif_cell *cell;
{
    char *ptr, *tmp;

    ptr = cell->instances.head;
    while (ptr != NIL(char)) {
	tmp = CIF_NEXT(ptr);
	cif_free_instance((struct cif_instance *) ptr);
	ptr = tmp;
    }
    
    ptr = cell->geos.head;
    while(ptr != NIL(char)) {
	tmp = CIF_NEXT(ptr);
	switch (CIF_TYPE(ptr)) {
	case CIF_BOX :
	    cif_free_box((struct cif_box *) ptr);
	    break;
	case CIF_POLYGON :
	    cif_free_polygon((struct cif_polygon *) ptr);
	    break;
	case CIF_WIRE :
	    cif_free_wire((struct cif_wire *) ptr);
	    break;
	case CIF_ROUND_FLASH :
	    cif_free_round_flash((struct cif_round_flash *) ptr);
	    break;
	case CIF_USER_EXTENSION :
	    cif_free_user_extension((struct cif_user_extension *) ptr);
	    break;
	case CIF_LAYER :
	    cif_free_layer((struct cif_layer *) ptr);
	    break;
	}
	ptr = tmp;
    }
    FREE(cell);
}

cif_mark_use(user, master)
struct cif_cell *user, *master;
{
    struct cif_chain *chain = ALLOC(struct cif_chain, 1);

    if (master->users != NIL(struct cif_chain) &&
	master->users->user == user) {
	/* already marked */
	return;
    }
    chain->master = master;
    chain->next_user = master->users;
    chain->last_user = NIL(struct cif_chain);
    master->users = chain;

    chain->user = user;
    chain->next_master = user->masters;
    chain->last_master = NIL(struct cif_chain);
    user->masters = chain;
}

    
