#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "cif.h"

#define USAGE "Usage: %s [input_file [output_file]]\n"

FILE *out_file;

char *optProgName;

main(argc, argv)
int argc;
char **argv;
{
    FILE *in_file;

    optProgName = argv[0];

    in_file = stdin;
    out_file = stdout;
    
    if (argc > 1) {
	in_file = fopen(argv[1], "r");
	if (in_file == NULL) {
	    (void) fprintf(stderr, "Cannot open input file %s\n", argv[1]);
	    (void) fprintf(stderr, USAGE, argv[0]);
	    exit(-1);
	}
	argc--;
	argv++;
    }
    
    if (argc > 1) {
	out_file = fopen(argv[1], "w");
	if (out_file == NULL) {
	    (void) fprintf(stderr, "Cannot open output file %s\n", argv[1]);
	    (void) fprintf(stderr, USAGE, argv[0]);
	    exit(-1);
	}
	argc--;
	argv++;
    }

    if (argc > 1) {
	(void) fprintf(stderr, "Too many arguments\n");
	(void) fprintf(stderr, USAGE, argv[0]);
	exit(-1);
    }

    cif_process(in_file, 1);
}
    
/*ARGSUSED*/
cif_layer(name, line)
char *name;
int line;
{
    (void) fprintf(out_file, "L %s;\n", name);
}

/*ARGSUSED*/
cif_user_extension(number, text, line)
int number;
char *text;
int line;
{
}

/*ARGSUSED*/
cif_box(length, width, center, dir, line)
int length;
int width;
struct cp_point center;
struct cp_point dir;
int line;
{
    (void) fprintf(out_file, "B %d %d %d %d", length, width, center.x, center.y);
    if (!(dir.y == 0 && dir.x > 0)) {
	(void) fprintf(out_file, " %d %d", dir.x, dir.y);
    }
    (void) fprintf(out_file, ";\n");
}

/*ARGSUSED*/
cif_round_flash(diameter, center, line)
int diameter;
cp_point center;
int line;
{
    (void) fprintf(out_file, "R %d %d %d;\n", diameter, center.x, center.y);
}

/*ARGSUSED*/
cif_wire(width, number, path, line)
int width;
int number;
cp_path *path;
int line;
{
    (void) fprintf(out_file, "W %d", width);
    while(path != NIL(cp_path)) {
	(void) fprintf(out_file, " %d %d", path->point.x, path->point.y);
	path = path->next;
    }
    (void) fprintf(out_file, ";\n");
}


/*ARGSUSED*/
cif_polygon(number, path, line)
int number;
cp_path *path;
int line;
{
    (void) fprintf(out_file, "P");
    
    while(path != NIL(cp_path)) {
	(void) fprintf(out_file, " %d %d", path->point.x, path->point.y);
	path = path->next;
    }
    
    (void) fprintf(out_file, ";\n");
}

/*ARGSUSED*/
cif_call(cell, cell_num, matrix, tran_x, tran_y, type, line)
struct cif_cell *cell;
int cell_num;
double matrix[2][2];
int32 tran_x, tran_y;
int type, line;
{
    (void) fprintf(stderr, "Internal Error: cif_call was called\n");
    exit(-1);
}
