#include "copyright.h"
#define DEFINE_CTO_EXTERN
#include "port.h"
#include "utility.h"
#include "oct.h"
#include "oh.h"
#include "cif.h"
#include "octmm.h"
#include "ciftooct.h"
#include "options.h"

optionStruct optionList[] = {
    { "s",	"scale",		"multiply all coordinates by this" },
    { "f",	"file",			"octmm program file"		},
    { OPT_RARG,	"cif_file",		"CIF input file"		},
    { OPT_OARG,	"root_cell][:view",	"root cell for oct and/or view name" },
    { 0,	0,			0				}
};

main(argc, argv)
int argc;
char **argv;
{
    FILE *cif_file;
    int option;
    extern int optind;
    extern char *optarg;

    scaleFactor = NIL(char);
    progScript = NIL(char);
    nullTopLevelGeos = nullTopLevelCalls = 0;

    while ((option = optGetOption(argc, argv)) != EOF) {
	switch (option) {
	    case 's':		/* scale factor */
		scaleFactor = util_strsav(optarg);
		break;
	    case 'f':		/* octmm program file */
		progScript = util_strsav(optarg);
		break;
	}
    }
    if (optind >= argc) optUsage();

    if (strcmp(argv[optind], "-") == 0) {
	cif_file = stdin;
    } else {
	if ((cif_file = fopen(argv[optind], "r")) == NULL) {
	    perror(argv[optind]);
	    exit(-1);
	}
    }
    
    ohUnpackDefaults(&current_facet, "w", ":physical:contents");
    if (++optind < argc) {
	if (ohUnpackFacetName(&current_facet, argv[optind++]) != OCT_OK) {
	    optUsage();
	}
    }
    if (optind != argc) optUsage();
    
    current_layer.type = OCT_LAYER;

    octBegin();

    if (NULL_TOP_LEVEL) {
	cif_process(cif_file, 0);

	if (nullTopLevelGeos > 0 || nullTopLevelCalls > 1) {
	    (void) fprintf(stderr,
			"%s: Warning: ignored top level had %d %s and %d %s\n",
			optProgName,
			nullTopLevelGeos,
			(nullTopLevelGeos == 1) ? "geometry" : "geometries",
			nullTopLevelCalls,
			(nullTopLevelCalls == 1) ? "call" : "calls");
	}
    } else {
	if (octOpenFacet(&current_facet) < OCT_OK) {
	    char buffer[100];
	    (void) sprintf(buffer, "Cannot open the cell %s:%s for writing",
			current_facet.contents.facet.cell,
			current_facet.contents.facet.view);
	    octError(buffer);
	    exit(-1);
	}

	cif_process(cif_file, 0);

	oct_mask_mod(optProgName, progScript, &current_facet, scaleFactor);
	
	if (octCloseFacet(&current_facet) != OCT_OK) {
	    char buffer[100];
	    (void) sprintf(buffer, "Cannot write out cell %s:%s",
			current_facet.contents.facet.cell,
			current_facet.contents.facet.view);
	    octError(buffer);
	    exit(-1);
	}
    }

    octEnd();
    
    exit(0);
}
