#include "port.h"
#include "utility.h"
#include "oct.h"
#include "oh.h"
#include "cif.h"
#include "octmm.h"
#include "ciftooct.h"
#include "options.h"

/*ARGSUSED*/
/*VARARGS1*/
cif_layer(name, line)
char *name;
int line;
{
    if (NULL_TOP_LEVEL) {
	nullTopLevelGeos++;
	return;
    }

    current_layer.contents.layer.name = name;
    if (octGetOrCreate(&current_facet, &current_layer) != OCT_OK) {
	octError("Can't create layer");
	exit(-1);
    }
}

cif_box(length, width, center, dir, line)
int length;
int width;
struct cp_point center;
struct cp_point dir;
int line;
{
    struct octObject box;

    if (NULL_TOP_LEVEL) {
	nullTopLevelGeos++;
	return;
    }

    if (!cif_current_layer_defined) {
	(void) fprintf(stderr,
		    "Warning: Box not on any layer at line %d is ignored\n",
		    line);
	return;
    }

    if (dir.y != 0) {
	if (dir.x == 0) {
	    /* switch the length and width */
	    int temp;

	    temp = width;
	    width = length;
	    length = temp;
	} else {
	    /* turn it into a polygon */
	    cp_path path[4];
	    double hypotT, sinT, cosT;

	    hypotT = 2 * sqrt((double) dir.x * dir.x + (double) dir.y *dir.y);
	    sinT = dir.y / hypotT;
	    cosT = dir.x / hypotT;

	    path[0].point.x = center.x + length*cosT - width*sinT;
	    path[0].point.y = center.y + width*cosT + length*sinT;
	    path[0].next = &path[1];
	    path[1].point.x = center.x - length*cosT - width*sinT;
	    path[1].point.y = center.y + width*cosT - length*sinT;
	    path[1].next = &path[2];
	    path[2].point.x = center.x - length*cosT + width*sinT;
	    path[2].point.y = center.y - width*cosT - length*sinT;
	    path[2].next = &path[3];
	    path[3].point.x = center.x + length*cosT + width*sinT;
	    path[3].point.y = center.y - width*cosT + length*sinT;
	    path[3].next = NIL(cp_path);

	    cif_polygon(4, path, line);
	    return;
	}
    }

    box.type = OCT_BOX;

    box.contents.box.lowerLeft.x = center.x - length/2;
    box.contents.box.lowerLeft.y = center.y - width/2;
    box.contents.box.upperRight.x = center.x + length/2;
    box.contents.box.upperRight.y = center.y + width/2;

    if (octCreate(&current_layer, &box) != OCT_OK) {
	octError("Error in oct: Adding a box");
	exit(-1);
    }
}

cif_round_flash(diameter, center, line)
int diameter;
cp_point center;
int line;
{
    struct octObject circle;

    if (NULL_TOP_LEVEL) {
	nullTopLevelGeos++;
	return;
    }

    if (!cif_current_layer_defined) {
	(void) fprintf(stderr,
		     "Warning: Circle not on any layer at line %d is ignored\n",
		     line);
	return;
    }

    circle.type = OCT_CIRCLE;

    circle.contents.circle.center.x = center.x;
    circle.contents.circle.center.y = center.y;
    circle.contents.circle.startingAngle = 0;
    circle.contents.circle.endingAngle = 3600;
    circle.contents.circle.innerRadius = 0;
    circle.contents.circle.outerRadius = diameter/2;

    if (octCreate(&current_layer, &circle) != OCT_OK) {
	octError("Error in oct: Adding a circle");
	exit(-1);
    }
}

cif_wire(width, number, path, line)
int width;
int number;
cp_path *path;
int line;
{
    struct octObject wire;
    struct octPoint *pt_array, *ptr;

    if (NULL_TOP_LEVEL) {
	nullTopLevelGeos++;
	return;
    }

    if (!cif_current_layer_defined) {
	(void) fprintf(stderr,
		   "Warning: Wire not on any layer at line %d is ignored\n",
		   line);
	return;
    }

    wire.type = OCT_PATH;
    wire.contents.path.width = width;
    
    if (octCreate(&current_layer, &wire) != OCT_OK) {
	octError("Error in oct: Adding a wire");
	exit(-1);
    }

    ptr = pt_array = ALLOC(struct octPoint, number);
    
    while(path != NIL(cp_path)) {
	ptr->x = path->point.x;
	ptr->y = path->point.y;
	path = path->next;
	ptr++;
    }
    
    if (octPutPoints(&wire, number, pt_array) != OCT_OK) {
	octError("Adding the points to a wire");
	exit(-1);
    }
    
    FREE(pt_array);
}


cif_polygon(number, path, line)
int number;
cp_path *path;
int line;
{
    struct octObject polygon;
    struct octPoint *pt_array, *ptr;

    if (NULL_TOP_LEVEL) {
	nullTopLevelGeos++;
	return;
    }

    if (!cif_current_layer_defined) {
	(void) fprintf(stderr,
		    "Warning: Polygon not on any layer at line %d is ignored\n",
		    line);
	return;
    }

    polygon.type = OCT_POLYGON;
    
    if (octCreate(&current_layer, &polygon) != OCT_OK) {
	octError("Error in oct: Adding a polygon");
	exit(-1);
    }

    ptr = pt_array = ALLOC(struct octPoint, number);
    
    while(path != NIL(cp_path)) {
	ptr->x = path->point.x;
	ptr->y = path->point.y;
	path = path->next;
	ptr++;
    }
    
    if (octPutPoints(&polygon, number, pt_array) != OCT_OK) {
	octError("Adding the points to a polygon");
	exit(-1);
    }
    
    FREE(pt_array);
}

cif_user_extension(number, text, line)
int number;
char *text;
int line;
{
    static void do_label();

    if (NULL_TOP_LEVEL) {
	nullTopLevelGeos++;
	return;
    }

    if (number == 94) {
	do_label(text, line);
    }
}

static void
do_label(text, line)
char *text;
int line;
{
    struct octObject layer, label, *layer_ptr;
    char layerName[10], labelText[1000];
    int32 x, y, width;
    int retval;

    retval = sscanf(text, "%999s %d %d %4s", labelText, &x, &y, layerName);
    switch (retval) {
    case 3:
	if (!cif_current_layer_defined) {
	    (void) fprintf(stderr,
		    "Warning: Label not on any layer at line %d is ignored\n",
		    line);
	    return;
	}
	layer_ptr = &current_layer;
	break;
    case 4:
	layer.type = OCT_LAYER;
	layer.contents.layer.name = layerName;
	if (octGetOrCreate(&current_facet, &layer) != OCT_OK) {
	    octError("Can't create layer");
	    exit(-1);
	}
	layer_ptr = &layer;
	break;
    default:
	(void) fprintf(stderr,
		    "Malformed label \"%d %s;\" in line %d is ignored\n",
		    94, text, line);
	return;
    }
	
    width = LABEL_HEIGHT * strlen(labelText);

    label.type = OCT_LABEL;

    label.contents.label.label = labelText;
    label.contents.label.region.lowerLeft.x = x;
    label.contents.label.region.lowerLeft.y = y;
    label.contents.label.region.upperRight.x = x + width;
    label.contents.label.region.upperRight.y = y + width;
    label.contents.label.textHeight = LABEL_HEIGHT;
    label.contents.label.vertJust = OCT_JUST_TOP;
    label.contents.label.horizJust = OCT_JUST_LEFT;
    label.contents.label.lineJust = OCT_JUST_LEFT;
    if (octCreate(layer_ptr, &label) != OCT_OK) {
	octError("Can't create label");
	exit(-1);
    }
}

char *make_cell_name();

cif_call(cell, cell_num, matrix, tran_x, tran_y, type, line)
struct cif_cell *cell;
int cell_num;
double matrix[2][2];
int32 tran_x, tran_y;
int type, line;
{
    char buffer[100];
    struct octObject instance;

    if (cell->flags.being_processed) {
	(void) fprintf(stderr,
		    "Fatal Error: recursive call of cell #%d at line %d\n",
		    cell_num, line);
	exit(-1);
    }

    if (!cell->flags.already_created) {
	int old_current_layer_defined;
	struct octObject old_layer, old_facet;
	char *ptr;
	struct cif_box *box;
	struct cif_layer *layer;
	struct cif_polygon *polygon;
	struct cif_user_extension *user_extension;
	struct cif_wire *wire;
	struct cif_round_flash *round_flash;
	
	cell->name = make_cell_name(cell);
	cell->name = util_strsav(cell->name);
	
	/* change only the cell name -- keep the rest the same */
	old_facet = current_facet;
	current_facet.contents.facet.cell = cell->name;

	if (octOpenFacet(&current_facet) < OCT_OK) {
	    (void) sprintf(buffer,"Creating the cell %s", cell->name);
	    octError(buffer);
	    exit(-1);
	}

	cell->flags.already_created = 1;
	cell->flags.being_processed = 1;
	
	ptr = cell->instances.head;
	while (ptr != NIL(char)) {
	    struct cif_cell *sub_cell;
	    struct cif_instance *inst = (struct cif_instance *) ptr;

	    if (!cif_lookup_cell(inst->cell_num, &sub_cell)) {
		(void) fprintf(stderr,
			    "Error: call of undefined cell #%d at line %d -- continuing without it\n",
			    inst->cell_num, cp_line_number);
		return;
	    }
	
	    cif_call(sub_cell, inst->cell_num, inst->matrix, inst->trans_x,
		    inst->trans_y, inst->trans_type, inst->line);
	    ptr = CIF_NEXT(ptr);
	}

	old_layer = current_layer;
	old_current_layer_defined = cif_current_layer_defined;
	cif_current_layer_defined = 0;
	
	ptr = cell->geos.head;
	while(ptr != NIL(char)) {
	    switch (CIF_TYPE(ptr)) {
	    case CIF_BOX :
		box = (struct cif_box *) ptr;
		cif_box(box->length, box->width, box->center, box->dir,
			box->line);
		break;
	    case CIF_POLYGON :
		polygon = (struct cif_polygon *) ptr;
		cif_polygon(polygon->number, polygon->path, polygon->line);
		break;
	    case CIF_WIRE :
		wire = (struct cif_wire *) ptr;
		cif_wire(wire->width, wire->number, wire->path, wire->line);
		break;
	    case CIF_ROUND_FLASH :
		round_flash = (struct cif_round_flash *) ptr;
		cif_round_flash(round_flash->diameter, round_flash->center,
				round_flash->line);
		break;
	    case CIF_USER_EXTENSION :
		user_extension = (struct cif_user_extension *) ptr;
		cif_user_extension(user_extension->number,
				   user_extension->text,
				   user_extension->line);
		break;
	    case CIF_LAYER :
		layer = (struct cif_layer *) ptr;
		cif_layer(layer->name);
		cif_current_layer_defined = 1;
		break;
	    }
	    ptr = CIF_NEXT(ptr);
	}
	
	oct_mask_mod(optProgName, progScript, &current_facet, scaleFactor);

	if (octCloseFacet(&current_facet) != OCT_OK) {
	    (void) sprintf(buffer, "Error while writing out the cell %s",
			cell->name);
	    octError(buffer);
	    exit(-1);
	}
	
	current_layer = old_layer;
	current_facet = old_facet;
	cif_current_layer_defined = old_current_layer_defined;
	
	cell->flags.being_processed = 0;
    }

    if (NULL_TOP_LEVEL) {
	nullTopLevelCalls++;
    } else {
	instance.type = OCT_INSTANCE;
	instance.contents.instance.name = NIL(char);
	instance.contents.instance.master = cell->name;
	instance.contents.instance.view = current_facet.contents.facet.view;
	instance.contents.instance.facet = "contents";
	instance.contents.instance.version = OCT_CURRENT_VERSION;
	instance.contents.instance.transform.translation.x = tran_x;
	instance.contents.instance.transform.translation.y = tran_y;
	instance.contents.instance.transform.transformType =
	  (octTransformType) type;
	
	if ((octTransformType) type == OCT_FULL_TRANSFORM) {
	    double (*mptr)[2] =
	      instance.contents.instance.transform.generalTransform;
	    mptr[0][0] = matrix[0][0]; mptr[0][1] = matrix[0][1];
	    mptr[1][0] = matrix[1][0]; mptr[1][1] = matrix[1][1];
	}
	
	if (octCreate(&current_facet, &instance) != OCT_OK) {
	    (void) sprintf(buffer, "Error in instantiating %s at line %d",
		    cell->name, cell->line);
	    octError(buffer);
	    exit(-1);
	}
    }
}

static char buffer[128];

char *fix_name();

char *
make_cell_name(cell)
struct cif_cell *cell;
{
    static cellNumber = 1;
    char *first_geo = cell->geos.head;
    
    if (CIF_TYPE(first_geo) == CIF_USER_EXTENSION) {
	struct cif_user_extension *ext =
		(struct cif_user_extension *) first_geo;
	
	if (ext->number == 9) {
	    (void) strcpy(buffer, ext->text);
	    return fix_name(buffer);
	}
    }
    (void) sprintf(buffer, "Symbol%d", cellNumber++);
    return buffer;
}
	

/*
 * remove leading blanks and any trailing words after the first in string
 */

char *fix_name(string)
char *string;
{
    char *ptr, *start;

    ptr = string;
    
    while (isspace(*ptr)) {
	ptr++;
    }
    start = ptr;
    while (*ptr != 0 && !isspace(*ptr)) {
	ptr++;
    }
    *ptr = '\0';
    return start;
}
