#!/bin/csh -f
#
set B = "fabprep:"

if ($?OCTTOOLS) then
        set octtools = $OCTTOOLS
else
	echo $B "Error: Environment variable OCTTOOLS is not set."
	exit 1
endif


if ( $?VOV_HOST_NAME ) then
	echo $B "I must disable tracing. Use vov_fabprep if you want tracing." 
	unsetenv VOV_HOST_NAME
endif

set SCRIPTS = scmos/octmm-scripts

set fullusage = 0
set usage = 0
set flatten = 1
set altCIFName = 0
set outViewName = selects
set well_type = nwell
set microns = 2u

while ($#argv)
	switch($1)
	case -o:
		set outViewName = $2
		shift
		breaksw
	case -P:
		set flatten = 0
		breaksw
	case -mic:
		set microns = $2
		shift
		breaksw
	case -cif:
		set cifName = $2
		set altCIFName = 1
		shift
		breaksw
	case -nwell:
		set well_type = nwell
		breaksw
	case -pwell:
		set well_type = pwell
		breaksw
	case -twinwell:
		set well_type = twinwell
		breaksw
	case -help:

		set fullusage = 1
		goto usage
		breaksw
	default:
        set option = `echo $1 | sed 's/^-/this is an option/'`
        if ( $#option > 1 ) then
            echo $B "unknown option" $1
	goto usage
        else 
            break
        endif
 	endsw
	shift
end

if ($#argv < 1) then
goto usage
endif

set cell = `echo $1 | sed 's/:.*//'`
set flatView = `echo $1 | sed 's/.*://'`

if (${altCIFName} == 0) then
set cifName = ${cell}.cif
endif


echo $B "Technology = ${microns}m scmos (${well_type})."
echo $B "CIF file name will be ${cifName}."
if (${flatten}) then
   echo $B Flattening $1.
   octflatten -t PHYSICAL -o :flat-select $1 >& /dev/null
   set flatView = flat-select
endif

echo $B "Adding select layers and fixing wells"
octmm -f ${SCRIPTS}/${well_type}.fabprep -o :$outViewName $cell\:$flatView
echo $B "Running vulcan"
vulcan -k ${cell}\:${outViewName}
echo $B "Creating bdnet file"
set BDNETFILE = ${cell}_addselects.bdnet
echo MODEL ${cell}\:fabprep \;  > $BDNETFILE
echo TECHNOLOGY scmos \; >> $BDNETFILE
echo VIEWTYPE SYMBOLIC \; >> $BDNETFILE
echo EDITSTYLE SYMBOLIC \; >> $BDNETFILE
echo INSTANCE $1 PROMOTE \; >> $BDNETFILE
echo INSTANCE ${cell}\:${outViewName} PROMOTE \; >> $BDNETFILE
echo ENDMODEL \; >> $BDNETFILE
echo $B "Running bdnet"
bdnet $BDNETFILE
echo $B "Running octtocif"
octtocif -f ${SCRIPTS}/${well_type}.fabprep.${microns} -o ${cifName} ${cell}\:fabprep

if ($flatten) then 
   goto cleanup
endif
exit 0

usage:
echo "Usage is: fabprep [options] cell:view"
if ($fullusage) then
echo "Options are"
echo "     -o :view   Specify the output view name. (:selects is the default)"
echo "     -P         Don't flatten the input cell. This should"
echo "                be used ONLY if the input cell is already a"
echo "                flat PHYSICAL view."
echo "     -cif name  Specify name for CIF file. (Default = cell.cif)."
echo "     -nwell     Use nwell CIF conversion script (default)."
echo "     -pwell     Use pwell CIF conversion script."
echo "     -twinwell  Use twinwell CIF conversion script."
echo "     -mic Nu    Set process geometry to N microns (default = 2u)."
echo "                (2u is also the only technology for now)."
echo "     -help      Print this message."
else 
echo "Use: fabprep -help for more information."
endif
exit 0

cleanup:
    echo $B "Cleaning up intermediate views and bdnet file."
    rm -rf $BDNETFILE
    rm -rf $cell\/flat-select
exit 0
