#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "oct.h"
#include "tr.h"
#include "oh.h"
#include "vov.h"
#include "errtrap.h"
#include "options.h"
#include "plotter.h"
#include "xforms.h"

double scale;
int xmin;
int ymin;
int xoffset;
int yoffset;
int userxminp = 0;
int useryminp = 0;
char *plotter = PLOTTERNAME;

extern void init();
extern void cleanup();
extern void move();
extern void draw();
extern void getPen();
extern void sendcoord();

int manhatWarn = 0; /* lets only warn once when we get a non-manhattan path */

struct colors {
    char name[128]; /* name of color */
    int pen;        /* pen number */
    int fillType;    /* fill/outline/both/neither*/
    int lineType;    /* line type */
    int fillAngle;    /* angle of fill lines */
    int fillSpace;    /* spacing (in .025 mm) between fill lines */
} colors[200];

int maxColors = 0;
int colorp = 0;

#define AUTOMATIC 0
#define PORTRAIT 1
#define LANDSCAPE 2
#define YES 1
#define NO 0


optionStruct optionList[] = {
    { "P",	"plotter",	"name of the plotter" },
    { "p",	0,		"plot in portrait mode, scaling to fit on the " },
    { OPT_CONT,	0,		"page or in the box defined by -b" },
    { "l",	0,		"plot in landscape mode, scaling to fit on the " },
    { OPT_CONT,	0,		"page or in the box defined by -b" },
    { "a",	0,		"choose -l or -p, which ever gives the biggest plot (default)" },
    { "n",	0,		"do not put the name on the plot" },
    { "s",	"scale",	"multiply the scale facet determined by -a, -l, or -p " },
    { OPT_CONT,	0,		"by `scale' (defaults to 1.0)" },
    { "c",	"colortable",	"name of the file that specifies pin numbers and fill information" },
    { OPT_CONT,	0,		"for the layers" },
    { "x",	"x",		"lower left coordinate value in oct units (x)" },
    { "y",	"y",		"lower left coordinate value in oct units (y)" },
    { "X",	"X",		"offset in 40ths of millimeters (x)" },
    { "X",	"X",		"offset in 40ths of millimeters (x)" },
    { OPT_RARG,	"cell:view[:facet]",	"facet defaults to `contents'" },
    { 0,	0,		0 }
};


main(argc,argv)
int argc;
char **argv;
{
    octObject facet, prop;
    octBox bb;
    double scalep, scalel, scalexp, scaleyp, scalexl, scaleyl;
    int i = 0;
    int mode = AUTOMATIC;
    int userscalep = 0;
    FILE *fcolors;
    char file[4096];
    int option;
    int header = YES;
    extern int optind;
    extern char *optarg;
    char linebuf[1000];

    xoffset = 0;
    yoffset = 0;

    while ((option = optGetOption(argc, argv)) != EOF) {
        switch (option) {
        case 'a':
            mode = AUTOMATIC;
            break;
        case 'p':
            mode = PORTRAIT;
            break;
        case 'l':
            mode = LANDSCAPE;
            break;
        case 'n':
            header=NO;
            break;
        case 's':
            scale = atof(optarg);
            userscalep = 1;
            break;
        case 'X':
            xoffset += atoi(optarg);
            break;
        case 'Y':
            yoffset += atoi(optarg);
            break;
        case 'x':
            xmin = atoi(optarg);
            userxminp = 1;
            break;
        case 'y':
            ymin = atoi(optarg);
            useryminp = 1;
            break;
        case 'c':
            colorp = 1;
            (void) strcpy(file, optarg);
            break;
        case 'P':
            plotter = optarg;
            break;
        }
    }

    if (argc == optind+1) {
	ohUnpackDefaults(&facet, "r", "::contents");
	if (ohUnpackFacetName(&facet, argv[optind]) != OCT_OK) {
	    optUsage();
	}
    } else {
        optUsage();
    }

    VOVbegin( argc, argv );

    VOVinputFacet( &facet );
    OH_ASSERT(octOpenFacet(&facet));

    if(colorp == 1) {
        /* user specified colormap file */
        if ((fcolors = VOVfopen(file, "r")) == NULL) {
            perror(file);
            VOVend(-1);
        }
    } 
    else {
        /* system colormap file */
        OH_ASSERT(ohGetByPropName(&facet,&prop,"TECHNOLOGY"));
        if(prop.contents.prop.type != OCT_STRING) {
            errRaise("hpgl",-1, "TECHNOLOGY property isn't a string!\n");
        }
        (void) sprintf(file,"%s/tech/%s/%s.map",
		       util_file_expand("$OCTTOOLS"),
		       prop.contents.prop.value.string,
		       PLOTTERTYPE);
        if ((fcolors = VOVfopen(file, "r")) == NULL) {
            perror(file);
            VOVend(-1);
        }
    }

    while (fgets(linebuf,1000,fcolors)) {
        if(linebuf[0]=='#') {
            /* comment */
        } else if (sscanf(linebuf, "%s %d %d %d %d %d\n",
                          colors[i].name,
			  &colors[i].pen,
			  &colors[i].fillType,
			  &colors[i].lineType,
			  &colors[i].fillAngle,
			  &colors[i].fillSpace) == 6) {
            /* valid line */
            i++;
        } else {
            (void) fprintf(stderr, "bad line in colormap file ignored:\n%s\n",linebuf);
        }
    }
    maxColors = i;

    OH_ASSERT(octBB(&facet, &bb));

    stack = tr_create();

    scalexp = (MAXX) / (bb.upperRight.x - bb.lowerLeft.x);
    scaleyp = (MAXY) / (bb.upperRight.y - bb.lowerLeft.y);
    scalep = MIN(scalexp, scaleyp);

    scalexl = (MAXY) / (bb.upperRight.x - bb.lowerLeft.x);
    scaleyl = (MAXX) / (bb.upperRight.y - bb.lowerLeft.y);
    scalel = MIN(scalexl, scaleyl);

    if (mode == AUTOMATIC) {
        if (scalel > scalep) {
            tr_do_op(stack,TR_ROT90,1);
            mode=PORTRAIT;
            if (userscalep == 0) {
                scale = scalel;
            }
            if(userxminp ==0) {
                xmin = -bb.upperRight.y;
            } 
            else {
                xmin = -xmin;
            }
            if(useryminp ==0) {
                ymin = bb.lowerLeft.x;
            }
            if(scalexl > scale) {
                ymin -= MAXY/(2*scale) - (bb.upperRight.x-bb.lowerLeft.x)/2;
            }
            if(scaleyl > scale) {
                xmin -= MAXX/(2*scale) - (bb.upperRight.y-bb.lowerLeft.y)/2;
            }
        } 
        else {
            mode=LANDSCAPE;
            if (userscalep == 0) {
                scale = scalep;
            }
            if(userxminp ==0) {
                xmin = bb.lowerLeft.x;
                if(scalexp > scale) {
                    xmin -= MAXX/(2*scale) - (bb.upperRight.x-bb.lowerLeft.x)/2;
                }
            }
            if(useryminp ==0) {
                ymin = bb.lowerLeft.y;
                if(scaleyp > scale) {
                    ymin -= MAXY/(2*scale) - (bb.upperRight.y-bb.lowerLeft.y)/2;
                }
            }
        }
    } 
    else if (mode == LANDSCAPE) {
        if (userscalep == 0) {
            scale = scalep;
        }
        if (userxminp == 0) {
            xmin = bb.lowerLeft.x;
        }
        if (useryminp  == 0) {
            ymin = bb.lowerLeft.y;
        }
    } 
    else {
        tr_do_op(stack,TR_ROT90,1);
        if (userscalep == 0) {
            scale = scalel;
        }
        if (userxminp  == 0) {
            xmin = -bb.upperRight.y;
        } 
        else {
            xmin = -xmin;
        }
        if (useryminp  == 0) {
            ymin = bb.lowerLeft.x;
        }
    }
    if ( mode==PORTRAIT) {
        (void) fprintf(stderr, "-x %d",-xmin);
    } else {
        (void) fprintf(stderr, "-x %d",xmin);
    }

    (void) fprintf(stderr, " -y %d -s %f\n",ymin,scale);
    init();

    if(header) {
        getPen(1);
        if(mode == LANDSCAPE) {
            move((int) (xmin-(200/scale)),ymin);
            (void) fprintf(PlotterFile,PORTLABELSETUP);
        } 
        else {
            move(xmin,(int) (ymin-(200/scale)));
            (void) fprintf(PlotterFile,LANDLABELSETUP);
        }
        (void) fprintf(PlotterFile,LABEL,facet.contents.facet.cell);
    }

    if (processCell(&facet) < OCT_OK) {

        cleanup();
        VOVend(-1);
    }
    cleanup();

    octEnd();
    VOVend( 0 );
}


int
processCell(facet)
octObject *facet;
{
    octObject instance, path, poly, box, circle, master, layer;
    static octPoint *points;
    static int pointsize = 0;
    octGenerator gen, congen;
    int  i;
    int32 numpoints;
    int pen;
    int fillType;
    int lineType;
    int fillAngle;
    int fillSpace;
    int colorCount = 0;

    if (pointsize == 0) {
	points = ALLOC(octPoint, 1024);
	pointsize = 1023;
    }

    for (;;) {
        if (colorCount < maxColors) {
            pen = colors[colorCount].pen;
            fillType = colors[colorCount].fillType;
            lineType = colors[colorCount].lineType;
            fillAngle = colors[colorCount].fillAngle;
            fillSpace = colors[colorCount].fillSpace;
            layer.type = OCT_LAYER;
            layer.contents.layer.name = colors[colorCount].name;
            colorCount++;
            if (octGetByName(facet, &layer) < OCT_OK) {
                /* layer does not exist, skip to the next one */
                continue;
            }
        } 
        else {
            break;
        }
        if(WhichPen != pen) getPen(pen);
        if(fillType & SOLID) {
            (void) fprintf(PlotterFile,"FT2;\n");
        } 
        else {
            (void) fprintf(PlotterFile,"FT3,%d,%d;\n",fillSpace,fillAngle);
        }
        if(lineType != 0) {
            (void) fprintf(PlotterFile,"LT%d;\n",lineType);
        } 
        else {
            (void) fprintf(PlotterFile,"LT;\n");
        }

        OH_ASSERT(octInitGenContents(&layer, OCT_PATH_MASK, &congen));

        while (octGenerate(&congen, &path) == OCT_OK) {
            int llx,lly,urx,ury;
            int lrx,lry,ulx,uly;
            int32 xx;
            double lx,ly,frac,width;

            numpoints = pointsize;
            (void) octGetPoints(&path, &numpoints, points);
            if(numpoints > pointsize) {
                FREE(points);
                points = ALLOC(octPoint, numpoints + 1024);
                pointsize = numpoints + 1023;
                (void) octGetPoints(&path,&numpoints,points);
            }

            if(path.contents.path.width <= 1) {
                MOVE(points[0].x,points[0].y);

                for (i = 1; i < numpoints; i++) {
                    DRAW(points[i].x,points[i].y,pen);
                }
            } 
            else {
                tr_transform(stack, (int32 *) &points[0].x, (int32 *) &points[0].y);
                xx = path.contents.path.width;
                tr_scaler_transform(stack,&xx);
                width = xx;
                for (i = 0; i < numpoints-1; i++) {
                    tr_transform(stack, (int32 *) &(points[i+1].x), (int32 *) &(points[i+1].y));
                    if(points[i].x == points[i+1].x) {
                        /* vertical manhattan */
                        urx=points[i].x+width/2;
                        llx=points[i].x-width/2;
                        if(points[i].y >points[i+1].y) {
                            if(i) {
                                ury=points[i].y+width/2;
                            } 
                            else {
                                ury=points[i].y;
                            }
                            lly = points[i+1].y;
                        } 
                        else {
                            if(i) {
                                lly = points[i].y-width/2;
                            } 
                            else {
                                lly = points[i].y;
                            }
                            ury = points[i+1].y;
                        }
                        if(fillType & FILL) {
                            if(lineType != 0) {
                                (void) fprintf(PlotterFile,"LT%d;\n",lineType);
                            } 
                            else {
                                (void) fprintf(PlotterFile,"LT;\n");
                            }
                            move(llx,lly);
                            (void) fprintf(PlotterFile,RECTFILL);
                            sendcoord(urx,ury);
                            (void) fprintf(PlotterFile,";\n");
                            (void) fprintf(PlotterFile,"LT;\n");
                        }
                        if(fillType & OUTLINE) {
                            move(llx,lly);
                            (void) fprintf(PlotterFile,RECTEDGE);
                            sendcoord(urx,ury);
                            (void) fprintf(PlotterFile,";\n");
                        }
                    } 
                    else if (points[i].y == points[i+1].y) {
                        ury = points[i].y+width/2;
                        lly = points[i].y-width/2;
                        if(points[i].x < points[i+1].x) {
                            if(i) {
                                llx = points[i].x-width/2;
                            } 
                            else {
                                llx = points[i].x;
                            }
                            urx = points[i+1].x;
                        } 
                        else {
                            if(i) {
                                urx = points[i].x + width/2;
                            } 
                            else {
                                urx = points[i].x;
                            }
                            llx = points[i+1].x;
                        }
                        if(fillType & FILL) {
                            if(lineType != 0) {
                                (void) fprintf(PlotterFile,"LT%d;\n",lineType);
                            } 
                            else {
                                (void) fprintf(PlotterFile,"LT;\n");
                            }
                            move(llx,lly);
                            (void) fprintf(PlotterFile,RECTFILL);
                            sendcoord(urx,ury);
                            (void) fprintf(PlotterFile,";\n");
                            (void) fprintf(PlotterFile,"LT;\n");
                        }
                        if(fillType & OUTLINE) {
                            move(llx,lly);
                            (void) fprintf(PlotterFile,RECTEDGE);
                            sendcoord(urx,ury);
                            (void) fprintf(PlotterFile,";\n");
                        }
                    } 
                    else {
                        if(!manhatWarn) {
                            (void) fprintf(stderr, "Warning: non-manhattan path(s) - corners may not match \n");
                            manhatWarn = 1;
                        }
                        frac = (width/2) / sqrt(
                        (double)(points[i].x-points[i+1].x)*
                            (double)(points[i].x-points[i+1].x)+
                            (double)(points[i].y-points[i+1].y)*
                            (double)(points[i].y-points[i+1].y));
                        lx = frac * (points[i+1].y-points[i].y);
                        ly = -frac * (points[i+1].x-points[i].x);
                        urx = points[i+1].x + lx;
                        ury = points[i+1].y + ly;
                        ulx = points[i+1].x - lx;
                        uly = points[i+1].y - ly;
                        llx = points[i].x - lx;
                        lly = points[i].y - ly;
                        lrx = points[i].x + lx;
                        lry = points[i].y + ly;
                        move(llx,lly);
                        (void) fprintf(PlotterFile,POLYMODE);
                        draw(lrx,lry,pen);
                        draw(urx,ury,pen);
                        draw(ulx,uly,pen);
                        (void) fprintf(PlotterFile,POLYDONE);
                        if(fillType & FILL) {
                            if(lineType != 0) {
                                (void) fprintf(PlotterFile,"LT%d;\n",lineType);
                            } 
                            else {
                                (void) fprintf(PlotterFile,"LT;\n");
                            }
                            (void) fprintf(PlotterFile,FILLPOLY);
                            (void) fprintf(PlotterFile,"LT;\n");
                        }
                        if(fillType & OUTLINE) {
                            move(llx,lly);
                            (void) fprintf(PlotterFile,EDGEPOLY);
                        }
                    }
                }
            }
        }

        OH_ASSERT(octInitGenContents(&layer, OCT_CIRCLE_MASK, &congen));

        while (octGenerate(&congen, &circle) == OCT_OK) {

            /* have to handle circles */
            do_circle(circle.contents.circle.center.x,
            circle.contents.circle.center.y,
            circle.contents.circle.innerRadius,
            circle.contents.circle.outerRadius,
            circle.contents.circle.startingAngle,
            circle.contents.circle.endingAngle,pen,fillType,lineType);

        }

        OH_ASSERT(octInitGenContents(&layer, OCT_POLYGON_MASK, &congen));

        while (octGenerate(&congen, &poly) == OCT_OK) {

            numpoints = pointsize;
            (void) octGetPoints(&poly, &numpoints, points);
            if(numpoints > pointsize) {
                FREE(points);
                points = ALLOC(octPoint, numpoints + 1024);
                pointsize = numpoints + 1023;
                (void) octGetPoints(&poly,&numpoints,points);
            }
            for(i=0;i<numpoints;i++) {
                tr_transform(stack,(int32 *) &points[i].x,(int32 *) &points[i].y);
            }
            move(points[0].x, points[0].y);
            (void) fprintf(PlotterFile,POLYMODE);
            for (i = 1; i < numpoints; i++) {
                draw(points[i].x, points[i].y,pen);
            }
            draw(points[0].x, points[0].y,pen);
            (void) fprintf(PlotterFile,POLYDONE);

            if(fillType & FILL) {
                if(lineType != 0) {
                    (void) fprintf(PlotterFile,"LT%d;\n",lineType);
                } 
                else {
                    (void) fprintf(PlotterFile,"LT;\n");
                }
                (void) fprintf(PlotterFile,FILLPOLY);
                (void) fprintf(PlotterFile,"LT;\n");
            }
            if(fillType & OUTLINE) {
                (void) fprintf(PlotterFile,EDGEPOLY);
            }

        }

        OH_ASSERT(octInitGenContents(&layer, OCT_BOX_MASK, &congen));

        while (octGenerate(&congen, &box) == OCT_OK) {


            if(fillType & FILL) {
                if(lineType != 0) {
                    (void) fprintf(PlotterFile,"LT%d;\n",lineType);
                } 
                else {
                    (void) fprintf(PlotterFile,"LT;\n");
                }
                MOVE( box.contents.box.lowerLeft.x, box.contents.box.lowerLeft.y);
                (void) fprintf(PlotterFile,RECTFILL);
                OUTPT(box.contents.box.upperRight.x,
                box.contents.box.upperRight.y);
                (void) fprintf(PlotterFile,";\n");
                (void) fprintf(PlotterFile,"LT;\n");
            } 
            else if (fillType & CROSS) {
                (void) fprintf(PlotterFile,"LT;\n");
                MOVE( box.contents.box.lowerLeft.x, box.contents.box.lowerLeft.y);
                DRAW(box.contents.box.upperRight.x, 
                box.contents.box.upperRight.y,pen);
                MOVE( box.contents.box.upperRight.x, box.contents.box.lowerLeft.y);
                DRAW(box.contents.box.lowerLeft.x, 
                box.contents.box.upperRight.y,pen);
            }
            if(fillType & OUTLINE) {
                MOVE( box.contents.box.lowerLeft.x, box.contents.box.lowerLeft.y);
                (void) fprintf(PlotterFile,RECTEDGE);
                OUTPT(box.contents.box.upperRight.x,
                box.contents.box.upperRight.y);
                (void) fprintf(PlotterFile,";\n");
            }


        }
    }

    OH_ASSERT(octInitGenContents(facet, OCT_INSTANCE_MASK, &gen));

    while (octGenerate(&gen, &instance) == OCT_OK) {
        if (openMaster(&instance, &master, "r") < OCT_OK) {
            octError("master open");
            return(OCT_ERROR);
        }

        /* tr stuff - yucky */
        tr_push(stack);
        tr_add_transform(stack, &instance.contents.instance.transform, 0);

        if (processCell(&master) < OCT_OK) {
            return(OCT_ERROR);
        }

        tr_pop(stack);
    }

    return(OCT_OK);
}


octStatus
openMaster(instance, master, mode)
octObject *instance;
octObject *master;
char *mode;
{
    master->type = OCT_FACET;
    master->contents.facet.facet = "contents";
    master->contents.facet.mode = mode;

    return(octOpenMaster(instance, master));
}

