#include "port.h"
#include "oct.h"
#include "mag2oct.h"

static    int    labelnum = 0;        /* non-decreasing label number */
static    char    labelname[16];        /* Used for ASCII of labelnum */

void
addlabel(facet, buf)
octObject    *facet;
char        *buf;
{
    int        n;
    char        label[BUFSIZ], labellayer[BUFSIZ];
    octObject    box;
    octObject    layer;
    octObject    prop;
    octObject    octlabel;

    if ((n = sscanf(buf, "rlabel %s %d %d %d %d %*d %s", labellayer,
            &box.contents.box.ll.x, &box.contents.box.ll.y,
            &box.contents.box.ur.x, &box.contents.box.ur.y,
            label)) != 6) {
        logmsg(-1, "parse error: %d != 6 (%s)", n, buf);
    }

    if (!keep_point_terminals &&
	(box.contents.box.ur.x - box.contents.box.ll.x) == 0 &&
        (box.contents.box.ur.y - box.contents.box.ll.y) == 0) {
	    return;
    }

    box.contents.box.ll.x *= LAM_SCALE;
    box.contents.box.ll.y *= LAM_SCALE;
    box.contents.box.ur.x *= LAM_SCALE;
    box.contents.box.ur.y *= LAM_SCALE;

    layer.type = OCT_LAYER;
    /* Place obox's on the overlap layer */
    layer.contents.layer.name = strcmp("obox", label) ?
                    labellayer : "overlap";

    if (octGetOrCreate(facet, &layer) != OCT_OK) {
        logmsg(-1, "Can not find layer: %s: %s\n",
            labellayer, octErrorString());
        return;
    }

    box.type = OCT_BOX;

    if (strcmp("space", labellayer) == 0) {
        /* No terminals necessary */

        if (octCreate(&layer, &box) != OCT_OK) {
            logmsg(-1, "Can not create label box: %s\n",
                octErrorString());
            return;
        }

        if (strcmp("obox", label) == 0)
            return;

        prop.type = OCT_PROP;
        prop.contents.prop.name = label;
        prop.contents.prop.type = OCT_NULL;

        if (octCreate(&box, &prop) != OCT_OK) {
            logmsg(-1, "Can not create prop for space lable: %s\n", label);
        }
        return;
    }

    (void) sprintf(labelname, "MAGIC#%d", labelnum++);

    octlabel.type = OCT_LABEL;
    octlabel.contents.label.label = labelname;
    octlabel.contents.label.region.upperRight.x = box.contents.box.ur.x;
    octlabel.contents.label.region.upperRight.y = box.contents.box.ur.y;
    octlabel.contents.label.region.lowerLeft.x = box.contents.box.ll.x;
    octlabel.contents.label.region.lowerLeft.y = box.contents.box.ll.y;
    octlabel.contents.label.textHeight = 20;
    octlabel.contents.label.vertJust = OCT_JUST_BOTTOM;
    octlabel.contents.label.horizJust = OCT_JUST_LEFT;
    octlabel.contents.label.lineJust = OCT_JUST_LEFT;

    if (octCreate(&layer, &octlabel) != OCT_OK) {
        logmsg(-1, "Can not create label box: %s\n", octErrorString());
        return;
    }

    prop.type = OCT_PROP;
    prop.contents.prop.name = label;
    prop.contents.prop.type = OCT_NULL;

    if (octCreate(&octlabel, &prop) != OCT_OK) {
	logmsg(-1, "can not create prop on label %s\n", labelname);
    }
    return;
}
