#include "port.h"
#include "oct.h"
#include "mag2oct.h"
#ifdef    GROW_OPTION
#include "region.h"
#ifdef    DO_TAP
#include "tap.h"
#endif
#endif

/*ARGSUSED*/
static void
convert(facet, layer, label, minsize)
octObject    *facet;
octObject    *layer;
octObject    *label;
octCoord    minsize;
{
    octObject    prop;
    octObject    terminal;

    if (octGenFirstContent(label, OCT_PROP_MASK, &prop) != OCT_OK) {
	logmsg(-1, "No prop on label %s: %s",
			label->contents.label.label, octErrorString());
	return;
    }

    terminal.type = OCT_TERM;
    terminal.contents.term.name = prop.contents.prop.name;
    terminal.contents.term.instanceId = oct_null_id;
    terminal.contents.term.width = 1;

    if (octGetOrCreate(facet, &terminal) != OCT_OK) {
        logmsg(-1, "Can not create terminal: %s\n", octErrorString());
        return;
    }

#ifdef    GROW_OPTION
    if ((label->contents.label.region.upperRight.x
       - label->contents.region.lowerLeft.x) < minsize
     || (label->contents.label.region.upperRight.y 
       - label->contents.region.lowerLeft.y) < minsize) {

        regGenerator    gen;
        octBox    area;
        octObject    *oldbox;

        area.ll.x = label->contents.label.region.lowerLeft.x;
        area.ll.y = label->contents.label.region.lowerLeft.y;
        area.ur.x = label->contents.label.region.upperRight.x;
        area.ur.y = label->contents.label.region.upperRight.y;

        if ((gen = regInit(layer, &area, OCT_BOX_MASK)) == 0) {
            logmsg(-1, "Can not create regional generator: %s\n", regError());
            return;
        }

        while (oldbox = regNext(gen, 0, 0, 0)) {
            if (octAttach(&terminal, oldbox) != OCT_OK) {
                logmsg(-1, "Can't attach terminal to existing box: %s\n",
                    octErrorString());
            }
        }

        regEnd(gen);
    } else
#endif
        {
        octObject    box;

        box.type = OCT_BOX;
        box.contents.box.ll.x = label->contents.label.region.lowerLeft.x;
        box.contents.box.ll.y = label->contents.label.region.lowerLeft.y;
        box.contents.box.ur.x = label->contents.label.region.upperRight.x;
        box.contents.box.ur.y = label->contents.label.region.upperRight.y;

        if (octCreate(layer, &box) != OCT_OK) {
            logmsg(-1, "Can not create terminal box: %s\n", octErrorString());
            return;
        }

        if (octAttach(&terminal, &box) != OCT_OK) {
            logmsg(-1, "Can't attach terminal to box: %s\n", octErrorString());
            return;
        }
    }
    if (octDelete(&prop) != OCT_OK) {
        logmsg(1, "Could not delete prop while fixing labels\n");
    }
    if (octDelete(label) != OCT_OK) {
        logmsg(1, "Could not delete label while fixing labels\n");
    }
    return;
}

void
fixlabels(facet)
octObject    *facet;
{
    extern    char    grow_terminals;
    octGenerator    layergen;
    octGenerator    labelgen;
    octObject    layer;
    octObject    label;
    octCoord    minsize = 0;

    if (octInitGenContents(facet, OCT_LAYER_MASK, &layergen) != OCT_OK) {
        logmsg(-1, "%s: Can't init layer generator on %s: %s\n",
            facet->contents.facet.cell, octErrorString());
        return;
    }

    while (octGenerate(&layergen, &layer) == OCT_OK) {
#ifdef    GROW_OPTION
        if (grow_terminals) {
#ifdef    DO_TAP
            minsize = tapGetMinWidth(&layer);
#else
            minsize = 2 * LAM_SCALE;
#endif
        } else {
            minsize = 0;
        }
#endif

        (void) octInitGenContents(&layer, OCT_LABEL_MASK, &labelgen);
        while (octGenerate(&labelgen, &label) == OCT_OK) {
            if (!strncmp(label.contents.label.label, "MAGIC#", 6)) {
                convert(facet, &layer, &label, minsize);
            }
        }
        (void) octFreeGenerator(&labelgen);
    }
    (void) octFreeGenerator(&layergen);
    return;
}
