#include "port.h"
#include <sys/stat.h>

#include "mag2oct.h"
#include "oct.h"

extern    char    local;

#define    FIND_TECH    1
#define    FIND_TIMESTAMP    2
#define    FIND_GROUP    3
#define    FIND_LABELS    4
#define    FIND_RECTS    5
#define    FIND_CHECKPAINT    6


static    octStatus
openfacet(facet)
octObject    *facet;
{
    octStatus    retval;
    char        *p;

    /*
     *    Return if open is successful
     *        or if already testing the locally located facet
     */

    if ((retval = octOpenFacet(facet)) >= 0 ||
        local || (p = strrchr(facet->contents.facet.cell, '/')) == NULL)
        return retval;

    p++;

    logmsg(1, "Couldn't open facet %s -- trying %s\n", facet->contents.facet.cell, p);

    /* WARNING: This is evil since p is in facet.contents.facet.cell */
    (void) strcpy(facet->contents.facet.cell, p);

    return octOpenFacet(facet);
}

char *
input(filename)
char    *filename;
{
    FILE        *fp;
    char        *magfile;
    char        buf[BUFSIZ];
    char        facetname[BUFSIZ];
    char        octfile[BUFSIZ];
    static    char    octfile_return[BUFSIZ];
    int        state = 0;
    octObject    facet;
    octObject    prop;
    char        *cellclass = "LEAF";
    char        *p;
    struct    stat    magstat;
    struct    stat    octstat;
    static    char    curlayer[BUFSIZ];    /* Only one is needed */
    extern    char    *findfile();
    extern    char    *getinput();
    extern    char    octmm_file[];
    extern    long    force;
    extern    void    oct_oct_mask_mod();
    extern    void    fixlabels();
    extern    void    addlabel();

    
    if ((p = strrchr(filename, '.')) == 0 || strcmp(".mag", p)) {
        logmsg(1, "Appending .mag to %s\n", filename);
        (void) strcat(filename, ".mag");
    }
        
    if ((magfile = findfile(filename)) == NULL) {
        logmsg(-1, "%s not found in magic search path.\n", filename);
        return NULL;
    }

    (void) strcpy(facetname, local
                ? ((p = strrchr(filename, '/')) ? p+1 : filename)
                : magfile);

    /* There is alway a .mag at the end by this time */
    *strrchr(facetname, '.') = '\0';

    facet.type = OCT_FACET;
    facet.contents.facet.view = "physical";
    facet.contents.facet.facet = "contents";
    facet.contents.facet.version = OCT_CURRENT_VERSION;
    facet.contents.facet.mode = "w";
    facet.contents.facet.cell = facetname;

#ifdef    DEBUG
    if (debug)
        logmsg(0, "Checking date on magic file: %s\n", magfile);
#endif

    if (stat(magfile, &magstat)) {
        extern    int    errno;
        logmsg(-1, "Can not stat magic file: %s (errno = %d)\n",
                magfile, errno);
    }

    (void) sprintf(octfile, "%s/%s/%s;",    facet.contents.facet.cell,
                    facet.contents.facet.view,
                    facet.contents.facet.facet);

#ifdef    DEBUG
    if (debug)
        logmsg(0, "Checking date on oct file: %s\n", octfile);
#endif

    if (stat(octfile, &octstat) == 0) {
        if (magstat.st_mtime < octstat.st_mtime) {
            logmsg(0, "%s (%d) older than %s (%d)\n",
                        magfile, magstat.st_mtime,
                        octfile, octstat.st_mtime);
            if (force && octstat.st_mtime < force) {
                logmsg(0, "Forcing update on %s\n",
                        facet.contents.facet.cell);
            } else {
                (void) strcpy(octfile_return,
                        facet.contents.facet.cell);
                return octfile_return;
            }
        }
    }

#ifdef    DEBUG
    if (debug)
        logmsg(0, "Opening magic file: %s\n", magfile);
#endif

    if ((fp = fopen(magfile, "r")) == NULL) {
        extern    int    errno;
        logmsg(-1, "Can not open magic file for reading: %s (errno = %d)\n",
                magfile, errno);
    }

    logmsg(0, "Processing %s\n", filename);

#ifdef    DEBUG
    if (debug)
        logmsg(0, "octOpenFacet:  facet.cell: %s\n",
            facet.contents.facet.cell);
#endif

    if (openfacet(&facet) < 0)
        logmsg(-1, "%s: octOpenFacet %s: %s\n",
                filename, facet.contents.facet.cell,
                octErrorString());

    while (getinput(buf, sizeof buf, fp)) {
        switch (state) {
        case 0:
            if (strncmp("magic", buf, 5))
                logmsg(-1, "%s: not a magic file\n", filename);

            state = FIND_TECH;
            continue;

        case FIND_TECH:
            if (! strncmp("tech", buf, 4)) {
                /* Work on tech... */
                state = FIND_TIMESTAMP;
                continue;
            }
/****/            /* tech is nmos by default */

            /* Fall though */

        case FIND_TIMESTAMP:
            state = FIND_GROUP;
            if (! strncmp("timestamp", buf, 9)) {
                prop.type = OCT_PROP;
                prop.contents.prop.name = "TIMESTAMP";
                prop.contents.prop.type = OCT_INTEGER;
                prop.contents.prop.value.integer =
                        atoi(&buf[9]);

                if (octCreate(&facet, &prop) != OCT_OK) {
                    logmsg(1, "Facet TIMESTAMP: %s\n",
                        octErrorString());
                    /* Continue on anyway... */
                }
                continue;
            }
            break;
        }

        /* Check for group headers (there are only 4) */

        if (! strncmp("<<", buf, 2)) {
            register int    i = 3;

            while (buf[i] && buf[i] != '\n' &&
                        strncmp(" >>", &buf[i], 3)) {
                curlayer[i-3] = buf[i];
                i++;
            }
            curlayer[i-3] = '\0';

            if (! (buf[i] && buf[i] != '\n')) {
                logmsg(1, "%s: Possible bad layer name: %s\n",
                    filename, curlayer);
            }

            if (strcmp(curlayer, "end") == 0)
                break;

            if (strcmp(curlayer, "checkpaint") == 0) {
                state = FIND_CHECKPAINT;
            } else if (strcmp(curlayer, "labels") == 0) {
                state = FIND_LABELS;
            } else {
                state = FIND_RECTS;
            }
        } else if (! strncmp("use", buf, 3)) {
#ifdef    DEBUG
            if (debug)
                logmsg(0, "%s: use file: %s\n", buf);
#endif
            subcell(&facet, filename, fp, buf);
            cellclass = "MODULE";
            state = FIND_GROUP;
        } else {
            octObject    box;
            box.type = OCT_BOX;
            switch (state) {
            case FIND_RECTS:
                if (strncmp("rect", buf, 4)) {
                    logmsg(1, "Unusual object in layer %s: %s",
                            curlayer, buf);
                } else if (strcmp(curlayer, "pad")) {
                    if (sscanf(buf, "rect %d %d %d %d\n",
                        &box.contents.box.ll.x,
                        &box.contents.box.ll.y,
                        &box.contents.box.ur.x,
                        &box.contents.box.ur.y) != 4) {
                        logmsg(1, "rect parse error: %s",
                                buf);
                    } else {
/** ** Start weird indentation ** **/
            octObject    layer;

            layer.type = OCT_LAYER;
            layer.contents.layer.name = curlayer;

            box.contents.box.ll.x *= LAM_SCALE;
            box.contents.box.ll.y *= LAM_SCALE;
            box.contents.box.ur.x *= LAM_SCALE;
            box.contents.box.ur.y *= LAM_SCALE;

            if (octGetOrCreate(&facet, &layer) != OCT_OK) {
                logmsg(-1, "Can't create layer: %s: %s\n",
                        curlayer, octErrorString());
            } else if (octCreate(&layer, &box) != OCT_OK) {
                logmsg(-1, "Can't create a box on layer: %s %s:\n",
                        curlayer, octErrorString());
            }
/** ** End weird indentation ** **/
                    }
                }
                break;
            case FIND_LABELS:
                if (strncmp("rlabel", buf, 6)) {
                    logmsg(1, "Unusual label: %s", buf);
                } else {
                    addlabel(&facet, buf);
                }
                break;
            case FIND_CHECKPAINT:
                /* Do nothing -- throw away */
                break;
            default:
                logmsg(1, "%s: Unknown group: %s", filename, buf);
            }
        }
    }

    oct_mask_mod(arg0, octmm_file, &facet, (char *)0);
    fixlabels(&facet);

    /* Now add a series of props to the facet */

    prop.type = OCT_PROP;
    prop.contents.prop.type = OCT_STRING;

    prop.contents.prop.name = "CELLCLASS";
    prop.contents.prop.value.string = cellclass;

    if (octCreate(&facet, &prop) != OCT_OK) {
        logmsg(1, "Can't create %s prop on facet: %s\n",
                prop.contents.prop.name,
                octErrorString());
        /* Continue on anyway... */
    }

    prop.contents.prop.name = "EDITSTYLE";
    prop.contents.prop.value.string = "PHYSICAL";

    if (octCreate(&facet, &prop) != OCT_OK) {
        logmsg(1, "Can't create %s prop on facet: %s\n",
                prop.contents.prop.name,
                octErrorString());
        /* Continue on anyway... */
    }

    prop.contents.prop.name = "TECHNOLOGY";
    prop.contents.prop.value.string = "scmos";

    if (octCreate(&facet, &prop) != OCT_OK) {
        logmsg(1, "Can't create %s prop on facet: %s\n",
                prop.contents.prop.name,
                octErrorString());
        /* Continue on anyway... */
    }

    prop.contents.prop.name = "VIEWTYPE";
    prop.contents.prop.value.string = "PHYSICAL";

    if (octCreate(&facet, &prop) != OCT_OK) {
        logmsg(1, "Can't create %s prop on facet: %s\n",
                prop.contents.prop.name,
                octErrorString());
        /* Continue on anyway... */
    }

#ifdef    DEBUG
    if (debug)
        logmsg(0, "Closing:  mag: %s  facet: %s\n",
            magfile, facet.contents.facet.cell);
#endif

    (void) strcpy(octfile_return, facet.contents.facet.cell);
    if (octCloseFacet(&facet) != OCT_OK) {
        logmsg(-1, "Can't close facet: %s", facet.contents.facet.cell);
    }
    (void) fclose(fp);
    return octfile_return;
}
