#include "port.h"
#include "utility.h"
#include "options.h"
#include <pwd.h>

char    octmm_file[BUFSIZ];
char    *arg0;
char    verbose = 0;
int	keep_point_terminals = 0;
char    *logfile = "mag2oct.log";
FILE    *fp_logfile = 0;
int    exit_val = 0;
char    local = 0;
long    force = 0;    /* 0 = check normally, t = time to check against */
char    grow_terminals = 0;    /* grow terminals to minimum terminal size */

#ifdef    DEBUG
char    debug = 0;
#endif

optionStruct    optionList[] = {
#ifdef    DEBUG
    { "d",        "mask",        "turn debugging on (-1 for full debugging)" },
#else
    { "d",        "mask",        "debugging not compiled -- option ignored" },
#endif
    { "f",        "",        "force update regardless of mag/oct timestamp" },
#ifdef    GROW_OPTION
    { "g",        "",        "grow terminals to minimum oct terminal size (not working)" },
#endif
    { "l",        "",        "local -- force oct facet to be made in the current dir" },
    { "L",        "log_file",    "Name of file to open for logging warnings and errors" },
    { "p",        "",        "keep zero sized terminals" },
    { "v",        "",        "verbose mode" },
    { OPT_RARG,    "magic_file",    ".mag file to be processed" },
    { OPT_ELLIP,    0,        0 },
    { OPT_DESC,    0,        "Converts magic files to oct files." },
    { OPT_DESC,    0,        "Optional environment varable MAG2OCT used to set layer translation table file." },
    { 0,        0,        0 }
};

main(argc, argv)
int    argc;
char    *argv[];
{
    extern    long    time();
    extern    char    *input();
    extern    void    getpath();
    extern    char    *tapRootDirectory();
    int        option;
    char        *p;
    char        buffer[2048];

    arg0 = argv[0];

    while ((option = optGetOption(argc, argv)) != EOF) {
        switch (option) {
        case 'd':
#ifdef    DEBUG
            logmsg(0, "Debugging on: Mask: 0x%x\n",
                debug = atoi(optarg));
#else
            logmsg(0, "Debugging not compiled in -- option ignored\n");
#endif
            break;
        case 'f':
            force = time((long *)0);
            break;
        case 'g':
#ifdef    GROW_OPTION
            grow_terminals = 1;
            logmsg(0, "-g option is experimental (not finished)\n");
#else
            logmsg(0, "Grow option not compiled -- option ignored\n");
#endif
            break;
        case 'l':
            local = 1;
            break;
        case 'L':
            logfile = optarg;
            break;
        case 'v':
            verbose = 1;
            break;
        case 'p':
            keep_point_terminals = 1;
            break;
        default:
            optUsage();
        }
    }

    if (optind >= argc) {
        optUsage();
        logmsg(-1, "No magic files specified\n");
    }

    if ((p = getenv("MAG2OCT")) == NULL) {
        (void) sprintf(buffer, "%s/mag2oct.table", tapRootDirectory(NIL(char)));
        p = util_strsav(buffer);
    }

    (void) strcpy(octmm_file, util_file_expand(p));

    getpath();

    while (optind < argc)
        (void) input(argv[optind++]);

    exit(exit_val);
}
