#include "port.h"
#include "utility.h"
#include <sys/file.h>
#include "mag2oct.h"

#define    MAXPATHS    256

static    char    *paths[MAXPATHS];
static    char    **lastpath = paths;

void
readpath(fp)
FILE    *fp;
{
    register char    *p;
    register char    *path;
    char        buf[BUFSIZ];
    int        quote;

    /* Read ALL lines in file -- there could be more than one path line */

    while (fgets(buf, sizeof buf, fp) != NULL) {
        if (strncmp(buf, "path", 4) == 0) {
            p = buf+5;
            quote = 0;

            while (*p && *p != '\n' && (quote || *p != ' ')) {
                if (*p == '"') {
                    quote = !quote;
                    p++;
                    continue;
                }

                path = p;

                while (*p && *p != '\n' && *p != ':' &&
                        *p != '"' && *p != ' ')
                    p++;

                if (p != path) {
		    char buffer[2048];
                    /* from path to p[-1] is the string */

                    if (*path == '~') {
			(void) strncpy(buffer, path, (unsigned) (p - path));
			buffer[(unsigned) (p - path)] = '\0';
			*lastpath = util_strsav(util_file_expand(buffer));
                    } else {
			(void) strncpy(buffer, path, (unsigned) (p - path));
			buffer[(unsigned) (p - path)] = '\0';
			*lastpath = util_strsav(buffer);
                    }
                    if (access(*lastpath, X_OK)) {
                        /* access NOT OK */
                        extern    int    errno;

                        logmsg(0, "Can't access %s: errno = %d\n", *lastpath, errno);
                        FREE(*lastpath);
                    } else {
                        /* access OK */

                        lastpath++;
                    }
                }

                if (*p && (*p != ' ' || quote))
                    p++;
            }
        }
    }
}

/*
 *    getpath()
 *        See if the following files exits and look for paths in them:
 *            .magic
 *            $HOME/.magic    (if $HOME not set, look it up)
 *            ~cad/lib/magic/sys/.magic
 */

void
getpath()
{
    FILE        *fp;
    char        *p;
    char        *file;

    /* Search current directory for .magic file */

    if (fp = fopen(".magic", "r")) {
        readpath(fp);
        (void) fclose(fp);
    }

    /* Search home directory for .magic file */

    if ((p = getenv("HOME")) != NIL(char)) {
	char buffer[2048];

	(void) sprintf(buffer, "%s/.magic", p);
	file = util_strsav(util_file_expand(buffer));

        if (fp = fopen(file, "r")) {
            readpath(fp);
            (void) fclose(fp);
        }
	FREE(file);
    } else {
	file = util_strsav(util_file_expand("~/.magic"));

        if (fp = fopen(file, "r")) {
            readpath(fp);
            (void) fclose(fp);
        }
	FREE(file);
    }

    /* Search cad's lib/magic/sys directory for .magic file */
    file = util_strsav(util_tilde_expand("~cad/lib/magic/sys/.magic"));
    if (fp = fopen(file, "r")) {
	readpath(fp);
	(void) fclose(fp);
    }
    FREE(file);
    
    /* If no paths specified, add . as default */

    if (lastpath == paths)
        *lastpath++ = ".";
}

char *
findfile(magfile)
char    *magfile;
{
    static    char    filename[BUFSIZ];
    register char    **path;

    if (magfile[0] == '/')
        return magfile;

    for (path = paths; *path; path++) {
        (void) sprintf(filename, "%s/%s", *path, magfile);
        if (access(filename, R_OK) == 0)
            return filename;
    }
    return NULL;
}
