#include "port.h"
#include "utility.h"
#include "options.h"
#include "oct.h"
#include "oh.h"
#include "vulcan.h"

optionStruct optionList[] = {
    {"o",	"[cell]:view",	"output cell:view"},
    { OPT_RARG,	"cell:view",	"facet defaults to `contents'" },
    {0, 0, 0}};

main(argc, argv)
int argc;
char **argv;
{
    char *input_name, *output_name;
    octObject outfacet, facet, instance, newinstance, layer, geo, connBag;
    octObject ifacet, prop;
    octGenerator gen;
    int option;
    extern int optind;
    extern char *optarg;
    octId firstId;
    int physicalp = 0, schematicp = 0;

    /* parse command line options*/
    output_name = NIL(char);
    
    while ((option = optGetOption(argc, argv)) != EOF) {
	switch(option) {
	    case 'o':
		output_name = util_strsav(optarg);
		break;
	}
    }

    /*
     *  Finish the command line argument parsing 
     */
    if (optind != argc-1) {
	optUsage();
    } else {
	input_name = argv[optind];
    }

    octBegin();
    
    /*
     *  Get the input and output facet names
     */
    ohUnpackDefaults(&facet, "a", "::contents");
    OH_ASSERT(ohUnpackFacetName(&facet, input_name));

    /*
     *  Unpack the output name (if present), and make a copy of the facet
     */
    if (output_name != NIL(char)) {
	outfacet = facet;
	if (ohUnpackFacetName(&outfacet, output_name) == OCT_OK) {
	    /* Copy the contents facet */
	    facet.contents.facet.facet = "contents";
	    outfacet.contents.facet.facet = "contents";
	    if (octCopyFacet(&outfacet, &facet) != OCT_OK) {
		octError("copying the contents facet");
		exit(-1);
	    }
	    facet = outfacet;
	    facet.contents.facet.mode = "a";
	}
    }

    /* 
     *  Open the facet 
     */
    if (octOpenFacet(&facet) < OCT_OK) {
	octError("can not open the facet");
	exit(-1);
    }

    /* remove the connectors */
    if (ohGetByBagName(&facet, &connBag, "CONNECTORS") == OCT_OK) {
	OH_ASSERT(octInitGenContents(&connBag, OCT_INSTANCE_MASK, &gen));
	while (octGenerate(&gen, &instance) == OCT_OK) {
	    octDelete(&instance);
	}
    }

    /* remove the layers and geometry */
    OH_ASSERT(octInitGenContents(&facet, OCT_LAYER_MASK, &gen));
    while (octGenerate(&gen, &layer) == OCT_OK) {
        ohRecursiveDelete(&layer, OCT_GEO_MASK);
    }

    OH_ASSERT(octInitGenContents(&facet, OCT_INSTANCE_MASK, &gen));

    firstId = oct_null_id;
    while (octGenerate(&gen, &instance) == OCT_OK) {
	if (octIdsEqual(instance.objectId, firstId)) {
	    break;
	}
	newinstance = instance;
        if (strcmp(instance.contents.instance.view, "schematic") == 0) {
	    newinstance.contents.instance.view = "physical";
	    physicalp = 1;
        } else {
	    newinstance.contents.instance.view = "schematic";
	    newinstance.contents.instance.facet = "interface";
	    schematicp = 1;
        }
	ohReplaceInstance(&newinstance, &instance);
	if (octIdIsNull(firstId)) {
	    firstId = newinstance.objectId;
	}
    }

    /* add properties */
    if (schematicp && physicalp) {
        fprintf(stderr, "Bad Design: physical and schematic cells intermixed\n");
	exit(-1);
    }
    OH_ASSERT(ohCreateOrModifyPropStr(&facet, &prop, "TECHNOLOGY", "scmos"));
    if (physicalp) {
        OH_ASSERT(ohCreateOrModifyPropStr(&facet, &prop, "VIEWTYPE", "SYMBOLIC"));
        OH_ASSERT(ohCreateOrModifyPropStr(&facet, &prop, "EDITSTYLE", "SYMBOLIC"));
    } else {
        OH_ASSERT(ohCreateOrModifyPropStr(&facet, &prop, "VIEWTYPE", "SCHEMATIC"));
        OH_ASSERT(ohCreateOrModifyPropStr(&facet, &prop, "EDITSTYLE", "SCHEMATIC"));
    }

    ifacet = facet;
    ifacet.contents.facet.facet = "interface";
    ifacet.contents.facet.mode = "w";
    octOpenFacet(&ifacet);

    vulcan(&facet, &ifacet);

    OH_ASSERT(octCloseFacet(&facet));
    exit(0);
}

