#include "copyright.h"
#include "port.h"
#include "oct.h"
#include "oh.h"

extern FILE *output;
extern FILE *VOVfopen();

extern int next_cell;

extern int hierarchy_level;
extern int max_hierarchy;
extern int force_bad_timestamp;
extern int strip_octflat_instances;
extern char *layout_directory;

extern void error(), warning();
extern void inst_to_master(), add_cell();

extern octStatus flatten_contacts();

#define NIL(type) ((type *) 0);

#define FPRINTF	(void) fprintf

#define CIF_MASK \
    OCT_BOX_MASK|OCT_CIRCLE_MASK|OCT_POLYGON_MASK|OCT_PATH_MASK|OCT_LABEL_MASK

#define M_SCALE(s) ((s) / 20)

typedef struct term_info {
  struct term_info *next;
  char *name;
  octObject box;
} term_info;

typedef struct master_info {
  unsigned int timestamp;
  struct octBox bbox;
  int instance_num;
  term_info *terminal_info;
} master_info;

char *MagicCellName();
char *MagicInstanceName();
char *MagicTransform();

static char scratch[200];

convert_facet(facet, cell_num)
    octObject *facet;
    int cell_num;
{
    octObject object, inst, master, layer;
    octObject term, prop, box, boundary, bound_layer;
    octGenerator inst_gen, gen, layer_gen;
    octGenerator term_gen, box_gen;
    int status, in_table;
    extern void do_mask_mod();
    int timestamp;
    char *technology;
    char magic_file_name[200];
    master_info *m_info, *tm_info;
    char *master_name;
    char *inst_name;
    char scratch[400];

    /*    
      Before modifying the facet, save its bounding box as a box object
      under the facet.  This will then be scaled with the rest of the
      design.
      */
    
    bound_layer.type = OCT_LAYER;
    bound_layer.contents.layer.name = "\nTheBoundLayer\n";
    if ((status = octCreate(facet, &bound_layer)) < OCT_OK) {
	error("creating bounding box layer in the facet");
    }
    boundary.type = OCT_BOX;
    boundary.objectId = oct_null_id;
    octBB(facet, &boundary.contents.box);
    if ((status = octCreate(&bound_layer, &boundary)) < OCT_OK) {
	error("creating bounding box in the facet");
    }

    if ((status = flatten_contact_terminals(facet)) < OCT_OK) {
	(void) sprintf(scratch, "flatten_contact_terminals(): %s",
		       octErrorString());
	error(scratch);
    }

    if ((status = flatten_contacts(facet)) < OCT_OK) {
	(void) sprintf(scratch, "flatten_contacts(): %s", octErrorString());
	error(scratch);
    }

    if (hierarchy_level <= max_hierarchy) {
	convert_subcells(facet);
    }
    
    /*
      Convert Oct layers into corresponding MAGIC layers
      */

    do_mask_mod(facet);

    /*
      Get the MAGIC technology and timestamp from the Oct facet
      */

    technology = "scmos";
    prop.type = OCT_PROP;
    prop.contents.prop.name = "TECHNOLOGY";
    if (octGetByName(facet, &prop) == OCT_NOT_FOUND) {
	warning("Warning:  no technology");
    }
    else {
	if (prop.contents.prop.type == OCT_STRING) {
	    technology = prop.contents.prop.value.string;
	}
	else {
	    warning("Warning:  the technology property must be a string");
	}
    }
    
    timestamp = 2;
    prop.type = OCT_PROP;
    prop.contents.prop.name = "TIMESTAMP";
    if (!force_bad_timestamp) {
	if (octGetByName(facet, &prop) == OCT_NOT_FOUND) {
	    sprintf( scratch ,"Warning:  no TIMESTAMP property in %s", ohFormatName(facet));
	    warning(scratch);
	} else {
	    if (prop.contents.prop.type == OCT_INTEGER) {
		timestamp = prop.contents.prop.value.integer;
	    } else {
		sprintf("Warning: the TIMESTAMP property in %s must be an integer",
			ohFormatName(facet));
		warning(scratch);
	    }
	}
    }

    /*
      Save information about the master for later use
      */

    m_info = (master_info *)malloc(sizeof(struct master_info));
    m_info->timestamp = timestamp;
    m_info->instance_num = 0;
    m_info->terminal_info = NIL(term_info);
    if (octGetById(&boundary) < OCT_OK) {
	error("octGenFirstContent");
    }
    m_info->bbox = boundary.contents.box;
    octDelete(&boundary);
    octDelete(&bound_layer);

    add_cell(facet, m_info);

    if (hierarchy_level > max_hierarchy) return;
    
    /*
      Open corresponding MAGIC file
      */

    if (layout_directory) {
	(void) sprintf(magic_file_name, "%s/%s.mag", layout_directory,
		       MagicCellName(facet->contents.facet.cell));
    }
    else {
	(void) sprintf(magic_file_name, "%s.mag",
		       MagicCellName(facet->contents.facet.cell));
    }

    if ((output = VOVfopen(magic_file_name, "w")) == NULL) {
	FPRINTF(stderr, "Cannot open magic output file %s.mag\n",
		magic_file_name);
	return;
    }

    FPRINTF(output, "magic\ntech %s\ntimestamp %d\n", technology, timestamp);

    /*
      Generate instances of MAGIC subcells
      */

    OH_ASSERT(octInitGenContents(facet, OCT_INSTANCE_MASK, &inst_gen));

    while ((status = octGenerate(&inst_gen, &inst)) == OCT_OK) {
	inst_to_master(&inst, &master);
	OH_ASSERT(octOpenMaster(&inst, &master));
	if (lookup_cell(&master, &tm_info) == 0) {
	    (void) sprintf(scratch, "lookup_cell(%s) failed in facet %s.\n",
			   inst.contents.instance.name,
			   facet->contents.facet.cell);
	    error(scratch);
	}

	if (force_bad_timestamp) {
	    timestamp = 3;
	} else {
	    timestamp = tm_info->timestamp;
	}
	
	/*
	  Determine the MAGIC name for the instance and its master
	  */
	
	master_name = MagicCellName(inst.contents.instance.master);
	inst_name = inst.contents.instance.name;
	if (!inst_name || inst_name[0] == '\000') {
	    (void) sprintf(scratch, "%s_%d", master_name,
			   tm_info->instance_num++);
	    inst_name = scratch;
	} else if (!strcmp(inst_name, master_name)) {
	    (void) sprintf(scratch, "%s_%d", master_name,
			   tm_info->instance_num++);
	    inst_name = scratch;
	} else if (strip_octflat_instances) {
	    inst_name = MagicInstanceName(inst_name, scratch);
	}
	
	/*
	  Put instance call into the MAGIC file
	  */
	
	FPRINTF(output, "use %s %s\n", master_name, inst_name);
	FPRINTF(output, "timestamp %d\n", timestamp);
	FPRINTF(output, "transform %s\n",
		MagicTransform(&inst.contents.instance.transform));
	FPRINTF(output, "box %d %d %d %d\n",
		M_SCALE(tm_info->bbox.lowerLeft.x),
		M_SCALE(tm_info->bbox.lowerLeft.y),
		M_SCALE(tm_info->bbox.upperRight.x),
		M_SCALE(tm_info->bbox.upperRight.y));
    }
        
    if (status != OCT_GEN_DONE) {
	error("instance generator");
    }

    /*
      Generate geometries of MAGIC subcells
      */

    OH_ASSERT(octInitGenContents(facet,OCT_LAYER_MASK, &layer_gen));

    while ((status = octGenerate(&layer_gen, &layer)) == OCT_OK) {
	
	if (octInitGenContents(&layer, CIF_MASK, &gen) != OCT_OK) {
	    error("generating geometries");
	}

	while ((status = octGenerate(&gen,&object)) == OCT_OK) {
	    process_object(&object, layer.contents.layer.name);
	}
	
	if (status != OCT_GEN_DONE) {
	    error("geo generator");
	}
    }
    
    if (status != OCT_GEN_DONE) {
	error("layer generator");
    }

    /*
      Generate all terminals on object, and create MAGIC rlabels
      */

    OH_ASSERT(octInitGenContents(facet, OCT_TERM_MASK, &term_gen));

    while ((status = octGenerate(&term_gen, &term)) == OCT_OK) {
	if (MagicLabelTerm(&term, m_info) < OCT_OK) {
	    octError("MagicLabelTerm() failed");
	}
    }

    if (status != OCT_GEN_DONE) {
	error("terminal generator");
    }

    /*
      Close up MAGIC file, and reset current layer
      */

    MagicLayer("end");
    MagicLayer(NULL);
    fclose(output);
    output = NIL(FILE);
}

convert_subcells(facet)
    octObject	*facet;
{
    octGenerator	inst_gen;
    octObject		inst;
    octObject		master;
    int			status, in_table;
    master_info		*m_info;

    OH_ASSERT(octInitGenContents(facet, OCT_INSTANCE_MASK, &inst_gen));

    while ((status = octGenerate(&inst_gen,&inst)) == OCT_OK) {
	inst_to_master(&inst, &master);
	status = octOpenMaster(&inst, &master);
	if (status < OCT_OK || status == OCT_NEW_FACET) {
	    char err_buf[300];
	    extern char *optProgName;
	    (void) sprintf(err_buf, "%s: Can't open %s:%s:%s:%s ", optProgName,
		    master.contents.facet.cell, master.contents.facet.facet,
		    master.contents.facet.view, master.contents.facet.version);
	    error(err_buf);
	}

	in_table = lookup_cell(&master, &m_info);
	
	if (status == OCT_ALREADY_OPEN && !in_table ||
	    status != OCT_ALREADY_OPEN && in_table) {
	  error("Panic: mismatch between st_lookup and ALREADY_OPEN");
	}
	
	if (!in_table) {
	  ++hierarchy_level;
	  convert_facet(&master, next_cell++);
	  --hierarchy_level;
	}
      }
    
    if (status != OCT_GEN_DONE) {
      error("instance generator");
    }
    
    octFreeGenerator(&inst_gen);
  }

char *MagicCellName(full_name)
     char *full_name;
{
  char *cell, *rindex();
  
  if (cell = rindex(full_name, '/'))
    ++cell;
  else
    cell = full_name;
  
  return (cell);
}

char *MagicInstanceName(full_name, new_name)
/*
 * If the instance name full_name begins with a name surrounded by
 * slashes '/', then search for the last occurrence of that name in
 * the string full_name, and return the trailing part containing the
 * name, including the preceding slash.
 */
	char		*full_name;
	char		*new_name;	/* The new name will go here. */
{
	char		*sub_name;
	char		*new_chars;
	register char	*slash;
	register char	*p;
	register char	*s;

	if (*full_name != '/') return (strcpy(new_name, full_name));

	sub_name = full_name + 1;
	new_chars = full_name;
	*new_name = '\000';
	for (slash = sub_name; *slash; ++slash) {
		s = slash;
		if (*s != '/') continue;

		*s = '\000';
		strcat(new_name, new_chars);
		*s = '/';

		/*
		 * Compare the strings following this slash
		 * and the preceding slash; stop when they
		 * are equal, or either equals a slash.
		 */
		for (++s, p = sub_name; *s == *p; ++p, ++s)
			if (*p == '/') {
				break;
			}
		if (*s != '_') strcat(new_name, "/");

		new_chars = s;
		sub_name = slash + 1;
	}

	strcat(new_name, new_chars);

	return (new_name);
}

char *MagicTransform(tr)
	struct octTransform	*tr;
{
	char	scratch[80];
	char	*format;


	if (tr->transformType != OCT_FULL_TRANSFORM) {
		/* A manhattan transformation was given. */
		/* Find the corresponding output format. */
		switch (tr->transformType) {
		case OCT_NO_TRANSFORM:
			format = "1 0 %d 0 1 %d";
			break;
		case OCT_MIRROR_X:
			format = "-1 0 %d 0 1 %d";
			break;
		case OCT_MIRROR_Y:
			format = "1 0 %d 0 -1 %d";
			break;
		case OCT_ROT90:
			format = "0 -1 %d 1 0 %d";
			break;
		case OCT_ROT180:
			format = "-1 0 %d 0 -1 %d";
			break;
		case OCT_ROT270:
			format = "0 1 %d -1 0 %d";
			break;
		case OCT_MX_ROT90:
			format = "0 -1 %d -1 0 %d";
			break;
		case OCT_MY_ROT90:
			format = "0 1 %d 1 0 %d";
			break;
		default:
			return ("BOGUS_TRANSFORM");
		}
		sprintf(scratch, format,
				M_SCALE(tr->translation.x),
				M_SCALE(tr->translation.y));
		return(scratch);
	}

	/* Handle the general transformation case. */
	sprintf(scratch, "%d %d %d %d %d %d",
			(int) tr->generalTransform[0][0],
			(int) tr->generalTransform[0][1],
			M_SCALE(tr->translation.x),
			(int) tr->generalTransform[1][0],
			(int) tr->generalTransform[1][1],
			M_SCALE(tr->translation.y));

	return (scratch);
}

#define	NO_TERM_IMPL	0
#define	BOX_TERM_IMPL	1
#define	INST_TERM_IMPL	2

MagicLabelTerm(term, m_info)
	octObject	*term;
	master_info	*m_info;
{
	int		status;
	int		valid_impl;
	master_info     *tm_info;
	term_info	*t_info, *t_new;
	octGenerator	implGen;
	octObject	term_impl;
	octObject	box, layer;
	octObject	inst, master;
	octObject	iterm;
	
	master.objectId = 0;
	
	OH_ASSERT(octInitGenContents(term, OCT_TERM_MASK | OCT_BOX_MASK,
			&implGen));

	valid_impl = NO_TERM_IMPL;
	
	while (octGenerate(&implGen, &term_impl) != OCT_GEN_DONE) {
	  
	  switch (term_impl.type) {
	  case OCT_BOX:
	    valid_impl = BOX_TERM_IMPL;
	    box = term_impl;
	    break;
	  case OCT_TERM:
	    inst.objectId = term_impl.contents.term.instanceId;
	    if ((status = octGetById(&inst)) < OCT_OK) return status;
	    inst_to_master(&inst, &master);
	    OH_ASSERT(octOpenMaster(&inst, &master));
	    if (lookup_cell(&master, &tm_info) == 0)
	      error(stderr, "Panic:  lost the cell for an instance");

	    for (t_info = tm_info->terminal_info; t_info;
		 t_info = t_info->next) {
	      if (!strcmp(t_info->name,
			  term_impl.contents.term.name))
		break;
	    }
	    if (!t_info) {
		error(stderr, "Terminal's name not found!!\n");
		continue;
	      }
	    
	    box = t_info->box;
	    octTransformGeo(&box,
			    &inst.contents.instance.transform);
	    
	    valid_impl = INST_TERM_IMPL;
	    break;
	  default:
	    continue;
	  }
	  
	  if (valid_impl > NO_TERM_IMPL) break;
	}

	if (valid_impl == NO_TERM_IMPL) return (status);

	/*
	 * If we've come this far, then there is a box
	 * implementing the terminal.
	 */
	status = octGenFirstContainer(&box, OCT_LAYER_MASK, &layer);
	if (status < OCT_OK) {
		/* There was no layer containing the box. */
		return (status);
	}

	/* Save the terminal implementation box. */


	t_info = (term_info *) malloc(sizeof(struct term_info));
	t_info->name = term->contents.term.name;
	t_info->box = box;
	t_info->next = m_info->terminal_info;
	m_info->terminal_info = t_info;
	
	if (valid_impl == INST_TERM_IMPL) {
		/* Create the layer that the terminal is on. */
		process_object(&box, layer.contents.layer.name);
	}

	MagicLayer("labels");
	FPRINTF(output, "rlabel %s %d %d %d %d 0 %s\n",
			layer.contents.layer.name,
			M_SCALE(box.contents.box.lowerLeft.x),
			M_SCALE(box.contents.box.lowerLeft.y),
			M_SCALE(box.contents.box.upperRight.x),
			M_SCALE(box.contents.box.upperRight.y),
			term->contents.term.name);
	CheckSpecialTermProps(term, &box, &layer);

	return (OCT_OK);
}

int
CheckSpecialTermProps(term, term_box, layer)
	octObject	*term;	   /* Terminal which may have props. */
	octObject	*term_box; /* Implementations of terminal props. */
	octObject	*layer;	   /* The layer that the box is on. */
{
	static octId	previous_id = oct_null_id;
	static int	has_bdsim_in_prop;
	octObject	prop;
	struct octBox	*box;
	int		cx, cy;

	if (!term) {
		/* Reset the property check. */
		previous_id = oct_null_id;
		return (0);
	}

	if (term->objectId != previous_id) {
		has_bdsim_in_prop = 0;
		prop.type = OCT_PROP;
		prop.contents.prop.name = "BDSIM_IN";
		if (octGetByName(term, &prop) == OCT_OK) {
			has_bdsim_in_prop = 1;
		}
	}

	if (!has_bdsim_in_prop) return (0);

	box = &term_box->contents.box;
	cx = (box->lowerLeft.x + box->upperRight.x) / 2;
	cy = (box->lowerLeft.y + box->upperRight.y) / 2;

	FPRINTF(output, "rlabel %s %d %d %d %d 0 %s\n",
			layer->contents.layer.name,
			M_SCALE(cx), M_SCALE(cy), M_SCALE(cx), M_SCALE(cy),
			"Cr:In");

	return (1);
}
