#include "copyright.h"
#include "port.h"
#include "oct.h"
#include "oh.h"
#include "tr.h"

octStatus flatten_contact_terminals(facet)
octObject *facet;
{
    octStatus s;
    octObject bag, inst, fterm, aterm;
    octGenerator ftermGen, atermGen;
    tr_stack *stack = tr_create();

    OH_ASSERT(octInitGenContents(facet, OCT_TERM_MASK, &ftermGen));

    while ((s = octGenerate(&ftermGen, &fterm)) == OCT_OK) {
	OH_ASSERT(octInitGenContents(&fterm, OCT_TERM_MASK, &atermGen));
	while ((s = octGenerate(&atermGen, &aterm)) == OCT_OK) {
	    if (aterm.contents.term.instanceId == oct_null_id) {
		continue;
	    }
	    inst.objectId = aterm.contents.term.instanceId;
	    if (octGetById(&inst) < OCT_OK) continue;
	    bag.type = OCT_BAG;
	    bag.contents.bag.name = "CONNECTORS";
	    if (octGetContainerByName(&inst, &bag) != OCT_OK) continue;
	    tr_push(stack);
	    tr_add_transform(stack, &inst.contents.instance.transform, 0);
	    s = flatten_contact_terminal(facet, &inst, stack, &fterm);
	    if (s != OCT_OK) return s;
	    tr_pop(stack);
	    if ((s = octDelete(&inst)) != OCT_OK) return s;
	}
    }

    tr_free(stack);

    return OCT_OK;
}


octStatus flatten_contact_terminal(facet, inst, stack, term)
octObject *facet;
octObject *inst;
tr_stack *stack;
octObject *term;
{
    octObject master, layer, new_layer, geo;
    octGenerator gen, gen1;
    octStatus s;
    struct octPoint *point, *pts;
    int32 i, num;

    s = ohOpenMaster(&master, inst, "contents", "r");
    if (s < OCT_OK) return s;

    s = octInitGenContents(&master, OCT_LAYER_MASK, &gen);
    while ((s = octGenerate(&gen, &layer)) == OCT_OK) {
	new_layer = layer;
	s = octGetOrCreate(facet, &new_layer);
	if (s != OCT_OK) return s;

	s = octInitGenContents(&layer, OCT_GEO_MASK, &gen1);
	while ((s = octGenerate(&gen1, &geo)) == OCT_OK) {
	    switch(geo.type) {
		case OCT_BOX:
		    point = &geo.contents.box.lowerLeft;
		    tr_transform(stack, &(point->x), &(point->y));
		    point = &geo.contents.box.upperRight;
		    tr_transform(stack, &(point->x), &(point->y));
		    if ((s = octCreate(&new_layer, &geo)) != OCT_OK) return s;
		    break;
		
		case OCT_CIRCLE:	/* better than nothing ... */
		    point = &geo.contents.circle.center;
		    tr_transform(stack, &(point->x), &(point->y));
		    if ((s = octCreate(&new_layer, &geo)) != OCT_OK) return s;
		    break;

		case OCT_PATH:
		    if ((s = octCreate(&new_layer, &geo)) != OCT_OK) return s;
		    ohGetPoints(&geo, &num, &pts);
		    for(i = 0; i < num; i++) {
			tr_transform(stack, &(pts[i].x), &(pts[i].y));
		    }
		    if ((s = octPutPoints(&geo, num, pts)) != OCT_OK) return s;
		    break;

		case OCT_POLYGON:
		    if ((s = octCreate(&new_layer, &geo)) != OCT_OK) return s;
		    ohGetPoints(&geo, &num, &pts);
		    for(i = 0; i < num; i++) {
			tr_transform(stack, &(pts[i].x), &(pts[i].y));
		    }
		    if ((s = octPutPoints(&geo, num, pts)) != OCT_OK) return s;
		    break;

		case OCT_LABEL:
		    point = &geo.contents.label.region.lowerLeft;
		    tr_transform(stack, &(point->x), &(point->y));
		    point = &geo.contents.label.region.upperRight;
		    tr_transform(stack, &(point->x), &(point->y));
		    if ((s = octCreate(&new_layer, &geo)) != OCT_OK) return s;
		    break;
	    }

	    if (term) {
		/* Attach the geometry to a terminal. */
		if ((s = octAttach(term, &geo)) != OCT_OK) return s;
	    }
	}

	if (s != OCT_GEN_DONE) return s;
    }
    if (s != OCT_GEN_DONE) return s;
    return OCT_OK;
}
