#include "copyright.h"
#include "port.h"
#include "oct.h"


#define M_SCALE(s) ((s) / 20)

#define FPRINTF	(void) fprintf

extern FILE *output;

process_object(obj, layer_name)
octObject *obj;
char *layer_name;
{
    int num_points;
    struct octPoint *arr;
    
    switch (obj->type) {
    case OCT_BOX:
	{
	    struct octBox *box = &obj->contents.box;
	    MagicLayer(layer_name);
	    FPRINTF(output, "rect %d %d %d %d\n",
			M_SCALE(box->lowerLeft.x), M_SCALE(box->lowerLeft.y),
			M_SCALE(box->upperRight.x),M_SCALE(box->upperRight.y));
	    break;
	}
    case OCT_PATH:
	{
	    struct octPath *path = &obj->contents.path;
	    int i;
	    int xl, yl, xh, yh;

	    if (get_points(obj, &num_points, &arr) != OCT_OK) {
		error("Getting points on a path");
	    }
	    --num_points;
	    MagicLayer(layer_name);
	    for (i = 0; i < num_points; i++) {
		if (arr[i].x == arr[i+1].x) {
			xl = arr[i].x - (path->width / 2);
			xh = xl + path->width;
			yl = arr[i].y;
			yh = arr[i+1].y;
			if (yh < yl) {
				yl = yh;
				yh = arr[i].y;
			}
		} else if (arr[i].y == arr[i+1].y) {
			yl = arr[i].y - (path->width / 2);
			yh = yl + path->width;
			xl = arr[i].x;
			xh = arr[i+1].x;
			if (xh < xl) {
				xl = xh;
				xh = arr[i].x;
			}
		} else {
		    warning("Warning: Non-manhattan path");
		    continue;
	        }

		FPRINTF(output, "rect %d %d %d %d\n",
				M_SCALE(xl), M_SCALE(yl),
				M_SCALE(xh), M_SCALE(yh));
	    }
	    break;
	}
    case OCT_POLYGON:
	{
	    warning("Warning: OCT polygons are not compatible with magic");
	    break;
	}
    case OCT_CIRCLE:
	{
	    warning("Warning: OCT circles are incompatible with magic");
	    break;
	}
    case OCT_LABEL:
	{
	    struct octLabel *label = &obj->contents.label;

	    MagicLayer("labels");
	    FPRINTF(output, "rlabel %s %d %d %d %d 1 %s;\n", layer_name,
			M_SCALE(label->region.lowerLeft.x),
			M_SCALE(label->region.lowerLeft.y),
			M_SCALE(label->region.upperRight.x),
			M_SCALE(label->region.upperRight.y),
			label->label);
	    break;
	}
    }
}

MagicLayer(layer_name)
	char	*layer_name;
{
	static char	*prev_name = NULL;

	if (!layer_name) {
		prev_name = layer_name;
		return;
	}
	if (prev_name && !strcmp(prev_name, layer_name)) return;
	FPRINTF(output, "<< %s >>\n", layer_name);
	prev_name = layer_name;
}
