/* out_verilog.c - output functions for oct2verilog */

#include "copyright.h"
#include "port.h"
#include "oct.h"
#include "st.h"
#include "oh.h"
#include "head.h"
    
extern char *cct_name;
extern char *outputFile;
extern struct IO_term *IO_in_head,*IO_in_cur;
extern struct IO_term *IO_out_head,*IO_out_cur;
extern struct IO_term *IO_inout_head,*IO_inout_cur;
extern struct INT_terms INT_terms;

char *buffer;
char *entity_name;

char *
bus_transpose(s)
char *s;
{
	char new[128];
	char *p;

	p = strchr(s, '[');
	if (p == NULL)
		return s;
	strcpy(new, p); 			/* Put bus size ("[x:y]") first */
	strcat(new, " ");
	strncat(new, s, p-s);
	return new;
}

char *
bus_truncate(s)
char *s;
{
	char new[128];
	char *p;

	p = strchr(s, '[');
	if (p == NULL)
		return s;
	strncpy(new, s, p-s);
	new[p-s] = '\0';
	return new;
}

hdl_wire_term(list)
struct IO_term *list;
{
	while (list != NULL) {
		if (list->netname && list->termname && 
			strcmp(list->netname, list->termname))
			fprintf(fp, "  assign %s = %s;\n", list->termname, list->netname);
		list = list->next;
	}
}

write_hdl() {
    
    if (outputFile==NULL) {
		outputFile=(char *)malloc(strlen(cct_name)+5);
		strcpy(outputFile,cct_name);
		strcat(outputFile,".hdl");
    }
    if((fp=fopen(outputFile,"w"))==NULL) {
		fprintf(stderr,"oct2verilog: unable to open %s for output\n",outputFile);
		exit(-1);
    }
    
    fprintf(stderr,"Your Verilog HDL Circuit is being stored on %s \n",outputFile);
    
    /* hdl_components(); */

	hdl_main_module();			/* print module header */
    hdl_gates();				/* print the netlist */

	hdl_wire_term(IO_in_head);
	hdl_wire_term(IO_out_head);
	hdl_wire_term(IO_inout_head);
    
	hdl_end();					/* end the module declaration */
}

hdl_print_terms(list, func, s, t)
struct IO_term *list;
char * (*func)();
char *s;
{
    struct sortlist *tmp, *slist;

    /* create a sortlist of the terminals */
    slist = (struct sortlist*) IO_2_sortlist(list);

    /* sort the list */
    sort_list(&slist);

    /* vectorize the list */
    slist = (struct sortlist*) vectorize(slist);
    
    /* write output signal names */
    for (tmp=slist;tmp!=NULL;tmp=tmp->next) {
		strcpy(buffer, s);
		strcat(buffer,(*func)(tmp->str));
		if (tmp->next != NULL)
			strcat(buffer, t);
		fputs(buffer, fp);
    }

	/* de-allocate the sorted list */
    free_sortlist(slist);
}

/* write the VERILOG style header for the circuit */
hdl_header() {
}

hdl_main_module()
{
    char *ptr1,*ptr2;
    int i,j,k;
    int linlen,strlength; 
    struct sortlist *tmp, *slist;
    
    entity_name = cct_name;

    /* check for leading digit in circuit name */
    if(isdigit(*cct_name)) {
		entity_name = (char *) malloc(strlen(cct_name)+2);
		strcpy(entity_name,"E");
		strcat(entity_name,cct_name);
		fprintf(stderr,"Your VERILOG Entity has been renamed %s \n\n",entity_name);
    }
    
    /* write header & circuti name */
    strcpy(buffer,"module ");
    strcat(buffer,entity_name);
    strcat(buffer," ( ");
	fputs(buffer, fp);
	*buffer = '\0';
    
	hdl_print_terms(IO_in_head, bus_truncate, "    ", ", \n ");
	if (IO_out_head != NULL)
		fputs(",\n", fp);
	hdl_print_terms(IO_out_head, bus_truncate, "    ", ",  \n");
	if (IO_inout_head != NULL)
		fputs(",\n", fp);
	hdl_print_terms(IO_inout_head, bus_truncate, "    ", ",  \n");
	fputs(");\n", fp);
		
	*buffer = '\0';

	hdl_print_terms(IO_in_head, bus_transpose, "input ", ";\n");
	if (IO_in_head != NULL)
		fputs(";\n", fp);
	hdl_print_terms(IO_out_head, bus_transpose, "output ", ";\n");
	if (IO_out_head != NULL)
		fputs(";\n", fp);
	hdl_print_terms(IO_inout_head, bus_transpose, "inout ", ";\n");
	if (IO_inout_head != NULL)
		fputs(";\n", fp);
    
	/* put a blank line out */
    strcpy(buffer,"\n");
    fputs(buffer,fp);
}

hdl_components() {
    int i,k;
    int dir;
    
    for (i=0;i<inst_cnt;i++) {

		/* Print out each component definition */
		strcpy(buffer,"module ");
		strcat(buffer,primitives[my_inst[i].prim_num].primitive_name);
		strcat(buffer," (");
		dir = my_inst[i].term_dir[0];

		/* Now get all the terminals for the component */
		for(k=0;k<my_inst[i].term_cnt;k++) {
	   		strcat(buffer,my_inst[i].term_name[k]);
	   		strcat(buffer,",");
	   		dir = my_inst[i].term_dir[k];
		}

		buffer[strlen(buffer)-1] = '\0'; /* gets rid of trailing ',' */
		fputs(buffer,fp);
		strcpy(buffer,");\nendmodule\n");
		fputs(buffer,fp);
    }
}

char *
fix_prim(s)
char *s;
{
	char new[128];
	char *p;

	for (p = new; *s; s++,p++)
		*p = toupper(*s);
	*p = '\0';
	return new;
}

hdl_gates() {
    
    int i,k;
    int prim_num;
    
    for (i=0;i<inst_cnt;i++) {
		prim_num = get_prim_num(my_inst[i].inst_numb);
		for(k=my_inst[i].gate;(k != -1);k=my_gate[k].next) {
	    	strcpy(buffer,"  "); 
	    	strcat(buffer,fix_prim(primitives[prim_num].primitive_name));
			strcat(buffer, " ");
	    	strcat(buffer,my_gate[k].g_name);
	    	strcat(buffer," (");
	    	hdl_term(k, i);
	    	buffer[strlen(buffer)-1] = '\0';
	    	strcat(buffer,");\n");
	    	fputs(buffer,fp);
		}
    }
}

hdl_term(g_num, i_num)
int  g_num;
int  i_num;
{
    int k,a=0;
    char c;
	char add_buff[128];
	char *find;
	char port_name[128];
    
    for (k=0;my_gate[g_num].term[k]!=NULL;k++) {
		strcpy(port_name, my_gate[g_num].term[k]);
		sprintf(add_buff, ".%s(%s),", my_inst[i_num].term_name[k], port_name);
		strcat(buffer,add_buff);
    }
}

hdl_end() {
    fprintf(fp,"endmodule\n");
}

