#include "oct.h"
#include "oh.h"
#include "options.h"
#include "errtrap.h"
#include "utility.h"

optionStruct optionList[] = {
    {"C", "old-new",     "Old and new layer name" },
    {OPT_RARG, "cell:view" , "" },
    { 0, 0 , 0}
};

main( argc, argv )
    int argc;
    char* argv[];
{
    octObject facet;
    char* pairs[100];		/* Enormous array. */
    int   countPairs = 0;	/* Count requested layer changes. */
    int c;			/* Parsed option. */
    int i;			/* Loop counter. */
    while ( ( c=optGetOption( argc, argv ))  != EOF ) {
	switch ( c ) {
	case 'C':
	    pairs[countPairs++] = optarg;
	    break;
	default:
	    optUsage();
	    break;
	}
    }
    if ( optind == argc ) {
	optUsage();
    }

    ohUnpackDefaults( &facet, "a", "::contents" );
    ohUnpackFacetName( &facet, argv[optind] );

    if ( octOpenFacet( &facet  ) != OCT_OLD_FACET ) {
	errRaise( "CHANGELAYER", 1, "Facet does not exist: %s", ohFormatName( &facet ) );
    }
    
    for ( i = 0 ; i < countPairs ; i++ ) {
	octObject layer;
	char* oldName = util_strsav( pairs[i] );
	char* newName = strchr( oldName, '-' );
	if ( !newName ) {
	    errRaise( "CHANGELAYER", 1,  "Illegal pair specification: %s\n", pairs[i] );
	}
	*newName++ = '\0';
	if ( ohGetByLayerName( &facet, &layer, oldName ) != OCT_OK  ) {
	    printf( "Non existing layer %s in %s\n", ohFormatName(&layer ), ohFormatName( &facet ) );
	    continue;
	}

	layer.contents.layer.name = newName;
	OH_ASSERT( octModify( &layer ) );
    }
    octCloseFacet( &facet );
    exit( 0 );
}
