#include "copyright.h"
#include "port.h"
#include "utility.h"
#include <setjmp.h>
#include "options.h"
#include "oct.h"
#include "oh.h"
#include "st.h"
#include "list.h"
#include "octdiff.h"

int markMatched(n1, id1List, n2, id2List)
int n1;			/* number of objects in facet1 group */
octId id1List[];	/* list of octIds of facet1 objects */
int n2;			/* number of objects in facet2 group */
octId id2List[];	/* list of octIds of facet2 objects */
{
    int i;
    int attFlags = 0;
    int markExactMatch();

    if (n1 != n2) attFlags |= DIFFERENT_P;

    /* squeeze out simple multiple attachment */
    while ((n1 >= 2) && (id1List[n1-1] == id1List[n1-2])) {
	n1--;
    }
    while ((n2 >= 2) && (id2List[n2-1] == id2List[n2-2])) {
	n2--;
    }

    if (n1 != n2) attFlags |= DIFFERENT_P;

    if (n1 == 1 && n2 == 1) {
	/* we have a definite correspondence */
	attFlags |= markExactMatch(id1List[0], id2List[0]);
    } else if (n1 == 0) {
	/* make sure unmatched objs are faulted into table */
	for (i = 0; i < n2; i++) {
	    (void) getMatchData(id2List[i]);
	}
    } else if (n2 == 0) {
	/* make sure unmatched objs are faulted into table */
	for (i = 0; i < n1; i++) {
	    (void) getMatchData(id1List[i]);
	}
    } else {
	attFlags |= SLOPPY_P;
    }

    return(attFlags);
}

static int markExactMatch(id1, id2)
octId id1;
octId id2;
{
    matchData *match1Ptr, *match2Ptr;

    match1Ptr = getMatchData(id1);
    match2Ptr = getMatchData(id2);

    if (!octIdIsNull(match1Ptr->matchId) || !octIdIsNull(match2Ptr->matchId)) {
	/* they must already be matched to each other */
	if (match1Ptr->matchId != id2 || match2Ptr->matchId != id1) {
	    PANIC("erroneous exact match");
	}
	return(0);
    } else {
	/* make new correspondence */
	match1Ptr->objFlags &= ~UNMATCHED_P;
	match2Ptr->objFlags &= ~UNMATCHED_P;
	if ( ! (curMatchTablePtr->flags & INEXACT)) {
	    match1Ptr->objFlags |= MATCHED_P;
	    match2Ptr->objFlags |= MATCHED_P;
	}
	match1Ptr->matchId = id2;
	match2Ptr->matchId = id1;

	lsNewEnd(curMatchTablePtr->matchedList, (lsGeneric) id1, LS_NH);
	return(0);
    }
}

void markUnmatchable(id)
octId id;
{
    matchData *dataPtr;

    if (curMatchTablePtr->flags & QUICK_CHECK) {
	longjmp(curMatchTablePtr->jmpBuf, 1);
    }
    curMatchTablePtr->errorP = 1;

    dataPtr = getMatchData(id);
    dataPtr->matchId = id;	/* give it a unique unmatched id -- its own */
    dataPtr->objFlags |= UNMATCHED_P;
}

void markObject(id, flags)
octId id;
int flags;
{
    getMatchData(id)->objFlags |= flags;
}

void markContents(id, flags)
octId id;
int flags;
{
    matchData *dataPtr;

    dataPtr = getMatchData(id);
    flags &= ~dataPtr->contentsFlags;

    if (flags & DIFFERENT_P) {
	if (curMatchTablePtr->flags & QUICK_CHECK) {
	    longjmp(curMatchTablePtr->jmpBuf, 1);
	}
	curMatchTablePtr->errorP = 1;
    }

    if (flags & SLOPPY_P) {
	lsNewEnd(curMatchTablePtr->sloppyList, (lsGeneric) id, LS_NH);
    }

    dataPtr->contentsFlags |= flags;
}

void markContainers(id, flags)
octId id;
int flags;
{
    matchData *dataPtr;

    dataPtr = getMatchData(id);
    flags &= ~dataPtr->containersFlags;

    if (flags & DIFFERENT_P) {
	if (curMatchTablePtr->flags & QUICK_CHECK) {
	    longjmp(curMatchTablePtr->jmpBuf, 1);
	}
	curMatchTablePtr->errorP = 1;
    }

    if (flags & SLOPPY_P) {
	lsNewEnd(curMatchTablePtr->sloppyList, (lsGeneric) id, LS_NH);
    }

    dataPtr->containersFlags |= flags;
}

matchData *getMatchData(id)
octId id;		/* id of reference object */
{
    matchData *doGetMatchData();

    return(doGetMatchData(curMatchTablePtr, id));
}

static matchData *doGetMatchData(tablePtr, id)
matchTable *tablePtr;	/* match table to be searched */
octId id;		/* id of reference object */
{
    matchData *dataPtr;
    octObject obj;
    octObject facet;
    char **slot;

    if (st_find_or_add(tablePtr->table, (char *) id, &slot)) {
	dataPtr = (matchData *) *slot;
    } else {
	dataPtr = ALLOC(matchData, 1);

	if (tablePtr->parent) {
	    *dataPtr = *doGetMatchData(tablePtr->parent, id);
	} else {
	    dataPtr->matchId = oct_null_id;
	    dataPtr->objFlags = 0;
	    dataPtr->contentsFlags = 0;
	    dataPtr->containersFlags = 0;

	    obj.objectId = id;
	    OCT_ASSERT(octGetById(&obj));
	    octGetFacet(&obj, &facet);
	    if (octIdsEqual(facet.objectId, facet1.objectId)) {
		dataPtr->objFlags |= FACET_1_P;
	    }
	}

	*slot = (char *) dataPtr;
    }
    return(dataPtr);
}

/*
 *	nextMatchedObj
 *
 *  Returns the objectId of the next object in facet1 that has a known
 *  corresponding object in facet2.  Returns oct_null_id if no more left.
 */
octId nextMatchedObj()
{
    lsGeneric dummy;

    if (lsDelBegin(curMatchTablePtr->matchedList, &dummy) == LS_NOMORE) {
	return(oct_null_id);
    } else {
	return((octId) dummy);
    }
}

int recycleSloppyObjs()
{
    int hadSome = 0;
    lsGeneric dummy;
    octId id;
    matchData *dataPtr;

    while (lsDelBegin(curMatchTablePtr->sloppyList, &dummy) != LS_NOMORE) {
	id = (octId) dummy;
	dataPtr = getMatchData(id);
	dataPtr->contentsFlags &= ~SLOPPY_P;
	dataPtr->containersFlags &= ~SLOPPY_P;
	lsNewEnd(curMatchTablePtr->matchedList, (lsGeneric) id, LS_NH);
	hadSome = 1;
    }

    return(hadSome);
}

/*
 *	recycleErrorObjs()
 *
 *   For each object with DIFFERENT_P marked on its contents or containers,
 *   erase the DIFFERENT_P mark and put the object in the matchList.  Also
 *   unmark any objects that were marked UNMATCH_P to let them try again.
 */
 
int recycleErrorObjs()
{
    int hadSome = 0;
    st_generator *gen;
    char *key, *val;
    octId id;
    matchData *dataPtr;

    gen = st_init_gen(curMatchTablePtr->table);
    while (st_gen(gen, &key, &val)) {
	id = (octId) key;
	dataPtr = (matchData *) val;

	if ((dataPtr->contentsFlags | dataPtr->containersFlags) & DIFFERENT_P) {
	    dataPtr->contentsFlags &= ~DIFFERENT_P;
	    dataPtr->containersFlags &= ~DIFFERENT_P;
	    lsNewEnd(curMatchTablePtr->matchedList, (lsGeneric) id, LS_NH);
	    hadSome = 1;
	}

	if (dataPtr->objFlags & UNMATCHED_P) {
	    dataPtr->objFlags &= ~UNMATCHED_P;
	    dataPtr->matchId = oct_null_id;
	}
    }

    return(hadSome);
}

void pushTable(newFlags)
int newFlags;
{
    matchTable *newTablePtr;

    newTablePtr = ALLOC(matchTable, 1);
    newTablePtr->parent = curMatchTablePtr;
    newTablePtr->table = st_init_table(st_numcmp, st_numhash);
    newTablePtr->matchedList = lsCreate();
    newTablePtr->sloppyList = lsCreate();
    newTablePtr->flags = newFlags;
    if (curMatchTablePtr) {
	newTablePtr->flags |= curMatchTablePtr->flags;
    }
    newTablePtr->errorP = 0;

    curMatchTablePtr = newTablePtr;
}

void popTable(saveP)
int saveP;
{
    st_generator *gen;
    char *key, *val;
    octId id;
    matchData *dataPtr;
    matchTable *oldTablePtr;

    if (saveP && curMatchTablePtr->parent == NIL(matchTable)) return;

    oldTablePtr = curMatchTablePtr;
    curMatchTablePtr = curMatchTablePtr->parent;

    if (saveP) {
	gen = st_init_gen(oldTablePtr->table);
	while (st_gen(gen, &key, &val)) {
	    id = (octId) key;
	    dataPtr = (matchData *) val;
	    if ((dataPtr->objFlags & FACET_1_P) &&
					(dataPtr->objFlags & MATCHED_P)) {
		if (markExactMatch(id, dataPtr->matchId) != 0) {
		    PANIC("rematching ids on context pop");
		}
		markContents(id, dataPtr->contentsFlags);
		markContainers(id, dataPtr->containersFlags);
	    }
	    FREE(dataPtr);
	}
	st_free_gen(gen);
    }

    st_free_table(oldTablePtr->table);
    lsDestroy(oldTablePtr->matchedList, (void (*)()) 0);
    lsDestroy(oldTablePtr->sloppyList, (void (*)()) 0);
    FREE(oldTablePtr);
}
