/* Modified by Andrea Casotto Feb 8 1988 */
#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "oct.h"
#include "oh.h"
#include "nle.h"
#include "options.h"
#include "vov.h"

#define TRUE         1
#define FALSE        0

extern char         *FLATversion();

char    *progName;
char    errmsg[256];
octObject RefFacet;        /* reference  Facet   */
int    referenceFacet = FALSE;
int    flatstop = 0;

#define LEV_PHYSICAL  0
#define LEV_SYMBOLIC  1
#define LEV_LEAF      2

int     debug = 0;

optionStruct optionList[] = {
    { "d",    0,        "provide debugging information" },
    { "o",    "[cell:]view",    "output facet (defaults to in place)" },
    { "r",    "[cell:]view",    "reference facet" },
    { "S",    0,        "do a SOFT flattening (default is HARD)"},
    { "F",    0,        "check for FLAT_STOP property" },
    { "t",    "TYPE",        "flatten until..." },
    { OPT_CONT,    0, "\n TYPE = PHYSICAL\t - there are no instances" },
    { OPT_CONT,    0, "\n*TYPE = SYMBOLIC\t - instances have VIEWTYPE = PHYSICAL" },
    { OPT_CONT,    0, "\n TYPE = LEAF\t - instances have CELLCLASS = LEAF" },
    { OPT_CONT,    0, "\n (default is SYMBOLIC)" },
    { OPT_RARG,    "cell:view",    "input facet" },
    { 0,        0,        0 },
};


int
parse_facet_spec(arg, cell, view)
char *arg, **cell, **view;
{
    char *colon;

    if ((colon = strchr(arg, ':')) == NULL) {
        *cell = arg;
        *view = "";
    } 
    else {
        *colon = '\0';
        *cell = arg;
        *view = colon+1;
    }
}


void
StopFlatNotify(InstP, reason)
octObject *InstP;
char    *reason;
{
    if (debug) {
	(void) printf("Instance %s of %s:%s not flattened because\n\t%s\n",
			InstP->contents.instance.name,
			InstP->contents.instance.master,
			InstP->contents.instance.view,
			reason);
    }
}


void
FatalError(es)
char *es;            /* error string */
{
    (void) fprintf(stderr,"\n\n%s: Fatal Error: %s\n" , progName, es);
    exit(-1);
}


void
FatalOctError(es)
char *es;            /* error string */
{
    char buffer[1024];
    (void) sprintf(buffer, "\n\n%s:FATAL: %s\n", progName, es);
    octError(buffer);
    exit(-1);
}


void
Warning(es)
char *es;            /* error string */
{
    (void) fprintf(stderr, "%s:WARN:%s\n", progName, es);
}


void
Message(es)
char *es;            /* error string */
{
    (void) printf("%s:INFO:%s\n", progName, es); fflush( stdout );
}


main(argc, argv)
int argc;
char **argv;
{
    extern int optind;
    extern char *optarg;
    int option;

    void	flatLoop();
    octObject     sourceFacet;
    octObject     workFacet, interface;
    octObject     net;
    octObject    prop;
    octGenerator     netgen;
    octStatus     stat;
    int         isdasho = 0;
    char	 *outFacetName = NIL(char);
    int        level = LEV_SYMBOLIC;        /* flat level param */
    int        nle_type = NLE_HARD;        /* of flattening */

    progName = argv[0];
    if (strrchr(progName,'/') != (char *) 0) {
        progName = strrchr(progName, '/') + 1;
    }
    (void) printf("%s: %s\n", progName , FLATversion());

    while ((option = optGetOption(argc, argv)) != EOF) {

        switch (option) {
	case 'F':
	    flatstop = 1;
	    break;
	case 'S':
	    nle_type = NLE_SOFT;
	    break;
        case 'd':
            debug = 1;
            break;

        case 't':
            if (strcmp(optarg,"LEAF")==0) {
                level = LEV_LEAF;
            } 
            else if (strcmp(optarg,"PHYSICAL")==0) {
                level = LEV_PHYSICAL;
            } 
            else if (strcmp(optarg,"SYMBOLIC")==0) {
                level = LEV_SYMBOLIC;
            } 
            else {
                optUsage();
            }
            break;

        case 'r':
	    ohUnpackDefaults(&RefFacet, "r", "::contents");
	    if (ohUnpackFacetName(&RefFacet, optarg) != OCT_OK) {
	        optUsage();
	    }
            referenceFacet = 1;
            break;

        case 'o':
            isdasho = 1;
            outFacetName = util_strsav(optarg);
            break;
        }
    }

    if (referenceFacet && level != LEV_SYMBOLIC) {
        FatalError("If you specify the reference facet you must do a symbolic flattening");
    }

    if (argc != optind+1) {
        optUsage();
    }


    if (nle_type == NLE_SOFT && level == LEV_PHYSICAL) {
	FatalError("Incompatible flags -S and -t PHYSICAL");
    }
    if (level == LEV_PHYSICAL) {
	nle_type = NLE_PHYSICAL;
    }


    if (!isdasho) {
	Warning("This will overwrite the input facet");
    }

    
    VOVbegin( argc, argv );
    octBegin();

    ohWorkFacet( &workFacet, argv[optind], outFacetName );

    /* start real processing */
    if (referenceFacet) {
        if ((stat = octOpenFacet(&RefFacet)) < OCT_OK) {
            FatalOctError("reference facet");
        }
    }

    flatLoop(&workFacet,level,nle_type);        /*  core of the program*/

    if (level == LEV_PHYSICAL) {
        /* Whoops -- symbolic will become physical */
        /* remove all nets */
        OH_ASSERT(octInitGenContents(&workFacet, OCT_NET_MASK, &netgen));
        while (octGenerate(&netgen, &net) == OCT_OK) {
            OH_ASSERT(octDelete(&net));
        }
        /* change VIEWTYPE and EDITSTYLE */
        prop.type = OCT_PROP;
        prop.contents.prop.name = "VIEWTYPE";
        prop.contents.prop.type = OCT_STRING;
        prop.contents.prop.value.string = "PHYSICAL";
        OH_ASSERT(octCreateOrModify(&workFacet, &prop));
        prop.contents.prop.name = "EDITSTYLE";
        OH_ASSERT(octCreateOrModify(&workFacet, &prop));

        (void) sprintf(errmsg, "View changed to PHYSICAL: updating interface facet",
        workFacet.contents.facet.cell, workFacet.contents.facet.view);
        Message(errmsg);
	interface = workFacet;
	interface.contents.facet.facet = "interface";
	VOVinputFacet( &interface );
	VOVoutputFacet( &interface );
	OH_ASSERT(octOpenFacet(&interface));
	(void) vulcan(&workFacet, &interface);
	OH_ASSERT(octCloseFacet(&interface));
    }
    OH_ASSERT(octCloseFacet(&workFacet));
    octEnd();

    VOVend(0);
}


/* Should this instance be flattened ? */
int
Should_I_flatten(InstanceP,level)
octObject     *InstanceP;
int        level;
{
    octObject    Master;
    octObject     refInst;    /* instance in the reference facet */
    octObject     viewtype;
    octObject     stopProp;
    octObject    prop;

    if (level == LEV_PHYSICAL) return(TRUE);

    OH_ASSERT(ohOpenMaster(&Master, InstanceP, "interface", "r"));

    /* check here to see if subInst should be collapsed */
    if (flatstop && ohGetByPropName(InstanceP, &stopProp, "FLAT_STOP") == OCT_OK) {
        StopFlatNotify(InstanceP,"Instance says FLAT_STOP");
        return(FALSE);
    } 
    else  if (flatstop && ohGetByPropName(&Master, &stopProp, "FLAT_STOP" ) == OCT_OK) {
        StopFlatNotify(InstanceP,"Master says FLAT_STOP");
        return(FALSE);
    } 
    else {
        if (referenceFacet) {
            refInst.type = OCT_INSTANCE;
            refInst.contents.instance.name = InstanceP->contents.instance.name;
            if (octGetByName(&RefFacet,&refInst) == OCT_OK) {
                StopFlatNotify(InstanceP,"Instance exists also in the reference facet");
                return(FALSE);
            }
        }
    }

    /* Additional conditions to stop flattening */
    if (level == LEV_SYMBOLIC) {
        viewtype.type = OCT_PROP;
        viewtype.contents.prop.name = "VIEWTYPE";
        if (octGetByName(&Master, &viewtype) != OCT_OK) {
            (void) sprintf(errmsg, "VIEWTYPE not found in %s:%s:%s",
            Master.contents.facet.cell,
            Master.contents.facet.view,
            Master.contents.facet.facet);
            FatalError(errmsg);
        }
        if (strcmp(viewtype.contents.prop.value.string, "PHYSICAL") == 0) {
            StopFlatNotify(InstanceP,"Master is PHYSICAL");
            return(FALSE);
        }
    } 
    if (level == LEV_LEAF) {
        prop.type = OCT_PROP;
        prop.contents.prop.name = "CELLCLASS";
        if (octGetByName(&Master, &prop) != OCT_OK) {
            (void) sprintf(errmsg,"CELLCLASS not found in %s:%s:%s",
            Master.contents.facet.cell,
            Master.contents.facet.view,
            Master.contents.facet.facet);
            FatalError(errmsg);
        }
        if (strcmp(prop.contents.prop.value.string, "LEAF") == 0) {
            StopFlatNotify(InstanceP,"Master is LEAF");
            return(FALSE);
        } 
    }

    /* All criteria to stop flattening failed, then flatten */
    return(TRUE);
}

#define I Instance.contents.instance


void
flatLoop(facet,level,nle_type)
octObject     *facet;        /* contents of instance to expand */
int        level;
int        nle_type;        /* of flattening */
{
    octGenerator     genInst;
    octGenerator     gen;
    octObject     Instance;
    octObject     InstanceBag;
    octObject     ConnectorBag;
    octObject    BagToFlatten;
    octObject    FlattenedBag;
    octObject    AlreadyConsideredInstances;
    int         flatten;    /* flatten subInst (copy up contents) if true */
    int        moreToFlat = 1;    /* greater than 0, to enter the flatten loop */

    InstanceBag.type = OCT_BAG;
    InstanceBag.contents.bag.name = "INSTANCES";
    OH_ASSERT(octGetOrCreate(facet, &InstanceBag));

    ConnectorBag.type = OCT_BAG;
    ConnectorBag.contents.bag.name = "CONNECTORS";
    OH_ASSERT(octGetOrCreate(facet, &ConnectorBag));

    BagToFlatten.type = OCT_BAG;
    BagToFlatten.contents.bag.name = "BAG_OF_INSTANCES_TO_FLATTEN";
    OH_ASSERT(octGetOrCreate(facet, &BagToFlatten));
    FlattenedBag.type = OCT_BAG;
    FlattenedBag.contents.bag.name = "BAG_OF_INSTANCES_OBTAINED_BY_FLAT";
    OH_ASSERT(octGetOrCreate(facet, &FlattenedBag));

    AlreadyConsideredInstances.type = OCT_BAG;
    AlreadyConsideredInstances.contents.bag.name = "INSTANCES_ALREADY_CONSIDERED";
    OH_ASSERT(octGetOrCreate(facet, &AlreadyConsideredInstances));

    while (moreToFlat) {
        /* printf("%3d",++flatPasses); */
        moreToFlat = 0;        /* unless some new instance is generated by flattening */
	if (level != LEV_PHYSICAL) {
	    OH_ASSERT(octInitGenContents(&InstanceBag,OCT_INSTANCE_MASK,&genInst));
	    while (octGenerate(&genInst,&Instance) == OCT_OK) {
		if (octIsAttached(&AlreadyConsideredInstances,&Instance)==OCT_OK) continue;
		flatten = Should_I_flatten(&Instance,level);
		if (flatten == TRUE) {
		    moreToFlat++;
		    OH_ASSERT(octAttach(&BagToFlatten,&Instance));
		    if (debug) {
			(void) sprintf(errmsg, "Flattening %s of %s:%s",
					       I.name, I.master, I.view);
			Message(errmsg);
		    }
		} else {
		    OH_ASSERT(octAttach(&AlreadyConsideredInstances,&Instance));
		}
	    }
	} else {
            OH_ASSERT(octInitGenContents(facet,OCT_INSTANCE_MASK,&gen));
            while (octGenerate(&gen,&Instance) == OCT_OK) {
                moreToFlat++;
                OH_ASSERT(octAttach(&BagToFlatten,&Instance));
                if (debug) {
                    (void) sprintf(errmsg, "Flattening %s of %s:%s",
					   I.name, I.master, I.view);
                    Message(errmsg);
                }
            }
        }

        if (moreToFlat) {
            if (debug) {
                (void) sprintf(errmsg,"%d instances to flatten", moreToFlat);
                Message(errmsg);
            }
            if (nleFlat(&BagToFlatten,&FlattenedBag, nle_type)!=NLE_OK)  {
                (void) fprintf(stderr,"error flattening some instance");
            }
        }
        if (debug) {
            Message("Flushing working facet");
            OH_ASSERT(octFlushFacet(facet));
        }
    }
    OH_ASSERT(octDelete(&BagToFlatten));
    OH_ASSERT(octDelete(&FlattenedBag));
    OH_ASSERT(octDelete(&AlreadyConsideredInstances));

    return;
}

