#ifndef lint
static char sccsid[] = "@(#)man.c	1.11 (Berkeley) 2/3/84";
#endif

#include "port.h"
#include "utility.h"
#include <sgtty.h>
#include "stdio.h"
#include "pwd.h"
#include "errtrap.h"
#include <sys/stat.h>
#include <string.h>
/*
 * octman: A stripped down version of 'cadman'
 *	-- with the added feature of printing previously formatted *.doc files
 *	(skipping nroff) located in a separate doc directory.
 *
 * man -- link also to apropos and whatis
 *
 * This version uses more for underlining and paging.
 * Which more can be changed with the PAGER environment variable.
 *
 * Made configurable so it can work in different man directories,
 *   with different sets of manual macros, Walter Scott, Feb 1985.
 *
 * Copyright (c) 1985 Regents of the University of California
 */

/*
 * Configuration: change these if you want man to look in a different
 * place than /usr/man, or use different macros.  Any of the following
 * may begin with a tilde.
 */
#define	MANDIR	  "$OCTTOOLS/man"		/* this is deeply embedded */
#define	DOCDIR	  "$OCTTOOLS/doc"		/* this is deeply embedded */

/* The following are tilde-expanded forms of the above */
char	ManPath[BUFSIZ];
char	DocPath[BUFSIZ];

/*
 * When nroff is run, the command will be of the form
 *	NROFFCAT MANMACS
 * or	NROFF MANMACS
 */
#define	NROFFCAT "nroff"		/* for nroffing to cat file */
#define	NROFF	"nroff"			/* for nroffing to tty */

/* Paging filters */
#define	MORE	"more -s"		/* paging filter */
#define	CAT_	"/bin/cat"		/* for when output is not a tty */
#define	CAT_S	"/bin/cat -s"		/* for '-' opt (no more) */

#define	ALLSECT	"1nl6823457po"	/* order to look through sections */
#define	SECT1	"1nlo"		/* sections to look at if 1 is specified */
#define	SUBSEC1	"mcgprv"	/* subsections to try in section 1 */
#define	SUBSEC2	"v"
#define	SUBSEC3	"jxmsnvcf"
#define	SUBSEC4	"pfvsn"
#define	SUBSEC8	"vc"

int	nomore;
int	cflag;
char	*CAT	= CAT_;
char	*manpath = MANDIR;
char	*docpath = DocPath;
struct	passwd *getpwnam();
char	*trim();
void	removeTmpFile();
int	section;
int	subsec;
int	killtmp;

#define	eq(a,b)	(strcmp(a,b) == 0)

usage()
{
    fprintf(stderr, "Usage: octman [ section ] name ...\n");
    exit(1);
}

main(argc, argv)
	int argc;
	char *argv[];
{
    errProgramName( "octman" );

	if (signal(SIGINT, SIG_IGN) == SIG_DFL) {
		signal(SIGINT, removeTmpFile);
		signal(SIGQUIT, removeTmpFile);
		signal(SIGTERM, removeTmpFile);
	}
	umask(0);
	if (argc <= 1) {
	    usage();
	}
	argc--, argv++;
	while (argc > 0 && argv[0][0] == '-') {
		switch(argv[0][1]) {

		case 0:
		    nomore++;
		    CAT = CAT_S;
		    break;

		case 'P':
		    argc--, argv++;
		    manpath = *argv;
		    break;

		}
		argc--, argv++;
	}

	if ( argc <= 0 ) {
	    usage();
	}
	/* FILE-expand everything */

	(void) strcpy(ManPath, util_file_expand(manpath));
	if (chdir(ManPath) < 0) {
		fprintf(stderr, "Can't chdir to %s.\n", ManPath);
		exit(1);
	}

	/* printf("<<<%s>>>\n", ManPath); */

	(void) strcpy(DocPath, util_file_expand(DOCDIR));

	if (nomore == 0 && !isatty(1))
		nomore++;
	section = 0;
	do {
		if (eq(argv[0], "doc")) {
			section = 'd';
			goto sectin;
		} else if (argv[0][0] >= '0' && argv[0][0] <= '9' && (argv[0][1] == 0 || argv[0][2] == 0)) {
			section = argv[0][0];
			subsec = argv[0][1];
sectin:
			argc--, argv++;
			if (argc == 0) {
				fprintf(stderr, "But what do you want from section %s?\n", argv[-1]);
				exit(1);
			}
			continue;
		}
		manual(section, argv[0]);
		argc--, argv++;
	} while (argc > 0);
	exit(0);
}

manual(sec, name)
	char sec;
	char *name;
{
	char section = sec;
	char work[100], work2[100], cmdbuf[150];
	int ss;
	struct stat stbuf, stbuf2;
	int last;
	char *sp = ALLSECT;

	if (section == 'd') {
		strcpy(work, docpath);
		strcat(work, "/");
		strcat(work, name);
		strcat(work, ".doc");
		printf("[%s]\n", work);
		sprintf(cmdbuf, "%s %s", nomore? CAT : MORE, work);
		system(cmdbuf);
		return;
	}

	strcpy(work, "manx/");
	strcat(work, name);
	strcat(work, ".x");
	last = strlen(work) - 1;
	if (section == '1') {
		sp = SECT1;
		section = 0;
	}
	if (section == 0) {
		ss = 0;
		for (section = *sp++; section; section = *sp++) {
			work[3] = section;
			work[last] = section;
			work[last+1] = 0;
			work[last+2] = 0;
			if (stat(work, &stbuf) >= 0)
				break;
			if (work[last] >= '1' && work[last] <= '8') {
				char *cp;
search:
				switch (work[last]) {
				case '1': cp = SUBSEC1; break;
				case '2': cp = SUBSEC2; break;
				case '3': cp = SUBSEC3; break;
				case '4': cp = SUBSEC4; break;
				case '8': cp = SUBSEC8; break;
				}
				while (*cp) {
					work[last+1] = *cp++;
					if (stat(work, &stbuf) >= 0) {
						ss = work[last+1];
						goto found;
					}
				}
				if (ss = 0)
					work[last+1] = 0;
			}
		}
		if (section == 0) {
			if (sec != 0)
				printf("No entry for %s in section %c of the manual.\n", name, sec);
			else {
				strcpy(work, docpath);
				strcat(work, "/");
				strcat(work, name);
				strcat(work, ".doc");
				if (stat(work, &stbuf) >= 0) {
					printf("[%s]\n", work);
					sprintf(cmdbuf, "%s %s",
						nomore? CAT : MORE, work);
					system(cmdbuf);
				} else
				printf("No manual entry for %s.\n", name);
			}
			return;
		}
	} else {
		work[3] = section;
		work[last] = section;
		work[last+1] = subsec;
		work[last+2] = 0;
		if (stat(work, &stbuf) < 0) {
			if ((section >= '1' && section <= '8') && subsec == 0) {
				sp = "\0";
				goto search;
			}
			printf("No entry for %s in section %c", name, section);
			if (subsec)
				putchar(subsec);
			printf(" of the manual.\n");
			return;
		}
	}
found:
	{
	    FILE *it;
	    char abuf[BUFSIZ];

	    if (!nomore) {
		if ((it = fopen(work, "r")) == NULL) {
		    perror(work);
		    exit(1);
		}
		if (fgets(abuf, BUFSIZ-1, it) &&
		    abuf[0] == '.' && abuf[1] == 's' &&
		    abuf[2] == 'o' && abuf[3] == ' ') {
		    register char *cp = abuf+strlen(".so ");
		    char *dp;

		    while (*cp && *cp != '\n')
		      cp++;
		    *cp = 0;
		    while (cp > abuf && *--cp != '/')
		      ;
		    dp = ".so man";
		    if (cp != abuf+strlen(dp)+1) {
		    tohard:
			fclose(it);
			nomore = 1;
			strcpy(work, abuf+4);
			goto hardway;
		    }
		    for (cp = abuf; *cp == *dp && *cp; cp++, dp++)
		      ;
		    if (*dp)
		      goto tohard;
		    strcpy(work, cp-3);
		}
		fclose(it);
	    }
	}
		strcpy(work2, "cat");
		work2[3] = work[3];
		work2[4] = 0;
		if (stat(work2, &stbuf2) < 0)
			goto hardway;
		strcpy(work2+4, work+4);
		if (stat(work2, &stbuf2) < 0 || stbuf2.st_mtime < stbuf.st_mtime) {
			if (nomore)
				goto hardway;
			printf("Reformatting page.  Wait...");
			fflush(stdout);
			unlink(work2);
			sprintf(cmdbuf,
		"%s %s %s > /tmp/man%d; trap '' 1 15; /bin/mv /tmp/man%d %s",
				NROFFCAT, "-man", work, getpid(),
				getpid(), work2);
			if (system(cmdbuf, 0)) {
				printf(" aborted (sorry)\n");
				removeTmpFile();
				/*NOTREACHED*/
			}
			printf(" done\n");
		}
		strcpy(work, work2);
hardway:
		nroff(work);
}

nroff(cp)
	char *cp;
{
	char cmd[BUFSIZ];
	char pagers[25];
	char *pager = pagers;

	if (getenv("PAGER") != 0)
		strcpy(pager, getenv("PAGER"));
		else strcpy(pager, MORE);

	if (cp[0] == 'c')
		sprintf(cmd, "%s %s", nomore? CAT : pager, cp);
	else
		sprintf(cmd, nomore? "%s %s %s" : "%s %s %s|%s",
			NROFF, "-man", cp, pager);
	system(cmd);
}

any(c, sp)
	register int c;
	register char *sp;
{
	register int d;

	while (d = *sp++)
		if (c == d)
			return (1);
	return (0);
}

void removeTmpFile()
{
    char name[15];

    sprintf(name, "/tmp/man%d", getpid());
    unlink(name);
    exit(1);
}

match(buf, str)
	char *buf, *str;
{
	register char *bp, *cp;

	bp = buf;
	for (;;) {
		if (*bp == 0)
			return (0);
		if (amatch(bp, str))
			return (1);
		bp++;
	}
}

amatch(cp, dp)
	register char *cp, *dp;
{

	while (*cp && *dp && lmatch(*cp, *dp))
		cp++, dp++;
	if (*dp == 0)
		return (1);
	return (0);
}

lmatch(c, d)
	char c, d;
{

	if (c == d)
		return (1);
	if (!isalpha(c) || !isalpha(d))
		return (0);
	if (islower(c))
		c = toupper(c);
	if (islower(d))
		d = toupper(d);
	return (c == d);
}

blklen(ip)
	register int *ip;
{
	register int i = 0;

	while (*ip++)
		i++;
	return (i);
}


char *
trim(cp)
	register char *cp;
{
	register char *dp;

	for (dp = cp; *dp; dp++)
		if (*dp == '/')
			cp = dp + 1;
	if (cp[0] != '.') {
		if (cp + 3 <= dp && dp[-2] == '.' && any(dp[-1], "cosa12345678npP"))
			dp[-2] = 0;
		if (cp + 4 <= dp && dp[-3] == '.' && any(dp[-2], "13") && isalpha(dp[-1]))
			dp[-3] = 0;
	}
	return (cp);
}

system(s)
char *s;
{
	int status, pid, w;
	SIGNAL_FN (*saveintr)(), (*savequit)();

	if ((pid = vfork()) == 0) {
		execl("/bin/sh", "sh", "-c", s, 0);
		_exit(127);
	}
	saveintr = signal (SIGINT, SIG_IGN);
	savequit = signal (SIGQUIT, SIG_IGN);
	while ((w = wait(&status)) != pid && w != -1)
		;
	(void) signal (SIGQUIT, saveintr);
	(void) signal (SIGQUIT, savequit);
	if (w == -1)
		status = -1;
	return (status);
}

texpand(file, result, rlen)
    char *file;
    char *result;
    int rlen;
{
    struct passwd *pwd;
    register int len;
    register char *p;
    char name[20];

    if (file[0] == '~')
    {
	p = file, len = 0;
	do
	{
	    p++;
	    if (isspace(*p) || *p=='\0' || *p=='/')
	    {
		/* end of name */
		name[len] = '\0';
		pwd = getpwnam(name);
		if (pwd == NULL)
		    break;

		/* return original if not enough space */
		if (strlen(pwd->pw_dir) + strlen(p) + 1 >= rlen)
		    break;

		strcpy(result, pwd->pw_dir);
		strcat(result, p);
		return;
	    }
	    name[len++] = *p;	/* add char to name */
	} while (len < sizeof name);
    }

    /* on name overflow use original file name */
    strncpy(result, file, rlen);
    result[rlen-1] = '\0';
}
