#include "copyright.h"
#include "port.h"
#include "oct.h"
#include "oh.h"

extern FILE *output;
extern int next_cell;

extern void error();
extern void inst_to_master(), add_cell();

#define FPRINTF	(void) fprintf

#define CIF_MASK \
    OCT_BOX_MASK|OCT_CIRCLE_MASK|OCT_POLYGON_MASK|OCT_PATH_MASK|OCT_LABEL_MASK

convert_facet(facet, cell_num)
octObject *facet;
int cell_num;
{
    octObject object, inst, master, layer;
    octGenerator inst_gen, gen, layer_gen;
    int status, in_table;
    char *cif_trans(), *fix_layer_name();
    extern void do_mask_mod();

    do_mask_mod(facet);
    
    OH_ASSERT(octInitGenContents(facet,OCT_INSTANCE_MASK, &inst_gen));

    while ((status = octGenerate(&inst_gen,&inst)) == OCT_OK) {
	inst_to_master(&inst, &master);
	if ((status = octOpenMaster(&inst, &master)) < 0 || status == OCT_NEW_FACET) {
	    char err_buf[300];
	    extern char *optProgName;
	    (void) sprintf(err_buf, "%s: Can't open %s", optProgName, ohFormatName(&master));
	    error(err_buf);
	}

	in_table = lookup_cell(&master);
	
	if (status != OCT_ALREADY_OPEN && in_table) {
	    error("Panic: mismatch between st_lookup and ALREADY_OPEN");
	}
	
	if (!in_table) {
	    add_cell(&master, next_cell);
	    convert_facet(&master, next_cell++);
	}
    }
        
    if (status != OCT_GEN_DONE) {
	error("instance generator");
    }

    FPRINTF(output, "DS %d;\n", cell_num);
    FPRINTF(output, "9 %s;\n", facet->contents.facet.cell);
    
    OH_ASSERT(octInitGenContents(facet,OCT_INSTANCE_MASK, &inst_gen));

    while ((status = octGenerate(&inst_gen, &inst)) == OCT_OK) {
	int sub_cell_num;
	
	inst_to_master(&inst, &master);
	OH_ASSERT(octOpenMaster(&inst, &master));

	if ((sub_cell_num = lookup_cell(&master)) == 0) {
	    error("Panic: lost the cell for an instance");
	}
	
	FPRINTF(output,"C %d %s;\n", sub_cell_num,
		cif_trans(inst.contents.instance.transform));
    }

    if (status != OCT_GEN_DONE) {
	error("instance generator");
    }
    
    OH_ASSERT(octInitGenContents(facet,OCT_LAYER_MASK, &layer_gen));

    while ((status = octGenerate(&layer_gen,&layer)) == OCT_OK) {
	char *layer_name = fix_layer_name(layer.contents.layer.name);
	
	FPRINTF(output,"L %s;\n", layer_name);
	
	if (octInitGenContents(&layer, CIF_MASK, &gen) != OCT_OK) {
	    error("generating geometries");
	}

	while ((status = octGenerate(&gen,&object)) == OCT_OK) {
	    process_object(&object, layer_name);
	}
	
	if (status != OCT_GEN_DONE) {
	    error("geo generator");
	}
    }
    
    if (status != OCT_GEN_DONE) {
	error("layer generator");
    }

    FPRINTF(output, "DF;\n\n");
}

char *trans_table[] = {
    " ",		
    "MX ",	
    "MY ",	
    "R 0 1 ",	
    "R -1 0 ",	
    "R 0 -1 ",	
    "MX R 0 1 ",
    "MY R 0 1 ",
    "too complicated for the moment"
};

#define LENGTH 1000		/* magnitude of the direction vector */

char *cif_trans(trans)
struct octTransform trans;
{
    static char buf[100];

    if (trans.transformType != OCT_FULL_TRANSFORM) {
	(void) strcpy(buf, trans_table[(int) trans.transformType]);
    } else {
	double (*arr)[2] = trans.generalTransform;

	buf[0] = '\0';
	if (arr[0][0] < 0) {
	    (void) strcat(buf, "MX ");
	    arr[0][0] = -arr[0][0];
	    arr[1][0] = -arr[1][0];
	}
	if (arr[1][1] < 0) {
	    (void) strcat(buf, "MY ");
	    arr[0][1] = -arr[0][1];
	    arr[1][1] = -arr[1][1];
	}

	(void) sprintf(buf+strlen(buf), "R %d %d ", (int) (LENGTH*arr[0][0]),
		(int) (LENGTH*arr[1][0]));
    }
    (void) sprintf(buf+strlen(buf), "T %d %d", trans.translation.x,
	    trans.translation.y);
    return buf;
}
