#include "copyright.h"
#include "port.h"
#include "options.h"
#include "oct.h"
#include "oh.h"
#include "octmm.h"
#include "vov.h"

#define USAGE 

#define FPRINTF	(void) fprintf
#define strsav(s)	strcpy(calloc((unsigned)(strlen(s)+1), sizeof(char)), s)
#define NIL(type) ((type *) 0)

FILE *output = stdout; 
char *optProgName;
static char *scale = NIL(char);
static char *fname = NIL(char);

int next_cell = 1;


optionStruct optionList[] = {
    { "o",	"cif_file",	"output cif file" },
    { "f",	"mod_file",	"octmm modification script" },
    { "s",	"scale",	"amount to scale the design" },
    { OPT_RARG,	"cell:view[:facet]",	"facet defaults to `contents'" },
    { 0,	0,		0},
};


main(argc, argv)
int argc;
char **argv;
{
    extern int optind;
    extern char *optarg;
    int option;
    octObject top_level;
    octStatus status;
    char* outputFile = NIL(char);

    while ((option = optGetOption(argc, argv)) != EOF) {
	switch (option) {
	case 'o':
	    outputFile = optarg;
	    break;
	case 's':
	    scale = strsav(optarg);
	    break;
	case 'f':
	    fname = strsav(optarg);
	    break;
	}
    }

    if (argc == optind+1) {
	ohUnpackDefaults(&top_level, "r", ":physical:contents");
	if (ohUnpackFacetName(&top_level, argv[optind]) != OCT_OK) {
	    optUsage();
	}
    }

    VOVbegin( argc, argv );

    if ( outputFile ) {
	output = VOVfopen(outputFile, "w");
	if (output == NULL) {
	    FPRINTF(stderr, "Cannot open output file %s\n", outputFile);
	    optUsage();
	}
    }



    init_cell_table();

    VOVinputFacet( &top_level );
    if (octOpenFacet(&top_level) < OCT_OK) {
	FPRINTF(stderr, "%s: Can't open %s:%s:%s:%s ", optProgName,
		top_level.contents.facet.cell,
		top_level.contents.facet.view,
		top_level.contents.facet.facet,
		top_level.contents.facet.version);
	octError("");
	optUsage();
    }

    if (status == OCT_INCONSISTENT) {
	ohPrintInconsistent(&top_level, stdout);
	VOVend(-1);
    }

    add_cell(&top_level, next_cell);
    convert_facet(&top_level, next_cell++);
    FPRINTF(output, "C 1;\nE\n");

    VOVend(0);
}


void do_mask_mod(facet)
octObject *facet;
{
    oct_mask_mod(optProgName, fname, facet, scale);
    return;
}

