#include "copyright.h"
#include "port.h"
#include "oct.h"

#define abs(x) ((x) < 0 ? -(x) : (x))

#define FPRINTF	(void) fprintf

extern FILE *output;

process_object(obj, layer_name)
octObject *obj;
char *layer_name;
{
    int num_points;
    struct octPoint *arr;
    
    switch (obj->type) {
    case OCT_BOX:
	{
	    struct octBox *box = &obj->contents.box;
	    FPRINTF(output, "    B %d %d %d %d;\n",
		    abs(box->upperRight.x - box->lowerLeft.x),
		    abs(box->upperRight.y - box->lowerLeft.y),
		    (box->upperRight.x + box->lowerLeft.x)/2,
		    (box->upperRight.y + box->lowerLeft.y)/2);
	    break;
	}
    case OCT_PATH:
	{
	    struct octPath *path = &obj->contents.path;
	    int i;
	    
	    FPRINTF(output,"    W %d", path->width);
	    if (get_points(obj, &num_points, &arr) != OCT_OK) {
		error("Getting points on a path");
	    }
	    for (i = 0; i < num_points; i++) {
		FPRINTF(output, " %d %d", arr[i].x, arr[i].y);
	    }
	    FPRINTF(output, ";\n");
	    break;
	}
    case OCT_POLYGON:
	{
	    int i;
	    
	    FPRINTF(output,"    P ");
	    if (get_points(obj, &num_points, &arr) != OCT_OK) {
		error("Getting points on a polygon");
	    }
	    for (i = 0; i < num_points; i++) {
		FPRINTF(output, " %d %d", arr[i].x, arr[i].y);
	    }
	    FPRINTF(output, ";\n");
	    break;
	}
    case OCT_CIRCLE:
	{
	    struct octCircle *circle = &obj->contents.circle;
	    if ((circle->endingAngle - circle->startingAngle != 3600) ||
		circle->innerRadius != 0) {
		    FPRINTF(stderr, "Ignoring non-circular circle\n");
		} else {
		    FPRINTF(output,"    R %d %d %d;\n", circle->outerRadius,
			    circle->center.x, circle->center.y);
		}
	    break;
	}
    case OCT_LABEL:
	    /* XXX needs to be fixed */
	{
	    struct octLabel *label = &obj->contents.label;
	    char *ptr;

	    if ((ptr = strchr(label->label, '\n')) != 0) {
		*ptr = '\0';
	    }
	    FPRINTF(output, "94 %s %d %d %s;\n", label->label,
		    label->region.lowerLeft.x, label->region.lowerLeft.y,
		    layer_name);
	    break;
	}
    }
}
