/*      ROUTINE FOR PRINTING 
*		BOUNDING CONSTRAINTS 
*		AND RELATED INFORMATION
*       ---------------------------------------
*
*   File:       bound_pr.c
*   Author:     Umakanta Choudhury
*   Date:       26th June 1989
*
*
*/

#include <stdio.h>
#include "layout.h"
#include "circuit.h"

extern struct Sens_link *find_stored_link();



REPORT bound_pr(layout,circuit,debug_file)
struct Layout *layout;
struct Circuit *circuit;
FILE *debug_file;
{
	struct Performance *perf;
	struct CriticalPairLink *critpair;
	struct Chan_info *chan_info;
	float pos_change[50];
	float neg_change[50];
	struct Sens_link *sens_link;
	float sens;
	int i;

	FILE *maxmin_temp;
	FILE *router_inp;
    char filename[50];

    /* Open the text file */
    sprintf(filename,"%s_%d","/tmp/parcar.maxmin_temp",getpid());
    if( (maxmin_temp = fopen(filename,"w")) == NULL) {
        printf("\nE: could not open file %s",filename);
        return ERROR;
    }
    sprintf(filename,"%s_%d","/tmp/parcar.router_inp",getpid());
    if( (router_inp = fopen(filename,"w")) == NULL) {
        printf("\nE: could not open file %s",filename);
        return ERROR;
    }

	fprintf(debug_file,"\nbound_pr.c\n");
	fprintf(debug_file,"cap_threshold = %g\n\n",layout->cap_threshold);
	for(perf = circuit->perf_list; perf != NULL; perf = perf->next){
		if(perf->pos_constraint == YES){
		/* constraint on maximum positive change present */
			fprintf(debug_file,"perf %s pos_eqn_num = %d\n",perf->name,
					perf->pos_eqn_num);
		}
		if(perf->neg_constraint == YES){
		/* constraint on maximum negative change present */
			fprintf(debug_file,"perf %s neg_eqn_num = %d\n",perf->name,
					perf->neg_eqn_num);
		}
		pos_change[perf->number] = 0; 
		neg_change[perf->number] = 0; 
	}
	fprintf(maxmin_temp,"number of nets %d\n",circuit->num_nodes);
	fprintf(maxmin_temp,"number of net pairs %d\n",
		layout->num_bound_constraints);

	for(critpair = layout->critical_pairs; critpair != NULL;
		critpair = critpair->next){
		for(chan_info = critpair->chan_info_list; chan_info != NULL;
			chan_info = chan_info->next){
			if(chan_info->C_max > layout->cap_threshold){
				fprintf(debug_file,"\npair (%s,%s) chan %s param_num = %d\n",
					critpair->netA_name,critpair->netB_name,
					chan_info->channel->name,chan_info->param_num);
				fprintf(debug_file,"C_bound[%d] = %g, max = %g, min = %g, shield_min = %g \n",
				chan_info->param_num,chan_info->C_bound,
				chan_info->C_max,chan_info->C_min,
				chan_info->C_min_shield);
				fprintf(maxmin_temp,"%s\t%s\t%.3e\t%.3e\n",
					critpair->netA_name,critpair->netB_name,
					chan_info->C_max,chan_info->C_min_shield);

				for(perf = circuit->perf_list; perf != NULL; 
					perf = perf->next){ 
					/* find the stored sensitivity link for 
						C(name1A,name2A) */
					if((sens_link = find_stored_link(perf,critpair->netA_name,critpair->netB_name)) == NULL){
						sens = 0;
					}
					else{
						sens = sens_link->sens;
					}
					if(sens >= 0){
						pos_change[perf->number] 
							+= chan_info->C_bound * sens;
					}
					else{
						neg_change[perf->number] += 
							chan_info->C_bound * sens;
					}
					fprintf(debug_file,"%s sens = %g\n",perf->name,sens);
				}
			}
		}
	}
	fprintf(debug_file,"\nprediction of linear analysis\n");
	for(i=1; i<=circuit->num_perf; i++){
		fprintf(debug_file,"%s pos_change=%g neg_change=%g tot_change=%g\n",
		*(layout->perf_name + i),pos_change[i],neg_change[i],
		pos_change[i]+neg_change[i]);
	}
	fprintf(debug_file,"\ninput for checking linear analysis \n");
	/*
	fprintf(router_inp,"channel\tnet1\tnet2\tCbound(fF)\tID(micron)\n\n");
	*/
	fprintf(router_inp,"channel\tnet1\tnet2\tCbound(fF)\n\n");
	for(critpair = layout->critical_pairs; critpair != NULL;
		critpair = critpair->next){
		for(chan_info = critpair->chan_info_list; chan_info != NULL;
			chan_info = chan_info->next){
			if(chan_info->C_max > layout->cap_threshold){
				fprintf(debug_file,"cap%s%s_%s %s %s %gfF\n",
					critpair->netA_name,critpair->netB_name,
					chan_info->channel->name,
					critpair->netA_name,critpair->netB_name,
					chan_info->C_bound);
				fprintf(router_inp,"%s\t\t%s\t\t%s\t\t%.2e\n",
					chan_info->channel->name,
					critpair->netA_name,critpair->netB_name,
					chan_info->C_bound);
			}
		}
	}
	fclose(maxmin_temp);
	fclose(router_inp);
	return(OK);
}
