/*		CAPACITANCE LIBRARY HEADER
*		--------------------------
*
*   File:	cap_library.h
*   Author: Umakanta Choudhury
*   Date:	11 June 1989.
*
*/
#ifndef  __CAPLIBRARY_HEADER__
#define  __CAPLIBRARY_HEADER__

/****************** STRUCTURES ****************************************/

/* CAPACITANCE LIBRARY DATA STRUCTURE 
 * ---------------------------------- 
 * Capacitance library obtained for a given technology
 * using curve fitting of data available from field 
 * simulator
 * sw1_inv = 1/(s+w1); sw2_inv = 1/(s+w2); sinv = 1/s12,
 * sinv_sq = 1/(s*s);
 *
 * For two line segments running parallel in X_LAYER the per 
 * unit length capacitance is
 * cap = coeffX0 + coeffX_sw1inv * sw1_inv + coeffX_sw2inv * sw2_inv
 *     + coeffX_sinv * sinv + coeffX_sinv_sq * sinv_sq
 *   
 *  where w1 and w2 are widths of the lines and s12 is
 * the distance between the lines 
 *
 * similarly for Y_LAYER and XY_LAYER (one line segment 
 * in X and one in Y layer)
 *
 * the crossover capacitance between two crossing
 * lines of width w1 (one in X_LAYER) and width
 * w2 (one in Y_LAYER) is given by
 *
 *		cap = coeffC0 + coeffC_w1 * w1 + coeffC_w2 * w2;
 *			+ coeffC_w1w2 * w1 * w2;
 *
 */

struct Cap_Library {
	char    *tek_name;			/* technology name for which
									the library has been built */
    /* various coefficients */
	double coeffX0;
	double coeffX_sinv;
	double coeffX_sinv_sq;
	double coeffX_sw1inv;
	double coeffX_sw2inv;
	double coeffY0;
	double coeffY_sinv;
	double coeffY_sinv_sq;
	double coeffY_sw1inv;
	double coeffY_sw2inv;
	double coeffC0;
	double coeffC_w1;
	double coeffC_w2;
	double coeffC_w1w2;
	double s_min; /* minimum s for which the model is valid */
	double s_max; /* maximum s for which the model is valid */
	double w_min; /* minimum w for which the model is valid */
	double w_max; /* maximum w for which the model is valid */

	} *cap_library ;

#endif
