/*      ROUTINE FOR ESTIMATING MAXIMUM AND
*       MINIMUM CROSSOVER CAPACITANCES BETWEEN 
*		TWO NETS 
*       -------------------------------------------
*
*   File:       ccross_maxmin.c
*   Author:     Umakanta Choudhury
*   Date:       15 July 1989
*
*
*/

#include <stdio.h>
#include "layout.h"

extern int strcmp();


REPORT ccross_maxmin(chan_info)
struct Chan_info *chan_info;
{
	struct Channel *channel;
	struct Channel *top_chan;
	struct Channel *bottom_chan;
	struct ChannelLink *top_chan_link;
	struct ChannelLink *bottom_chan_link;
	struct Net *net1;
	struct Net *net2;
	double left; 
	double right; 
	double W1; 
	double W2; 
	double minW; 
	double max_common_span; /* maximum common span in x direction of 
								the two nets */
	double min_common_span; /* minimum common span in x direction of 
								the two nets */
	float chan_net_sep;

	channel = chan_info->channel;
	chan_net_sep = channel->chan_net_separation;
	net1 = chan_info->netA;
   	net2 = chan_info->netB;
	W1 = 2*(net1->true_half_width);
	W2 = 2*(net2->true_half_width);
	minW = MIN(W1,W2);


	/* compute the maximum common span */
	left = MAX(net1->max_span_left,net2->max_span_left);
	right = MIN(net1->max_span_right,net2->max_span_right);
	if(right < left) max_common_span = 0;
	else max_common_span = right - left;

	chan_info->Ccross_1above2 = 0;
	chan_info->Ccross_2above1 = 0;

	if(max_common_span != 0){

		/* compute the maximum crossover capacitance due to
			fixed pins */

		if(fixpin_cross(chan_info,left,right) != OK){
			return(ERROR);
		}

		/* compute the maximum crossover capacitance due to
		floating pins */

		/* scan through the top channels */
		for(top_chan_link = channel->top_chan_list; 
			top_chan_link != NULL;
			top_chan_link = top_chan_link->next){

			top_chan = top_chan_link->channel;

			if((top_chan->top - chan_net_sep  > left) && 
				(top_chan->bottom + chan_net_sep  < right)){
				/* intersects with the common span */
				if(flpin_cross(chan_info,top_chan,TOP) 
					!= OK){
					return(ERROR);
				}
			}
		}
		for(bottom_chan_link = channel->bottom_chan_list; 
			bottom_chan_link != NULL;
			bottom_chan_link = bottom_chan_link->next){

			bottom_chan = bottom_chan_link->channel;

			if((bottom_chan->top - chan_net_sep > left) && 
				(bottom_chan->bottom + chan_net_sep < right)){
				/* intersects with the common span */
				if(flpin_cross(chan_info,bottom_chan,BOTTOM)
					!= OK){
					return(ERROR);
				}
			}
		}
	}

	chan_info->Ccross_max = MAX(chan_info->Ccross_1above2,
		chan_info->Ccross_2above1);


	/* compute the minimum common span */
	left = MAX(net1->min_span_left,net2->min_span_left);
	right = MIN(net1->min_span_right,net2->min_span_right);
	if(right < left) min_common_span = 0;
	else min_common_span = right - left;

	chan_info->Ccross_1above2 = 0;
	chan_info->Ccross_2above1 = 0;

	if(min_common_span != 0){

		/* compute the minimum crossover capacitance due to
			fixed pins */

		if(fixpin_cross(chan_info,left,right) != OK){
			return(ERROR);
		}
		/* compute the minimum crossover capacitance due to
			floating pins */
	
		/* scan through the top channels */
		for(top_chan_link = channel->top_chan_list; 
			top_chan_link != NULL;
			top_chan_link = top_chan_link->next){

			top_chan = top_chan_link->channel;


			if((top_chan->top - chan_net_sep - minW < right) && 
				(top_chan->bottom + chan_net_sep + minW > left)){
				/* must intersect the minimum common span */
				if(flpin_cross(chan_info,top_chan,TOP) 
					!= OK){
					return(ERROR);
				}
			}
		}
		/* scan through the bottom channels */
		for(bottom_chan_link = channel->bottom_chan_list; 
			bottom_chan_link != NULL; 
			bottom_chan_link = bottom_chan_link->next){

			bottom_chan = bottom_chan_link->channel;

			if((bottom_chan->top - chan_net_sep - minW < right) 
				&& (bottom_chan->bottom + chan_net_sep + minW > left)){
				/* must intersect the minimum common span */
				if(flpin_cross(chan_info,bottom_chan,BOTTOM)
					!= OK){
					return(ERROR);
				}
			}
		}
	}
	
	chan_info->Ccross_min = MIN(chan_info->Ccross_1above2,
		chan_info->Ccross_2above1);

	return(OK);
}

