/*      ROUTINE FOR PRINTING 
*		BOUNDING CONSTRAINTS 
*		AND SENSITIVITIES FOR CRITICAL PARASITICS
*       -----------------------------------------
*
*   File:       constraints_pr.c
*   Author:     Umakanta Choudhury
*   Date:       21st Feb 1990
*
*
*/

#include <stdio.h>
#include "layout.h"
#include "circuit.h"

extern struct Sens_link *find_link();

void
  constraint_pr(layout,circuit, out_file)
struct Layout *layout;
struct Circuit *circuit;
FILE *out_file;
{
    FILE *constraint_file;
    struct Performance *perf;
    struct CriticalPairLink *critpair;
    struct Chan_info *chan_info;
    struct Node *nodeA, *nodeB;
    float pos_change[50];
    float neg_change[50];
    struct Sens_link *sens_link;
    float sens;
    char filename[128];
    char *nameA, *nameB;
    char *name1, *name2;
    int i;
    int imax;

    sprintf(filename, "%s_%d","/tmp/parcar.constraints", getpid()); 
    constraint_file  = fopen(filename, "w");

    fprintf(constraint_file,"number of performance functions %d \n",
	    circuit->num_perf);
    fprintf(out_file,"%d\t%d\n",layout->num_bound_constraints,
	    circuit->num_perf);
    for(perf = circuit->perf_list; perf != NULL; perf = perf->next){
	if(perf->pos_constraint == YES){
	    /* constraint on maximum positive change present */
	    fprintf(constraint_file,"%s\tdelta+ = %.3e",perf->name,
		    perf->max_pos_change);
	    fprintf(out_file,"%.3e\t",perf->max_pos_change);
	}
	else{
	    /* -1 indicates no constraint */
	    fprintf(constraint_file,"%s\tdelta+ = -1",perf->name);
	    fprintf(out_file,"-1\t",perf->name);
	}
	if(perf->neg_constraint == YES){
	    /* constraint on maximum negative change present */
	    fprintf(constraint_file,"\tdelta- = %.3e\n",
		    (-1)*perf->max_neg_change);
	    fprintf(out_file,"%.3e\n",(-1)*perf->max_neg_change);
	}
	else{
	    /* -1 indicates no constraint */
	    fprintf(constraint_file,"\tdelta- = -1\n");
	    fprintf(out_file,"-1\n");
	}
    }
    fprintf(constraint_file,"number of bounding constraints %d \n",
		layout->num_bound_constraints);
    
    for(critpair = layout->critical_pairs; critpair != NULL;
	critpair = critpair->next){
	nameA = critpair->netA_name;
	nameB = critpair->netB_name;
	if(avl_lookup(circuit->node_tree,nameA,&nodeA) == NULL){
	    printf("node %s not found in the circuit\n",nameA);
	}
	if(avl_lookup(circuit->node_tree,nameB,&nodeB) == NULL){
	    printf("node %s not found in the circuit\n",nameB);
	}
	imax = 1;
	if((nodeA->matched) || (nodeB->matched)) imax = 2;
	if((nodeA->matched) && (nodeB->matched)) imax = 1;
	/* need to print the information for the matched pair 
	   also */
	
	for(chan_info = critpair->chan_info_list; chan_info != NULL;
	    chan_info = chan_info->next){
	    
	    if(chan_info->C_max > layout->cap_threshold){
		for(i=1; i<= imax; i++){
		    if(i==1){
			name1 = nodeA->name;
			name2 = nodeB->name;
		    }
		    else if(i==2){
			/* get the node names for the matched pair */
			if(nodeA->matched) 
			  name1 = nodeA->matched_node->name;
			if(nodeB->matched) 
			  name2 = nodeB->matched_node->name;
		    }
		    else{
			printf("constraint_pr: cannot occur\n");
			exit( -1 );
		    }
		    fprintf(constraint_file,"%s\t%s\tC_bound = %.3e",
			    name1,name2,chan_info->C_bound);
		    fprintf(out_file,"%s\t%s\t%.3e",name1,name2,
			    chan_info->C_bound);
		    
		    for(perf = circuit->perf_list; perf != NULL; 
			perf = perf->next){ 
			/* find the positive sensitivity link for 
			   C(nameA,nameB) */
			/*
			  if((sens_link = find_link(perf,nameA,nameB,
			  POS_LIST)) == NULL){
			  sens = 0;
			  }
			  else{
			  sens = sens_link->sens;
			  }
			  */
                        sens = *(critpair->pos_sens + perf->number);
			fprintf(constraint_file,"\tS+ = %.3e",sens);
			fprintf(out_file,"\t%.3e",sens);
                        sens = *(critpair->neg_sens + perf->number);
			fprintf(constraint_file,"\tS- = %.3e",
				(-1)*sens);
			fprintf(out_file,"\t%.3e",(-1)*sens);
		    }
		    fprintf(constraint_file,"\n");
		    fprintf(out_file,"\n");
		}
	    }
	}
    }
    fclose( constraint_file );
}
