/*		ROUTINE FOR ESTIMATING MAXIMUM AND
 *		MINIMUM CAPACITANCES IN X-DIRECTION BETWEEN 
 *		TWO NETS 
 *       -------------------------------------------
 *
 *   File:       cx_maxmin.c
 *   Author:     Umakanta Choudhury
 *   Date:       15 July 1989
 *   Last modified:  Enrico Malavasi,  Aug. 1992
 *
 *
 */

#include <stdio.h>
#include <string.h>
#include "layout.h"

extern double cap_parallel();
extern float chan_net_com();


REPORT
  cx_maxmin(chan_info)
struct Chan_info *chan_info;
{
	struct Channel *channel;
	struct ChannelLink *top_chan_link;
	struct ChannelLink *bottom_chan_link;
	struct Channel	*top_chan;
	struct Channel	*bottom_chan;
	float net_separation;
	float chan_net_sep;
	float channel_width;
	struct Net *net1;
	struct Net *net2;
	float W1; /* width of net1 */
	float W2; /* width of net2 */
	float left; 
	float right; 
	float d_min; /* minimum distance between two adjacent nets running
				parallel used for computation of maximum capacitance */
	float cap; 
	float max_common_span; 
	int	first_pin;
	float	chan_intersect;
	float	Cx_max_top_12;
	float	Cx_max_top_21;
	float	Cx_max_bottom_12;
	float	Cx_max_bottom_21;
	float	Cx_max_top;
	float	Cx_max_bottom;

#ifdef DEBUG
	printf("cx_maxmin\n");
#endif 

	channel = chan_info->channel;
	net_separation = channel->default_net_separation;
	chan_net_sep= channel->chan_net_separation;
	channel_width = channel->top - channel->bottom;
	net1 = chan_info->netA;
	net2 = chan_info->netB;
	W1 = 2*(net1->true_half_width);
	W2 = 2*(net2->true_half_width);


	/* compute the maximum common span */
	left = MAX(net1->max_span_left,net2->max_span_left);
	right = MIN(net1->max_span_right,net2->max_span_right);
	if(right < left) max_common_span = 0;
	else max_common_span = right - left;

	/* first the coupling inside the channel */
	d_min = net_separation;
	cap = cap_parallel(W1,W2,d_min,X_LAYER);
	chan_info->Cx_max = max_common_span * cap;
	chan_info->Cx_min = 0; /* when the separation is very large */

	chan_info->Lx_max += max_common_span;


#ifdef DEBUG
	printf("net1 %s ; net2  %s\n",net1->name,net2->name);
	printf("net1_max_left = %f net1_max_right = %f\n",
		net1->max_span_left,net1->max_span_right);
	printf("net2_max_left = %f net2_max_right = %f\n",
		net2->max_span_left,net2->max_span_right);
	printf("max_common_span = %f W1 = %f W2 = %f\n",max_common_span,
			W1,W2);
	printf("d_min = %f cap = %f \n",d_min,cap);
#endif 
									
	/* then consider the inter channel coupling w.r.t top and 
		bottom channels (coupling w.r.t left and right channels 
		will be considered while considering those channels to 
		avoid duplication) */

	/* compute maximum coupling w.r.t top channels */

	Cx_max_top_12 = 0; /* net1 inside and net2 outside */
	Cx_max_top_21 = 0; /* net2 inside and net1 outside */

	for(top_chan_link = channel->top_chan_list; top_chan_link != NULL;
			top_chan_link = top_chan_link->next){

		top_chan = top_chan_link->channel;

		/* look for coupling between 
			net1 inside and net2 in top_chan */ 
		if(cx_top_maxmin(chan_info,top_chan,net1,net2) != OK) {
		    return(ERROR);
		}
		Cx_max_top_12 += chan_info->Cx_max_top;
#ifdef DEBUG
		printf("Cx_max_top_12 = %g \n",Cx_max_top_12);
#endif 

		/* look for coupling between 
			net2 inside and net1 in top_chan */ 
		if(cx_top_maxmin(chan_info,top_chan,net2,net1) != OK){
		    return(ERROR);
		}
		Cx_max_top_21 += chan_info->Cx_max_top;
#ifdef DEBUG
		printf("Cx_max_top_21 = %g \n",Cx_max_top_21);
#endif 

	}

	/* compute maximum coupling w.r.t bottom channels */

	Cx_max_bottom_12 = 0; /* net1 inside and net2 outside */
	Cx_max_bottom_21 = 0; /* net2 inside and net1 outside */

	for(bottom_chan_link = channel->bottom_chan_list; bottom_chan_link 
		!= NULL; bottom_chan_link = bottom_chan_link->next){

		bottom_chan = bottom_chan_link->channel;

		/* look for coupling between 
			net1 inside and net2 in bottom_chan */ 
		if(cx_bot_maxmin(chan_info,bottom_chan,net1,net2) != OK){
		    return(ERROR);
		}
		Cx_max_bottom_12 += chan_info->Cx_max_bottom;

		/* look for coupling between 
			net2 inside and net1 in bottom_chan */ 
		if(cx_bot_maxmin(chan_info,bottom_chan,net2,net1) != OK){
		    return(ERROR);
		}
		Cx_max_bottom_21 += chan_info->Cx_max_bottom;

	}

	/*
	if(max_common_span == 0){
		Cx_max_top = Cx_max_top_12 + Cx_max_top_21;
		Cx_max_bottom = Cx_max_bottom_12 + Cx_max_bottom_21;
	}
	else{
		Cx_max_top = MAX(Cx_max_top_12, Cx_max_top_21);
		Cx_max_bottom = MAX(Cx_max_bottom_12, Cx_max_bottom_21);
	}
	*/

	Cx_max_top = Cx_max_top_12 + Cx_max_top_21;
	Cx_max_bottom = Cx_max_bottom_12 + Cx_max_bottom_21;

	chan_info->Cx_max += Cx_max_top + Cx_max_bottom;
	return(OK);
}

