/*		READ TEXT FILE
*		--------------
*
*   File:	layout_read.c
*   Original Author:	Nicholas Weiner
*   Date:	13th March 1988.
*   Modified by Umakanta Choudhury
*   Date:   9th June
*
*   Reads data from named file. 
*
*   "Mighty" input syntax has been adopted. All of the "mighty" 
*   input is accepted - some is ignored. On top of the "mighty"
*   input, further data may be supplied for "roadrunner".
*/
#include <stdio.h>
#include "utility.h"
#include "layout.h"

#define MAX_LINE_LENGTH 200
#define PRINT_LINE_DATA {printf("%s: line %d\n",text_file_name,linenum);}
#define CHECK_RETURN(X) \
  {int R; R=(X); if(R!=OK){PRINT_LINE_DATA;if(R==ERROR) exit( -1 );}}
#define DECREMENT_NUMBER_OF_ITEMS \
  {if( (--number_of_items) == 0) readin_status = READING_NORMAL;}

/*  READIN STATUS DEFINITIONS */
#define READING_NORMAL 0
#define READING_PINS 1
#define READING_LEFT_CROSSINGS 2
#define READING_RIGHT_CROSSINGS 3
#define READING_BOTTOM_CROSSINGS 4
#define READING_TOP_CROSSINGS 5
#define READING_SENSITIVE_NETS 6
#define READING_CORNERS 7
#define READING_OBSTACLES 8
#define READING_MATCHED_PAIRS 9
#define READING_CRITICAL_PAIRS 10

extern int strcmp();
/*
*  READ TEXT FILE
*  
*  Expected input syntax is made up of that from "mighty" and extra
*  "roadrunner" syntax:
*
*  "number of channels n"
*
*  Channel Info(Mighty/Roadrunner Syntax)
*  of Channel1
*  
*  .end
*
*  Channel Info(Mighty/Roadrunner Syntax)
*  of Channel2
*  
*  .end
*
*  .
*  .
*  .
*  .
*  .
*
*  Channel Info(Mighty/Roadrunner Syntax)
*  of Channeln
*  
*  .end
*
*  "mighty" input syntax:
*  ----------------------
*
*    "number_of_nets integer"    - Self evident;
*    "rectagoncorners integer"   - For "roadrunner", the integer must be
*				   2 or 4. Immediately after this line there 
*				   should be correponding corner positions:
*    "x-position y-position"	 - Corner coordinate (floating point).
*    "number_of_pins integer"	 - Number of pins on top and bottom of
*				   channel.  Immediately after this line there 
*				   should be the correctnumber of line:
*    "net_name x-pos y-pos [layer]"- Net name may be any alpha-numeric string
*				   with no spaces. x-pos and x-pos are 
*				   floats. The layer is ignored by "road-
*				   runner" and so is optional.
*    "left_list integer"	 - Gives number of nets crossing left end of
*				   channel. This line should be immediately
*				   followed by:
*    "net_name net_name .." 	 - Any number of net names per line until
*				   the correct numebr of nets has been given.
*    "right_list integer"	 - Gives number of nets crossing right end of
*				   channel.  This line should be immediately
*				   followed by:
*    "net_name net_name .." 	 - Any number of net names per line until
*				   the correct numebr of nets has been given.
*    "sensitive_nets integer"    - Gives number of nets with total length,
*				   secondary layer length, and number of
*				   cross-overs to be minimised. This line 
*				   should be immediately followed by:
*    "net_name net_name .." 	 - Any number of net names per line until
*				   the correct numebr of nets has been given.
*    "obstacles integer"	 - Roadrunner does not cope with obstacles.
*				   If such a line is encoutered - a warning
*				   will be given, and the next integer
*                		   lines will be ignored.
*  "roadrunner" input syntax:
*  --------------------------
*
*    blank line			 - Ignored.
*    line beggining with "!"     - Comment line: ignored.
*    "read file_name"	 	 - Read text from specified file. 
*    "net_width float"		 - Subsequently declared nets have this width.
*				   Value applies until next net_width statement.
*    "net_width net_name float"  - Sets width of given net only (creates new
*				   net if net_name not already declared.
*    "net_separation float"      - Sets minimum separation between wires.
*    "contact_overhang float"    - Overhang around the wire inserect.
*    "layout_resolution float"   - Sets layout resolution. Layout will be 
*				   described in integers, where the unit 
*				   value is as set by this line.
*    "matched_pairs integer"	 - Gives number of matched pairs in channel.
*				   channel.  This line should be immediately
*				   followed by integer line of:
*    "net_name net_name" 	 - Pair of signal wires to be matched.
*    "critical_pairs integer"	 - Gives number of pairs of wires with critical
*				   coupling capacitances. This line should be 
*				   immediately followed by integer line of:
*    "net_name net_name float"	 - Pair wires and bound on capacitance.
*    "screen net_name"		 - (Usually will use Vdd for screening.) This
*				   line may be repeated and sets the current
*				   "default screen". Reading through the file
*				   the most recently specified screen will be
*				   the current default screen.
*    "screen_net net_name [net_name]- Screen the first net with the second.
*				   If only one net is specified then the 
*				   current default screen is used.
*
*
*    
*
*  Returns OK or ERROR.
*/


REPORT
 layout_read(text_file_name,layout)
char *text_file_name;		/* The input text file			*/
struct Layout *layout;	/* layout structure		*/
{
    FILE	*text_file;
    char	text_line[MAX_LINE_LENGTH];
    char	*net_list;
    int		linenum = 1;
    int		prev_linenum = 1;
    int		readin_status = READING_NORMAL;
    int		number_of_items = 0;
    float	xpos, ypos, distance, capacitance;
    char	net_name[30], another_net_name[30];
    char	chan_name[30]; 
    char	name[30]; 
    char	orient[30]; 
    char	another_file_name[100];
	struct  Channel *channel;
	struct  ChannelLink *chan_list;
	struct  ChannelLink *chan_link;
    int		num_chan;
    int		current_chan_num;
    int		i;
	FILE *file;


    /* Open the text file */
    if( (text_file = fopen(text_file_name,"r")) == NULL)     {
	printf("\nE: \"%s\" file not found.\n",text_file_name);
	return ERROR;
    }

    
    if(fgets(text_line,MAX_LINE_LENGTH,text_file) == NULL) {
	printf("nothing in the file %s\n",text_file_name);
	exit(-1);
    }
    if(sscanf(text_line,"number of channels %d",&num_chan) == 1){
	if(ds_MakeChanList(num_chan,layout) != OK) return ERROR;
    } else {
	printf("first line should specify number of channels\n");
	exit(-1);
    }
    prev_linenum = 1;
    for(chan_link = layout->chan_list; chan_link != NULL; chan_link = chan_link->next) {
	
	if(ds_MakeChannel(chan_link,&channel) != OK) return ERROR;
	
    	/* Read file, line by line */
    	for(linenum = prev_linenum + 1; ;linenum++)  {
	    if(fgets(text_line,MAX_LINE_LENGTH,text_file) == NULL)      break;
	    
	    /* Check for comment or empty line */
	    if(text_line[0] == '!') continue;
	    if(strlen(text_line) == 1) continue;
	    if(sscanf(text_line,".end %s",name) == 1)  {
		prev_linenum = linenum;
		break;/* read the next channel */
	    }
	    
	    /* Status determines whether aything specific is expected next */
	    switch(readin_status) {
		case(READING_NORMAL):
		  {
		      if(sscanf(text_line,"net_width %s %g",net_name,&distance) == 2) {
			  CHECK_RETURN(rr_dsSetNetWidth(net_name,distance,channel));
		      } else if(sscanf(text_line,"net_width %g",&distance) == 1) {
			  CHECK_RETURN(rr_dsSetDefaultNetWidth(distance,channel));
		      } else if(sscanf(text_line,"net_separation %g",&distance) == 1) {
			  CHECK_RETURN(rr_dsSetDefaultNetSeparation(distance,channel));
		      } else if(sscanf(text_line,"chan_net_separation %g",&distance) == 1) {
			  CHECK_RETURN(rr_dsSetChanNetSeparation(distance,channel));
		      } else if(sscanf(text_line,"influence distance %g",&distance) == 1) {
			  CHECK_RETURN(rr_dsSetInfluenceDistance(distance,channel));
		      } else if(sscanf(text_line,"contact_overhang %g",&distance) == 1)	{
			  /* ignored */
			  /*
			    CHECK_RETURN(rr_dsSetContactOverhang(distance,channel));
			    */
		      } else if(sscanf(text_line,"layout_resolution %g",&distance) == 1) {
			  CHECK_RETURN(rr_dsSetLayoutResolution(distance,channel));
		      } else if(sscanf(text_line,"number_of_nets %d",&number_of_items) == 1) {
			  CHECK_RETURN(rr_dsSetNumberOfNets(number_of_items,channel));
		      } else if(sscanf(text_line,"rectagoncorners %d",&number_of_items)==1) {
			  CHECK_RETURN(rr_dsSetNumberOfCorners(number_of_items,channel));
			  readin_status = READING_CORNERS;
		      } else if(sscanf(text_line,"channel name %s",chan_name)==1) {
			  channel->name = util_strsav(chan_name);
		      } else if(sscanf(text_line,"channel orientation %s",orient)==1) {
			  CHECK_RETURN(rr_dsSetChanOrient(orient,channel));
		      } else if(sscanf(text_line,"number_of_pins %d",&number_of_items)==1) {
			  readin_status = READING_PINS;
		      } else if(sscanf(text_line,"left_list %d",&number_of_items)==1) {
			  readin_status = READING_LEFT_CROSSINGS;
		      } else if(sscanf(text_line,"right_list %d",&number_of_items)==1) {
			  readin_status = READING_RIGHT_CROSSINGS;
		      } else if(sscanf(text_line,"top_list %d",&number_of_items)==1) {
			  readin_status = READING_TOP_CROSSINGS;
		      } else if(sscanf(text_line,"bottom_list %d",&number_of_items)==1)	{
			  readin_status = READING_BOTTOM_CROSSINGS;
		      } else if(sscanf(text_line,"sensitive_nets %d",&number_of_items)==1)
			{
			    readin_status = READING_SENSITIVE_NETS;
			}
		      else if(sscanf(text_line,"obstacles %d",&number_of_items)==1)
			{
			    sprintf(RRwarning,"%s, line %d - ",text_file_name,linenum);
			    sprintf(RRwarning,"%s Obstacles are ignored by Roadrunner.\n",
				    RRwarning);
			    rrPrintWarning(W_OBSTACLES_IGNORED);
			    readin_status = READING_OBSTACLES;
			}
		      else if(sscanf(text_line,"matched_pairs %d",&number_of_items)==1)
			{
			    readin_status = READING_MATCHED_PAIRS;
			}
		      else if(sscanf(text_line,"critical_pairs %d",&number_of_items)==1) {
			  readin_status = READING_CRITICAL_PAIRS;
		      } else if(sscanf(text_line,"read %s",another_file_name)==1)	{
			  CHECK_RETURN(layout_read(another_file_name,layout));
		      } else {
			  PRINT_LINE_DATA;
			  printf("** %s **: unrecognized command.\n", text_line);
			  return ERROR;
		      }
		      break;
		  }
		case(READING_CORNERS):
		  {
		      if(sscanf(text_line,"%g %g",&xpos,&ypos)==2)
			{
			    CHECK_RETURN(rr_dsSetCorner(xpos,ypos,channel));
			    DECREMENT_NUMBER_OF_ITEMS;
			}
		      else
			{
			    PRINT_LINE_DATA;
			    printf("Expecting more corner data\n");
			    return ERROR;
			}
		      break;
		  }
		case(READING_PINS):
		  {
		      if(sscanf(text_line,"%s %g %g",net_name,&xpos,&ypos)==3) {
			  CHECK_RETURN(rr_dsRecordPin(net_name,xpos,ypos,channel));
			  DECREMENT_NUMBER_OF_ITEMS;
		      } else {
			  PRINT_LINE_DATA;
			  printf("Expecting more pin data\n");
			  return ERROR;
		      }
		      break;
		  }
		case(READING_LEFT_CROSSINGS):
			    {
			    net_list = text_line;
			    while(sscanf(net_list,"%s",net_name) == 1)
				{
			        CHECK_RETURN(rr_dsRecordCrossing(net_name,LEFT,channel));
			        DECREMENT_NUMBER_OF_ITEMS;
				while(net_list[0]==' ')net_list++;
				net_list = net_list + strlen(net_name);
				}
			    break;
			    }
		case(READING_RIGHT_CROSSINGS):
			    {
			    net_list = text_line;
			    while(sscanf(net_list,"%s",net_name) == 1)
				{
			        CHECK_RETURN(rr_dsRecordCrossing(net_name,RIGHT,channel));
			        DECREMENT_NUMBER_OF_ITEMS;
				while(net_list[0]==' ')net_list++;
				net_list = net_list + strlen(net_name);
				}
			    break;
			    }
		case(READING_BOTTOM_CROSSINGS):
			    {
			    net_list = text_line;
			    while(sscanf(net_list,"%s",net_name) == 1)
				{
			        CHECK_RETURN(rr_dsRecordCrossing(net_name,BOTTOM,channel));
			        DECREMENT_NUMBER_OF_ITEMS;
				while(net_list[0]==' ')net_list++;
				net_list = net_list + strlen(net_name);
				}
			    break;
			    }
		case(READING_TOP_CROSSINGS):
			    {
			    net_list = text_line;
			    while(sscanf(net_list,"%s",net_name) == 1)
				{
			        CHECK_RETURN(rr_dsRecordCrossing(net_name,TOP,channel));
			        DECREMENT_NUMBER_OF_ITEMS;
				while(net_list[0]==' ')net_list++;
				net_list = net_list + strlen(net_name);
				}
			    break;
			    }
		case(READING_SENSITIVE_NETS):
			    {
				/* ignored */
			    net_list = text_line;
			    while(sscanf(net_list,"%s",net_name) == 1)
				{
				/*
			        CHECK_RETURN(rr_dsRecordSensitiveNet(net_name,channel));
				*/
			        DECREMENT_NUMBER_OF_ITEMS;
				while(net_list[0]==' ')net_list++;
				net_list = net_list + strlen(net_name) + 1;
				}
			    break;
			    }
		case(READING_OBSTACLES):
			  {
			    /* Obstacles are ignored */
			    DECREMENT_NUMBER_OF_ITEMS;
			    break;
			    }
		case(READING_MATCHED_PAIRS):
			    {
				/* ignored */
			    if(sscanf(text_line,"%s %s",net_name,another_net_name)==2)
				{
				/*
				CHECK_RETURN(rr_dsRecordMatchedPair(net_name,another_net_name,
				   channel));
				*/
				DECREMENT_NUMBER_OF_ITEMS;
				}
			    else
				{
				PRINT_LINE_DATA;
				printf("Expecting more matched pairs.\n");
				return ERROR;
				}
			    break;
			    }
		case(READING_CRITICAL_PAIRS):
		  {
		      /* ignored */
		      if(sscanf(text_line,"%s %s %g",net_name,another_net_name,&capacitance)==3) {
			  /*
			    CHECK_RETURN(rr_dsRecordCriticalPair(net_name,another_net_name,
			    capacitance,channel));
			    */
			  DECREMENT_NUMBER_OF_ITEMS;
		      } else {
			  PRINT_LINE_DATA;
			  printf("Expecting more critical pairs.\n");
			  return ERROR;
		      }
		      break;
		  }
	      default:
		{
		    PRINT_LINE_DATA;
		    printf("Program error. Nothing expected - Why not?\n");
		    return ERROR;
		}
	    } /* End of switch */
	} /* End of text line processor */
	prev_linenum = linenum;
    }

    return OK;
    }

