/*    debug.c
*
*
*   Author: Umakanta Choudhury, Enrico Malavasi
*   Last update: July 1992
*/


#include <stdio.h>
#include "layout.h"
#include "circuit.h"
/* #include "cap_library.h" */

extern struct Sens_link *find_link();

void
  layout_pr(layout,debug_file)
struct Layout *layout;	/* the layout structure */
FILE *debug_file;
/*    PRINT LAYOUT INFORMATION  */
{
    struct ChannelLink	*chan_link;
    struct Channel	*channel;
    struct NetLink	*net_link;
    struct Net		*net;
    struct PinLink	*pin_link;
    struct Pin		*pin;
    int	i;


    fprintf(debug_file,"\nlayout_pr.c\n");
    
    for(chan_link = layout->chan_list; chan_link != NULL; chan_link = chan_link->next) { 
	
	channel = chan_link->channel;
	
	fprintf(debug_file,"chan_name  %s :\n",channel->name);
	if(channel->orient == TOP_BOTTOM) 
	  fprintf(debug_file,"orientation TOP_BOTTOM \n");
	else if(channel->orient == LEFT_RIGHT) 
	  fprintf(debug_file,"orientation LEFT_RIGHT \n");
	fprintf(debug_file,"chan_left = %g :\n",channel->left);
	fprintf(debug_file,"chan_right = %g :\n",channel->right);
	fprintf(debug_file,"chan_bottom = %g :\n",channel->bottom);
	fprintf(debug_file,"chan_top = %g :\n",channel->top);
	
	for(net_link = channel->net_list; net_link != NULL; net_link = 
	    net_link->next){
	    net = net_link->net;
	    fprintf(debug_file,"net_name  %s :\n",net->name);
	    fprintf(debug_file,"pin_info:\n");
	    for(pin_link = net->pins; pin_link != NULL; pin_link = 
		pin_link->next){
		pin = pin_link->pin;
		fprintf(debug_file,"x_position=%g y_position = %g ",
			pin->x_position,pin->y_position);
		if(pin->edge == TOP) 
		  fprintf(debug_file,"on TOP edge\n");
		else if(pin->edge == BOTTOM) 
		  fprintf(debug_file,"on BOTTOM edge\n");
		else if(pin->edge == LEFT) 
		  fprintf(debug_file,"on LEFT edge\n");
		else if(pin->edge == RIGHT) 
		  fprintf(debug_file,"on RIGHT edge\n");
	    }
	}
    }
}

void
 perf_pr(circuit, layout, debug_file)
struct Circuit *circuit;	/* the circuit structure */
struct Layout  *layout;	/* the layout structure */
FILE           *debug_file;
/*  Print circuit and performance info  */
{
    struct Matched_pair	*matchpair;
    struct Performance *performance;
    struct Node *node;
    int	i;

    fprintf(debug_file,"\nperf_pr.c\n");

    fprintf(debug_file,"num_nodes = %d\n",circuit->num_nodes);
    for(node = circuit->node_list; node != NULL; node = node->next){ 
	fprintf(debug_file,"node name = %s\n",node->name);
	if(node->matched == YES){
	    fprintf(debug_file,"matched\n");
	}
	else{
	    fprintf(debug_file,"not matched\n");
	}
    }
    for(matchpair = circuit->matched_pair_list; matchpair != NULL; 
	matchpair = matchpair->next){ 
	fprintf(debug_file,"match pair (%s,%s) \n",
		matchpair->nodeA->name,matchpair->nodeB->name);
    }
    for(performance = circuit->perf_list; performance != NULL; 
	performance = performance->next){ 
	fprintf(debug_file,"performance name = %s\n",performance->name);
	if(performance->pos_constraint == YES){
	    fprintf(debug_file,"max_pos_change = %g\n",performance->
		    max_pos_change);
	}
	if(performance->neg_constraint == YES){
	    fprintf(debug_file,"max_neg_change = %g\n",performance->
		    max_neg_change);
	}
    }
}

void
  sens_pr(circuit,layout,debug_file)
struct Circuit *circuit;	/* the circuit structure */
struct Layout *layout;	/* the layout structure */
FILE *debug_file;
/*   PRINT   SENSITIVITY INFORMATION  */
{
	struct Performance *perf;
	struct Sens_link *sens_link;
	struct Chan_info *chan_info;

	fprintf(debug_file,"\nsens_pr.c\n");

	fprintf(debug_file,"num_nodes = %d\n",circuit->num_nodes);
	for(perf = circuit->perf_list; perf != NULL; 
				perf = perf->next){ 
		fprintf(debug_file,"performance name = %s\n",perf->name);
		fprintf(debug_file,"node1	node2	sensitivity\n");
		for(sens_link = perf->pos_sens_list; sens_link != NULL;
			sens_link = sens_link->next){
			fprintf(debug_file,"%s		%s		%g\n",
				sens_link->nodeA->name,sens_link->nodeB->name,
				sens_link->sens);
		}
		for(sens_link = perf->neg_sens_list; sens_link != NULL;
			sens_link = sens_link->next){
			fprintf(debug_file,"%s 		%s 		%g\n",
				sens_link->nodeA->name,sens_link->nodeB->name,
				sens_link->sens);
		}
	}
}

void
  sens_match_pr(circuit,layout,debug_file)
struct Circuit *circuit;	/* the circuit structure */
struct Layout *layout;	/* the layout structure */
FILE *debug_file;
/*   Print matched sensitivity info  */
{
	struct Performance *perf;
	struct Sens_link *sens_link;

	fprintf(debug_file,"\nsens_match_pr.c\n");

	fprintf(debug_file,"num_nodes = %d\n",circuit->num_nodes);
	for(perf = circuit->perf_list; perf != NULL; 
				perf = perf->next){ 
		fprintf(debug_file,"performance name = %s\n",perf->name);
		fprintf(debug_file,"node1	node2	sensitivity\n");
		for(sens_link = perf->pos_sens_list; sens_link != NULL;
			sens_link = sens_link->next){
			fprintf(debug_file,"%s		%s		%g\n",
				sens_link->nodeA->name,sens_link->nodeB->name,
				sens_link->sens);
		}
		for(sens_link = perf->neg_sens_list; sens_link != NULL;
			sens_link = sens_link->next){
			fprintf(debug_file,"%s 		%s 		%g\n",
				sens_link->nodeA->name,sens_link->nodeB->name,
				sens_link->sens);
		}
	}
}

void
  crit_pr(circuit,layout,debug_file)
struct Circuit *circuit;	/* the circuit structure */
struct Layout *layout;	/* the layout structure */
FILE *debug_file;
/*  Print critical pair info  */
{
    struct CriticalPairLink	*critpair_link;
    struct Chan_info	*chan_info;
    int	i;

    fprintf(debug_file,"\ncrit_pr.c\n");
    
    for(critpair_link = layout->critical_pairs; critpair_link != NULL; 
	critpair_link = critpair_link->next){ 
	fprintf(debug_file,"\ncrit_pair (%s,%s) present in:\n",
		critpair_link->netA_name,critpair_link->netB_name);
	for(chan_info = critpair_link->chan_info_list; 
	    chan_info != NULL; chan_info = chan_info->next){
	    fprintf(debug_file,"channel %s\n",chan_info->channel->name);
	    fprintf(debug_file,"netA %s netB %s\n",chan_info->netA->name,
		    chan_info->netB->name);
	    fprintf(debug_file,"Lx_max = %g, Ly_max = %g\n",
		    chan_info->Lx_max,chan_info->Ly_max);
	}
	for(i=1; i<=circuit->num_perf; i++){
	    fprintf(debug_file,"%s pos_sens = %g neg_sens = %g\n",
		    *(layout->perf_name + i),*(critpair_link->pos_sens + i),
		    *(critpair_link->neg_sens + i));
	}
    }
    fprintf(debug_file,"cap_threshold = %g\n",layout->cap_threshold);
}


void
  maxmin_pr(layout,debug_file)
struct Layout *layout;	/* the layout structure */
FILE *debug_file;
/*  Print maxmin cap info  */
{
    struct CriticalPairLink	*critpair;
    struct Chan_info	*chan_info;
    struct Channel	*channel;

	fprintf(debug_file,"\nmaxmin_pr.c\n");

	for(critpair = layout->critical_pairs; critpair != NULL; 
				critpair = critpair->next){ 
		fprintf(debug_file,"\ncritical pair (%s,%s) :\n",
			critpair->netA_name,critpair->netB_name);
		for(chan_info = critpair->chan_info_list; chan_info != NULL;
			chan_info = chan_info->next){

			channel = chan_info->channel;
			fprintf(debug_file,"\nchannel %s :\n",channel->name);
			fprintf(debug_file,"Cx_max %g \n",chan_info->Cx_max);
			fprintf(debug_file,"Cx_min %g \n",chan_info->Cx_min);
			fprintf(debug_file,"Cy_max %g \n",chan_info->Cy_max);
			fprintf(debug_file,"Cy_min %g \n",chan_info->Cy_min);
			fprintf(debug_file,"Ccross_max %g \n",chan_info->Ccross_max);
			fprintf(debug_file,"Ccross_min %g \n",chan_info->Ccross_min);
			fprintf(debug_file,"C_max %g \n",chan_info->C_max);
			fprintf(debug_file,"C_min %g \n",chan_info->C_min);
		}
	}
}

void
  delta_min_pr(layout,circuit,debug_file)
struct Layout *layout;
struct Circuit *circuit;
FILE *debug_file;
{
	struct Performance *perf;


	fprintf(debug_file,"\ndelta_min_pr.c\n");

	fprintf(debug_file,"before shielding:\n");
	for(perf = circuit->perf_list; perf != NULL; perf = perf->next){
		fprintf(debug_file,"performance %s :\n",perf->name);
		if(perf->pos_constraint == YES){
		/* constraint on maximum positive change present */
			fprintf(debug_file,"max_pos_change = %f,delta_pos_min = %f\n"
				,perf->max_pos_change,perf->delta_pos_min);
		}
		if(perf->neg_constraint == YES){
		/* constraint on maximum negative change present */
			fprintf(debug_file,"max_neg_change = %f,delta_neg_min = %f\n"
				,perf->max_neg_change,perf->delta_neg_min);
		}
	}
}

void
  bound_pr(layout,circuit,debug_file)
struct Layout *layout;
struct Circuit *circuit;
FILE *debug_file;
{
    struct Performance *perf;
    struct CriticalPairLink *critpair;
    struct Chan_info *chan_info;
    float pos_change[50];
    float neg_change[50];
    struct Sens_link *sens_link, *find_stored_link();
    float sens;
    int i;

    FILE *maxmin_temp;
    FILE *router_inp;
    char filename[50];
    
    /* Open the text file */
    sprintf(filename,"%s_%d","/tmp/parcar.maxmin_temp",getpid());
    if( (maxmin_temp = fopen(filename,"w")) == NULL) {
        printf("\nE: could not open file %s",filename);
	exit( -1 );
    }
    sprintf(filename,"%s_%d","/tmp/parcar.router_inp",getpid());
    if( (router_inp = fopen(filename,"w")) == NULL) {
        printf("\nE: could not open file %s",filename);
	exit( -1 );
    }

    DEBUGG( fprintf(debug_file,"\nbound_pr.c\n") );
    DEBUGG( fprintf(debug_file,"cap_threshold = %g\n\n",layout->cap_threshold) );
    for(perf = circuit->perf_list; perf != NULL; perf = perf->next){
	if(perf->pos_constraint == YES){
	    /* constraint on maximum positive change present */
	    DEBUGG( fprintf(debug_file,"perf %s pos_eqn_num = %d\n",perf->name, perf->pos_eqn_num) );
	}
	if(perf->neg_constraint == YES){
	    /* constraint on maximum negative change present */
	    DEBUGG( fprintf(debug_file,"perf %s neg_eqn_num = %d\n",perf->name, perf->neg_eqn_num) );
	}
	pos_change[perf->number] = 0; 
	neg_change[perf->number] = 0; 
    }
    fprintf(maxmin_temp, "number of nets %d\n", circuit->num_nodes);
    fprintf(maxmin_temp, "number of net pairs %d\n", layout->num_bound_constraints);
    
    for(critpair = layout->critical_pairs; critpair != NULL;
	critpair = critpair->next){
	for(chan_info = critpair->chan_info_list; chan_info != NULL;
	    chan_info = chan_info->next){
	    if(chan_info->C_max > layout->cap_threshold){
		DEBUGG( fprintf(debug_file,"\npair (%s,%s) chan %s param_num = %d\n",
				critpair->netA_name,critpair->netB_name,
				chan_info->channel->name,chan_info->param_num));
		DEBUGG( fprintf(debug_file,"C_bound[%d] = %g, max = %g, min = %g, shield_min = %g \n",
				chan_info->param_num,chan_info->C_bound,
				chan_info->C_max,chan_info->C_min,
				chan_info->C_min_shield));
		fprintf(maxmin_temp,"%s\t%s\t%.3e\t%.3e\n",
			critpair->netA_name,critpair->netB_name,
			chan_info->C_max,chan_info->C_min_shield);
		
		for(perf = circuit->perf_list; perf != NULL; 
		    perf = perf->next){ 
		    /* find the stored sensitivity link for C(name1A,name2A) */
		    if((sens_link=find_stored_link(perf, critpair->netA_name, critpair->netB_name)) == NULL) {
			sens = 0;
		    } else {
			sens = sens_link->sens;
		    }
		    if(sens >= 0){
			pos_change[perf->number] += chan_info->C_bound * sens;
		    } else {
			neg_change[perf->number] += chan_info->C_bound * sens;
		    }
		    DEBUGG( fprintf(debug_file,"%s sens = %g\n",perf->name,sens) );
		}
	    }
	}
    }
    DEBUGG( fprintf(debug_file,"\nprediction of linear analysis\n") );
    for(i=0; i<circuit->num_perf; i++) {
	DEBUGG( fprintf(debug_file,"%s pos_change=%g neg_change=%g tot_change=%g\n",
			layout->perf_name[i], pos_change[i], neg_change[i],
			pos_change[i]+neg_change[i]));
    }
    DEBUGG( fprintf(debug_file,"\ninput for checking linear analysis \n") );
    fprintf(router_inp,"channel\tnet1\tnet2\tCbound(fF)\n\n");
    for(critpair = layout->critical_pairs; critpair != NULL;
	critpair = critpair->next){
	for(chan_info = critpair->chan_info_list; chan_info != NULL;
	    chan_info = chan_info->next){
	    if(chan_info->C_max > layout->cap_threshold){
		DEBUGG( fprintf(debug_file,"cap%s%s_%s %s %s %gfF\n",
				critpair->netA_name,critpair->netB_name,
				chan_info->channel->name,
				critpair->netA_name,critpair->netB_name,
				chan_info->C_bound));
		fprintf(router_inp,"%s\t\t%s\t\t%s\t\t%.2e\n",
			chan_info->channel->name,
			critpair->netA_name,critpair->netB_name,
			chan_info->C_bound);
	    }
	}
    }
    fclose(maxmin_temp);
    fclose(router_inp);
}

void
  constraint_pr(layout, circuit, out_file)
struct Layout *layout;
struct Circuit *circuit;
FILE *out_file;
{
    FILE *constraint_file;
    struct Performance *perf;
    struct CriticalPairLink *critpair, *c2;
    struct Chan_info *chan_info;
    struct Node *nodeA, *nodeB;
    float pos_change[50];
    float neg_change[50];
    struct Sens_link *sens_link;
    float sens;
    char filename[128];
    char *nameA, *nameB;
    char *name1, *name2;
    int i;
    int imax;

    sprintf(filename, "%s_%d","/tmp/parcar.constraints", getpid()); 
    constraint_file  = fopen(filename, "w");

    fprintf(constraint_file, "number of performance functions %d \n", circuit->num_perf);
    fprintf(out_file, "%d\t%d\n", layout->num_bound_constraints, circuit->num_perf);
    for(perf=circuit->perf_list; perf!=NULL; perf=perf->next) {
	if(perf->pos_constraint == YES) {
	    /* constraint on maximum positive change present */
	    fprintf(constraint_file,"%s\tdelta+ = %.3e",perf->name, perf->max_pos_change);
	    fprintf(out_file,"%.3e\t",perf->max_pos_change);
	} else {
	    /* -1 indicates no constraint */
	    fprintf(constraint_file,"%s\tdelta+ = -1",perf->name);
	    fprintf(out_file,"-1\t",perf->name);
	}
	if(perf->neg_constraint == YES){
	    /* constraint on maximum negative change present */
	    fprintf(constraint_file,"\tdelta- = %.3e\n", (-1)*perf->max_neg_change);
	    fprintf(out_file,"%.3e\n",(-1)*perf->max_neg_change);
	}
	else{
	    /* -1 indicates no constraint */
	    fprintf(constraint_file,"\tdelta- = -1\n");
	    fprintf(out_file,"-1\n");
	}
    }
    fprintf(constraint_file,"number of bounding constraints %d \n",
		layout->num_bound_constraints);
    

    /*  Sort all critical pairs */
    for(critpair=layout->critical_pairs ; critpair!=NULL ; critpair=critpair->next) {
	if(strcmp(critpair->netA_name, critpair->netB_name) > 0) {
	    SWAP( char *, critpair->netA_name, critpair->netB_name);
	}
    }

    for(critpair = layout->critical_pairs ; critpair != NULL ; critpair = critpair->next) {
	nameA = critpair->netA_name;
	nameB = critpair->netB_name;
	if(avl_lookup(circuit->node_tree,nameA,&nodeA) == NULL){
	    printf("node %s not found in the circuit\n",nameA);
	}
	if(avl_lookup(circuit->node_tree,nameB,&nodeB) == NULL){
	    printf("node %s not found in the circuit\n",nameB);
	}
	imax = 1;
	if((nodeA->matched) || (nodeB->matched)) imax = 2;
	if((nodeA->matched) && (nodeB->matched)) imax = 1;
	/* need to print the information for the matched pair too */
	
	for(chan_info = critpair->chan_info_list; chan_info != NULL;
	    chan_info = chan_info->next){
	    
	    if(chan_info->C_max > layout->cap_threshold){
		for(i=1; i<= imax; i++){
		    if(i==1){
			name1 = nodeA->name;
			name2 = nodeB->name;
		    }
		    else if(i==2){
			/* get the node names for the matched pair */
			if(nodeA->matched) 
			  name1 = nodeA->matched_node->name;
			if(nodeB->matched) 
			  name2 = nodeB->matched_node->name;
		    }
		    else{
			printf("constraint_pr: cannot occur\n");
			exit( -1 );
		    }
		    fprintf(constraint_file,"%s\t%s\tC_bound = %.3e", name1,name2,chan_info->C_bound);
		    fprintf(out_file,"%s\t%s\t%.3e",name1,name2, chan_info->C_bound);
		    
		    for(perf = circuit->perf_list; perf != NULL; perf = perf->next) {
			/* find the positive sensitivity link for 
			   C(nameA,nameB) */
                        sens = *(critpair->pos_sens + perf->number);
			fprintf(constraint_file,"\tS+ = %.3e",sens);
			fprintf(out_file,"\t%.3e",sens);
                        sens = *(critpair->neg_sens + perf->number);
			fprintf(constraint_file,"\tS- = %.3e", (-1)*sens);
			fprintf(out_file,"\t%.3e",(-1)*sens);
		    }
		    fprintf(constraint_file,"\n");
		    fprintf(out_file,"\n");
		}
	    }
	}
    }
    fclose( constraint_file );
}

void
  link_chan_pr(layout,debug_file)
struct Layout *layout;	/* the layout structure */
FILE *debug_file;
{
    struct ChannelLink	*chan_link;
    struct Channel	*channel;
    struct ChannelLink	*top_chan;
    struct ChannelLink	*bottom_chan;

	fprintf(debug_file,"\nlink_chan_pr.c\n");


	for(chan_link = layout->chan_list; chan_link != NULL; 
				chan_link = chan_link->next){
		channel = chan_link->channel;
		fprintf(debug_file,"channel  %s :\n",channel->name);
		if(channel->left_chan != NULL){
			fprintf(debug_file,"\t left_chan = %s \n",
				channel->left_chan->name);
		}
		if(channel->right_chan != NULL){
			fprintf(debug_file,"\t right_chan = %s \n",
				channel->right_chan->name);
		}
		for(top_chan = channel->top_chan_list; top_chan != NULL; 
			top_chan = top_chan->next){
			fprintf(debug_file,"\t top_chan = %s \n",
				top_chan->channel->name);
		}
		for(bottom_chan = channel->bottom_chan_list; 
			bottom_chan != NULL; bottom_chan = bottom_chan->next){
			fprintf(debug_file,"\t bottom_chan = %s \n",
				bottom_chan->channel->name);
		}
	}
}

void
  sublist_pr(layout,debug_file)
struct Layout *layout;	/* the layout structure */
FILE *debug_file;
{
    struct ChannelLink	*chan_link;
    struct Channel	*channel;
    struct Pin_list_link	*pin_list_link;
    struct PinLink	*pin_link;

	fprintf(debug_file,"\nsublist_pr.c\n");


	for(chan_link = layout->chan_list; chan_link != NULL; 
				chan_link = chan_link->next){

		channel = chan_link->channel;
		fprintf(debug_file,"channel  %s :\n",channel->name);

		for(pin_list_link = channel->top_pin_lists; 
			pin_list_link != NULL; pin_list_link = pin_list_link->next){
			if(pin_list_link->channel != NULL)
				fprintf(debug_file,"\t sublist after top channel %s \n",
				pin_list_link->channel->name);
			else fprintf(debug_file,"\t first sublist on top \n");
			for(pin_link = pin_list_link->pin_list; 
				pin_link != NULL; pin_link = pin_link->next){
				fprintf(debug_file,"\t (%g,%g) \n",
					pin_link->pin->x_position,
					pin_link->pin->y_position);
			}
		}

		for(pin_list_link = channel->bottom_pin_lists; 
			pin_list_link != NULL; pin_list_link = pin_list_link->next){
			if(pin_list_link->channel != NULL)
				fprintf(debug_file,"\t sublist after bottom channel %s\
				\n",pin_list_link->channel->name);
			else fprintf(debug_file,"\t first sublist on bottom \n");
			for(pin_link = pin_list_link->pin_list; 
				pin_link != NULL; pin_link = pin_link->next){
				fprintf(debug_file,"\t (%g,%g) \n",
					pin_link->pin->x_position,
					pin_link->pin->y_position);
			}
		}
		if(channel->last_top_chan_link != NULL){
			fprintf(debug_file,"\t last_top_chan %s\n",
				channel->last_top_chan_link->channel->name);
		}
		if(channel->last_bottom_chan_link != NULL){
			fprintf(debug_file,"\t last_bottom_chan %s\n",
				channel->last_bottom_chan_link->channel->name);
		}
	}
}

void
  span_pr(layout,debug_file)
struct Layout *layout;	/* the layout structure */
FILE *debug_file;

{
    struct ChannelLink	*chan_link;
    struct Channel	*channel;
    struct NetLink	*net_link;
    struct Net	*net;

    fprintf(debug_file,"\nspan_pr.c\n");

    for(chan_link = layout->chan_list; chan_link != NULL; 
	chan_link = chan_link->next){ 
	
	channel = chan_link->channel;
	fprintf(debug_file,"\nchannel %s:\n",channel->name);
	
	for(net_link = channel->net_list; net_link != NULL; 
	    net_link = net_link->next){ 
	    
	    net = net_link->net;
	    fprintf(debug_file,"net %s :\n",net->name);
	    fprintf(debug_file,"max_left = %g, max_right = %g :\n"
		    ,net->max_span_left,net->max_span_right);
	    fprintf(debug_file,"min_left = %g, min_right = %g :\n"
		    ,net->min_span_left,net->min_span_right);
	}
    }
}

void
  allpair_pr(layout, debug_file)
struct Layout *layout;	/* the layout structure */
FILE *debug_file;

{
    struct Chan_info	*chan_info;
    struct Channel	*channel;
	struct Chan_info_list_link *chan_info_list_link;

	fprintf(debug_file,"\nallpair_pr.c\n");


	for(chan_info_list_link = layout->chan_info_list_link;
		chan_info_list_link != NULL;
		chan_info_list_link = chan_info_list_link->next){

		chan_info = chan_info_list_link->chan_info_list;
		if(chan_info != NULL)
			fprintf(debug_file,"\npair (%s,%s) :\n",
				chan_info->netA->name,chan_info->netB->name);
		for(chan_info = chan_info_list_link->chan_info_list; 
			chan_info != NULL; chan_info = chan_info->next){

			channel = chan_info->channel;
			fprintf(debug_file,"\nchannel %s :\n",channel->name);
			fprintf(debug_file,"Cx_max %g \n",chan_info->Cx_max);
			fprintf(debug_file,"Cx_min %g \n",chan_info->Cx_min);
			fprintf(debug_file,"Cy_max %g \n",chan_info->Cy_max);
			fprintf(debug_file,"Cy_min %g \n",chan_info->Cy_min);
			fprintf(debug_file,"Ccross_max %g \n",chan_info->Ccross_max);
			fprintf(debug_file,"Ccross_min %g \n",chan_info->Ccross_min);
			fprintf(debug_file,"C_max %g \n",chan_info->C_max);
			fprintf(debug_file,"C_min %g \n",chan_info->C_min);
		}
	}
}

void
  shield_pr(layout,circuit,debug_file)
struct Layout *layout;
struct Circuit *circuit;
FILE *debug_file;
{
    struct Performance *perf;
    struct CriticalPairLink *critpair;
    struct Chan_info *chan_info;
    

    fprintf(debug_file,"\nshield_pr.c\n");
    
    fprintf(debug_file,"after shielding:\n");
    for(perf = circuit->perf_list; perf != NULL; perf = perf->next){
	fprintf(debug_file,"performance %s :\n",perf->name);
	if(perf->pos_constraint == YES){
	    /* constraint on maximum positive change present */
	    fprintf(debug_file,"max_pos_change = %f,delta_pos_min = %f\n"
		    ,perf->max_pos_change,perf->delta_pos_min);
	}
	if(perf->neg_constraint == YES){
	    /* constraint on maximum negative change present */
	    fprintf(debug_file,"max_neg_change = %f,delta_neg_min = %f\n"
		    ,perf->max_neg_change,perf->delta_neg_min);
	}
    }
    fprintf(debug_file,"shielding information:\n");
    for(critpair = layout->critical_pairs; critpair != NULL;
	critpair = critpair->next){
	for(chan_info = critpair->chan_info_list; chan_info 
	    != NULL; chan_info = chan_info->next){
	    if(chan_info->shielded == YES){
		fprintf(debug_file,"crit pair (%s,%s) chan %s\n",
			critpair->netA_name,critpair->netB_name,chan_info->
			channel->name);
	    }
	}
    }
}
