/*   		WARNINGS 
*		--------
*   
*   Functions: 	rrPrintWarning
*   File:	warnings.c
*   Author:	Nicholas Weiner
*   Date:	18th March 1988
*
*   The routine in this files prints out warnings, and keeps track
*   of the number of each type of warning given.
*   No warnings are given if RRquiet is set. If RRquiet is
*   not set then warnings are printed to standard output, up to a limit
*   for each type. If RRdebug is set, then warnings are printed
*   in unlimited numbers.
*
*   To add a new warning type:
*   a) Edit header file -
*       i) Increment the value NUMBER_OF_WARNING_TYPES;
*      ii) Define a new warning type with number one larger than previous 
*          largest number;
*   b) Edit this file -
*     iii) Add lines to initialisation routine to give limit and 
*          descriptive string for new warning type.
*/

#include <stdio.h>
#include "layout.h"

/* DECLARATION OF GLOBAL CHARACTER ARRAY */
char RRwarning[200];
int RRquiet;
int RRdebug;

/* ARRAY OF WARNING DATA - global to this file */
struct WarningData {
    int allocation;
    char *description; };

static struct WarningData Warnings[NUMBER_OF_WARNING_TYPES];


/*
*  WARNINGS
*
*  All warnings are passed by use of the GLOBAL CHARACTER ARRAY - RRwarning.
*
*  The static array WarningAllocations is initialised with the maximum
*  number of warning required of each type for the normal mode.
*  When ever a warning is given an array member is decremented. When an
*  array reaches zero a warningf suppression message is given and all
*  further calls for a warning of the specific type are ignored.
*
*  To change the maximum number of warnings of any type, just change the
*  initialisation part of this routine.
*  
* 
*  All warnings out-of-range types are printed.
*
* Returns WARNING if warning is given, OK if it is not.
*/

REPORT
  rrPrintWarning(type)
int type;			/* One of those defined in the header file */

{
    static int WarningsInitialised = FALSE;

    /* Initialise the warning allocations during the first call to this routine */
    if(WarningsInitialised == FALSE) 
      {
	  rrInitWarnings();
	  WarningsInitialised = TRUE;
      }
    
    /* Nothing to do if in quiet mode */
    if(RRquiet == TRUE) return OK;
    
    if(  (type >= NUMBER_OF_WARNING_TYPES) ||(type < 0)
       ||(RRdebug) )
      {
	  /* Print warning with no questions asked */
	  printf("W: %s\n",RRwarning);
	  return WARNING;
      }
    
    /* Any more of this warning type required? */
    if(Warnings[type].allocation > 0)
      {
	  printf("W: %s\n",RRwarning);
	  Warnings[type].allocation--;
	  return WARNING;
      }
    else if(Warnings[type].allocation == 0)
      {
	  printf(
		 "Further \"%s\" warnings being suppressed.\n\n",Warnings[type].description);
	  Warnings[type].allocation--;
	  return OK;
      }
    
    return OK;
}
    

/*
*   INITIALISE THE WARNING DATA ARRAY
*
*  All allocations are initialised to 10. Do not need to specifiy allocation
*  again if this number is acceptible.
*
*/
rrInitWarnings()
{
    char *default_description = "???";
    int  default_allocation = 10;
    int  type;
    
    /* Play safe - set up all types */
    for(type=0;type<NUMBER_OF_WARNING_TYPES;type++)
      {
	  Warnings[type].allocation = default_allocation;
	  Warnings[type].description = default_description;
      }
    
    /* Now work through type definitions */ 
    Warnings[W_OBSTACLES_IGNORED].description = "obstacles ignored";
    Warnings[W_NUMBER_OF_CORNERS].description = "number of corners";
    Warnings[W_NOT_ENOUGH_CONNECTIONS].description = "not enough connections";
    Warnings[W_PINS_INTERSECT].description = "intersecting pins";
}
