/*      ROUTINE FOR GENERATING BOUNDING CONSTRAINTS 
*		ON THE PARASITICS (CALLS THE QP PACKAGE)
*       ---------------------------------------
*
*   File:       bound_gen.c
*   Author:     Umakanta Choudhury
*   Date:       26th June 1989
*
*
*/

#include <stdio.h>
#include "layout.h"
#include "circuit.h"
#define NOT_PRESENT -1



REPORT bound_gen(layout,circuit,debug_file)
struct Layout *layout;
struct Circuit *circuit;
FILE *debug_file;
{
    struct Performance *perf;
    struct CriticalPairLink *critpair;
    struct Chan_info *chan_info;
    struct CriticalPairLink *critpair1;
    struct Chan_info *chan_info1;
    struct CriticalPairLink *critpair2;
    struct Chan_info *chan_info2;
    struct Node *nodeA, *nodeB;
    double sens_mag;
    double sens;
    int	max_eqn_num = 0;
    int	max_param_num = 0;
    int	nq = 0;
    int	mge = 0;
    int	me = 0;
    int	mle = 0;
    int	mmeq = 0;
    int	info;
    double	*q;
    double	*QP_cq;
    double	*QP_hq;
    double	*QP_bq;
    double	*QP_gq;
    double	*QP_qmult;
    double	*QP_bndu;
    double	*QP_bndl;
    double	*QP_x;
    double	*QP_qw;
    double	*x0;
    int	*QP_iqw;
    int	nqw;
    int	i;
    int	j;
    int	all_zero;
    double C_max;
    double C_min;
    double C_sq;
    double cost;
    char *netA_name, *netB_name;
    
    fprintf(debug_file,"\nbound_gen.c\n");
    for(perf = circuit->perf_list; perf != NULL; perf = perf->next){
	if(perf->pos_constraint == YES){
	    /* constraint on maximum positive change present */
	    
	    /* check that all the nonnegative sensitivities 
	       are not zero */
	    all_zero = YES;
	    for(critpair = layout->critical_pairs; critpair != NULL;
		critpair = critpair->next){
		sens = *(critpair->pos_sens + perf->number); 
		if(sens != 0) {
		    all_zero = NO;
		    break;
		}
	    }
	    if(all_zero == NO){
				perf->pos_eqn_num = max_eqn_num;
				max_eqn_num ++;
			    }
	    else{
		perf->pos_eqn_num = NOT_PRESENT;
	    }
	}
	if(perf->neg_constraint == YES){
	    /* constraint on maximum negative change present */
	    
	    /* check that all the nonpositive sensitivities 
	       are not zero */
	    for(critpair = layout->critical_pairs; critpair != NULL;
		critpair = critpair->next){
		sens = *(critpair->neg_sens + perf->number); 
		all_zero = YES;
		if(sens != 0) {
		    all_zero = NO;
		    break;
		}
	    }
	    if(all_zero == NO){
		perf->neg_eqn_num = max_eqn_num;
		max_eqn_num ++;
	    }
	    else{
		perf->neg_eqn_num = NOT_PRESENT;
	    }
	}
    }
    for(critpair = layout->critical_pairs; critpair != NULL;
	critpair = critpair->next){
	for(chan_info = critpair->chan_info_list; chan_info != NULL;
	    chan_info = chan_info->next){
	    if(chan_info->C_max > layout->cap_threshold){
		chan_info->param_num = max_param_num;
		max_param_num ++;
	    }
	}
    }


    mle = max_eqn_num;
    mmeq = mle;
    nq = max_param_num;
    nqw = nq*(mle+3*nq+21)+15*mle+160;
    
    QP_cq = (double *) malloc(sizeof(double) * nq);
    QP_x = (double *) malloc(sizeof(double) * nq);
    QP_hq = (double *) malloc(sizeof(double) * nq * nq);
    QP_bndu = (double *) malloc(sizeof(double) * nq);
    QP_bndl = (double *) malloc(sizeof(double) * nq);
    q = (double *) malloc(sizeof(double) * nq);
    
    QP_bq = (double *) malloc(sizeof(double) * mle);
    QP_gq = (double *) malloc(sizeof(double) * mle * nq);
    QP_qw = (double *) malloc(sizeof(double) * nqw);
    QP_iqw = (int *) malloc(sizeof(int) * nqw);
    QP_qmult = (double *) malloc(sizeof(double) * (mle+nq+nq));
    
    x0 = (double *) malloc(sizeof(double) * nq);
    /*
     * Assemble the bndu/bndl and cq vector
     */
    
    for(critpair = layout->critical_pairs; critpair != NULL;
	critpair = critpair->next){
	for(chan_info = critpair->chan_info_list;chan_info != NULL;
	    chan_info = chan_info->next){
	    if(chan_info->C_max > layout->cap_threshold){
		
		i = chan_info->param_num;
		C_min = chan_info->C_min_shield;
		C_max = chan_info->C_max;
		
                /* variable transformed */
                x0[i] = C_min;
		
                C_max = C_max - x0[i];
                C_min = 0;
		
		*(QP_bndl + i) = C_min;
		*(QP_bndu + i) = C_max;
		C_sq = (C_max - C_min) * (C_max - C_min);
		if(C_sq == 0){
		    printf("program error : C_max = C_min cannot happen");
		    return(ERROR);
		}
		q[i] = 2/(C_sq); /* factor of 2 because QP 
				    minimizes 0.5*quadratic term */
		
                /* linear part of the cost */
		*(QP_cq + i) = (-1) * (2/(C_max - C_min) + 2 * C_min/(C_sq));
		
                /* result of transformation */
		
		*(QP_cq + i) -=  2*q[i]*x0[i];
	    }
	}
    }
	
    /*
     * Assemble the hq matrix(quadratic part of the cost : 
     * should be positive definite)
     */
    for(critpair1 = layout->critical_pairs; critpair1 != NULL;
	critpair1 = critpair1->next){
	for(chan_info1 = critpair1->chan_info_list; chan_info1 != NULL;
	    chan_info1 = chan_info1->next){
	    if(chan_info1->C_max > layout->cap_threshold){
		i = chan_info1->param_num;
		*(QP_x+i) = 0;
		
		for(critpair2 = layout->critical_pairs; critpair2 != NULL;
		    critpair2 = critpair2->next){
		    for(chan_info2 = critpair2->chan_info_list; 
			chan_info2 != NULL; chan_info2 = chan_info2->next){
			if(chan_info2->C_max > layout->cap_threshold){
			    j = chan_info2->param_num;
			    
			    if(i == j){
				*(QP_hq + nq * i + j) = q[i];
			    }
			    else{
				*(QP_hq + nq * i + j) = 0;
			    }
			}
		    }
		}
	    }
	}
    }
    
    /*
     * Assemble the bq vector(contains the magnitudes of 
     * maximum changes allowed in performance functions 
     */
    for(perf = circuit->perf_list; perf != NULL; perf = perf->next){
	if(perf->pos_constraint == YES){
	    /* constraint on maximum positive change present */
	    if(perf->pos_eqn_num != NOT_PRESENT){
		i = perf->pos_eqn_num;
		*(QP_bq + i) = perf->max_pos_change;
	    }
	}
	if(perf->neg_constraint == YES){
	    /* constraint on maximum negative change present */
	    if(perf->neg_eqn_num != NOT_PRESENT){
		i = perf->neg_eqn_num;
		*(QP_bq + i) = (-1) * perf->max_neg_change;
	    }
	}
    }
    
    /*
     * Assemble the gq matrix (contains the magnitudes of 
     * sensitivities with respect to parasitics)
     */
    for(critpair = layout->critical_pairs; critpair != NULL;critpair = critpair->next){
	for(chan_info = critpair->chan_info_list; chan_info != NULL;
	    chan_info = chan_info->next){
            if(chan_info->C_max > layout->cap_threshold){
		j = chan_info->param_num;
		for(perf = circuit->perf_list; perf != NULL; 
		    perf = perf->next){
		    if(perf->pos_constraint == YES){
			/* constraint on maximum positive change present */
			if(perf->pos_eqn_num != NOT_PRESENT){
			    i = perf->pos_eqn_num;
			    sens_mag = *(critpair->pos_sens + perf->number); 
			    *(QP_gq + nq * i + j) = sens_mag;
			    
                            /* additional terms due to transformation */
			    *(QP_bq + i) += sens_mag * x0[j];
                            
			}
		    }
		    if(perf->neg_constraint == YES){
			/* constraint on maximum negative change present */
			if(perf->neg_eqn_num != NOT_PRESENT){
			    i = perf->neg_eqn_num;
			    sens_mag = (-1) * *(critpair->neg_sens + perf->number);
			    *(QP_gq + nq * i + j) = sens_mag;
			    
                            /* additional terms due to transformation */
			    *(QP_bq + i) += sens_mag * x0[j];
			}
		    }
		}
	    }
	}
    }

    fprintf(debug_file,"before qpr2\n");
    fprintf(debug_file,"nqw = %d, info = %d, cost = %g\n",nqw,info,cost);
    for(i=0; i<nq; i++){
	fprintf(debug_file,"QP_cq[%d] = %g\n",i,*(QP_cq+i));
	fprintf(debug_file,"QP_x[%d] = %g\n",i,*(QP_x+i));
	fprintf(debug_file,"QP_bndu[%d] = %g\n",i,*(QP_bndu+i));
	fprintf(debug_file,"QP_bndl[%d] = %g\n",i,*(QP_bndl+i));
	fprintf(debug_file,"QP_qmult[%d] = %g\n",i,*(QP_qmult+i));
	for(j=0; j<nq; j++){
	    fprintf(debug_file,"QP_hq[%d][%d] = %g\n",i,j,*(QP_hq+nq*i+j));
	}
    }
    for(i=0; i<mle; i++){
	fprintf(debug_file,"QP_bq[%d] = %g\n",i,*(QP_bq+i));
	for(j=0; j<nq; j++){
	    fprintf(debug_file,"QP_gq[%d][%d] = %g\n",i,j,*(QP_gq+nq*i+j));
	}
    }

    /*
     * Call the QP solver
     */
    qpr2_(&nq, &mge, &mle, &me, &mmeq, QP_cq, QP_hq, QP_bq, QP_gq, &nqw,
	  QP_qw, QP_iqw, QP_x, QP_bndu, QP_bndl, QP_qmult, &cost, &info);

    /*
     * store the bounding constraints
     */
    for(critpair = layout->critical_pairs; critpair != NULL ; critpair = critpair->next){
	for(chan_info = critpair->chan_info_list; chan_info != NULL;
	    chan_info = chan_info->next){
            j = chan_info->param_num;
            if(chan_info->C_max > layout->cap_threshold){
		layout->num_bound_constraints++;
		if(avl_lookup(circuit->node_tree,critpair->netA_name, &nodeA) == NULL){
		    printf("node %s not found in circuit\n",netA_name);
		}
		if(avl_lookup(circuit->node_tree,critpair->netB_name, &nodeB) == NULL) {
		    printf("node %s not found in circuit\n",netB_name);
		}
		
		if(((nodeA->matched)&&!(nodeB->matched)) || (!(nodeA->matched)&&(nodeB->matched))) {
		    layout->num_bound_constraints++;
                }
		
                /* back transformation */
		QP_x[j] +=  x0[j];
		if(QP_x[j] < layout->cap_threshold) {
		    chan_info->C_bound = 0;   /* layout->cap_threshold; */
		} else {
		    chan_info->C_bound = QP_x[j];
		}
		if(chan_info->C_bound < chan_info->C_min) {
		    fprintf(stderr, "*Warning* - Lower bound for the cap. between nodes %s and %s is too tight\n",
			    critpair->netA_name, critpair->netB_name);
		}
		if(chan_info->C_bound > chan_info->C_max) {
		    fprintf(stderr, "*Warning* - Upper bound for the cap. between nodes %s and %s is too tight\n",
			    critpair->netA_name, critpair->netB_name);
		}
	    }
	}
    }
    
    
    fprintf(debug_file,"after qpr2\n");
    fprintf(debug_file,"info = %d cost = %g\n",info,cost);
    for(i=0; i<nq; i++){
	fprintf(debug_file,"x[%d] = %g\n",i,QP_x[i]);
    }
    
    return(OK);
}
