/*		ROUTINE FOR ESTIMATING MAXIMUM AND
*		MINIMUM CAPACITANCES IN X-DIRECTION BETWEEN 
*		TWO NETS(NET1 INSIDE CHANNEL AND NET2 IN THE
*		BOTTOM CHANNEL)
*       -------------------------------------------
*
*   File:       cx_bot_maxmin.c
*   Author:     Umakanta Choudhury
*   Date:       20 July 1989
*
*
*/

#include <stdio.h>
#include "layout.h"

extern int strcmp();
extern double cap_parallel();
extern float chan_net_com();


REPORT
  cx_bot_maxmin(chan_info,bottom_chan,net1,net2)
struct Chan_info *chan_info;
struct Channel *bottom_chan;
struct Net *net1;
struct Net *net2;
{

	struct Channel *channel;
	struct Channel *chan;
	struct ChannelLink *bot_chan_toplink;
	struct ChannelLink *bot_chan_botlink;
	struct Pin *pin;
	struct PinLink *pin_link;
	struct NetLink *net_link;
	struct Net *bottom_chan_net2;
	float influence_reg_bot;
	float W1;
	float W2;
	float d_min;
	float d;
	float chan_intersect;
	float chan_net_sep;
	float cap;
	float Cx_max_bot_toppins; 
	float Cx_max_bot_botpins; 
	int	FOUND_NET2 = 0;

#ifdef DEBUG
	printf("cx_bot_maxmin\n");
#endif 

	channel = chan_info->channel;
	chan_net_sep= channel->chan_net_separation;
	W1 = 2*(net1->true_half_width);
	W2 = 2*(net2->true_half_width);
	d_min = HUGE;

	/* check whether bottom_chan intersects the span of net1 */

	if((bottom_chan->top > net1->max_span_left) && 
		(bottom_chan->bottom < net1->max_span_right)){

		/* calculate the intersection of channel with 
			net1 span */

		chan_intersect = chan_net_com(bottom_chan,net1);

		/* check whether any net2 pin in bottom channel 
		is in the region of influence of net1 */ 

		influence_reg_bot = bottom_chan->right + chan_net_sep
			- bottom_chan->influence_dist;

		chan_info->Cx_max_bottom = 0;
		Cx_max_bot_toppins = 0;/* maximum coupling due to 
								top pins of bot_chan */
		Cx_max_bot_botpins = 0;/* maximum coupling due to 
								bottom pins of bot_chan */

		/* scan through floating pins of net2 on top and
		bottom edge of bot_chan if present in reverse order */
		for(bot_chan_toplink = bottom_chan->last_top_chan_link;
			bot_chan_toplink != NULL; 
			bot_chan_toplink = bot_chan_toplink->prev){
			chan = bot_chan_toplink->channel;
			if(net_present(chan->left_crosses,net2)){
				if((chan->top - chan_net_sep) 
					< influence_reg_bot){
					/* crossed the region of influence */
					break;
				}
				else{
					d = bottom_chan->right - chan->top 
						+ (2 * chan_net_sep);
					d_min = MIN(d,d_min);
					break;
	
				}
			}
		}
		for(bot_chan_botlink = bottom_chan->last_bottom_chan_link;
			bot_chan_botlink != NULL; 
			bot_chan_botlink = bot_chan_botlink->prev){
			chan = bot_chan_botlink->channel;
			if(net_present(chan->right_crosses,net2)){
				if((chan->top - chan_net_sep) 
					< influence_reg_bot){
					/* crossed the region of influence */
					break;
				}
				else{
					d = bottom_chan->right - chan->top 
						+ (2 * chan_net_sep);
					d_min = MIN(d,d_min);
					break;
	
				}
			}
		}

		/* get net2 in bottom_chan if present */
		FOUND_NET2 = 0;
		for(net_link = bottom_chan->net_list; net_link != NULL;
			net_link = net_link->next){
			if(strcmp(net_link->net->name,net2->name) 
				== 0){ 
				bottom_chan_net2 = net_link->net;
				FOUND_NET2 = 1;
				break;
			}
		}

		if(!FOUND_NET2) goto end;

		/* get the nearest net2 pin */

		pin = bottom_chan_net2->last_pin_link->pin;
		if((pin->x_position + 0.5 * W2) 
			> influence_reg_bot){
			d = bottom_chan->right + chan_net_sep
				- pin->x_position - 0.5 * W2;
			d_min = MIN(d,d_min);
		}
	}
end:	;
	cap = 0;
	if(d_min != HUGE){
		cap = cap_parallel(W1,W2,d_min,X_LAYER);
		chan_info->Lx_max += chan_intersect;
	}
	chan_info->Cx_max_bottom = chan_intersect * cap;
#ifdef DEBUG
	printf("\nCx_bottom_maxmin: net1 %s net2 %s bottom_chan %s\n",net1->name,net2->name,bottom_chan->name); 
	printf("d_min = %g, cap = %g,chan_intersect = %g\n",d_min,cap,chan_intersect); 
#endif 
	return(OK);
}

