/*		ROUTINE FOR ESTIMATING MAXIMUM AND
*		MINIMUM CAPACITANCES IN X-DIRECTION BETWEEN 
*		TWO NETS(NET1 INSIDE CHANNEL AND NET2 IN THE
*		TOP CHANNEL)
*       -------------------------------------------
*
*   File:       cx_top_maxmin.c
*   Author:     Umakanta Choudhury
*   Date:       20 July 1989
*
*
*/

#include <stdio.h>
#include "layout.h"

extern int strcmp();
extern double cap_parallel();
extern float chan_net_com();


REPORT cx_top_maxmin(chan_info,top_chan,net1,net2)
struct Chan_info *chan_info;
struct Channel *top_chan;
struct Net *net1;
struct Net *net2;
{

	struct Channel *channel;
	struct Channel *chan;
	struct ChannelLink *top_chan_toplink;
	struct ChannelLink *top_chan_botlink;
	struct Pin *pin;
	struct PinLink *pin_link;
	struct NetLink *net_link;
	struct Net *top_chan_net2;
	float influence_reg_top;
	float W1;
	float W2;
	float d_min;
	float d;
	float chan_intersect;
	float chan_net_sep;
	float cap;
	float Cx_max_top_toppins;
	float Cx_max_top_botpins;
	int	FOUND_NET2 = 0;

#ifdef DEBUG
	printf("cx_top_maxmin\n");
	printf("inside net %s outside net %s\n",net1->name,net2->name);
#endif 

	channel = chan_info->channel;
	chan_net_sep= channel->chan_net_separation;
	W1 = 2*(net1->true_half_width);
	W2 = 2*(net2->true_half_width);
	d_min = HUGE;

	/* check whether top_chan intersects the span of net1 */

	if((top_chan->top > net1->max_span_left) && 
		(top_chan->bottom < net1->max_span_right)){
#ifdef DEBUG
		printf("cx_top_maxmin 54\n");
#endif 
		/* calculate the intersection of channel with 
			net1 span */

		chan_intersect = chan_net_com(top_chan,net1);

		/* check whether any net2 pin in top channel 
		is in the region of influence of net1 */ 

		influence_reg_top = top_chan->left - chan_net_sep
			+ top_chan->influence_dist;

		chan_info->Cx_max_top = 0;
		Cx_max_top_toppins = 0;/* maximum coupling due to 
								top pins of top_chan */
		Cx_max_top_botpins = 0;/* maximum coupling due to 
								bottom pins of top_chan */

		/* scan through floating pins of net2 on top and
		bottom edge of top_chan if present */
		for(top_chan_toplink = top_chan->top_chan_list;
			top_chan_toplink != NULL; 
			top_chan_toplink = top_chan_toplink->next){
			chan = top_chan_toplink->channel;
			if(net_present(chan->left_crosses,net2)){
				if((chan->bottom + chan_net_sep) 
					> influence_reg_top){
					/* crossed the region of influence */
					break;
				}
				else{
					d = chan->bottom - top_chan->left 
						+ (2 * chan_net_sep);
					d_min = MIN(d,d_min);
					break;
	
				}
			}
		}
		for(top_chan_botlink = top_chan->bottom_chan_list;
			top_chan_botlink != NULL; 
			top_chan_botlink = top_chan_botlink->next){
			chan = top_chan_botlink->channel;
			if(net_present(chan->right_crosses,net2)){
				if((chan->bottom + chan_net_sep) 
					> influence_reg_top){
					/* crossed the region of influence */
					break;
				}
				else{
					d = chan->bottom - top_chan->left 
						+ (2 * chan_net_sep);
					d_min = MIN(d,d_min);
					break;
				}
			}
		}

		/* get net2 in top_chan if present */
		FOUND_NET2 = 0;
		for(net_link = top_chan->net_list; net_link != NULL;
			net_link = net_link->next){
			if(strcmp(net_link->net->name,net2->name) 
				== 0){ 
				top_chan_net2 = net_link->net;
				FOUND_NET2 = 1;
				break;
			}
		}

		if(!FOUND_NET2) goto end;

		/* get the nearest net2 pin */

		pin = top_chan_net2->pins->pin;
		if((pin->x_position - 0.5 * W2) 
			< influence_reg_top){
			d = pin->x_position - 0.5 * W2
				- top_chan->left + chan_net_sep;
			d_min = MIN(d,d_min);

		}
	}
end:	;
	cap = 0;
	if(d_min != HUGE){
		cap = cap_parallel(W1,W2,d_min,X_LAYER);
		chan_info->Lx_max += chan_intersect;
	}
	chan_info->Cx_max_top = chan_intersect * cap;
#ifdef DEBUG
	printf("\nCx_top_maxmin: net1 %s net2 %s top_chan %s\n",net1->name,net2->name,top_chan->name); 
	printf("d_min = %g, cap = %g, chan_intersect = %g\n",d_min,cap,chan_intersect); 
#endif 
	return(OK);
}

