/*   delta_min.c
 *
 *   Author:     Umakanta Choudhury
 *   Date:       22nd June 1989
 *   Last modified:  Enrico Malavasi,  Aug. 1992
 *
 *   Purpose:    Compute all performance degradations due to the minimum possible
 *               capacitances (without shielding)
 */
#include <stdio.h>
#include "layout.h"
#include "circuit.h"

void
  delta_min(layout,circuit)
struct Layout *layout;
struct Circuit *circuit;
{
    struct Performance *perf;
    struct CriticalPairLink *critpair;
    int perf_num;
    float max_pos_change;
    float max_neg_change;
    struct Chan_info *chan_info;
    double delta_min;
    struct Channel *channel;
    

    for(perf = circuit->perf_list; perf != NULL; perf = perf->next){
	perf_num = perf->number;
	if(perf->pos_constraint == YES){
	    /* constraint on maximum positive change present */
	    max_pos_change = perf->max_pos_change;
	    delta_min = 0;  
	    for(critpair=layout->critical_pairs; critpair != NULL; critpair=critpair->next){
		/* consider only positive sensitivities */
		for(chan_info = critpair->chan_info_list; chan_info!=NULL; chan_info = chan_info->next){
		    /* increment minimum possible change */
		    delta_min += (critpair->pos_sens[perf_num] * chan_info->C_min);
		}
	    }
	    perf->delta_pos_min = delta_min;
	}
	if(perf->neg_constraint == YES){
	    max_neg_change = perf->max_neg_change;
	    delta_min = 0;  
	    for(critpair=layout->critical_pairs; critpair != NULL; critpair=critpair->next){
		/* consider only negative sensitivities */
		for(chan_info = critpair->chan_info_list; chan_info!=NULL; chan_info = chan_info->next){
		    /* increment minimum possible change */
		    delta_min += (critpair->neg_sens[perf_num] * chan_info->C_min);
		}
	    }
	    perf->delta_neg_min = delta_min;
	}
    }
}
