/*      ROUTINE FOR ESTIMATING MAXIMUM AND
*       MINIMUM CROSSOVER CAPACITANCES BETWEEN 
*		TWO NETS DUE TO FIXED PINS IN COMMON SPAN 
*       -------------------------------------------
*
*   File:       fixpin_cross.c
*   Author:     Umakanta Choudhury
*   Date:       16 July 1989
*
*
*/

#include <stdio.h>
#include "layout.h"

extern int strcmp();
extern double cap_cross();


REPORT fixpin_cross(chan_info,left,right)
struct 	Chan_info *chan_info;
float	left;
float	right;
{
	struct Net *net1;
	struct Net *net2;
	struct PinLink *pin_link;
	struct Pin *pin;
	double W1; /* width of net1 */
	double W2; /* width of net2 */
	double x_position; 
	double cap; 

	net1 = chan_info->netA;
   	net2 = chan_info->netB;
	W1 = 2*(net1->true_half_width);
	W2 = 2*(net2->true_half_width);

	for(pin_link = net1->pins; pin_link != NULL;
					pin_link = pin_link->next){
		x_position = pin_link->pin->x_position;
		pin = pin_link->pin;
		if(((x_position + (0.5 * W1)) > left) && 
			((x_position - (0.5 * W1)) < right)){
			/* pin lies in common span */ 
			if(pin->edge == TOP){

				/* crossing occurs when net2 is above net1; 
				 * crossing is between net2 segment in 
				 * x direction and net1 segment in y 
				 * direction 
				 */
				cap = cap_cross(W2,W1); 
				chan_info->Ccross_2above1 += cap;
			}
			else if(pin->edge == BOTTOM){

				/* crossing occurs when net1 is above net2; 
				 * crossing is between net2 segment in 
				 * x direction and net1 segment in y 
				 * direction 
				 */

				cap = cap_cross(W2,W1); 
				chan_info->Ccross_1above2 += cap;
			}
		}
	}
	for(pin_link = net2->pins; pin_link != NULL;
					pin_link = pin_link->next){
		x_position = pin_link->pin->x_position;
		if(((x_position + (0.5 * W2)) > left) && 
			((x_position - (0.5 * W2)) < right)){
			pin = pin_link->pin;
			/* pin lies in common span */ 
			if(pin->edge == TOP){

				/* crossing occurs when net1 is above net2; 
				 * crossing is between net1 segment in 
				 * x direction and net2 segment in y 
				 * direction 
				 */

				cap = cap_cross(W1,W2); 
				chan_info->Ccross_1above2 += cap;
			}
			else if(pin->edge == BOTTOM){

				/* crossing occurs when net2 is above net1; 
				 * crossing is between net1 segment in 
				 * x direction and net2 segment in y 
				 * direction 
				 */

				cap = cap_cross(W1,W2); 
				chan_info->Ccross_2above1 += cap;
			}
		}
	}
	return(OK);
}
