/*   circuit.h
 *
 *	Author:	Umakanta Choudhury
 *      Date:	8 June 1989.
 *      Last modified:  Enrico Malavasi,  Aug. 1992
 */
#ifndef  __CIRCUIT_HEADER__
#define  __CIRCUIT_HEADER__

#include "avl.h"


/****************** STRUCTURES ****************************************/

struct Circuit {
    char    *name;			/* Circuit name			*/
    int	     num_nodes;	/* number of nodes */
    int	     num_matched_pairs;	/* number of matched node-pairs */
    char    *out_node1_name; /* name of the first output node */
    char    *out_node2_name; /* name of the second output node */
    struct Node	*node_list;	/* linked list of nodes in the circuit */
    avl_tree    *node_tree;	/* binary tree of nodes */
    struct Matched_pair	*matched_pair_list;	/* linked list of matched 
						   pairs of nodes in the circuit */
    int	     num_perf;	/* number of performance functions */
    
    struct  Performance *perf_list;	/* array of performance 
					   structures */
    avl_tree *perf_tree;	/* binary tree of performance functions  */
    float	max_process_var_percent;	/* process variation in 
						   values of routing parasitics (in percent) */
    float	max_mismatch_percent;	/* maximum mismatch between  
					   matched routing parasitics (in percent) */
    char *technologyName;
    char *sim_name;
    char *deckFile;
};

struct Node {
    struct	Node	*prev; 
    struct	Node	*next; 
    char	*name; 
    unsigned	matched:1;	/* whether it is a matched node */
    struct	Node	*matched_node;
};

struct Performance {
    struct Performance	*prev;
    struct Performance	*next;
    char    *name;			/* Performance name	*/
    char    *ckt_file_name;	/* name of the circuit file*/
    int      number;			/* Performance number	*/
    int      pos_eqn_num; /* eqn number associated with positive 
			     change constraint while generating 
			     bounds on parasitics */
    int      neg_eqn_num; /* eqn number associated with negative 
			     change constraint while generating 
			     bounds on parasitics */
    unsigned	type:1;/* flag indicating whether 
			  the performance is a 
			  standard or a nonstandard one */
    unsigned	pos_constraint:1;/* flag indicating whether 
				    positive constraint is present */
    unsigned	neg_constraint:1;/* flag indicating whether 
				    negative constraint is present */
    float   max_pos_change;	/* maximum positive change allowed */
    float   max_neg_change;	/* maximum negative change allowed */
    float   delta_pos_min;	/* minimum positive change possible */
    float   delta_neg_min;	/* minimum negative change possible */
    struct	Sens_link	*pos_sens_list;	   /* linked list of positive sensitivities and the
						      associated node pair info */
    struct	Sens_link	*neg_sens_list;	   /* linked list of negative sensitivities 
						      associated node pair info */
    struct	Sens_link	*pos_sens_stored_list;	/* linked list of positive sensitivities and the
							   associated node pair info (stored for future
							   reference) */
    struct	Sens_link	*neg_sens_stored_list;	/* linked list of negative sensitivities and the
							   associated node pair info (stored for future
							   reference) */
    struct _range {
	float min;
	float max;
    } freqRange;
};

struct Matched_pair {
    struct Matched_pair	*prev;
    struct Matched_pair	*next;
    struct Node	*nodeA;
    struct Node	*nodeB;
    unsigned	processed:1; /* a flag indicating whether a matched pair is processed */
};

struct Sens_link {
    struct Sens_link	*prev;
    struct Sens_link	*next;
    struct Node	*nodeA;
    struct Node	*nodeB;
    float	sens; /* sensitivity w.r.t cap(nodeA,nodeB) */
    struct	Chan_info	*chan_info_list;
};

#endif
