/*		CONSTRAINT HEADER
*		-----------------
*
*   File:	layout.h
*   Date:	8 June 1989.
*
*/
#ifndef  __LAYOUT_HEADER__
#define  __LAYOUT_HEADER__

#include "avl.h"
#include "macros.h"


/* CHANNEL 
   ------- */

#define  UNKNOWN_CHANNEL_TYPE    0
#define  MIGHTY_CHANNEL_TYPE     1
#define  ROADRUNNER_CHANNEL_TYPE 2

struct Layout {
    char    *name;		  /* Layout name			*/
    int     mode;                 /* NORMAL OR CHANNEL_ROUTING */    
    int     num_chan;		  /* number of channels */
    char  **perf_name;	          /* array of performance names */
    struct  ChannelLink *chan_list;	/* Linear linked list of channels */
    struct  Chan_info_list_link *chan_info_list_link;	/* Linear linked list 
							   of chan_info_list structures of 
							   all the net pairs */
    int		num_critpairs;		/* number of critical pairs */
    int		num_match_constraints;	/* number of matching constraints */
    int		num_bound_constraints;	/* number of bounding constraints */
    struct  CriticalPairLink   *critical_pairs;    /* List of pointers to critical pairs*/
    float	cap_threshold; /* threshold below which capacitances can be
				  considered as zero ;  chosen such that
				  max performance degradation caused when
				  all the critical capacitances are below
				  this threshold is 1% of maximum 
				  degradation allowed */
};

struct Channel {
    char    *name;			/* Channel name			*/

    float   top, bottom,		/* Positions of channel edges	*/
            left, right;
    float   default_net_width;
    float   default_net_separation;
    float   chan_net_separation; /* separation between channel edge
				    and any net */
    float	influence_dist; /* maximum distance between two parallel 
				   lines for which capacitive coupling 
								is considered */
    float   layout_resolution;
    unsigned orient:1; /* channel orientation, can be TOP_BOTTOM 
			  (fixed pins on top and bottom) ;
						or LEFT_RIGHT (fixed pins on left and 
						right) */
    
    int  	num_nets;		/* Number of nets in channel	*/
    avl_tree *net_tree;		/* Binary tree of nets		*/
    struct  NetLink *net_list;	/* Linear linked list of nets	*/
    
    int	num_top_pins;		/* Num of pins on top edge   	*/
    struct 	PinLink *top_pins;	/* Ordered by x-position	*/
    struct 	PinLink *last_top_pin_link;	/* pointer to last (right most)
						   top pinlink */
    
    int	num_bottom_pins;	/* Num of pins on bottom edge   */
    struct 	PinLink *bottom_pins;	/* Ordered by x-position	*/
    struct 	PinLink *last_bottom_pin_link;	/* pointer to last 
						   (right most) bottom pinlink */
    
    int	num_right_pins;		/* Num of pins on right edge   	*/
    struct 	PinLink *right_pins;	/* Ordered by x-position	*/
    
    int	num_left_pins;		/* Num of pins on left edge   	*/
    struct 	PinLink *left_pins;	/* Ordered by x-position	*/
    
    int	num_left_crosses;	/* Num of nets crossing left end*/
    struct  NetLink *left_crosses;	/* List of pointers to nets	*/
    
    int	num_right_crosses;	/* Num of nets crossing rght end*/
    struct  NetLink *right_crosses;	/* List of pointers to nets*/
    
    int	num_bottom_crosses;	/* Num of nets crossing bottom end*/
    struct  NetLink *bottom_crosses;	/* List of pointers to nets*/
    
    int	num_top_crosses;	/* Num of nets crossing top end*/
    struct  NetLink *top_crosses;	/* List of pointers to nets*/
    
    struct	Channel	*left_chan; /* pointer to left channel */
    struct	Channel	*right_chan; /* pointer to left channel */
    struct	ChannelLink	*top_chan_list; /* list of pointers to top
						   channels */
    struct	ChannelLink	*last_top_chan_link; /* pointer to last (right most)
							top channel*/
    struct	ChannelLink	*bottom_chan_list; /* list of pointers to bottom
						      channels */
    struct	ChannelLink	*last_bottom_chan_link; /* pointer to last (right 
							   most) bottom channel*/
    struct	Pin_list_link	*top_pin_lists; /* pointer to the link of
						   sublists of top pins before each 
						   top channel */
    struct	Pin_list_link	*bottom_pin_lists; /* pointer to the link of
						      sublists of bottom pins before each 
						      bottom channel */
    
};

struct Pin_list_link {
	struct	Pin_list_link	*prev; /* pointer to the previous 
					pin list link */
	struct	Pin_list_link	*next; /* pointer to the 
					next pin list link */
	struct	PinLink	*pin_list; /* pointer to the pins in the sublist */
	struct	PinLink	*last_pin_link; /* pointer to last pin in the sublist */
	struct	Channel	*channel; /* pointer to the top channel immediately
									to the left of the list */
	};
struct Net {
	char *name;			/* Name of net			*/
	struct Channel *channel;	/* In which net exists		*/
	struct PinLink *pins;		/* Pins of this net		*/
	struct PinLink	*last_pin_link;		/* pointer to the last pin of this net	(ordered in x direction) */
    int left_end_crossing,		/* Flags - set if net crosses	*/
	right_end_crossing,		/* respective ends of channel	*/
	bottom_end_crossing,		/* respective ends of channel	*/
	top_end_crossing;		/* respective ends of channel	*/

	float true_half_width;		/* Half of real net width	*/
	float true_left, true_right;	/* Real end positions of net	*/
	float half_width;		/* Half of "effective" net width*/
	float left, right;		/* Effective end positions	*/
	float max_span_left,max_span_right; /* left and right positions for
										maximum span */
	float min_span_left,min_span_right; /* left and right positions for
										maximum span */
	};


struct Pin {
	float x_position; 		/* X-Coordinate of pin		*/
	float y_position; 		/* Y-Coordinate of pin		*/
	int edge;			/* TOP,BOTTOM,RIGHT or LEFT edge of channel*/
	struct Net *net;		/* Net connected to pin		*/
	float true_half_width;		/* Half of real pin width	*/
	float true_left, true_right;	/* Real pininterval		*/
	float half_width;		/* Half of effective width	*/
	float left, right;		/* Effective pin interval	*/
	};

struct PinLink {
	struct PinLink *prev, *next;
	struct Pin *pin;
	};

struct NetLink {
	struct NetLink *prev, *next;
	struct Net *net;
	};

struct ChannelLink {
	struct ChannelLink *prev, *next;
	struct Channel *channel;
	};

struct CriticalPairLink {		/* Critical coupling capac-	*/
    struct CriticalPairLink *prev, *next;
    char *netA_name;
    char *netB_name;
    float *pos_sens;			/* array of positive sensitivities 
					   of the various performance functions 
					   w.r.t the coupling capacitance 
					   between the nets */
    float *neg_sens;			/* array of negative sensitivities 
					   of the various performance functions 
					   w.r.t the coupling capacitance 
					   between the nets */
    struct Chan_info *chan_info_list;	/* array of channel_info 
					   structures for the channels which carry
					   both the nets of the critical pair */	
};

struct Chan_info {		
    struct Chan_info *prev;             /* pointer to the previous chan_info structure  */
    struct Chan_info *next;             /* pointer to the next chan_info structure  */
    struct Channel *channel;            /* pointer to the channel */
    struct Net *netA, *netB;	        /* critical pair of nets of the channel */
    int    param_num;		        /* parameter number associated with capacitance of the
					   critical pair in that channel */
    float  C_bound;			/* Capacitance constraint	*/
    float  Cx_max;			/* maximum Capacitance possible due to adjacent segments
					   running in x direction */
    float  Cx_min;			/* minimum Capacitance possible due to adjacent segments
					   running in x direction */
    float  Cx_max_top;		        /* maximum Capacitance possible due to adjacent segments
					   running in x direction (one net inside the channel and
					   the other in top channel)*/
    float  Cx_max_bottom;		/* maximum Capacitance possible due to
					   adjacent segments running in x direction (one net inside
					   the channel and the other in bottom channel)*/
    float  Cy_max;			/* maximum Capacitance possible due to adjacent segments running 
					   in y direction */
    float  Cy_min;			/* minimum Capacitance possible due to adjacent segments running 
					   in y direction */
    float  Ccross_max;		        /* maximum cross over capacitance possible */

    float  Ccross_min;		        /* minimum cross over capacitance possible */

    float  Ccross_1above2;		/* cross over capacitance when horizontal segment of net1 is 
					   above that of net2 */

    float  Ccross_2above1;		/* cross over capacitance when horizontal segment of net2 is 
					   above that of net1 */

    float  C_max;		        /* maximum total capacitance possible */
    float  C_min;		        /* minimum total capacitance possible */
    float  C_min_shield;		/* minimum total capacitance possible with shielding */
    unsigned shielded:1;                /* flag which specifies whether the two nets of the corresponding
					   critical pair in the given channel should be shielded */
    float Lx_max;		        /* maximum length for which two adjacent lines 
					   can run in x direction */
    float Ly_max;		        /* maximum length for which two adjacent lines 
					   can run in y direction */

};
struct Chan_info_list_link {	/* linked lists of all the 
								chan_information for a net pair */
	struct Chan_info_list_link *prev, *next;

	struct Chan_info *chan_info_list;	/* array of channel_info 
							structures for the channels which carry
							both the nets of the pair */	
};

struct Crit_temp {		/* temporary structure for sorting purpose */ 
	struct Crit_temp *prev, *next;
	char *netA_name;
	char *netB_name;
	float sens;			
	};

struct CriticalIndPairLink {		/* Critical coupling ind-	*/
	struct CriticalIndPairLink *prev, *next;
	struct Net *netA, *netB;	/* itance between these two nets*/
	float  L_bound;			/* Capacitance constraint	*/
	float  Lx_max;			/* maximum Inductance possible due to
								adjacent segments running 
										in x direction */
	float  Lx_min;			/* minimum Inductance possible due to
								adjacent segments running 
										in x direction */
	float  Ly_max;			/* maximum Inductance possible due to
								adjacent segments running 
										in y direction */
	float  Ly_min;			/* minimum Inductance possible due to
								adjacent segments running 
										in y direction */
	float *sens;			/* array of sensitivities of the various
								perofrmance functions w.r.t the coupling
								inductance between the nets */
	};


/****************** DEFINITIONS ***************************************/

#define LEFT 1
#define RIGHT 2
#define TOP 3
#define BOTTOM 4
#define NO_EDGE 5
#define ABOVE 6
#define BELOW 7

#define X_LAYER 1
#define Y_LAYER 2
#define XY_LAYER 3

#define YES 1
#define NO 0

#define LEFT_RIGHT 1
#define TOP_BOTTOM 0

#endif
