/*      ROUTINE FOR COMPUTING THE MAXIMUM
*		INFLUENCE DISTANCE
*       (MODIFICATIONS NECESSARY)
*       ---------------------------------------
*
*   File:       infl_dist.c
*   Author:     Umakanta Choudhury
*   Date:       11th August 1989
*
*
*/

#include <stdio.h>
#include "layout.h"
#include "circuit.h"

extern double cap_parallel();



REPORT
  infl_dist(layout,circuit,debug_file)
struct Layout *layout;
struct Circuit *circuit;
FILE *debug_file;
{
	struct Performance *perf;
	struct CriticalPairLink *critpair;
	int perf_num;
	float max_pos_change;
	float max_neg_change;
	struct Chan_info *chan_info;
	double sens;
	double delta_max;
	struct Channel *channel;
	float init_dist;
	float influence_dist;
	float W1;
	float W2;
	float cap_x;
	float cap_y;

	if(layout->chan_list == NULL) {
	    fprintf(stderr, "** Layout not defined\n");
	    exit( -1 );
	}

	init_dist = layout->chan_list->channel->influence_dist;
	for(influence_dist=init_dist ; influence_dist>=0 ; influence_dist -= (0.1*init_dist)){

	    for(perf = circuit->perf_list; perf != NULL; perf = perf->next){
		perf_num = perf->number;
		if(perf->pos_constraint == YES){
		    /* constraint on maximum positive change present */
		    max_pos_change = perf->max_pos_change;
		    delta_max = 0;  
		    for(critpair = layout->critical_pairs; critpair != NULL;
			critpair = critpair->next){
			/* consider only positive sensitivities */
			sens = *(critpair->pos_sens + perf_num);
			for(chan_info = critpair->chan_info_list; chan_info 
			    != NULL; chan_info = chan_info->next){
			    W1 = 2 * (chan_info->netA->true_half_width);
			    W2 = 2 * (chan_info->netB->true_half_width);
			    cap_x = cap_parallel(W1,W2,influence_dist,X_LAYER);
			    cap_y = cap_parallel(W1,W2,influence_dist,Y_LAYER);
			    /* increment maximum possible change */
			    delta_max += sens * (cap_x + cap_y);  
			}
		    }
		    if(delta_max >= 0.01 * max_pos_change){
			fprintf(debug_file,"influence_dist = %g\n",influence_dist);
			fprintf(debug_file,"delta_max = %g\n",delta_max);
			return(OK);
		    }
		}
		if(perf->neg_constraint == YES){
		    max_neg_change = perf->max_neg_change;
		    delta_max = 0;  
		    for(critpair = layout->critical_pairs; critpair != NULL;
			critpair = critpair->next){
			/* consider only negative sensitivities */
			sens = *(critpair->neg_sens + perf_num);
			for(chan_info = critpair->chan_info_list; chan_info 
			    != NULL; chan_info = chan_info->next){
			    W1 = 2 * (chan_info->netA->true_half_width);
			    W2 = 2 * (chan_info->netB->true_half_width);
			    cap_x = cap_parallel(W1,W2,influence_dist,X_LAYER);
			    cap_y = cap_parallel(W1,W2,influence_dist,Y_LAYER);
			    /* increment maximum possible change */
			    delta_max += sens * (cap_x + cap_y);  
			}
		    }
		    if(delta_max <= 0.01 * max_neg_change){
			fprintf(debug_file,"influence_dist = %g\n",influence_dist);
			fprintf(debug_file,"delta_max = %g\n",delta_max);
			return(OK);
		    }
		}
	    }
	}
	return(OK);
}
