/*
*   LINEAR LIST FACILITIES
*
*   Functions: 	nwAddLink, nwInsertLink, nwRemoveLink, nwDeleteLink
*		nwDeleteList, nwConcatLists
*   File:	linearlists.c
*   Author:	Nick Weiner 
* 		22/10/85
*		nwConcatLists added 3/29/86
*
*   Here are some general purpose functions for handling linear 
*   linked lists made up with links of the form:
*
*   struct Link {struct Link *prev, *next; ...... };
*/

#include <stdio.h>

    /* Define general link template */

struct 	Link {struct Link *prev, *next; };

/*
*   nwAddLink
*
*   Create link (mallocs space), and insets link into specified place 
*   in given list.
*
*   RETURNS pointer to new link.
*/
struct Link *
  nwAddLink(linksize,listpointer,prev,next)
int		linksize;		/* In bytes			*/
struct	Link	**listpointer;		/* Addr of pointer to 1st link	*/
struct	Link	*prev, *next;		/* Prev and next links in list	*/
{
    struct Link *newlink;		/* Pointer to new link		*/

    newlink  = (struct Link *) malloc(linksize);
    if (prev != NULL) prev->next = newlink;
    else *listpointer = newlink;
    if (next!= NULL) next->prev = newlink;
    newlink->prev = prev;
    newlink->next = next;
    return newlink;
}


/*
*   nwInsertLink
*
*   Insets link into specified place in given list.
*
*   RETURNS pointer to given link.
*/
struct Link *nwInsertLink(link,listpointer,prev,next)
struct	Link	*link;			/* To be inserted		*/
struct	Link	**listpointer;		/* Addr of pointer to 1st link	*/
struct	Link	*prev, *next;		/* Prev and next links in list	*/
{
    if (prev != NULL) prev->next = link;
    else *listpointer = link;
    if (next!= NULL) next->prev = link;
    link->prev = prev;
    link->next = next;
    return link;
}

/*
*   nwRemoveLink
*
*   Removes link from list (but does not delete it).
*
*   RETURNS pointer to previous link in list.
*/
struct Link *nwRemoveLink(exitinglink,listpointer)
struct	Link	*exitinglink;		/* Link to be deleted		*/
struct	Link	**listpointer;		/* Addr of pointer to 1st link	*/
{

    struct Link *prevlink;

    prevlink = exitinglink->prev;
    if (exitinglink->prev != NULL) 
	exitinglink->prev->next = exitinglink->next;
    else *listpointer = exitinglink->next;
    if (exitinglink->next != NULL) 
	exitinglink->next->prev = exitinglink->prev;
    return prevlink;
}

/*
*   nwDeleteLink
*
*   Removes link from list and then deletes it (frees space).
*
*   RETURNS pointer to previous link if end of function is reached (!!).
*/
struct Link *nwDeleteLink(retiringlink,listpointer)
struct	Link	*retiringlink;		/* Link to be deleted		*/
struct	Link	**listpointer;		/* Addr of pointer to 1st link	*/
{

    struct Link *prevlink;

    prevlink = retiringlink->prev;
    if (retiringlink->prev != NULL) 
	retiringlink->prev->next = retiringlink->next;
    else *listpointer = retiringlink->next;
    if (retiringlink->next != NULL) 
	retiringlink->next->prev = retiringlink->prev;
    free(retiringlink);
    return prevlink;
}

/*
*   DELETE LIST
*
*   Deletes an entire linked list.
*/
void nwDeleteList(listpointer)
struct Link **listpointer;		/* Gives start of list	*/
{
while(*listpointer != NULL)
    nwDeleteLink(*listpointer,listpointer);
}




/*  
*   CONCATENATE LISTS
*
*   Given last link in list A and first link in list B, joins the two
*   lists. Expects lastinA to be finite (NOT NULL). firatinB may be NULL.
*   returns lastinA.
*/

struct Link *nwConcatLists(lastinA,firstinB)

struct Link 	*lastinA, 		/* Last link in first list	*/
		*firstinB;		/* First link in second list	*/

{

/* See what we have been given */
if(lastinA == NULL)
    {
    printf("PROGRAM ERROR  nwConcatList: lastinA is NULL.\n");
    return lastinA;
    }
if(firstinB == NULL)
    {
    /* Nothing to do */
    return lastinA;
    }

/* OK to perform the concatenation */
lastinA->next = firstinB;
if(firstinB != NULL)
    firstinB->prev = lastinA;

return lastinA;
}
