/*      ROUTINE FOR DETECTING 
*		THE UP,DOWN,LEFT AND RIGHT
*		CHANNELS FOR EACH CHANNEL
*       ---------------------------------------
*
*   File:       link_chan.c
*   Author:     Umakanta Choudhury
*   Date:       5th July 1989
*
*
*/

#include <stdio.h>
#include "layout.h"
#include "circuit.h"


REPORT link_chan(layout)
struct Layout *layout;
{
    struct Channel *channel1;
    struct Channel *channel2;
    struct ChannelLink *chan_link1;
    struct ChannelLink *chan_link2;
    struct ChannelLink *top_chan;
    struct ChannelLink *bottom_chan;
    float left1;
    float right1;
    float top1;
    float bottom1;
    float left2;
    float right2;
    float top2;
    float bottom2;

    for(chan_link1 = layout->chan_list ; chan_link1!=NULL ; chan_link1 = chan_link1->next){
	channel1 = chan_link1->channel;
	left1 = channel1->left;
	right1 = channel1->right;
	top1 = channel1->top;
	bottom1 = channel1->bottom;
	for(chan_link2 = layout->chan_list; chan_link2 != NULL; chan_link2 = chan_link2->next){
	    channel2 = chan_link2->channel;
	    /* check whether channel2 touches channel1 */
	    
	    if(channel1->orient != channel2->orient){
		/* channels of same orientation cannot touch in
		   channel routing scheme */
		
		left2 = channel2->left;
		right2 = channel2->right;
		top2 = channel2->top;
		bottom2 = channel2->bottom;
		
		/* check for left channel */ 
		if(top2 == left1){
		    if((right2 >= top1) && (left2 <= bottom1)){
			/* touching on the left */ 
			if(channel1->left_chan == NULL){
			    channel1->left_chan = channel2;
			} else {
			    fprintf(stderr, "error: both channel %s and channel %s are left channels of channel %s\n",
				    channel1->left_chan->name,channel2->name,
				    channel1->name);
			    return(ERROR);
			}
		    }
		    if( ((left2 < top1) && (left2 > bottom1)) ||
		       ((right2 < top1) && (right2 > bottom1)) ) {
			/* error in input */ 
			fprintf(stderr, "error: channel %s only half touching \
						channel %s on the left\n",channel2->name,
				channel1->name); 
			return(ERROR);
		    }
		}
		
		/* check for right channel */ 
		if(bottom2 == right1){
		    if((right2 >= top1) && (left2 <= bottom1)){
			/* touching on the right */ 
			if(channel1->right_chan == NULL){
			    channel1->right_chan = channel2;
			}
			else{
			    fprintf(stderr, "error: both channel %s and channel %s \
								are right channels of channel %s\n",
				    channel1->right_chan->name,channel2->name
				    ,channel1->name);
			    return(ERROR);
			}
		    }
		    if( ((right2 < top1) && (right2 > bottom1)) ||
		       ((left2 < top1) && (left2 > bottom1)) ) {
			/* error in input */ 
			printf("error: channel %s only half touching \
						channel %s on the right\n",channel2->name,
			       channel1->name); 
			return(ERROR);
		    }
		}
		
		/* check for top channels */ 
		if(left2 == top1){
		    if((bottom2 >= left1) && (top2 <= right1)){
			/* touching on the top */ 
			if(ds_MakeChanLink(channel1,TOP,&top_chan) != OK) return ERROR;
			channel1->top_chan_list->channel = channel2;
		    }
		    if( ((left1 > bottom2) && (left1 < top2)) ||
		       ((right1 > bottom2) && (right1 < top2)) ) {
			/* error in input */ 
			printf("error: channel %s only half touching \
						channel %s on the top\n",channel2->name,
			       channel1->name); 
			return(ERROR);
		    }
		    /* error if any due to overlapping top channels 
		     * will be detected while sorting them 
					 */
		}
		
		/* check for bottom channels */ 
		if(right2 == bottom1){
		    if((bottom2 >= left1) && (top2 <= right1)){
			/* touching on the bottom */ 
			if(ds_MakeChanLink(channel1,BOTTOM,&bottom_chan)
			   != OK) return ERROR;
			channel1->bottom_chan_list->channel = channel2;
		    }
		    if( ((left1 > bottom2) && (left1 < top2)) ||
		       ((right1 > bottom2) && (right1 < top2)) ) {
			/* error in input */ 
			printf("error: channel %s only half touching \
						channel %s on the bottom\n",channel2->name,
			       channel1->name); 
			return(ERROR);
		    }
		    /* error if any due to overlapping bottom channels 
		     * will be detected while sorting them 
		     */
		}
	    }
	}
    }
    return OK;
}
