/*		READ TEXT FILE
*		--------------
*
*   File:	maxmin_read.c
*   Author:	Umakanta Choudhury
*   Date:	20th Feb 1990.
*
*   Reads maximum and minimum capacitances from named file. 
*
*/
#include <stdio.h>
#include <string.h>
#include "layout.h"
#include "circuit.h"
#include "avl.h"
#define MAX_LINE_LENGTH 200

/*
 *  n1 n2 cmax cmin
 *  ...
 */

#define   DEFAULT_MAXCAP   1000.0
#define   DEFAULT_MINCAP   0.0

static void
  setDefaultMaxMin( cmax, cmin, layout )
float	cmax;
float   cmin;
struct Layout *layout;
{
    struct  Chan_info_list_link *link;
    struct  Chan_info *chan_info;

    for(link=layout->chan_info_list_link ; link!=NULL ;	link=link->next) {
	if((chan_info = link->chan_info_list) != NULL) {
	    chan_info->C_max = cmax;
	    chan_info->C_min = cmin;
	    chan_info->C_min_shield = cmin;
	}
    }
}

void
  maxmin_read(text_file_name, layout, circuit)
char *text_file_name;	        /* The input text file	*/
struct Layout *layout;	        /* layout structure	*/
struct Circuit *circuit;	/* circuit structure	*/
{
    FILE       *fin;		       /* The input text file  */
    char	text_line[MAX_LINE_LENGTH];
    float	cmax, cmin;
    char	nameA[64];
    char	nameB[64];
    struct  Chan_info_list_link *link;
    struct  Chan_info *chan_info;
    struct  Net *netA, *netB;
    int     pairNo;

    setDefaultMaxMin( DEFAULT_MAXCAP, DEFAULT_MINCAP, layout );

    /* Open the text file */
    if( (fin = fopen(text_file_name,"r")) == NULL) {
	VERBOSE( fprintf(stderr, "File %s not found\n", text_file_name));
	VERBOSE( fprintf(stderr, "Capacitance max/min estimates are automatically drawn.\n"));
	return;
    }
    
    VERBOSE( fprintf(stderr, "Reading the maxmin file '%s'\n", text_file_name));
    for(pairNo=0 ; nextWords( fin, text_line, 1 )!=NULL ; pairNo++)  {
	
	if(text_line[0] == '!') {
	    while( getc( fin ) != '\n')  ;
	    pairNo--;
	    continue;  /*  Comment  */
	}

	sscanf( text_line, "%s ", nameA );

	if((! strcmp( nameA, "default" )) || (! strcmp( nameA, "DEFAULT" ))) {
	    if( nextWords( fin, text_line, 2 ) == NULL) {
		fprintf(stderr, "** %s: short file\n", text_file_name);
		exit( -1 );
	    }
	    sscanf( text_line, "%g %g", &cmax, &cmin );
	    VERBOSE( fprintf(stderr, " Default max/min values set to (max:%g, min:%g)\n", cmax, cmin));
	    setDefaultMaxMin( cmax, cmin, layout );

	    pairNo--;
	    continue;
	}


	if( nextWords( fin, text_line, 3 ) == NULL) {
	    fprintf(stderr, "** %s: short file\n", text_file_name);
	    exit( -1 );
	}

	sscanf( text_line, "%s %g %g", nameB, &cmax, &cmin );
	DEBUGG( fprintf(stderr, " %s %s %g %g\n", nameA, nameB, cmax, cmin ));

	if(avl_lookup(layout->chan_list->channel->net_tree, nameA, &netA) == NULL){
	    fprintf(stderr, "Net %s not found in circuit\n", nameA);
	    exit( -1 );
	}
	if(avl_lookup(layout->chan_list->channel->net_tree, nameB,&netB) == NULL){
	    fprintf(stderr, "Net %s not found in circuit\n", nameB);
	    exit( -1 );
	}
	/* check if a chan_info_list_link was formed 
	   corresponding to this pair (because of a 
	   nonzero entry in the sensitivity file) */
	
	if( ds_FindChanInfoListLink(nameA,nameB, &link,layout) == YES) {
	    /* set the maximum and minimum cap values */
	    chan_info = link->chan_info_list;
	    chan_info->C_max = cmax;
	    chan_info->C_min = cmin;
	    chan_info->C_min_shield = cmin;
	}
    }
    VERBOSE( fprintf(stderr, " Found %d max/min capacitance estimates\n", pairNo ));
}
