/*		ROUTINE FOR ESTIMATING MAXIMUM AND
*		MINIMUM CAPACITANCES IN Y-DIRECTION BETWEEN 
*		NETS OF TWO ADJACENT PINS OF A CHANNEL(NET1 
*		PIN FIXED AND NET2 PIN FIXED OR FLOATING)
*       -------------------------------------------
*
*   File:       net1fix_net2.c
*   Author:     Umakanta Choudhury
*   Date:       14 July 1989
*
*
*/

#include <stdio.h>
#include "layout.h"

extern int strcmp();
extern double cap_parallel();


REPORT
  net1fix_net2(chan_info)
struct Chan_info *chan_info;
{
    struct Channel *channel;
    struct Channel *chan;
    struct Net	*net1;
    struct Net	*net2;
    struct PinLink *net1_pin_link;
    struct PinLink *pin_link;
    struct NetLink *list;
    struct Pin_list_link *pin_lists;
    struct Pin_list_link *pin_list_link;
    float net_separation;
    float chan_net_sep;
    float channel_width;
    float W1; /* width of net1 */
    float W2; /* width of net2 */
    float d_left;      /* distance between net1 and parallel 
			  segment of net2 on left used for computation 
			  of maximum capacitance */
    float d_min_left;  /* minimum distance between net1 and parallel 
			  segment of net2 on left used for computation 
			  of maximum capacitance */
    float d_right;     /* distance between net1 and parallel 
			  segment of net2 on right used for computation 
			  of maximum capacitance */
    float d_min_right; /* minimum distance between net1 and parallel 
			  segment of net2 on right used for computation 
			  of maximum capacitance */
    float d_min_left_or_right; /* minimum distance between net1 and 
				  parallel segment of net2 on left or right (but
				  not both) used for computation 
				  of maximum capacitance */
    float influence_reg_left; 
    float influence_reg_right; 
    float net1_x; 
    float net1_x_left; 
    float net1_x_right; 
    float net2_x; 
    float net2_x_left; 
    float net2_x_right; 
    float x_left; 
    float x_right; 
    float cap; 
    int net1_edge;
    
    DEBUGG(printf("net1fix_net2\n"));
    
    channel        = chan_info->channel;
    net_separation = channel->default_net_separation;
    chan_net_sep   = channel->chan_net_separation;
    channel_width  = channel->top - channel->bottom;
    net1 = chan_info->netA;
    net2 = chan_info->netB;
    W1   = 2*(net1->true_half_width);
    W2   = 2*(net2->true_half_width);
    /* scan through fixed pins of net1 */

    for(net1_pin_link = net1->pins; net1_pin_link != NULL; net1_pin_link = net1_pin_link->next){
	net1_x = net1_pin_link->pin->x_position;
	net1_x_left = net1_x - (0.5 * W1);
	net1_x_right = net1_x + (0.5 * W1);
	net1_edge = net1_pin_link->pin->edge;
	
	influence_reg_left = net1_x_left - channel->influence_dist;
	influence_reg_left = MAX(channel->left, influence_reg_left);
	influence_reg_right = net1_x_right + channel->influence_dist;
	influence_reg_right = MIN(channel->right, influence_reg_right);
	
	d_min_left = HUGE;
	d_min_right = HUGE;
	d_min_left_or_right = HUGE;
	
	
	if(net1_edge == TOP) pin_lists = channel->top_pin_lists;
	else pin_lists = channel->bottom_pin_lists;
	
#ifdef DEBUG
	printf("net1 %s\n",net1->name);
	printf("net1fix_net2 pin x_position = %g\n",net1_x);
	if(net1_edge == TOP) printf("edge = TOP\n");
	else printf("edge = BOTTOM\n");
#endif 
	
	for(pin_list_link = pin_lists; pin_list_link != NULL; pin_list_link = pin_list_link->next){
	    
	    /* consider the channel to the left of pinlist if any 
	       on the same edge  */
	    chan = pin_list_link->channel;
	    if(chan != NULL){
		/* channel present to the left of pin_list */
		if(net1_edge == TOP) list = chan->left_crosses;
		else list = chan->right_crosses;
		if(net_present(list,net2) == YES){
		    if((chan->bottom + chan_net_sep)
		       > influence_reg_right){
			/* crossed the region of influence 
			   of net1 */
			break;
		    }
		    else if((chan->top - chan_net_sep)
			    > influence_reg_left){
			/* intersects the region of influence */
			
			/* check on which side of the net1 */
			if(chan->top < net1_x_left){
			    /* on its left */
			    d_left = net1_x_left - chan->top 
			      + chan_net_sep;
			    d_min_left = MIN(d_left,d_min_left);
			}
			else{
			    /* on its right */
			    d_right = chan->bottom + chan_net_sep - net1_x_right;
			    d_min_right = MIN(d_right,d_min_right);
			}
		    }
		}
	    }

	    /* now consider the pin_list */
	    for(pin_link = pin_list_link->pin_list; pin_link 
		!= NULL; pin_link = pin_link->next){
		if(pin_link->pin->x_position - 0.5 * W2 > influence_reg_right){
		    /* crossed the region of influence */
		    break;
		}
		else if(strcmp(pin_link->pin->net->name,net2->name) == 0) {
		    net2_x = pin_link->pin->x_position;
		    net2_x_left = net2_x - (0.5 * W2);
		    net2_x_right = net2_x + (0.5 * W2);
		    if(net2_x_right > influence_reg_left){
			/* in the region of influence */
			/* check on which side of the net1 */
			if(net2_x < net1_x_left) {
			    /* on its left */
			    d_left = net1_x_left - net2_x_right;
			    d_min_left = MIN(d_left,d_min_left);
			} else {
			    /* on its right */
			    d_right = net2_x_left - net1_x_right;
			    d_min_right = MIN(d_right,d_min_right);
			}
		    }
		}
	    }
	}
	
	x_right = 0.99*(net1_x + 0.5 * (W1 + W2) + net_separation);
	x_left = 1.01*(net1_x - 0.5 * (W1 + W2) - net_separation);
	
	/* vertical segments of pins on opposite edge 
	   lying between x_right and x_left can not
	   run parallel to the pin; hence get the net2 pins 
	   immediate to the right of x_right and 
	   immediate to the left of x_left. For an opposite channel 
	   containing net2 in the corresponding list, if it intersects 
	   (x_left,x_right) the minimum separation between net1 
	   and net2 vertical segments can be net_separation */
	
	if(net1_edge == TOP) pin_lists = channel->bottom_pin_lists;
	else pin_lists = channel->top_pin_lists;
	
	for(pin_list_link = pin_lists; pin_list_link
	    != NULL; pin_list_link = pin_list_link->next){
	    
	    /* consider the channel to the left of pinlist if any 
	       on the opposite edge  */
		    chan = pin_list_link->channel;
		    if(chan != NULL){
			/* channel present to the left of pin_list */
			if(net1_edge == TOP) list = chan->right_crosses;
			else list = chan->left_crosses;
			
			if(net_present(list,net2) == YES){
			    if((chan->bottom + chan_net_sep)
			       > influence_reg_right){
				/* crossed the region of influence 
				   of net1 */
				break;
			    }
			    else if((chan->top - chan_net_sep)
				    > influence_reg_left){
						/* intersects the region of influence */
				
				/* check on which side of (x_left,x_right) */
				if((chan->top - chan_net_sep - (0.5 * W2))
				   < x_left){
				    /* on its left */
				    d_left = net1_x_left - chan->top 
				      + chan_net_sep;
				    d_min_left = MIN(d_left,d_min_left);
				}
				else if((chan->bottom + chan_net_sep 
					 + (0.5 * W2)) > x_right){
							/* on its right */
				    d_right = chan->bottom + chan_net_sep - net1_x_right;
				    d_min_right = MIN(d_right,d_min_right);
				}
				else{
				    /* intersects (x_left,x_right) */
				    d_min_left_or_right = net_separation;
				}
			    }
			}
		    }
		    /* now consider the pin_list */
		    for(pin_link = pin_list_link->pin_list; pin_link 
			!= NULL; pin_link = pin_link->next){
			if(pin_link->pin->x_position - 0.5 * W2 
			   > influence_reg_right){
			    /* crossed the region of influence */
			    break;
			}
			else{
			    if(strcmp(pin_link->pin->net->name,net2->name) ==0) {
				net2_x = pin_link->pin->x_position;
				net2_x_left = net2_x - (0.5 * W2);
				net2_x_right = net2_x + (0.5 * W2);
				if(net2_x_right > influence_reg_left){
				    /* in the region of influence */
				    /* check on which side of the
				       (x_left,x_right) */
				    if(net2_x < x_left){
					/* on its left */
					d_left = net1_x_left - net2_x_right;
					d_min_left = MIN(d_left,d_min_left);
					
				    }
				    else if(net2_x > x_right){
					/* on its right */
					d_right = net2_x_left - net1_x_right;
					d_min_right = MIN(d_right,d_min_right);
					
				    }
				} 
			    }
			}
		    }
		}
		cap = 0;
		/* intersecting channel contributes either to left or to
		right but not both */
		if(d_min_left_or_right != HUGE){ 
			if(d_min_left > d_min_right) 
				d_min_left = MIN(d_min_left,d_min_left_or_right);
			else 	
				d_min_right = MIN(d_min_right,d_min_left_or_right);
		}
		if(d_min_left != HUGE){ 
			cap += cap_parallel(W1,W2,d_min_left,Y_LAYER);
			chan_info->Ly_max += channel_width;
		}
		if(d_min_right != HUGE){ 
		    cap += cap_parallel(W1, W2, d_min_right, Y_LAYER);
		    chan_info->Ly_max += channel_width;
		}
		chan_info->Cy_max += channel_width * cap;
		chan_info->Cy_min = 0;
	
#ifdef DEBUG
		printf("\nnet1fix_net2: net1 %s net1_x= %g,net2 %s\n",net1->name,net1_x,net2->name); 
		printf("d_min_left = %g,d_min_right = %g cap = %g\n",d_min_left,d_min_right,cap); 
#endif 
	}
	return(OK);
}

