/*		ROUTINE FOR ESTIMATING MAXIMUM AND
*		MINIMUM CAPACITANCES IN Y-DIRECTION BETWEEN 
*		NETS OF TWO ADJACENT PINS OF A CHANNEL(NET1 
*		PIN FLOATING AND NET2 PIN FIXED OR FLOATING)
*       -------------------------------------------
*
*   File:       net1fl_net2fl.c
*   Author:     Umakanta Choudhury
*   Date:       15 July 1989
*
*
*/

#include <stdio.h>
#include "layout.h"

extern int strcmp();
extern double cap_parallel();


REPORT
  net1fl_net2(chan_info,net1_edge)
struct Chan_info *chan_info;
int	net1_edge;
{
	struct Channel *channel;
	struct Channel *net1_chan;
	struct ChannelLink *net1_chan_link;
	struct Channel *net2_chan;
	struct ChannelLink *net2_chan_link;
	struct Net	*net1;
	struct Net	*net2;
	struct NetLink *list;
	struct ChannelLink *chan_list;
	struct Pin_list_link *pin_lists;
	struct Pin_list_link *op_pin_lists;
	struct Pin_list_link *pin_list_link;
	struct PinLink *pin_link;
	float net2_x;
	float net2_x_left;
	float net2_x_right;
	float net_separation;
	float chan_net_sep;
	float channel_width;
	float W1; /* width of net1 */
	float W2; /* width of net2 */
	float d_left; /* distance between net1 and parallel 
						segment of net2 on left used for computation 
						of maximum capacitance */
	float d_min_left; /* minimum distance between net1 and parallel 
						segment of net2 on left used for computation 
						of maximum capacitance */
	float d_right; /* distance between net1 and parallel 
						segment of net2 on right used for computation 
						of maximum capacitance */
	float d_min_right; /* minimum distance between net1 and parallel 
						segment of net2 on right used for computation 
						of maximum capacitance */
	float d_min_left_or_right; /* minimum distance between net1 and 
						parallel segment of net2 on left or right (but
						not both) used for computation 
						of maximum capacitance */
	float influence_reg_left; 
	float influence_reg_right; 
	float dist; 
	float cap; 

#ifdef DEBUG
	printf("net1fix_net2\n");
#endif 
	channel = chan_info->channel;
	net_separation = channel->default_net_separation;
	chan_net_sep= channel->chan_net_separation;
	channel_width = channel->top - channel->bottom;
	net1 = chan_info->netA;
   	net2 = chan_info->netB;
	W1 = 2*(net1->true_half_width);
	W2 = 2*(net2->true_half_width);

	if(net1_edge == TOP){ 
		chan_list = channel->top_chan_list;
		pin_lists = channel->top_pin_lists;
		op_pin_lists = channel->bottom_pin_lists;
	}
	else{
		chan_list = channel->bottom_chan_list;
		pin_lists = channel->bottom_pin_lists;
		op_pin_lists = channel->top_pin_lists;
	}

	/* scan throgh floating pins of net1 on the same edge */

	for(net1_chan_link = chan_list; net1_chan_link != NULL; 
		net1_chan_link = net1_chan_link->next){

		net1_chan = net1_chan_link->channel;


		if(net1_edge == TOP) list = net1_chan->left_crosses;
		else list = net1_chan->right_crosses;

		/* check whether net1 is present in the list */

		if(net_present(list,net1) == YES){

			influence_reg_left = net1_chan->bottom + chan_net_sep
				- channel->influence_dist;
			influence_reg_left = MAX(influence_reg_left,
				channel->left);
			influence_reg_right = net1_chan->top - chan_net_sep
				+ channel->influence_dist;
			influence_reg_right = MIN(influence_reg_right,
				channel->right);

			d_min_left = HUGE;
			d_min_right = HUGE;
			d_min_left_or_right = HUGE;


			/* check whether net2 is present in the channel within 
			region of influence net1_chan on the same edge */

			for(pin_list_link = pin_lists; pin_list_link 
				!= NULL; pin_list_link = pin_list_link->next){

				/* consider the channel to the left of pinlist if any 
					on the same edge  */
				net2_chan = pin_list_link->channel;
				if(net2_chan != NULL){
					/* channel present to the left of pin_list */

					if(net1_edge == TOP) list = net2_chan->left_crosses;
					else list = net2_chan->right_crosses;


					if(net_present(list,net2) == YES){
	
						if((net2_chan->bottom + chan_net_sep) >
							influence_reg_right){
							/* crossed the region of influence */
							break;
						}
						else{
							if((net2_chan->top - chan_net_sep) >
								influence_reg_left){
								/* in the region of influence */
	
								/* check on which side of net1_chan */
								if(net2_chan->top 
									< net1_chan->bottom){
									/* on its left */
									d_left = net1_chan->bottom 
										- net2_chan->top
										+ (2 * chan_net_sep);
									d_min_left = MIN(d_left,d_min_left);
								}
								else if (net2_chan->bottom 
									> net1_chan->top){
									/* on its right */
									d_right = net2_chan->bottom 
										- net1_chan->top
										+ (2 * chan_net_sep);
									d_min_right = MIN(d_right,d_min_right);
								}
								else{
									/* same channel */
									d_min_left_or_right = net_separation;
								}
							}
						}
					}
				}
				/* now consider the pin_list */
				for(pin_link = pin_list_link->pin_list; pin_link 
					!= NULL; pin_link = pin_link->next){
					if(pin_link->pin->x_position - 0.5 * W2 
						> influence_reg_right){
						/* crossed the region of influence */
						break;
					}
					else{
						if(strcmp(pin_link->pin->net->name,net2->name)
												==0){
							net2_x = pin_link->pin->x_position;
							net2_x_left = net2_x - (0.5 * W2);
							net2_x_right = net2_x + (0.5 * W2);
							if(net2_x_right > influence_reg_left){
								/* in the region of influence */
								/* check on which side of the net1 */
								if(net2_x_right < net1_chan->bottom){
									/* on its left */
									d_left = net1_chan->bottom + 
										chan_net_sep - net2_x_right;
									d_min_left = MIN(d_left,d_min_left);
								}
								else{
									/* on its right */
									d_right = net2_x_left - 
										net1_chan->top + chan_net_sep;
									d_min_right = MIN(d_right,d_min_right);
								}
							} 
						}
					}
				}
			}

			/* check whether net2 is present in the channel within 
			region of influence net1_chan on the opposite edge */

			for(pin_list_link = op_pin_lists; pin_list_link 
				!= NULL; pin_list_link = pin_list_link->next){

				/* consider the channel to the left of pinlist if any 
					on the opposite edge  */
				net2_chan = pin_list_link->channel;
				if(net2_chan != NULL){
					/* channel present to the left of pin_list */

					if(net1_edge == TOP) 
						list = net2_chan->right_crosses;
					else 
						list = net2_chan->left_crosses;
	
					if(net_present(list,net2) == YES){
	
						if((net2_chan->bottom + chan_net_sep) >
							influence_reg_right){
							/* crossed the region of influence */
							break;
						}
						else{
							if((net2_chan->top - chan_net_sep) >
								influence_reg_left){
								/* in the region of influence */
	
								/* check on which side of net1_chan */
								if((dist = net1_chan->bottom - 
									net2_chan->top + 
									(2 * chan_net_sep))
									> net_separation){
									/* on the left */
									d_left = dist;
									d_min_left = MIN(d_left,d_min_left);
								}
								else if((dist = net2_chan->bottom - 
									net1_chan->top + 
									(2 * chan_net_sep))
									> net_separation){
									/* on the right */
									d_right = dist;
									d_min_right = MIN(d_right,d_min_right);
								}
								else{
									d_min_left_or_right = net_separation;
								}
							}
						}
					}
				}
				/* now consider the pin_list */
				for(pin_link = pin_list_link->pin_list; pin_link 
					!= NULL; pin_link = pin_link->next){
					if(pin_link->pin->x_position - 0.5 * W2 
						> influence_reg_right){
						/* crossed the region of influence */
						break;
					}
					else{
						if(strcmp(pin_link->pin->net->name,net2->name)
												==0){
							net2_x = pin_link->pin->x_position;
							net2_x_left = net2_x - (0.5 * W2);
							net2_x_right = net2_x + (0.5 * W2);
							if(net2_x_right > influence_reg_left){
								/* in the region of influence */
								/* check on which side of net1_chan */
								if((dist = net1_chan->bottom +
									chan_net_sep - net2_x_right)  
									> net_separation){
									/* on the left */
									d_left = dist;
									d_min_left = MIN(d_left,d_min_left);
								}
								else if((dist = net2_x_left -
									net1_chan->top + chan_net_sep )  
									> net_separation){
									/* on the right */
									d_right = dist;
									d_min_right = MIN(d_right,d_min_right);
								}
								else{
									d_min_left_or_right = net_separation;
								}
							} 
						}
					}
				}
			}
			cap = 0;
			/* intersecting channel contributes either to left or to
			right but not both */
			if(d_min_left_or_right != HUGE){ 
				if(d_min_left > d_min_right) 
					d_min_left = MIN(d_min_left,d_min_left_or_right);
				else 	
					d_min_right = MIN(d_min_right,d_min_left_or_right);
			}
			if(d_min_left != HUGE){ 
				cap += cap_parallel(W1,W2,d_min_left,Y_LAYER);
				chan_info->Ly_max += channel_width;
			}
			if(d_min_right != HUGE){ 
				cap += cap_parallel(W1,W2,d_min_right,Y_LAYER);
				chan_info->Ly_max += channel_width;
			}
			chan_info->Cy_max += channel_width * cap;
			chan_info->Cy_min = 0;
#ifdef DEBUG
			printf("\nnet1fl_net2fl: net1 %s net1_chan %s net2 %s\n",net1->name,net1_chan->name,net2->name); 
			printf("d_min_left = %g,d_min_right = %g cap = %g\n",d_min_left,d_min_right,cap); 
#endif 
		}
	}
	return(OK);
}
