/*		ROUTINE FOR ESTIMATING MAXIMUM AND
*		MINIMUM CAPACITANCES IN Y-DIRECTION BETWEEN 
*		NETS OF TWO PINS OF A CHANNEL(NET2 
*		PIN FIXED AND NET1 PIN FLOATING ; FIXED NET1
*		PINS ALREADY HANDLED IN SUBROUTINE net1fix_net2)
*       -------------------------------------------
*
*   File:       net2fix_net1fl.c
*   Author:     Umakanta Choudhury
*   Date:       14 July 1989
*
*
*/

#include <stdio.h>
#include "layout.h"

extern int strcmp();
extern double cap_parallel();


REPORT net2fix_net1fl(chan_info,cap_library)
struct Chan_info *chan_info;
struct Cap_library *cap_library;
{
	struct Channel *channel;
	struct Channel *chan;
	struct Net	*net1;
	struct Net	*net2;
	struct PinLink *net2_pin_link;
	struct NetLink *list;
	struct ChannelLink *chan_list;
	struct ChannelLink *chan_link;
	float net_separation;
	float chan_net_sep;
	float channel_width;
	float W1; /* width of net1 */
	float W2; /* width of net2 */
	float d_left; /* distance between net2 and parallel 
						segment of net1 on left used for computation 
						of maximum capacitance */
	float d_min_left; /* minimum distance between net2 and parallel 
						segment of net1 on left used for computation 
						of maximum capacitance */
	float d_right; /* distance between net2 and parallel 
						segment of net1 on right used for computation 
						of maximum capacitance */
	float d_min_right; /* minimum distance between net2 and parallel 
						segment of net1 on right used for computation 
						of maximum capacitance */
	float d_min_left_or_right; /* minimum distance between net2 and 
						parallel segment of net1 on left or right (but
						not both) used for computation 
						of maximum capacitance */
	float influence_reg_left; 
	float influence_reg_right; 
	float net1_x; 
	float net1_x_left; 
	float net1_x_right; 
	float net2_x; 
	float net2_x_left; 
	float net2_x_right; 
	float x_left; 
	float x_right; 
	float cap; 
	int net2_edge;

#ifdef DEBUG
	printf("net2fix_net1fl.c\n");
#endif 
	channel = chan_info->channel;
	net_separation = channel->default_net_separation;
	chan_net_sep= channel->chan_net_separation;
	channel_width = channel->top - channel->bottom;
	net1 = chan_info->netA;
   	net2 = chan_info->netB;
	W1 = 2*(net1->true_half_width);
	W2 = 2*(net2->true_half_width);
	/* scan through fixed pins of net2 */

	for(net2_pin_link = net2->pins; net2_pin_link != NULL;
					net2_pin_link = net2_pin_link->next){
		net2_x = net2_pin_link->pin->x_position;
		net2_x_left = net2_x - (0.5 * W2);
		net2_x_right = net2_x + (0.5 * W2);
		net2_edge = net2_pin_link->pin->edge;

		influence_reg_left = net2_x_left - channel->influence_dist;
		influence_reg_left = MAX(channel->left,
			influence_reg_left);
		influence_reg_right = net2_x_right + channel->influence_dist;
		influence_reg_right = MIN(channel->right,
			influence_reg_right);

		d_min_left = HUGE;
		d_min_right = HUGE;
		d_min_left_or_right = HUGE;


#ifdef DEBUG
		printf("net2 %s\n",net2->name);
		printf("net2fix_net1fl pin x_position = %g\n",net2_x);
		if(net2_edge == TOP) printf("edge = TOP\n");
		else printf("edge = BOTTOM\n");
#endif 

		if(net2_edge == TOP) chan_list = channel->top_chan_list;
		else chan_list = channel->bottom_chan_list;

		for(chan_link = chan_list; chan_link != NULL; 
			chan_link = chan_link->next){

			chan = chan_link->channel;

			if(net2_edge == TOP) list = chan->left_crosses;
			else list = chan->right_crosses;
			if(net_present(list,net1) == YES){
				if((chan->bottom + chan_net_sep)
					> influence_reg_right){
					/* crossed the region of influence 
						of net2 */
					break;
				}
				else if((chan->top - chan_net_sep)
					> influence_reg_left){
					/* intersects the region of influence */

					/* check on which side of the net2 */
					if(chan->top < net2_x_left){
						/* on its left */
						d_left = net2_x_left - chan->top 
							+ chan_net_sep;
						d_min_left = MIN(d_left,d_min_left);
					}
					else{
						/* on its right */
						d_right = chan->bottom + chan_net_sep
							- net2_x_right;
						d_min_right = MIN(d_right,d_min_right);
					}
				}
			}
		}

		x_right = 0.99*(net2_x + 0.5 * (W1 + W2) 
			+ net_separation);
		x_left = 1.01*(net2_x - 0.5 * (W1 + W2)
			- net_separation);

		/* for an opposite channel containing net2, if it intersects 
		(x_left,x_right) the minimum separation between net2 and net1 
		vertical segments can be net_separation */


		if(net2_edge == TOP) chan_list = channel->bottom_chan_list;
		else chan_list = channel->top_chan_list;

		for(chan_link = chan_list; chan_link != NULL; 
			chan_link = chan_link->next){

			chan = chan_link->channel;

			if(net2_edge == TOP) list = chan->right_crosses;
			else list = chan->left_crosses;

			if(net_present(list,net1) == YES){
				if((chan->bottom + chan_net_sep)
					> influence_reg_right){
					/* crossed the region of influence 
						of net2 */
					break;
				}
				else if((chan->top - chan_net_sep)
					> influence_reg_left){
					/* intersects the region of influence */

					/* check on which side of the net2 */
					if((chan->top - chan_net_sep - (0.5 * W1))
						< x_left){
						/* on its left */
						d_left = net2_x_left - chan->top 
							+ chan_net_sep;
						d_min_left = MIN(d_left,d_min_left);
					}
					else if((chan->bottom + chan_net_sep 
						+ (0.5 * W1)) > x_right){
						/* on its right */
						d_right = chan->bottom + chan_net_sep
							- net2_x_right;
						d_min_right = MIN(d_right,d_min_right);
					}
					else{
						/* intersects (x_left,x_right) */
						d_min_left_or_right = net_separation;
					}
				}
			}
		}
		cap = 0;
		/* intersecting channel contributes either to left or to
		right but not both */
		if(d_min_left_or_right != HUGE){ 
			if(d_min_left > d_min_right) 
				d_min_left = MIN(d_min_left,d_min_left_or_right);
			else 	
				d_min_right = MIN(d_min_right,d_min_left_or_right);
		}
		if(d_min_left != HUGE) 
			cap += cap_parallel(W1,W2,d_min_left,Y_LAYER);
		if(d_min_right != HUGE) 
			cap += cap_parallel(W1,W2,d_min_right,Y_LAYER);
		chan_info->Cy_max += channel_width * cap;
		chan_info->Cy_min = 0;
#ifdef DEBUG
		printf("\nnet2fix_net1fl: net1 %s net2_x = %g net2 %s\n",net1->name,net2_x,net2->name); 
		printf("d_min_left = %g,d_min_right = %g cap = %g\n",d_min_left,d_min_right,cap); 
#endif 
	}
	return(OK);
}
