/*    printall.c
 *
 *   Author: Umakanta Choudhury
 *   Completely rewritten by:  Enrico Malavasi,  Aug. 1992
 */
#include <stdio.h>
#include "layout.h"
#include "circuit.h"

void
 perf_pr(circuit, layout, debug_file)
struct Circuit *circuit;	/* the circuit structure */
struct Layout  *layout;	/* the layout structure */
FILE           *debug_file;
/*  Print circuit and performance info  */
{
    struct Matched_pair	*matchpair;
    struct Performance *performance;
    struct Node *node;
    int	i;

    fprintf(debug_file,"\nperf_pr.c\n");

    fprintf(debug_file,"num_nodes = %d\n",circuit->num_nodes);
    for(node = circuit->node_list; node != NULL; node = node->next){ 
	fprintf(debug_file,"node name = %s\n",node->name);
	if(node->matched == YES){
	    fprintf(debug_file,"matched\n");
	}
	else{
	    fprintf(debug_file,"not matched\n");
	}
    }
    for(matchpair = circuit->matched_pair_list; matchpair != NULL; 
	matchpair = matchpair->next){ 
	fprintf(debug_file,"match pair (%s,%s) \n",
		matchpair->nodeA->name,matchpair->nodeB->name);
    }
    for(performance = circuit->perf_list; performance != NULL; 
	performance = performance->next){ 
	fprintf(debug_file,"performance name = %s\n",performance->name);
	if(performance->pos_constraint == YES){
	    fprintf(debug_file,"max_pos_change = %g\n",performance->
		    max_pos_change);
	}
	if(performance->neg_constraint == YES){
	    fprintf(debug_file,"max_neg_change = %g\n",performance->
		    max_neg_change);
	}
    }
}

void
  sens_match_pr(circuit,layout,debug_file)
struct Circuit *circuit;	/* the circuit structure */
struct Layout *layout;	/* the layout structure */
FILE *debug_file;
/*   Print matched sensitivity info  */
{
    struct Performance *perf;
    struct Sens_link *sens_link;

    fprintf(debug_file,"\nsens_match_pr.c\n");
    
    fprintf(debug_file,"num_nodes = %d\n",circuit->num_nodes);
    for(perf = circuit->perf_list; perf != NULL; 
	perf = perf->next){ 
	fprintf(debug_file,"performance name = %s\n",perf->name);
	fprintf(debug_file,"node1	node2	sensitivity\n");
	for(sens_link = perf->pos_sens_list; sens_link != NULL;
	    sens_link = sens_link->next){
	    fprintf(debug_file,"%s		%s		%g\n",
		    sens_link->nodeA->name,sens_link->nodeB->name,
		    sens_link->sens);
	}
	for(sens_link = perf->neg_sens_list; sens_link != NULL;
	    sens_link = sens_link->next){
	    fprintf(debug_file,"%s 		%s 		%g\n",
		    sens_link->nodeA->name,sens_link->nodeB->name,
		    sens_link->sens);
	}
    }
}

void
  crit_pr(circuit,layout,debug_file)
struct Circuit *circuit;	/* the circuit structure */
struct Layout *layout;	/* the layout structure */
FILE *debug_file;
/*  Print critical pair info  */
{
    struct CriticalPairLink	*critpair_link;
    struct Chan_info	*chan_info;
    int	i;

    fprintf(debug_file,"\ncrit_pr.c\n");
    
    for(critpair_link = layout->critical_pairs; critpair_link != NULL; 
	critpair_link = critpair_link->next){ 
	fprintf(debug_file,"\ncrit_pair (%s,%s) present in:\n",
		critpair_link->netA_name,critpair_link->netB_name);
	for(chan_info = critpair_link->chan_info_list; 
	    chan_info != NULL; chan_info = chan_info->next){
	    fprintf(debug_file,"channel %s\n",chan_info->channel->name);
	    fprintf(debug_file,"netA %s netB %s\n",chan_info->netA->name,
		    chan_info->netB->name);
	    fprintf(debug_file,"Lx_max = %g, Ly_max = %g\n",
		    chan_info->Lx_max,chan_info->Ly_max);
	}
	for(i=1; i<=circuit->num_perf; i++){
	    fprintf(debug_file,"%s pos_sens = %g neg_sens = %g\n",
		    *(layout->perf_name + i),*(critpair_link->pos_sens + i),
		    *(critpair_link->neg_sens + i));
	}
    }
    fprintf(debug_file,"cap_threshold = %g\n",layout->cap_threshold);
}


void
  maxmin_pr(layout,debug_file)
struct Layout *layout;	/* the layout structure */
FILE *debug_file;
/*  Print maxmin cap info  */
{
    struct CriticalPairLink	*critpair;
    struct Chan_info	*chan_info;
    struct Channel	*channel;

	fprintf(debug_file,"\nmaxmin_pr.c\n");

	for(critpair = layout->critical_pairs; critpair != NULL; 
				critpair = critpair->next){ 
		fprintf(debug_file,"\ncritical pair (%s,%s) :\n",
			critpair->netA_name,critpair->netB_name);
		for(chan_info = critpair->chan_info_list; chan_info != NULL;
			chan_info = chan_info->next){

			channel = chan_info->channel;
			fprintf(debug_file,"\nchannel %s :\n",channel->name);
			fprintf(debug_file,"Cx_max %g \n",chan_info->Cx_max);
			fprintf(debug_file,"Cx_min %g \n",chan_info->Cx_min);
			fprintf(debug_file,"Cy_max %g \n",chan_info->Cy_max);
			fprintf(debug_file,"Cy_min %g \n",chan_info->Cy_min);
			fprintf(debug_file,"Ccross_max %g \n",chan_info->Ccross_max);
			fprintf(debug_file,"Ccross_min %g \n",chan_info->Ccross_min);
			fprintf(debug_file,"C_max %g \n",chan_info->C_max);
			fprintf(debug_file,"C_min %g \n",chan_info->C_min);
		}
	}
}

void
  delta_min_pr(layout,circuit,debug_file)
struct Layout *layout;
struct Circuit *circuit;
FILE *debug_file;
{
	struct Performance *perf;


	fprintf(debug_file,"\ndelta_min_pr.c\n");

	fprintf(debug_file,"before shielding:\n");
	for(perf = circuit->perf_list; perf != NULL; perf = perf->next){
		fprintf(debug_file,"performance %s :\n",perf->name);
		if(perf->pos_constraint == YES){
		/* constraint on maximum positive change present */
			fprintf(debug_file,"max_pos_change = %f,delta_pos_min = %f\n"
				,perf->max_pos_change,perf->delta_pos_min);
		}
		if(perf->neg_constraint == YES){
		/* constraint on maximum negative change present */
			fprintf(debug_file,"max_neg_change = %f,delta_neg_min = %f\n"
				,perf->max_neg_change,perf->delta_neg_min);
		}
	}
}

void
  bound_pr(layout,circuit,debug_file)
struct Layout *layout;
struct Circuit *circuit;
FILE *debug_file;
{
    struct Performance *perf;
    struct CriticalPairLink *critpair;
    struct Chan_info *chan_info;
    float pos_change[50];
    float neg_change[50];
    struct Sens_link *sens_link, *find_stored_link();
    float sens;
    int i;

    FILE *maxmin_temp;
    FILE *router_inp;
    char filename[50];
    
    /* Open the text file */
    sprintf(filename,"%s_%d","/tmp/parcar.maxmin_temp",getpid());
    FOPEN( maxmin_temp, filename, "w");
    sprintf(filename,"%s_%d","/tmp/parcar.router_inp",getpid());
    FOPEN( router_inp, filename, "w");

    DEBUGG( fprintf(debug_file,"\nbound_pr.c\n") );
    DEBUGG( fprintf(debug_file,"cap_threshold = %g\n\n",layout->cap_threshold) );
    for(perf = circuit->perf_list; perf != NULL; perf = perf->next){
	if(perf->pos_constraint == YES){
	    /* constraint on maximum positive change present */
	    DEBUGG( fprintf(debug_file,"perf %s pos_eqn_num = %d\n",perf->name, perf->pos_eqn_num) );
	}
	if(perf->neg_constraint == YES){
	    /* constraint on maximum negative change present */
	    DEBUGG( fprintf(debug_file,"perf %s neg_eqn_num = %d\n",perf->name, perf->neg_eqn_num) );
	}
	pos_change[perf->number] = 0; 
	neg_change[perf->number] = 0; 
    }
    fprintf(maxmin_temp, "number of nets %d\n", circuit->num_nodes);
    fprintf(maxmin_temp, "number of net pairs %d\n", layout->num_bound_constraints);
    
    for(critpair = layout->critical_pairs; critpair != NULL;
	critpair = critpair->next){
	for(chan_info = critpair->chan_info_list; chan_info != NULL;
	    chan_info = chan_info->next){
	    if(chan_info->C_max > layout->cap_threshold){
		DEBUGG( fprintf(debug_file,"\npair (%s,%s) chan %s param_num = %d\n",
				critpair->netA_name,critpair->netB_name,
				chan_info->channel->name,chan_info->param_num));
		DEBUGG( fprintf(debug_file,"C_bound[%d] = %g, max = %g, min = %g, shield_min = %g \n",
				chan_info->param_num,chan_info->C_bound,
				chan_info->C_max,chan_info->C_min,
				chan_info->C_min_shield));
		fprintf(maxmin_temp,"%s\t%s\t%.3e\t%.3e\n",
			critpair->netA_name,critpair->netB_name,
			chan_info->C_max,chan_info->C_min_shield);
		
		for(perf = circuit->perf_list; perf != NULL; 
		    perf = perf->next){ 
		    /* find the stored sensitivity link for C(name1A,name2A) */
		    if((sens_link=find_stored_link(perf, critpair->netA_name, critpair->netB_name)) == NULL) {
			sens = 0;
		    } else {
			sens = sens_link->sens;
		    }
		    if(sens >= 0){
			pos_change[perf->number] += chan_info->C_bound * sens;
		    } else {
			neg_change[perf->number] += chan_info->C_bound * sens;
		    }
		    DEBUGG( fprintf(debug_file,"%s sens = %g\n",perf->name,sens) );
		}
	    }
	}
    }
    DEBUGG( fprintf(debug_file,"\nprediction of linear analysis\n") );
    for(i=0; i<circuit->num_perf; i++) {
	DEBUGG( fprintf(debug_file,"%s pos_change=%g neg_change=%g tot_change=%g\n",
			layout->perf_name[i], pos_change[i], neg_change[i],
			pos_change[i]+neg_change[i]));
    }
    DEBUGG( fprintf(debug_file,"\ninput for checking linear analysis \n") );
    fprintf(router_inp,"channel\tnet1\tnet2\tCbound(fF)\n\n");
    for(critpair = layout->critical_pairs; critpair != NULL;
	critpair = critpair->next){
	for(chan_info = critpair->chan_info_list; chan_info != NULL;
	    chan_info = chan_info->next){
	    if(chan_info->C_max > layout->cap_threshold){
		DEBUGG( fprintf(debug_file,"cap%s%s_%s %s %s %gfF\n",
				critpair->netA_name,critpair->netB_name,
				chan_info->channel->name,
				critpair->netA_name,critpair->netB_name,
				chan_info->C_bound));
		fprintf(router_inp,"%s\t\t%s\t\t%s\t\t%.2e\n",
			(chan_info->channel->name == NULL) ? "noName" : chan_info->channel->name,
			critpair->netA_name,critpair->netB_name,
			chan_info->C_bound);
	    }
	}
    }
    FCLOSE(maxmin_temp);
    FCLOSE(router_inp);
}

#define   MAX_WEIGHT   100
void
  constraint_pr(layout, circuit, fileName)
struct Layout *layout;
struct Circuit *circuit;
char      *fileName;
{
    FILE *fil;
    struct Performance *perf;
    struct CriticalPairLink *critpair, *c2;
    struct Chan_info *chan_info;
    struct Node *nodeA, *nodeB;
    char *nameA, *nameB;
    float w;
    int i;

    if( fileName == NULL )  fil = stdout;
    else FOPEN( fil, fileName, "w" );

    fprintf(fil, "%d\t%d\n", layout->num_bound_constraints, circuit->num_perf);
    for(perf=circuit->perf_list; perf!=NULL; perf=perf->next) {
	/* constraint on maximum positive change present */
	fprintf(fil,"%s\t%.3g\t",
		perf->name,
		(perf->pos_constraint == YES) ? perf->max_pos_change : -1.0);
	
	/* constraint on maximum negative change present */
	fprintf(fil,"%.3g\n",
		(perf->neg_constraint == YES) ? (- perf->max_neg_change) : -1.0);
    }

    /*  Sort all critical pairs */
    for(critpair=layout->critical_pairs ; critpair!=NULL ; critpair=critpair->next) {
	if(strcmp(critpair->netA_name, critpair->netB_name) > 0) {
	    SWAP( char *, critpair->netA_name, critpair->netB_name);
	}
    }
    for(critpair=layout->critical_pairs ; critpair!=NULL ; critpair=critpair->next) {
	for(c2=critpair->next ; c2!=NULL ; c2=c2->next) {
	    if(((i=strcmp(critpair->netA_name, c2->netA_name)) > 0) ||
	       ((i == 0) && (strcmp(critpair->netB_name, c2->netB_name) > 0))) {

		if(c2->prev != NULL)        c2->prev->next = c2->next;
		if(c2->next != NULL) 	    c2->next->prev = c2->prev;
		c2->next = critpair;
		c2->prev = critpair->prev;
		if(critpair->prev != NULL)  critpair->prev->next = c2;
		critpair->prev = c2;
		if(c2->prev == NULL) layout->critical_pairs = c2;
		SWAP(struct CriticalPairLink *, critpair, c2);
	    }
	}
    }


    for(critpair=layout->critical_pairs ; critpair!=NULL ; critpair=critpair->next) {
	nameA = critpair->netA_name;
	nameB = critpair->netB_name;
	if(avl_lookup(circuit->node_tree, nameA, &nodeA) == NULL){
	    VERBOSE( fprintf(stderr, " Node %s ignored\n", nameA));
	    continue;
	}
	if(avl_lookup(circuit->node_tree, nameB, &nodeB) == NULL){
	    VERBOSE( fprintf(stderr, " Node %s ignored\n", nameB));
	    continue;
	}
	/* need to print the information for the matched pair too */
	
	for(chan_info=critpair->chan_info_list ; chan_info!=NULL ; chan_info=chan_info->next) {
	    
	    if(chan_info->C_max > layout->cap_threshold){
		
		fprintf(fil,"%s\t%s", nameA, nameB);
		if(output_format & BOUND_FORMAT) fprintf(fil,"\t%.3e", chan_info->C_bound);
		if(output_format & WEIGHT2_FORMAT) {
		    w = (chan_info->C_max-chan_info->C_bound)/
		      (chan_info->C_max-chan_info->C_min);
		    fprintf(fil,"\t%.3f", 1+(MAX_WEIGHT-1)*w*w);
		}

		if(output_format & SENS_FORMAT) {
		    for(perf = circuit->perf_list; perf != NULL; perf = perf->next) {
			/* find the positive sensitivity link for C(nameA,nameB) */
			fprintf(fil,"\t%.3e", critpair->pos_sens[perf->number]);
			fprintf(fil,"\t%.3e",(- critpair->neg_sens[perf->number]));
		    }
		}
		fprintf(fil, "\n");
	    }
	}
    }

    FCLOSE( fil );
}

void
  shield_pr(layout,circuit,debug_file)
struct Layout *layout;
struct Circuit *circuit;
FILE *debug_file;
{
    struct Performance *perf;
    struct CriticalPairLink *critpair;
    struct Chan_info *chan_info;
    

    fprintf(debug_file,"\nshield_pr.c\n");
    
    fprintf(debug_file,"after shielding:\n");
    for(perf = circuit->perf_list; perf != NULL; perf = perf->next){
	fprintf(debug_file,"performance %s :\n",perf->name);
	if(perf->pos_constraint == YES){
	    /* constraint on maximum positive change present */
	    fprintf(debug_file,"max_pos_change = %f,delta_pos_min = %f\n"
		    ,perf->max_pos_change,perf->delta_pos_min);
	}
	if(perf->neg_constraint == YES){
	    /* constraint on maximum negative change present */
	    fprintf(debug_file,"max_neg_change = %f,delta_neg_min = %f\n"
		    ,perf->max_neg_change,perf->delta_neg_min);
	}
    }
    fprintf(debug_file,"shielding information:\n");
    for(critpair = layout->critical_pairs; critpair != NULL;
	critpair = critpair->next){
	for(chan_info = critpair->chan_info_list; chan_info 
	    != NULL; chan_info = chan_info->next){
	    if(chan_info->shielded == YES){
		fprintf(debug_file,"crit pair (%s,%s) chan %s\n",
			critpair->netA_name,critpair->netB_name,chan_info->
			channel->name);
	    }
	}
    }
}
