c    la01ed - given an lp tableau consisting of an m by n matrix
c      a augumented by the first l columns of the identity matrix,
c      la01ed selects m columns from the tableau that form a
c      basis and returns the inverse of the basis in transposed form.
c         the caller can force columns into the basis by setting
c      nbasis and the array ind.
c
      subroutine la01ed(m,n,l,a,ia,nbasis,ind,u,iu,iec)
      implicit double precision(a-h,o-z)
      double precision a(ia,1),u(iu,1)
      integer ind(1)
c
c    m      integer scalar; (user:*); no. of rows in matrix a.
c    n      integer scalar; (user:*); no. of cols in matrix a.
c    l      integer scalar; (user:*); no. of identity matrix cols.
c    a      real array(m by n); (user:*); the matrix a.
c    ia     integer scalar; (user:*); 1st dimension of array a.
c    nbasis integer scalar; (user:*); the no. of cols defined
c           in ind to be forced into the basis.
c    ind    integer array(m+1); (user:la01ed); on entry gives the
c           cols to be forced into the basis, on return it will
c           give the m columns that were put into the basis.
c           ind(m+1) will always be returned set to n+l+1.
c    u      real array(m+1 by m+1); (*:la01ed); will contain the transpo
c           of the basis inverse.  la01ed also sets the (m+1)th row to
c           zeros and u(m+1,m+1) to one, the rest of the (m+1)th
c           col is used as workspace.
c    iu     integer scalar; (user:*); 1st dimension of u.
c    iec    integer scalar; (*:la01ed); error code, it will be
c           set to -1 on a successful inversion and to the
c           number of independent columns if la01ed is unable to
c           find m independent cols.
c
c
c      constants:
c
cc
cc    double precision zero/0d0/,one/1d0/
cc
cc  the previous line was refused by f77; replaced by the next 2
cc
      double precision zero, one
      data zero/0d0/, one/1d0/
cc
c
      m1=m+1
c
c      is caller supplying all or part of the basis ?
      if(nbasis.le.0) go to 10
c      yes: set up to select the first nbasis cols from ind
      l1=1
c      no. of cols to be scanned
      nn=nbasis
      kflag=-1
      if(nbasis.eq.m) kflag=1
      ind(m1)=n+l+1
      go to 20
c
c      la01b selects the basis
   10 kflag=0
c      no. of cols to be scanned
      nn=n
c      enter slacks plus an extra column from identity matrix
      l1=l+1
   20 do 40 j=1,l1
      do 30 i=1,m
   30 u(j,i)=zero
      u(j,j)=one
      if(kflag.eq.0) ind(j)=n+j
   40 continue
c      test for m slacks, if so we have an initial basis
      if(l1.gt.m) go to 170
c
c      if the caller is not forcing columns select m-l  columns
c      from a to complete basis, otherwise put his set in first
c      and then include the rest of the tableau.
c
      do 160 i=l1,m
c      for stability choose the column to come in which
c      has  max(dabs(ith row of inverse  into  column))
      jk=0
      amax=zero
      do 70 kl=1,nn
      k=kl
      if(kflag.ne.0) k=ind(kl)
      if(k.le.n) go to 50
c      slack column
      if(k-n.gt.i) go to 70
      sum=u(k-n,i)
      go to 60
c      column from a
   50 sum=fm01ad(i,u(1,i),a(1,k))
   60 if(dabs(sum).le.amax) go to 70
      jk=kl
      amax=dabs(sum)
   70 continue
c
c      did we find a column ?
      if(jk.eq.0) go to 190
c      yes: are we re-inverting ?
      if(kflag.eq.0) go to 90
c      yes: have we chosen this one before ?
      if(jk.lt.i) go to 190
c      no: do we have to interchange basis indicators ?
      if(jk.eq.i) go to 80
      isave=ind(jk)
      ind(jk)=ind(i)
      ind(i)=isave
   80 jk=ind(i)
      go to 110
c      build inverse: now if we have selected a column that we have
c      already chosen it means we cannot find m independent cols.
   90 do 100 j=1,i
      if(jk.eq.ind(j)) go to 190
  100 continue
c      set basis indicators
      ind(i+1)=ind(i)
      ind(i)=jk
c
c      compute vector = inverse  into  selected column
c      also initialize (i+1)th column of inverse
c
  110 do 130 j=1,m
      u(i+1,j)=zero
      if(jk.le.n) go to 120
c      slack column
      u(j,m1)=u(jk-n,j)
      go to 130
c      column from a
  120 u(j,m1)=fm01ad(i,u(1,j),a(1,jk))
      if(j.gt.i) u(j,m1)=u(j,m1)+a(j,jk)
  130 continue
      u(i+1,i+1)=one
c
c      update inverse
c
      do 150 j=1,i
      delta=u(j,i)/u(i,m1)
      do 140 k=1,m
  140 u(j,k)=u(j,k)-delta*u(k,m1)
  150 u(j,i)=delta
c      is caller forcing cols into the basis ?
      if(kflag.ge.0) go to 160
c      yes: have we included all of them yet ?
      if(i.lt.nbasis) go to 160
c      yes: arrange to get the rest from the lp tableau
      kflag=0
c      the no. of cols to be scanned
      nn=n+l
      ind(i+1)=nn+1
c
  160 continue
c
c      set error code to success
  170 iec=-1
c
c      return to caller
c
  180 return
c
c      la01ed unable to find m independent cols
c
  190 iec=i-1
      go to 180
      end
