c CHANGES:
c   (7-3-84)  The following to avoid potential problems with, for example,
c             SPICE (all are 1,$s/old/new/g):
c                   blanks --> QQblan
c                   findv  --> QQfinv
c                   gencon --> QQgenc
c                   getcon --> QQgetc
c                   result --> QQresu
c                   rnorm  --> QQrnor
c                   rsol   --> QQrsol
c                   shiftv --> QQshiv
c                   simcon --> QQsimc   (conflict with spint/simcon())
c                   storev --> QQstov   (conflict with D.SIMNON-storev())
c                   title  --> QQtitl
c
c END CHANGES.
c
      subroutine lcqp(n, ngc, lngc, lcon, lh, agc, lagc, ibound,
     *   bl, bu, lb, istate, initpt, msglvl, iupr, iutty, maxit,
     *   hesl, hesd, c, x, f, na, umin, r, u, index, el, lel,
     *   qt, lqt, ell, dd, inform, iw, liw, w, lw)
      implicit double precision (a-h,o-z)
      integer n, ngc, lngc, lcon, lh, lagc, ibound, lb, initpt,
     *   msglvl, iupr, iutty, maxit, na, lel, lqt, inform, liw, lw
      integer istate(lngc), index(lcon), iw(liw)
      dimension agc(lagc, lngc), bl(lb), bu(lb),
     *   hesl(lh), hesd(n), c(n), x(n), r(lcon), u(n),
     *   el(lel), qt(lqt, n), ell(lh), dd(n), w(lw)
c
c  *********************************************************************
c
c  subroutine lcqp, npl algorithms library ref. no. e4/40/f.
c  lcqp finds the minimum of a quadratic function (of the form
c  0.5*xt.h.x + xt.c) of n variables subject to linear constraints.
c  it is assumed that lower and upper bounds have been supplied for
c  all variables.  the inclusion of general constraints is optional.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  september 1978
c
c  *********************************************************************
c
      integer i, iatp, ib, ibnds, icos, idflag, ifmt1, ifmt2, ifmt3,
     *   ifmt4, ig, ineg, ip, iprint, irat, irtol, is, iscale, isin,
     *   is1, itype, iw1, iw2, iw3, iw4, iy, kadd, lfmt, liwork,
     *   lwork, lzt, m, mlevel, nbschg, ncon, ndel, nequal, nf, nina,
     *   niter, nlm, nngc, npiter, npseud, nviol, n2
      logical ellmod, feas, lmtest, lmtiny, nomove, pickup, psdcon,
     *   ref, vertex
      integer name(6)
c
c  the following standard functions and system functions are used
c  in lcqp
c
      double precision dot, dsqrt, x02aaf
      external lcimon, lcmon
c
      data name(1), name(2), name(3), name(4), name(5), name(6)
     *   /1hl, 1hc, 1hq, 1hp, 1h , 1h /
c
c  a machine-dependent constant is set here.  epsmch is the smallest
c  positive double precision number such that 1.0 + epsmch .gt. 1.0.
c
      epsmch = x02aaf(epsmch)
      rteps = dsqrt(epsmch)
      n2 = n + n
      nngc = n + ngc
      ncon = nngc + nngc
      lfmt = 160
      liwork = n + nngc + lfmt
      lwork = 9*n + 7*ngc
      mlevel = msglvl/1000
      if(mlevel .gt. 0) call QQtitl(iupr, iutty, name)
c
c  chqpin checks the input parameters and, optionally, sets the bounds
c  for the simple constraints.
c
      feas = .false.
      call chqpin(n, ngc, initpt, ibound, liwork, lwork, liw, lw,
     *   lcon, lagc, lngc, lqt, lel, lh, maxit, bl, bu, lb, istate,
     *   inform)
      if(inform .eq. 0) go to 10
      if(mlevel .gt. 0) call QQresu(inform, iupr, iutty, 40, feas)
      return
c
c  initialize the counters etc. and allocate the workspace.
c
   10 iprint = msglvl - mlevel*1000
      m = mlevel
      if(m .ge. 4) m = m - 4
      niter = 0
      npiter = iprint
      call zeroiv(ncon, index)
      ndel = 1
      iw1 = 1
      iw2 = iw1 + n
      iw3 = ncon + 1
      iw4 = iw3 + n
      ip = iw3 + ncon
      iy = ip + n
      ig = iy + n
      irtol = ig + n
      iscale = irtol + ncon
      irat = iw1
      iatp = iw3
      isin = iw1
      icos = iw2
      ibnds = 1
      itype = n + 1
      ifmt1 = itype + nngc
      ifmt2 = ifmt1 + 40
      ifmt3 = ifmt2 + 20
      ifmt4 = ifmt3 + 35
c
c  scale the constraint matrix and the bounds vectors for the general
c  constraints.
c
      if(ngc .eq. 0) go to 20
      call scle(n, ngc, lngc, agc, lagc, bl, bu,
     *   lb, istate, w(iscale))
      call storiv(ngc, istate, iw(itype))
c
c  call phase1 to ensure that the starting point is feasible.
c
   20 call zeroiv(n, iw(ibnds))
      vertex = .false.
      call phase1(initpt, n, ngc, nngc, lcon, bl, bu, agc, lagc, lngc,
     *   iw(itype), iprint, mlevel, iupr, iw(ifmt1), lfmt, lcimon,
     *   lcmon, maxit, epsmch, rteps, vertex, iw(ibnds), x, feas,
     *   nviol, na, nina, lzt, w(ig), u, r, w(iscale), w(irtol), qt,
     *   lqt, el, lel, index, ncon, nequal, inform, iw, liw, w, lw)
      if(inform .eq. 0) go to 30
      if(mlevel .gt. 0) call QQresu(inform, iupr, iutty, 40, feas)
      return
c
c  when feasibility has been established, prepare for the second phase
c  of the algorithm.  a successful call of phase1 will have set vertex
c  true if there are exactly n equality constraints active but false
c  if there are fewer.
c
   30 psdcon = .false.
      lmtest = .false.
      lmtiny = .false.
      ellmod = .false.
      is = 0
      ineg = 0
      if(.not. vertex) go to 40
      call qfun(n, x, f, w(ig), hesl, lh, hesd, c, iw, liw, w, lw)
      nf = 1
      go to 230
   40 nf = 0
      umin = 0.0e+0
      umax = 0.0e+0
      atp = 1.0e+6
      npseud = lzt
      psdcon = npseud .gt. 0
      if(psdcon) call addpsd(n, ngc, lcon, agc, lagc, lngc,
     *   na, nina, lzt, el, lel, qt, lqt, index, r, w(irtol),
     *   iw(ibnds), w, lw)
      nomove = .false.
      pickup = .false.
      idflag = 0
c
c  the main iteration loop begins here.
c
   50 call qfun(n, x, f, w(ig), hesl, lh, hesd, c, iw, liw, w, lw)
      nf = nf + 1
      nbschg = 0
      go to 70
c
c  check the number of basis changes.
c
   60 inform = 3
      if(nbschg .ge. n2) go to 220
c
c  check the number of iterations.
c
   70 inform = 2
      if(niter .ge. maxit) go to 220
      inform = 0
      niter = niter + 1
c
c  print out information at prescribed intervals.
c
      if(m .eq. 0 .or. iprint .eq. 0) go to 90
      if(npiter .ne. iprint) go to 80
      npiter = 1
      if(psdcon .and. m .ge. 3) call psdres(n, ngc, nina, iw, x,
     *   bl, lb, r, lcon)
      call lcmon(m, iupr, iw(ifmt1), iw(ifmt2), iw(ifmt3), iw(ifmt4),
     *   n, ngc, na, nequal, ncon, lcon, nngc, .true., nviol,
     *   niter, nf, f, x, 0.0e+0, umin, umax, is, u, r,
     *   w(iscale), lngc, index, iw(itype), 1, .false.,
     *   0.0e+0, .false., .false., iw, liw, w, lw)
      go to 90
   80 npiter = npiter + 1
c
c  if x is a minimum on the current subspace (that is, if no constraint
c  was added to the active set in the previous iteration) or if there
c  are now n constraints active, try to delete a constraint.
c
c  if the basis still contains pseudo-constraints, call delpsd to
c  select one of these for deletion.
c
   90 lmtest = .false.
      lmtiny = .false.
      is = 0
      if(pickup .and. na .lt. n) go to 130
      if(.not. psdcon) go to 100
      call delpsd(n, na, index, ncon, npseud, iw(ibnds), is)
      i = index(is)
      ib = i + ngc
      rlast = x(i) - bl(ib)
      if(npseud .eq. 0) call zeroiv(n, iw(ibnds))
      go to 120
c
c  if there are no pseudo-constraints in the basis, compute and test
c  the lagrange multipliers for all the active inequalities.
c
  100 if(na .le. nequal) go to 220
      rlast = 0.0e+0
      call lagmul(n, na, el, lel, w(ig), qt, lqt, u, w(iw1))
  110 call chslm1(na, nequal, u, is, umax, umin)
      lmtest = .true.
c
c  terminate if all the multipliers are indisputably positive.
c  otherwise delete the constraint with the negative multiplier of
c  largest modulus, calling delcon to modify the rtq factorization of
c  the matrix of active constraints, delprm to update the numbers of
c  active and inactive constraints and columns in z and to make the
c  necessary permutations in the lists of constraints, residuals and
c  tolerances and ldlast to compute the final row of the cholesky
c  factors of the new projected hessian.  check the sign and the
c  modulus of the added diagonal element.
c
      test = 1.0e+1*epsmch*dabs(umax)
      if(umin .eq. umax) test = 1.0e+2*epsmch
      if(umin .gt. test) is = 0
      if(npiter .eq. 1 .and. m .ge. 1) call lcmon(m, iupr, iw(ifmt1),
     *   iw(ifmt2), iw(ifmt3), iw(ifmt4), n, ngc, na, nequal,
     *   ncon, lcon, nngc, .true., nviol, niter, nf, f, x,
     *   0.0e+0, umin, umax, is, u, r, w(iscale), lngc, index,
     *   iw(itype), 1, .true., 0.0e+0, .false., .false., iw, liw,
     *   w, lw)
      if(umin .gt. test) go to 220
      lmtiny = umin .ge. (- test)
      nbschg = nbschg + 1
  120 call delcon(n, na, is, el, lel, qt, lqt)
      call delprm(na, nina, lzt, ncon, is, index, w(irtol), r)
      if(rlast .gt. 0.0e+0) r(nina) = rlast
      call ldlast(n, na, qt, lqt, hesl, lh, hesd, ell, dd,
     *   w(iw1), w(iw2), ellmod)
      idflag = 0
      dlast = dd(lzt)
      test = 1.0e+1*epsmch
      if(dlast .lt. test) idflag = 1
      if(dlast .lt. 0.0e+0) idflag = 2
      if(dlast .lt. (- 1.0e+1)) idflag = 3
c
c  if the only suspect multipliers were all close to zero and the new
c  diagonal factor of the projected hessian is positive definite,
c  try deleting another constraint with a tiny multiplier.
c
      if(.not. lmtiny .or. idflag .ge. 1) go to 130
      if(na .le. nequal) go to 220
      is1 = is + 1
      nlm = na - is + 1
      if(nlm .gt. 0) call QQstov(nlm, u(is1), u(is))
      is = 0
      go to 110
c
c  compute the search direction vector and its euclidean norm.
c  try to ensure that the direction is both downhill and feasible.
c
  130 call eltsol(lzt, lh, lzt, ell, w(iw1))
      alpha = 1.0e+0
      call zpacmp(n, lzt, alpha, qt, lqt, w(iw1), w(ip), ptp)
      pnorm = dsqrt(ptp) + epsmch
      gtp = dot(n, w(ig), w(ip))
      test = epsmch*pnorm
c
c  if the projected gradient is small and the projected hessian is
c  positive definite, try deleting another constraint.
c
      if(dabs(gtp) .ge. test .or. idflag .ge. 1) go to 140
      pickup = .false.
      psdcon = npseud .gt. 0
      if(psdcon) go to 90
      if(na .le. nequal) go to 220
      rlast = 0.0e+0
      if(.not. lmtest) go to 100
      is1 = is + 1
      nlm = na - is + 1
      if(nlm .gt. 0) call QQstov(nlm, u(is1), u(is))
      is = 0
      go to 110
c
c  if no real constraint has been deleted, choose the sign of p to
c  give a descent direction.
c
  140 if(is .gt. 0 .and. .not. psdcon) go to 150
      psdcon = npseud .gt. 0
      if(gtp .gt. 0.0e+0) go to 170
      go to 180
c
c  when a real constraint has been deleted, choose the sign of p to
c  give a feasible direction.
c
  150 atp = 1.0e+6
      ineg = 0
      call chkatp(n, ngc, ncon, ndel, index, agc, lagc, lngc,
     *   w(ip), test, atp, ineg)
      if(dabs(atp) .gt. test) go to 160
      if(gtp .gt. 0.0e+0) go to 170
      go to 180
  160 if(ineg .eq. 1 .and. (gtp .ge. test .or. (gtp .gt. (- test)
     *   .and. idflag .ge. 2) .or. lmtiny)) go to 170
      if(ineg .eq. 0 .and. (gtp .le. test .or. (gtp .lt. test .and.
     *   idflag .ge. 2) .or. lmtiny)) go to 180
c
c  if it is demonstrably impossible to find a feasible descent
c  direction, terminate.
c
      inform = 5
      go to 220
  170 call negvec(n, w(ip))
      gtp = - gtp
c
c  QQgetc computes alpha as the distance to the nearest inactive
c  constraint and returns kadd as the index of its residual.  if there
c  is no constraint in the positive p direction and the projected
c  hessian is not positive definite, an error indicator is set and
c  the program terminates.
c
  180 alpha = pnorm
      rho = 1.0e+0/(epsmch*pnorm)
      call QQgetc(n, na, ngc, ncon, w(irtol), rteps, rho,
     *   index, agc, lagc, lngc, w(ip), r, kadd, alpha,
     *   feas, w(iatp), w(irat))
      if(alpha .lt. rho .or. idflag .eq. 0) go to 190
      inform = 6
      if(psdcon) go to 230
      go to 220
c
c  if the projected hessian is positive definite, alpha is replaced by
c  the unit step rho whenever this is feasible.
c
  190 if(idflag .eq. 0) rho = - gtp/dlast
      pickup = alpha .le. rho
      if(.not. pickup) alpha = rho
      nomove = alpha .eq. 0.0e+0
      if(nomove .and. pickup) go to 200
      if(nomove) go to 60
c
c  if the step length is non-zero, vecsum updates the x values and
c  upresd uses the inner products computed during QQgetc to update
c  the residuals.
c
      call vecsum(n, alpha, w(ip), x, x)
      call upresd(na, ncon, alpha, w(iatp), r)
      if(.not. pickup) go to 50
c
c  when a new constraint is added, addprm updates the permutations
c  of constraints, residuals and tolerances, addcon updates the rtq
c  factorization of the matrix of active constraints and addphf
c  modifies the cholesky factors of the projected hessian.  if the
c  last element of the diagonal factor was previously less than - 10,
c  ldlast recomputes the final rows of the new l and d.
c
  200 call addprm(na, nina, lzt, kadd, index, ncon, w(irtol), r)
      call addcon(n, na, ngc, kadd, .false., agc, lagc, lngc,
     *   el, lel, qt, lqt, w(isin), w(icos), gamma)
      if(na .eq. n) go to 210
      call addphf(n, na, ell, lh, dd, n, w(isin), w(icos), w(iw3),
     *   n2)
      if(ellmod .or. idflag .eq. 3) call ldlast(n, na, qt, lqt,
     *   hesl, lh, hesd, ell, dd, w(iw3), w(iw4), ellmod)
      idflag = 0
      dlast = dd(lzt)
      test = 1.0e+1*epsmch
      if(dlast .lt. test) idflag = 1
      if(dlast .lt. 0.0e+0) idflag = 2
      if(dlast .lt. (- 1.0e+1)) idflag = 3
  210 if(na .eq. n) idflag = 0
c
c  if a zero step has been taken, update the number of
c  basis changes.
c
      if(.not. nomove) go to 50
      nbschg = nbschg + 1
      go to 60
c
c  when all the lagrange multipliers are positive, refine x to satisfy
c  the active constraints more accurately, unscale the constraint matrix
c  and the bounds vectors, recompute all the residuals and print out
c  information if required.
c
  220 if(.not. psdcon) call refcon(n, na, ngc, ncon, w(irtol),
     *   index, agc, lagc, lngc, bl, bu, lb, el, lel, qt, lqt, x,
     *   ref, w(iw3), w(iw4))
  230 if(ngc .gt. 0) call unscle(n, ngc, lngc, agc, lagc, bl, bu,
     *   lb, istate, w(iscale))
      call allres(n, ngc, lngc, istate, agc, lagc, bl, bu, lb,
     *   x, r, lcon)
      if(mlevel .eq. 0) return
      if(psdcon .and. mlevel .ge. 3) call psdres(n, ngc, nina, iw, x,
     *   bl, lb, r, lcon)
      call lcmon(mlevel, iupr, iw(ifmt1), iw(ifmt2), iw(ifmt3),
     *   iw(ifmt4), n, ngc, na, nequal, ncon, lcon, nngc,
     *   .true., nviol, niter, nf, f, x, 0.0e+0, umin, umax,
     *   is, u, r, w(iscale), lngc, index, iw(itype), 1,
     *   lmtest, 0.0e+0, .false., .true., iw, liw, w, lw)
      call QQresu(inform, iupr, iutty, 40, feas)
      return
c
c  end of subroutine lcqp
c
      end
      subroutine phase1(iac, n, ngc, nngc, lcon, bl, bu, agc, lagc,
     *   lagct, istate, iprint, mlevel, iupr, ifmt, lfmt, imonit,
     *   monit, maxit, epsmch, eps, vertex, ibnds, x, feas, nviol,
     *   na, nina, lzt, g, u, r, scale, rtol, qt, lqt, el,
     *   lel, index, ncon, nequal, ifail, iw, liw, w, lw)
      implicit double precision (a-h,o-z)
      integer iac, n, ngc, nngc, lcon, lagc, lagct, iprint, mlevel,
     *   iupr, lfmt, maxit, nviol, na, nina, lzt, lqt, lel, ncon,
     *   nequal, ifail, liw, lw
      logical vertex, feas
      integer istate(nngc), ifmt(lfmt), ibnds(n), index(lcon), iw(liw)
      dimension bl(nngc), bu(nngc), agc(lagc, lagct),
     *   x(n), g(n), u(n), r(lcon), scale(lagct),
     *   rtol(lcon), qt(lqt, n), el(lel), w(lw)
      external monit
c
c  *********************************************************************
c
c  phase1 attempts to find a feasible initial point or, optionally, if
c  the logical variable vertex has been set to true, a feasible initial
c  vertex for the linearly-constrained minimization routines.  all the
c  equality constraints are made active.  when the user has supplied a
c  starting point (iac = 0, 1 or 2) and not requested a vertex, the
c  subroutine chooses the feasible point which lies nearest to the given
c  point.  the error indicator ifail is set to 4 if no feasible solution
c  exists.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  march 1979
c
c  *********************************************************************
c
      integer idrop, ifmti, ifmtr, ifmt1, ifmt2, ifmt3, ifmt4, ip,
     *   isimbd, iwa, iwb, iy, l, lblock, lw1, lw2, m, nbschg, nf,
     *   niter, npiter
      logical final, lmtest, nomove, term
c
c  allocate the workspace.
c
      l = n + ngc
      lblock = l + l
      iwa = 1
      iwb = iwa + lblock
      ip = iwb + lblock
      iy = ip + n
      lw1 = iwa
      lw2 = lw1 + n
      isimbd = ngc + 1
      ifmt1 = 1
      ifmt2 = ifmt1 + 40
      ifmt3 = ifmt2 + 20
      ifmt4 = ifmt3 + 35
      ifmti = ifmt4 + 35
      ifmtr = ifmti + 10
c
c  initialize the counters.
c
      na = 0
      nf = 0
      niter = 0
      nviol = 0
      m = mlevel
      if(m .ge. 4) m = m - 4
      npiter = iprint
      ifail = 0
c
c  initialize the arrays and other parameters.
c
      call setqu(n, u, qt, lqt)
      f = 0.0e+0
      ymod = 0.0e+0
      lmtest = .false.
      umin = 0.0e+0
      umax = 0.0e+0
      idrop = 0
c
c  compute the tolerances for the constraint residuals.
c
      call tolcmp(n, ngc, eps, nngc, bl, bu, istate,
     *   ncon, rtol, lcon)
c
c  put all the equality constraints into the basis.
c
      call equcon(n, ngc, nngc, istate, bl, agc, lagc, lagct, el, lel,
     *   qt, lqt, na, index, ncon, w(iwb), ifail, w, lw)
      nequal = na
c
c  if a vertex is required, fill up the basis with inequalities.  set
c  the error indicator if there are fewer than n genuine constraints.
c
      if(.not. vertex) go to 10
      call zerovc(n, x)
      if(nequal .eq. n .or. ifail .ne. 0) go to 20
      call fillup(nequal, n, ngc, nngc, ibnds, istate, bl, bu, agc,
     *   lagc, lagct, el, lel, qt, lqt, na, index, ncon, w(iwb),
     *   w, lw)
      if(na .lt. n) ifail = 7
      go to 20
c
c  if fewer than n constraints are now active, set the starting point
c  as requested by the user.
c
   10 if(iac .eq. 1) call QQstov(n, bl(isimbd), x)
      if(iac .eq. 2) call QQstov(n, bu(isimbd), x)
      if(iac .eq. 3) call zerovc(n, x)
c
c  print out the starting point and a key to the constraint indices.
c
   20 if(mlevel .gt. 0) call imonit(iupr, n, ngc, nngc, x, istate,
     *   ifmt(ifmti), ifmt(ifmtr), ifmt(ifmt1), ifmt(ifmt2),
     *   ifmt(ifmt3), ifmt(ifmt4))
      if(ifail .ne. 0) return
      if(na .eq. 0) go to 30
c
c  compute the vector of least length which satisfies the active
c  constraints.  whenever possible, modify the solution to minimize
c  the distance from the user-supplied starting point.
c
      call QQfinv(n, na, el, lel, qt, lqt, w(iwb), w(lw2), w(lw1))
      if(na .eq. n) call QQstov(n, w(lw1), x)
      if(na .lt. n) call QQshiv(n, na, x, qt, lqt, w(lw2), w(lw1),
     *   w(iwb))
c
c  record the indices and compute the residuals of the inactive
c  constraints.
c
   30 call settol(n, na, ngc, lcon, istate, nngc, iw, index,
     *   rtol, w, lw)
      call inacrs(n, na, ngc, ncon, rtol, index, agc, lagc, lagct,
     *   bl, bu, nngc, x, r)
      nina = ncon - na
      lzt = n - na
c
c  check for feasibility and set up a pseudo-gradient if necessary.
c
      call psdgrd(n, na, ngc, ncon, rtol, index, agc, lagc, lagct,
     *   r, feas, nviol, g)
      nomove = .false.
      if(nequal .eq. n .and. .not. feas) go to 50
      if(feas) go to 60
c
c  call minlof to minimize the pseudo-gradient.  then check again
c  and continue if possible until feasibility is achieved.
c
   40 call minlof(n, ngc, nngc, lcon, ncon, rtol, agc, lagc, lagct,
     *   istate, iprint, m, iupr, ifmt, lfmt, monit, iwa, iwb, maxit,
     *   epsmch, eps, f, x, ymod, feas, nviol, lmtest, umin, umax,
     *   idrop, nf, niter, npiter, na, nequal, nina, lzt, g,
     *   w(iy), w(ip), u, r, scale, qt, lqt, el, lel, index,
     *   nomove, nbschg, term, ifail, iw, liw, w, lw)
      if(term .and. ifail .ne. 0) go to 70
      call psdgrd(n, na, ngc, ncon, rtol, index, agc, lagc, lagct,
     *   r, feas, nviol, g)
      if(feas) go to 60
      if(.not. term) go to 40
c
c  set the error indicator if a feasible point cannot be found.
c
   50 ifail = 4
      go to 70
c
c  if phase1 has terminated successfully, set the auxiliary function
c  value and the norm of its projected gradient to zero.
c
   60 if(m .eq. 0) return
      f = 0.0e+0
      ymod = 0.0e+0
      lmtest = .false.
      final = .false.
      go to 80
c
c  print out information as required.
c
   70 if(ngc .gt. 0) call unscle(n, ngc, ngc, agc, lagc,
     *   bl, bu, nngc, istate, scale)
      call allres(n, ngc, lagct, istate, agc, lagc, bl, bu, nngc,
     *   x, r, lcon)
      if(mlevel .eq. 0) return
      m = mlevel
      final = .true.
   80 if(.not. nomove) niter = niter + 1
      call monit(m, iupr, ifmt(ifmt1), ifmt(ifmt2), ifmt(ifmt3),
     *   ifmt(ifmt4), n, ngc, na, nequal, ncon, lcon, nngc,
     *   .false., nviol, niter, nf, f, x, ymod, umin, umax, idrop,
     *   u, r, scale, lagct, index, istate, 2, lmtest, 0.0e+0,
     *   .false., final, iw, liw, w, lw)
      return
c
c  end of subroutine phase1
c
      end
      subroutine fillup(nequal, n, ngc, nngc, ibnds, istate, bl, bu,
     *   agc, lagc, lagct, el, lel, qt, lqt, na, index, lin, rhs,
     *   w, lw)
      implicit double precision (a-h,o-z)
      integer nequal, n, ngc, nngc, lagc, lagct, lel, lqt, na, lin,
     *   lw
      integer ibnds(n), istate(nngc), index(lin)
      dimension bl(nngc), bu(nngc), agc(lagc, lagct),
     *   el(lel), qt(lqt, n), rhs(n), w(lw)
c
c  *********************************************************************
c
c  fillup completes the set of n active constraints by drawing first on
c  the simple constraints with finite bounds and then, if necessary, on
c  the general inequality constraints, provided that enough of these
c  are available.
c  the length of the workspace vector w must be at least 2*n.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  october 1977
c
c  *********************************************************************
c
      if(nequal .ge. n) return
      na = nequal
      call QQsimc(n, ngc, nngc, ibnds, bl, bu, istate, agc, lagc,
     *   lagct, el, lel, qt, lqt, na, index, lin, rhs, w, lw)
      if(na .ge. n .or. ngc .eq. 0) return
      call QQgenc(n, ngc, nngc, istate, bl, bu, agc, lagc,
     *   lagct, el, lel, qt, lqt, na, index, lin, rhs, w, lw)
      return
c
c  end of subroutine fillup
c
      end
      subroutine addpsd(n, ngc, ncon, agc, lagc, lagct, na, nina,
     *   lzt, el, lel, qt, lqt, index, r, rtol, ipseud, w, lw)
      implicit double precision (a-h,o-z)
      integer n, ngc, ncon, lagc, lagct, na, nina, lzt, lel, lqt, lw
      integer index(ncon), ipseud(n)
      dimension agc(lagc, lagct), el(lel),
     *   qt(lqt, n), r(ncon), rtol(ncon), w(lw)
c
c  *********************************************************************
c
c  if there are fewer than n constraints in the active set, addpsd
c  fills up the basis with pseudo-constraints by fixing the required
c  number of variables on temporary lower bounds consisting of their
c  current values and making the entry of - 2 in each of the
c  corresponding positions of the integer vector ipseud.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
      integer i, iadd, ii, lw1, lw2
c
c  the following standard function and system function are used by
c  addpsd
c
c
      if(na .eq. n) return
c
c  a machine-dependent constant is set here.  epsmch is the smallest
c  positive double precision number such that 1.0 + epsmch .gt. 1.0.
c
      epsmch = x02aaf(epsmch)
      rteps = dsqrt(epsmch)
      lw1 = 1
      lw2 = lw1 + n
      call zeroiv(n, ipseud)
      do 30 i = 1, n
c
c  check if the addition would make the basis singular.
c
         call q2mult(n, ngc, i, na, agc, lagc, lagct,
     *      qt, lqt, gamma, w(lw1))
         if(gamma .le. rteps) go to 30
c
c  locate the relevant index in the list of inactive constraints.
c
         ii = na
         do 10 iadd = 1, nina
            ii = ii + 1
            if(index(ii) .eq. i) go to 20
   10    continue
         go to 30
c
c  add the new pseudo-constraint to the basis.
c
   20    call addprm(na, nina, lzt, iadd, index, ncon, rtol, r)
         call addcon(n, na, ngc, iadd, .true., agc, lagc, lagct,
     *      el, lel, qt, lqt, w(lw1), w(lw2), gamma)
         ipseud(i) = - 2
         if(na .eq. n) return
   30 continue
      return
c
c  end of subroutine addpsd
c
      end
      subroutine equcon(n, ngc, nngc, istate, bl, agc, lagc, lagct, el,
     *   lel, qt, lqt, na, index, lin, rhs, ifail, w, lw)
      implicit double precision (a-h,o-z)
      integer n, ngc, nngc, lagc, lagct, lel, lqt, na, lin, ifail, lw
      integer istate(nngc), index(lin)
      dimension bl(nngc), agc(lagc, lagct), el(lel),
     *   qt(lqt, n), rhs(n), w(lw)
c
c  *********************************************************************
c
c  equcon scans the general constraints for any which is specified as
c  an equality and makes it active by recording its index in the vector
c  index.  it then enters its right-hand side in the vector rhs and
c  calls the auxiliary routine addcon to update the rtq factors of the
c  matrix of active constraints.  the subroutine returns with the error
c  indicator ifail = 3 if it detects more than n equality constraints
c  and with ifail = 100 + iadd if the basis becomes singular.
c  the length of the workspace vector w must be at least 2*n.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  october 1977
c
c  *********************************************************************
c
      integer i, iadd, lw1, lw2, nequal, n2
c
c  the following standard function and system function are used
c  by equcon
c
      double precision x02aaf
c
c  a machine-dependent constant is set here.  epsmch is the smallest
c  positive double precision number such that 1.0 + epsmch .gt. 1.0.
c
      epsmch = x02aaf(epsmch)
      rteps = dsqrt(epsmch)
      ifail = 0
      nequal = 0
      lw1 = 1
      lw2 = lw1 + n
      n2 = n + n
      do 20 i = 1, nngc
         if(istate(i) .ne. 0) go to 20
         if(nequal .eq. n) go to 30
         iadd = n2 + i
         if(i .gt. ngc) iadd = i - ngc
c
c  if the new constraint is linearly dependent on the active set,
c  give up.
c
         call q2mult(n, ngc, iadd, nequal, agc, lagc, lagct,
     *      qt, lqt, gamma, w(lw1))
         if(gamma .gt. rteps) go to 10
         ifail = 100 + iadd
         return
c
c  add the new constraint to the basis and update the rtq factors.
c
   10    nequal = nequal + 1
         index(nequal) = iadd
         rhs(nequal) = bl(i)
         call addcon(n, nequal, ngc, iadd, .true., agc, lagc, lagct,
     *      el, lel, qt, lqt, w(lw1), w(lw2), gamma)
   20 continue
      na = nequal
      return
   30 ifail = 3
      return
c
c  end of subroutine equcon
c
      end
      subroutine QQgenc(n, ngc, nngc, istate, bl, bu, agc, lagc,
     *   lagct, el, lel, qt, lqt, na, index, lin, rhs, w, lw)
      implicit double precision (a-h,o-z)
      integer n, ngc, nngc, lagc, lagct, lel, lqt, na, lin, lw
      integer istate(nngc), index(lin)
      dimension bl(nngc), bu(nngc), agc(lagc, lagct),
     *   el(lel), qt(lqt, n), rhs(n), w(lw)
c
c  *********************************************************************
c
c  QQgenc scans the general constraints for any which is not specified
c  as an equality and makes it active by recording its index in the
c  vector index.  it then enters its right-hand side in the vector rhs
c  and calls the auxiliary routine addcon to update the rtq factors of
c  the matrix of active constraints.  for each added constraint, the
c  corresponding entry in istate is set to the negative of its original
c  value.  the process terminates when n constraints are active.
c  the length of the workspace vector w must be at least 2*n.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  october 1977
c
c  *********************************************************************
c
      integer i, iadd, isi, lw1, lw2, next, n2
c
c  the following standard function and system function are used by
c  QQgenc
c
c
      if(na .ge. n) return
c
c  a machine-dependent constant is set here.  epsmch is the smallest
c  positive double precision number such that 1.0 + epsmch .gt. 1.0.
c
      epsmch = x02aaf(epsmch)
      rteps = dsqrt(epsmch)
      n2 = n + n
      lw1 = 1
      lw2 = lw1 + n
      next = na + 1
      do 30 i = 1, ngc
         isi = istate(i)
         if(isi .eq. 0) go to 30
         if(isi .eq. 2) go to 10
         iadd = n2 + i
         rhs(next) = bl(i)
         go to 20
   10    iadd = n2 + ngc + i
         rhs(next) = - bu(i)
c
c  check the basis for singularity.
c
   20    call q2mult(n, ngc, iadd, na, agc, lagc, lagct,
     *      qt, lqt, gamma, w(lw1))
         if(gamma .le. rteps) go to 30
         na = next
         index(next) = iadd
         istate(i) = - isi
c
c  add the new constraint to the basis and update the rtq factors.
c
         call addcon(n, na, ngc, iadd, .true., agc, lagc, lagct, el,
     *      lel, qt, lqt, w(lw1), w(lw2), gamma)
         if(na .eq. n) return
         next = next + 1
   30 continue
      return
c
c  end of subroutine QQgenc
c
      end
      subroutine lcimon(iupr, n, ngc, nngc, x, istate,
     *   ifmti, ifmtr, ifmt01, ifmt02, ifmt03, ifmt04)
      implicit double precision (a-h,o-z)
      integer iupr, n, ngc, nngc
      integer istate(nngc), ifmti(10), ifmtr(20), ifmt01(40),
     *   ifmt02(20), ifmt03(35), ifmt04(35)
      dimension x(n)
c
c  *********************************************************************
c
c  lcimon prints out information at the beginning of the linearly-
c  constrained minimization routines.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  september 1978
c
c  *********************************************************************
c
      integer i, iis, ilb, isi, iub
      call setfmt(ifmti, ifmtr, ifmt01, ifmt02, ifmt03, ifmt04)
      write(iupr, 2010)
      write(iupr, ifmt02) n
      write(iupr, 2020)
      write(iupr, ifmt02) ngc
      write(iupr, 2030)
      do 10 i = 1, n
         write(iupr, ifmt01) i, x(i)
   10 continue
      write(iupr, 2040)
      iis = ngc
      iub = n
      do 30 i = 1, n
         iis = iis + 1
         iub = iub + 1
         if(istate(iis) .eq. 0) go to 20
         write(iupr, ifmt03) i, i, iub
         go to 30
   20    write(iupr, ifmt03) i, i
         write(iupr, 2050)
   30 continue
      write(iupr, 2060) n, n
      if(ngc .eq. 0) go to 80
      ilb = n + n
      iub = ilb + ngc
      write(iupr, 2070)
      do 70 i = 1, ngc
         ilb = ilb + 1
         iub = iub + 1
         isi = istate(i)
         if(isi .ne. 0) go to 40
         write(iupr, ifmt03) i, ilb
         write(iupr, 2050)
         go to 70
   40    if(isi .ne. 1) go to 50
         write(iupr, ifmt03) i, ilb
         write(iupr, 2080)
         go to 70
   50    if(isi .ne. 2) go to 60
         write(iupr, ifmt04) i, iub
         write(iupr, 2090)
         go to 70
   60    write(iupr, ifmt03) i, ilb, iub
   70 continue
   80 write(iupr, 2100)
      call modfmt(ifmti, ifmtr, ifmt03, ifmt04)
      return
c
c  non-executable statements
c
c  the following are examples of the formats created for
c  the cdc 6000/7000 series.
c
c2001 format(4x, i5, 6x, 1pe22.14)
c2002 format(10(1x, i5))
c2003 format(14x, i5, 2(20x, i5))
c2004 format(14x, i5, 45x, i5)
 2010 format(49h0details of linear constraints and starting point/)
 2020 format(1h+, 6x, 21hindependent variables/)
 2030 format(1h+, 6x, 19hgeneral constraints// 3x, 8hvariable,
     *   9x, 14hstarting point)
 2040 format(43h0key to indices of constraints in print-out/
     *   1h0, 10x, 8hvariable, 3x, 22hindex for simple lower,
     *   3x, 22hindex for simple upper/ 25x,
     *   16hbound constraint, 9x, 16hbound constraint)
 2050 format(1h+, 60x, 8hequality)
 2060 format(28h0** for example - the index , i3,
     *   39h refers to the lower bound on variable , i3)
 2070 format(19h0general constraint, 4x, 21hindex for lower bound,
     *   4x, 21hindex for upper bound)
 2080 format(1h+, 64x, 4hnone)
 2090 format(1h+, 39x, 4hnone)
 2100 format(1h1)
c
c  end of subroutine lcimon
c
      end
      subroutine ldlast(n, na, qt, lqt, hesl, lh, hesd,
     *   ell, dd, w1, w2, ellmod)
      implicit double precision (a-h,o-z)
      integer n, na, lqt, lh
      logical ellmod
      dimension qt(lqt, n), hesl(lh), hesd(n),
     *   ell(lh), dd(n), w1(n), w2(n)
c
c  *********************************************************************
c
c  given a constant hessian matrix with its diagonal elements in hesd
c  and its subdiagonal elements stored by rows in hesl, ldlast computes
c  the final row of the cholesky factors of its projection in the null
c  space of active constraints when a constraint has been deleted from
c  the active set.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
      integer ileast
c
c  the following system function is used by ldlast
c
c
      ileast = n - na
      delta = x02aaf(delta)
      call symhx(n, lh, hesl, hesd, qt(1, na + 1), w1)
      call delphf(n, na, w1, qt, lqt, ileast, delta,
     *   ell, lh, dd, n, ellmod, w2)
      return
c
c  end of subroutine ldlast
c
      end
      subroutine minlof(n, ngc, nngc, lcon, ncon, rtol, agc, lagc,
     *   lngc, istate, iprint, mlevel, iupr, ifmt, lfmt, monit, iwa,
     *   iwb, maxit, epsmch, eps, f, x, ymod, feas, nviol, lmtest,
     *   umin, umax, is, nf, niter, npiter, na, nequal, nina, lzt, g,
     *   y, p, u, r, scale, qt, lqt, el, lel, index, nomove,
     *   nbschg, term, ifail, iw, liw, w, lw)
      implicit double precision (a-h,o-z)
      integer n, ngc, nngc, lcon, ncon, lagc, lngc, iprint, mlevel,
     *   iupr, lfmt, iwa, iwb, maxit, nviol, is, nf, niter, npiter,
     *   na, nequal, nina, lzt, lqt, lel, nbschg, ifail, liw, lw
      logical feas, lmtest, nomove, term
      integer istate(nngc), ifmt(lfmt), index(lcon), iw(liw)
      dimension rtol(lcon), agc(lagc, lngc),
     *   x(n), g(n), y(n), p(n), u(n), r(lcon), scale(lngc),
     *   qt(lqt, n), el(lel), w(lw)
c
c  *********************************************************************
c
c  minlof executes a single complete iteration of a non-simplex linear
c  programming algorithm using a projection of the negatived gradient
c  as the direction of search.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  september 1978
c
c  *********************************************************************
c
      integer ifmt1, ifmt2, ifmt3, ifmt4, ipflag, iv, kadd
      logical pickup
c
c  the following standard functions and system function are used
c  by minlof
c
      double precision dot, dsqrt
c
      ifmt1 = 1
      ifmt2 = ifmt1 + 40
      ifmt3 = ifmt2 + 20
      ifmt4 = ifmt3 + 35
      nina = ncon - na
      lzt = n - na
      term = .true.
      if(nomove) go to 10
      f = dot(n, x, g)
      nf = nf + 1
      nbschg = 0
      go to 20
c
c  check the number of basis changes.
c
   10 ifail = 3
      if(nbschg .ge. n + n) return
c
c  check the number of iterations.
c
   20 ifail = 2
      if(niter .ge. maxit) return
c
c  proceed with the next iteration.
c
      if(nomove) go to 30
c
c  compute the projected gradient and its euclidean norm ymod.
c
      yty = 0.0e+0
      if(lzt .gt. 0) call rvztg(n, na, qt, lqt, g, y, yty)
      ymod = dsqrt(yty)
   30 term = .false.
      ifail = 0
      niter = niter + 1
c
c  print out information at prescribed intervals.
c
      if(mlevel .eq. 0 .or. iprint .eq. 0) go to 50
      if(npiter .ne. iprint) go to 40
      npiter = 1
      call monit(mlevel, iupr, ifmt(ifmt1), ifmt(ifmt2), ifmt(ifmt3),
     *   ifmt(ifmt4), n, ngc, na, nequal, ncon, lcon, nngc, feas,
     *   nviol, niter, nf, f, x, ymod, umin, umax, is, u, r, scale,
     *   lngc, index, istate, 2, .false., 0.0e+0, .false., .false.,
     *   iw, liw, w, lw)
      go to 50
   40 npiter = npiter + 1
c
c  if the current x is either a feasible point or an infeasible
c  vertex, compute and test the lagrange multipliers for all the
c  inequalities.
c
   50 nomove = .true.
      lmtest = .false.
      is = 0
      ipflag = 0
      if(feas .or. na .eq. n) go to 60
c
c  when the projected gradient vanishes at an infeasible point, ysmall
c  tries to find a violated constraint which lies outside the subspace
c  spanned by the active constraints.  if successful it sets ipflag = 1.
c
      if(ymod .ge. epsmch) go to 90
      call ysmall(n, ngc, na, lzt, epsmch, r,
     *   ncon, rtol, index, agc, lagc, lngc, qt, lqt, w(iwa),
     *   iv, ipflag)
      if(ipflag .eq. 1) go to 90
   60 if(na .le. nequal) go to 70
      call lagmul(n, na, el, lel, g, qt, lqt, u, w(iwa))
      call chslm1(na, nequal, u, is, umax, umin)
      lmtest = .true.
      if(umin .gt. (- eps)) is = 0
      if(npiter .eq. 1 .and. mlevel .ge. 1) call monit(mlevel, iupr,
     *   ifmt(ifmt1), ifmt(ifmt2), ifmt(ifmt3), ifmt(ifmt4), n, ngc,
     *   na, nequal, ncon, lcon, nngc, feas, nviol, niter, nf, f, x,
     *   ymod, umin, umax, is, u, r, scale, lngc, index, istate,
     *   2, .true., 0.0e+0, .false., .false., iw, liw, w, lw)
c
c  delete the constraint with the negative multiplier of largest
c  modulus.
c
      if(umin .le. (- eps)) go to 80
c
c  if all the multipliers are positive, continue minimizing unless the
c  current point is a vertex or the objective function is linearly
c  dependent on na .lt. n active constraints.
c
   70 if(ymod .gt. 1.0e-2*eps*(1.0e+0 + dabs(f))) go to 90
      term = .true.
      return
c
c  delcon modifies the rtq factorization of the matrix of active
c  constraints.
c
   80 call delcon(n, na, is, el, lel, qt, lqt)
c
c  delprm updates the numbers of active and inactive constraints and of
c  columns in z and makes the necessary permutations in the lists of
c  constraints, residuals and tolerances.
c
      call delprm(na, nina, lzt, ncon, is, index, rtol, r)
c
c  delztg updates the projected gradient.
c
      call delztg(n, na + 1, lzt, qt, lqt, g, y, ymod)
      nbschg = nbschg + 1
      if(.not. feas) ipflag = - 1
c
c  pcomp computes the n-dimensional search direction vector and the
c  square of its euclidean norm.
c
   90 call pcomp(ipflag, n, na, lzt, epsmch, qt, lqt, y,
     *   w(iwa), p, ptp)
      pnorm = dsqrt(ptp) + epsmch
c
c  chkatp checks that the new search direction is not infeasible with
c  respect to the constraint just deleted.
c
      if(is .gt. 0) call chkatp(n, ngc, ncon, 1, index, agc,
     *   lagc, lngc, p, 1.0e-2*eps*pnorm, atp, ifail)
      if(ifail .eq. 0) go to 100
      ifail = 5
      lmtest = .false.
      term = .true.
      return
c
c  QQgetc computes alpha as the distance to the inactive constraint
c  with its residual indexed by kadd.
c
  100 alpha = pnorm
      stepmx = 1.0e+0/(epsmch*pnorm)
      call QQgetc(n, na, ngc, ncon, rtol, eps, stepmx, index,
     *   agc, lagc, lngc, p, r, kadd, alpha, feas,
     *   w(iwa), w(iwb))
      pickup = kadd .ne. 0
      if(feas .and. .not. pickup) ifail = 6
      nomove = alpha .eq. 0.0e+0
      if(nomove) go to 110
c
c  if the step length is non-zero, update the x values.  upresd uses
c  the inner products computed during QQgetc to update the residuals.
c
      call vecsum(n, alpha, p, x, x)
      call upresd(na, ncon, alpha, w(iwa), r)
      if(.not. pickup) return
c
c  if a new constraint is added, addprm updates the permutations of
c  constraints, residuals and tolerances...
c
  110 call addprm(na, nina, lzt, kadd, index, ncon, rtol, r)
c
c  ...addcon updates the rtq factorization of the matrix of active
c  constraints.
c
      call addcon(n, na, ngc, kadd, .false., agc, lagc, lngc, el,
     *   lel, qt, lqt, w(iwa), w(iwb), gamma)
      if(.not. nomove) return
c
c  if a zero step is taken, mdztg uses the givens matrices generated
c  by addcon to modify the projected gradient.
c
      call mdztg(n, na, w(iwa), w(iwb), y, ymod)
      nbschg = nbschg + 1
      return
c
c  end of subroutine minlof
c
      end
      subroutine QQsimc(n, ngc, nngc, ibnds, bl, bu, istate, agc, lagc,
     *   lagct, el, lel, qt, lqt, na, index, lin, rhs, w, lw)
      implicit double precision (a-h,o-z)
      integer n, ngc, nngc, lagc, lagct, lel, lqt, na, lin, lw
      integer ibnds(n), istate(nngc), index(lin)
      dimension bl(nngc), bu(nngc), agc(lagc, lagct),
     *   el(lel), qt(lqt, n), rhs(n), w(lw)
c
c  *********************************************************************
c
c  QQsimc scans the variables for any with a finite bound and adds the
c  relevant simple constraint to the active set by recording its index
c  in the vector index.  when both bounds are finite, a presetting of
c  ibnds(i), based on information about the objective function, may
c  optionally be used to determine the choice of bound for x(i).  the
c  subroutine next enters the appropriate right-hand side in the vector
c  rhs and calls the auxiliary routine addcon to update the rtq factors
c  of the matrix of active constraints.  for each added constraint,
c  ibnds(i) is set to - 1 if x(i) = bl(i) or + 1 if x(i) = bu(i).  if n
c  constraints have become active before the scanning is completed, the
c  subroutine returns immediately.
c  the length of the workspace vector w must be at least 2*n.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  september 1978
c
c  *********************************************************************
c
      integer i, iadd, ib, lw1, lw2, next
c
c  the following standard functions and system function are used
c  by QQsimc
c
      double precision dsqrt, x02aaf
c
c  a machine-dependent constant is set here.  epsmch is the smallest
c  positive double precision number such that 1.0 + epsmch .gt. 1.0.
c
      epsmch = x02aaf(epsmch)
      rteps = dsqrt(epsmch)
      lw1 = 1
      lw2 = lw1 + n
      next = na + 1
      ib = ngc
      do 50 i = 1, n
         ib = ib + 1
         if(istate(ib) .eq. 0) go to 50
         bli = bl(ib)
         bui = bu(ib)
         absbli = dabs(bli)
         absbui = dabs(bui)
         if(absbli .ge. 1.0e+3) go to 10
         if(ibnds(i) .lt. 0 .and. absbui .lt. 1.0e+3) go to 20
         iadd = i
         rhs(next) = bli
         ibnds(i) = - 1
         go to 30
   10    if(absbui .lt. 1.0e+3) go to 20
         ibnds(i) = 0
         go to 50
   20    iadd = n + i
         rhs(next) = - bui
         ibnds(i) = 1
c
c  check the basis for singularity.
c
   30    call q2mult(n, ngc, iadd, na, agc, lagc, lagct,
     *      qt, lqt, gamma, w(lw1))
         if(gamma .gt. rteps) go to 40
         ibnds(i) = 2
         go to 50
c
c  add the new constraint to the basis and update the rtq factors.
c
   40    na = next
         index(next) = iadd
         call addcon(n, na, ngc, iadd, .true., agc, lagc, lagct, el,
     *      lel, qt, lqt, w(lw1), w(lw2), gamma)
         if(na .eq. n) return
         next = next + 1
   50 continue
      return
c
c  end of subroutine QQsimc
c
      end
      subroutine addcon(n, na, ngc, iadd, vknown, agc, lagc, lagct,
     *   el, lel, qt, lqt, v, u, gamma)
      implicit double precision (a-h,o-z)
      integer n, na, ngc, lagc, lagct, lel, lqt, iadd
      logical vknown
      dimension agc(lagc, lagct), el(lel), qt(lqt,n),
     *   v(n), u(n)
c
c  *********************************************************************
c
c  this subroutine is intended for use in the linearly constrained
c  minimization of functions with constant hessians.  it is called,
c  whenever a new constraint has been added, to update the rtq factors
c  of the matrix of active constraints by means of givens transform-
c  ations.  these are stored as sine/cosine pairs in the vectors v and
c  u for subsequent use if required.
c
c  philip e gill, walter murray, susan m picken, enid m r long
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  september 1978
c
c  *********************************************************************
c
      integer i, igclb, igcub, iq1, ixub, j, k, l, lnewq1, lnewq2,
     *   loldq1, loldq2, m, mplus
c
c  the following standard and library functions are called in addcon
c
      double precision dot, dsqrt, x02aaf
c
c  a machine-dependent constant is set here.  epsmch is the smallest
c  positive double precision number such 1.0 + epsmch .gt. epsmch.
c
      epsmch = x02aaf(epsmch)
c
c  set parameters defining lengths of vectors.
c
      lnewq1 = na
      lnewq2 = n - lnewq1
      loldq1 = lnewq1 - 1
      loldq2 = lnewq2 + 1
c
c  determine the type of the new constraint.
c
      ixub = iadd - n
      igclb = ixub - n
      igcub = igclb - ngc
c
c  if it has not already been done, call q2mult
c  to premultiply the new constraint by the old q2
c  and to calculate the euclidean length of the resulting vector v.
c
      if(.not. vknown) call q2mult(n, ngc, iadd, loldq1, agc, lagc,
     *   lagct, qt, lqt, gamma, v)
      k = lnewq1*loldq1/2
      if(loldq1 .eq. 0) go to 60
c
c  to update the r factor, premultiply the
c  new constraint by the old q1 and store the resulting vector in the
c  next vacant positions of el.
c
      do 50 iq1 = 1, loldq1
         if(iadd .gt. n) go to 10
         sum = qt(iadd,iq1)
         go to 40
   10    if(igclb .gt. 0) go to 20
         sum = - qt(ixub,iq1)
         go to 40
   20    if(igcub .gt. 0) go to 30
         sum = dot(n, agc(1, igclb), qt(1, iq1))
         go to 40
   30    sum = - dot(n, agc(1, igcub), qt(1, iq1))
   40    l = k + iq1
         el(l) = sum
   50 continue
c
c  complete the updating of the r factor by entering the euclidean
c  norm of v in the final position of the current el.
c
   60 l = k + lnewq1
      el(l) = gamma
c
c  form symmetric givens matrices, rotating in the planes (i, i + 1),
c  i = lnewq2, ..., 2, 1, to reduce the last lnewq2 elements of v to
c  zero.
c
      vi = v(loldq2)
      if(lnewq1 .lt. n) go to 80
c
c  special case where n constraints are now active.
c
      if(vi .ge. 0.0e+0) return
      do 70 j = 1, n
         qt(j,n) = - qt(j,n)
   70 continue
      return
   80 do 120 l = 1, lnewq2
         i = loldq2 - l
         viplus = vi
         vi = v(i)
         if(dabs(viplus) .ge. epsmch .or. dabs(vi) .ge. epsmch) go to 90
         cs = 1.0e+0
         if(vi .lt. 0.0e+0) cs = - 1.0e+0
         sn = 0.0e+0
         go to 100
   90    d = dsqrt(viplus*viplus + vi*vi)
         sn = 1.0e+0/d
         cs = vi*sn
         sn = viplus*sn
         vi = d
  100    v(i) = sn
         u(i) = cs
c
c  update the qt matrix by applying the givens transformations to the
c  last loldq2 rows.
c
         m = n - l
         mplus = m + 1
         do 110 j = 1, n
            zi = qt(j,m)
            ziplus = qt(j,mplus)
            qt(j,mplus) = sn*zi - cs*ziplus
            qt(j,m) = cs*zi + sn*ziplus
  110    continue
  120 continue
      return
c
c  end of subroutine addcon
c
      end
      subroutine chqpin(n, ngc, iac, ibound, liwmin, lwmin,
     *   liw, lw, lcon, lagc, lngc, lqt, lel, lh, maxit,
     *   bl, bu, lb, istate, ifail)
      implicit double precision (a-h,o-z)
      integer n, ngc, iac, ibound, liwmin, lwmin, liw, lw, lcon,
     *   lagc, lngc, lqt, lel, lh, maxit, lb, ifail
      integer istate(lngc)
      dimension bl(lb), bu(lb)
c
c  *********************************************************************
c
c  chqpin checks the input parameters for the linearly constrained
c  quadratic programming routine.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  july 1978
c
c  *********************************************************************
c
      integer lhmin
      ifail = 1
c
c  check the length of the vectors for the hessian matrix.
c
      lhmin = n*(n - 1)/2
      if(lh .le. 0 .or. lh .lt. lhmin) return
      ifail = 0
c
c  chlpin checks the input parameters common to all the linearly
c  constrained routines and, if requested by the user, sets the
c  bounds vectors for the simple constraints.
c
      call chlpin(n, ngc, iac, ibound, liwmin, lwmin, liw, lw, lcon,
     *   lagc, lngc, lqt, lel, maxit, bl, bu, lb, istate, ifail)
      return
c
c  end of subroutine chqpin
c
      end
      subroutine delphf(n, na, hz, qt, lqt, nsphi, delta, ell, lell,
     *   dd, ldd, ellmod, w)
      implicit double precision (a-h,o-z)
      integer n, na, lqt, nsphi, lell, ldd
      logical ellmod
      dimension hz(n), qt(lqt, n), ell(lell), dd(ldd),
     *   w(n)
c
c  *********************************************************************
c
c  this subroutine updates the factors of a projected hessian matrix
c  when a constraint is deleted from the active set.
c  the factors updated are la and da, where ha = zt*h*z
c  = la*da*lat. the unit lower triangular matrix la is stored row by row
c  in ell(i), i = 1,2,...,(n - na)*(n - na - 1)/2 and the diagonal
c  elements of the diagonal matrix da are stored in dd(i),
c  i = 1,2,...,n - na. the matrix z is
c  given by the last n - na columns of the n*n orthogonal matrix qt in
c  reverse order. the vector h*znew is input in hz, where znew is the
c  column (na + 1) of the matrix qt and h is the hessian matrix. hz is
c  overwritten. the variables delta and nsphi given by the subroutine
c  frmchl are also updated. the new element dd(n - na) is not modified
c  if it is small or negative. this may be done by calling subroutine
c  dpos.
c  if the new element of dd is going to be negative the elements
c  of the new row in ell may be modified to ensure they are not
c  too large. if this modification has been done ellmod is set
c  true.
c  workspace w of length n is used in this subroutine.
c  lqt = n, lell = (n - na)*(n - na - 1)/2, ldd = n - na.
c
c  philip e. gill, walter murray, susan m. picken, enid m. r. long
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  september 1978
c
c  *********************************************************************
c
      integer i, i1, i2, j, k
c
c  the following standard and library functions are called
c
      double precision dsqrt, x02aaf
      ellmod = .false.
      epsmch = x02aaf(epsmch)
      del = delta
      i2 = n - na
      call rvztg(n, na, qt, lqt, hz, w, temp)
      do 10 i = 1, i2
         temp = w(i)
         temp = dabs(temp)*epsmch
         if(temp .gt. del) del = temp
   10 continue
      sum = w(i2)
      zthz2 = 2.0e+0*dabs(sum)
      if(i2 .eq. 1) goto 40
      k = i2 - 1
      call lsol(k, lell, ell, w, hz)
      j = k*(k - 1)/2
      do 30 i = 1, k
         ddi = dd(i)
         temp = hz(i)/ddi
         i1 = i + j
         if(temp*temp*ddi .lt. zthz2 + 1.0e+1) goto 20
         temp2 = dsqrt((zthz2 + 1.0e+1)/ddi)
         if(temp .lt. 0.0e+0) temp2 = - temp2
         temp = temp2
         ellmod = .true.
   20    sum = sum - temp*temp*ddi
         ell(i1) = temp
   30 continue
   40 dd(i2) = sum
      if(sum .le. del) nsphi = i2
      delta = del
      return
c
c  end of subroutine delphf
c
      end
      subroutine setfmt(ifmti, ifmtr, ifmt01, ifmt02, ifmt03, ifmt04)
      implicit double precision (a-h,o-z)
      integer ifmti(10), ifmtr(20),
     *   ifmt01(40), ifmt02(20), ifmt03(35), ifmt04(35)
c
c  *********************************************************************
c
c  subroutine setfmt builds the format arrays ifmt01, ifmt02, ifmt03
c  and ifmt04 from the standard integer format array (sifa) and standard
c  double precision format array (srfa) assigned in fieldi and fieldr respective
c
c  the format arrays contain the following hollerith text-
c        ifmt01             (3x,sifa,2x,srfa)
c        ifmt02             (10(sifa))
c        ifmt03             (13x,sifa,2(19x,sifa))
c        ifmt04             (13x,sifa,44x,sifa)
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england.
c  crown copyright reserved
c  september 1978
c
c  *********************************************************************
c
      integer i, iblank, icomma, ilpar, irpar, itemp, ix,
     *   i0, i1, i2, i3, i4, i9
      data iblank /1h /
      data icomma /1h,/
      data ilpar  /1h(/
      data irpar  /1h)/
      data ix     /1hx/
      data i0     /1h0/
      data i1     /1h1/
      data i2     /1h2/
      data i3     /1h3/
      data i4     /1h4/
      data i9     /1h9/
      ifmt01(1) = ilpar
      ifmt01(2) = i3
      ifmt01(3) = ix
      ifmt01(4) = icomma
      ifmt01(15) = icomma
      ifmt01(16) = i2
      ifmt01(17) = ix
      ifmt01(18) = icomma
      ifmt01(39) = irpar
      ifmt01(40) = iblank
      ifmt02(1) = ilpar
      ifmt02(2) = i1
      ifmt02(3) = i0
      ifmt02(4) = ilpar
      ifmt02(15) = irpar
      ifmt02(16) = irpar
      ifmt03(1) = ilpar
      ifmt03(2) = i1
      ifmt03(3) = i3
      ifmt03(4) = ix
      ifmt03(5) = icomma
      ifmt03(16) = icomma
      ifmt03(17) = i2
      ifmt03(18) = ilpar
      ifmt03(19) = i1
      ifmt03(20) = i9
      ifmt03(21) = ix
      ifmt03(22) = icomma
      ifmt03(33) = irpar
      ifmt03(34) = irpar
      ifmt03(35) = iblank
      ifmt04(1) = ilpar
      ifmt04(2) = i1
      ifmt04(3) = i3
      ifmt04(4) = ix
      ifmt04(5) = icomma
      ifmt04(16) = icomma
      ifmt04(17) = i4
      ifmt04(18) = i4
      ifmt04(19) = ix
      ifmt04(20) = icomma
      ifmt04(31) = irpar
      call fieldi(ifmti)
      call fieldr(ifmtr)
      do 10 i = 1, 10
         itemp = ifmti(i)
         ifmt01(i + 4) = itemp
         ifmt02(i + 4) = itemp
         ifmt03(i + 5) = itemp
         ifmt03(i + 22) = itemp
         ifmt04(i + 5) = itemp
         ifmt04(i + 20) = itemp
   10 continue
      do 20 i = 1, 20
         ifmt01(i + 18) = ifmtr(i)
   20 continue
      call QQblan(4, ifmt02(17))
      call QQblan(4, ifmt04(32))
      return
c
c  end of subroutine setfmt
c
      end
      subroutine QQshiv(n, na, x, qt, lqt, s, v, w)
      implicit double precision (a-h,o-z)
      integer n, na, lqt
      dimension x(n), qt(lqt, n), s(n),
     *   v(n), w(n)
c
c  *********************************************************************
c
c  QQshiv is called, when na .lt. n and an initial point x has been
c  supplied by the user, to compute and add to the vector v obtained by
c  QQfinv the vector s = z.zt.x which gives v + s as the nearest
c  feasible point to x.  on exit v + s is stored in x.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  october 1977
c
c  *********************************************************************
c
      integer lzt
      if(na .ge. n) return
      lzt = n - na
      scalar = 1.0e+0
c
c  rvztg computes w = zt.x
c
      call rvztg(n, na, qt, lqt, x, w, wtw)
c
c  zpacmp computes s = z.(zt.x)
c
      call zpacmp(n, lzt, scalar, qt, lqt, w, s, sts)
c
c  vecsum stores v + s in x
c
      call vecsum(n, scalar, v, s, x)
      return
c
c  end of subroutine QQshiv
c
      end
      subroutine ysmall(n, ngc, na, lzt, eps, r, ncon, rtol, index,
     *   agc, lagc, lagct, qt, lqt, v, iv, iflag)
      implicit double precision (a-h,o-z)
      integer n, ngc, na, lzt, ncon, lagc, lagct, lqt, iv, iflag
      integer index(ncon)
      dimension r(ncon), rtol(ncon), agc(lagc, lagct),
     *   qt(lqt, n), v(n)
c
c  *********************************************************************
c
c  when the projected gradient vanishes at an infeasible point, ysmall
c  scans the violated constraints for one which does not lie in the
c  subspace spanned by the active constraints.  if it finds one, it sets
c  iflag = 1 and returns to the calling routine immediately with the
c  non-zero projection of the violated constraint in v.  otherwise it
c  returns with iflag = - 1 and v reset to the zero vector.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
      integer i, ia, ic, ii, iq, j, k, l, nina, n2, n2ngc
c
c  the following system function and standard function are used in
c  ysmall
c
c
      iflag = 0
      test = 0.0e+0
      nina = ncon - na
      n2 = n + n
      n2ngc = n2 + ngc
      ii = na
      do 70 i = 1, nina
         ii = ii + 1
         if(r(i) .gt. (- rtol(ii))) go to 70
         ic = index(ii)
         if(ic .gt. n) go to 20
c
c  simple constraint with lower bound.
c
         iq = ic
         k = na
         l = lzt + 1
         do 10 j = 1, lzt
            k = k + 1
            l = l - 1
            v(l) = qt(iq, k)
   10    continue
         test = QQrnor(lzt, v)
         go to 60
   20    if(ic .gt. n2) go to 40
c
c  simple constraint with upper bound.
c
         iq = ic - n
         k = na
         l = lzt + 1
         do 30 j = 1, lzt
            k = k + 1
            l = l - 1
            v(l) = - qt(iq, k)
   30    continue
         test = QQrnor(lzt, v)
         go to 60
   40    if(ic .gt. n2ngc) go to 50
c
c  general constraint with lower bound.
c
         ia = ic - n2
         call rvztg(n, na, qt, lqt, agc(1, ia), v, test)
         test = dsqrt(test)
         go to 60
c
c  general constraint with upper bound.
c
   50    ia = ic - n2ngc
         call rvztg(n, na, qt, lqt, agc(1, ia), v, test)
         call negvec(lzt, v)
         test = dsqrt(test)
   60    if(test .gt. eps) go to 80
   70 continue
c
c  the projections of the violated constraints are uniformly zero.
c
      iflag = - 1
      call zerovc(lzt, v)
      return
c
c  iv indexes the violated constraint with non-zero projection.
c
   80 iv = ic
      iflag = 1
      return
c
c  end of subroutine ysmall
c
      end
      subroutine allres(n, ngc, lngc, istate, agc, lagc, bl, bu, lb,
     *   x, r, lr)
      implicit double precision (a-h,o-z)
      integer n, ngc, lngc, lagc, lb, lr
      integer istate(lngc)
      dimension agc(lagc, lngc), bl(lb), bu(lb), x(n), r(lr)
c
c  ********************************************************************
c
c  forms the residuals of all the constraints at the point x and
c  stores them in r(i), i=1,2,...,2*n+2*ngc.
c  lngc = ngc or 1 if ngc = 0, lagc = n. lb = ngc + n.
c
c  philip e. gill, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright reserved
c  february 1976
c
c  *********************************************************************
c
      integer k, k1, k2, k3, k4
c
c  the following function is called
c
      do 10 k = 1, n
         k1 = ngc + k
         r(k) = x(k) - bl(k1)
   10 continue
      k1 = n + 1
      k2 = n + n
      do 20 k = k1, k2
         k3 = k - n
         k4 = ngc + k3
         r(k) = - x(k3) + bu(k4)
   20 continue
      k1 = k2 + 1
      k2 = k2 + ngc
      if(k2 .lt. k1) return
      do 40 k = k1, k2
         k3 = k - n - n
         if(istate(k3) .eq. 2) goto 30
         sum = dot(n, agc(1, k3), x)
         sum = sum - bl(k3)
         r(k) = sum
         goto 40
   30    r(k) = 0.0e+0
   40 continue
      k1 = k2 + 1
      k2 = k2 + ngc
      do 60 k = k1, k2
         k3 = k - k1 + 1
         if(istate(k3) .eq. 1) goto 50
         sum = dot(n, agc(1, k3), x)
         sum = bu(k3) - sum
         r(k) = sum
         goto 60
   50    r(k) = 0.0e+0
   60 continue
      return
c
c  end of subroutine allres
c
      end
      subroutine chkatp(n, ngc, m, numdel, index, agc, lagc,
     *   lagct, p, tol, atpdel, ifail)
      implicit double precision (a-h,o-z)
      integer n, ngc, m, numdel, lagc, lagct, ifail
      integer index(m)
      dimension agc(lagc, lagct), p(n)
c
c  *********************************************************************
c
c  checks that the inner product of the constraint(s) just deleted
c  with the direction of search p is positive or sufficiently small.
c  set ifail = 1 if not.
c
c  philip e. gill, walter murray, susan m. picken, enid m. r. long
c  d.n.a.c., national physical laboratory, england.
c  crown copyright
c  september 1977
c
c  *********************************************************************
c
      integer i, i2, i3, i4
c
c  the following function is called
c
      ifail = 0
      do 50 i = 1, numdel
         i2 = m - numdel + 1
         i3 = index(i2)
         sum = 0.0e+0
         if(i3 .gt. n) goto 10
         sum = p(i3)
         goto 40
   10    if(i3 .gt. n + n) goto 20
         i4 = i3 - n
         sum = - p(i4)
         goto 40
   20    if(i3 .gt. n + n + ngc) goto 30
         i4 = i3 - n - n
         sum = dot(n, agc(1, i4), p)
         goto 40
   30    i4 = i3 - n - n - ngc
         sum = dot(n, agc(1, i4), p)
         sum = - sum
   40    if(sum .lt. (- tol)) ifail = 1
         atpdel = sum
   50 continue
      return
c
c  end of subroutine chkatp
c
      end
      subroutine chlpin(n, ngc, iac, ibound, liwmin, lwmin,
     *   liw, lw, lcon, lagc, lngc, lqt, lel, maxit,
     *   bl, bu, lb, istate, ifail)
      implicit double precision (a-h,o-z)
      integer n, ngc, iac, ibound, liwmin, lwmin, liw, lw,
     *   lcon, lagc, lngc, lqt, lel, maxit, lb, ifail
      integer istate(lngc)
      dimension bl(lb), bu(lb)
c
c  *********************************************************************
c
c  chlpin checks the input parameters common to all the linearly
c  constrained routines and, if requested by the user, sets the
c  bounds vectors for the simple constraints.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  january 1978
c
c  *********************************************************************
c
      integer i, isi, lbmin, lelmin, m
      ifail = 1
c
c  check the control parameters and the array dimensions.
c
      lbmin = n + ngc
      m = lbmin + lbmin
      lelmin = n*(n + 1)/2
      if(n .lt. 1 .or. ngc .lt. 0 .or. iac .lt. 0 .or. iac .ge. 4 .or.
     *   ibound .lt. 0 .or. ibound .ge. 4 .or. liw .lt. liwmin .or.
     *   lw .lt. lwmin .or. lcon .lt. m .or. lagc .lt. 1 .or.
     *   (lagc .lt. n .and. ngc .gt. 0) .or. lngc .lt. ngc .or.
     *   lngc .lt. 1 .or. lqt .lt. n .or. lel .lt. lelmin .or.
     *   lb .lt. lbmin .or. maxit .lt. 1) return
      if(ngc .eq. 0) go to 20
      do 10 i = 1, ngc
         isi = istate(i)
         if(isi .lt. 0 .or. isi .ge. 4) return
   10 continue
   20 ifail = 0
c
c  setbnd sets the bounds vectors for the simple constraints.
c
      if(ibound .gt. 0) call setbnd(n, ibound, 1.0e+6, bl(ngc + 1),
     *   bu(ngc + 1))
c
c  chkbnd checks the relationship between the lower and the upper
c  bound wherever this is relevant.
c
      call chkbnd(n, ngc, bl, bu, lb, istate, lngc, ifail)
      return
c
c  end of subroutine chlpin
c
      end
      subroutine delcon(n, na, is, el, lel, qt, lqt)
      implicit double precision (a-h,o-z)
      integer n, na, is, lel, lqt
      dimension el(lel), qt(lqt, n)
c
c  ******************************************************************
c
c  this subroutine modifies, by means of givens transformations,
c  l and q due to the deletion of row is from the matrix of active
c  constraints, at, where q*a = (l 0)t (t denotes transpose).  the
c  new lower-triangular matrix l has (na - 1) rows and is stored row
c  by row in the array el(i), i=1,2,...,na*(na-1)/2, and the transpose
c  of the orthogonal matrix q is stored in the array qt.
c
c  philip e. gill, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory,england.
c  crown copyright reserved
c  february 1976
c
c  ********************************************************************
c
      integer i, i1, ip1, j, k, k1, km1, km2, m, mp1, nam1, nam2
c
c  the following standard function is called within delcon.
c
c
      k = is*(is + 1)/2 + 1
      nam1 = na - 1
      nam2 = na - 2
      if(nam1 .lt. is) return
      do 50 m = is, nam1
         if(is .lt. 2) goto 20
         do 10 j = 2, is
            k1 = k - m
            el(k1) = el(k)
            k = k + 1
   10    continue
   20    km2 = k + m - is
         km1 = km2 + 1
         k = km1 + 1
         el1 = el(km2)
         el2 = el(km1)
         cs = 1.0e+0/dsqrt(el1*el1 + el2*el2)
         sn = el2*cs
         cs = el1*cs
         k1 = km2 - m
         el(k1) = cs*el1 + sn*el2
         mp1 = m + 1
         do 30 i = 1, n
            q1 = qt(i, m)
            q2 = qt(i, mp1)
            qt(i, m) = cs*q1 + sn*q2
            qt(i, mp1) = sn*q1 - cs*q2
   30    continue
         ip1 = km1
         if(nam2 .lt. m) goto 50
         do 40 j = m, nam2
            i = ip1 + j
            ip1 = i + 1
            el1 = el(i)
            el2 = el(ip1)
            i1 = i - 1 - j
            el(i1) = cs*el1 + sn*el2
            el(ip1) = sn*el1 - cs*el2
   40    continue
   50 continue
      return
c
c  end of subroutine delcon
c
      end
      subroutine delztg(n, naplus, lzt, qt, lqt, g, y, ymod)
      implicit double precision (a-h,o-z)
      integer n, naplus, lzt, lqt
      dimension qt(lqt,n), g(n), y(n)
c
c  *********************************************************************
c
c  delztg updates the projected gradient and its norm when a constraint
c  has been deleted.
c
c  philip e gill, walter murray, susan m picken, enid m r long
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
c
c  the following standard and library functions are called by delztg
c
c
      sum = dot(n, qt(1, naplus), g)
      y(lzt) = sum
      ymod = dsqrt(ymod*ymod + sum*sum)
      return
c
c  end of subroutine delztg
c
      end
      subroutine fieldi(ifmti)
      implicit double precision (a-h,o-z)
      integer ifmti(10)
c
c  *********************************************************************
c
c  fieldi is a machine-dependent subroutine in which the standard
c  integer format array is assigned to the integer array ifmti(j),
c  j=1(1)10.  all unused elements of ifmti must be filled with blanks.
c
c                        important
c                        *********
c
c  the total field width defined in the format array ifmti must be
c  exactly equal to six.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england.
c  july 1978
c
c  *********************************************************************
c
      integer iblank, icomma, ii, ix, i1, i5, j
c
c  the following data statements assign the hollerith constants which
c  make up the format string
c                   1x,i5
c  for the cdc 6000/7000 series.
c
      data icomma /1h,/
      data ii     /1hi/
      data ix     /1hx/
      data i1     /1h1/
      data i5     /1h5/
c
      data iblank /1h /
c
c  the following statements assign the format string
c                         1x,i5
c  for the cdc 6000/7000 series.
c
      ifmti(1) = i1
      ifmti(2) = ix
      ifmti(3) = icomma
      ifmti(4) = ii
      ifmti(5) = i5
      do 10 j = 6, 10
         ifmti(j) = iblank
   10 continue
      return
c
c  end of subroutine fieldi
c
      end
      subroutine fieldr(ifmtr)
      implicit double precision (a-h,o-z)
      integer ifmtr(20)
c
c  *********************************************************************
c
c  fieldr is a machine-dependent subroutine in which the standard double precisi
c  format array is assigned to the integer array fmtr(j), j=1(1)20.
c  all unused elements of ifmtr must be filled with blanks.
c
c                        important
c                        *********
c
c  the total field width defined in the format array ifmtr must be
c  exactly equal to 30.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england.
c  july 1978
c
c  *********************************************************************
c
      integer iblank, icomma, ie, ip, ipoint, ix, i1, i2, i4, j
c
c  the following data statements assign the hollerith constants which
c  make up the format string
c                   4x,1pe22.14,4x
c  for the cdc 6000/7000 series
c
      data icomma /1h,/
      data ie     /1he/
      data ip     /1hp/
      data ipoint /1h./
      data ix     /1hx/
      data i1     /1h1/
      data i2     /1h2/
      data i4     /1h4/
c
      data iblank /1h /
c
c  the following statements assign the format string
c                         4x,1pe22.14,4x
c  for the cdc 6000/7000 series
c
      ifmtr(1) = i4
      ifmtr(2) = ix
      ifmtr(3) = icomma
      ifmtr(4) = i1
      ifmtr(5) = ip
      ifmtr(6) = ie
      ifmtr(7) = i2
      ifmtr(8) = i2
      ifmtr(9) = ipoint
      ifmtr(10) = i1
      ifmtr(11) = i4
      ifmtr(12) = icomma
      ifmtr(13) = i4
      ifmtr(14) = ix
      do 10 j = 15, 20
         ifmtr(j) = iblank
   10 continue
      return
c
c  end of subroutine fieldr
c
      end
      subroutine QQfinv(n, na, el, lel, qt, lqt, rhs, s, v)
      implicit double precision (a-h,o-z)
      integer n, na, lel, lqt
      dimension el(lel), qt(lqt, n), rhs(na), s(na), v(n)
c
c  *********************************************************************
c
c  QQfinv finds v, the vector of least length which satisfies the na
c  equations rt.q.v = rhs, where rt is a lower-triangular matrix and q
c  is an na*n matrix such that q.qt = i.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  october 1977
c
c  *********************************************************************
c
c  rtsol solves rt.s = rhs for s
c
      call rtsol(na, lel, el, rhs, s)
c
c  ax computes v = qt.s
c
      call ax(n, na, qt, lqt, s, v)
      return
c
c  end of subroutine QQfinv
c
      end
      subroutine QQgetc(n, na, ngc, ncon, rtol, eps, stepmx,
     *   index, agc, lagc, lagct, p, r, kadd, rlamk,
     *   feas, v, rlam)
      implicit double precision (a-h,o-z)
      integer n, na, ngc, ncon, lagc, lagct, kadd
      logical feas
      integer index(ncon)
      dimension rtol(ncon), agc(lagc, lagct),
     *   p(n), r(ncon), v(ncon), rlam(ncon)
c
c  *********************************************************************
c
c  to determine which new constraint should become active, QQgetc
c  computes, for every member of the active set, rlam(i) as the
c  ratio of - r(i), the negative of its residual, and v(i), the inner
c  product of its normal with the search direction vector p.  the
c  columns of agc which give the inactive constraints are identified
c  by the elements index(i), i = na + 1, na + 2, ..., ncon.  on entry
c  rlamk contains norm(p).  on exit it returns rlam(kadd) as the length
c  of the step to be taken along the vector p.  if there is no inactive
c  constraint in the positive p direction, then rlamk = stepmx.  when
c  rlamk .lt. stepmx, kadd returns the index of the constraint to be
c  added to the active set.  if at the current point there are some
c  constraints with small residuals which have not been included in the
c  active set (zres = true) and if moving in the direction p would
c  violate any of them, then kadd will correspond to one of these and
c  rlamk will be set to zero.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  september 1978
c
c  *********************************************************************
c
      integer i, ic, igenlb, igenub, ii, infeas, isimub,
     *   kmax, kmin, nina, n2, n2ngc
      logical zr
c
c  the following standard function and system function are called
c  within QQgetc
c
      double precision dot
c
      nina = ncon - na
      kadd = 0
      if(nina .ne. 0) go to 10
      rlamk = stepmx
      return
   10 kmin = 0
      kmax = 0
      rlkmin = 0.0e+0
      rlkmax = 0.0e+0
      atpmin = 1.0e+0
      pnminv = 1.0e+0/rlamk
      n2 = n + n
      n2ngc = n2 + ngc
c
c  compute the inner products of the normals of the inactive
c  constraints with the search direction vector p.
c
      ii = na
      do 100 i = 1, nina
         ii = ii + 1
         ic = index(ii)
c
c  simple constraints with lower bounds.
c
         if(ic .gt. n) go to 20
         atp = p(ic)
         go to 50
c
c  simple constraints with upper bounds.
c
   20    if(ic .gt. n2) go to 30
         isimub = ic - n
         atp = - p(isimub)
         go to 50
c
c  general constraints with lower bounds.
c
   30    if(ic .gt. n2ngc) go to 40
         igenlb = ic - n2
         atp = dot(n, agc(1, igenlb), p)
         go to 50
c
c  general constraints with upper bounds.
c
   40    igenub = ic - n2ngc
         atp = - dot(n, agc(1, igenub), p)
c
c  prepare to compute and compare the ratios.
c
   50    atpnmd = atp*pnminv
         res = r(i)
         absres = dabs(res)
c
c  set rlam(i) = - 1.0 whenever the i-th ratio is infinite.
c
         if(absres .lt. stepmx*dabs(atp)) go to 60
         rlam(i) = - 1.0e+0
         go to 70
c
c  compute the finite ratios.
c
   60    rlam(i) = - res/atp
c
c  locate the constraints with zero residuals which were not included
c  in the previous active set.
c
   70    tol = rtol(ii)
c
c  find the inequality with zero residual which would be most seriously
c  violated by a non-zero step along p.
c
         if((feas .and. res .ge. tol) .or. (.not. feas .and. absres
     *      .ge. tol)) go to 90
         if(atpnmd .ge. atpmin) go to 80
         atpmin = atpnmd
         kmin = i
   80    rlam(i) = - 1.0e+0
   90    v(i) = atp
  100 continue
c
c  take a zero step when necessary.
c
      zr = atpmin .lt. (- 1.0e-2*eps)
      if(zr) go to 150
c
c  determine the new active constraint and the distance to it.  first
c  look for the nearest satisfied constraint and count the number of
c  violated constraints in the direction p.
c
      kmin = 0
      infeas = 0
      ii = na
      do 130 i = 1, nina
         ii = ii + 1
         ratio = rlam(i)
         if(ratio .le. 0.0e+0) go to 130
         if(r(i) .le. (- rtol(ii))) go to 120
         if(kmin .ne. 0) go to 110
         rlkmin = ratio
         kmin = i
         go to 130
  110    if(ratio .ge. rlkmin) go to 130
         rlkmin = ratio
         kmin = i
         go to 130
  120    infeas = infeas + 1
  130 continue
      if(kmin .eq. 0) rlkmin = stepmx
c
c  in the case of infeasibility, look for the furthest violated
c  constraint that does not lie beyond the nearest satisfied one.
c
      if(infeas .eq. 0) go to 150
      ii = na
      do 140 i = 1, nina
         ii = ii + 1
         ratio = rlam(i)
         if(ratio .gt. rlkmin .or. ratio .lt. rlkmax) go to 140
         if(r(i) .gt. (- rtol(ii))) go to 140
         rlkmax = ratio
         kmax = i
  140 continue
      if(kmax .eq. 0) go to 150
      kadd = kmax
      rlamk = rlkmax
      return
  150 kadd = kmin
      rlamk = rlkmin
      return
c
c  end of subroutine QQgetc
c
      end
      subroutine inacrs(n, na, ngc, m, rtol, index,
     *   agc, lagc, lagct, bl, bu, lb, x, r)
      implicit double precision (a-h,o-z)
      integer n, na, ngc, m, lagc, lagct, lb
      integer index(m)
      dimension rtol(m), agc(lagc, lagct), bl(lb), bu(lb),
     *   x(n), r(m)
c
c  *********************************************************************
c
c  forms the residuals corresponding to the inactive constraints at
c  the point x and stores them in r(i), i=1,2,...,m - na.
c  lagc = n, lagct = ngc or 1 if ngc = 0.
c  lb = ngc + n.
c
c  philip e. gill, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright reserved
c  february 1976
c
c  *********************************************************************
c
      integer i, i2, i4, i5, i6, i7, i8, ina, mna
c
c  the following standard and library functions are called in inacrs
c
c
      mna = m - na
      do 30 i = 1, mna
         i2 = i + na
         i2 = index (i2)
         i4 = i2 - 2*n
         i5 = i4 - ngc
         i6 = i2 + ngc
         i7 = i2 - n
         i8 = i7 + ngc
         if(i2 .le. n) temp = x(i2) - bl(i6)
         if(i2 .gt. n .and. i4 .le. 0) temp = - x(i7) + bu(i8)
         if(i4 .le. 0) goto 20
         if(i5 .gt. 0) goto 10
         temp = dot(n, agc(1, i4), x)
         temp = temp - bl(i4)
          goto 20
   10    temp = dot(n, agc(1, i5), x)
         temp = bu(i5) - temp
   20    r(i) = temp
         ina = i + na
   30 continue
      return
c
c  end of subroutine inacrs
c
      end
      subroutine lagmul(n, na, el, lel, g, qt, lqt, rlag, w)
      implicit double precision (a-h,o-z)
      integer n, na, lel, lqt
      dimension el(lel), g(n), qt(lqt, n), rlag(na), w(na)
c
c  *********************************************************************
c
c  forms lagrange multipliers in rlag(i), i = 1,2,...,na by solving
c  lt*rlag = q1*g. l is an lower-triangular matrix (lt denotes its
c  transpose). l is stored row by row in el(i), i = 1,2,...,
c  na*(na + 1)/2. q1 is the matrix formed from the first na columns of
c  the n*n matrix qt, transposed.
c  g is the gradient vector of length n.
c  w is workspace of length na. lqt = n. lel = na*(na + 1)/2.
c
c  philip e. gill, walter murray and susan m picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  september 1977
c
c  *********************************************************************
c
      if(na .lt. 1) return
      call atx(n, na, qt, lqt, g, w)
      call QQrsol(na, lel, el, w, rlag)
      return
c
c  end of subroutine lagmul
c
      end
      subroutine mdztg(n, na, v, u, y, ymod)
      implicit double precision (a-h,o-z)
      integer n, na
      dimension v(n), u(n), y(n)
c
c  *********************************************************************
c
c  mdztg updates the projected gradient when a zero step has been taken.
c  it must always be preceded by the call of addcon, since it uses
c  permutations of givens transformations which are generated during
c  addcon and stored as sine/cosine pairs in the vectors v and u.
c
c  philip e gill, walter murray, susan m picken, enid m r long
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
      integer i, k, lnewzt
c
c  the following standard function is called within mdztg
c
c
      sum = 0.0e+0
      if(na .eq. n) go to 20
      yknext = y(1)
      lnewzt = n - na
      do 10 k = 1, lnewzt
         i = lnewzt - k + 1
         sn = v(i)
         cs = u(i)
         yk = yknext
         ykplus = y(k + 1)
         yknext = cs*ykplus + sn*yk
         yk = sn*ykplus - cs*yk
         y(k) = yk
         sum = sum + yk*yk
   10 continue
   20 ymod = dsqrt(sum)
      return
c
c  end of subroutine mdztg
c
      end
      subroutine modfmt(ifmti, ifmtr, ifmt03, ifmt04)
      implicit double precision (a-h,o-z)
      integer ifmti(10), ifmtr(20), ifmt03(35), ifmt04(35)
c
c  *********************************************************************
c
c  subroutine modfmt refills the format arrays ifmt03 and ifmt04.
c
c  the format arrays contain the following hollerith text-
c        ifmt03             (sifa,1x,3(sifa))
c        ifmt04             (2(srfa),3x,l1)
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england.
c  crown copyright
c  september 1978
c
c  *********************************************************************
c
      integer i, icomma, il, ilpar, irpar, itemp, ix, i1, i2, i3
      data icomma /1h,/
      data il     /1hl/
      data ilpar  /1h(/
      data irpar  /1h)/
      data ix     /1hx/
      data i1     /1h1/
      data i2     /1h2/
      data i3     /1h3/
      ifmt03(1) = ilpar
      ifmt03(12) = icomma
      ifmt03(13) = i1
      ifmt03(14) = ix
      ifmt03(15) = icomma
      ifmt03(16) = i3
      ifmt03(17) = ilpar
      ifmt03(28) = irpar
      ifmt03(29) = irpar
      ifmt04(1) = ilpar
      ifmt04(2) = i2
      ifmt04(3) = ilpar
      ifmt04(24) = irpar
      ifmt04(25) = icomma
      ifmt04(26) = i3
      ifmt04(27) = ix
      ifmt04(28) = icomma
      ifmt04(29) = il
      ifmt04(30) = i1
      ifmt04(31) = irpar
      do 10 i = 1, 10
         itemp = ifmti(i)
         ifmt03(i + 1) = itemp
         ifmt03(i + 17) = itemp
   10 continue
      do 20 i = 1, 20
         ifmt04(i + 3) = ifmtr(i)
   20 continue
      call QQblan(6, ifmt03(30))
      call QQblan(4, ifmt04(32))
      return
c
c  end of subroutine modfmt
c
      end
      subroutine pcomp(ipflag, n, na, lzt, eps, qt, lqt,
     *   y, v, p, ptp)
      implicit double precision (a-h,o-z)
      integer ipflag, n, na, lzt, lqt
      dimension qt(lqt, n), y(n), v(n), p(n)
c
c  *********************************************************************
c
c  pcomp computes p, the search direction vector, and ptp, the square
c  of its euclidean norm, for the linear programming algorithm.  the
c  setting of ipflag determines whether p is derived from the projected
c  gradient (ipflag = 0), from a violated constraint (ipflag = 1) or
c  from the final column of z (ipflag = - 1).
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
c
c  the following system function is used by pcomp
c
c
      if(ipflag .ne. 0) go to 10
      call zpacmp(n, lzt, - 1.0e+0, qt, lqt, y, p, ptp)
      return
   10 if(ipflag .lt. 0) go to 20
      call zpacmp(n, lzt, 1.0e+0, qt, lqt, v, p, ptp)
      return
   20 call QQstov(n, qt(1, na + 1), p)
      test = y(lzt)
      if(test .gt. (- eps)) call negvec(n, p)
      ptp = dot(n, p, p)
      return
c
c  end of subroutine pcomp
c
      end
      subroutine psdgrd(n, na, ngc, m, rtol, index,
     *   agc, lagc, lagct, r, feas, nviol, g)
      implicit double precision (a-h,o-z)
      integer n, na, ngc, m, lagc, lagct, nviol
      logical feas
      integer index(m)
      dimension rtol(m), agc(lagc, lagct), r(m), g(n)
c
c  *********************************************************************
c
c  forms the gradient vector in g(i), i=1,2,...,n corresponding to
c  the pseudo-objective function that is set up when the current point
c  is infeasible. the logical variable feas = .true. if the current
c  point is feasible with respect to the constraints within a tolerance
c  of rtol.
c
c  philip e. gill, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright reserved
c  february 1976
c
c  *********************************************************************
c
      integer i, i1, i2, i3, ina, infeas, mna
      logical fs
      fs = .true.
      infeas = 0
      mna = m - na
      do 10 i = 1, n
   10    g(i) = 0.0e+0
      do 50 i = 1, mna
         i1 = i + na
         i1 = index(i1)
         i2 = i1 - n - n
         i3 = i2 - ngc
         temp = r(i)
         ina = i + na
         rtoli = rtol(ina)
         if(i1 .gt. n) goto 20
         if(temp .ge. (- rtoli)) goto 50
         fs = .false.
         infeas = infeas + 1
         g(i1) = g(i1) - 1.0e+0
         goto 50
   20    if(i2 .gt. 0) goto 30
         if(temp .ge. (- rtoli)) goto 50
         fs = .false.
            infeas = infeas + 1
         i1 = i1 - n
         g(i1) = g(i1) + 1.0e+0
         goto 50
   30    if(i3 .gt. 0) goto 40
         if(temp .ge. (- rtoli)) goto 40
         call vecsum(n, - 1.0e+0, agc(1, i2), g, g)
         fs = .false.
         infeas = infeas + 1
         goto 50
   40    if(temp .ge. (- rtoli)) goto 50
         call vecsum(n, 1.0e+0, agc(1, i3), g, g)
         fs = .false.
         infeas = infeas + 1
   50 continue
      feas = fs
      nviol = infeas
      return
c
c  end of subroutine psdgrd
c
      end
      subroutine qfun(n, x, f, g, hesl, lh, hesd, c,
     *   iw, liw, w, lw)
      implicit double precision (a-h,o-z)
      integer n, lh, liw, lw
      integer iw(liw)
      dimension x(n), g(n), hesl(lh),
     *   hesd(n), c(n), w(lw)
c
c  *********************************************************************
c
c  qfun computes the value and the gradient of the quadratic function
c  0.5*xt.h.x + xt.c when the cholesky factors of the hessian matrix
c  h are stored in the vectors hesl and hesd.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
c  the following system function is called by qfun
c
c
c  symhx computes w = h.x
c
      call symhx(n, lh, hesl, hesd, x, w)
c
c  the function is evaluated.
c
      f = 5.0e-1*dot(n, x, w) + dot(n, x, c)
c
c  vecsum forms the gradient g = h.x + c
c
      call vecsum(n, 1.0e+0, w, c, g)
      return
c
c  end of subroutine qfun
c
      end
      subroutine q2mult(n, ngc, iadd, lq1, agc, lagc, lagct,
     *   qt, lqt, gamma, v)
      implicit double precision (a-h,o-z)
      integer n, ngc, iadd, lq1, lagc, lagct, lqt
      dimension agc(lagc, lagct), qt(lqt, n), v(n)
c
c  *********************************************************************
c
c  when a new constraint is to be added to the basis, q2mult pre-
c  multiplies it by the current q2 matrix and computes gamma as the
c  euclidean norm of the resulting vector v.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
      integer i, igclb, igcub, iqt, ixub, lq2
c
c  the following system function and standard function are used by
c  q2mult
c
c
      ixub = iadd - n
      igclb = ixub - n
      igcub = igclb - ngc
      iqt = lq1
      lq2 = n - iqt
      gamma2 = 0.0e+0
      do 50 i = 1, lq2
         iqt = iqt + 1
         if(iadd .gt. n) go to 10
         sum = qt(iadd, iqt)
         go to 40
   10    if(igclb .gt. 0) go to 20
         sum = - qt(ixub, iqt)
         go to 40
   20    if(igcub .gt. 0) go to 30
         sum = dot(n, agc(1, igclb), qt(1, iqt))
         go to 40
   30    sum = - dot(n, agc(1, igcub), qt(1, iqt))
   40    v(i) = sum
         gamma2 = gamma2 + sum*sum
   50 continue
      gamma = dsqrt(gamma2)
      return
c
c  end of subroutine q2mult
c
      end
      subroutine refcon(n, na, ngc, m, rtol, index, agc, lagc, lagct,
     *   bl, bu, lb, el, lel, qt, lqt, x, ref, delx1, w)
      implicit double precision (a-h,o-z)
      integer n, na, ngc, m, lagc, lagct, lb, lel, lqt
      integer index(m)
      logical ref
      dimension rtol(m), agc(lagc, lagct), bl(lb), bu(lb),
     *   el(lel), qt(lqt,n), x(n), delx1(n), w(n)
c
c  ********************************************************************
c
c  subroutine refcon refines the approximate solution in x(i),
c  i = 1,2,...,n to ensure that  at*x = b (t denotes
c  transpose). at is the na*n set of active constraints and b the set of
c  corresponding right-hand sides. these constraints are identified by
c  the elements index(i), i = 1,2,...,na of the integer array index,
c  together with the matrix of general constraints agc and the bounds
c  in bl and bu.
c  qt and l are such that a = qt*(l 0)t, where q is an orthogonal
c  matrix and where the lower-triangular matrix l
c  is stored row by row in el(i), i = 1,2,...,na*(na + 1)/2.
c  x is updated and the vector that has been added to x is also
c  stored in delx1.
c  the logical ref is set true if any residual has an absolute
c  value greater than the corresponding element of rtol. if ref
c  is false no refinement has been applied to x.
c  lagc = n. lagct = ngc or 1 if ngc .eq. 0.
c  lb = ngc + n, lel = na*(na + 1)/2, lqt = n and lw = n.
c
c  philip e. gill, walter murray, susan m. picken, enid m. r. long
c  d.n.a.c., national physical laboratory, england.
c  crown copyright
c  september 1978
c
c  ********************************************************************
c
      integer i, i1, i2, i3, i4, i5
c
c  the following standard and library functions are called
c
      double precision dot
      ref = .false.
      if(na .lt. 1) return
c
c  form r = b - at*x
c
      do 50 i = 1, na
         i1 = index(i)
         rtoli = rtol(i)
         i2 = i1 - n - n
         i3 = i2 - ngc
         if(i1 .gt. n) goto 10
         i4 = i1 + ngc
         sum = bl(i4) - x(i1)
         goto 40
   10    if(i2 .gt. 0) goto 20
         i4 = i1 + ngc - n
         i5 = i1 - n
         sum = - bu(i4) + x(i5)
         goto 40
   20    if(i3 .gt. 0) goto 30
         sum = dot(n, agc(1, i2), x)
         sum = bl(i2) - sum
         goto 40
   30    sum = dot(n, agc(1, i3), x)
         sum = sum - bu(i3)
   40    delx1(i) = sum
         if(dabs(sum) .gt. rtoli) ref = .true.
   50 continue
      if(.not. ref) return
c
c  solve l*s = r for s
c
      call rtsol(na, lel, el, delx1, w)
c
c  form delx1 = q1t*s, where q1t is the matrix composed of the
c  first na columns of qt.
c
      call ax(n, na, qt, lqt, w, delx1)
c
c  add delx1 to x.
c
      call vecsum(n, 1.0e+0, delx1, x, x)
      return
c
c  end of subroutine refcon
c
      end
      double precision function QQrnor(n, r)
      implicit double precision (a-h,o-z)
      integer n
      dimension r(n)
c
c  *********************************************************************
c
c  this double precision function calculates the norm of the n-vector r.
c
c  philip e. gill, walter murray, susan m. picken and
c  nicholas i. m. gould.
c  d.n.a.c., national physical laboratory, england.
c  crown copyright reserved
c  august 1976
c
c  *********************************************************************
c
      integer i
c
c  the following standard function is called
c
      rnrmsq = 0.0e+0
      do 10 i = 1, n
         rr = r(i)
         rnrmsq = rnrmsq + rr*rr
   10 continue
      QQrnor = dsqrt(rnrmsq)
      return
c
c  end of double precision function QQrnor.
c
      end
      subroutine rvztg(n, na, qt, lqt, g, y, yty)
      implicit double precision (a-h,o-z)
      integer n, na, lqt
      dimension qt(lqt, n), g(n), y(n)
c
c  *********************************************************************
c
c  forms y = zt*g and stores yt*y in yty. the matrix z is given by the
c  last n-na columns of the n*n matrix qt in reverse order. vector g is
c  stored in g(i), and the vector y in y(i), i = 1,2,...,n - na.
c
c  philip e. gill, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england.
c  crown copyright reserved
c  september 1977
c
c  *********************************************************************
c
      integer i, i1, i2, k
c
c  the following library function is called within ztg.
c
c
      sum = 0.0e+0
      k = n + 1
      i1 = n - na
      if (i1 .lt. 1) goto 30
      do 20 i = 1, i1
         i2 = k - i
         sum2 = dot(n, qt(1, i2), g)
         y(i) = sum2
         sum = sum + sum2*sum2
   20 continue
   30 yty = sum
      return
c
c  end of subroutine rvztg
c
      end
      subroutine scle(n, ngc, lngc, agc, lagc, bl, bu, lb, istate,
     *   scale)
      implicit double precision (a-h,o-z)
      integer n, ngc, lngc, lagc, lb
      integer istate(lngc)
      dimension agc(lagc, lngc), bu(lb), bl(lb), scale(lngc)
c
c  *********************************************************************
c
c  scales the matrix of general constraints agc so that each column
c  has a norm of 1.0. elements of bu and bl are scaled correspondingly.
c  the scaling vector is stored in scale.
c  if the original column has a norm of less than epsmch then no
c  scaling takes place.
c
c  philip e gill, walter murray, susan m picken, enid m r long
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
      integer i, isti, j
c
c  the following library and standard functions are used
c
      if(ngc .eq. 0) return
      epsmch = x02aaf(epsmch)
      do 20 i = 1, ngc
         sci = dot(n, agc(1, i), agc(1, i))
         sci = dsqrt(sci)
         if(sci .lt. epsmch) sci = 1.0e+0
         do 10 j = 1, n
   10       agc(j, i) = agc(j, i)/sci
         isti = istate(i)
         if(isti .ne. 2) bl(i) = bl(i)/sci
         if(isti .ne. 1) bu(i) = bu(i)/sci
         scale(i) = sci
   20 continue
      return
c
c  end of subroutine scle
c
      end
      subroutine setqu(n, u, qt, lqt)
      implicit double precision (a-h,o-z)
      integer n, lqt
      dimension u(n), qt(lqt, n)
c
c  *********************************************************************
c
c  setqu sets all the elements of the lagrange-multiplier vector to
c  zero and initializes qt as the identity matrix.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  october 1977
c
c  *********************************************************************
c
      integer i
      call zerovc(n, u)
      call zeroar(n, n, qt, lqt)
      do 10 i = 1, n
         qt(i, i) = 1.0e+0
   10 continue
      return
c
c  end of subroutine setqu
c
      end
      subroutine settol(n, na, ngc, lcon, istate, lst, ibnds, index,
     *   rtol, v, lv)
      implicit double precision (a-h,o-z)
      integer n, na, ngc, lcon, lst, lv
      integer istate(lst), ibnds(n), index(lcon)
      dimension rtol(lcon), v(lv)
c
c  *********************************************************************
c
c  when a starting point has been found, settol rearranges the entries
c  in the residual tolerance vector rtol to correspond with those in
c  index for all the active constraints and assigns the appropriate
c  values to the positions of index and rtol relating to the inactive
c  constraints.  it also restores the original entries in the vectors
c  istate and ibnds.
c  lv, the length of the workspace vector v, must be at least
c  2*(n + ngc).
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
      integer i, iin, iis, isi, iv, m
      m = 2*(n + ngc)
      call QQstov(m, rtol, v)
c
c  active constraints.
c
      if (na .eq. 0) go to 20
      do 10 i = 1, na
         iv = index(i)
         rtol(i) = v(iv)
   10 continue
c
c  inactive simple constraints.
c
   20 iin = na
      iis = ngc
      do 30 i = 1, n
         iis = iis + 1
         if(ibnds(i) .lt. 0 .or. istate(iis) .eq. 0) go to 30
         iin = iin + 1
         index(iin) = i
         rtol(iin) = v(i)
   30 continue
      iv = n
      iis = ngc
      do 40 i = 1, n
         iv = iv + 1
         iis = iis + 1
         if(ibnds(i) .gt. 0 .or. istate(iis) .eq. 0) go to 40
         iin = iin + 1
         index(iin) = iv
         rtol(iin) = v(iv)
   40 continue
      call zeroiv(n, ibnds)
c
c  inactive general constraints.
c
      if(ngc .lt. 1) return
      do 50 i = 1, ngc
         iv = iv + 1
         isi = istate(i)
         if(isi .le. 0 .or. isi .eq. 2) go to 50
         iin = iin + 1
         index(iin) = iv
         rtol(iin) = v(iv)
   50 continue
      do 70 i = 1, ngc
         iv = iv + 1
         isi = istate(i)
         if(isi .ge. 2) go to 60
         if(isi .lt. 0) istate(i) = - isi
         if(isi .ne. (- 3)) go to 70
   60    iin = iin + 1
         index(iin) = iv
         rtol(iin) = v(iv)
   70 continue
      return
c
c  end of subroutine settol
c
      end
      subroutine tolcmp(n, ngc, eps, nngc, bl, bu, istate,
     *   ncon, rtol, lrtol)
      implicit double precision (a-h,o-z)
      integer n, ngc, nngc, ncon, lrtol
      integer istate(nngc)
      dimension bl(nngc), bu(nngc), rtol(lrtol)
c
c  *********************************************************************
c
c  tolcmp computes the tolerances for the constraint residuals and
c  stores them in the positions of rtol which correspond to the actual
c  constraints.  ncon returns the number of constraints contained in
c  the problem.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  september 1978
c
c  *********************************************************************
c
      integer i, ib, il, isi, iu, kount, m
c
c  the following standard function is called by tolcmp
c
c
c  compute the tolerances for the simple constraints.
c
      kount = 0
      m = 2*(n + ngc)
      call zerovc(m, rtol)
      ib = ngc
      il = 0
      iu = n
      do 10 i = 1, n
         ib = ib + 1
         isi = 3
         bli = bl(ib)
         bui = bu(ib)
         if((bui - bli) .le. eps) isi = 0
         istate(ib) = isi
         kount = kount + 1 + isi/3
         il = il + 1
         iu = iu + 1
         rtol(il) = eps*(dabs(bli) + 1.0e+0)
         if(isi .eq. 3) rtol(iu) = eps*(dabs(bui) + 1.0e+0)
   10 continue
      if(ngc .lt. 1) go to 40
c
c  compute the tolerances for the general constraints.
c
      il = n + n
      iu = il + ngc
      do 30 i = 1, ngc
         il = il + 1
         iu = iu + 1
         isi = istate(i)
         bli = bl(i)
         bui = bu(i)
         if(isi .eq. 2) go to 20
         kount = kount + 1
         rtol(il) = eps*(dabs(bli) + 1.0e+0)
         if(isi .ne. 3) go to 30
         if((bui - bli) .gt. eps) go to 20
         istate(i) = 0
         go to 30
   20    kount = kount + 1
         rtol(iu) = eps*(dabs(bui) + 1.0e+0)
   30 continue
   40 ncon = kount
      return
c
c  end of subroutine tolcmp
c
      end
      subroutine addphf(n, na, ell, lell, dd, ldd, v, u, w, lw)
      implicit double precision (a-h,o-z)
      integer n, na, lell, ldd, lw
      dimension ell(lell), dd(ldd), v(n), u(n), w(lw)
c
c  *********************************************************************
c
c  addphf modifies the cholesky factors of the projected hessian after
c  a constraint has been added.  it must always be preceded by the call
c  of addcon, since it uses permutations of givens transformations which
c  are generated during addcon and stored as sine/cosine pairs in the
c  vectors v and u.
c  the projected hessian enters with nz+1 rows and columns, where
c  nz = n - na. the value of na has already been reduced by one.
c  on exit only the first nz rows and columns of lell are needed.
c  addphf uses 2*n words of workspace.
c
c  philip e gill, walter murray, susan m picken, enid m r long
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  september 1978
c
c  *********************************************************************
c
      integer i, im1, iplus2, iplusn, ir, irmi, j, k, nz
      logical ddpos
c
c  the following library and system functions are called within
c  subroutine addphf.
c
c
c  pre-multiply lower-triangular matrix by permuted givens rotations
c  to form a lower-hessenberg matrix. during the multiplication, the
c  square root of the diagonal matrix dd is multiplied into ell.
c  the super-diagonals are stored in  w(n+1) onward, the diagonals are
c  stored in dd.
c
      if(na .eq. n) return
c
c  compute the dimension of the null space of the matrix of active
c  constraints.
c
      nz = n - na
      ddpos = dd(nz + 1) .gt. 0.0e+0
      rtdiag = dsqrt(dd(1))
      dd(1) = rtdiag
      w(1)  = rtdiag
      ir = 1
      iplusn = n
      do 30 i = 1, nz
         iplusn = iplusn + 1
c
c  extract the sine and cosine pairs from storage. they were stored
c  during the execution of the subroutine addcon.
c
         k = nz - i + 1
         sn = v(k)
         cs = u(k)
         if(i .eq. 1) go to 20
         im1 = i - 1
c
c  form linear combination of elements from the ith. and i+1th. rows
c
         do 10 j = 1, im1
            irmi = ir - im1
            ellrmi = ell(irmi)
c
c  scale with respect to the diagonal before applying the rotation.
c
            ellr = ell(ir)*w(j)
            ell(irmi) = - cs*ellrmi + sn*ellr
            ell(ir) = sn*ellrmi + cs*ellr
            ir = ir + 1
   10    continue
   20    ddi = dd(i)
         rtdiag = dsqrt(dabs(dd(i+1)))
         ddip1 = rtdiag
c
c  store the square root of the original diagonal element.
c
         w(i+1) = rtdiag
c
c   compute the super-diagonal element.
c
         w(iplusn) = ddip1*sn
         ellr = ell(ir)*w(i)
         ell(ir) = ddi*sn + cs*ellr
         dd(i) = - cs*ddi + sn*ellr
         dd(i+1) = cs*ddip1
         ir = ir + 1
   30 continue
c
c  post-multiply the lower hessenberg matrix by a sequence of givens
c  matrices which reduce the super-diagonal elements to zero.
c  simultaneously scale the columns of ell by the elements of dd.
c
      k = 0
      iplusn = n
      do 50 i = 1, nz
         iplusn = iplusn + 1
         wi = w(iplusn)
         ddi = dd(i)
         if(i .eq. nz) go to 50
         ddip1 = dd(i+1)
         rhosq = wi*wi + ddi*ddi
         rho = dsqrt(rhosq)
         sn = wi/rho
         cs = ddi/rho
         dd(i) = rhosq
         k = k + i
         ellk = ell(k)
         ell(k) = (cs*ellk + sn*ddip1)/rho
         dd(i+1) = sn*ellk - cs*ddip1
         if(i .eq. nz - 1) go to 50
         iplus2 = i + 2
         ir = k + i
         do 40 j = iplus2, nz
            ellr = ell(ir)
            ellrp1 = ell(ir+1)
            ell(ir) = (cs*ellr + sn*ellrp1)/rho
            ell(ir+1) = sn*ellr - cs*ellrp1
            ir = ir + j - 1
   40    continue
   50 continue
      wi = wi*wi
      if(.not. ddpos) wi = - wi
      dd(nz) = ddi*ddi + wi
c
      return
c
c  end of subroutine addphf
c
      end
      subroutine addprm(na, nina, lzt, iadd, index, m, rtol, r)
      implicit double precision (a-h,o-z)
      integer na, nina, lzt, iadd, m
      integer index(m)
      dimension rtol(m), r(m)
c
c  *********************************************************************
c
c  after the addition of a constraint, addprm deletes its residual
c  r(kmin) from the residual vector and shifts all subsequent entries
c  forwards by one position.  it also transfers its index from its
c  place in the list of active constraints, moving the index of each
c  intervening constraint backwards by one position in the index vector.
c  iadd has the value kmin on entry but on exit returns the original
c  index of the new constraint.
c
c  philip e gill, walter murray, susan m picken, enid m r long
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
      integer i, j, nak
      nak = na + iadd
      na = na + 1
      nina = nina - 1
      lzt = lzt - 1
      if(iadd .gt. nina) go to 20
      do 10 i = iadd, nina
         r(i) = r(i + 1)
   10 continue
   20 iadd = index(nak)
      if(nak .eq. 1) return
      rtolk = rtol(nak)
      nak = nak - 1
      do 30 i = na, nak
         j = nak + na - i
         index(j + 1) = index(j)
         rtol(j + 1) = rtol(j)
   30 continue
      index(na) = iadd
      rtol(na) = rtolk
      return
c
c  end of subroutine addprm
c
      end
      subroutine atx(n, m, a, la, x, y)
      implicit double precision (a-h,o-z)
      integer n, m, la
      dimension a(la, m), x(n), y(m)
c
c  *********************************************************************
c
c  forms the product of the transpose of the matrix a with the vector
c  x and stores the result in y.
c
c  philip e. gill, enid m. r. long,
c  walter murray and susan picken.
c  d.n.a.c., national physical laboratory, england
c  crown copyright reserved
c  november 1976
c
c  *********************************************************************
c
      integer i, j
      do 20 i = 1, m
         sum = 0.0e+0
         do 10 j = 1, n
   10       sum = sum + a(j, i)*x(j)
         y(i) = sum
   20 continue
      return
c
c  end of subroutine atx.
c
      end
      subroutine ax(n, m, a, la, x, y)
      implicit double precision (a-h,o-z)
      integer n, m, la
      dimension a(la, m), x(m), y(n)
c
c  *********************************************************************
c
c  forms the product of the n*m matrix a with the vector x and stores
c  the result in vector y.
c
c  philip e. gill, walter murray, susan m. picken, enid m. r. long
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  january 1977
c
c  *********************************************************************
c
      integer i, j
      do 20 i = 1, n
         sum = 0.0e+0
         do 10 j = 1, m
   10       sum = sum + a(i, j)*x(j)
         y(i) = sum
   20 continue
      return
c
c  end of subroutine ax
c
      end
      subroutine QQblan(n, iv)
      implicit double precision (a-h,o-z)
      integer n
      integer iv(n)
c
c  *********************************************************************
c
c  subroutine QQblan fills the integer array iv(i), i=1(1)n with
c  blanks.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england.
c  crown copyright, june 1978
c
c  *********************************************************************
c
      integer i, iblank
      data iblank /1h /
      do 10 i = 1, n
         iv(i) = iblank
   10 continue
      return
c
c  end of subroutine QQblan
c
      end
      subroutine chkbnd(n, ngc, bl, bu, lb, istate, lngc, ifail)
      implicit double precision (a-h,o-z)
      integer n, ngc, lb, lngc, ifail
      integer istate(lngc)
      dimension bl(lb), bu(lb)
c
c  *********************************************************************
c
c  chkbnd checks that the lower bound does not exceed the upper bound
c  of any inequality constraint and that the lower bound is equal to
c  the upper bound for every constraint specified as an equality.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  january 1978
c
c  *********************************************************************
c
      integer i, ib, isi
      ifail = 1
c
c  simple constraints.
c
      ib = ngc
      do 10 i = 1, n
         ib = ib + 1
         if(bu(ib) .lt. bl(ib)) return
   10 continue
      if(ngc .eq. 0) go to 30
c
c  general constraints.
c
      do 20 i = 1, ngc
         isi = istate(i)
         if(isi .ne. 0 .and. isi .ne. 3) go to 20
         bli = bl(i)
         bui = bu(i)
         if((isi .eq. 0 .and. bui .ne. bli) .or. (isi .eq. 3
     *      .and. bui .lt. bli)) return
   20 continue
   30 ifail = 0
      return
c
c  end of subroutine chkbnd
c
      end
      subroutine chslm1(na, nequal, rlag, imin, rlmax, rlmin)
      implicit double precision (a-h,o-z)
      integer na, nequal, imin
      dimension rlag(na)
c
c  *********************************************************************
c
c  this subroutine gives imin and rlmin, where rlag(imin) = rlmin is
c  the minimum multiplier corresponding to an inequality constraint.
c  rlmax gives the maximim value of rlag(i) corresponding to an
c  inequality constraint.
c
c  philip e. gill, walter murray, susan m. picken, enid m. r. long
c  d.n.a.c., national physical laboratory, england.
c  crown copyright
c  february 1978
c
c  *********************************************************************
c
      integer i, istart
      rlmax = - 1.0e+5
      imin = 0
      rlmin = 1.0e+5
      if(nequal .ge. na) return
      istart = nequal + 1
      do 20 i = istart, na
         temp = rlag(i)
         if(temp .ge. rlmin .and. i .ne. istart) go to 10
         imin = i
         rlmin = temp
   10    if(temp .gt. rlmax .or. i .eq. istart) rlmax = temp
   20 continue
      return
c
c  end of subroutine chslm1.
c
      end
      subroutine delprm(na, nina, lzt, m, idel, index, rtol, r)
      implicit double precision (a-h,o-z)
      integer na, nina, lzt, m, idel
      integer index(m)
      dimension rtol(m), r(m)
c
c  *********************************************************************
c
c  after the deletion of a constraint, delprm transfers its index from
c  its place in the list of active constraints to the bottom of the
c  list of inactive constraints, moving the index of each subsequent
c  constraint forwards by one position in the index vector.  it assigns
c  the residual of this new inactive constraint, currently zero, to the
c  first unused position of the residual vector.
c
c  philip e gill, walter murray, susan m picken, enid m r long
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
      integer j, k, l
      na = na - 1
      nina = nina + 1
      lzt = lzt + 1
      r(nina) = 0.0e+0
      k = index(idel)
      rtoli = rtol(idel)
      l = idel + 1
      do 10 j = l, m
         index(j - 1) = index(j)
         rtol(j - 1) = rtol(j)
   10 continue
      index(m) = k
      rtol(m) = rtoli
      return
c
c  end of subroutine delprm
c
      end
      subroutine delpsd(n, na, index, lin, npseud, ipseud, is)
      implicit double precision (a-h,o-z)
      integer n, na, lin, npseud, is
      integer index(lin), ipseud(n)
c
c  *********************************************************************
c
c  when the basis still contains pseudo-constraints (marked by entries
c  of - 2 in the vector ipseud), delpsd selects one of these for
c  deletion, records its basis index in is and decrements npseud, the
c  number of pseudo-constraints in the active set.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
      integer i, ii
      do 10 i = 1, na
         ii = index(i)
         if(ii .gt. n) go to 10
         if(ipseud(ii) .eq. (- 2)) go to 20
   10 continue
      return
   20 is = i
      npseud = npseud - 1
      ipseud(ii) = 0
      return
c
c  end of subroutine delpsd
c
      end
      double precision function dot(n, a, b)
      implicit double precision (a-h,o-z)
      integer n
      dimension a(n), b(n)
c
c  *********************************************************************
c
c  this double precision function calculates the inner product of the n-vectors
c  a and b.
c
c  philip e. gill, walter murray, susan m. picken, shedmond r. graham
c  and margaret h. wright
c  d.n.a.c., national physical laboratory, england.
c  crown copyright reserved
c  september 1975
c
c  *********************************************************************
c
      integer i
      atb = 0.0e+0
      do 10 i = 1, n
         atb = atb + a(i)*b(i)
   10 continue
      dot = atb
      return
c
c  end of double precision function dot.
c
      end
      subroutine eltsol(n, lel, is, el, p)
      implicit double precision (a-h,o-z)
      integer n, lel, is
      dimension el(lel), p(n)
c
c  *********************************************************************
c
c  solves lt*p = es for p, where lt is the transpose of l,
c  an n*n unit lower-triangular matrix stored by rows
c  omitting the unit diagonal in the array el(i),
c  i = 1(1)n*(n-1)/2. es is an n-vector with its isth element
c  unity and all others zero.
c
c  philip e. gill, walter murray, susan m. picken, shedmond r. graham
c  and margaret h. wright
c  d.n.a.c., national physical laboratory, england.
c  crown copyright reserved
c  september 1975
c
c  *********************************************************************
c
      integer i, ip1, iq, is1, isp1, it, j, j1, k, l, np1
      is1 = is
      it = is1*(is1 - 1)/2 + 1
      np1 = n + 1
      isp1 = is1 + 1
      do 40 j = 1, n
         i = np1 - j
         if(i .gt. is1) go to 30
         iq = it
         it = it - 1
         sum = 0.0e+0
         if(i .eq. is1) sum = 1.0e+0
         ip1 = i + 1
         if(ip1 .gt. is1) go to 20
         l = isp1 - ip1
         do 10 j1 = 1, l
            k = isp1 - j1
            sum = sum - p(k)*el(iq)
            iq = iq + 2 - k
   10    continue
   20    p(i) = sum
         go to 40
   30    p(i) = 0.0e+0
   40 continue
      return
c
c  end of subroutine eltsol.
c
      end
      subroutine lcmon(mlevel, iupr, ifmt01, ifmt02, ifmt03,
     *   ifmt04, n, ngc, na, nequal, ncon, lcon, nngc,
     *   feas, nviol, niter, nf, f, x, pgnorm, umin, umax,
     *   idrop, u, r, scale, lscale, index, istate, iprog,
     *   lmtest, cond, posdef, final, iw, liw, w, lw)
      implicit double precision (a-h,o-z)
      integer mlevel, iupr, n, ngc, na, nequal, ncon, lcon,
     *   nngc, nviol, niter, nf, idrop, lscale, iprog, liw, lw
      logical feas, lmtest, posdef, final
      integer ifmt01(40), ifmt02(20), ifmt03(35), ifmt04(35),
     *   index(lcon), istate(nngc), iw(liw)
      dimension x(n), u(n), r(lcon), scale(lscale),
     *   w(lw)
c
c  *********************************************************************
c
c  lcmon prints out information for the linearly-constrained
c  minimization routines at the level requested by the user.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  september 1978
c
c  *********************************************************************
c
      integer i, id, iis, ini, iphase, ir, isi, j, j0, j1, j2,
     *   nact, na1, ni, npseud, n2, n2i, n2ngc, n2ngci
      if(mlevel .eq. 0) return
      n2 = n + n
      n2ngc = n2 + ngc
      if(lmtest .and. .not. final) go to 320
c
c  the following abbreviated headings are used at level 1
c        phase   = 1 if program currently finding feasible point,
c                  2 otherwise
c        niter   number of iteration in current phase
c        nfun    number of function evaluations in phase 2
c        nact    number of constraints in active set
c        obj f   value of objective function being minimized
c        proj grad norm
c                euclidean norm of gradient of function projected
c                into null space of active constraints
c        posdef  = t if projected hessian matrix positive definite,
c                  f otherwise
c
      iphase = 1
      if(feas) iphase = 2
      npseud = 0
      j0 = nequal + 1
      j1 = j0
      j2 = nequal
      go to (10, 50, 60, 70), iprog
c
c  for lcqp.
c
   10 nact = na
      if(nact .le. nequal .or. lmtest) go to 40
      do 30 i = j0, nact
         ini = index(i)
         if(ini .gt. n) go to 20
         if(iw(ini) .ne. (- 2)) go to 20
         npseud = npseud + 1
         go to 30
   20    if(npseud .eq. 0) j2 = j2 + 1
   30 continue
      nact = nact - npseud
      j0 = j2 + npseud + 1
   40 write(iupr, 2010)
      write(iupr, ifmt03) iphase, niter, nact
      write(iupr, 2020)
      write(iupr, ifmt04) f
      go to 80
c
c  for lp and feaspt.
c
   50 write(iupr, 2010)
      write(iupr, ifmt03) iphase, niter, na
      write(iupr, 2030)
      write(iupr, ifmt04) f, pgnorm
      go to 80
c
c  for lcqndr and lcqndf.
c
   60 write(iupr, 2040)
      write(iupr, ifmt03) iphase, niter, nf, na
      write(iupr, 2030)
      write(iupr, ifmt04) f, pgnorm
      go to 80
c
c  for lcmna and lcmnaf.
c
   70 if(na .eq. n) go to 60
      write(iupr, 2040)
      write(iupr, ifmt03) iphase, niter, nf, na
      write(iupr, 2050)
      write(iupr, ifmt04) f, pgnorm, posdef
c
c  print out additional information at level 2.
c
   80 if(mlevel .eq. 1) go to 320
c
c  print out the current solution.
c
      write(iupr, 2060)
      do 90 i = 1, n
         write(iupr, ifmt01) i, x(i)
   90 continue
c
c  list the equality constraints.
c
      if(nequal .eq. 0) go to 100
      write(iupr, 2070)
      write(iupr, ifmt02) (index(i), i = 1, nequal)
c
c  if the set of active inequalities is not empty, list its contents.
c  in phase 1 give the number of violated constraints.
c
  100 if(j1 .gt. j2 .and. j0 .gt. na) go to 110
      write(iupr, 2080)
      if(j1 .le. j2) write(iupr, ifmt02) (index(i), i = j1, j2)
      if(j0 .le. na) write(iupr, ifmt02) (index(i), i = j0, na)
  110 if(feas) go to 120
      write(iupr, 2090)
      write(iupr, ifmt02) nviol
      write(iupr, 2100)
c
c  for lcqndr, lcqndf, lcmna and lcmnaf, give the estimated condition
c  number of the projected hessian matrix.
c
  120 if(iprog .lt. 3 .or. na .eq. n) go to 130
      write(iupr, 2110)
      write(iupr, ifmt04) cond
c
c  at level 3 add the residuals of the inactive constraints or, at the
c  final call before exit, the residuals of all the constraints with
c  an indication of their status.
c
  130 if(mlevel .eq. 2) go to 320
c
c  print out the residuals of the inactive constraints.
c
      if(final) go to 190
      na1 = na + 1
      write(iupr, 2120)
      ir = 0
      do 170 i = na1, ncon
         ir = ir + 1
         ini = index(i)
         if(ini .gt. n2) go to 150
         write(iupr, ifmt01) ini, r(ir)
         if(ini .gt. n) go to 140
         write(iupr, 2130)
         go to 170
  140    write(iupr, 2140)
         go to 170
  150    if(ini .gt. n2ngc) go to 160
         j = ini - n2
         res = r(ir)*scale(j)
         write(iupr, ifmt01) ini, res
         write(iupr, 2150)
         go to 170
  160    j = ini - n2ngc
         res = r(ir)*scale(j)
         write(iupr, ifmt01) ini, res
         write(iupr, 2160)
  170 continue
      if(npseud .eq. 0) go to 320
      ir = ncon - na
      do 180 i = 1, n
         if(iw(i) .ne. (- 2)) go to 180
         ir = ir + 1
         write(iupr, ifmt01) i, r(ir)
         write(iupr, 2130)
  180 continue
      go to 320
c
c  print out the residuals of all the simple constraints.
c
  190 write(iupr, 2170)
      iis = ngc
      do 220 i = 1, n
         iis = iis + 1
         isi = istate(iis)
         write(iupr, ifmt01) i, r(i)
         if(isi .ne. 0) write(iupr, 2130)
         if(isi .eq. 0) write(iupr, 2180)
         if(na .eq. 0 .or. iw(i) .eq. (- 2)) go to 210
         do 200 j = 1, na
            if(index(j) .ne. i) go to 200
            write(iupr, 2190)
            go to 220
  200    continue
  210    write(iupr, 2200)
  220 continue
      ni = n
      iis = ngc
      do 250 i = 1, n
         ni = ni + 1
         iis = iis + 1
         if(istate(iis) .eq. 0) go to 250
         write(iupr, ifmt01) ni, r(ni)
         write(iupr, 2140)
         if(na .eq. 0) go to 240
         do 230 j = 1, na
            if(index(j) .ne. ni) go to 230
            write(iupr, 2190)
            go to 250
  230    continue
  240    write(iupr, 2200)
  250 continue
c
c  print out the residuals of all the general constraints.
c
      if(ngc .eq. 0) go to 320
      n2i = n2
      do 280 i = 1, ngc
         n2i = n2i + 1
         isi = istate(i)
         if(isi .eq. 2) go to 280
         write(iupr, ifmt01) n2i, r(n2i)
         if(isi .ne. 0) write(iupr, 2150)
         if(isi .eq. 0) write(iupr, 2180)
         if(na .eq. 0) go to 270
         do 260 j = 1, na
            if(index(j) .ne. n2i) go to 260
            write(iupr, 2190)
            go to 280
  260    continue
  270    write(iupr, 2200)
  280 continue
      n2ngci = n2ngc
      do 310 i = 1, ngc
         n2ngci = n2ngci + 1
         isi = istate(i)
         if(isi .lt. 2) go to 310
         write(iupr, ifmt01) n2ngci, r(n2ngci)
         write(iupr, 2160)
         if(na .eq. 0) go to 300
         do 290 j = 1, na
            if(index(j) .ne. n2ngci) go to 290
            write(iupr, 2190)
            go to 310
  290    continue
  300    write(iupr, 2200)
  310 continue
c
c  print out information about the lagrange multipliers.
c
  320 if(.not. lmtest .or. na .le. nequal) return
      write(iupr, 2210)
      write(iupr, ifmt04) umin, umax
      if(mlevel .lt. 3) return
      id = idrop
      if(idrop .eq. 0) id = n + 1
      write(iupr, 2220)
      j0 = nequal + 1
      do 370 i = j0, na
         ini = index(i)
         write(iupr, ifmt01) ini, u(i)
         if(ini .gt. n) go to 330
         write(iupr, 2130)
         go to 360
  330    if(ini .gt. n2) go to 340
         write(iupr, 2140)
         go to 360
  340    if(ini .gt. n2ngc) go to 350
         write(iupr, 2150)
         go to 360
  350    write(iupr, 2160)
  360    if(i .eq. id) write(iupr, 2230)
  370 continue
      return
c
c  non-executable statements.
c
c  the following are examples of the formats created for
c  the cdc 6000/7000 series.
c
c2001 format(4x, i5, 6x, 1pe22.14)
c2002 format(10(1x, i5))
c2003 format(1x, i5, 1x, 3(1x, i5))
c2004 format(2(4x, 1pe22.14, 4x), 3x, l1)
c
 2010 format(///// 19h0phase  niter  nact)
 2020 format(1h0, 12x, 5hobj f)
 2030 format(1h0, 12x, 5hobj f, 21x, 14hproj grad norm)
 2040 format(///// 25h0phase  niter  nfun  nact)
 2050 format(1h0, 12x, 5hobj f, 21x, 14hproj grad norm, 5x,
     *   6hposdef)
 2060 format(11h0  variable, 8x, 16hcurrent solution)
 2070 format(39h0indices of active equality constraints)
 2080 format(41h0indices of active inequality constraints)
 2090 format(1h )
 2100 format(1h+, 6x, 22hconstraint(s) violated)
 2110 format(48h0estimated condition number of projected hessian,
     *   7h matrix)
 2120 format(10h0 inactive/ 11h constraint, 12x,
     *   8hresidual, 19x, 4htype)
 2130 format(1h+, 42x, 18hsimple lower bound)
 2140 format(1h+, 42x, 18hsimple upper bound)
 2150 format(1h+, 41x, 19hgeneral lower bound)
 2160 format(1h+, 41x, 19hgeneral upper bound)
 2170 format(11h0constraint, 12x, 8hresidual, 19x, 4htype,
     *   10x, 6hstatus)
 2180 format(1h+, 47x, 8hequality)
 2190 format(1h+, 63x, 6hactive)
 2200 format(1h+, 62x, 8hinactive)
 2210 format(37h0lagrange multipliers (reduced costs),
     *   19h have been computed// 7x,
     *   18hminimum multiplier, 12x, 18hmaximum multiplier)
 2220 format(9h0  active/ 11h constraint, 7x,
     *   19hlagrange multiplier, 13x, 4htype)
 2230 format(1h+, 62x, 7hdropped)
c
c  end of subroutine lcmon
c
      end
      subroutine lsol(n, lel, el, b, y)
      implicit double precision (a-h,o-z)
      integer n, lel
      dimension el(lel), b(n), y(n)
c
c  *********************************************************************
c
c  solves l*y = b for y, where l is an n*n unit lower-triangular matrix
c  stored row by row in el(i), i = 1,2,...,n*(n+1)/2, omitting the unit
c  diagonal. the right-hand-side is stored in b(i) and the solution in
c  y(i), i = 1,2,...,n.
c
c  philip e. gill, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england.
c  crown copyright reserved
c  february 1976
c
c  *********************************************************************
c
      integer i, i1, j, k
      j = 1
      do 30 i = 1, n
         sum = b(i)
         i1 = i - 1
         if (i1 .lt. 1) goto 20
         do 10 k = 1, i1
            sum = sum - y(k)*el(j)
            j = j + 1
   10    continue
   20    y(i) = sum
   30 continue
      return
c
c  end of subroutine lsol
c
      end
      subroutine negvec(n, v)
      implicit double precision (a-h,o-z)
      integer n
      dimension v(n)
c
c  *********************************************************************
c
c  negvec negatives every element of the vector v.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  july 1977
c
c  *********************************************************************
c
      integer i
      do 10 i = 1, n
         v(i) = - v(i)
   10 continue
      return
c
c  end of subroutine negvec
c
      end
      subroutine psdres(n, ngc, nina, ipseud, x, bl, lb, r, lr)
      implicit double precision (a-h,o-z)
      integer n, ngc, nina, lb, lr
      integer ipseud(n)
      dimension x(n), bl(lb), r(lr)
c
c  *********************************************************************
c
c  psdres computes the residuals for the pseudo-constraints in lcqp.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  april 1978
c
c  *********************************************************************
c
      integer i, ib, ir
      ib = ngc
      ir = nina
      do 10 i = 1, n
         ib = ib + 1
         if(ipseud(i) .ne. (- 2)) go to 10
         ir = ir + 1
         r(ir) = x(i) - bl(ib)
   10 continue
      return
c
c  end of subroutine psdres
c
      end
      subroutine QQresu(inform, iupr, iutty, isub, feas)
      implicit double precision (a-h,o-z)
      integer inform, iupr, iutty, isub
      logical feas
c
c  *********************************************************************
c
c  called from one of the linearly-constrained minimization routines,
c  result gives a brief indication of the reason for its termination.
c  output is directed to a file (iupr) and, optionally, to the local
c  terminal (iutty) as well.
c
c  philip e. gill, walter murray, susan m. picken, enid m. r. long and
c  brian t. hinde
c  d.n.a.c.s., national physical laboratory, england
c  crown copyright
c  september 1978
c
c  *********************************************************************
c
      integer iout, j, last1, last2
c
c  the following standard function is used by result
c
      integer mod
c
      iout = iupr
   10 if(inform .ge. 0) go to 20
      write(iout, 2000)
      go to 140
   20 write(iout, 2010) inform
      if(inform .gt. 100) go to 110
      if(inform .ne. 0) go to (30, 40, 50 ,60, 70, 80, 90, 100),
     *   inform
      write(iout, 2020)
      go to 140
   30 write(iout, 2030)
      go to 140
   40 if(isub .ge. 40) write(iout, 2040)
      if(isub .lt. 40) write(iout, 2050)
      go to 140
   50 write(iout, 2060)
      go to 140
   60 write(iout, 2070)
      go to 140
   70 if(isub .ge. 40) write(iout, 2080)
      if(isub .lt. 40 .and. .not. feas) write(iout, 2090)
      go to 140
   80 if(isub .lt. 40) write(iout, 2100)
      if(isub .ge. 40) write(iout, 2110)
      go to 140
   90 if(isub .ge. 43) write(iout, 2120)
      if(isub .lt. 40) write(iout, 2130)
      go to 140
  100 if(isub .eq. 44) write(iout, 2140)
      if(isub .lt. 40) write(iout, 2150)
      go to 140
  110 j = inform - 100
      last1 = mod(j, 10)
      last2 = mod(j, 100)
      if(last1 .ne. 1 .or. last2 .eq. 11) go to 120
      write(iout, 2160) j
      go to 140
  120 if(last1 .ne. 2 .or. last2 .eq. 12) go to 130
      write(iout, 2170) j
      go to 140
  130 write(iout, 2180) j
  140 if(iout .eq. iutty) return
      iout = iutty
      go to 10
c
c  non-executable statements
c
 2000 format(///// 17h ***** end of run//
     *   19h inform is negative//
     *   40h termination from user-supplied routine.)
 2010 format(///// 17h ***** end of run//
     *   9h inform =, i5)
 2020 format(26h0program ran successfully.)
 2030 format(27h0error in input parameters.)
 2040 format(34h0there have been maxit iterations.)
 2050 format(34h0there have been maxfun iterations,
     *   11h in phase 1/ 17x, 30hor maxfun function evaluations,
     *   12h in phase 2.)
 2060 format(23h0redundant constraints.)
 2070 format(19h0no feasible point.)
 2080 format(34h0rounding error has resulted in an,
     *   28h ascent direction of search.)
 2090 format(34h0rounding error has resulted in an,
     *   38h ascent direction of search (phase 1).)
 2100 format(32h0required degree of accuracy not,
     *   12h attainable.)
 2110 format(20h0no finite solution.)
 2120 format(31h0vertex requested but no finite,
     *   15h vertex exists.)
 2130 format(27h0the subroutine has found a,
     *   18h stationary point.)
 2140 format(25h0only one feasible point.)
 2150 format(20h0danger of overflow.)
 2160 format(1h0, i5, 26hst constraint is redundant,
     *   10h equality.)
 2170 format(1h0, i5, 26hnd constraint is redundant,
     *   10h equality.)
 2180 format(1h0, i5, 26hth constraint is redundant,
     *   10h equality.)
c
c  end of subroutine QQresu
c
      end
      subroutine QQrsol(n, lel, el, b, y)
      implicit double precision (a-h,o-z)
      integer n, lel
      dimension el(lel), b(n), y(n)
c
c  *********************************************************************
c
c  solves r*y = b for y, where r is the transpose of a lower-triangular
c  matrix stored row by row in el(i), i = 1,2,...,n*(n+1)/2. the right-
c  hand-side is stored in b(i) and the solution in y(i), i = 1,2,...,n.
c
c  philip e. gill, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england.
c  crown copyright reserved
c  february 1976
c
c  *********************************************************************
c
      integer i, ir, is, it, i1, j, k
      it = n*(n + 1)/2 + 1
      do 30 i1 = 1, n
         i = n + 1 - i1
         it = it - 1
         is = it
         sum = b(i)
         j = i + 1
         if (n .lt. j) goto 20
         do 10 k = j, n
            ir = n + j - k
            sum = sum - y(ir)*el(is)
            is = is + 1 - ir
   10    continue
   20    y(i) = sum/el(is)
   30 continue
      return
c
c  end of subroutine QQrsol
c
      end
      subroutine rtsol(n, lel, el, b, y)
      implicit double precision (a-h,o-z)
      integer n, lel
      dimension el(lel), b(n), y(n)
c
c  *********************************************************************
c
c  solves rt*y = b for y, where rt is lower-triangular matrix stored
c  row by row in el(i), i = 1,2,...,n*(n+1)/2. the right-hand-side is
c  stored in b(i) and the solution in y(i), i = 1,2,...,n.
c
c  philip e. gill, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england.
c  crown copyright reserved
c  february 1976
c
c  *********************************************************************
c
      integer i, i1, j, k
      j = 1
      do 30 i = 1, n
         sum = b(i)
         i1 = i - 1
         if (i1 .lt. 1) goto 20
         do 10 k = 1, i1
            sum = sum - y(k)*el(j)
            j = j + 1
   10    continue
   20    y(i) = sum/el(j)
         j = j + 1
   30 continue
      return
c
c  end of subroutine rtsol.
c
      end
      subroutine setbnd(n, ibound, bndmax, bl, bu)
      implicit double precision (a-h,o-z)
      integer n, ibound
      dimension bl(n), bu(n)
c
c  *********************************************************************
c
c  setbnd sets the lower and upper bounds on all variables uniformly
c  to the values indicated by the parameter ibound.  the value of
c  bndmax is not machine-dependent and must not be altered by the user.
c
c  philip e. gill, walter murray, susan m. picken, margaret h. wright
c  and enid m. r. long
c  d.n.a.c., national physical laboratory, england
c  crown copyright reserved
c  september 1976
c
c  *********************************************************************
c
      integer i
      if(ibound .le. 0) return
      bndmax = 1.0e+6
      go to (10, 30, 50, 10), ibound
c
c  set lower bounds to large negative number and upper bounds to large
c  positive number.
c
   10 do 20 i = 1, n
         bl(i) = - bndmax
         bu(i) = bndmax
   20 continue
      return
c
c  set lower bounds to zero and upper bounds to large positive number.
c
   30 do 40 i = 1, n
         bl(i) = 0.0e+0
         bu(i) = bndmax
   40 continue
      return
c
c  set lower bounds and upper bounds to numbers specified by user.
c
   50 if(n .eq. 1) return
      bndl = bl(1)
      bndu = bu(1)
      do 60 i = 2, n
         bl(i) = bndl
         bu(i) = bndu
   60 continue
      return
c
c  end of subroutine setbnd
c
      end
      subroutine QQstov(n, a, b)
      implicit double precision (a-h,o-z)
      integer n
      dimension a(n), b(n)
c
c  *********************************************************************
c
c  stores a(i) in b(i), i = 1(1)n
c
c  philip e. gill, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england.
c  crown copyright reserved
c  september 1976
c
c  *********************************************************************
c
      integer i
      if(n .eq. 0) return
      do 10 i = 1, n
         b(i) = a(i)
   10 continue
      return
c
c  end of QQstov
c
      end
      subroutine storiv(n, jvec, kvec)
      implicit double precision (a-h,o-z)
      integer n
      integer jvec(n), kvec(n)
c
c  *********************************************************************
c
c  storiv stores the elements of the integer vector jvec in the
c  integer vector kvec.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  february 1978
c
c  *********************************************************************
c
      integer i
      if(n .le. 0) return
      do 10 i = 1, n
         kvec(i) = jvec(i)
   10 continue
      return
c
c  end of subroutine storiv
c
      end
      subroutine symhx(n, lh, hesl, hesd, x, y)
      implicit double precision (a-h,o-z)
      integer n, lh
      dimension hesl(lh), hesd(n), x(n), y(n)
c
c  *********************************************************************
c
c  multiplies the n*n symmetric matrix by the vector x and
c  stores the result in y. the matrix has its lower triangle
c  stored row by row in hesl and its diagonal stored in hesd.
c
c  philip e. gill, walter murray, susan m. picken, enid m. r. long
c  d.n.a.c., national physical laboratory, england.
c  crown copyright
c  february 1977
c
c  *********************************************************************
c
      integer i, incm, i1, j, l, m, minit
      l = 1
      minit = 1
      do 50 i = 1, n
         yi = hesd(i)*x(i)
         if(i .eq. 1) goto 20
         i1 = i - 1
         do 10 j = 1, i1
            yi = hesl(l)*x(j) + yi
            l = l + 1
   10    continue
   20    if(i .eq. n) goto 40
         i1 = i + 1
         incm = i
         m = minit
         minit = minit + i1
         do 30 j = i1, n
            yi = hesl(m)*x(j) + yi
            m = m + incm
            incm = incm + 1
   30    continue
   40    y(i) = yi
   50 continue
      return
c
c  end of subroutine symhx
c
      end
      subroutine QQtitl(iupr, iutty, name)
      implicit double precision (a-h,o-z)
      integer iupr, iutty
      integer name(6)
c
c  *********************************************************************
c
c  subroutine QQtitl outputs to device iupr and iutty a heading
c  for the npl numerical optimization software library which
c  includes the name of the subroutine being used, its version
c  number and its date of completion.
c
c  philip e. gill, walter murray, susan m. picken, enid m. r. long
c  and brian t. hinde
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  march 1979
c
c  *********************************************************************
c
      integer i, iyear, j, k, month
      integer mnths(36)
      data  mnths(1),mnths(2),mnths(3),mnths(4),mnths(5),mnths(6),
     *    mnths(7),mnths(8),mnths(9),mnths(10),mnths(11),mnths(12),
     *    mnths(13),mnths(14),mnths(15),mnths(16),mnths(17),mnths(18),
     *    mnths(19),mnths(20),mnths(21),mnths(22),mnths(23),mnths(24),
     *    mnths(25),mnths(26),mnths(27),mnths(28),mnths(29),mnths(30),
     *    mnths(31),mnths(32),mnths(33),mnths(34),mnths(35),mnths(36)
     *   /1hj,1ha,1hn, 1hf,1he,1hb, 1hm,1ha,1hr, 1ha,1hp,1hr,
     *    1hm,1ha,1hy, 1hj,1hu,1hn, 1hj,1hu,1hl, 1ha,1hu,1hg,
     *    1hs,1he,1hp, 1ho,1hc,1ht, 1hn,1ho,1hv, 1hd,1he,1hc/
c
c  set version number, month and year
c
      versn = 1.3e+0
      month = 3
      iyear = 1979
      k = 3*month
      j = k - 2
      write(iupr, 1000) name, versn, (mnths(i), i = j, k), iyear
      if(iupr .eq. iutty) return
      write(iutty, 1000) name, versn, (mnths(i), i = j, k), iyear
      return
c
c   non-executable statement
c
 1000 format(28h1       n p l  n o s l ---- , 6a1, 1x, 8h version,
     *   f5.1, 4x, 3a1, 2x, i4/22h        = = =  = = = =)
c
c  end of subroutine QQtitl
c
      end
      subroutine unscle(n, ngc, lngc, agc, lagc, bl, bu, lb, istate,
     *   scale)
      implicit double precision (a-h,o-z)
      integer n, ngc, lngc, lagc, lb
      integer istate(lngc)
      dimension agc(lagc, lngc), bu(lb), bl(lb), scale(lngc)
c
c  *********************************************************************
c
c  unscales the scaling done in subroutine scle.
c
c  philip e gill, walter murray, susan m picken, enid m r long
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
      integer i, isti, j
      if(ngc .eq. 0) return
      do 20 i = 1, ngc
         sci = scale(i)
         do 10 j = 1, n
   10       agc(j, i) = agc(j, i)*sci
         isti = istate(i)
         if(isti .ne. 2) bl(i) = bl(i)*sci
         if(isti .ne. 1) bu(i) = bu(i)*sci
   20 continue
      return
c
c  end of subroutine unscle
c
      end
      subroutine upresd(na, m, alpha, atp, r)
      implicit double precision (a-h,o-z)
      integer na, m
      dimension atp(m), r(m)
c
c  *********************************************************************
c
c  upresd updates the residuals of the inactive constraints whenever a
c  step of non-zero length alpha has been taken.
c   a positive value of kmin indicates that
c  the constraint with its residual in r(kmin) has become active.
c
c  philip e gill, walter murray, susan m picken, enid m r long
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
      integer i, nina
      nina = m - na
      do 10 i = 1, nina
         ri = r(i)
         ri = ri + alpha*atp(i)
         r(i) = ri
   10 continue
      return
c
c  end of subroutine upresd
c
      end
      subroutine vecsum(n, alpha, a, b, c)
      implicit double precision (a-h,o-z)
      integer n
      dimension a(n), b(n), c(n)
c
c  *********************************************************************
c
c  forms the vector sum c(i) = alpha*a(i) + b(i), for i = 1,2,...,n,
c  where alpha is a double precision variable.
c
c  philip e. gill, walter murray, susan m. picken, enid m. r. long
c  d.n.a.c., national physical laboratory, england.
c  crown copyright
c  april 1977
c
c  *********************************************************************
c
      integer i
      if (n .lt. 1) return
      do 10 i = 1, n
         c(i) = alpha*a(i) + b(i)
   10 continue
      return
c
c  end of subroutine vecsum
c
      end
      double precision function x02aaf(x)
      implicit double precision (a-h,o-z)
c
c  *********************************************************************
c
c  returns the value of epsmch, where epsmch is the smallest possible
c  double precision number such that 1.0 + epsmch .gt. 1.0.
c
c  for cdc 6000/7000 series
c
c  d.n.a.c. national physical laboratory, england.
c  crown copyright 1977.
c
c  *********************************************************************
c
      x02aaf = 2.0**(-47)
      return
c
c  end of x02aaf
c
      end
      subroutine zeroar(m, n, a, la)
      implicit double precision (a-h,o-z)
      integer m, n, la
      dimension a(la,n)
c
c  *********************************************************************
c
c  sets array a(i,j), i = 1(1)m, j = 1(1)n to zero.
c
c  philip e. gill, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england.
c  crown copyright reserved
c  september 1976
c
c  *********************************************************************
c
      integer i, j
      if(n .eq. 0 .or. m .eq. 0) return
      do 10 j = 1, n
         do 10 i = 1, m
            a(i,j) = 0.0e+0
         continue
   10 continue
      return
c
c  end of zeroar
c
      end
      subroutine zeroiv(liv, iv)
      implicit double precision (a-h,o-z)
      integer liv
      integer iv(liv)
c
c  *********************************************************************
c
c  zeroiv sets the elements of the integer vector iv(i), i = 1(1)liv,
c  to zero.
c
c  philip e. gill, enid m. r. long, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  october 1977
c
c  *********************************************************************
c
      integer i
      if(liv .le. 0) return
      do 10 i = 1, liv
         iv(i) = 0
   10 continue
      return
c
c  end of subroutine zeroiv
c
      end
      subroutine zerovc(n, a)
      implicit double precision (a-h,o-z)
      integer n
      dimension a(n)
c
c  *********************************************************************
c
c  sets vector a(i), i = 1(1)n to zero
c
c  philip e. gill, walter murray and susan m. picken
c  d.n.a.c., national physical laboratory, england.
c  crown copyright reserved
c  september 1976
c
c  *********************************************************************
c
      integer i
      if(n .eq. 0) return
      do 10 i = 1, n
         a(i) = 0.0e+0
   10 continue
      return
c
c  end of zerovc
c
      end
      subroutine zpacmp(n, lzt, signum, qt, lqt, pa, zpa, zpasq)
      implicit double precision (a-h,o-z)
      integer n, lzt, lqt
      dimension qt(lqt,n), zpa(n), pa(n)
c
c  *********************************************************************
c
c  starting with the search direction vector pa of dimension lzt,
c  obtained as the solution to equations using the ldlt factors of the
c  projected hessian, zpacmp premultiplies it by the matrix z to give
c  a search direction zpa of dimension n.  it also computes the square
c  of the euclidean norm of zpa.  the parameter signum should be set to
c  + 1 or - 1 to ensure a descent direction.
c
c  philip e gill, walter murray, susan m picken, enid m r long
c  d.n.a.c., national physical laboratory, england
c  crown copyright
c  november 1977
c
c  *********************************************************************
c
      integer i, j, k, l
      sumsq = 0.0e+0
      k = n + 1
      do 20 i = 1, n
         zpi = 0.0e+0
         do 10 j = 1, lzt
            l = k - j
            zpi = zpi + qt(i,l)*pa(j)
   10    continue
         zpa(i) = signum*zpi
         sumsq = sumsq + zpi*zpi
   20 continue
      zpasq = sumsq
      return
c
c  end of subroutine zpacmp
c
      end

