cc
cc    subroutine qpr2
cc    ===============
cc
cc    purpose
cc    =======
cc
cc    qpr2 solves the following quadratic programme:
cc    minimise  0.5*<cq,x>+<x,hq*x> (with respect to x)
cc    subject to: <g1,x>  = b1
cc                <g2,x> >= b2
cc                <g3,x> <= b3
cc                bndl(i)<=x(i)<=bndu(i)  for i=1...nq,
cc    where: cq and x are vectors of length nq;
cc           hq is an nqxnq matrix;
cc           g1 is an nqxme matrix;
cc           g2 is an nqxmge matrix;
cc           g3 is an nq*mle matrix;
cc           b1 is an me vector;
cc           b2 is an mge vector;
cc           b3 is an mle vector;
cc           bndl and bndu are n vectors;
cc    qpr2 also returns the lagrange multipliers .
cc
cc    fortran calling sequence
cc    ========================
cc
cc    subroutine qpr2(nq,mge,mle,me,mmeq,cq,hq,bq,gq,nqw,qw,iqw,xq,
cc                    bndu,bndl,qmult,cost,info)
cc
cc    arguments
cc    =========
cc
cc    1)arguments assigned on entry to qpr2:
cc    nq-dimension of the vector x;
cc    mge-number of inequality constraints of the form <g2,x> >= b2
cc    mle-number of inequality constraints of the form <g3,x> <= b3
cc    me-number of equality constraints of the form <g1,x> = b1
cc    mmeq=mge+mle+me where me is the number of equality constraints;
cc    cq-nq vector, the linear part of the cost;
cc    hq-nqxnq matrix, quadratic part of the cost;
cc                 T
cc    bq-the vector (b1,b2,b3) of length mmeq;
cc    gq-the nqxmmeq matrix (g1)
cc                 (g2)
cc                 (g3);
cc    nqw=nq*(mmeq+3*nq+21)+15*mmeq+160;
cc    bndu-nq vector of upper bounds on the basic variable;
cc    bndl-nq vector of lower bounds on the basic variable;
cc    xq-nq vector, must be initialized (e.g. xq(i)=0., i+1, nq).
cc
cc    2)arguments returned on exit from qpr2:
cc    xq-nq vector which contains the minimising x in the quadratic
cc    programme;
cc    qmult-vector of length (me+mge+mle+n+n). in its first me (mge, mle,nq,nq) 
cc    places it contains the set of lagrange multipliers corresponding 
cc    to the active equality (greater or equal, less or equal, simple greater
cc    or equal, simple less or equal) constraints at the solution;
cc    cost-cost function
cc    info-a flag indicator which operates as follows:
cc    info=0=> successful return from lcqp.
cc    info=1=> ill-defined input parameters on lcqp.
cc    info=2=> eccessive iterations
cc    info=3=> some of the constraints are redundant.
cc    info=4=> the program does not admit a feasible solution.
cc             for further information see prologue to lcqp.
cc
cc    3)other arguments
cc    qw-vector of lenght nqw which is used as workspace;
cc    iqw-integer vector of lenght nqw which is used as workspace.
cc
      subroutine qpr2(nq,mge,mle,me,mmeq,cq,hq,bq,gq,nqw,qw,
     +                iqw,xq,bndu,bndl,qmult,cost,info)
      implicit double precision (a-h , o-z)
c
cX    dimension xq(nq),qmult(mge+me+mle+2*n)
cX: The above expression-dimension failed on VMS.
c
      dimension xq(nq),qmult(1)
      dimension cq(nq),hq(nq,nq),bq(mmeq),gq(nq,mmeq),qw(nqw)
      dimension iqw(nqw),bndu(nq),bndl(nq)
      ngc=mle+mge+me
      lngc=mmeq
      lcon=2*(ngc+nq)
      lh=max(nq*(nq-1)/2,1)
      lagc=nq
      if(ngc.eq.0) lagc=1
      ibound=0
      lb=nq+ngc
      initpt=0
      msglvl=0000
      iupr=6
      iutty=6
      maxit=50*nq
      lel=nq*(nq+1)/2
      lqt=nq
      liw=2*nq+ngc+160
      lw=9*nq+7*ngc
c
c     set up g' (transpose)
c
      ii=1
      if(ngc.gt.0) then
        do 10 j=1,mmeq
          do 10 i=1,nq
            qw(ii)=gq(i,j)
10      ii=ii+1
      endif
      n1=mmeq*nq+1
c
c     set up array of lenght nq which contains the diagonal
c     elements of matrix hq
c
      ii=n1
      do 20 i=1,nq
        qw(ii)=hq(i,i)
20    ii=ii+1
      n2=n1+nq
c
c     set up array of lenght lh which contains the
c     subdiagonal elements of matrix hq.
c
      ii=n2
      if(nq.gt.1) then
        do 30 j=2,nq
          do 30 i=1,j-1
            qw(ii)=hq(i,j)
            ii=ii+1
30      continue
      endif
      n3=n2+lh
c
c      set up integer array of lenght mmeq giving
c      the type of each general constraint;
c               0  if  g'*x eq b
c               1  if  g'*x ge b
c               2  if  g'*x le b
c
       if(ngc.gt.0) then
         ii=n3
         do 40 i=1,mmeq
           iqw(ii)=2
           if(i.le.(mge+me)) iqw(ii)=1
           if(i.le.me) iqw(ii)=0
           ii=ii+1
40       continue
       endif
      n4=mmeq+n3
c
c      setup the upper and lower bounds on the general
c      constraint and basic variables.
c
      n5=n4+lb
      ii=n4
      jj=n5
      if(me.gt.0) then 
        do 50 i=1,me
          qw(ii)=bq(i)
          qw(jj)=bq(i)
          ii=ii+1
          jj=jj+1
50      continue
      endif
      if(mge.gt.0) then
        do 51 i=1,mge
          qw(jj)=bq(i+me)
          ii=ii+1
          jj=jj+1
51      continue
      endif
      if(mle.gt.0) then
        do 52 i=1,mle
          qw(ii)=bq(i+me+mge)
          ii=ii+1
          jj=jj+1
52      continue
      endif
c
      do 60 i=1,nq
        qw(ii)=bndu(i)
        qw(jj)=bndl(i)
        ii=ii+1
        jj=jj+1
60    continue
c
      n6=n5+lb
      n7=n6+lcon
      n8=n7+nq
      n9=n8+lcon
      n10=n9+lel
      n11=n10+nq*lqt
      n12=n11+lh
      n13=n12+nq
      n14=n13+liw
      n15=n14+lw
c
      call lcqp(nq,ngc,lngc,lcon,lh,qw(1),lagc,ibound,
     +         qw(n5),qw(n4),lb,iqw(n3),initpt,
     +         msglvl,iupr,iutty,maxit,qw(n2),qw(
     +         n1),cq,xq,cost,na,umin,qw(n6),qw(n7),
     +         iqw(n8),qw(n9),lel,qw(n10),lqt,
     +         qw(n11),qw(n12),info,iqw(n13),liw,
     +         qw(n14),lw)
      if(info.ne.0.or.na.eq.0) return
c
      nq2=nq*2
      ngcnq2=ngc+nq2
      do 65 i=1,ngcnq2
        qw(i)=0.
65    qmult(i)=0.
c
      do 70 k=1,nq
        if(k.le.me) then
          qw(k)=qw(n7+k-1)
        else
          nco=iqw(n8+k-1)
          if(nco.le.nq2) then
            qw(ngc+nco)=qw(n7+k-1)
            go to 70
          endif
          if(nco.le.ngcnq2) then
            qw(nco-nq2)=qw(n7+k-1)
            go to 70
          endif
          if(nco.le.2*(ngc+nq)) then
            qw(nco-ngcnq2)=qw(n7+k-1)
            go to 70
          endif
        endif
70    continue
      if(ngc.gt.0) then
        do 80 j=1,ngc
          scale=0.
          do 90 i=1,nq
90        scale=scale+gq(i,j)**2
          scale=dsqrt(scale)
80      qmult(j)=qw(j)/scale
      endif
      do 100 j=ngc+1,ngcnq2
100   qmult(j)=qw(j)
      return
      end

