/*  sens_read.c
 *
 *   Author: Umakanta Choudhury
 *   Date:   29th June
 *   Last modified:  Enrico Malavasi,  Aug. 1992
 *
 *   Purpose: Read sensitivities from the output of senscalc
 */
#include <stdio.h>
#include <string.h>
#include "layout.h"
#include "circuit.h"
#include "avl.h"
#define MAX_LINE_LENGTH 200
#define PRINT_LINE_DATA {printf("%s: line %d\n",text_file_name,linenum);}
#define DECREMENT_NUMBER_OF_ITEMS \
  {if( (--number_of_items) == 0) readin_status = READING_NORMAL;}

/*  READIN STATUS DEFINITIONS */
#define READING_NORMAL 0
#define READING_PERFORMANCE 1
#define READING_NODES 2

/*
 *  "sensitivity file" input syntax:
 *  -----------------------------
 *
 *	performance name number_of_items 
 *	node1_name	node2_name	sensitivity
 *	.
 *	.
 *	(repeated format)
 *
 */

void
  sens_read(text_file_name, circuit, layout)
char *text_file_name;		/* The input text file			*/
struct Circuit *circuit;	/* The circuit structure */
struct Layout *layout;	/* The layout structure */
{
    FILE *text_file;		/* The input text file			*/
    char	text_line[MAX_LINE_LENGTH];
    int		linenum = 1;
    int		readin_status = READING_NORMAL;
    int		number_of_items = 0;
    float  value;
    struct	Performance *perf; 
    char	name[30];
    char	*nameA[30];
    char	*nameB[30];
    struct	Node	*node;
    int	number;
    struct Sens_link	*sens_link;
    struct Sens_link	*stored_link;
    struct Node	*nodeA;
    struct Node	*nodeB;
    struct Channel	*channel;
    struct ChannelLink	*chan_link;
    struct Chan_info	*chan_info;
    struct Chan_info_list_link	*chan_info_list_link;
    int	present;

    /* Open the text file */
    FOPEN( text_file, text_file_name, "r");
    VERBOSE( fprintf(stderr, "Reading the sensitivy file '%s'\n", text_file_name));

    /* Read file, line by line */
    for(linenum = 1 ; ; linenum++) {
        if(fgets(text_line,MAX_LINE_LENGTH,text_file) == NULL)
	  break;
	
        /* Check for comment or empty line */
        if(text_line[0] == '!') continue;
	if(strlen(text_line) == 1) continue;
	/* first read the performance data */	
	
	/* Status determines whether aything specific is expected next */
	switch(readin_status) {
	    case(READING_NORMAL): {
		if(sscanf(text_line,"number of %s sensitivities = %d", name, &number_of_items) == 2){
		    readin_status = READING_PERFORMANCE;
		    if(avl_lookup(circuit->perf_tree,name,&perf) == NULL){
			printf("performance %s not found in input\n",name);
			exit( -1 );
		    }
		} else {
		    PRINT_LINE_DATA;
		    printf("Text line = %s\n",text_line);
		    printf("Unrecognized text \n");
		    exit( -1 );
		}
	    break;
	    }
	    case(READING_PERFORMANCE): {
		if(sscanf(text_line,"%s %s %g",nameA,nameB,&value)==3) {
		    /* Get nodeA and nodeB */
		    if(avl_lookup(circuit->node_tree,nameA,&nodeA) == NULL) {
			printf("node %s not found in circuit\n",nameA);
			exit( -1 );
		    }
		    if(avl_lookup(circuit->node_tree,nameB,&nodeB) == NULL) {
			printf("node %s not found in circuit\n",nameB);
			exit( -1 );
		    }
		    
		    ds_MakeSensLink(perf,value, nodeA, nodeB, &sens_link,&stored_link);
		    
		    /* check if there is a chan_info_list_link
		       already present for the pair */
		    
		    if( ds_FindChanInfoListLink(nameA,nameB, &chan_info_list_link,layout) == NO) {
			ds_MakeChanInfoListLink(layout,nameA,nameB, &chan_info_list_link);
		    }
		    sens_link->chan_info_list   = chan_info_list_link->chan_info_list;
		    stored_link->chan_info_list = chan_info_list_link->chan_info_list;
		    
		    DECREMENT_NUMBER_OF_ITEMS;
		}
		else{
		    PRINT_LINE_DATA;
		    printf("Unrecognized text.\n");
		    exit( -1 );
		}
		break;
	    }
	  default: {
	      PRINT_LINE_DATA;
	      printf("Program error. Nothing expected - Why not?\n");
	      exit( -1 );
	  }
 	} /* End of switch */
    } 
}
