/*      ROUTINE FOR DECIDING WHICH CROSSOVERS
*		TO SHIELD
*       ---------------------------------------
*
*   File:       shield_decide.c
*   Author:     Umakanta Choudhury
*   Date:       24th June 1989
*
*
*/

#include <stdio.h>
#include "utility.h"
#include "layout.h"
#include "circuit.h"


void
  shield_decide(layout,circuit)
struct Layout *layout;
struct Circuit *circuit;
{
    struct Performance *perf;
    struct CriticalPairLink *critpair;
    int perf_num;
    float max_pos_change;
    float max_neg_change;
    struct Chan_info *chan_info;
    double sens;
    float delta_pos_min;
    float delta_neg_min;
    struct Channel *channel;
    struct Crit_temp *crit_temp_list;
    struct Crit_temp *crit_temp;
    char netA_name[30];
    char netB_name[30];
    
    for(perf = circuit->perf_list; perf != NULL; perf = perf->next){
	perf_num = perf->number;
	
	if(perf->pos_constraint == YES){
	    /* constraint on maximum positive change present */
	    max_pos_change = perf->max_pos_change;
	    delta_pos_min = perf->delta_pos_min;
	    /* first subtract the contributions of
	       all the crossovers which have been 
	       marked shielded */
	    for(critpair = layout->critical_pairs; critpair != NULL;
		critpair = critpair->next){
		sens = *(critpair->pos_sens + perf_num);
		for(chan_info = critpair->chan_info_list;chan_info != NULL; chan_info = chan_info->next){
		    if(chan_info->shielded == YES){
			delta_pos_min -= sens * chan_info->Ccross_min;
			if(delta_pos_min < max_pos_change){
			    /* delta_pos_min taken care of */
			    perf->delta_pos_min = delta_pos_min;
			    goto next;
			}
		    }
		}
	    }
	    if(delta_pos_min >= max_pos_change){
		/* shielding necessary */
		crit_temp_list = NULL;
		/* form a temporary list for sorting the sensitivities */
		for(critpair = layout->critical_pairs; critpair != NULL; critpair = critpair->next){
		    sens = *(critpair->pos_sens + perf_num);
		    ds_MakeCritTemp(&crit_temp_list,&crit_temp);
		    crit_temp->netA_name =  util_strsav(critpair->netA_name);
		    crit_temp->netB_name =  util_strsav(critpair->netB_name);
		    crit_temp->sens = (- sens);
		    /* want the most positive sensitivity to be the first in linked list */
		}
		sens_order(&(crit_temp_list));
		for(crit_temp = crit_temp_list; crit_temp != NULL; crit_temp = crit_temp->next){
		    strcpy(netA_name,crit_temp->netA_name);
		    strcpy(netB_name,crit_temp->netB_name);
		    /* find the critical pair link with the same nets */
		    for(critpair = layout->critical_pairs; critpair 
			!= NULL; critpair = critpair->next){
			if((strcmp(critpair->netA_name,netA_name) == 0) &&
			   (strcmp(critpair->netB_name,netB_name) == 0)){
			    sens = *(critpair->pos_sens + perf_num);
			    for(chan_info = critpair->chan_info_list; chan_info!=NULL; chan_info = chan_info->next){
				/* shield the crossovers between netA and netB in this channel */
				chan_info->C_min_shield -= chan_info->Ccross_min;
				delta_pos_min -= sens * chan_info->Ccross_min;
				chan_info->shielded = YES;
				/* take into account shieldings
				   causd by other performance functions 
				   (not yet implemented) */
				if (delta_pos_min <	max_pos_change){
				    perf->delta_pos_min = delta_pos_min;
				    goto next;/* delta_pos_min taken care of */
				}
			    }
			}
		    }
		}
	    }
	}

next:	if(perf->neg_constraint == YES){
	    /* constraint on maximum negative change present */
	    max_neg_change = perf->max_neg_change;
	    delta_neg_min = perf->delta_neg_min;
	    /* first subtract the contributions of
	       all the crossovers which have been 
	       marked shielded */
	    for(critpair = layout->critical_pairs; critpair != NULL; critpair = critpair->next){
		sens = *(critpair->neg_sens + perf_num);
		for(chan_info = critpair->chan_info_list; chan_info != NULL; 
		    chan_info = chan_info->next){
		    if(chan_info->shielded == YES){
			delta_neg_min -= sens * chan_info->Ccross_min;
			if(delta_neg_min > max_neg_change){
			    /* delta_neg_min taken care of */
			    perf->delta_neg_min = delta_neg_min;
			    goto last;
			}
		    }
		}
	    }
	    if(delta_neg_min <= max_neg_change){
		/* shielding necessary */
		crit_temp_list = NULL;
		/* form a temporary list for sorting the sensitivities */
		for(critpair = layout->critical_pairs; critpair != NULL; critpair = critpair->next){
		    sens = *(critpair->neg_sens + perf_num);
		    ds_MakeCritTemp(&crit_temp_list,&crit_temp);
		    crit_temp->netA_name =  util_strsav(critpair->netA_name);
		    crit_temp->netB_name =  util_strsav(critpair->netB_name);
		    crit_temp->sens = sens; 
		}
		sens_order(&(crit_temp_list));
		for(crit_temp = crit_temp_list; crit_temp != NULL; crit_temp = crit_temp->next){
		    strcpy(netA_name,crit_temp->netA_name);
		    strcpy(netB_name,crit_temp->netB_name);
		    /* find the critical pair link with
		       the same nets */
		    for(critpair = layout->critical_pairs; critpair 
			!= NULL; critpair = critpair->next){
			if((strcmp(critpair->netA_name,netA_name) == 0)
			   && (strcmp(critpair->netB_name,netB_name) == 0)){
			    sens = *(critpair->neg_sens + perf_num);
			    for(chan_info = critpair->chan_info_list;chan_info != NULL; 
				chan_info = chan_info->next){
				/* shield the crossovers between 
				   netA and netB in this channel */
				chan_info->C_min_shield -= chan_info->Ccross_min;
				delta_neg_min -= sens * chan_info->Ccross_min;
				chan_info->shielded = YES;
				if (delta_neg_min > max_neg_change) {
				    perf->delta_neg_min = delta_neg_min;
				    goto last;/* delta_neg_min taken care of */
				}
			    }
			}
		    }
		}
	    }
	}
last: ;	
    }
}
