/*		ROUTINE FOR ESTIMATING MAXIMUM AND
*		MINIMUM SPANS OF THE NETS
*       OF EACH CHANNEL
*       --------------------------------
*
*   File:       span_maxmin.c
*   Author:     Umakanta Choudhury
*   Date:       9th June 1989
*
*
*/

#include <stdio.h>
#include "macros.h"
#include "layout.h"

extern int strcmp();

void
  span_maxmin(layout)
struct Layout *layout;
{
    struct ChannelLink *chan_link;
    struct ChannelLink *top_chan_link;
    struct ChannelLink *bottom_chan_link;
    struct Channel	*channel;
    struct Channel	*top_chan;
    struct Channel	*bottom_chan;
    struct NetLink *net_link;
    struct Net *net;
    struct Pin *pin;
    struct Pin *left_most_pin;
    struct Pin *right_most_pin;
    float left; 
    float right; 
    float max_span_left; 
    float min_span_left; 
    float max_span_right; 
    float min_span_right; 
    float chan_net_sep; 
    
    for(chan_link = layout->chan_list; chan_link != NULL;
	chan_link = chan_link->next){
	
	channel = chan_link->channel;
	chan_net_sep = channel->chan_net_separation;
	DEBUGG(	printf("span_maxmin: channel %s:\n",channel->name) );

	/* scan through each net and order the pins */
	for(net_link = channel->net_list; net_link != NULL; 
	    net_link = net_link->next){
	    
	    net = net_link->net;
	    order_pins(&(net->pins),&(net->last_pin_link));
	    DEBUGG( printf("span_maxmin: net %s:\n",net->name) );
	    
	    if(net->left_end_crossing == FALSE) {
		
		/* need to find left most fixed pin */
		left_most_pin = net->pins->pin;
		net->max_span_left = left_most_pin->x_position 
		  - net->true_half_width;
		net->min_span_left = left_most_pin->x_position
		  - net->true_half_width;
		
		DEBUGG( printf("span_maxmin: max_span_left %g:\n",net->max_span_left));

		/* scan through the top channels to find the leftmost 
		   floating pin */
		for(top_chan_link = channel->top_chan_list; 
		    top_chan_link != NULL; 
		    top_chan_link = top_chan_link->next){
		    
		    top_chan = top_chan_link->channel;
		    
		    /* check whether the net is present in the top 
		       channel left_list */
		    if(top_chan != NULL){
			if(net_present(top_chan->left_crosses,net) == YES) {
			    
			    /* left position for maximum span */
			    net->max_span_left 
			      = MIN(top_chan->bottom + chan_net_sep,
				    net->max_span_left);
			    
			    /* left position for minimum span */
			    net->min_span_left 
			      = MIN(top_chan->top - chan_net_sep
				    - (2 * net->true_half_width), 
				    net->min_span_left);
			    
			    break;
			}
		    }
		}

		for(bottom_chan_link = channel->bottom_chan_list; 
		    bottom_chan_link != NULL; 
		    bottom_chan_link = bottom_chan_link->next){
		  
		    bottom_chan = bottom_chan_link->channel;
		  
		    /* check whether the net is present in the 
		       bottom channel right_list */
		    if(bottom_chan != NULL){
			if(net_present(bottom_chan->right_crosses,net) == YES){ 
			  
			  /* left position for maximum span */
			  net->max_span_left 
			    = MIN(bottom_chan->bottom 
				  + chan_net_sep,
				  net->max_span_left);
			  
			  /* left position for minimum span */
			  net->min_span_left 
			    = MIN(bottom_chan->top - chan_net_sep
				  - (2 * net->true_half_width), 
				  net->min_span_left);
			  break;
		      }
		  }
	      }
	    }
	    else{
		net->max_span_left = channel->left;
		net->min_span_left = channel->left;
	    }
	    
	    if(net->right_end_crossing == FALSE){
		/* need to find right most fixed pin */
		right_most_pin = net->last_pin_link->pin;
		net->max_span_right = right_most_pin->x_position
		  + net->true_half_width;
		net->min_span_right = right_most_pin->x_position
		  + net->true_half_width;
		
		/* scan through the top channels in reverse order 
		   to find the rightmost floating pin */
		for(top_chan_link = channel->last_top_chan_link; 
		    top_chan_link != NULL;
		    top_chan_link = top_chan_link->prev){
		    
		    top_chan = top_chan_link->channel;
		    
		    /* check whether the net is present in the top 
		       channel left_list */
		    if(top_chan != NULL) {
			if(net_present(top_chan->left_crosses,net) == YES){ 
			    
			    /* right position for maximum span */
			    net->max_span_right = MAX(top_chan->top
						      - chan_net_sep, net->max_span_right);
			    
			    /* right position for minimum span */
			    net->min_span_right = MAX(top_chan->bottom
						      + chan_net_sep 
						      + (2 * net->true_half_width),
						      net->min_span_right);
			    
			    break;
			}
		    }
		}

		for(bottom_chan_link = channel->last_bottom_chan_link; 
		    bottom_chan_link != NULL; 
		    bottom_chan_link = bottom_chan_link->prev){
		    
		    bottom_chan = bottom_chan_link->channel;
		    
		    /* check whether the net is present in the 
		       bottom channel right_list */
		    if(bottom_chan != NULL){
			if(net_present(bottom_chan->right_crosses,net) 
			   == YES){ 
			    
			    /* right position for maximum span */
			    net->max_span_right = MAX(bottom_chan->top
						      - chan_net_sep, net->max_span_right);
			    
			    /* right position for minimum span */
			    net->min_span_right = MAX(bottom_chan->bottom
						      + chan_net_sep 
						      + (2 * net->true_half_width),
						      net->min_span_right);
			    
			    break;
			}
		    }
		}
	    }
	    else{
		net->max_span_right = channel->right;
		net->min_span_right = channel->right;
	    }
	}
    }
}
