/*      ROUTINE FOR CREATING AND LINKING 
*		THE PIN SUBLISTS
*       ---------------------------------------
*
*   File:       sublist.c
*   Author:     Umakanta Choudhury
*   Date:       10th July 1989
*
*
*/

#include <stdio.h>
#include "layout.h"
#include "circuit.h"

REPORT sublist(layout)
struct Layout *layout;
{
    struct Channel *chan;
    struct Channel *top_chan;
    struct Channel *bottom_chan;
    struct PinLink *pin_link;
    struct PinLink *prev_pin_link;
    struct PinLink *new_pin_link;
    struct Pin_list_link *pin_list_link;
    struct ChannelLink *chan_link1;
    struct ChannelLink *chan_link2;
    struct ChannelLink *last_link;
    float top_chan_right;
    float bottom_chan_right;
    int	last_pin;


    for(chan_link1=layout->chan_list ; chan_link1!=NULL ; chan_link1 = chan_link1->next) {
	chan = chan_link1->channel;
	
	/* order the top pins of the channel in increasing x positions */
	order_pins(&(chan->top_pins),&(chan->last_top_pin_link));
	
	/* order the bottom pins of the channel in increasing x positions */
	order_pins(&(chan->bottom_pins),&(chan->last_bottom_pin_link));
	
	/* order the top channels of the channel in increasing x positions */
	order_chans(&(chan->top_chan_list),&(chan->last_top_chan_link));
	
	/* order the bottom channels of the channel in increasing x positions */
	order_chans(&(chan->bottom_chan_list), &(chan->last_bottom_chan_link));

	/* get the last top pin link */
	pin_link = prev_pin_link = chan->last_top_pin_link;
	
	/* get the last top channel link */
	last_link = chan->last_top_chan_link;
	
	/* scan through the top channels in the reverse order */
	for(chan_link2 = last_link; chan_link2 != NULL; chan_link2 = chan_link2->prev){
	    
	    top_chan = chan_link2->channel;
	    
	    /* form a new pin list link */
	    ds_MakePinListLink(chan,TOP,&pin_list_link);
	    pin_list_link->channel = top_chan;

	    /* form a sublist of pins for each top channel */
	    /* right position of top channel with respect  to channel coordiante system */
	    top_chan_right = top_chan->top;

	    last_pin = YES;
	    for(pin_link = prev_pin_link; pin_link != NULL;
		pin_link = pin_link->prev){
		
		if(pin_link->pin->x_position < top_chan_right){
		    /* crossed to the left of the top channel */
		    prev_pin_link = pin_link;
		    break;
		}
		else{
		    /* form a new pin link in the sub list */
		    ds_MakePinLink(pin_list_link,&new_pin_link);
		    
		    pin_list_link->pin_list->pin = pin_link->pin;
		    if(last_pin == YES){
			pin_list_link->last_pin_link = pin_link;
			last_pin = NO;
		    }
		}
	    }
	}
	
	/* form the sublist of pins to the left of first top channel */
	if(pin_link != NULL){
	    
	    /* form a new pin list link */
	    ds_MakePinListLink(chan,TOP,&pin_list_link);
	    
	    pin_list_link->channel = NULL; /* no top channel to 
					      the left */
	    
	    last_pin = YES;
	    for(pin_link = prev_pin_link; pin_link != NULL;
		pin_link = pin_link->prev){
		
		/* form a new pin link in the sub list */
		ds_MakePinLink(pin_list_link,&new_pin_link);
		
		pin_list_link->pin_list->pin = pin_link->pin;
		
		if(last_pin == YES){
		    pin_list_link->last_pin_link = pin_link;
		    last_pin = NO;
		}
	    }
	}
	
	/* get the last bottom pin link */
	
	prev_pin_link = chan->last_bottom_pin_link;
	pin_link = prev_pin_link;
	
	/* get the last bottom channel link */
	
	last_link = chan->last_bottom_chan_link;
	
	
	/* scan through the bottom channels in the reverse order */
	
	for(chan_link2 = last_link; chan_link2 != NULL; chan_link2 = chan_link2->prev){
	    
	    bottom_chan = chan_link2->channel;
	    
	    /* form a new pin list link */
	    ds_MakePinListLink(chan,BOTTOM,&pin_list_link);
	    pin_list_link->channel = bottom_chan;
	    
	    
	    /* form a sublist of pins for each bottom channel */
	    
	    bottom_chan_right = bottom_chan->bottom; /* right position of bottom 
							channel with respect  to channel coordiante system */
	    
	    last_pin = YES;
	    for(pin_link = prev_pin_link; pin_link != NULL;
		pin_link = pin_link->prev){
		
		if(pin_link->pin->x_position < bottom_chan_right){
		    /* crossed to the left of the bottom channel */
		    prev_pin_link = pin_link;
		    break;
		}
		else{
		    /* form a new pin link in the sub list */
		    ds_MakePinLink(pin_list_link,&new_pin_link);
		    
		    pin_list_link->pin_list->pin = pin_link->pin;
		    if(last_pin == YES){
			pin_list_link->last_pin_link = pin_link;
			last_pin = NO;
		    }
		}
	    }
	}

	/* form the sublist of pins to the left of first bottom channel */
	if(pin_link != NULL){
	    ds_MakePinListLink(chan,BOTTOM,&pin_list_link);
	    
	    pin_list_link->channel = NULL; /* no bottom channel to 
					      the left */
	    
	    last_pin =YES;
	    for(pin_link = prev_pin_link; pin_link != NULL;
		pin_link = pin_link->prev){
		
		/* form a new pin link in the sub list */
		ds_MakePinLink(pin_list_link,&new_pin_link);
		
		pin_list_link->pin_list->pin = pin_link->pin;
		if(last_pin == YES){
		    pin_list_link->last_pin_link = pin_link;
		    last_pin = NO;
		}
	    }
	}
    }
    return OK;
}
