%{
#include <stdio.h>
#include "utility.h"
#include <ctype.h>

extern char yytext[1024];
%}

%union {
    int    intval;
    double doubval;
    char  *strval;
}

%start file

%token <intval>
 BGN
 END
 IGNORE
 WORD
 NAME
 PLUS
 MINUS
 OPEN_SQUARE
 CLOSE_SQUARE
 OPEN_ROUND
 CLOSE_ROUND
 COMMA

 CHANNEL
 DECKFILE
 SIMULATOR
 MATCHNODES
 OUTNODES
 VARIATION
 MISMATCH
 RANGE
/*  record tokens  */
 SIMULATION
 PERFORMANCE
 TECHNOLOGY
 LAYOUT

 EQUAL
 NODENUMBER
 NUMBER
 MAXVAR

%type <strval>
 word
 bound

%%

file   :    lline
       |    file lline
       ;

lline  :
            simulation_record
       |    technology_record
       |    performance_record
       |    matchnodes_record
       |    layout_record
       ;

simulation_record :
            BGN SIMULATION
                simulation_fields
            END SIMULATION
       ;

simulation_fields : /* empty */
       |    simulation_fields SIMULATOR word
            {  setSimulator( $3 ); }
       |    simulation_fields matchnodes_record
       |    simulation_fields DECKFILE  word
            {  setDeckfile( $3 );  }
       |    simulation_fields OUTNODES word word
            {  setOutNodes( $3, $4 ); }
       ;

technology_record:
            BGN TECHNOLOGY
                technology_fields
            END TECHNOLOGY
       ;

technology_fields: /* empty */
       |    technology_fields NAME word
            { setTechnologyName( $3 ); }
       |    technology_fields VARIATION  word
            { setProcessVariation( $3 ); }
       |    technology_fields MISMATCH   word
            { setProcessMismatch( $3 ); }
       ;

performance_record:
            BGN PERFORMANCE
                performance_fields
            END PERFORMANCE
            {    endPerformance();  }
       ;

performance_fields:  /* empty */
       |    performance_fields NAME word
            {  setPerfName( $3 );  }
       |    performance_fields maxvar
       |    performance_fields RANGE  open_bracket word COMMA word close_bracket
            {  setPerfRange( $4, $6 );  }
       ;

layout_record:
            BGN LAYOUT
                layout_fields
            END LAYOUT
       ;

layout_fields: /* empty */
       |    layout_fields CHANNEL word
            { readChannel( $3 ); }
       |    layout_fields "layout_resolution" word
            { lyParameter( "layout_resolution", $3 ); }
       ;

maxvar:      MAXVAR  open_bracket bound COMMA bound close_bracket
             {  setPerfChange( $3, $5 ); }
       ;

bound:  /* empty */
             { $$ = util_strsav( "infinity"); }
       |   word
             { $$ = util_strsav( $1 ); }
       ;

matchnodes_record :
            BGN MATCHNODES
               matchnodes_fields
            END MATCHNODES
       ;

matchnodes_fields : /* empty */
       |    matchnodes_fields open_bracket word COMMA word close_bracket
            {   setMatchedPair( $3, $5 );  }
       ;
 
otherlines: /* empty */
       |    otherlines line
       ;

line   :    /* empty */
       |    |    word line
       ;

word   :     WORD
             { $$ = util_strsav(yytext); }
       ;

open_bracket: OPEN_SQUARE
       |      OPEN_ROUND
       ;

close_bracket: CLOSE_SQUARE
       |       CLOSE_ROUND
       ;
