/*
 *  FILE:
 *
 *  DESCRIPTION:
 *
 *
 *  CREATION DATE: 10/13/87		AUTHOR: A.Casotto
 *
 *  FUNCTIONS:
 */
#include "rpc.h"
#include "nle.h"

#define VM(m) vemMessage(m, MSG_DISP|MSG_NOLOG)

#ifndef MAX
#define MAX(x,y) (x > y) ? x : y
#endif

#ifndef MIN
#define MIN(x,y) (x > y) ? y : x
#endif

/*
 *   GLOBAL variables for this file
 */
RPCArg		*gArg1;		/* argument passed by vem (a BAG) */
octObject 	gFacet;		/* facet to work on */

/*
 *  rpcSoftFlat() :: flat only instances and nets, no geometry
 */
rpcSoftFlat(spot, cmdList , userOptionWord)
     RPCSpot *spot;
     lsList cmdList;
     long userOptionWord;
{
  octObject iBag;
  octObject OutputBag;
  struct octBox redrawBox;
  

  vemMessage("Making Soft Flat ...\n", MSG_DISP|MSG_NOLOG);

  if (checkFlatArgs(spot, cmdList) != NLE_OK) return(RPC_OK);

  gFacet.objectId = spot->facet;
  octGetById(&gFacet);
  iBag.objectId = gArg1->argData.objArg.theBag;
  octGetById(&iBag);
  
  OutputBag.type = OCT_BAG;
  OutputBag.contents.bag.name = "BAG TO CONTAIN ALL THE NEW INSTANCES FROM FLAT";
  octCreate(&gFacet,&OutputBag);

  getBagBB(&iBag,&redrawBox);	/* For redisplay */

  if (nleFlat(&iBag, &OutputBag, NLE_SOFT) != NLE_OK) {
    VM("Problems with FLAT\n");
  } else {
    VM("Flat done\n");
  }

  /*
   * Redisplay chip
   */
  vemWnQRegion(&gFacet,&redrawBox);
  vemWnFlush();
  octDelete(&OutputBag);

  return(RPC_OK);
}



/*
 *  nleHardFlat() :: flat "with geometry"
 */
rpcHardFlat(spot, cmdList , userOptionWord)
     RPCSpot *spot;
     lsList cmdList;
     long userOptionWord;
{
  octObject iBag;
  octObject OutputBag;
  struct octBox redrawBox;
  
  VM("Making Hard Flat ...\n");

  if (checkFlatArgs(spot, cmdList) != NLE_OK) return(RPC_OK);

  gFacet.objectId = spot->facet;
  octGetById(&gFacet);
  iBag.objectId = gArg1->argData.objArg.theBag;
  octGetById(&iBag);
  
  OutputBag.type = OCT_BAG;
  OutputBag.contents.bag.name = "BAG TO CONTAIN ALL THE NEW INSTANCES FROM FLAT";
  octCreate(&gFacet,&OutputBag);

  getBagBB(&iBag,&redrawBox);

  if (nleFlat(&iBag, &OutputBag, NLE_HARD) != NLE_OK) {
    VM("Problems with FLAT\n");
  } else {
    VM("Flat done\n");
  }

  /*
   * Redisplay chip
   */
  vemWnQRegion(&gFacet,&redrawBox);
  vemWnFlush();
  octDelete(&OutputBag);

  return(RPC_OK);
}



getBagBB(BagP,redrawBoxP)
    octObject *BagP;
    struct octBox *redrawBoxP;
{
  octGenerator genInst;
  octObject	Inst;
  struct octBox	tmpBox;
  int		counter = 0;	/* counts the objects with BBox */

  octInitGenContents(BagP,OCT_INSTANCE_MASK,&genInst);
  while (octGenerate(&genInst,&Inst) == OCT_OK) {
    if (counter == 0) {
      if (octBB(&Inst,redrawBoxP)==OCT_OK) counter++;
    } else {
      if (octBB(&Inst,&tmpBox)==OCT_OK) {
	counter++;
	redrawBoxP->lowerLeft.x = MIN(redrawBoxP->lowerLeft.x, tmpBox.lowerLeft.x);
	redrawBoxP->lowerLeft.y = MIN(redrawBoxP->lowerLeft.y, tmpBox.lowerLeft.y);
	redrawBoxP->upperRight.x = MAX(redrawBoxP->upperRight.x, tmpBox.upperRight.x);
	redrawBoxP->upperRight.y = MAX(redrawBoxP->upperRight.y, tmpBox.upperRight.y);
      }
    }
  }
  if (counter == 0) {
    return(NLE_NOT_OK);
  }
  return(NLE_OK);
}


rpcFlatUsage()
{
  vemMessage("format: [object set] : flat\n", MSG_DISP|MSG_NOLOG);
}

checkFlatArgs(spot, cmdList)
     RPCSpot *spot;
     lsList cmdList;
{
  octObject Facet;
  
  if (lsLength(cmdList) != 1) {
    rpcFlatUsage();
    return(NLE_NOT_OK);
  }
  Facet.objectId = spot->facet;
  if (octGetById(&Facet) != OCT_OK) {
    Warning("Could not find facet");
    return(NLE_NOT_OK);
  }
  lsDelBegin(cmdList, (Pointer *) &gArg1);
  if (gArg1->argType != VEM_OBJ_ARG) {
    rpcFlatUsage();
    return(NLE_NOT_OK);
  }
  return(NLE_OK);
}



rpcFlatToPhysical(spot, cmdList , userOptionWord)
     RPCSpot *spot;
     lsList cmdList;
     long userOptionWord;
{
  octObject iBag;
  octObject OutputBag;
  struct octBox redrawBox;
  

  vemMessage("Flattening to physical ...\n", MSG_DISP|MSG_NOLOG);

  if (checkFlatArgs(spot, cmdList) != NLE_OK) return(RPC_OK);

  gFacet.objectId = spot->facet;
  octGetById(&gFacet);
  iBag.objectId = gArg1->argData.objArg.theBag;
  octGetById(&iBag);
  
  OutputBag.type = OCT_BAG;
  OutputBag.contents.bag.name = "BAG TO CONTAIN ALL THE NEW INSTANCES FROM FLAT";
  octCreate(&gFacet,&OutputBag);

  getBagBB(&iBag,&redrawBox);	/* For redisplay */

  if (nleFlat(&iBag, &OutputBag, NLE_PHYSICAL) != NLE_OK) {
    VM("Problems with FLAT\n");
  } else {
    VM("Flat done\n");
  }

  /*
   * Redisplay chip
   */
  vemWnQRegion(&gFacet,&redrawBox);
  vemWnFlush();
  octDelete(&OutputBag);

  return(RPC_OK);
}
