#include "rpc.h"
#include "oh.h"

static vemSelSet errSet = 0;

void rpcBegin( s )
    char* s;
{
    char buf[128];
    if ( errSet ) {
	vemFreeSelSet( errSet );
	errSet = 0;
    }
    sprintf( buf, "\nsymhelp: %s\n", s );
    VEMMSG( buf );
}

void rpcEnd( s , errcount)
    char* s;
    int errcount;
{
    char buf[128];
    if ( errcount ) {
	sprintf( buf, "symhelp: %d errors found.\n", errcount );
    } else {
	sprintf( buf, "symhelp: %s done: all seems OK.\n", s );
    }
    VEMMSG( buf );
    vemPrompt();
}

static int doall( facet, fixp )
    octObject* facet;
    int fixp;
{
    int errcount = 0;

    VEMMSG( "symhelp: doing names\n" );
    if ( fixp ) {
	errcount = symcleanAddNames( facet );
    }
    errcount = symcleanCheckNames( facet, fixp );

    VEMMSG( "symhelp: doing lambdas\n" );
    errcount = symcleanLambdas( facet, fixp );

    VEMMSG( "symhelp: doing nets\n");
    errcount += symcleanNets( facet, fixp );
    
    VEMMSG( "symhelp: doing formals\n");
    errcount += symcleanFormals( facet, fixp );

    VEMMSG( "symhelp: doing segments\n");
    errcount += symcleanSegments( facet, fixp);

    VEMMSG( "symhelp: doing instances\n");
    errcount += symcleanInstances( facet, fixp );

    VEMMSG( "symhelp: doing garbage\n");
    errcount += symcleanGarbage( facet, fixp );


    if ( ! fixp ) {
	VEMMSG( "symhelp: doing policy\n" );
	errcount += scCheck( facet );
    }
    return errcount;
}

int SRfixall( spot, cmdList, uow )
    RPCSpot* spot;
    lsList  cmdList;
    long  uow;
{
    octObject facet;
    int errcount = 0;
    int fixp = 1;

    rpcBegin( "fixall" );
    ohGetById( &facet, spot->facet );
  
    errcount += doall( &facet, fixp );
    
    rpcEnd( "fixall", errcount );
    return RPC_OK;
}

int SRfixnets( spot, cmdList, uow )
    RPCSpot* spot;
    lsList  cmdList;
    long  uow;
{
    octObject facet;
    int errcount = 0;
    int fixp = 1;

    rpcBegin( "fixnets" );
    ohGetById( &facet, spot->facet );
  
    errcount += symcleanNets( &facet, fixp );
    
    rpcEnd( "fixnets", errcount );
    return RPC_OK;
}

int SRcheckall( spot, cmdList, uow )
    RPCSpot* spot;
    lsList  cmdList;
    long  uow;
{
    octObject facet;
    int errcount = 0;
    int fixp = 0;

    rpcBegin( "checkall" );
    ohGetById( &facet, spot->facet );
    errcount += doall( &facet, fixp );
    rpcEnd( "checkall", errcount );
    return RPC_OK;
}

int SRpolicy( spot, cmdList, uow )
    RPCSpot* spot;
    lsList  cmdList;
    long  uow;
{
    octObject facet;
    int errcount = 0;
    int fixp = 0;

    rpcBegin( "policy" );
    ohGetById( &facet, spot->facet );
    errcount += scCheck( &facet );
    rpcEnd( "policy", errcount );
    return RPC_OK;
}


int SRchecknets( spot, cmdList, uow )
    RPCSpot* spot;
    lsList  cmdList;
    long  uow;
{
    octObject facet;
    int errcount = 0;
    int fixp = 0;

    rpcBegin( "checknets" );
    ohGetById( &facet, spot->facet );
  
    errcount += symcleanNets( &facet, fixp );
    
    rpcEnd( "checknets", errcount );
    return RPC_OK;
}

int SRchecksegments( spot, cmdList, uow )
    RPCSpot* spot;
    lsList  cmdList;
    long  uow;
{
    octObject facet;
    int errcount = 0;
    int fixp = 0;

    rpcBegin( "checksegments" );
    ohGetById( &facet, spot->facet );
  
    errcount += symcleanSegments( &facet, fixp );
    
    rpcEnd( "checksegments", errcount );
    return RPC_OK;
}

int SRfixsegments( spot, cmdList, uow )
    RPCSpot* spot;
    lsList  cmdList;
    long  uow;
{
    octObject facet;
    int errcount = 0;
    int fixp = 1;

    rpcBegin( "fixsegments" );
    ohGetById( &facet, spot->facet );
  
    errcount += symcleanSegments( &facet, fixp );
    
    rpcEnd( "fixsegments", errcount );
    return RPC_OK;
}

int SRaddnames( spot, cmdList, uow )
    RPCSpot* spot;
    lsList  cmdList;
    long  uow;
{
    octObject facet;
    int count = 0;

    rpcBegin( "addnames" );
    ohGetById( &facet, spot->facet );
    count = symcleanAddNames( &facet );
    rpcEnd( "addnames", count);
    return RPC_OK;
}

int SRchecknames( spot, cmdList, uow )
    RPCSpot* spot;
    lsList  cmdList;
    long  uow;
{
    octObject facet;
    int count = 0;

    rpcBegin( "checknames" );
    ohGetById( &facet, spot->facet );
    count = symcleanCheckNames( &facet, 0 );
    rpcEnd( "checknames", count);
    return RPC_OK;
}

int SRfixlambda( spot, cmdList, uow )
    RPCSpot* spot;
    lsList  cmdList;
    long  uow;
{
    octObject facet;
    int errcount = 0;
    rpcBegin( "fixlambdas" );
    ohGetById( &facet, spot->facet );
    errcount = symcleanLambdas( &facet, 1 );
    rpcEnd( "fixlambdas", errcount );
    return RPC_OK;
}

int SRchecklambda( spot, cmdList, uow )
    RPCSpot* spot;
    lsList  cmdList;
    long  uow;
{
    octObject facet;
    int errcount = 0;
    rpcBegin( "checklambdas" );
    ohGetById( &facet, spot->facet );
    errcount = symcleanLambdas( &facet, 0 );
    rpcEnd( "checklambdas", errcount );
    return RPC_OK;
}

int SRclear( spot, cmdList, uow )
    RPCSpot* spot;
    lsList  cmdList;
    long  uow;
{
    rpcBegin( "Clearing done" );
    vemPrompt();
    return RPC_OK;
}

int SRexit( spot, cmdList, uow )
    RPCSpot* spot;
    lsList  cmdList;
    long  uow;
{
    rpcBegin( "Exiting" );
    RPCExit(RPC_OK);
}

extern long SRcapacitance();
extern long SRcrossCapacitance();
extern long SRresistance();
extern long SRpepInfo();

extern long labelFormalSupply();
extern long labelFormalGround();
extern long labelFormalInput();
extern long labelFormalOutput();


RPCFunction CommandArray[] = {
    { SRfixall , "FIX", "fix-all", 0, 0 },
    { SRfixnets, "FIX", "fix-nets", 0, 0 },
    { SRfixsegments, "FIX", "fix-segments", 0, 0 },
    { SRfixlambda, "FIX", "fix-lambda", 0, 0 },
    { SRaddnames, "FIX", "add-names", 0, 0 },
    { SRcheckall, "CHECK", "check-all", 0, 0 },
    { SRpolicy, "CHECK", "check-policy", 0, 0 },
    { SRchecknets, "CHECK", "check-nets", 0, 0 },
    { SRchecksegments, "CHECK", "check-segments", 0, 0 },
    { SRchecklambda, "CHECK", "check-lambda", 0, 0 },
    { SRchecknames, "CHECK", "check-names", 0, 0 },
    { SRpepInfo, "PARASITICS", "INFORMATION", 0, 0 },
    { SRcapacitance, "PARASITICS", "capacitance", 0, 0 },
    { SRcrossCapacitance, "PARASITICS", "cross-cap", 0, 0 },
    { SRresistance, "PARASITICS", "resistance", 0, 0 },
    { labelFormalSupply, "TERM-PROPERTIES", "supply-term", "", 0 },
    { labelFormalGround, "TERM-PROPERTIES", "ground-term", "", 0 },
    { labelFormalInput, "TERM-PROPERTIES", "input-term", "", 0 },
    { labelFormalOutput, "TERM-PROPERTIES", "output-term", "", 0 },
    { SRclear, "MISC", "clear-highlight", 0 , 0 },
    { SRexit, "MISC", "exit-symhelp", 0 , 0 }
};


void rpcErrorReport( obj, msg )
    octObject* obj;
    char* msg;
{
    if ( obj != 0 ) {
	if ( errSet == 0 ) {
	    octObject facet;
	    octGetFacet( obj, &facet );
	    errSet = vemNewSelSet( facet.objectId, 60000, 60000, 10000, 1, 1, 4, 4,
				  "0001 0110 0110 1000" );
	}
	vemAddSelSet( errSet, obj->objectId );
    }
    VEMMSG( msg );
}


/*ARGSUSED*/
long UserMain( display, spot, cmdList, uow, array )
    char* display;
    RPCSpot* spot;
    lsList cmdList;
    long  uow;
    RPCFunction* array[];
{
    *array = CommandArray;

    VEMMSG(  "symhelp: version 1.2: ready.\n" );
    vemPrompt();
    symcleanSetErrorReport( rpcErrorReport );

    return(sizeof(CommandArray) / sizeof(RPCFunction));
}

