/*
 * Symbolic Policy Checking Library
 *
 * David Harrison
 * University of California, Berkeley
 * 1990
 */

#ifndef SYMCK_H
#define SYMCK_H

#include "ansi.h"
#include "oct.h"

extern char *sc_pkg_name;

/* Error classes */
#define WARN	0
#define ERR	1
#define FATAL	2

/* Specific errors */
#define MFP	0		/* Missing facet property */
#define BFP	1		/* Bad facet property     */
#define BCE	2		/* Bag contents error     */
#define MCT	3		/* Missing celltype property */
#define MIF	4		/* Missing interface facet */
#define MAF	5		/* Mismatched actual/formal */
#define BJP	6		/* Bad joined policy       */
#define TNF	7		/* Terminal net failure    */
#define DAT	8		/* Dangling actual terminal */
#define JTF	9		/* Jump term failure        */
#define SLF	10		/* Segment layer failure    */
#define STF	11		/* Segment terminal failure */
#define SFE	12		/* Segment on facet error   */
#define BGE	13		/* Bad geometry error       */
#define SPE	14		/* Segment points error     */
#define SNF	15		/* Segment net failure      */
#define IOT	16		/* Illegal object type      */
#define CER	17		/* Bad connectivity         */
#define TNE	18		/* Trivial net error        */
#define DTN	19		/* Duplicate terminal names */
#define UOT	20		/* Unexpected object type   */
#define FIE	21		/* Formal terminal implementation error */
#define FNN	22		/* Formal terminal with no net */
#define BPV	23		/* Bad property value */

#define EOE	24		/* Last error */

extern char *scErrClass[EOE];	/* Error class names */

extern int scCheck
  ARGS((octObject *fct));
  /* Checks the facet `fct' for symbolic policy errors */

typedef struct sc_err_func_defn {
    void (*err_func)
      ARGS((int severity, int class, octObject *obj, char *msg, char *user_data));
    char *user_data;
} scErrFunc;

extern void scSetErrorFunc
  ARGS((scErrFunc *newf, scErrFunc *old));
  /* Sets the error function to `newf' and returns the old one in `old' */

#endif /* SYMCK_H */
