/*
 * Written by Tu-Cuong(Clark) Nguyen and Haifang Guo.
 *
 * 'ComputeGradSENS' is the main multiplexing program which decides
 * which algorithms are necessary for a given performance function
 * and proceeds to determine sensitivities.  Calls are made to
 * 'spice' for waveform and circuit variable sensitivity information,
 * and the algorithms discussed in "Performance Sensitivity
 * Computation for Interconnect Parasitics" are implemented.
 *
 */

#include <stdio.h>
#include <math.h>
#include "formulate.h"
#include "macros.h"

ComputeGradSENS(num_incards,node_file_name)
int num_incards;    /* numer of INCARDS */
char *node_file_name;
{
    INCARD *incard;        /* pointer to an INCARD */
    GRAD_info *grad_info;       /* pointer to a GRAD_info */
    char *label;         /* type of constraint */
    SIM_data sim_data; /* dummy variable to hold simulation data */
    SIM_data ac_read(), tran_read();
    double test;
    double slope(), zdbfreqval(), tdbfreqval();
    double low_freq_val, low_freq_gain;
    double zdbfreq_val, tdbfreq_val;
    int zdbfreq_index, tdbfreq_index;
    int num_nodes;
    double pvmpw, pvppw, pvmpc, pvppc;
    char **node_names;
    int i,j,k,l;        /* dummy variables */

    /* initialize the structures to hold the gradient info */
    Gradients = (GRAD_info *) malloc(sizeof(GRAD_info)*num_incards);
    if(node_read(node_file_name,&(node_names),&(num_nodes)) != OK){
	print_error("main.c: error in node_read()\n");
    }

    /* calculate the gradients and store them in GRAD_infos */
    for (i=0; i < num_incards ; i++) {
    grad_info = Gradients+i;
    label = grad_info->label;
      incard = Incards+i;
    if(strcmp(incard->label,"lfg")==0){
        strcpy(label,incard->label);
        sim_data = ac_read(&sim_data,incard->spiceinf);
        low_freq_val = sim_data.scale[0]; 
        low_freq_gain = sim_data.waveform1[0]; 
        GetPvPc(incard->spiceinf,low_freq_val,AC,&node_names,num_nodes,sim_data);
        l=0;
        for (j=1; j <= num_nodes; j++) {
            for (k=1; k <= num_nodes; k++) {
            pvmpc = pvmpc_data[j][k];
            strcpy((grad_info->grad_card)[l].node1,*(node_names+j));
            strcpy((grad_info->grad_card)[l].node2,*(node_names+k));
            (grad_info->grad_card)[l].grad = pvmpc;
            l++;
            }
        }
        grad_info->num_grads = l;
        SortGrad(grad_info);
    }
    else if(strcmp(incard->label,"ugb")==0){
        strcpy(label,incard->label);
        sim_data = ac_read(&sim_data,incard->spiceinf);
        zdbfreq_index = zdbfreqindex(&sim_data);
        zdbfreq_val = zdbfreqval(&sim_data);
        GetPvPc(incard->spiceinf,zdbfreq_val,AC,&node_names,num_nodes,sim_data);
        pvmpw = slope(zdbfreq_index,&sim_data,MAG);
        l=0;
        for (j=1; j <= num_nodes; j++) {
            for (k=1; k <= num_nodes; k++) {
            pvmpc = pvmpc_data[j][k];
            strcpy((grad_info->grad_card)[l].node1,*(node_names+j));
            strcpy((grad_info->grad_card)[l].node2,*(node_names+k));
            (grad_info->grad_card)[l].grad = - pvmpc/pvmpw;
            l++;
            }
        }
        grad_info->num_grads = l;
        SortGrad(grad_info);
    }
    else if(strcmp(incard->label,"pm")==0){
        strcpy(label,incard->label);
        sim_data = ac_read(&sim_data,incard->spiceinf);
        zdbfreq_index = zdbfreqindex(&sim_data);
        zdbfreq_val = zdbfreqval(&sim_data);
        GetPvPc(incard->spiceinf,zdbfreq_val,AC,&(node_names),num_nodes,sim_data);
        /* magnitude slope in /Hz */
        pvmpw = slope(zdbfreq_index,&sim_data,MAG);

        /* phase slope in degree/Hz */
        pvppw = slope(zdbfreq_index,&sim_data,PHASE);
         
        l=0;
        for (j=1; j <= num_nodes; j++) {
            for (k=1; k <= num_nodes; k++) {
            pvmpc = pvmpc_data[j][k];
             pvppc = pvppc_data[j][k];
            strcpy((grad_info->grad_card)[l].node1,*(node_names+j));
            strcpy((grad_info->grad_card)[l].node2,*(node_names+k));
            (grad_info->grad_card)[l].grad = pvppc - 
              pvppw*pvmpc/pvmpw;
            l++;
            }
        }
        grad_info->num_grads = l;
        SortGrad(grad_info);
    }
    else if(strcmp(incard->label,"3db")==0){
        strcpy(label,incard->label);
        sim_data = ac_read(&sim_data,incard->spiceinf);
        tdbfreq_index = tdbfreqindex(&sim_data);
        tdbfreq_val = tdbfreqval(&sim_data);
        GetPvPc(incard->spiceinf,tdbfreq_val,AC,&node_names,num_nodes,sim_data);
        /*
        printf("tdbindex, tdbval = %d,%g\n",tdbfreq_index,tdbfreq_val);
        */
        pvmpw = slope(tdbfreq_index-1,&sim_data,MAG);
            /* tdbfreq_index-1 to handle thecase when tdbfreq_index
               is the last one */
        l=0;
        for (j=1; j <= num_nodes; j++) {
            for (k=1; k <= num_nodes; k++) {
            pvmpc = pvmpc_data[j][k];
            strcpy((grad_info->grad_card)[l].node1,*(node_names+j));
            strcpy((grad_info->grad_card)[l].node2,*(node_names+k));
            (grad_info->grad_card)[l].grad = - pvmpc/pvmpw;
            l++;
            }
        }

        /*

        low_freq_val = sim_data.scale[0]; 
        low_freq_gain = sim_data.waveform1[0]; 
        GetPvPc(incard->spiceinf,low_freq_val,AC,&node_names,num_nodes,sim_data);
        l=0;
        for (j=1; j <= num_nodes; j++) {
            for (k=1; k <= num_nodes; k++) {
            pvmpc = pvmpc_data[j][k];
            strcpy((grad_info->grad_card)[l].node1,*(node_names+j));
            strcpy((grad_info->grad_card)[l].node2,*(node_names+k));
            (grad_info->grad_card)[l].grad += 0.7071 * pvmpc/pvmpw;
            l++;
            }
        }
        */
        grad_info->num_grads = l;
        SortGrad(grad_info);
    }
    else if(strcmp(incard->label,"sr")==0){
        strcpy(label,incard->label);
        sim_data = tran_read(&sim_data,incard->spiceinf);
        GetPvPc(incard->spiceinf,*(sim_data.scale+10),
                    TRAN,&node_names,num_nodes,sim_data);
        l=0;
        for (j=1; j <= num_nodes; j++) {
            for (k=1; k <= num_nodes; k++) {
            pvmpc = pvmpc_data[j][k];
            strcpy((grad_info->grad_card)[l].node1,*(node_names+j));
            strcpy((grad_info->grad_card)[l].node2,*(node_names+k));
            (grad_info->grad_card)[l].grad = pvmpc/
              (*(sim_data.scale+10));
            l++;
            }
        }
        grad_info->num_grads = l;
        SortGrad(grad_info);
    }
	else{
		printf("unknown performance %s\n",incard->label);
		exit(-1);
	}
    }
    return(num_nodes);
}

/*
 *
 * 'zdbfreqindex' takes a pointer to a SIM_data structure (containing
 * waveform data from a circuit simulation) and returns the index
 * of the 0 dB frequency of the magnitude waveform (contained in 
 * 'waveform1' in the SIM_data structure).
 *
 */
zdbfreqindex(sim_data)
SIM_data *sim_data;
{
    double *scale, *waveform;  /* dummies to point to parts of SIM_data */
    double sample;      /* variable to hold interpolated zero dB freq. */
    int i;        /* dummy variables */

    scale = sim_data->scale;
    waveform = sim_data->waveform1;

    for (i=0; i <= sim_data->len; i++) {
    if (*(waveform+i) > 1.0) 
        continue;
    else
        return(i);
    }
    printf("senscalc:  can't find 0 dB frequency\n");
    return(-1.0);
}

/*
double
izdbfreqval(sim_data)
SIM_data *sim_data;
{
    double *scale, *waveform;  
    double sample;     
    int i;        

    scale = sim_data->scale;
    waveform = sim_data->waveform1;

    for (i=0; i <= sim_data->len; i++) {
    if (*(waveform+i) > 1.0) continue;
    if (*(waveform+i) == 1.0)
      return(*(scale+i));  
    else {   
        sample = (*(scale+i-1)) + 
              (*(scale+i)-(*(scale+i-1)))/
              (*(waveform+i-1)-(*(waveform+i))) *
              (*(waveform+i-1)-1.0);
        return(sample);    
    }
    }
    printf("senscalc:  can't find 0 dB frequency\n");
    return(-1.0);
}  
*/

/*
 *
 * 'zdbfreqval' takes a pointer to a SIM_data structure (containing
 * waveform data from a circuit simulation) and returns the spice
 * scale frequency closest to an interpolated value 
 * of the 0 dB frequency of the magnitude waveform (contained in 
 * 'waveform1' in the SIM_data structure).
 *
 */
double zdbfreqval(sim_data) 
SIM_data *sim_data; 
{ 
    double *scale,*waveform; /* dummies to point to parts of SIM_data */ 
    double sample;  /* variable to hold interpolated zero dB freq. */ 
    int i;        /* dummy variables */

    scale = sim_data->scale; waveform = sim_data->waveform1;

    for (i=0; i <= sim_data->len; i++) {
     if (*(waveform+i) > 1.0) continue;
     if (*(waveform+i) == 1.0) { /* rare case */
        return(*(scale+i));
     }
     else { /* return freq. closest to interpolated value */
        sample = (*(scale+i-1)) + 
              (*(scale+i)-(*(scale+i-1)))/
              (*(waveform+i-1)-(*(waveform+i))) *
              (*(waveform+i-1)-1.0);
          if (FABS(*(scale+i) - sample) < FABS(*(scale+i-1) - sample))
          return(*(scale+i));
        else
          return(*(scale+i-1));
    }
    }
    printf("senscalc:  can't find 0 dB frequency\n");
    return(-1.0);
}  

/*
 *
 * 'tdbfreqindex' takes a pointer to a SIM_data structure (containing
 * waveform data from a circuit simulation) and returns the index
 * of the 3 dB frequency of the magnitude waveform (contained in 
 * 'waveform1' in the SIM_data structure).
 *
 */
tdbfreqindex(sim_data)
SIM_data *sim_data;
{
    double *scale, *waveform;  /* dummies to point to parts of SIM_data */
    double sample;      /* variable to hold interpolated zero dB freq. */
    int i;        /* dummy variables */
    double tdb_value;   /* 3db value */

    scale = sim_data->scale;
    waveform = sim_data->waveform1;

    tdb_value = (*waveform) / sqrt(2.0);  /* 3db below dc_gain */       
    for (i=0; i <= sim_data->len; i++) {
    if (*(waveform+i) > tdb_value) 
        continue;
    else
        return(i);
    }
    printf("senscalc:  can't find 3 dB frequency\n");
    return(-1.0);
}


/*
 *
 * 'zdbfreqval' takes a pointer to a SIM_data structure (containing
 * waveform data from a circuit simulation) and returns the spice
 * scale frequency closest to an interpolated value 
 * of the 0 dB frequency of the magnitude waveform (contained in 
 * 'waveform1' in the SIM_data structure).
 *
 */
double tdbfreqval(sim_data)
SIM_data *sim_data; 
{ 
    double *scale,*waveform; /* dummies to point to parts of SIM_data */ 
    double sample;  /* variable to hold interpolated zero dB freq. */ 
    int i;        /* dummy variables */
    double tdb_value;

    scale = sim_data->scale; 
    waveform = sim_data->waveform1;

    tdb_value = (*waveform) / sqrt(2.0);  /* 3db below dc_gain */       
    for (i=0; i <= sim_data->len; i++) {
     if (*(waveform+i) > tdb_value) continue;
     if (*(waveform+i) == tdb_value) { /* rare case */
        return(*(scale+i));
     }
     else { /* return freq. closest to interpolated value */
        sample = (*(scale+i-1)) + 
              (*(scale+i)-(*(scale+i-1)))/
              (*(waveform+i-1)-(*(waveform+i))) *
              (*(waveform+i-1)-tdb_value);
          if (FABS(*(scale+i) - sample) < FABS(*(scale+i-1) - sample))
          return(*(scale+i));
        else
          return(*(scale+i-1));
    }
    }
    printf("senscalc:  can't find 3 dB frequency\n");
    return(-1.0);
}  


/*
 * This program takes a SPICE input file include .tf card,
 * can contain .print .plot.. As long as it doesn't contain
 * .sens .save ...
 * then run SPICE to get a raw spice  file which contains dc  gain value.
 * Then a shell command is called to get the dc gain value and put into a file
 *
 * File: dc_gain_read.c
 * Date: 12/11/90 
 */

/*
double dc_gain_read(spice_file_name)

char *spice_file_name;

{
    char command[100];
    double dc_gain;
    char inline[MAX_INLINE];
    FILE *f1;
    char *fgetsnoret();

    sprintf(command,"%s -r raw < %s > out",sim_name,spice_file_name);

#ifdef DEBUG
    printf("before spice in dc_gain_read\n");
#endif 

    system(command);

#ifdef DEBUG
    printf("after spice in dc_gain_read\n");
#endif 

    system("sed '1,12d' raw | egrep '^0' | sed 's/^0        //' > dcgain");


    f1 = fopen("dcgain", "r");
    fgetsnoret(inline,  MAX_INLINE, f1);
    dc_gain = atof(inline);

#ifdef DEBUG
    printf("the value for dc_gain %e \n", dc_gain);
#endif 

    system("rm raw out dcgain");

    return(dc_gain);
}
*/

/*
 *
 * 'findfreqindex' takes a pointer to a SIM_data structure (containing
 * waveform data from a circuit simulation), and a frequency point,
 * and returns the index of the frequency
 */

findfreqindex(sim_data,freq_point)
SIM_data *sim_data;
double freq_point;

{
    double *scal; /* dummies to point to parts of SIM_data */
    double sample;     /* variable to hold interpolated zero dB freq. */
    int i;        /* dummy variables */

    scal = sim_data->scale;

    for (i=0; i <= sim_data->len; i++) {
        if (*(scal+i) < freq_point)
               continue;
        else{
            if ((*(scal+i) - freq_point) < (freq_point - *(scal+i-1)))
                   return(i);
            else return((i-1));
        }
    }
    printf("senscalc:  can't find freq_point\n");
    return(-1.0);
}


/*
 * This program computes the slope of the given waveform (stored in
 * a array) at the given point (ie zdbfreq_val).
 * type indicates which waveform to look at in the SIM_data structure
 *
 */

double slope(zdbfreq_index,sim_data, type)
int zdbfreq_index;
SIM_data *sim_data;
int type;

{
    int nzdbfreq_index;
    double *scale, *waveform1, *waveform2;
    double d_scale;
    double d_waveform1;
    double d_waveform2;
    double slope_waveform1;
    double slope_waveform2;

    scale = sim_data->scale;
    waveform1 = sim_data->waveform1;
    waveform2 = sim_data->waveform2;

    nzdbfreq_index = zdbfreq_index+1;
    /*
    printf("scale1, scale2 = %g,%g\n",scale[zdbfreq_index],scale[nzdbfreq_index]);
    printf("w1, w2 = %g,%g\n",waveform1[zdbfreq_index],waveform1[nzdbfreq_index]);
    */
    d_scale = scale[zdbfreq_index] - scale[nzdbfreq_index];
    if (type == MAG) {
    d_waveform1 = waveform1[zdbfreq_index] - waveform1[nzdbfreq_index];
    slope_waveform1 = d_waveform1 / d_scale; 
    return(slope_waveform1);
    }
    else if (type == PHASE) {
        d_waveform2 = waveform2[zdbfreq_index] - waveform2[nzdbfreq_index];
    slope_waveform2 = d_waveform2 / d_scale;
    return(slope_waveform2);
    }
    else {
        printf("\n Undefined type \n");
    }
}



/*
 * This program takes a SPICE input file, a data structure and 
 * performs two tasks -- task one, call SPICE and get a spice output file;
 * task two, read the relevant part of the SPICE output file into the data structure.
 *
 * File: ac_read.c
 * Date: 12/6/90 
 */

extern int strcmp();
extern char *strsave();

#define MAX_ROW_NUM 1000
#define MAX_COL_NUM 2
#define MAX_LINE_LENGTH 200


SIM_data ac_read(sim_data, spice_file_name)

SIM_data *sim_data;
char *spice_file_name;        /* the SPICE input deck */

{

    FILE *text_file;
    FILE *mag_file;
    char text_line[MAX_LINE_LENGTH];
    char *next_entry;
    char *begin_ptr;
    int    row_index = -1;
    int    col_index = 0;
    double mag_info[MAX_ROW_NUM][MAX_COL_NUM];        
    double ph_info[MAX_ROW_NUM][MAX_COL_NUM];        
    int    number;
    int    i;
    double value;
    double real;
    double imag;
    double phase;
    double arg;
    char dummy[10];
    int line_num;

    char command[100];    /* command line for system call to run SPICE */
    char spice_out_name[50]; /* spice_output_file name */


    /* call spice to get the output files */
    sprintf(spice_out_name, "%s.out", spice_file_name);
    sprintf(command,"%s < %s >  %s", sim_name,spice_file_name,spice_out_name);
#ifdef DEBUG
    printf("before spice in ac_read\n");
#endif

    system(command);

#ifdef DEBUG
    printf("after spice in ac_read\n");
#endif

     
    mag_file = fopen("store","A");
    /* read input data and set up the data structures */

    
    /* Open the SPICE output file */
    if( (text_file = fopen(spice_out_name,"r")) == NULL) {
        printf("\nE: \"%s\" file not found.\n",spice_out_name);
    /*return ERROR; */
    }

    /* Read file, line by line */
    for(line_num = 1; ;line_num++) {
    if(fgets(text_line,MAX_LINE_LENGTH,text_file) == NULL)
            break;
    
    /* Check for comment or empty line */
    if(text_line[0] == ';') continue;
    if(strlen(text_line) == 1) continue;

    if(sscanf(text_line,"Index %s",dummy) == 1){
        if(fgets(text_line,MAX_LINE_LENGTH,text_file) == NULL){
            printf("sens_tran_ac 79: \
              no sensitivity info present \n");
             break;
        }
        for(i=0; i<=MAX_LINE_LENGTH-1; i++){
        /* checking '-' line */
        if((text_line[i]==' ') ||
            (text_line[i]=='-')){
            continue;
            }
        else if (text_line[i]=='\n') break;
        else {
        printf("something wrong in output format");
        exit(-1);
        }
        }
next_frequency_point:

        if(fgets(text_line,MAX_LINE_LENGTH,text_file) == NULL){
        /****
         * printf("sens_tran_ac 98: \
         *    no sensitivity info present \n");
         ****/
            break;
        }

        begin_ptr = text_line;

        number = FIRST;
        for(;;){
            if (*begin_ptr=='\n'){
             goto next_frequency_point;
             }
            if (*begin_ptr==' '){
             begin_ptr++;
             continue;
            }
            if (*begin_ptr==','){
             begin_ptr++;
             continue;
                }    
            /* check if the next entry is a number (positive 
               or negative) or something else 
               (go to next line if it is something else)*/

            if(((*begin_ptr > '9') || (*begin_ptr < '0'))&&
                ((*begin_ptr != '-')||
                (*(begin_ptr+1) > '9') || (*(begin_ptr+1) < '0')))
                goto next_frequency_point;

            value = atof(begin_ptr);
 
                /*
                  * first one is index, the second one is time stamp
                 * the third one is the magnitude
                 */
             if(number == FIRST) {
                 number = SECOND;
             row_index++ ;
                 fprintf(mag_file, "row_index %d \n ", row_index);
             }
             else if(number == SECOND) {
                number = THIRD;
                col_index = 0;
                sim_data->scale[row_index] = value;
                /***
                 * mag_info[row_index][col_index] = value;
                 * ph_info[row_index][col_index] = value;
                 ***/
                fprintf(mag_file, "scale %e \n", sim_data->scale[row_index]);
             }
             else if(number == THIRD) {
                number = FOURTH;
             }
             else if(number == FOURTH) {
                number = FIFTH;    
                col_index = 1;
                sim_data->waveform1[row_index] = value;
                /* mag_info[row_index][col_index] = value; */
                /*
                fprintf(mag_file, "waveform1 %e \n", sim_data->waveform1[row_index]);
                */
             }
             else if(number == FIFTH) {
                col_index = 1;
                /* ph_info[row_index][col_index] = value; */
                /*
                fprintf(mag_file, "waveform2 %e \n", sim_data->waveform2[row_index]);
                */
                real = sim_data->waveform1[row_index];
                imag = value;
                sim_data->waveform1[row_index]=sqrt(real*real+imag*imag);
                /* compute phase in degree */
                arg = imag/real;
                phase = (180/3.1415)*atan(FABS(arg));
                if((real > 0) && (imag < 0)) phase *= -1;
                if((real < 0) && (imag > 0)) phase = 180 - phase;
                if((real < 0) && (imag < 0)) phase = -1*(180 - phase);

                sim_data->waveform2[row_index]=phase;
             }
             for(next_entry = begin_ptr; ;next_entry++){
                if (*next_entry == '\t')  break;
                if (*next_entry == '\n'){
                    goto next_line;
                }
            }
            begin_ptr = next_entry + 1;
            sim_data->len = row_index;
        }
    }
next_line: ;
    }

#ifdef DEBUG
    printf("len %d \n", sim_data->len);
#endif

    fclose(mag_file);
    return(*sim_data);
}

/*
 * This program takes a SPICE input file, a data structure and
 * performs two tasks -- task one, call SPICE and get a spice output file;
 * task two, read the relevant part of the SPICE output file into the data structure.
 * The analysis is transient analysis.
 *
 * File: tran_read.c
 * Date: 12/6/90 
 */

SIM_data
tran_read(sim_data, spice_file_name)

SIM_data *sim_data;
char *spice_file_name;        /* the SPICE input deck */

{

    FILE *text_file;
    FILE *mag_file;
    char text_line[MAX_LINE_LENGTH];
    char *next_entry;
    char *begin_ptr;
    int    row_index = -1;
    int    col_index = 0;
    int    number;
    int    i;
    double value;
    char dummy[10];
    int line_num;

    char command[100];    /* command line for system call to run SPICE */
    char spice_out_name[50]; /* spice_output_file name */


    /* call spice to get the output files */
    sprintf(spice_out_name, "%s.out", spice_file_name);
    sprintf(command, " %s < %s >  %s", sim_name,spice_file_name,spice_out_name);
#ifdef DEBUG
    printf("before spice in tran_read()\n");
#endif

    system(command);

#ifdef DEBUG
    printf("after spice in tran_read()\n");
#endif

     
    mag_file = fopen("store","A");
    /* read input data and set up the data structures */

    
    /* Open the SPICE output file */
    if( (text_file = fopen(spice_out_name,"r")) == NULL) {
        printf("\nE: \"%s\" file not found.\n",spice_out_name);
    /*return ERROR; */
    }

    /* Read file, line by line */
    for(line_num = 1; ;line_num++) {
    if(fgets(text_line,MAX_LINE_LENGTH,text_file) == NULL)
            break;
    
    /* Check for comment or empty line */
    if(text_line[0] == ';') continue;
    if(strlen(text_line) == 1) continue;

    if(sscanf(text_line,"Index %s",dummy) == 1){
        if(fgets(text_line,MAX_LINE_LENGTH,text_file) == NULL){
            printf("sens_tran_ac 79: \
              no sensitivity info present \n");
             break;
        }
        for(i=0; i<=MAX_LINE_LENGTH-1; i++){
        /* checking '-' line */
        if((text_line[i]==' ') ||
            (text_line[i]=='-')){
            continue;
            }
        else if (text_line[i]=='\n') break;
        else {
        printf("something wrong in output format");
        exit(-1);
        }
        }
next_frequency_point:

        if(fgets(text_line,MAX_LINE_LENGTH,text_file) == NULL){
        /****
         * printf("sens_tran_ac 98: \
         *    no sensitivity info present \n");
         ****/
            break;
        }

        begin_ptr = text_line;

        number = FIRST;
        for(;;){
            if (*begin_ptr=='\n'){
             goto next_frequency_point;
             }
            if (*begin_ptr==' '){
             begin_ptr++;
             continue;
            }
            if (*begin_ptr==','){
             begin_ptr++;
             continue;
                }    
            /* check if the next entry is a number (positive 
               or negative) or something else 
               (go to next line if it is something else)*/

            if(((*begin_ptr > '9') || (*begin_ptr < '0'))&&
                ((*begin_ptr != '-')||
                (*(begin_ptr+1) > '9') || (*(begin_ptr+1) < '0')))
                goto next_frequency_point;

            value = atof(begin_ptr);
 
                /*
                  * first one is index, the second one is time stamp
                 * the third one is the transient output 
                 */
             if(number == FIRST) {
                 number = SECOND;
             row_index++ ;
             }
             else if(number == SECOND) {
                number = THIRD;
                col_index = 0;
                sim_data->scale[row_index] = value;
                fprintf(mag_file, "sim_data->scale[row_index] %e \n", sim_data->scale[row_index]);
             }
             else if(number == THIRD) {
                col_index = 1;
                sim_data->waveform1[row_index] = value;
                fprintf(mag_file, "sim_data->waveform1[row_index] %e \n", sim_data->waveform1[row_index]);
             }
             for(next_entry = begin_ptr; ;next_entry++){
                if (*next_entry == '\t')  break;
                if (*next_entry == '\n'){
                    goto next_line;
                }
             }
            begin_ptr = next_entry + 1;
            sim_data->len = row_index;
        }
    }
next_line: ;
    }
    fprintf(mag_file, "sim_data->len %d \n", sim_data->len);
    fclose(mag_file);
    return(*sim_data);
}

/*
 *
 * 'SortGrad' takes a pointer to a GRAD_info and sorts the GRAD_card
 * array such that the gradient values (double grad) are sorted in 
 * decreasing order.
 *
 */
SortGrad(grad_info)
GRAD_info *grad_info;
{
    GRAD_card *grad_card;
    int gap,i,j;
    GRAD_card temp;
    int n;

    n = grad_info->num_grads;
    grad_card = grad_info->grad_card;

    for (gap = n/2; gap > 0; gap /= 2)
    for (i=gap; i < n; i++)
        for (j=i-gap; j>=0 && grad_card[j].grad < grad_card[j+gap].grad; 
         j-=gap) {
        temp = grad_card[j];
        grad_card[j] = grad_card[j+gap];
        grad_card[j+gap] = temp;
        }
}
    
