/*
 *
 * 'sens_tran_read' reads spice output and gets the transient sensitivity
 * information.
 *
 * Written by Umakanta Choudhury and modified by Tu-Cuong(Clark) Nguyen
 * and Haifang Guo.
 *
 */

extern int strcmp();
extern char *strsave();


#include <stdio.h>
#include <math.h>
#include "formulate.h"
#include "macros.h"

#define MAX_LINE_LENGTH 200
#define name1   *(node_names + node1_index)
#define name2   *(node_names + node2_index)

int sens_tran_read(text_file_name,node_names,time_point,
        node1_index_list,node2_index_list,num_params,
        num_entries,sim_data,out_mode)

char *text_file_name;        /* The input text file            */
char **node_names;
double time_point;
int *node1_index_list;
int *node2_index_list;
int num_params;
int *num_entries;
SIM_data sim_data;
int out_mode;

{
    FILE *text_file;
    FILE *tran_sens_file;
    char text_line[MAX_LINE_LENGTH];
    char filename[50];
    char *next_entry;
    char *begin_ptr;
    int    line_num = 1;
    int number;
    int sens_type = REAL1;
    int first_line = YES;
    int    read_sens = NO;
    int blank;
    int i;
    float sens;
    double value;
    char dummy[30];
    int param_num = 0;
    int node1_index = 0;
    int node2_index = 0;
    int entries = 0;
    double s1;
    double s2;
    double s;
    double abs_vt;
    double abs_t;

    /*** add row_index, col_index ***/
    int row_index;
    int col_index;


    sprintf(filename,"%s_%d","/tmp/parcar.outfile1",getpid()); 
    tran_sens_file = fopen(filename,"A");
    /* read input data and set up the data structures */

    /* Open the text file */
    if( (text_file = fopen(text_file_name,"r")) == NULL) {
        printf("\nE: \"%s\" file not found.\n",text_file_name);
        return ERROR;
    }

    /* Read file, line by line */
    for(line_num = 1; ;line_num++) {
        if(fgets(text_line,MAX_LINE_LENGTH,text_file) == NULL)
        break;
    
    /* Check for comment or empty line */
    if(text_line[0] == ';') continue;
    if(strlen(text_line) == 1) continue;

    if(sscanf(text_line,"Index %s \n",dummy) == 1){
        if(fgets(text_line,MAX_LINE_LENGTH,text_file) == NULL){
            printf("sens_tran_read 83: \
                   no sensitivity info present \n");
            break;
        }
        for(i=0; i<=MAX_LINE_LENGTH-1; i++){
        /* checking '-' line */
            if((strcmp(text_line[i],' ') == 0) ||
            (strcmp(text_line[i],'-') == 0)){
                continue;
            }
            else if (strcmp(text_line[i],'\n') == 0) break;
            else {
            printf("something wrong in output format");
            exit(-1);
            }
        }

next_time_point:
        if(fgets(text_line,MAX_LINE_LENGTH,text_file) == NULL){
		/*
            printf("sens_tran_read 101: \
            no sensitivity info present \n");
			*/
            break;
        }

        begin_ptr = text_line;

        number = FIRST;
        read_sens = NO;
        for(;;){
            if (strcmp(*begin_ptr,'\n') == 0){
                goto next_time_point;
            }
            if (strcmp(*begin_ptr,' ') == 0){
                begin_ptr++;
                continue;
            }
            /* check if the next entry is a number (positive 
            or negative) or something else 
            (go to next line if it is something else)*/

            if(((*begin_ptr > '9') || (*begin_ptr < '0'))&&
                ((*begin_ptr != '-')||
                (*(begin_ptr+1) > '9') || (*(begin_ptr+1) < '0')))
                goto next_time_point;

            value = atof(begin_ptr);
            /* first two entries are index and time point*/
            if(number == FIRST) {
                number = SECOND;
            }
            else if(number == SECOND) {
                number = THIRD;
/***
                printf(" in SensTranRead.c value %e data %e %e \n",value, time_point, abs(value-time_point));
***/
                abs_vt = FABS(value-time_point);
                abs_t = FABS(time_point);
				if(abs_vt < 0.001*abs_t){ 
				/*
                if(FABS(value-time_point) 
                        < 0.001*(FABS(time_point))){
						*/
                /* correct time point */
                    read_sens = YES;
                }
                else{
                    goto next_time_point;
                }
            }
            else if (read_sens == YES){

                if((sens_type == REAL1) && (out_mode == OUT1_ZERO)){ 
                    s1 = 0;
                    sens_type == REAL2;
                }

                if(sens_type == REAL1) {
    
                    /* for node1 */
    
                    s1 = 1e-15*value;    

                    if(out_mode == OUT2_ZERO) {
                        sens_type = REAL1; 
                        s2 = 0;
                        goto calculate;
                    }
                    else sens_type = REAL2;
                    goto next_line;
                }
                else if(sens_type == REAL2) {
    
                    /* for node2 */
    
                    s2 = 1e-15*value;
    
calculate:          s = s1 - s2;

                    param_num++ ;
                    node1_index 
                    = *(node1_index_list + param_num);
                    node2_index 
                    = *(node2_index_list + param_num);

                    /* print the sensitivity 
                     * in v/fF ; value is in v/F 
                     */
                    entries += 1;
                
                    pvmpc_data[node1_index][node2_index] = s;
                    fprintf(tran_sens_file,"name1  %s name2 %s s %g \n",
                            (name1),(name2),s);
                    fprintf(tran_sens_file,"node1_index %d node2_index %d s1 %g s2 %g \n",
                    node1_index,node2_index,s1,s2);


                    sens_type = REAL1;
                    goto next_line;
               }
               else{
                   printf("sens_read.c: program error,\
                   unknown sens_type\n");
                   return(ERROR);
               }
        }
        else{
            printf("sens_read.c: program error,\
                unknown number\n");
                return(ERROR);

        }
        for(next_entry = begin_ptr; ;next_entry++){
            if (*next_entry == '\t')  break;
            if (*next_entry == '\n'){
                goto next_line;
            }
        }
        begin_ptr = next_entry + 1;
        }
        }
next_line:   ;
    }
end:    
    fclose(tran_sens_file);
    *num_entries += entries;
    return(OK);
}
