/*
 * defaults.c
 *
 * set sim2facet command line defaults.
 */

#include "sim2facet.h"

extern s_options Options;
extern s_cells Cells;
extern octObject oContentsFacet;
extern octObject oInterfaceFacet;
extern octObject oSimFacet;
extern octObject oPalette;
extern char *DefaultTechnology;

/* path to palette				*/
static char PaletteDefault[] =
		"/tech/schematic/crystal/palette:symbolic:contents";

/* path to schematic schematic symbols directory				*/
static char CellDefault[] =
		"/tech/schematic/symbols/";

/*
 * initDefaults
 *
 * initialize the options structure before command line parsing.
 */
initDefaults()
{

    Options.paletteName = NIL(char);
    Options.simFileName = NIL(char);
    Options.simFacetName = NIL(char);
    Options.contentsFacetName = NIL(char);
    DefaultTechnology = "schematic";

}

/*
 * setDefaults
 *
 * set any options unset after command line parsing.
 */
setDefaults()
{
    char *st;

    setCellNames();

    OH_ASSERT(ohUnpackDefaults( &oContentsFacet, "r", ":physical:contents"));
    OH_ASSERT(ohUnpackFacetName( &oContentsFacet, Options.contentsFacetName));

    OH_ASSERT(ohUnpackDefaults( &oInterfaceFacet, "r", ":physical:interface"));
    OH_ASSERT(ohUnpackFacetName( &oInterfaceFacet, Options.contentsFacetName));

    OH_ASSERT(ohUnpackDefaults( &oSimFacet, "w", "::crystal"));
    oSimFacet.contents.facet.cell = oContentsFacet.contents.facet.cell;
    oSimFacet.contents.facet.view = oContentsFacet.contents.facet.view;
    if( Options.simFacetName != NIL(char)){
	oSimFacet.contents.facet.facet = Options.simFacetName;
	if(( st = strchr( Options.simFacetName, (int) ':')) != NIL(char)){
	    st = '\0';
	    ++st;
	    oSimFacet.contents.facet.version = st;
	}
    }

    if( Options.simFileName == NIL(char)){
	/* default to input cell name plus ".sim"			*/
	Options.simFileName = ALLOC( char,
			    strlen( oContentsFacet.contents.facet.cell) +
			    strlen( oContentsFacet.contents.facet.view) + 6);
	(void) sprintf(Options.simFileName, "%s_%s.sim", 
			    oContentsFacet.contents.facet.cell,
			    oContentsFacet.contents.facet.view);
    }

    OH_ASSERT(ohUnpackDefaults( &oPalette, "r", ":symbolic:contents"));
    if( Options.paletteName == NIL(char)){
	st = ALLOC( char, strlen("$OCTTOOLS") + strlen( PaletteDefault) + 1 );
	(void) sprintf( st, "%s%s", "$OCTTOOLS", PaletteDefault);
	OH_ASSERT(ohUnpackFacetName( &oPalette, st));
    } else {
	OH_ASSERT(ohUnpackFacetName( &oPalette, Options.paletteName));
    }
}


setCellNames()
{
    int pl;

    pl = strlen( "$OCTTOOLS") + strlen( CellDefault) + 1;

    Cells.nchanFet.name = "NCHAN_FET";
    Cells.nchanFet.master = ALLOC( char, pl + strlen( "nmos"));
    (void) sprintf( Cells.nchanFet.master, "%s%s%s", "$OCTTOOLS", CellDefault, "nmos");
    Cells.nchanFet.view = "schematic";

    Cells.pchanFet.name = "PCHAN_FET";
    Cells.pchanFet.master = ALLOC( char, pl + strlen( "pmos"));
    (void) sprintf( Cells.pchanFet.master, "%s%s%s", "$OCTTOOLS", CellDefault, "pmos");
    Cells.pchanFet.view = "schematic";

    Cells.nenhFet.name = "NENH_FET";
    Cells.nenhFet.master = ALLOC( char, pl + strlen( "nmos"));
    (void) sprintf( Cells.nenhFet.master, "%s%s%s", "$OCTTOOLS", CellDefault, "nmos");
    Cells.nenhFet.view = "schematic";

    Cells.ndepFet.name = "NDEP_FET";
    Cells.ndepFet.master = ALLOC( char, pl + strlen( "nmos-dep"));
    (void) sprintf( Cells.ndepFet.master, "%s%s%s", "$OCTTOOLS", CellDefault, "nmos-dep");
    Cells.ndepFet.view = "schematic";

    Cells.capacitor.name = "CAPACITOR";
    Cells.capacitor.master = ALLOC( char, pl + strlen( "capacitor"));
    (void) sprintf(Cells.capacitor.master, "%s%s%s", "$OCTTOOLS", CellDefault,"capacitor");
    Cells.capacitor.view = "schematic";
}
