/*
 * oct.c
 *
 * This contains routines that perform the utility operations of
 * interfacing with the oct database, without knowing about the
 * semantics of sim.
 */

#include "sim2facet.h"
#include "st.h"

/* global imports							*/
octObject oSimFacet;
extern char *optProgName;
extern s_cells Cells;
extern octObject oSimFacet;
extern octObject oPalette;

/* globals for the sim facet						*/
octObject oInstances;
octObject oFetWidProp;
octObject oFetLenProp;
octObject oFetModProp;

/*
 * getFetInst
 *
 */
getFetInst( fet, oInst)
s_simFet *fet;
octObject *oInst;
{
    char *name;

    switch( fet->type ){
	case NCHAN:
	    name = Cells.nchanFet.name;
	    break;
	case PCHAN:
	    name = Cells.pchanFet.name;
	    break;
	case NENH:
	    name = Cells.nenhFet.name;
	    break;
	case NDEP:
	    name = Cells.ndepFet.name;
	    break;
	default:
	    fprintf( stderr,"%s: unknown FET type %d, Skipping\n",
			optProgName, fet->type);
	    break;
    }

    if( ohGetByInstName( &oPalette, oInst, name) < OCT_OK){
	fprintf(stderr,"%s: Fatal Error: couldn't get %s from palette %s:%s:%s: %s\n",
			optProgName,
			oInst->contents.instance.name,
			oPalette.contents.facet.cell,
			oPalette.contents.facet.view,
			oPalette.contents.facet.facet,
			octErrorString());
    }

    /* set the instance's transform					*/
    oInst->contents.instance.transform.translation.x = (octCoord) fet->xloc*20;
    oInst->contents.instance.transform.translation.y = (octCoord) fet->yloc*20;

    /* create the instance in the facet					*/
    OH_ASSERT( octCreate( &oSimFacet, oInst));

    /* assume its an INSTANCES instance					*/
    OH_ASSERT( octAttach( &oInstances, oInst));
}

/*
 * getCapacitorInst
 *
 */
getCapacitorInst( oInst)
octObject *oInst;
{
    if( ohGetByInstName( &oPalette, oInst, Cells.capacitor.name) < OCT_OK){
	fprintf(stderr,"%s: Fatal Error: couldn't get %s from palette %s:%s:%s: %s\n",
			optProgName,
			oInst->contents.instance.name,
			oPalette.contents.facet.cell,
			oPalette.contents.facet.view,
			oPalette.contents.facet.facet,
			octErrorString());
    }

    /* create the instance in the facet					*/
    OH_ASSERT( octCreate( &oSimFacet, oInst));

    /* assume its an INSTANCES instance					*/
    OH_ASSERT( octAttach( &oInstances, oInst));
}

/*
 * getNodeNet
 *
 */
getNodeNet( name, oNet)
char *name;
octObject *oNet;
{
    oNet->type = OCT_NET;
    oNet->contents.net.name = name;
    oNet->contents.net.width = 1;
    if( octGetByName( &oSimFacet, oNet) == OCT_NOT_FOUND){
	OH_ASSERT( octCreate(&oSimFacet, oNet)); 
    }
}
