/*
 * octSim.c
 *
 * Routines to map individule .sim statements into Oct equivalents.
 */

#include "sim2facet.h"

/* macros used here							*/
#define GATE "GATE"
#define GATE1 "GATE1"
#define DRAIN "DRAIN"
#define SOURCE "SOURCE"
#define BULK "BULK" /* ???: its in the symbols/*:schemtic facets	*/
#define CAP_TERM1 "TOP"
#define CAP_TERM2 "BOTTOM"

extern s_options Options;
extern octObject oSimFacet;
extern octObject oInterfaceFacet;
extern octObject oContentsFacet;
extern octObject oPalette;
extern octObject oInstances;
extern octObject oFetLenProp;
extern octObject oFetWidProp;
extern octObject oFetModProp;
extern char *optProgName;
extern char *VddName;
extern char *GroundName;
extern char *DefaultTechnology;

/*
 * initFacets
 *
 * Open the contents file associated with the .sim file.  This must exist.
 * Open the facet to be created. This may or may not already exits.
 * Add the INSTANCES bag to the new facet and fill out values of global
 * properties.
 */
initFacets( )
{
    octObject oProp;

    if( octOpenFacet( &oContentsFacet ) != OCT_OLD_FACET ){
	errRaise( optProgName, -1,
		"Could not open existing facet `%s:%s:%s:%s': %s\n",
		 oContentsFacet.contents.facet.cell,
		 oContentsFacet.contents.facet.view,
		 oContentsFacet.contents.facet.facet,
		 oContentsFacet.contents.facet.version,
		 octErrorString());
    }

    if( octOpenFacet( &oInterfaceFacet ) != OCT_OLD_FACET ){
	errRaise( optProgName, -1,
		"Could not open existing facet `%s:%s:%s:%s': %s\n",
		 oInterfaceFacet.contents.facet.cell,
		 oInterfaceFacet.contents.facet.view,
		 oInterfaceFacet.contents.facet.facet,
		 oInterfaceFacet.contents.facet.version,
		 octErrorString());
    }

    if( octOpenFacet( &oSimFacet ) < OCT_OK ){
	errRaise( optProgName, -1,
		"Could not open facet `%s:%s:%s:%s': %s\n",
		 oSimFacet.contents.facet.cell,
		 oSimFacet.contents.facet.view,
		 oSimFacet.contents.facet.facet,
		 oSimFacet.contents.facet.version,
		 octErrorString());
    }

    if( octOpenFacet( &oPalette ) < OCT_OK ){
	errRaise( optProgName, -1,
		"Could not open palette `%s:%s:%s:%s': %s\n",
		 oPalette.contents.facet.cell,
		 oPalette.contents.facet.view,
		 oPalette.contents.facet.facet,
		 oPalette.contents.facet.version,
		 octErrorString());
    }

    oInstances.type = OCT_BAG;
    oInstances.contents.bag.name = "INSTANCES";
    OH_ASSERT( octCreate( &oSimFacet, &oInstances));

    oFetWidProp.type = OCT_PROP;
    oFetWidProp.contents.prop.type = OCT_REAL;
    oFetWidProp.contents.prop.name = "MOSFETWIDTH";

    oFetLenProp.type = OCT_PROP;
    oFetLenProp.contents.prop.type = OCT_REAL;
    oFetLenProp.contents.prop.name = "MOSFETLENGTH";

    oFetModProp.type = OCT_PROP;
    oFetModProp.contents.prop.type = OCT_STRING;
    oFetModProp.contents.prop.name = "SIM_MODEL";

    OH_ASSERT( ohCreatePropStr( &oSimFacet, &oProp, "VIEWTYPE", "SCHEMATIC"));
    OH_ASSERT( ohCreatePropStr( &oSimFacet, &oProp, "EDITSTYLE", "SCHEMATIC"));

    if( ohGetByPropName( &oInterfaceFacet, &oProp, "TECHNOLOGY") == OCT_OK){
	OH_ASSERT( octCreate( &oSimFacet, &oProp));
    } else {
	OH_ASSERT( ohCreatePropStr( &oSimFacet, &oProp, "TECHNOLOGY",
			DefaultTechnology));
    }

    /*** ??? 
     *** the crystal reader will look at the interface facet anyway,
     *** so why bother with this stuff.
    if( ohGetByPropName( &oInterfaceFacet, &oProp, "CELLCLASS") == OCT_OK){
	OH_ASSERT( octCreate( &oSimFacet, &oProp));
    } else {
	OH_ASSERT( ohCreatePropStr( &oSimFacet, &oProp, "CELLCLASS", "MODULE"));
    }
    if( ohGetByPropName( &oInterfaceFacet, &oProp, "CELLTYPE") == OCT_OK){
	OH_ASSERT( octCreate( &oSimFacet, &oProp));
    } else {
	OH_ASSERT( ohCreatePropStr( &oSimFacet, &oProp, "CELLTYPE",
			"SIM_UNKNOWN"));
    }
    */
}

/*
 * octPutFet
 *
 * add a transistor to the output facet.
 */
octPutFet( fet )
s_simFet *fet;
{
    octObject oInst;
    octObject oSnet, oGnet, oDnet;
    octObject oTerm;
    octGenerator gTerm;
    octStatus sTerm;
    int result = TRUE;

    /* get an instance of the appropriate FET type			*/
    getFetInst( fet, &oInst);

    /* get the nets for these node names				*/
    getNodeNet( fet->source, &oSnet);
    getNodeNet( fet->gate, &oGnet);
    getNodeNet( fet->drain, &oDnet);

    /* generate all the terminals of this instance and attach to nets	*/
    OH_ASSERT( octInitGenContents( &oInst, OCT_TERM_MASK, &gTerm));
    while(( sTerm = octGenerate( &gTerm, &oTerm)) == OCT_OK){
	if( !strcmp( oTerm.contents.term.name, GATE)
	    || !strcmp( oTerm.contents.term.name, GATE1)){
	    OH_ASSERT( octAttach( &oGnet, &oTerm));
	} else if( !strcmp( oTerm.contents.term.name, DRAIN)){
	    OH_ASSERT( octAttach( &oDnet, &oTerm));
	} else if( !strcmp( oTerm.contents.term.name, SOURCE)){
	    OH_ASSERT( octAttach( &oSnet, &oTerm));
	} else if( strcmp( oTerm.contents.term.name, BULK)){
	    fprintf(stderr,"%s: FET instance %s: unknown terminal %s\n",
		optProgName, ohFormatName( &oInst), ohFormatName( &oTerm));
	    result = FALSE;
	}
    }
    OH_ASSERT( sTerm );

    /* attach length/width/model name props				*/
    /* convert microns to meters					*/
    oFetWidProp.contents.prop.value.real = (double) (fet->width * 1e-6);
    OH_ASSERT( octCreate( &oInst, &oFetWidProp));

    oFetLenProp.contents.prop.value.real = (double) (fet->length * 1e-6);
    OH_ASSERT( octCreate( &oInst, &oFetLenProp));

    if( fet->modelName != NIL(char)){
	oFetModProp.contents.prop.value.string = fet->modelName;
	OH_ASSERT( octCreate( &oInst, &oFetModProp));
    }

    return( result );
}

/*
 * octPutCapacitor
 *
 * add a capacitor to the output facet.
 */
int
octPutCapacitor( cap )
s_simCapacitor *cap;
{
    octObject oNet1, oNet2, oInst, oTerm;
    octGenerator gTerm;
    octStatus sTerm;
    int result = TRUE;

    /* get an instance of a capacitor					*/
    getCapacitorInst( &oInst);

    /* get the nets							*/
    getNodeNet( cap->node1, &oNet1);
    getNodeNet( cap->node2, &oNet2);

    OH_ASSERT( octInitGenContents( &oInst, OCT_TERM_MASK, &gTerm));
    while(( sTerm = octGenerate( &gTerm, &oTerm)) == OCT_OK){
	if( !strcmp( oTerm.contents.term.name, CAP_TERM1)){
	    OH_ASSERT( octAttach( &oNet1, &oTerm));
	} else if( !strcmp( oTerm.contents.term.name, CAP_TERM2)){
	    OH_ASSERT( octAttach( &oNet2, &oTerm));
	} else {
	    fprintf(stderr,"%s: FET instance %s: unknown terminal %s\n",
		optProgName, ohFormatName( &oInst), ohFormatName( &oTerm));
	    result = FALSE;
	}
    }
    OH_ASSERT( sTerm );

    /* attache cap prop to capacitor (reusing oNet1 var, here as prop)	*/
    /* convert pico-farads to farads					*/
    OH_ASSERT( ohCreatePropReal( &oInst, &oNet1, "CAPACITANCE",
				(double) (cap->cap * 1e-12) ));

    return( result );
}

/*
 * octPutNodeRes
 *
 * add resistance to a net in the output facet.
 */
octPutNodeRes( res )
s_simNodeRes *res;
{
    octObject oNet, oProp;

    getNodeNet( res->node, &oNet);

    OH_ASSERT( ohCreatePropReal( &oNet, &oProp, "SIM_RESISTANCE",
		(double) res->res));

    return( TRUE );
}

/*
 * octPutNodeCap
 *
 * add capacitance to a net in the output facet.
 */
octPutNodeCap( ncap )
s_simNodeCap *ncap;
{
    octObject oNet, oProp;

    getNodeNet( ncap->node, &oNet);

    /* add props, convert micron and sq. micron to meter and sq. meter	*/
    OH_ASSERT( ohCreatePropReal( &oNet, &oProp, "SIM_DIFF_AREA",
		(double) (ncap->da * 1e-12) ));
    OH_ASSERT( ohCreatePropReal( &oNet, &oProp, "SIM_DIFF_PERIM",
		(double) (ncap->dp * 1e-6) ));
    OH_ASSERT( ohCreatePropReal( &oNet, &oProp, "SIM_POLY_AREA",
		(double) (ncap->pa * 1e-12) ));
    OH_ASSERT( ohCreatePropReal( &oNet, &oProp, "SIM_POLY_PERIM",
		(double) (ncap->pp * 1e-6) ));
    OH_ASSERT( ohCreatePropReal( &oNet, &oProp, "SIM_MET_AREA",
		(double) (ncap->ma * 1e-12) ));
    OH_ASSERT( ohCreatePropReal( &oNet, &oProp, "SIM_MET_PERIM",
		(double) (ncap->mp * 1e-6) ));

    return( TRUE );
}

/*
 * attachFormals
 *
 * Generate all the formal terms and link them to the nets with
 * the same name.  Also add NETTYPE and TERMTYPE props to
 * supply and ground nets and terms.
 */
attachFormals()
{
    octObject oITerm , oCTerm, oSTerm, oCNet, oSNet, oProp;
    octGenerator gITerm;
    octStatus sITerm;
    int netFound = 0;

    oSNet.type = OCT_NET;
    oProp.type = OCT_PROP;
    oSTerm.type = OCT_TERM;
    OH_ASSERT( octInitGenContents( &oContentsFacet, OCT_TERM_MASK, &gITerm));
    while(( sITerm = octGenerate( &gITerm, &oCTerm)) == OCT_OK){

	/* set the name field of the Sim facet net and term		*/
	oSNet.contents.net.name = oSTerm.contents.term.name
		= oCTerm.contents.term.name;

	/* attach term to net in Sim facet				*/
	OH_ASSERT( octGetByName( &oSimFacet, &oSTerm));
	if ( octGetByName( &oSimFacet, &oSNet) != OCT_OK) {
	  continue;
	} 
	OH_ASSERT( octAttach(&oSNet, &oSTerm));

	oITerm = oCTerm;
	OH_ASSERT( octGetByName(&oInterfaceFacet,&oITerm));

	/* check for props on interface facet term and net		*/

	/* get the net from the contents facet				*/
	if (octGenFirstContainer( &oCTerm, OCT_NET_MASK, &oCNet) == OCT_OK &&
	    ohGetByPropName( &oCNet, &oProp, "NETTYPE") == OCT_OK) {
	    OH_ASSERT( octCreate( &oSNet, &oProp));
	    if( !strcmp(oProp.contents.prop.value.string, "SUPPLY")){
		netFound |= 1;
	    } else if( !strcmp(oProp.contents.prop.value.string, "GROUND")){
		netFound |= 2;
	    }
	} else if( ohGetByPropName( &oITerm, &oProp, "TERMTYPE") == OCT_OK){
	    OH_ASSERT( octCreate( &oSTerm, &oProp));
	    if( !strcmp(oProp.contents.prop.value.string, "SUPPLY")){
		netFound |= 1;
	    } else if( !strcmp(oProp.contents.prop.value.string, "GROUND")){
		netFound |= 2;
	    }
	}
    }
    OH_ASSERT( sITerm);
    /* see if there was a Vdd and/or GND net				*/
    if( !( netFound & 1) || !( netFound & 2)){
	if( !( netFound & 1)){
	    fprintf(stderr,"%s: Warning: SUPPLY term or net not found\n",
			optProgName);
	}
	if( !( netFound & 2)){
	    fprintf(stderr,"%s: Warning: GROUND term or net not found\n",
			optProgName);
	}
	fprintf(stderr,"\tSupply and ground nets are recognized by proptery in %s.\n",
		optProgName);
	fprintf(stderr,"\tIf you want a net to be supply or ground,\n");
	fprintf(stderr,"\tadd a NETTYPE or TERMTYPE prop with value \"SUPPLY\" or \"GROUND\"\n");
	fprintf(stderr,"\tto the appropriate term or net in the interface facet.\n");
    }
}
