#include "copyright.h"
#include "port.h"
#include "oct.h"
#include "st.h"
#include "vulcan.h"
#include "options.h"
#include "oh.h"
#include "region.h"
#include "utility.h"


#define PRINTF	if (!quietp) (void) printf

optionStruct optionList[] = {
    { "d",	0,		"do the modifications" },
    { "u",	0,		"do not delete unimplemented formals (good for bdnet output)" },
    { "n",	0,		"warn about actual terminals not on nets" },
    { "r",      0,              "recursive" },
    { "v",	0,		"do not run vulcan" },
    { "l",      0,              "do lambda snapping only" },
    { "N",	0,		"allow the checking of facets that do not have VIEWTYPE equal to SCHEMATIC or SYMBOLIC" },
    { "q",	0,		"quiet, no messages" },
    { OPT_RARG,	"cell:view",	"name of the view to clean" },
    { 0,	0,		0 },
};

static void printerror( obj, msg )
    octObject* obj;
    char* msg;
{
    /* Print only if there is something to be printed */
    if ( msg && msg[0] != '\0' ) {
	fprintf( stderr, "%s\n", msg );
    }
}

static int doall( facet, fixp )
    octObject* facet;
    int fixp;
{
    int errcount = 0;

    printf( "symhelp: doing lambdas\n" );
    errcount = symcleanLambdas( facet, fixp );

    printf( "symhelp: doing instance and net names\n" );
    errcount = symcleanCheckNames( facet, fixp );

    printf( "symhelp: doing nets\n");
    errcount += symcleanNets( facet, fixp );
    
    printf( "symhelp: doing formals\n");
    errcount += symcleanFormals( facet, fixp );

    printf( "symhelp: doing segments\n");
    errcount += symcleanSegments( facet, fixp);

    printf( "symhelp: doing instances\n");
    errcount += symcleanInstances( facet, fixp );

    printf( "symhelp: doing garbage\n");
    errcount += symcleanGarbage( facet, fixp );


    printf( "symhelp: doing spc\n" );
    errcount += scCheck( facet );
    return errcount;
}


main(argc, argv)
int argc;
char **argv;
{
    octObject interface, facet, net, bag, term, prop;
    octObject aterm, layer, path, geo, master, instance;
    octObject fterm, joined, joinedterm, ajoinedterm, oldnet;
    octObject instancesbag, connectorsbag;
    struct octBox bb, origBB, finalBB;
    octGenerator gen, gen2, gen3;
    st_table *TermTable, *NetTable;
    int cnt, again = 0, oncethrough = 0, schematicp = 0;
    int option;
    extern int optind;
    extern char *optarg;
    int modifyp = 0, keepformals = 0, vulcanp = 1, fullvulcanp = 0;
    int actualp = 0, strictp = 1;
    int fixp = 0;
    int quietp = 0;
    int errcount = 0;
    int recursive = 0;
    int snapOnly = 0;
    char *version = "1.2";

    while ((option = optGetOption(argc, argv)) != EOF) {
	switch (option) {
	case 'd': modifyp = 1; fixp = 1; break;
	case 'r': recursive = 1; break;
	case 'q': quietp = 1; break;
	case 'N': strictp = 0; break;
	case 'n': actualp = 1; break;
	case 'l': snapOnly = 1; break;
	}
    }
    
    PRINTF("\nSYMHELP (version %s)\n\n", version);
    symcleanSetErrorReport( printerror );
    if (argc == optind + 1) {
	ohUnpackDefaults(&facet, (modifyp ? "a" : "r"), "::contents");
	if (ohUnpackFacetName(&facet, argv[optind]) != OCT_OK) {
	    optUsage();
	}
    } else {
	optUsage();
    }

    if (octOpenFacet(&facet) != OCT_OLD_FACET) {
	(void) printf("%s:%s does not exist\n",
		      facet.contents.facet.cell,
		      facet.contents.facet.view);
	exit(-1);
    }

    if ( snapOnly ) {
	printf( "symhelp: doing lambdas\n" );
	errcount = symcleanLambdas( &facet, fixp );
    } else {
	errcount = doall( &facet, fixp );
    }
    if ( recursive ) {
	octObject bag,inst,master;
	octGenerator gen;
	ohGetByBagName( &facet, &bag, "INSTANCES" );
	octInitGenContents( &bag, OCT_INSTANCE_MASK, &gen );
	while ( octGenerate( &gen, &inst ) == OCT_OK ) {
	    int err = 0;
	    OH_ASSERT( ohOpenMaster( &master, &inst, "contents", (modifyp?"a":"r") ));
	    
	    PRINTF("Doing %s\n", ohFormatName(&master) );
	    if ( snapOnly ) {
		err += symcleanLambdas( &master, fixp );
	    } else {
		err += doall( &master, fixp );
	    }
	    if ( err > 0 ) {
		OH_ASSERT( octFlushFacet( &master )) ;
	    }
	}
    }

    if ( errcount > 0 && fixp ) {
	OH_ASSERT(octFlushFacet( &facet ));
    }

    exit( errcount );
}


