/*
 * VEM Dialog Manager Interface Routines
 * 
 * David Harrison
 * University of California,  Berkeley
 * 1986, 1989, 1990
 *
 * This file contains easy to use programming interfaces for common
 * dialog operations.  These are usually short moded dialogs which
 * ask questions that require an answer immediately.  The prefix of
 * these routines is "dm".
 *
 * This package is now implemented on top of the more powerful
 * Xt toolkit based package "dds".  It is provided for
 * compatibility.  The library must be initialized using
 * dmCompatInit().
 *
 * The routines defined here are internal for use by the rest of
 * the package.
 */

#include "vemDMint.h"

char *vemdm_pkg_name = "vemdm";
int _vemdm_initialized = 0;
ddsEvtHandler _dmCompatHandler;



void dmCompatInit(evt_handler)
int (*evt_handler)();
/*
 * Application top level widget for resource gathering.
 */
{
    _vemdm_initialized = 1;
    _dmCompatHandler.other_handler = evt_handler;
    _dmCompatHandler.user_data = (ddsData) 0;
}


/*
 * Standard control button callback functions
 */

static void std_cb(item, preserve, status)
ddsHandle item;
int preserve;
vemStatus status;
/*
 * Standard callback template for pushing "Ok" or "Cancel" buttons.
 * Sets the status in the callback data and unposts (or destroys)
 * the dialog.
 */
{
    ddsControl data;
    con_callback *cb_info;

    dds_get(item, (ddsData) &data);
    cb_info = (con_callback *) data.user_data;
    cb_info->status = status;
    if (cb_info->help) dds_destroy(cb_info->help);
    if (preserve) {
	dds_unpost(item);
    } else {
	dds_destroy(item);
    }
}

void _vemdm_general_ok(item)
ddsHandle item;
/*
 * Called when the "Ok" button is pressed in dmWhichOne or dmMultiWhich.
 */
{
    std_cb(item, 0, VEM_OK);
}

void _vemdm_general_cancel(item)
ddsHandle item;
/*
 * Called when the "Ok" button is pressed in dmWhichOne or dmMultiWhich.
 */
{
    std_cb(item, 0, VEM_NOSELECT);
}

void _vemdm_general_no(item)
ddsHandle item;
/*
 * Called when "no" button is pushed in dmConfirm
 */
{
    std_cb(item, 0, VEM_FALSE);
}

void _vemdm_preserve_ok(item)
ddsHandle item;
/*
 * Called when the "Ok" button is pressed in dmMultiText.
 */
{
    std_cb(item, 1, VEM_OK);
}

void _vemdm_preserve_cancel(item)
ddsHandle item;
/*
 * Called when the "Ok" button is pressed in dmMultiText.
 */
{
    std_cb(item, 1, VEM_NOSELECT);
}



