/*
 * Fill Pattern Manipulation Header
 *
 * David Harrison
 * University of California, 1985, 1987, 1989
 *
 * This file contains declarations for use of the fill pattern manipulation
 * package implemented in fillPat.c.  Other files which should be included
 * before this one:
 */

#ifndef FILLPAT
#define FILLPAT

#include "ansi.h"
#include "general.h"
#include "X11/Xlib.h"

/* Can be typedef'ed to void if supported */
typedef struct fill_dummy_defn {
    int dummy;
} fill_dummy;

typedef fill_dummy *flPattern;	/* Internal representation of a fill pattern */

extern flPattern flNewPattern
  ARGS((int width, int height, STR dataStr));
  /* Create a new fill pattern from external form */

extern void flSize
  ARGS((flPattern pat, int *width, int *height));
  /* Returns the current size of a fill pattern */

extern vemStatus flIsSolid
  ARGS((flPattern pat));
  /* Returns VEM_OK if the pattern is a solid fill,  VEM_FALSE otherwise */

extern void flXBitmap
  ARGS((Display *disp, Visual *vis, flPattern pat, int *width, int *height,
	char **data));
  /* Returns pattern in format suitable for making X bitmaps and pixmaps */

extern void flXLineStyle
  ARGS((flPattern pat, int *offset, int *len, char **list));
  /* Returns pattern in format suitable for using as X line style */

extern void flFreePattern
  ARGS((flPattern fillPat));
  /* Releases the resources associated with a pattern created by vemNewPattern */

extern flPattern flCopyPattern
  ARGS((flPattern pat));
  /* Returns a copy of `pat' */

extern flPattern flChangeSize
  ARGS((flPattern pat, int new_width, int new_height));
  /* Sizes a pattern to a specified size using clipping and replication */

extern vemStatus flEqual
  ARGS((flPattern pat1, flPattern pat2));
  /* Compares two fill patterns to see if they are equal */

#endif
