/*
 * Technology Maintainence Package Header File
 *
 * David Harrison
 * University of California,  Berkeley
 * 1987
 *
 * This file contains definitions for using the technology
 * maintainence package.  This package is built on top
 * of TAP (technology access package) written by T. Laidig.
 */

#ifndef TECHHEADER
#define TECHHEADER

#include "general.h"		/* General VEM defns */
#include "oct.h"		/* Database          */
#include "attributes.h"		/* Attribute handler */

/* Exported type for handing a technology */

/* Can be typedef'ed to void if supported */
typedef struct tk_dummy_def {
    int dummy;
} tk_dummy;

typedef tk_dummy *tkHandle;

/* Exported type for a layer in a technology */
typedef struct tk_lyr_defn {
    int priority;		/* Layer priority */
} tk_lyr;

typedef tk_lyr *tkLayer;


/*
 * Exported functions and procedures
 */

extern tkHandle tkGet ARGS((octObject *theFacet));
  /* Returns a technology given an Oct facet */
extern void tkFree ARGS((tkHandle tech));
  /* Frees resources associated with a technology handle */
extern void tkWipe ARGS(());
  /* Remove cached information on all known technologies */

extern vemStatus tkIsTapFacet ARGS((octObject *fct));
  /* Returns VEM_OK if the supplied facet is a tap facet */

extern tkLayer tkLyr ARGS((tkHandle tech, octObject *lyrObj));
  /* Retrieves a layer from a technology given an Oct layer */

extern atrHandle tkAttr ARGS((tkLayer lyr, int alt));
  /* Gets an attribute from a technology layer */

extern vemStatus tkMinWidth ARGS((tkHandle tech, STR lyrName, octCoord *minwidth));
  /* Minimum width of a named layer */

#endif
