/*
 * Object Selection Set Header
 *
 * David Harrison
 * University of California,  Berkeley
 * 1987
 *
 * This file contains header definitions for mulitple object selection
 * set highlighting.  The implementation of the module is in the
 * file objsel.c.  The module exports routines for highlighting
 * any number of objects using any number of colors in all
 * appropriate windows.
 *
 * The philosophy used in this module is based on the vemObjects
 * structure defined in commands.h.  The module supplies routines
 * for modifying this structure that also have the side-effect
 * of causing the appropriate highlighting to occur.  This is
 * a low-level interface used by the selection and highlighting
 * modules.
 */

#ifndef OBJSEL_HEADER
#define OBJSEL_HEADER

#include "commands.h"		/* Dispatch module */

extern void objOn
  ARGS((vemObjects *objset));
  /* Turns on the highlighting of objects in a set */

extern void objOff
  ARGS((vemObjects *objset));
  /* Turns off the highlighting of objects in a set */

extern void objAdd
  ARGS((vemObjects *objset, octObject *obj));
  /* Adds a new object to an existing set   */

extern void objSub
  ARGS((vemObjects *objset, octObject *obj));
  /* Removes an object from an existing set */

extern void objRepl
  ARGS((vemObjects *objset, octObject *oldobj, octObject *newobj));
  /* Special routine to replace an object   */

extern void objTran
  ARGS((vemObjects *objset, struct octTransform *newtran));
  /* Changes the transformation of a set   */

extern vemStatus objMove
  ARGS((vemObjects *objset, octCoord xoff, octCoord yoff));
  /* Attempts to move object set as a unit */

extern vemStatus objBB
  ARGS((vemObjects *objset, struct octBox *bb));
  /* Returns the bounding box of a set     */

extern vemStatus objTranBB
  ARGS((vemObjects *objset, struct octBox *bb));
  /* Returns transformed bounding box      */

extern void objRedraw();
  /* Redraws selected objects in a region */

#endif
