/* file name     : writeoct.c */


/* WriteOct - routines for translating a module in YAL format to oct format.
 
   Copyright (C) 1988 by Yoshihito Nishizaki.  All rights reserved.

 */


#include "copyright.h"
#include "port.h"
#include "benchtypes.h"
#include "oct.h"
#include "vulcan.h"


/* define some stuff */
#define OCT_PER_MICRON  20
#define BSIZE 512
#define MAXINT (2147483647)
#define OCTERR(errMsg) \
{ \
(void) fprintf(stderr, "ERROR oct database call at %d:\"%s\"\n",\
	__LINE__, __FILE__);\
octError(errMsg); \
}
#define OCTCHKOK(octCall, errMsg) \
{ \
if (octCall!=OCT_OK) { \
(void) fprintf(stderr, "ERROR oct database call at %d:\"%s\"\n",\
        __LINE__, __FILE__);\
octError(errMsg); \
} }


/*ARGSUSED*/
WriteOct(Name, ModType, Dimensions, IOListHead,
	  NetworkListHead, PlacementListHead,CriticalNetListHead)

NameType Name;
ModuleType ModType;
CoordPair *Dimensions;
IOList *IOListHead;
NetworkList *NetworkListHead;
PlacementList *PlacementListHead;
CriticalNetList *CriticalNetListHead;


{

    IOList	*IOListPin;
    IOList	*IOListPin1;
    CoordPair	*CurrentPair;
    NetworkList	*CurrentNetworkEntry;
    SignalList	*CurrentSignal;
    CriticalNetList *CurrentCriticalNetEntry;

    /* oct data definition */
    octObject       octCell;
    octObject       octInst;
    octObject       octNet;
    octObject       octTerm;
    octObject       oBox, octOutline;
    octObject       pdif_layer,ndif_layer,poly_layer,met1_layer,met2_layer;
    octObject       place_layer;
    octObject       octTermtype,octDirection,octLength,octNettype,octClass,octEdge;
    octObject       octMobility,octPosition;
    octObject	octJoined,octJoinedterms,octInstances,octFloorplan;
    octObject	octProp;
    octObject	octCurrent;

    octObject       octInt;
    octObject       octIntProp;
    octObject       octIntTerm;
    octGenerator    genIntTerm;

    octGenerator    genProp;

    octObjectMask   octMask;


    int             term_No,sub_No;
    int             numDup;
    float           xmax,ymax;
    float           xmin,ymin;
    octCoord        x, y;
    char buf[BSIZE];
    NameType	previous_name;


    /*********************/
    /* processing begins */
    /*********************/

    /* Open cell facet */
    /*-----------------*/
    octCell.type = OCT_FACET;
    octCell.contents.facet.cell = Name;
    if (ModType == PARENT) {
	octCell.contents.facet.view = "symbolic";
    } else {
	octCell.contents.facet.view = "physical";
    }
    octCell.contents.facet.facet   = "contents";
    octCell.contents.facet.mode    = "w";
    octCell.contents.facet.version = OCT_CURRENT_VERSION;
    if (octOpenFacet(&octCell)<OCT_OK)
      OCTERR("cannot open facet");


    /* create oct properties */
    /*-----------------------*/
    octProp.type = OCT_PROP;
    octProp.contents.prop.type = OCT_STRING;

    octProp.contents.prop.name = "EDITSTYLE";
    octProp.contents.prop.value.string =
      (ModType==PARENT ? "SYMBOLIC" : "PHYSICAL");
    OCTCHKOK(octCreate(&octCell, &octProp),"cannot add property");
    octProp.contents.prop.name = "VIEWTYPE";
    OCTCHKOK(octCreate(&octCell, &octProp),"cannot add property");
    octProp.contents.prop.name = "TECHNOLOGY";
    octProp.contents.prop.value.string = "scmos";
    OCTCHKOK(octCreate(&octCell, &octProp),"cannot add property");

    if (ModType == PAD) {
	octClass.type = OCT_PROP;
	octClass.contents.prop.name = "PLACEMENT_CLASS";
	octClass.contents.prop.type = OCT_STRING;
	octClass.contents.prop.value.string = "PAD";
	OCTCHKOK(octCreate(&octCell, &octClass),"cannot add property");
    }


    /* create bags */
    /*-------------*/
    if (ModType == PARENT) {
	octInstances.type = OCT_BAG;
	octInstances.contents.bag.name = "INSTANCES";
	OCTCHKOK(octCreate(&octCell, &octInstances),"cannot create a bag");
    } else {
	octJoinedterms.type = OCT_BAG;
	octJoinedterms.contents.bag.name = "JOINEDTERMS";
	OCTCHKOK(octCreate(&octCell, &octJoinedterms),"cannot create a bag");
    }

    /* create each layer */
    /*-------------------*/
    pdif_layer.type = OCT_LAYER;
    pdif_layer.contents.layer.name = "PDIF";
    if (octCreate(&octCell,&pdif_layer)<OCT_OK)
      OCTERR("cannot create layer");

    ndif_layer.type = OCT_LAYER;
    ndif_layer.contents.layer.name = "NDIF";
    if (octCreate(&octCell,&ndif_layer)<OCT_OK)
      OCTERR("cannot create layer");

    poly_layer.type = OCT_LAYER;
    poly_layer.contents.layer.name = "POLY";
    if (octCreate(&octCell,&poly_layer)<OCT_OK)
      OCTERR("cannot create layer");

    met1_layer.type = OCT_LAYER;
    met1_layer.contents.layer.name = "MET1";
    if (octCreate(&octCell,&met1_layer)<OCT_OK)
      OCTERR("cannot create layer");

    met2_layer.type = OCT_LAYER;
    met2_layer.contents.layer.name = "MET2";
    if (octCreate(&octCell,&met2_layer)<OCT_OK)
      OCTERR("cannot create layer");

    place_layer.type = OCT_LAYER;
    place_layer.contents.layer.name = "PLACE";
    if (octCreate(&octCell,&place_layer)<OCT_OK)
      OCTERR("cannot create layer");


    /* create outline of the cell */
    /*----------------------------*/
    CurrentPair = Dimensions;
    octOutline.type = OCT_BOX;
    octOutline.contents.box.lowerLeft.x = MAXINT;
    octOutline.contents.box.lowerLeft.y = MAXINT;
    octOutline.contents.box.upperRight.x = -MAXINT;
    octOutline.contents.box.upperRight.y = -MAXINT;

    while(CurrentPair != CoordPairNull) {
        x = CurrentPair->X * OCT_PER_MICRON;
        y = CurrentPair->Y * OCT_PER_MICRON;
        if (x<octOutline.contents.box.lowerLeft.x)
	  octOutline.contents.box.lowerLeft.x = x;
        if (y<octOutline.contents.box.lowerLeft.y)
	  octOutline.contents.box.lowerLeft.y = y;
        if (x>octOutline.contents.box.upperRight.x)
	  octOutline.contents.box.upperRight.x = x;
        if (y>octOutline.contents.box.upperRight.y)
	  octOutline.contents.box.upperRight.y = y;
	CurrentPair = CurrentPair->Link;
    }

    if (ModType != PARENT) {
	if (octCreate(&place_layer,&octOutline)<OCT_OK)
	  OCTERR("cannot create box");
    }


    /*****************************/
    /* process each I/O terminal */
    /*****************************/
    octTermtype.type = OCT_PROP;
    octTermtype.contents.prop.name = "TERMTYPE";
    octTermtype.contents.prop.type = OCT_STRING;

    octDirection.type = OCT_PROP;
    octDirection.contents.prop.name = "DIRECTION";
    octDirection.contents.prop.type = OCT_STRING;

    octEdge.type = OCT_PROP;
    octEdge.contents.prop.name = "TERM_EDGE";
    octEdge.contents.prop.type = OCT_STRING;

    octMobility.type = OCT_PROP;
    octMobility.contents.prop.name = "MOBILITY";
    octMobility.contents.prop.type = OCT_STRING;
    octMobility.contents.prop.value.string = "FIXED";

    octPosition.type = OCT_PROP;
    octPosition.contents.prop.name = "TERM_RELATIVE_POSITION";
    octPosition.contents.prop.type = OCT_REAL;

    octJoined.type = OCT_BAG;
    octJoined.contents.bag.name = "JOINED";

    octCurrent.type = OCT_PROP;
    octCurrent.contents.prop.name = "PEAKCURRENT";
    octCurrent.contents.prop.type = OCT_REAL;

    octNettype.type = OCT_PROP;
    octNettype.contents.prop.name = "NETTYPE";
    octNettype.contents.prop.type = OCT_STRING;

    /* First, check and save the min and max location of terminals */
    /*-------------------------------------------------------------*/
    IOListPin = IOListHead;
    xmax = ymax = (float)-MAXINT;
    xmin = ymin = (float)MAXINT;

    if ((IOListPin->XPosition != NOPOSITION) && (ModType == PARENT)) {
	/* save max and min of x and y of the PARENT terminal location */
	/*	for later processing */
	while (IOListPin != IOListNull) {
	    if ((IOListPin->XPosition == TOP) ||
		(IOListPin->XPosition == BOTTOM)) {
		if (xmax < IOListPin->YPosition) {
		    xmax = IOListPin->YPosition;
		}
		if (xmin > IOListPin->YPosition) {
		    xmin = IOListPin->YPosition;
		}
	    } else {
		if ((IOListPin->XPosition == LEFT) ||
		    (IOListPin->XPosition == RIGHT)) {
		    if (ymax < IOListPin->YPosition) {
			ymax = IOListPin->YPosition;
		    }
		    if (ymin > IOListPin->YPosition) {
			ymin = IOListPin->YPosition;
		    }
		} else {
		    if (xmax < IOListPin->XPosition) {
			xmax = IOListPin->XPosition;
		    }
		    if (xmin > IOListPin->XPosition) {
			xmin = IOListPin->XPosition;
		    }
		    if (ymax < IOListPin->YPosition) {
			ymax = IOListPin->YPosition;
		    }
		    if (ymin > IOListPin->YPosition) {
			ymin = IOListPin->YPosition;
		    }
		}
	    }
	    (void) strcpy(previous_name,IOListPin->SignalName);
	    IOListPin = IOListPin->Link;
	}
    }

    if (xmin > (float)octOutline.contents.box.lowerLeft.x / OCT_PER_MICRON)
      xmin = (float)octOutline.contents.box.lowerLeft.x / OCT_PER_MICRON;
    if (xmax < (float)octOutline.contents.box.upperRight.x / OCT_PER_MICRON)
      xmax = (float)octOutline.contents.box.upperRight.x / OCT_PER_MICRON;
    if (ymin > (float)octOutline.contents.box.lowerLeft.y / OCT_PER_MICRON)
      ymin = (float)octOutline.contents.box.lowerLeft.y / OCT_PER_MICRON;
    if (ymax < (float)octOutline.contents.box.upperRight.y / OCT_PER_MICRON)
      ymax = (float)octOutline.contents.box.upperRight.y / OCT_PER_MICRON;


    /* Next, do all the other processing for each terminal */
    /*-----------------------------------------------------*/
    IOListPin = IOListHead;
    term_No = 0;
    sub_No = 0;

    while (IOListPin != IOListNull) {

	/* create each terminal */
	/*----------------------*/
	octTerm.type = OCT_TERM;
	octTerm.contents.term.instanceId = oct_null_id;
	octTerm.contents.term.width = 1;
	if ((term_No == 0) && (sub_No == 0)) {
	    ++term_No;
	    ++sub_No;
	} else {
            if (!strcmp(IOListPin->SignalName,previous_name)) {
	    	++sub_No;
	    } else {
	    	++term_No;
	    	sub_No = 1;
	    }
	}
	(void) sprintf(buf, "%d_%d", term_No,sub_No);

	/* Naming depends on module type ---> This is to be discussed */
	if (ModType == PARENT) {

	    IOListPin1 = IOListHead;
	    numDup = 0;
	    while (IOListPin1 != IOListPin) {
		if (!strcmp(IOListPin1->SignalName,IOListPin->SignalName)) {
		    ++numDup;
		}
		IOListPin1 = IOListPin1->Link;
	    }
	    if (numDup != 0) {
	    /* This means there appeared a pin of the same name */
	    /*--------------------------------------------------*/
		if (numDup == 1) {
		/* Modify the first terminal with the same name */
		    (void) printf("Terminal name changed from %s to %s_%d because of duplication\n",
			IOListPin->SignalName, IOListPin->SignalName, numDup);
		    (void) sprintf(buf, "%s_%d", IOListPin->SignalName, 
			numDup);
		    octTerm.contents.term.name = IOListPin->SignalName;
		    OCTCHKOK(octGetByName(&octCell,&octTerm),
			 "cannot get a terminal");
		    octTerm.contents.term.name = buf;
		    OCTCHKOK(octModify(&octTerm), "cannot get a terminal");
		}
		(void) printf("Terminal name changed from %s to %s_%d because of duplication\n",
			IOListPin->SignalName, IOListPin->SignalName, numDup+1);
		(void) sprintf(buf, "%s_%d", IOListPin->SignalName, numDup+1);
		octTerm.contents.term.name = buf;
	    } else {
		octTerm.contents.term.name = IOListPin->SignalName;
	    }
	} else {
	    octTerm.contents.term.name = buf;
	}

	if (octCreate(&octCell,&octTerm)<OCT_OK)
	  OCTERR("cannot create terminal");

	/* In case of primitives, check if they are electrically equivalent */
	if ((ModType != PARENT) && (sub_No >= 2)) {
	    if (sub_No == 2) {
		OCTCHKOK(octCreate(&octJoinedterms,&octJoined),
			 "cannot create a bag");
		(void) sprintf(buf, "%d_1", term_No);
		octTerm.contents.term.name = buf;
		OCTCHKOK(octGetByName(&octCell,&octTerm),
			 "cannot get a terminal");
		OCTCHKOK(octAttach(&octJoined,&octTerm),
			 "cannot attach a terminal");
	    }
	    (void) sprintf(buf, "%d_%d", term_No,sub_No);
	    octTerm.contents.term.name = buf;
	    OCTCHKOK(octGetByName(&octCell,&octTerm),
		     "cannot get a terminal");
	    OCTCHKOK(octAttach(&octJoined,&octTerm),
		     "cannot attach a terminal");
	}


	/* In case of PARENT, create a net and attach the terminal to the net */
	/*--------------------------------------------------------------------*/

	if (ModType == PARENT) {
	    octNet.type = OCT_NET;
	    octNet.contents.net.width = 1;
	    octNet.contents.net.name = IOListPin->SignalName;
	    if (octGetOrCreate(&octCell,&octNet)<OCT_OK)
		OCTERR("cannot getOrCreate a net");
	    if (octAttach(&octNet,&octTerm)<OCT_OK)
		OCTERR("cannot attach a terminal to a net");
	  
	    switch(IOListPin->Terminal) {
	    case PWR:
		octNettype.contents.prop.value.string = "SUPPLY";
		if (octGetOrCreate(&octNet,&octNettype)<OCT_OK)
		    OCTERR("cannot create property");
		break;
	    case GND:
		octNettype.contents.prop.value.string = "GROUND";
		if (octGetOrCreate(&octNet,&octNettype)<OCT_OK)
		    OCTERR("cannot create property");
		break;
	    default:
		break;
	    }
	}

	/* add properties to the terminal */
	/*--------------------------------*/
	switch(IOListPin->Terminal) {
	case I:
	    octDirection.contents.prop.value.string = "INPUT";
	    if (octCreate(&octTerm,&octDirection)<OCT_OK)
	      OCTERR("cannot create property");
	    octTermtype.contents.prop.value.string = "SIGNAL";
	    if (octCreate(&octTerm,&octTermtype)<OCT_OK)
	      OCTERR("cannot create property");
	    break;
	case O:
	    octDirection.contents.prop.value.string = "OUTPUT";
	    if (octCreate(&octTerm,&octDirection)<OCT_OK)
	      OCTERR("cannot create property");
	    octTermtype.contents.prop.value.string = "SIGNAL";
	    if (octCreate(&octTerm,&octTermtype)<OCT_OK)
	      OCTERR("cannot create property");
	    break;
	case B:
	    octDirection.contents.prop.value.string = "INOUT";
	    if (octCreate(&octTerm,&octDirection)<OCT_OK)
	      OCTERR("cannot create property");
	    octTermtype.contents.prop.value.string = "SIGNAL";
	    if (octCreate(&octTerm,&octTermtype)<OCT_OK)
	      OCTERR("cannot create property");
	    break;
	case PI:
	    octDirection.contents.prop.value.string = "INPUT";
	    if (octCreate(&octTerm,&octDirection)<OCT_OK)
	      OCTERR("cannot create property");
	    octTermtype.contents.prop.value.string = "SIGNAL";
	    if (octCreate(&octTerm,&octTermtype)<OCT_OK)
	      OCTERR("cannot create property");
	    OCTCHKOK(octCreate(&octTerm,&octMobility),
		     "cannot create a property");
	    break;
	case PO:
	    octDirection.contents.prop.value.string = "OUTPUT";
	    if (octCreate(&octTerm,&octDirection)<OCT_OK)
	      OCTERR("cannot create property");
	    octTermtype.contents.prop.value.string = "SIGNAL";
	    if (octCreate(&octTerm,&octTermtype)<OCT_OK)
	      OCTERR("cannot create property");
	    OCTCHKOK(octCreate(&octTerm,&octMobility),
		     "cannot create a property");
	    break;
	case PB:
	    octDirection.contents.prop.value.string = "INOUT";
	    if (octCreate(&octTerm,&octDirection)<OCT_OK)
	      OCTERR("cannot create property");
	    octTermtype.contents.prop.value.string = "SIGNAL";
	    if (octCreate(&octTerm,&octTermtype)<OCT_OK)
	      OCTERR("cannot create property");
	    OCTCHKOK(octCreate(&octTerm,&octMobility),
		     "cannot create a property");
	    break;
	case F:
	    break;
	case PWR:
	    octTermtype.contents.prop.value.string = "SUPPLY";
	    if (octCreate(&octTerm,&octTermtype)<OCT_OK)
	      OCTERR("cannot create property");
	    break;
	case GND:
	    octTermtype.contents.prop.value.string = "GROUND";
	    if (octCreate(&octTerm,&octTermtype)<OCT_OK)
	      OCTERR("cannot create property");
	    break;
	default:
	    break;
	}

    /* Set TERM_EDGE and TERM_RELATIVE_POSITION property
       using xmin, xmax, ymin and ymax saved before */
	if ((IOListPin->XPosition != NOPOSITION) && (ModType == PARENT)) {
	    if (IOListPin->XPosition == TOP) {
		octEdge.contents.prop.value.string = "TOP";
		octPosition.contents.prop.value.real 
		  = (double)(IOListPin->YPosition - xmin) 
		    / (double)(xmax-xmin);
	    }
	    else if (IOListPin->XPosition == BOTTOM) {
		octEdge.contents.prop.value.string = "BOTTOM";
		octPosition.contents.prop.value.real 
		  = (double)(IOListPin->YPosition - xmin) 
		    / (double)(xmax-xmin);
	    }
	    else if (IOListPin->XPosition == LEFT) {
		octEdge.contents.prop.value.string = "LEFT";
	    	octPosition.contents.prop.value.real 
		  = (double)(IOListPin->YPosition - ymin) 
		    / (double)(ymax-ymin);
	    }
	    else if (IOListPin->XPosition == RIGHT) {
		octEdge.contents.prop.value.string = "RIGHT";
	    	octPosition.contents.prop.value.real 
		  = (double)(IOListPin->YPosition - ymin) 
		    / (double)(ymax-ymin);
	    }
	    else if (IOListPin->XPosition == xmin) {
		octEdge.contents.prop.value.string = "LEFT";
	    	octPosition.contents.prop.value.real 
		  = (double)(IOListPin->YPosition - ymin) 
		    / (double)(ymax-ymin);
	    }
	    else if (IOListPin->XPosition == xmax) {
		octEdge.contents.prop.value.string = "RIGHT";
	    	octPosition.contents.prop.value.real 
		  = (double)(IOListPin->YPosition - ymin) 
		    / (double)(ymax-ymin);
	    }
	    else if (IOListPin->YPosition == ymin) {
		octEdge.contents.prop.value.string = "BOTTOM";
	    	octPosition.contents.prop.value.real 
		  = (double)(IOListPin->XPosition - xmin)
		    / (double)(xmax-xmin);
	    }
	    else if (IOListPin->YPosition == ymax) {
		octEdge.contents.prop.value.string = "TOP";
	    	octPosition.contents.prop.value.real 
		  = (double)(IOListPin->XPosition - xmin)
		    / (double)(xmax-xmin);
	    }
	    OCTCHKOK(octCreate(&octTerm,&octEdge), "cannot create a property");
	    if (octCreate(&octTerm,&octPosition)<OCT_OK)
	      OCTERR("cannot create property");
	}

	if (IOListPin->Current != NOCURRENT) {
	    octCurrent.contents.prop.value.real = IOListPin->Current;
	    OCTCHKOK(octCreate(&octTerm,&octCurrent),
		     "cannot create a property");
	}

	/* get the position of terminal */
	oBox.type = OCT_BOX;
	if ((IOListPin->XPosition != NOPOSITION) && (ModType != PARENT)) {

            if (IOListPin->XPosition == BOTTOM) {
                oBox.contents.box.lowerLeft.x
		  = oBox.contents.box.upperRight.x
		    = octOutline.contents.box.lowerLeft.x
		      + IOListPin->YPosition * OCT_PER_MICRON;
                oBox.contents.box.upperRight.y
		  = oBox.contents.box.lowerLeft.y
		    = octOutline.contents.box.lowerLeft.y;
            }
            else if (IOListPin->XPosition == RIGHT) {
                oBox.contents.box.lowerLeft.x
		  = oBox.contents.box.upperRight.x
		    = octOutline.contents.box.upperRight.x;
                oBox.contents.box.upperRight.y
		  = oBox.contents.box.lowerLeft.y
		    = octOutline.contents.box.lowerLeft.y
		      + IOListPin->YPosition * OCT_PER_MICRON;
            }
            else if (IOListPin->XPosition == TOP) {
                oBox.contents.box.lowerLeft.x
		  = oBox.contents.box.upperRight.x
		    = octOutline.contents.box.lowerLeft.x
		      + IOListPin->YPosition * OCT_PER_MICRON;
                oBox.contents.box.upperRight.y
		  = oBox.contents.box.lowerLeft.y
		    = octOutline.contents.box.upperRight.y;
            }
            else if (IOListPin->XPosition == LEFT) {
                oBox.contents.box.lowerLeft.x
		  = oBox.contents.box.upperRight.x
		    = octOutline.contents.box.lowerLeft.x;
                oBox.contents.box.upperRight.y
		  = oBox.contents.box.lowerLeft.y
		    = octOutline.contents.box.lowerLeft.y
		      + IOListPin->YPosition * OCT_PER_MICRON;
            }
            else {
                oBox.contents.box.lowerLeft.x
		  = oBox.contents.box.upperRight.x
		    = IOListPin->XPosition * OCT_PER_MICRON;
                oBox.contents.box.upperRight.y
		  = oBox.contents.box.lowerLeft.y
		    = IOListPin->YPosition * OCT_PER_MICRON;
            }

	}

	/* add displacement to the center value  and get a box shape */
	if ((IOListPin->Width != NOWIDTH) && (ModType != PARENT)) {
	    if (oBox.contents.box.lowerLeft.x ==
		octOutline.contents.box.lowerLeft.x) {
		if (oBox.contents.box.lowerLeft.y ==
		    octOutline.contents.box.lowerLeft.y) {
		    oBox.contents.box.lowerLeft.x
		    -= IOListPin->Width / 2.0 * OCT_PER_MICRON;
		    oBox.contents.box.upperRight.x
		    += IOListPin->Width / 2.0 * OCT_PER_MICRON;
		    oBox.contents.box.upperRight.y
		    += IOListPin->Width * OCT_PER_MICRON;
		} else {
		    if (oBox.contents.box.lowerLeft.y ==
			octOutline.contents.box.upperRight.y) {
			oBox.contents.box.lowerLeft.x
                        -= IOListPin->Width / 2.0 * OCT_PER_MICRON;
			oBox.contents.box.lowerLeft.y
                        -= IOListPin->Width * OCT_PER_MICRON;
			oBox.contents.box.upperRight.x
                        += IOListPin->Width / 2.0 * OCT_PER_MICRON;
		    } else {
			oBox.contents.box.lowerLeft.y 
			-= IOListPin->Width / 2.0 * OCT_PER_MICRON; 
			oBox.contents.box.upperRight.x 
			+= IOListPin->Width * OCT_PER_MICRON;
			oBox.contents.box.upperRight.y 
			+= IOListPin->Width / 2.0 * OCT_PER_MICRON;
		    }
		}
	    }
	    if (oBox.contents.box.lowerLeft.x ==
		octOutline.contents.box.upperRight.x) {
		if (oBox.contents.box.lowerLeft.y ==
		    octOutline.contents.box.lowerLeft.y) {
		    oBox.contents.box.lowerLeft.x
		    -= IOListPin->Width / 2.0 * OCT_PER_MICRON;
		    oBox.contents.box.upperRight.x
		    += IOListPin->Width / 2.0 * OCT_PER_MICRON;
		    oBox.contents.box.upperRight.y
		    += IOListPin->Width * OCT_PER_MICRON;
		} else {
		    if (oBox.contents.box.lowerLeft.y ==
			octOutline.contents.box.upperRight.y) {
			oBox.contents.box.lowerLeft.x
                        -= IOListPin->Width / 2.0 * OCT_PER_MICRON;
			oBox.contents.box.lowerLeft.y
                        -= IOListPin->Width * OCT_PER_MICRON;
			oBox.contents.box.upperRight.x
                        += IOListPin->Width / 2.0 * OCT_PER_MICRON;
		    } else {
			oBox.contents.box.lowerLeft.x 
			-= IOListPin->Width * OCT_PER_MICRON;
			oBox.contents.box.lowerLeft.y 
			-= IOListPin->Width / 2.0 * OCT_PER_MICRON; 
			oBox.contents.box.upperRight.y 
			+= IOListPin->Width / 2.0 * OCT_PER_MICRON;
		    }
		}
	    }
	    if (oBox.contents.box.upperRight.y ==
		octOutline.contents.box.lowerLeft.y) {
		oBox.contents.box.lowerLeft.x 
		-= IOListPin->Width / 2.0 * OCT_PER_MICRON;
		oBox.contents.box.upperRight.x 
		+= IOListPin->Width / 2.0 * OCT_PER_MICRON;
		oBox.contents.box.upperRight.y 
		+= IOListPin->Width * OCT_PER_MICRON;
	    }
	    if (oBox.contents.box.lowerLeft.y ==
		octOutline.contents.box.upperRight.y) {
		oBox.contents.box.lowerLeft.x 
		-= IOListPin->Width / 2.0 * OCT_PER_MICRON;
		oBox.contents.box.lowerLeft.y 
		-= IOListPin->Width * OCT_PER_MICRON; 
		oBox.contents.box.upperRight.x 
		+= IOListPin->Width / 2.0 * OCT_PER_MICRON;
	    }


	    if (IOListPin->Layer != NOLAYER) {
		/* create geometry of the terminal */
		/*---------------------------------*/
		switch (IOListPin->Layer) {
		case PDIFF:
		    if (octCreate(&pdif_layer,&oBox)<OCT_OK)
		      OCTERR("cannot create box");
		    break;
		case NDIFF:
		    if (octCreate(&ndif_layer,&oBox)<OCT_OK)
		      OCTERR("cannot create box");
		    break;
		case POLY:
		    if (octCreate(&poly_layer,&oBox)<OCT_OK)
		      OCTERR("cannot create box");
		    break;
		case METAL1:
		    if (octCreate(&met1_layer,&oBox)<OCT_OK)
		      OCTERR("cannot create box");
		    break;
		case METAL2:
		    if (octCreate(&met2_layer,&oBox)<OCT_OK)
		      OCTERR("cannot create box");
		    break;
		default:
		    break;
		}
		if (octAttach(&octTerm,&oBox)<OCT_OK)
		  OCTERR("cannot attach a box to a terminal");
	    }
	}

	(void) strcpy(previous_name,IOListPin->SignalName);
	IOListPin = IOListPin->Link;
    }


    /********************/
    /* process each net */
    /********************/
    if (NetworkListHead != NetworkListNull) {
	CurrentNetworkEntry = NetworkListHead;

	while (CurrentNetworkEntry != NetworkListNull) {
	    /* create an instance first and attach it to a bag */
	    /*-------------------------------------------------*/
	    octInst.type = OCT_INSTANCE;
	    octInst.contents.instance.name = CurrentNetworkEntry->InstanceName;
	    octInst.contents.instance.master = CurrentNetworkEntry->ModuleName;
	    octInst.contents.instance.view = "physical";
	    octInst.contents.instance.facet = "contents";
	    octInst.contents.instance.version = OCT_CURRENT_VERSION;
	    octInst.contents.instance.transform.transformType 
	      = OCT_NO_TRANSFORM;
	    octInst.contents.instance.transform.translation.x = 0; 
	    octInst.contents.instance.transform.translation.y = 0; 

	    if (octCreate(&octCell,&octInst)<OCT_OK) {
		/* child modules should be physical (not PARENT)
		   otherwise terminal retrieval does not work for its naming convention above */
		/*		octInst.contents.instance.view = "symbolic";
				if (octCreate(&octCell,&octInst)<OCT_OK) { */
		OCTERR("cannot create instance");
		/*		} */
	    }
	    OCTCHKOK(octAttach(&octInstances,&octInst),
		     "cannot attach an instance to a bag");

	    term_No = 1;
	    CurrentSignal = CurrentNetworkEntry->SignalListHead;
	    while(CurrentSignal != SignalListNull) {

		/* get or create a net and create properties */
		/*-------------------------------------------*/
		octNet.type = OCT_NET;
		octNet.contents.net.width = 1;
		octNet.contents.net.name = CurrentSignal->SignalName;

		if (octGetOrCreate(&octCell,&octNet)<OCT_OK)
		  OCTERR("cannot getOrCreate a net");
		(void) sprintf(buf, "%d_1", term_No);
		octTerm.contents.term.name = buf;
		if (octGetByName(&octInst,&octTerm)<OCT_OK)
		  OCTERR("cannot get by name");
		if (octAttach(&octNet,&octTerm)<OCT_OK)
		  OCTERR("cannot attach terminal to a net");

		CurrentSignal = CurrentSignal->Link;
		++term_No;
	    }
	    CurrentNetworkEntry = CurrentNetworkEntry->Link;
	}
    }


    /* add properties to nets */
    /*------------------------*/
    octLength.type = OCT_PROP;
    octLength.contents.prop.name = "NET_MAX_LENGTH";
    octLength.contents.prop.type = OCT_INTEGER;
    octNet.type = OCT_NET;
    if (CriticalNetListHead != CriticalNetListNull) {
	CurrentCriticalNetEntry = CriticalNetListHead;
	while (CurrentCriticalNetEntry != CriticalNetListNull) {
	    octNet.contents.net.name = CurrentCriticalNetEntry->SignalName;
	    OCTCHKOK(octGetByName(&octCell,&octNet),"cannot get net");
	    octLength.contents.prop.value.integer = 
	      CurrentCriticalNetEntry->MaximumLength;
	    OCTCHKOK(octCreate(&octNet,&octLength),
		     "cannot create property");
	    CurrentCriticalNetEntry = CurrentCriticalNetEntry->Link;
	}
    }
    

    /* flush facet and run vulcan to create an interface facet */
    if (octFlushFacet(&octCell)<OCT_OK)
      OCTERR("cannot flush facet");

    octInt.type = OCT_FACET;
    octInt.contents.facet.cell = Name;
    octInt.contents.facet.view = octCell.contents.facet.view;
    octInt.contents.facet.facet   = "interface";
    octInt.contents.facet.mode    = "w";
    octInt.contents.facet.version = OCT_CURRENT_VERSION;
    if (octOpenFacet(&octInt)<OCT_OK)
      OCTERR("cannot open facet");
    (void) vulcan(&octCell,&octInt);


    /* add terminal properties and other properties to the interface facet */
    /*---------------------------------------------------------------------*/
    place_layer.type = OCT_LAYER;
    place_layer.contents.layer.name = "PLACE";
    if (octGetOrCreate(&octInt,&place_layer)<OCT_OK)
      OCTERR("cannot getOrCreate layer");

    if (ModType == PARENT) {
	octFloorplan.type = OCT_BAG;
	octFloorplan.contents.bag.name = "FLOORPLAN";
	OCTCHKOK(octCreate(&octInt, &octFloorplan),"cannot create a bag");

	if (octOutline.contents.box.lowerLeft.x != MAXINT) {
	    if (octCreate(&place_layer,&octOutline)<OCT_OK)
	      OCTERR("cannot create box");
	    if (octAttach(&octFloorplan,&octOutline)<OCT_OK)
	      OCTERR("cannot attach box");
	}
    }


    octMask = OCT_TERM_MASK;
    if (octInitGenContents(&octInt,octMask,&genIntTerm)<OCT_OK)
      OCTERR("cannot generate contents");

    while ((octGenerate(&genIntTerm, &octIntTerm)) == OCT_OK) {

	octTerm.contents.term.name = octIntTerm.contents.term.name;
	if (octGetByName(&octCell,&octTerm)<OCT_OK)
	  OCTERR("cannot get by name");
	octMask = OCT_PROP_MASK;
	if (octInitGenContents(&octTerm,octMask,&genProp)<OCT_OK)
	  OCTERR("cannot generate contents");
	while ((octGenerate(&genProp, &octIntProp)) == OCT_OK) {

	    if (octCreate(&octIntTerm, &octIntProp)<OCT_OK)
	      OCTERR("cannot create property");
	}
    }

    if (octCloseFacet(&octCell)<OCT_OK)
      OCTERR("cannot close facet");
    if (octCloseFacet(&octInt)<OCT_OK)
      OCTERR("cannot close facet");


}
