.TH VEM OCTTOOLS "1 August 1990"
.SH NAME
VEM - Interactive Graphics for Oct Designs (version 8-1)
.SH SYNOPSIS
vem [-F cell[:view[:facet]]] [-G WxH+X+Y] [-R [host,]path] [-display host:display] [name=value ...]
.SH DESCRIPTION
VEM is an interactive graphics shell/editor for IC designs represented in the
.B Oct
CAD/VLSI data representation system.
The primary purpose of VEM
is to provide a means of looking at the graphics representation of
.B Oct
views and invoking various CAD tools on these views.
VEM also provides standard graphics editing capabilities
for physical (mask-level),
symbolic, and schematic designs.  Some understanding of the
.B Oct
data manager is required to use VEM.  See the Octtools Reference Manual
for more information about
.B Oct.
.PP
VEM is a multi-window
application designed to run on bitmap based graphics workstations.
VEM will run on any such system that has a reasonably complete implementation
of the
.I X
Window System version 11 release 4 or later.
Currently, there are 
.I X 
implementations for a wide range of desktop workstations
including all DEC and Sun Microsystems computers
as well as some HP, Apollo, and Masscomp computers.
.PP
VEM may be started by simply typing
"vem".
VEM looks at the value of the DISPLAY environment variable to determine
what host and display to use for output.  The user may override
the display choice by providing the host and display on the command
line.  
VEM may be run in the background without affecting the program operation.
.PP
The 
.B -F, 
.B -G,
and 
.B -R
command line options allow a user to specify a certain
startup window configuration for VEM.
These three options are considered triplets that specify the initial cell, 
position and size, and remote application respectively for a window.
There is no limit to the number of triplets that may be specified.
The 
.B -F
flag marks the start of each new triplet.  The corresponding
.B -G 
and 
.B -R 
flags after the 
.B -F 
flag are optional.  If the 
.B -G 
flag is omitted,
VEM will not specify a location for the window and most window
managers will interactively prompt for the window location.
If the 
.B -R 
flag
is omitted,  no remote application will be started in the window.
The 
.B -F 
flag can be omitted from the first triplet.  In this case, the
.B -G 
and 
.B -R 
flags apply to the console window.
For example,  the following command starts VEM with its console window
at (100,100) with a size of 500 by 200,  and one window looking at
the cell "test:physical" at (100,300) with a size of 500 by 500 running
the "rpc-symhelp" remote application:
.RS
vem -G 500x200+100+100 -F test:physical -G 500x500+100+300 -R $OCTTOOLS/bin/rpc/rpc-symhelp
.RE
.PP
VEM is a highly customizable editor.  Nearly all of the colors, font
styles and fill patterns VEM uses can be changed by the user.
Normally, these parameters are read from the user's X resources
(usually kept in the files ~/.Xdefaults or ~/.Xresources).  However,
one can set certain parameters on the command line using the = (equal)
command line option.  For example, to start VEM without opening a
browser window one could issue the following command:
.RS
vem initialBrowser=0
.RE
A list of all configurable parameters can be found in the document
"Customizing VEM".
.PP
If no location for the console window has been provided on the command line,
VEM will not specify a location for the console window and most window
managers will interactively prompt for a console window location.
This window echoes user input and outputs various help and status messages.  
After starting,  the
console will display a prompt and wait for input.
.PP
After the console window appears, VEM opens a
.I browser window
that lists
.B Oct
cells in the current working directory.
The directory name is show in the title bar at the top of the window.
The body of the window lists all of the cells in the directory along
with any subdirectories that may be present.
Clicking once on a cell or directory selects it.  
Double clicking (clicking quickly twice) will open the cell or
directory.
Opening an
.B Oct
cell will create a new VEM graphics window that displays the contents
of the cell.
Opening a new directory will create a new browser looking at the
contents of the directory.
At the bottom of the window are three control buttons.  Clicking on
.I Dismiss
will close the browser window.
The browser does not automatically track changes to the directory it
displays.  Clicking on 
.I Refresh
will cause the browser to reexamine the directory and update the cells
and directories displayed in the main body.
Clicking on
.I Help
will display a message describing the features of the browser.
.PP
Once the console window has been opened,  new windows can be
created using the browser or the
.I open-window
command (see the command reference below).  
Each window has exactly
one associated cell.  
Mouse action with the cursor positioned inside
a window cause operations to occur to the window's associated cell.
Any number of windows can be created with the same or different
associated cells.  
More than one window may have the same associated
cell.  
In this case,  all of the windows are attached to the
.I same
cell.  
Thus,  a change to one of the windows may cause updates
to other windows which look at the same cell.
.PP
VEM assumes the presence of a three-button mouse.  
The left button is used for entry of graphics information.  
The middle button is used for the primary menu of commands.  
The right button is used to modify graphics information entered using
the left button.
.PP
Commands to VEM are specified in post-fix form.  
The user
builds an argument list first and then selects a command.  
Commands can be selected in four ways:  pop-up menus,  single
keystrokes,  using the last command button, or by typing in the command name.
Pressing and releasing the middle button in a graphics window causes
a VEM menu to appear.  
The user can use the mouse to riffle through
the options until the desired choice is highlighted.  
Pressing and releasing the mouse button activates the selected command.  
Pressing and releasing the mouse outside the menu cancels the selection.  
Normally,  pressing and releasing the middle button
causes a VEM menu to appear.  
However,  expert users can define a parameter
which tells VEM to execute the last command specified by menu if the time
between the press and release of the middle button is less
than the parameter value (see 
.I menu.delay
in the customization document). 
A number of common commands
can be selected via a single keystroke.  
Key bindings for various commands are 
shown next to the corresponding entry in the VEM menu,
are listed in the command reference below,
and can be queried interactively using the
.I bindings
command.
Finally,  typing a colon (:) allows
the user to type in the command name (or a user defined alias).  
The standard line editing keys can be used while typing the command name.
The interface supports automatic command completion.  
Typing a
tab will complete the command if it is unique or offer a list of
alternatives if it is not unique.  
The command is selected by
typing a carriage return.
.PP
There are five types of input to VEM:  points, boxes, lines, text,
and objects.  
Points are entered by pressing and releasing the left
button of the mouse.  
Boxes are entered by pressing, dragging, and
releasing the left button.  
Lines are entered by pressing, dragging
and releasing the left button over a previously created point or
line.  
Text is entered by typing the text enclosed in double 
quotes.
If entering a filename,  typing a TAB character
will cause VEM to try to complete the name if it is unique or offer a list of
alternatives if it is not unique.
Objects are entered using the 
.I select-objects 
and 
.I unselect-objects 
commands.
The last item on an argument list can be deleted using the standard
character for delete.  The last group of items can be deleted using
the word erase character control-W.  The entire argument list can be
deleted using the standard kill-line character (usually control-X or 
control-U).
.PP
Once entered, graphics arguments (points, boxes, and lines) can be 
modified in various ways.  
For all arguments,  a point can be moved
by moving the cursor over the point
and pressing, dragging, and releasing the right mouse button.
New points can be added to a group of lines by moving over a point
in the segment,
depressing, dragging, and releasing the left mouse button.  
This will
insert a new point after the point.
It is also possible to interactively move, rotate, and mirror objects 
arguments (selected items).  
See the description of the
.I transform
command in the command reference below.
.PP
This version of VEM supports three basic editing styles:  
physical, symbolic, and schematic.
Physical editing involves the entry and editing of basic geometry
and the creation of interface terminals.  
This style is used to
build leaf cells.  
Symbolic editing involves the placement of
instances of leaf cells and the interconnection of these instances.  
Most 
.B Oct
CAD tools are designed to operate on symbolic views.
Schematic editing is an extension of symbolic where the primitive
cells are schematic symbols and wire width is insignificant.
Schematic cells can be used as input to switch level and electrical
simulators.
.PP
When VEM opens a new cell,  a dialog will appear asking for
three property values:  TECHNOLOGY, VIEWTYPE, and EDITSTYLE.
The TECHNOLOGY and VIEWTYPE properties determine the location
of the technology facet (see tap(3)).  
Layer display and design
rule information is read from this facet.  
The EDITSTYLE property is used by VEM to determine the set of
commands available for editing the cell.  
Currently,  the
legal editing styles are PHYSICAL, SYMBOLIC, or SCHEMATIC.
When the dialog appears,  VEM fills in these fields with default
values.  
In most cases,  these defaults are acceptable and
the user can click in the area labeled OK to continue.
It is also possible for the user to specify default values
for these fields. See the customization document for details.
.SH USING DIALOG BOXES
.PP
Some commands require information that cannot be expressed easily
using post-fix notation.  Examples include destructive commands
that require an explicit confirmation and commands that require complex
non-graphic information.  VEM uses
.I dialog boxes
to handle these situations.  
Dialog boxes are windows that resemble business forms.
These windows contain labeled fields for entering text, changing
numerical values, and selecting options.
This section describes how to use dialog boxes.
.PP
All dialog boxes in VEM have the same form.  
At the top of all dialogs is a one line title indicating the purpose
for the dialog.
The middle of the dialog (known as the body) contains fields for
displaying and editing information of various kinds.
At the bottom of the dialog are a number of
.I control buttons.
Each control button represents a command.
The arguments to the command are the values of the fields displayed in
the body.
Thus, operating a dialog consists of editing or changing fields in the
body and then selecting a command by activating a control button.
Six kinds of fields may appear in the body of a VEM dialog: editable
text, non-editable text, enumerated value, numerical value, exclusive
lists, and non-exclusive lists.
A description of each field type is given in the paragraphs that follow.
.PP
Editable text fields are used to enter and edit text.  
Visually, an edit text field consists of a box containing a caret
cursor, an optional scrollbar, and a label to the left of the box
indicating the purpose for the field.
Only one editable text field is active in any one dialog.
The active editable text field has a dark border.
Typing text with the mouse positioned anywhere in the dialog inserts
text into the active editable text field.
Most of the basic emacs editing commands can be used to modify the
text in the field:
.RS
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82 83
.nr 80 0
.nr 38 \wKey
.if \n(80<\n(38 .nr 80 \n(38
.nr 38 \wCtrl-A
.if \n(80<\n(38 .nr 80 \n(38
.nr 38 \wCtrl-B
.if \n(80<\n(38 .nr 80 \n(38
.nr 38 \wCtrl-N
.if \n(80<\n(38 .nr 80 \n(38
.nr 38 \wCtrl-V
.if \n(80<\n(38 .nr 80 \n(38
.nr 38 \wCtrl-D
.if \n(80<\n(38 .nr 80 \n(38
.nr 38 \wCtrl-K
.if \n(80<\n(38 .nr 80 \n(38
.nr 38 \wCtrl-Y
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 81 0
.nr 38 \wAction
.if \n(81<\n(38 .nr 81 \n(38
.nr 38 \wBeginning of Line
.if \n(81<\n(38 .nr 81 \n(38
.nr 38 \wBackward character
.if \n(81<\n(38 .nr 81 \n(38
.nr 38 \wNext line
.if \n(81<\n(38 .nr 81 \n(38
.nr 38 \wNext page
.if \n(81<\n(38 .nr 81 \n(38
.nr 38 \wDelete next character
.if \n(81<\n(38 .nr 81 \n(38
.nr 38 \wDelete to end of line
.if \n(81<\n(38 .nr 81 \n(38
.nr 38 \wYank deleted text
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 82 0
.nr 38 \wKey
.if \n(82<\n(38 .nr 82 \n(38
.nr 38 \wCtrl-E
.if \n(82<\n(38 .nr 82 \n(38
.nr 38 \wCtrl-F
.if \n(82<\n(38 .nr 82 \n(38
.nr 38 \wCtrl-P
.if \n(82<\n(38 .nr 82 \n(38
.nr 38 \wMeta-V
.if \n(82<\n(38 .nr 82 \n(38
.nr 38 \wMeta-D
.if \n(82<\n(38 .nr 82 \n(38
.nr 38 \wCtrl-S
.if \n(82<\n(38 .nr 82 \n(38
.nr 38 \wMeta-I
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 83 0
.nr 38 \wAction
.if \n(83<\n(38 .nr 83 \n(38
.nr 38 \wEnd of Line
.if \n(83<\n(38 .nr 83 \n(38
.nr 38 \wForward character
.if \n(83<\n(38 .nr 83 \n(38
.nr 38 \wPrevious line
.if \n(83<\n(38 .nr 83 \n(38
.nr 38 \wPrevious page
.if \n(83<\n(38 .nr 83 \n(38
.nr 38 \wDelete next word
.if \n(83<\n(38 .nr 83 \n(38
.nr 38 \wSearch forward
.if \n(83<\n(38 .nr 83 \n(38
.nr 38 \wInclude file
.if \n(83<\n(38 .nr 83 \n(38
.83
.rm 83
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr 43 \n(82+(3*\n(38)
.nr 83 +\n(43
.nr TW \n(83
.if t .if \n(TW>\n(.li .tm Table at line 339 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0
.vs \n(36u
.ta \n(80u \n(81u \n(82u \n(83u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'Key\h'|\n(41u'Action\h'|\n(42u'Key\h'|\n(43u'Action
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0
.vs \n(36u
.ta \n(80u \n(81u \n(82u \n(83u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'Ctrl-A\h'|\n(41u'Beginning of Line\h'|\n(42u'Ctrl-E\h'|\n(43u'End of Line
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0
.vs \n(36u
.ta \n(80u \n(81u \n(82u \n(83u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'Ctrl-B\h'|\n(41u'Backward character\h'|\n(42u'Ctrl-F\h'|\n(43u'Forward character
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0
.vs \n(36u
.ta \n(80u \n(81u \n(82u \n(83u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'Ctrl-N\h'|\n(41u'Next line\h'|\n(42u'Ctrl-P\h'|\n(43u'Previous line
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0
.vs \n(36u
.ta \n(80u \n(81u \n(82u \n(83u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'Ctrl-V\h'|\n(41u'Next page\h'|\n(42u'Meta-V\h'|\n(43u'Previous page
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0
.vs \n(36u
.ta \n(80u \n(81u \n(82u \n(83u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'Ctrl-D\h'|\n(41u'Delete next character\h'|\n(42u'Meta-D\h'|\n(43u'Delete next word
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0
.vs \n(36u
.ta \n(80u \n(81u \n(82u \n(83u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'Ctrl-K\h'|\n(41u'Delete to end of line\h'|\n(42u'Ctrl-S\h'|\n(43u'Search forward
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0
.vs \n(36u
.ta \n(80u \n(81u \n(82u \n(83u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'Ctrl-Y\h'|\n(41u'Yank deleted text\h'|\n(42u'Meta-I\h'|\n(43u'Include file
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0
.vs \n(36u
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-20


.RE
The insert position in the field may also be changed by pressing the
left mouse button when the mouse cursor is over the desired position.
Any editable text field can be made active by clicking the left mouse
button inside the editable area.
Alternatively, one can use the <Tab> key to make the next text field
active and <Meta-Tab> to make the previous field active.
Editable text fields that display large amounts of text have a
scrollbar to the left of the text area.  
Pressing the left and right
mouse buttons when the mouse cursor is in a scrollbar will scroll the
text down and up respectively in proportion to the distance between
the mouse cursor and the top of the scrollbar.
As an example, pressing the left mouse button near the bottom of the
scrollbar will scroll down the text almost one screen.
Pressing and releasing the middle mouse button scrolls the text to
a relative position based on how far the mouse cursor is from the top
of the scrollbar.
Holding down the middle mouse button will interactively scroll through
the text.
.PP
Non-editable text fields are used to display text messages.
They consist of a box containing text and an optional scrollbar.
The scrollbar operates just like those used in editable text fields.
.PP
Enumerated value fields are used to specify one value out of a small
list of values.
They consist of a value displayed
inside a box and a descriptive label to the left of the value.
The border of the value highlights as the mouse cursor moves over it.
Depressing and holding the left mouse button inside the value box
causes a menu to appear that displays all possible values.
The choices will highlight as the mouse cursor moves over them.
To select a new value, release the mouse button when the desired
choice is highlighted.  The new value will appear in the value box.
One can leave the value unchanged by releasing the mouse button
outside the menu boundary.
.PP
The numerical value field is used to specify a magnitude between a
predetermined minimum and maximum.
Visually, it consists of a box containing a numerical value, a
horizontal scrollbar to the right of the box for changing the value,
and a label to the left describing the value.
The magnitude of the value is changed by operating the scrollbar.
Pressing the left and right buttons in the scrollbar decrement and
increment the value by one unit.
Pressing the middle button changes the value based on the distance
between the mouse cursor and the left edge of the scroll bar.
The middle button may be pressed and held to interactively modify the
value. 
Most people use the middle button to set the value roughly then use
the left and right buttons to make the value precise.
.PP
Exclusive lists are used to choose one possible value out of a
(possibly quite large) list of values.
These values are displayed in a box with a scrollbar on the left edge
of the box.
Each value consists of a button box on the left and a descriptive label
to the right.
As the mouse moves over a button box, it will highlight to indicate it
can be activated.
The button box of the selected item will appear dark while all others
will remain light.
If there are too many values to display in the box at one time, the
scrollbar can be used to scroll through the possible values.
The scrollbar operates in the same way as described for editable text
fields.
.PP
Non-exclusive lists are used to choose zero or more possible values
from a (possibly quite large) list of values.
A non-exclusive list resembles an exclusive list both in appearance
and operation.
However, unlike an exclusive list, one can choose any number of items
in a non-exclusive list.
Visually, the two lists are distinguished by the appearance of the
button boxes.
Exclusive button boxes resemble radio buttons.  Non-exclusive button
boxes resemble check marks.
Pressing the left button in a non-exclusive button box causes the
value to toggle (i.e., if it was selected it becomes unselected, if it
was unselected it becomes selected).
.PP
Control buttons cause the dialog to carry out some operation.
They consist of a text label surrounded by a box.
Control buttons are activated in one of two ways:
pressing and releasing the left mouse button when the mouse cursor is
positioned inside the button boundary, and through keystrokes.
Not all control buttons can be activated using keystrokes.
Those that can be activated in this fashion display the key in
parentheses under the button label.
Although there are exceptions, most dialogs support the following
keyboard accelerators:
.RS
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81
.nr 80 0
.nr 38 \w\f3Keys\fP
.if \n(80<\n(38 .nr 80 \n(38
.nr 38 \w<Ret>,<Meta-Ret>,<F1>
.if \n(80<\n(38 .nr 80 \n(38
.nr 38 \w<Del>,<Meta-Del>,<F2>
.if \n(80<\n(38 .nr 80 \n(38
.nr 38 \w<Help>,<F3>
.if \n(80<\n(38 .nr 80 \n(38
.nr 38 \w<F4>
.if \n(80<\n(38 .nr 80 \n(38
.nr 38 \w<F5>
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 81 0
.nr 38 \w\f3Action\fP
.if \n(81<\n(38 .nr 81 \n(38
.nr 38 \wOk
.if \n(81<\n(38 .nr 81 \n(38
.nr 38 \wCancel
.if \n(81<\n(38 .nr 81 \n(38
.nr 38 \wHelp
.if \n(81<\n(38 .nr 81 \n(38
.nr 38 \wAll
.if \n(81<\n(38 .nr 81 \n(38
.nr 38 \wClear
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr TW \n(81
.if t .if \n(TW>\n(.li .tm Table at line 450 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3Keys\f\n(31\h'|\n(41u'\f3Action\f\n(31
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\v'-1p'\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0\v'2p'\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0\v'-1p'
.vs \n(36u
.ta \n(80u \n(81u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'<Ret>,<Meta-Ret>,<F1>\h'|\n(41u'Ok
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0
.vs \n(36u
.ta \n(80u \n(81u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'<Del>,<Meta-Del>,<F2>\h'|\n(41u'Cancel
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0
.vs \n(36u
.ta \n(80u \n(81u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'<Help>,<F3>\h'|\n(41u'Help
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0
.vs \n(36u
.ta \n(80u \n(81u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'<F4>\h'|\n(41u'All
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0
.vs \n(36u
.ta \n(80u \n(81u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'<F5>\h'|\n(41u'Clear
.nr 36 \n(.v
.vs \n(.vu-\n(.sp
\h'|0'\s\n(33\l'|\n(TWu\(ul'\s0
.vs \n(36u
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-15
.RE
.PP
Dialogs may be both
.I moded
and
.I unmoded.
Moded dialogs are those requiring a response before processing can
proceed.
VEM uses these kinds of dialogs to ask for confirmation before
proceeding.
On the other hand, unmoded dialogs remain active until explicitly
dismissed by the user.
Other commands may be invoked freely while unmoded dialogs are
visible.
Most non-confirmation dialogs in VEM are unmoded.
.PP
.SH GENERAL COMMANDS
Below is a reference for all VEM commands.  
The first section outlines
general commands available for editing all types of cells.  
The next section describes the general selection mechanism.  
The third section describes property and bag editing features.
The next three sections describe editing commands for physical, symbolic,
and schematic cells respectively.
The summary includes the
name of the command as it appears on a menu (this is also its default
command name for command type-in),  its default keyboard
binding,  and the syntax of the argument list passed to it.
The phrase <no-name> implies it has no default menu or command name binding,
the symbol <*> implies the command has no default key binding.
In general,  the commands
used most often have key and menu bindings.  
Less often used commands may have
only command name bindings.
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3<no-name>\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3<del>,^H\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3Any Argument List\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 488 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3<no-name>\f\n(31\h'|\n(41u'\f3<del>,^H\f\n(31\h'|\n(42u'\f3Any Argument List\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command deletes the last item of the last argument on the
argument list.  Thus,  if the last argument is 10 boxes,  it
will delete the last box entered and the argument list will be
modified to contain 9 boxes.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3<no-name>\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3^W\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3Any Argument List\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 500 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3<no-name>\f\n(31\h'|\n(41u'\f3^W\f\n(31\h'|\n(42u'\f3Any Argument List\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command is similar to the one above, but deletes the entire last argument
on the argument list.   Thus,  if the last argument is 5 lines,
it will delete all 5 lines and leave the remaining arguments
unchanged.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3<no-name>\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3^X or ^U\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3Any Argument List\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 512 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3<no-name>\f\n(31\h'|\n(41u'\f3^X or ^U\f\n(31\h'|\n(42u'\f3Any Argument List\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command erases the entire argument list allowing the user
to start over.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3alt-buffer\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3<*>\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[pnt] {"cell:view:{facet:version}"}\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 522 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3alt-buffer\f\n(31\h'|\n(41u'\f3<*>\f\n(31\h'|\n(42u'\f3[pnt] {"cell:view:{facet:version}"}\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command makes the facet associated with the window containing
the cursor the alternate facet for the window under the point in
the argument list.  The origin of the alternate facet will be aligned
with this point.  If the last argument is text,  it will be interpreted
as the alternate facet instead of the one under the spot.
.PP
In later releases,  this capability will allow VEM to edit masters
of instances in context.  Currently,  it is provided for use by remote
applications.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3bindings\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3b\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3Saves Arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 539 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3bindings\f\n(31\h'|\n(41u'\f3b\f\n(31\h'|\n(42u'\f3Saves Arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command asks the user for a command and displays all of its
current key, menu, and alias bindings.  The command will display
a prompt (vem bindings> ) and the user can specify a command using
any of the four means of normally specifying commands (via menu,
single keystroke, type-in, or last command).
The command also outputs a one line description of the command
for help purposes.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3browser\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3B\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3"directory"\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 554 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3browser\f\n(31\h'|\n(41u'\f3B\f\n(31\h'|\n(42u'\f3"directory"\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command creates a new browser window in the specified directory.
If no directory is specified, VEM displays a browser looking at the
current working directory.
A browser window displays a list of Oct cells and directories.
Clicking the left mouse button on a cell or directory selects it.
Double clicking (clicking quickly twice) will open a cell or
directory.
Opening a directory will create a new browser window that displays the
contents of that directory.
Opening an
.B Oct
cell will create a new graphics window that displays the contents of
the cell.
At the bottom of the window are three control buttons.
Clicking on
.I Dismiss
will close the browser window.
Clicking on the
.I Refresh
button will cause VEM to examine the contents of the directory and
update the list of cells and directories displayed in the browser.
Finally, clicking on the
.I Help
button will display a message briefly describing how to operate the
browser window.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3close-window\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3^D\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No Arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 587 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3close-window\f\n(31\h'|\n(41u'\f3^D\f\n(31\h'|\n(42u'\f3No Arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I close-window
command closes the window the cursor was in when the command
was invoked.  This DOES NOT flush the contents of the window to disk.
Even after all windows looking at a facet are closed,  the contents
are not saved on disk.  This must be done using the 
.I save-window
or 
.I save-all
commands.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3deep-reread\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3<*>\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 605 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3deep-reread\f\n(31\h'|\n(41u'\f3<*>\f\n(31\h'|\n(42u'\f3[objs]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I deep-reread
command is a specialized form of the re-read command.  With no arguments,
it re-reads a facet and all of master facets of its subcells (instances).
Both the contents and interface facets of the instances are re-read.
If a set of objects is specified,  the command re-reads the master
cells of the instances in the object set.  Only the master cells of
the instances are re-read;  cells are not re-read recursively when
using this form.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3interrupt\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3^C\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 622 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3interrupt\f\n(31\h'|\n(41u'\f3^C\f\n(31\h'|\n(42u'\f3No arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This routine interrupts (deactivates) the window containing the cursor.
No drawing will be done in the window until a full redraw requested
by the user (using pan, zoom, or redraw-window) is done.  
The key binding for this command
can also be used while a window is drawing to immediately
stop drawing in that window.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3kill-buffer\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3<*>\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3"cell view {facet} {version}"\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 636 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3kill-buffer\f\n(31\h'|\n(41u'\f3<*>\f\n(31\h'|\n(42u'\f3"cell view {facet} {version}"\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I kill-buffer
command flushes a facet out of memory
.I without
saving its contents.  
If the string specification of the facet is missing,  the facet
is determined by the window containing the cursor.
.I All
windows looking at this facet are destroyed.  There are no
key or menu bindings for the command and it will ask the user
for confirmation before carrying out the command.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3list-palettes\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3L\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3Saves arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 655 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3list-palettes\f\n(31\h'|\n(41u'\f3L\f\n(31\h'|\n(42u'\f3Saves arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I list-palettes
command outputs a list of valid palette names for the cell containing
the pointer to the console window.
These names can be used with the
.I palette
command (see below).
.RE
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3log-bindings\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3<*>\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3Saves arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 669 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3log-bindings\f\n(31\h'|\n(41u'\f3<*>\f\n(31\h'|\n(42u'\f3Saves arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I log-bindings
command writes out a description of all type-in, menu, and key bindings for all
commands in the editing style of the window containing the cursor.
This description is written to the log file for the session.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3name-objects\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3n\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] "name"\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 682 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3name-objects\f\n(31\h'|\n(41u'\f3n\f\n(31\h'|\n(42u'\f3[objs] "name"\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command names all of the objects selected in the first argument
to the name given in the last argument.  This is used most often in the
symbolic and schematic editing styles to name instances and nets
for reference after simulation.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3open-window\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3o\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[box] or "cell:{view:{facet:{version}}}"\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 694 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3open-window\f\n(31\h'|\n(41u'\f3o\f\n(31\h'|\n(42u'\f3[box] or "cell:{view:{facet:{version}}}"\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I open-window
command is primary way to create new graphics windows in VEM.
It takes a string specifying the cell to open.  
When specifying the cell portion of the name,  typing a TAB
will attempt to complete the string as a file or offer alternatives
if the name is not unique.
If this string is
absent,  it will duplicate the window containing the cursor.  
Normally,  the extent of the duplicated window is the same as
the parent window.  However, if
the user specifies a box,  the duplicated window will be zoomed
to that extent (see zoom-in).
The string specifying the cell contains four fields.  The last three
are optional and default to "physical", "contents", 
and the null string respectively.
It is possible to specify your own defaults for these fields.  See
the customization document for details.
Newly created windows are always zoomed to contain all geometry in the cell.
If the cell does not exist,  it will be created.
When creating new cells,  VEM prompts the user for required cell
properties.  See the introduction for details.  Most of the time, the
defaults presented in this dialog are acceptable and activating the
.I Ok
button is sufficient to proceed.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3palette\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3P\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3{"palette-name"}\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 727 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3palette\f\n(31\h'|\n(41u'\f3P\f\n(31\h'|\n(42u'\f3{"palette-name"}\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I palette
command opens a new window onto a previously created facet which contains
standard layers or instances for a given technology.  This window can
be used to select layers for creating geometry or instances for
instantiation.
The command takes one argument:  the name
of the palette.  If omitted,  it defaults to "layer".
A list of valid palette names for a cell can be obtained by
using the
.I list-palettes
command.
.PP
Palette cells are found using the function tapGetPalette (see
tap(3)).  For all standard technologies and viewtypes,  there is
a "layer" palette.  In the symbolic editing style,  there are
also "mosfet" and "connector" palettes which
display mosfets and connectors respectively.  In the schematic
editing style,  there are "device" and "gate"
palettes which contain device level and gate level schematic
primitives.  New palettes can be added easily.  See ``Customizing VEM''
for details.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3pan\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3p\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[any arguments] [pnt]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 757 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3pan\f\n(31\h'|\n(41u'\f3p\f\n(31\h'|\n(42u'\f3[any arguments] [pnt]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I pan
command centers the window containing the cursor around the last point
on the argument list.  The window will be redrawn so that the argument
list point is now the center of the window.  The point need not be
in the same window as the cursor.  Thus,  a user can point in a window
showing a large portion of a cell and invoke the command in
a more detailed window for a magnifying glass effect.  If the point
is omitted,  the command assumes the cursor position is also the
desired center point.  This is the fast way to pan in a single
window.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3pop-context\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3)\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No Arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 776 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3pop-context\f\n(31\h'|\n(41u'\f3)\f\n(31\h'|\n(42u'\f3No Arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command pops off an input context from the context stack
and replaces the current context with that context.  See the
.I push-context
command for details.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3push-context\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3(\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 788 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3push-context\f\n(31\h'|\n(41u'\f3(\f\n(31\h'|\n(42u'\f3No arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command pushes the current argument list context onto the context
stack and gives the user a new context.  This can be used to do other
commands while preserving entered arguments.  Note that the current
arguments remain displayed.  The old context can be restored using the
.I pop-context
command.
Four context levels are supported in the current version of VEM.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3push-master\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3i\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3{"facet"}\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 803 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3push-master\f\n(31\h'|\n(41u'\f3i\f\n(31\h'|\n(42u'\f3{"facet"}\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I push-master
command opens a new window on the master of the instance under the
mouse cursor.
This command can be used all editing styles.
If a facet name is supplied,  the command will use that facet
name instead of "contents".
.RE

.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3recover-facet\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3<*>\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No Arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 819 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3recover-facet\f\n(31\h'|\n(41u'\f3<*>\f\n(31\h'|\n(42u'\f3No Arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
Unless directed otherwise,
VEM saves all cells occasionally in case of a system crash or some
other unforseen disaster.
Whenever a new cell is opened, VEM checks to see if the last
automatically saved version is more recent than the user saved
version.
If the automatically saved version is more recent, a warning is
produced and the user version is loaded.
One can use the
.I recover-facet
command to replace the cell with the more recent automatically saved
version.
The command displays a dialog containing a list of all of the saved
versions.
Generally, there are two possible alternative versions for a cell.
The
.I autosave
version is written by vem automatically after a certain number of
changes to the cell.
The
.I crashsave
version is written when VEM detects a serious error.
Note that the crashsave version may itself be corrupt since a serious
error occurred just before it was written.
This command is destructive: it replaces the cell with the selected
alternate.
One can use the
.I Cancel
button to abort the recovery.
Before using the
.I recover-facet
command, it is often useful to view the alternate cells.
This can be done by specifying the version (either "autosave" or
"crashsave" as the last field in the cell specification to
.I open-window.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3re-read\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3R\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No Arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 862 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3re-read\f\n(31\h'|\n(41u'\f3R\f\n(31\h'|\n(42u'\f3No Arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I re-read
command flushes the facet associated with the window containing the
cursor out of memory and reads it back in from the disk.  This
can be used to see changes to a cell that were done outside VEM
or to revert back to the cell before changes were made.  This
is a dangerous command and VEM asks for confirmation before
proceeding.
.RE


.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3redraw-window\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3^L\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[box]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 880 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3redraw-window\f\n(31\h'|\n(41u'\f3^L\f\n(31\h'|\n(42u'\f3[box]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command redraws the contents of the window containing the cursor.
It does not effect the argument list (i.e. it can be done regardless
of the argument list contents).  If a box is provided,  only the
portion of the window in the box is redrawn.  If the window is interrupted,
this command will reactivate it.  However,  if the box form is used,
VEM will only draw the specified area and leave the window deactivated.
This can be used to selectively draw portions of a deactivated window.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3same-scale\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3= \fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[any arguments] [pnt]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 895 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3same-scale\f\n(31\h'|\n(41u'\f3= \f\n(31\h'|\n(42u'\f3[any arguments] [pnt]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command changes the scale of the window containing the mouse to
the same scale as the window containing the last point on the argument
list.  It is commonly used to compare the sizes of two facets.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3save-all\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3S\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 906 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3save-all\f\n(31\h'|\n(41u'\f3S\f\n(31\h'|\n(42u'\f3No arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I save-all
command spawns a dialog to ask the user which buffers he would like
to save.  Only those chosen will be flushed to disk.  The command
takes no arguments and can be invoked only from the console window.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3save-window\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3S\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[any arguments]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 919 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3save-window\f\n(31\h'|\n(41u'\f3S\f\n(31\h'|\n(42u'\f3[any arguments]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command saves the contents of the facet associated with the
window where the command was invoked.  It asks for confirmation
and does not effect the argument list.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3set-path-width\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3w\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[box] [line] [point] or ["size string"]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 930 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3set-path-width\f\n(31\h'|\n(41u'\f3w\f\n(31\h'|\n(42u'\f3[box] [line] [point] or ["size string"]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command sets the current path width for a given layer on a
window-by-window basis.
It takes one argument which may be points, lines, boxes,
or text.  For points and lines,  the argument length must
be two.  The path width is set to the maximum manhattan
distance between the points.  For boxes,  only one box
is allowed at the new width is the larger of the two
dimensions.  For text, the string should contain the path
width in lambda.
If no width is specified,  the path width will be set to
the minimum layer width as specified in the technology.
.PP
The layer for the path width command is determined by looking
at the object under the cursor.
If there are objects on more than one layer under the cursor,
a dialog will be presented and the user should choose one
of the listed layers and press "OK" to continue.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3show-all\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3f\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No Arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 955 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3show-all\f\n(31\h'|\n(41u'\f3f\f\n(31\h'|\n(42u'\f3No Arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I show-all
command causes VEM to zoom the window containing the cursor
so that all of the cell is displayed in the window.  The
key binding is an abbreviation for "full".  It does not effect
the argument list.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3swap-bufs\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3<*>\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 969 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3swap-bufs\f\n(31\h'|\n(41u'\f3<*>\f\n(31\h'|\n(42u'\f3No arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command swaps the alternate and primary buffers of the window
containing the cursor when the command is invoked.  The
.I alt-buffer
command is used to establish alternate buffers for a window.  This
command is not meant to be used by users at the current time.  It
is exported for use by remote applications.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3switch-facet\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3^S\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3["cell view {facet {version}}"] or [pnt]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 983 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3switch-facet\f\n(31\h'|\n(41u'\f3^S\f\n(31\h'|\n(42u'\f3["cell view {facet {version}}"] or [pnt]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command replaces the facet in the window containing the mouse
with a different facet.  The first form replaces the window's facet
with the named facet.  The second form replaces the window's facet
with the facet of the window containing the point.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3toggle-expansion\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3e\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No Arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 995 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3toggle-expansion\f\n(31\h'|\n(41u'\f3e\f\n(31\h'|\n(42u'\f3No Arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I toggle-expansion
command toggles the window under the cursor between expanded
and unexpanded mode.  In unexpanded mode,  all instances are shown
by displaying the interface facet of the instance master.  In expanded
mode,  all instances are shown by displaying the contents facet
of the instance master.  In later releases,  this command will allow
the user to specify only particular instances for expansion.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3toggle-grid\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3g\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No Arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1011 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3toggle-grid\f\n(31\h'|\n(41u'\f3g\f\n(31\h'|\n(42u'\f3No Arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I toggle-grid
command toggles the visibility of the grid in the window containing
the cursor.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3version\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3V\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1023 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3version\f\n(31\h'|\n(41u'\f3V\f\n(31\h'|\n(42u'\f3No arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command outputs the current versions of VEM and the RPC server
to the console window.  
This command can only be invoked in the console window.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3where\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3?\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1034 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3where\f\n(31\h'|\n(41u'\f3?\f\n(31\h'|\n(42u'\f3No arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I where 
command can be used to find out the position of the cursor
in terms of 
.B Oct
units.  It also displays a textual representation
of the objects under the cursor.  The command can be issued while building an
argument list without effecting the list.
Alternatively,  if the argument list includes an object set,  the
where command will print textual descriptions of the selected items.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3write-window\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3W\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3"cell:view" [any arguments]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1052 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3write-window\f\n(31\h'|\n(41u'\f3W\f\n(31\h'|\n(42u'\f3"cell:view" [any arguments]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I write-window
command saves the contents of the cell associated with the window where
the command was invoked under another name.
This alternate name is specified by the "cell:view" argument.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3zoom-in\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3z\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[any arguments] [box]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1065 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3zoom-in\f\n(31\h'|\n(41u'\f3z\f\n(31\h'|\n(42u'\f3[any arguments] [box]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I zoom-in 
command zooms the window containing the mouse to the extent
indicated by the last box on the argument list.  The box
and the zoom window must be in the same facet.  
However,  the extent may be in a different window from the mouse
which can be used to achieve a magnifying glass effect.
If the box is not provided,  it zooms the window containing
the mouse in by a factor of two.  If provided,  the command removes the box
from the argument list but leaves other arguments untouched.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3zoom-out\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3Z\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[any arguments] [box]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1083 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3zoom-out\f\n(31\h'|\n(41u'\f3Z\f\n(31\h'|\n(42u'\f3[any arguments] [box]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I zoom-out 
command is the opposite of zoom-in.  This command zooms
the window containing the mouse out far enough so that the OLD
contents of the window are contained in the extent of the box
provided on the argument list.  Thus,  smaller boxes zoom out
farther than larger ones.  If the box is omitted,  the window
containing the mouse is zoomed out by a factor of two.
.RE
.SH OPTIONS
The options commands below all post form windows which allow a user
to change display parameters interactively.  The default values
for these parameters can be changed in your ~/.Xdefaults file.
See the document "Customizing VEM" for details.
.PP
All of the options dialogs below are
.I unmoded.
This means that the user can do other things while the dialog is
posted.  They will not go away until explicitly closed by the user.
Details on the operation of dialog boxes can be found in the section
.I Using Dialog Boxes
at the start of the manual.
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3window-options\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3<*>\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1111 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3window-options\f\n(31\h'|\n(41u'\f3<*>\f\n(31\h'|\n(42u'\f3No arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command posts a dialog which presents a number of window related
options each of which can be modified by the user on a window-by-window
basis.  Two kinds of options are presented in this dialog: flag options
and value options.  Flag options have a check box on the left of
the option and can be either on or off.  Pressing the left mouse button
in the check box toggles the value of the option.
Value options display a numerical value in a box with a descriptive
label to the left and a scrollbar to the right for changing the value.
At the bottom of the dialog are four control buttons: 
.I OK,
.I Dismiss,
.I Apply,
and
.I Help.
Pressing the left mouse button inside the
.I OK
button saves any options you may have changed and closes the window.
The
.I Dismiss
button does not save any options and closes the window.
The
.I Apply
button saves any changed options but does not close the window.
The
.I Help
button will open a window containing a brief description
of the dialog.
.PP
Each of the options and their meanings are given below:
.TP
.I Visible Grid
If set, a grid will be shown in the window.
.TP
.I Dotted Grid
If the grid is visible and this option is set,
the grid will be drawn as dots rather than
lines.
.TP
.I Manhattan Argument Entry
If this option is set, entering line arguments and dragging
option sets will be restricted to manhattan angles.  This
option is set by default in the symbolic and schematic
editing styles.
.TP
.I Argument Gravity
If set,  all lines entered using the left button whose endpoints
are near an actual terminal will be snapped to that terminal.
This is especially useful when editing
schematic diagrams (VEM automatically turns this option on
when the edit style is set to schematic).
The .Xdefault parameter
.I vem.gravity
specifies the maximum distance 
between line endpoints and terminals
for gravity to have an effect (by default, 10 pixels).
.TP
.I Show Instance Bounding Boxes
If this option is set,  VEM will display bounding boxes
around all instances.
The instance name will be displayed in the center of
these bounding boxes.
.TP
.I Show Actual Terminals
When viewing very large designs, drawing the highlighting around actual terminals
can be expensive.  If this option is turned off,
VEM will not draw highlighting around actual terminals.
.TP
.I Expand Instances
If on,  the contents facet of instance masters will be displayed.
Otherwise,  the interface facet will be displayed.  This has
the same effect as the
.I toggle-expansion
command.
.TP
.I Visible Title Bar
If this option is set,  VEM will display its own title bar above each
graphics window.  If the option is turned off,  the title bar will
be turned off.
.TP
.I Oct units per Lambda
This parameter specifies the number of 
.B Oct 
units per lambda.  The
output of the where command and the coordinate displays in the title
bar are displayed according to the value of this parameter.  By default,
VEM uses 20 
.B Oct 
units per lambda.
.TP
.I Snap
All graphic input into the window will be snapped to multiples of
this parameter (given in 
.B Oct
units).  By default,  VEM snaps to
one lambda (20 
.B Oct
unit) intervals.
.TP
.I Major Grid Spacing
This parameter specifies the grid spacing of major grid lines
in 
.B Oct 
units.  If logarithmic grids are turned on,  it specifies
a multiplying factor for major grid line spacing (see Base below).
.TP
.I Minor Grid Spacing
This parameter specifies the grid spacing of minor grid lines
in 
.B Oct 
units.  If logarithmic grids are turned on,  it specifies
a multiplying factor for minor grid line spacing (see Base below).
.TP
.I Minimum Grid Threshold
This parameter specifies the smallest allowable space (in pixels) 
between grid lines before VEM stops drawing them.
.TP
.I Log Grid Base
If non-zero,  this option selects a logarithmic grid.  Normally,
there are two grids drawn at a fixed number of 
.B Oct 
units (major
and minor grid lines).  In a logarithmic grid,  grids are drawn at
some constant (specified by the Major Grid and Minor Grid parameters
above) times the nearest integral power of the grid base.  For
example,  if the constant is two and the base 10,  grids will be
drawn at 2, 20, 200, etc.
.TP
.I Log Grid Minimum Base
This parameter specifies the smallest base to be used for drawing
logarithmic grids (see above).
.TP
.I Solid Fill Threshold
This parameter specifies the size (in pixels) before a shape is
drawn using solid fill rather than stipple fill.
A large number specifies all geometry should be drawn solid
regardless of its size.
.TP
.I BB Name Threshold
Label text drawn in bounding boxes (e.g. instances or terminals)
may or may not be drawn depending on the size of the bounding box.
This parameter specifies how many times wider the text may be than the
box before the label is not drawn.
.TP
.I BB Abstraction Threshold
This parameter specifies the maximum size of a bounding box (in pixels)
where it is acceptable to draw a filled box rather than an outline
to speed the drawing process.
.TP
.I Contact Abstraction Threshold
This parameter specifies the maximum size of a contact (connector)
where it is acceptable to draw an abstraction of the contact
rather than its full definition.
This can be used to tell VEM to draw contacts quickly.
.TP
.I Interface Facet
This string parameter specifies the name of the displayed
interface facet.
This facet will be used to draw instances in unexpanded mode.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3layer-display\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3<*>\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No Arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1277 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3layer-display\f\n(31\h'|\n(41u'\f3<*>\f\n(31\h'|\n(42u'\f3No Arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command posts a dialog which can be used to selectively turn on or
off the display of geometry on any layer.
At the bottom of the dialog are six control buttons.
The
.I Ok,
.I Dismiss,
.I Apply,
and
.I Help
buttons work in the same way as described for
.I window-options.
The
.I All
button automatically selects all of the layers displayed in the body
of the dialog.
The
.I Clear
button automatically unselects all of the layers displayed in the body
of the dialog.
Above the control buttons there is a list of all layers displayed
in the window.  
The layers currently shown in the window have
buttons to the left of the layer name
that have check marks.
Those that are not shown have buttons to the left of
the layer name that appear empty.  The state of a layer
can be changed by moving the cursor over the corresponding button 
and depressing the left mouse button.
Note that no window update will occur until either
.I Ok
or
.I Apply
are pressed.
.RE
.SH SELECTION
.PP
The selection commands described below are used to manipulate
object arguments on the argument list.
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3select-layer\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3.\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] [pnts] [lines] [boxes] "layer"\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1321 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3select-layer\f\n(31\h'|\n(41u'\f3.\f\n(31\h'|\n(42u'\f3[objs] [pnts] [lines] [boxes] "layer"\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I select-layer
command is similar to the
.I select-objects
command but allows the user to select only the geometries
on a particular layer.  This layer can be specified in two
ways:  the layer name can be typed in as the last argument
or the command will try to determine the layer by looking
at the geometry under the cursor when the command is invoked.
If the spot for the layer is ambiguous,  VEM will post a dialog
presenting a choice between the layers.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3select-objects\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3s\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] [pnts] [lines] [boxes] "layer"\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1340 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3select-objects\f\n(31\h'|\n(41u'\f3s\f\n(31\h'|\n(42u'\f3[objs] [pnts] [lines] [boxes] "layer"\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I select-objects
command is used for placing collections of objects on
the argument list for further processing by other commands.
It takes as arguments any number of points,  lines,  or boxes.
Points select items under the point,  lines select objects which
cross the line,  and boxes select objects inside the box.  
If
.I select-objects
is not given any point, line, or box arguments,  it will try
to select items under the cursor where the command was invoked.
These semantics are described in detail below.
.PP
For each
point,  the command adds zero or more of the objects under the
point to the list.  
If there is more than one object under the point,
a dialog will be posted with buttons representing each of the objects
under the point.  Clicking the mouse in one of the buttons highlights
the object.  Once the user has clicked on the desired objects,  the 
"ok" button is used to actually select the items.
.PP
For each line argument,  the command adds all objects which intersect
the line.  This is useful for schematic drawings where paths (wires)
are zero width.
Selection using lines works best if the entered lines are manhattan.
Non-manhattan lines may select more objects than intended.
.PP
For each box,  the command adds all objects completely contained
in the box to the list.
Note that an object is considered contained if and only if its
.I "bounding box"
is
.I "completely inside"
the given box.
The 
.I select-objects
command is incremental;  i.e. it may be called many
times,  each time adding to the selected set.  All items selected
are highlighted in the VEM highlight color.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3select-terms\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3^T\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs][pnts][lines][boxes]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1388 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3select-terms\f\n(31\h'|\n(41u'\f3^T\f\n(31\h'|\n(42u'\f3[objs][pnts][lines][boxes]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
This command selects all terminals (both actual and formal) whose
implementations intersect the objects found by examining the argument
list.  The semantics for specifying the objects is identical to
that described for
.I select-objects.
The items on the argument list will be replaced with the
set of terminals found by examining the items.
This command is useful for deleting formal terminals, specifying
actual terminals for use by 
.I edit-property
or
.I select-bag,
or creating new symbolic formal terminals using
.I promote.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3transform\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3t\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1409 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3transform\f\n(31\h'|\n(41u'\f3t\f\n(31\h'|\n(42u'\f3[objs]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command takes a selected set of objects built by selection
commands and transforms them.  The objects remain on the selected
set.  It is important to note that the actual objects in the database
are not affected by this command.  The transformation is associated
with the object set not with the objects themselves.
.PP
The command takes (up to) three arguments:  a set of objects to
work on,  a text rotation specification,  and two points indicating
a relative translation.  The object set must be supplied.
The rotation specification is a list of keywords separated by
colons:
.TP
.I mx
Mirror around the Y axis.
.TP
.I my
Mirror around the X axis
.TP
.I 90
Rotate counter-clockwise 90 degrees.
.TP
.I 180
Rotate counter-clockwise 180 degrees.
.TP
.I 270
Rotate counter-clockwise 270 degrees.
.PP
If both a rotation and a translation are given,  the rotation specification
should come first.  If no rotation and translation are given,  the
command rotates the items 90 degrees counter-clockwise.
.PP
The
.I transform
command is incremental.
This means it can be applied many times with each command having a
relative effect on the current selected set.
For example,  invoking 
.I transform
without arguments twice is the same as invoking it once and specifying
the rotation string "180".
.PP
Once the command completes,  the highlighted form of the objects
will reflect the specified transformation.  This can be used as
a reference for further rotations or translations.
.PP
Translations are specified by two points.  A relative translation
is applied to the current transformation based on the vector formed
by the two points.
It is also possible to interactively specify the translation of
a selected set.
This is done by moving the cursor to a reference point and
pressing, dragging, and releasing the right mouse
button.  While the right button is depressed,  the selected objects
will track the cursor motion.
This can be used to drag items around interactively.
.PP
After completing a transformation with transform,  the objects
are not actually changed until a command that manipulates objects
is invoked (e.g. move-objects, copy-objects, drag-instance, etc).
See the
.I move-objects
and
.I copy-objects
commands below for more information.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3unselect-objects\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3u\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] [pnts] [lines] [boxes]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1481 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3unselect-objects\f\n(31\h'|\n(41u'\f3u\f\n(31\h'|\n(42u'\f3[objs] [pnts] [lines] [boxes]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I unselect-objects
command is used to remove items from a previously created
object argument (select-objects operation).  
Any number of points, lines, and boxes may be
specified.  
The semantics for mapping these arguments to objects is the same as
.I select-objects.
For each point,  zero or more items beneath the point that are
part of the selected set are removed.  For each line,  all
items in the selected set intersecting the line are removed 
from the set.  For each box,  all items
completely contained in the box that are part of the selected
set are removed from the set.  
.PP
It is important to note that this command is intended to
be used to unselect small sets of already selected objects.
To unselect all items on the argument list,  use control-W
or control-U.
.PP
.SH PROPERTY AND BAG EDITING
.PP
The
.B Oct
Data Manager supports two kinds of annotation: properties and bags.
Properties are named objects which may have an arbitrary value and
can be attached to any other
.B Oct
object.
Properties are used most often to represent parameters or other
kinds of design documentation.
Bags are named objects which can contain any number of other
.B Oct
objects.
Bags are used to represent common collections of objects (instances
for example) that can be accessed efficiently.
The commands below can be used to create, edit, and delete properties
and bags.
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3attach-to-bag\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3<*>\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] {"bag name"}\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1525 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3attach-to-bag\f\n(31\h'|\n(41u'\f3<*>\f\n(31\h'|\n(42u'\f3[objs] {"bag name"}\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command attaches the items on the argument list to the
specified  bag.  
The command uses an involved process to try to determine the
right bag if no bag name is supplied.  First,  if the object
under the spot where the command was issued is ambiguous,  it
will present a dialog which lists all of the objects.  The
user can choose one by clicking the mouse in the check box
next to the object name.  The selected object is highlighted.
When the right object is found,  the user clicks the "ok" button
to continue.
Then,  the command examines the object for attached bags.  If
it has both parent and child bags,  it will ask the user which
ones he would like to see.  It then presents a list of these bags.
As the user selects bags in this dialog,  the objects connected
to the bag are highlighted.  Once a decision has been made,  the
user can click the "ok" button to continue.  The user is then
asked whether he would like to use the specified bag
bag or look through the bags connected to it.  In this way,  the
user can reach any bag in any hierarchy of bags.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3create-bag\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3<*>\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] ["bag name"]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1552 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3create-bag\f\n(31\h'|\n(41u'\f3<*>\f\n(31\h'|\n(42u'\f3[objs] ["bag name"]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command creates a new bag which is initially empty.
The command takes two arguments:  a collection of objects
specifying the parents (containers) of the bag,  and a bag
name.  If the parent object list is absent,  the parent
is assumed to be the facet of the window.  Objects can
be added to the bag using the
.I attach-to-bag
command.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3detach-from-bag\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3<*>\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] {"bag-name"}\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1568 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3detach-from-bag\f\n(31\h'|\n(41u'\f3<*>\f\n(31\h'|\n(42u'\f3[objs] {"bag-name"}\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command detaches the items in the specified object set from
the bag specified by bag-name.  If bag-name is omitted,  the
method used to discover the bag is the same as that described for
.I attach-to-bag.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3edit-property\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3^E\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1580 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3edit-property\f\n(31\h'|\n(41u'\f3^E\f\n(31\h'|\n(42u'\f3[objs]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I edit-property
command edits properties attached to objects supplied
in an object set.
If the object set is omitted, the object under the
cursor is used.
If there is no object under the cursor,  the facet of
the window where the command was involked is used.
.PP
If there is more than one property attached to
an object,  VEM will present a selection dialog that
presents a list of all properties
attached to the item.
The user can specify one of the properties by clicking
the mouse in the check box next to the property name.
Once a property has been selected,  the user can
edit the property using the
.I Edit
button, spawn a new selection dialog with the
specified property as the top level using the
.I Edit Children
button, create a new property attached to
the object using the
.I New
button,
or cancel the dialog using the
.I Cancel
button.
Property editing is described in the next paragraph.
.PP
The
.I Edit
and
.I New
options of the selection dialog spawn a property editor dialog
which is used to change the name or value of a property.
The property editor has check boxes for setting the
type of the property, type-in fields for the property
name and value, and six control buttons: 
.I Save,
.I New,
.I Cancel,
.I Edit Children,
.I Delete
and
.I Help
The name and value fields of the property can be changed
by moving the mouse cursor into the field and using
Emacs like editing commands to change the value of
the property.
The type of the property can be changed by clicking in
the check box next to the desired type.
.PP
The control buttons determine what should be done with the
property after changes have been made to its contents.
The
.I Save
button stores the property into 
.B Oct.
Invoking the
.I New
operation will create a new property with the name, value,
and type specified (this has no effect on the property
you may have been editing before).
The
.I Cancel
operation will end the dialog without making any changes
to the database.
The
.I Edit Children
button causes VEM to edit properties that are attached
to the displayed property.
If there is more than one,  a selection dialog will
be posted to allow the user to choose a property to edit.
The
.I Delete
operation will remove the displayed property from the
database.
.PP
All of the property dialogs are
.I modeless.
This means the user can do other things while property selection
or property editor dialogs are active.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3sel-bag-contents\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3B\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3["bag name"]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1671 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3sel-bag-contents\f\n(31\h'|\n(41u'\f3B\f\n(31\h'|\n(42u'\f3["bag name"]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command traces the contents of a bag and adds the items connected
to it to the current argument list.
If a bag name is specified,  then the source bag is determined by
finding the bag named "bag name" attached to the facet.  If the bag
name is not specified,  it uses the same mechanism as attach-to-bag
to determine the source bag from the object under the spot where
the command was invoked.  This command is often used to put the
current contents of a bag on the argument list for editing before
updating the bag using bag-replace.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3select-bag\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3b\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3["bag name"]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1688 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3select-bag\f\n(31\h'|\n(41u'\f3b\f\n(31\h'|\n(42u'\f3["bag name"]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command is similar to select-bag-contents except the bag itself
is placed on the argument list.  This command is used to create
bags which contain bags and to delete bags.  Bags that contain bags
can be constructed by selecting the bag and then using create-bag.
Bags can be deleted by selecting the bag and invoking the delete-objects
command.
.RE
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3show-property\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3^P\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1701 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3show-property\f\n(31\h'|\n(41u'\f3^P\f\n(31\h'|\n(42u'\f3[objs]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I show-property
command produces a list of all the properties associated
with the object under the cursor when the command was invoked
(or all objects in the selected set).  
If there are many objects under the cursor,  the command will
present a dialog which lists each object.  The user can select
one by clicking the mouse in the check box next to the object name.
The object will be highlighted.  When the right object is found,
the user can click "ok" to show the properties attached to the
object.
If the cursor is not over an object and nothing is in the
selected set, the properties attached to
the facet are shown.
The
properties are shown in the form:  xid: name (type) = value.  
The
.I xid
is the external identifier for the property and can be
used in evaluated labels.
The command also echos the type of the object each property is connected to.
.RE
.SH PHYSICAL EDITING COMMANDS
.PP
The physical editing style in VEM is used to enter and edit 
mask-level geometry.
This editing style is most often used to generate low-level
primitives that are instantiated in symbolic or schematic cells.
However,  physical editing can also be used as a general purpose
graphics editing system.
The commands described below are available in addition to the
common commands described in the previous section.
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3alter-geometry\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3a\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[box] [lines] or [points]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1740 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3alter-geometry\f\n(31\h'|\n(41u'\f3a\f\n(31\h'|\n(42u'\f3[box] [lines] or [points]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command replaces the box, path, or polygon under the cursor 
with the new specification supplied on the argument list.
This can be used to "stretch" geometry or change their composition.
For example,  to make a box slightly larger,  enter the slightly
larger box onto the argument list,  move the mouse over the old
box,  and invoke 
.I alter-geometry.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3change-layer\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3l\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] or [pnts][lines][boxes]"layer"\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1755 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3change-layer\f\n(31\h'|\n(41u'\f3l\f\n(31\h'|\n(42u'\f3[objs] or [pnts][lines][boxes]"layer"\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I change-layer
command detaches geometry from its current layer and attaches it
to a different layer.
The geometry can be specified either as an object set constructed
by the selection commands,  or directly by drawing points on them,
drawing lines through them,  or drawing boxes around them.
Normally,  the target layer is determined by looking at the geometry 
under the cursor when the command was invoked.
However,  if the last argument to the command is text,  it
is interpreted as the name of the target layer.
.RE
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3copy-objects\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3x\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] {pnt pnt}\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1773 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3copy-objects\f\n(31\h'|\n(41u'\f3x\f\n(31\h'|\n(42u'\f3[objs] {pnt pnt}\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I copy-objects
command copies a set of objects from one place to another.
The command takes an object argument that should contain a list
of objects to be moved (this is built with 
.I select-objects
and 
.I unselect-objects ).
The command assumes the object set has been transformed using the
.I transform
command or interactively dragged to a new location with the right
mouse button.  The command makes a copy of the objects where are transformed
according to this translation.  For example,  to copy objects from
one location to another,  the user first selects the objects using
.I select-objects,
then interactively drags the objects using the right button (transformation),
then invokes the
.I copy-objects
command to make a copy at the new location.
Since the items remain selected, new copies can be made without reselecting
the objects.
.PP
The optional second argument should be two points which specify the source
and destination points of the copy.  This alternative can be used if
the object set is too large for interactive dragging or one wishes to
copy objects from one facet into another.  If the copy is from one
facet to another facet,  terminals will not be copied and
the objects will be copied in a manner
that preserves the position of the objects relative
to the source point.
The default key binding for this command is short for
"xerox".
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3create-circle\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \wC
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[line] [pnts] "layer"\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1813 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3create-circle\f\n(31\h'|\n(41u'C\h'|\n(42u'\f3[line] [pnts] "layer"\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
Since VEM does not have a circle argument type,  a special circle
drawing command has been added.  For most types of geometry,  the
user should use
.I create-geometry.
Circles are specified in one of two ways.  The first is a line
followed by up to two points.  The line specifies a filled circle
with the first point being the center and the second its
outer radius.  If the first point is supplied,  an arc is assumed
with an angle formed by the second point of the line,  the center
point and the newly specified point.  The angle is measured counter-clockwise.
If the last point is supplied,  it specifies the inner radius of the circle
(otherwise the inner radius is zero).
The second form takes two points and an optional point.  It specifies
a circle where the inner and outer radius are the same.  If the last
point is supplied,  an arc with the same semantics as the first form
is assumed.  Finally,  the layer of the circle is determined from
the cursor position or by a final text argument specifying the layer
directly.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3create-geometry\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \wc
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[pnts] [lines] [boxes] [text] "layer"\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1839 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3create-geometry\f\n(31\h'|\n(41u'c\h'|\n(42u'\f3[pnts] [lines] [boxes] [text] "layer"\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I create-geometry
command creates new geometry.  It takes any number of
points, lines, boxes, or text and a layer specification.  A points argument
creates a closed polygon.  A line argument creates a multi-point
path.  Box arguments create boxes.  Finally,  text arguments
create labels.  When creating labels,  the point set after the
label is interpreted as the target points for the label.
All the geometry is created
on the same layer.  This layer may be specified as the final text
argument to the command or by invoking the command over an object
attached to a layer.  If the layer is ambiguous,  the command will
present a choice of layers in a dialog box.  The palette
command can be used to create a window which offers all possible
layers for creating geometry.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3create-instance\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3I\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[pnt] {"master:view name"}\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1862 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3create-instance\f\n(31\h'|\n(41u'\f3I\f\n(31\h'|\n(42u'\f3[pnt] {"master:view name"}\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
In most cases,  the leaf cells designed with the physical editing
style are not hierarchical.
Instead,  instances of the low-level cells are connected together
using the symbolic editing style.
However,  those who would like to use VEM as a purely physical
design editor require instances in physical cells.
This command places instances in the physical editing style.
.PP
The instance is placed relative to the point supplied to the command.
The master of the instance can be specified in two ways.
In the first form,  the user supplies a text argument which contains
the cell, view, and instance names separated by spaces.  The instance
name is optional.
The second form determines the master of the instance by looking at
the instance under the cursor when the command is invoked.
This instance can be in the same cell or in another cell.
A common practice is to build a cell of primitives and use this
cell as a menu for placing physical instances (see the
.I palette
command).
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3create-terminal\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3T\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3["term name"]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1890 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3create-terminal\f\n(31\h'|\n(41u'\f3T\f\n(31\h'|\n(42u'\f3["term name"]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command creates a new formal terminal named "term name".  The
implementation of the terminal is determined by constructing
a list of all geometries under the spot where the command was
invoked and choosing the smallest coincident boxes from this
list.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3delete-objects\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3D\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] or [pnts] [boxes] "layer"\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1903 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3delete-objects\f\n(31\h'|\n(41u'\f3D\f\n(31\h'|\n(42u'\f3[objs] or [pnts] [boxes] "layer"\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The delete command removes objects from a cell.  The command
has two forms.  The first takes an object argument constructed
using selection commands and deletes all of the items in this
set.
The second takes any number of point, line, and box arguments and
a layer specification.  This form deletes all objects under
the points,  all objects which intersect the lines, and all 
objects completely contained in the boxes
that are attached to the specified layer.  The layer may also
be specified by placing the cursor over some other object attached
to a layer when the command is invoked.  If no layer is specified,
all of the geometry is deleted.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3edit-label\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3E\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[pnt] { "LAYER" } or [objs]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1923 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3edit-label\f\n(31\h'|\n(41u'\f3E\f\n(31\h'|\n(42u'\f3[pnt] { "LAYER" } or [objs]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I edit-label
command creates and edits labels.
The command has two forms.  The first form creates a new label at the
specified point on the given layer.  If the layer is not specified,
it will be determined by looking at the object under the cursor.
The second form edits labels selected using the
.I select
command.
.PP
Labels in Oct are represented as a box where the text is drawn
entirely inside the box subject to justification and text
height parameters.
The edit-label command builds the box automatically by examining
the text height and text itself.
Thus, the user can control the justification, text height,
and the label text.
These parameters are set using the edit-label dialog box.
This is a modeless dialog box that is posted when the
user invokes the edit-label command.
.PP
The edit-label dialog box consists of three check-box areas for
adjusting the label justification, and two type-in fields
for adjusting the text height and the text itself.
The justifications are computed in relation to the point
the user specified when the label was first created.
Thus,  the horizontal justification specifies whether the
point should be to the left, center, or right of the text.
Similarly,  the vertical justification specifies whether
the point should be on the bottom, center, or top of the
text.
Finally, the line justification specifies how the lines
should be justified within the text block when there is
more than one line.
The text height of the text is given in Oct units.
Note that the X window system does not directly support
fully scalable fonts.
Thus,  VEM uses a strategy where it will pick the closest
font from a set of fonts that can be specified as a startup
parameter (see the document "Customizing VEM" for details).
Finally,  the last type in field can be used to enter
the text for the label.  The label can have as many lines
as necessary.
.PP
At the bottom of the dialog are four control buttons:
.I Ok,
.I Apply,
.I Dismiss,
and
.I Help.
The
.I Ok
button updates the value of the label and causes the dialog
to close.  The
.I Apply
button updates the value of the label (showing the effects
in the graphics window) but does not close the dialog.
This allows the user to adjust the label several times
if necessary.
The
.I Dismiss
button closes the dialog without updating the value of the label.
Finally, the
.I Help
button displays some help about how to use the dialog.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3move-objects\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3m\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] {pnt pnt}\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 1996 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3move-objects\f\n(31\h'|\n(41u'\f3m\f\n(31\h'|\n(42u'\f3[objs] {pnt pnt}\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I move-objects
command moves a set of objects from one place to another.
The command takes an object argument that should contain a list
of objects to be moved (this is built with 
.I select-objects and 
.I unselect-objects ).
The command assumes the object set has been transformed using the
.I transform
command or interactively dragged to a new location using the right
mouse button.  The command moves the objects to a new location based on
this transformation.  For example,  to move objects from
one location to another,  the user first selects the objects using
.I select-objects,
then interactively drags the objects using the right button (transformation),
then invokes the
.I move-objects
command to actually move the items to the new location.  The items remain
selected for further moves if necessary.
.PP
The optional second argument should be two points which specify the source
and destination points of the move.  This alternative can be used if
the object set is too large for interactive dragging.  Unlike the
.I copy-objects
command,  objects cannot be moved from one facet to another.
.RE
.RE
.SH SYMBOLIC EDITING COMMANDS
.PP
The symbolic editing commands in VEM allow the user to edit
.B Oct
symbolic views.
Symbolic views are used to represent layout in a form
suitable for compaction and simulation.
.PP
The Berkeley symbolic layout style consists of instances
which are interconnected using single segment paths on
a restricted set of layers.
All interconnecting paths have user-defined widths.
Connection between single segment paths is represented
using connector instances.
These instances represent same-layer as well as between-layer connections.
.PP
All symbolic views have terminals.
Terminals are objects which represent legal connection points
to the cell when it is instantiated elsewhere.
These terminals are often referred to as "formal terminals".
When the cell is instantiated in another cell,  a copy
of these terminals is made and these terminals are
called "actual terminals".
All connections in a symbolic cell are made by creating
paths which connect actual terminals together.
New formal terminals can be created so that the cell
can be used in higher level symbolic cells.
.PP
Interconnectivity is automatically maintained in symbolic
cells.
As connections between actual terminals are made,  information
is stored in the database about the connection.
This makes it easy to verify the connectivity of your
cell interactively.
Objects which are connected are automatically placed on
objects called "nets".
These objects can be selected in VEM to show the user
what instances are connected together.
.PP
All symbolic views have two representations: the cell itself and
its abstraction.
The cell itself describes the cell hierarchically (as
a cell made up of other cells which have been connected
together).
The abstraction of a cell is used to allow tools (including
VEM) to deal with the cell without looking at its detailed
definition.
The abstraction is usually non-hierarchical and much simpler
than the cell definition itself.
Cell abstractions can be automatically generated using
the program vulcan (1OCTTOOLS).
.PP
The current symbolic layout style encourages a bottom-up
design methodology.
First,  new primitive cells with formal terminals are
created (either automatically or using the VEM physical
editing style).
Often,  these cells are placed in a standard location
so everyone can use them (see the palette command).
Next,  new symbolic cells are generated by instantiating
primitive cells and connecting them together.
New formal terminals are defined so that these cells
can be used in higher level cells.
Vulcan is used to automatically generate protection
frames for each cell.
Higher level symbolic cells are constructed from these lower level
cells in the same manner.
.PP
The commands described below support the creation and manipulation
of symbolic cells.
These commands are available in addition to those described in
the general, selection, and options section.
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3update-interface\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3none\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3"No arguments"\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2102 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3update-interface\f\n(31\h'|\n(41u'\f3none\f\n(31\h'|\n(42u'\f3"No arguments"\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
Run vulcan on the facet in the specified window. Vulcan creates
the protection frames in for the facet. The in-memory representation
of the interface is updated. You will need to save the interface
explicitly.
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3create\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3c\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3(See Below)\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2112 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3create\f\n(31\h'|\n(41u'\f3c\f\n(31\h'|\n(42u'\f3(See Below)\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I create
command allows the user to add new objects to a symbolic cell.
Different arguments given to 
.I create
will produce different objects.
.PP
.B "Formal Terminals -"
\fB"terminal-name [type] [direction]"\fP
.B " : create"
.PP
When 
.I create
is given a single argument string, the name of a new formal terminal,
a formal terminal is created.  
The implementation of the formal terminal is taken to be the actual
terminal currently under the mouse (note: a connector terminal can also be used
for this purpose).
Formal terminal names must be unique within the cell.
If a formal terminal of the given name already exists,
VEM will display a dialog box asking whether or not you
wish to replace the old terminal.
.PP
Two optional pieces of annotation can be placed on the terminal:
type and direction.
The type can be one of SIGNAL, CLOCK, GROUND, SUPPLY, or TRISTATE.
If not provided,  SIGNAL will be assumed.
The direction can be one of IN, OUT, or INOUT.  If not specified,
INOUT will be assumed.
If either of these values are not provided in the terminal
name specification, VEM will post a dialog box containing fields
for entering both the terminal type and direction.  Pressing the left
mouse button in the value area for these fields will cause a menu
of the possible choices to appear.  New values can be selected
by releasing the mouse button over the desired value.  Once
appropriate values are selected, activating the
.I Ok
button at the bottom of the dialog will save the annotations.
Activating the
.I Dismiss
option will leave the annotations unspecified.
These annotations can be edited later using the edit-property command.
.PP
.B "Instances - points ["\fB"[master:view] [instance-name]"\fB
.B "] : create"
.PP
If the arguments to 
.I create
are a number of points followed by an optional string, instances will
be created with their origins at those points.
If the name of the master is not specified textually,
it will be inferred from the instance under the mouse.
.PP
The string argument has two parts: the instance specification and name.
Both of these fields are optional.  Specifying a null string is considered
to be the same as no string at all.  The instance specification is a
master-view pair, such as "chip:symbolic".  If this field is left
out, 
the master is inferred from the instance under the mouse.  
Otherwise, an attempt
is made to locate the instance by the master-view pair.  If the
name field is given, the instance will be given the specified name.
.PP
NOTES: Newly created instances whose actual terminals intersect actual terminals
of other instances will be automatically connected.
In this case,  no path is required between the terminals.
Rotated and mirrored forms of instances can be created by instantiating
a new instance and using the transform and move-objects or transform
and copy-objects commands.
.PP
.B "Paths - lines ["
\fB"layer-name"\fP
.B "] : create"
.PP
This command creates new segments for connecting together instance
actual terminals.
A new series of segments will be created on the specified layer.
If no layer is specified,  a layer will be inferred from the
layer of the object under the cursor.
Connector instances will be placed automatically at all
jog points.
The width of the new path is layer-dependent and can be changed
using the set-path-width command.
.RE
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3replace-instance\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3I\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] or [pnts,lines,boxes] ["cell:view"]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2204 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3replace-instance\f\n(31\h'|\n(41u'\f3I\f\n(31\h'|\n(42u'\f3[objs] or [pnts,lines,boxes] ["cell:view"]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command takes a set of instances specified using points, lines,
boxes, or objects and replaces them with an instance of a master
specified by text ("cell:view") or by the master of the instance
under the cursor.  This command is most useful for replacing instances
of a master whose terminals have been changed.  The command uses a
number of heuristics to attempt to reconnect the new instance
properly.  However,  the user should carefully check the instance
after the command has completed to make sure that the instances
are appropriately connected.
.PP
NOTE: The 
.I replace-instance
command automatically re-reads both the interface and
the contents facet of the replacement master.  
Any changes made to the
master since it was last saved will be lost.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3reconnect\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3C\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] or [pnts, lines, boxes] \fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2228 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3reconnect\f\n(31\h'|\n(41u'\f3C\f\n(31\h'|\n(42u'\f3[objs] or [pnts, lines, boxes] \f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command takes as an argument a set of instances.  These instances
are specified explicitly in an object set constructed using
.I select-objects
or implicitly as the instances under the specified points,
intersecting the specified lines, or inside the specified boxes.
The terminals of each instance are examined for connectivity
correctness.  Any inappropriate segments connected to any of the
terminals will be removed.  Any unconnected segments that overlap
suitable terminals of the selected instances will be connected.
This command is often used to connect instances that were not properly
connected initially using other symbolic editing commands.  See the
symbolic editing tutorial for situations where this might occur.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3change-segment\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3l\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] or [pnts, lines, boxes] ["layer"]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2248 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3change-segment\f\n(31\h'|\n(41u'\f3l\f\n(31\h'|\n(42u'\f3[objs] or [pnts, lines, boxes] ["layer"]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command changes either the width, the layer, or both of those
segments specified on the argument list to the same width and layer 
of the layer specified as the last argument. 
If no layer is specified,  one is inferred by looking at the layer of
the object under the cursor.
The command will change the types of the connectors at segment
endpoints appropriately.
This command could be used to speed path entry.
The user can enter a long routing sequence on one layer using
create and then change the appropriate segments to a new
layer for routing purposes.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3delete-objects\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3d\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] or [pnts, lines, boxes] ["layer"]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2267 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3delete-objects\f\n(31\h'|\n(41u'\f3d\f\n(31\h'|\n(42u'\f3[objs] or [pnts, lines, boxes] ["layer"]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command takes either an object list created with
the 
.I select-objects
and
.I unselect-objects
commands, or points, lines, and boxes with an optional layer-name.
The resulting objects are deleted from the cell.
Deleting objects from a symbolic cell may result in
a connection dividing into two connections.
VEM will print a warning if this occurs.
The user may wish to make sure that the remaining connections
are still named appropriately.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3promote\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3P\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2287 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3promote\f\n(31\h'|\n(41u'\f3P\f\n(31\h'|\n(42u'\f3[objs]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command takes a set of actual terminals selected using
.I select-terms
and promotes them to formal terminals with the same name as
the actual terminals.
If a formal terminal with the same name as one of the new terminals
already exists,  a dialog box will appear.  This dialog offers
the option of replacing the existing terminal with the new one
or leaving the existing terminal intact.  If the existing terminal
is left intact,  no new terminal will be created.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3select-major-net\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3^N\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3pnts, lines, or boxes ["net name"]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2304 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3select-major-net\f\n(31\h'|\n(41u'\f3^N\f\n(31\h'|\n(42u'\f3pnts, lines, or boxes ["net name"]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command finds the net associated with the object under
the points, intersecting the lines, or inside the boxes
and highlights all objects on that net.
If no points, boxes, or lines are specified,  the object
under the cursor will be examined.
Alternatively,  if a net name is provided,  objects on
the named net are highlighted.
This command can be used to check the connectivity of
a symbolic cell.
The command is incremental (i.e. multiple nets can be
selected).
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3move-objects\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3m\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[objs] {pnt pnt}\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2323 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3move-objects\f\n(31\h'|\n(41u'\f3m\f\n(31\h'|\n(42u'\f3[objs] {pnt pnt}\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I move-objects
command moves a set of objects from one place to another.
The command takes a set of objects (specified using select-objects)
and a transformation.
This transformation can be specified in two ways: as two points
specifying a relative translation or using the transform command.
The transform command allows general mirroring and rotation
to be specified in addition to a translation.
It is also possible to specify the translation of an object
set by interactively moving the set using the right mouse
button (see the transform command).
After a move operation,  the objects remain on the argument list for
further move or copy operations.
.PP
For example,  a set of objects can be moved using the following
sequence:  select the objects using the select-objects command.
Move the cursor over a reference point.  Depress and hold the
right mouse button and drag the items to a new location.
Use the transform command to rotate or mirror the object
set.  Once everything is correct,  invoke the move-objects
command to actually move the objects.
.PP
The connectivity of the items moved by this command is not changed.
This means an instance moved using this command also causes the
segments attached to its terminals to move as well.  Moving
objects between facets is not supported.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3copy-objects\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3x\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3(See Below)\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2358 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3copy-objects\f\n(31\h'|\n(41u'\f3x\f\n(31\h'|\n(42u'\f3(See Below)\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I copy-objects
command copies a set of objects from one place to another.
This command has the same form as 
.I move-objects 
(see above) except that
the objects are copied not moved.
Like 
.I move-objects,  
the objects copied remain on the argument
list for further move and copy operations.
However, unlike
.I move-objects,
the connections to the objects in the selected set are not copied.
Instead, the connectivity between the selected items is copied
along with the objects.
.RE
.SH SCHEMATIC
The schematic editing style is an extension of symbolic.
The symbolic commands
move-objects, copy-objects, delete-objects, and drag-instance
also work on schematics.
However,  the primitives used in schematic are usually schematic
symbols (at the gate or device level)
and all wires are zero width (see the palette command).
In addition to the general, selection, options, and symbolic
editing commands,  the following commands are specific to
the schematic editing style:
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3create\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3c\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3(See Below)\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2393 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3create\f\n(31\h'|\n(41u'\f3c\f\n(31\h'|\n(42u'\f3(See Below)\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The 
.I create
command allows the user to add new objects to a schematic cell.
Different arguments given to 
.I create
will produce different objects.
.PP
.B "Formal Terminals -"
\fB"terminal-name [type] [direction]"\fP
.B " : create"
.PP
When 
.I create
is given a single argument string, the name of a new formal terminal,
a formal terminal is created.  
The implementation of the formal terminal is taken to be the actual
terminal currently under the mouse (note: a connector terminal can also be used
for this purpose).
Since terminals in schematic may be quite small,  this routine
will try to find nearby terminals if it doesn't find a terminal
directly beneath the cursor.
.PP
Formal terminal names must be unique within the cell.
If a formal terminal of the given name already exists,
VEM will display a dialog box asking whether or not you
wish to replace the old terminal.
.PP
Two optional pieces of annotation can be placed on the terminal:
type and direction.
The type can be one of SIGNAL, CLOCK, GROUND, SUPPLY, or TRISTATE.
If not provided,  SIGNAL will be assumed.
The direction can be one of IN, OUT, or INOUT.  If not specified,
INOUT will be assumed.
If either of these values are not provided in the terminal
name specification, VEM will post a dialog box containing fields
for entering both the terminal type and direction.  Pressing the left
mouse button in the value area for these fields will cause a menu
of the possible choices to appear.  New values can be selected
by releasing the mouse button over the desired value.  Once
appropriate values are selected, activating the
.I Ok
button at the bottom of the dialog will save the annotations.
Activating the
.I Dismiss
option will leave the annotations unspecified.
These annotations can be edited later using the edit-property command.
.PP
.B "Instances - points ["\fB"[master:view] [instance-name]"\fB
.B "] : create"
.PP
If the arguments to 
.I create
are a number of points followed by an optional string, instances will
be created with their origins at those points.
If the name of the master is not specified textually,
it will be inferred from the instance under the mouse.
.PP
The string argument has two parts: the instance specification and name.
Both of these fields are optional.  Specifying a null string is considered
to be the same as no string at all.  The instance specification is a
master-view pair, such as "amp:schematic".  If this field is left
out, 
the master is inferred from the instance under the mouse.  
Otherwise, an attempt
is made to locate the instance by the master-view pair.  If the
name field is given, the instance will be given the specified name.
.PP
NOTES: Newly created instances whose actual terminals intersect actual terminals
of other instances will be automatically connected.
In this case,  no path is required between the terminals.
Rotated and mirrored forms of instances can be created by instantiating
a new instance and using the transform and move-objects or transform
and copy-objects commands.
.PP
.B "Paths - lines" : create
.PP
This command creates new segments for connecting together instance
actual terminals.
A new series of segments will be created on a predefined layer
(WIRING).
Connector instances will be placed automatically at all
jog points.
The width of the new path is always zero.
.PP
Normally,  the schematic editing style has a feature turned on
called "gravity".
When you draw segments with gravity turned on,  VEM will try
to connect the segments to a nearby terminal if you miss
a terminal by a small amount.
This is useful when editing a large cell.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3edit-label\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3E\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3[pnt] { "LAYER" } or [objs]\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2491 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3edit-label\f\n(31\h'|\n(41u'\f3E\f\n(31\h'|\n(42u'\f3[pnt] { "LAYER" } or [objs]\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I edit-label
command creates and edits labels.
The command has two forms.  The first form creates a new label at the
specified point on the given layer.  If the layer is not specified,
it will be determined by looking at the object under the cursor.
The second form edits labels selected using the
.I select
command.
.PP
Labels in Oct are represented as a box where the text is drawn
entirely inside the box subject to justification and text
height parameters.
The edit-label command builds the box automatically by examining
the text height and text itself.
Thus, the user can control the justification, text height,
and the label text.
These parameters are set using the edit-label dialog box.
This is a modeless dialog box that is posted when the
user invokes the edit-label command.
.PP
The edit-label dialog box consists of three check-box areas for
adjusting the label justification, and two type-in fields
for adjusting the text height and the text itself.
The justifications are computed in relation to the point
the user specified when the label was first created.
Thus,  the horizontal justification specifies whether the
point should be to the left, center, or right of the text.
Similarly,  the vertical justification specifies whether
the point should be on the bottom, center, or top of the
text.
Finally, the line justification specifies how the lines
should be justified within the text block when there is
more than one line.
The text height of the text is given in Oct units.
Note that the X window system does not directly support
fully scalable fonts.
Thus,  VEM uses a strategy where it will pick the closest
font from a set of fonts that can be specified as a startup
parameter (see the document "Customizing VEM" for details).
Finally,  the last type in field can be used to enter
the text for the label.  The label can have as many lines
as necessary.
.PP
At the bottom of the dialog are four control buttons:
.I Ok,
.I Apply,
.I Dismiss,
and
.I Help.
The
.I Ok
button updates the value of the label and causes the dialog
to close.  The
.I Apply
button updates the value of the label (showing the effects
in the graphics window) but does not close the dialog.
This allows the user to adjust the label several times
if necessary.
The
.I Dismiss
button closes the dialog without updating the value of the label.
Finally, the
.I Help
button displays some help about how to use the dialog.
.RE
.RE
.SH REMOTE APPLICATION COMMANDS
The following commands apply only if a remote application is running.
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3kill-application\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3<*>\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No Arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2567 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3kill-application\f\n(31\h'|\n(41u'\f3<*>\f\n(31\h'|\n(42u'\f3No Arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
The
.I kill-application
command kills the remote application which has control of the
window where the command was invoked.  This can be used to
terminate runaway remote applications.  Note it has no
standard menu or key bindings;  it is a type-in command only.
This command may not work on all machines.
.RE
.PP
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3rpc-any\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3r\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3"host pathname"\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2582 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3rpc-any\f\n(31\h'|\n(41u'\f3r\f\n(31\h'|\n(42u'\f3"host pathname"\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
This command starts up a remote application which is not on
the standard list of applications in the VEM menu.  "host"
specifies the network location for the application and
"path" specifies the path to the executable on "host".
If the host specification is omitted,  the local machine
is assumed.
.RE
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3rpc-signal\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3^Z\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3"number"\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2595 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3rpc-signal\f\n(31\h'|\n(41u'\f3^Z\f\n(31\h'|\n(42u'\f3"number"\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
Send the specified signal to the RPC application.
The signal is specified by a number, just as you would do from the 
shell with the kill command. 
.RE
.TS
.if \n+(b.=1 .nr d. \n(.c-\n(c.-1
.de 35
.ps \n(.s
.vs \n(.vu
.in \n(.iu
.if \n(.u .fi
.if \n(.j .ad
.if \n(.j=0 .na
..
.nf
.nr #~ 0
.if n .nr #~ 0.6n
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.fc
.nr 33 \n(.s
.rm 80 81 82
.nr 80 0
.nr 38 \w\f3rpc-reset\fP
.if \n(80<\n(38 .nr 80 \n(38
.80
.rm 80
.nr 38 20n
.if \n(80<\n(38 .nr 80 \n(38
.nr 81 0
.nr 38 \w\f3^R\fP
.if \n(81<\n(38 .nr 81 \n(38
.81
.rm 81
.nr 38 10n
.if \n(81<\n(38 .nr 81 \n(38
.nr 82 0
.nr 38 \w\f3No Arguments\fP
.if \n(82<\n(38 .nr 82 \n(38
.82
.rm 82
.nr 38 1n
.nr 79 0
.nr 40 \n(79+(0*\n(38)
.nr 80 +\n(40
.nr 41 \n(80+(3*\n(38)
.nr 81 +\n(41
.nr 42 \n(81+(3*\n(38)
.nr 82 +\n(42
.nr TW \n(82
.if t .if \n(TW>\n(.li .tm Table at line 2605 file Input is too wide - \n(TW units
.fc  
.nr #T 0-1
.nr #a 0-1
.eo
.de T#
.ds #d .d
.if \(ts\n(.z\(ts\(ts .ds #d nl
.mk ##
.nr ## -1v
.ls 1
.ls
..
.ec
.ta \n(80u \n(81u \n(82u 
.nr 31 \n(.f
.nr 35 1m
\&\h'|\n(40u'\f3rpc-reset\f\n(31\h'|\n(41u'\f3^R\f\n(31\h'|\n(42u'\f3No Arguments\f\n(31
.fc
.nr T. 1
.T# 1
.35
.TE
.if \n-(b.=0 .nr c. \n(.c-\n(d.-3
.RS
.PP
If an application terminates abnormally,  VEM may not recognize
that the window no longer has an application running in it.
This command forces VEM to reset the state of the window
so that new applications can be run in it.

.SH CUSTOMIZATION
.PP
VEM is a highly customizable editor.  Nearly all of the colors,  font
styles,  and fill patterns it uses can be configured to a user's
taste.  See "Customizing VEM" in the Octtools manual for details.
.PP
Furthermore,  VEM is meant to be an extensible program.
New commands can be written that are either tightly bound to
the editor or that run in a separate process space.  There
are documents describing both tightly bound and remotely
bound extensions to VEM.  Contact the author for information.
.SH FILES
~/.Xdefaults - Aesthetic parameters read by VEM
.br
$OCTTOOLS/doc/vem/vemcustom.dvi - Customizing VEM
.br
$OCTTOOLS/doc/vem/vemphytut.dvi - Physical Editing with VEM
.br
$OCTTOOLS/doc/vem/vemsymtut.dvi - Symbolic Editing with VEM
.br
$OCTTOOLS/lib/vem/vem.bindings - Keyboard and menu bindings
.SH SEE ALSO
$OCTTOOLS/doc/oct/objects.me - Structure of OCT
.br
.SH AUTHOR
David S. Harrison
.SH BUGS
.br
Opening a facet that is inconsistent (either out of date or one
with conflicting terminals) isn't handled very gracefully.
You can open an inconsistent facet by opening it again.
.br
Bounding boxes may not be drawn if there is no geometry in the cell.
.br
The set path width command doesn't work if you use a palette
to specify the layer.
.br
Editing the master of an instance displayed in some other window
sometimes causes the instance to disappear.  Re-reading the
cell with the instance gets around the problem.
